Package: ggRandomForests
Type: Package
Title: Visually Exploring Random Forests
Version: 3.5.0
Date: 2026-08-04
Authors@R: person("John", "Ehrlinger",
  role = c("aut", "cre"),
  email = "john.ehrlinger@gmail.com",
  comment = c(ORCID = "0000-0002-5340-5154"))
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/ehrlinger/ggRandomForests,
        https://ehrlinger.github.io/ggRandomForests/
BugReports: https://github.com/ehrlinger/ggRandomForests/issues
Description: Graphic elements for exploring Random Forests using the
    'randomForest' or 'randomForestSRC' package for survival, regression
    and classification forests and 'ggplot2' package plotting. Implements
    visualizations of the methods described in Breiman (2001)
    <doi:10.1023/A:1010933404324> and Ishwaran, Kogalur, Blackstone, and
    Lauer (2008) <doi:10.1214/08-AOAS169>.
Depends: R (>= 4.4.0)
Imports: randomForestSRC (>= 3.4.0), randomForest, varPro (>= 3.1.0),
        igraph, survival, tidyr, dplyr, ggplot2, patchwork, stringr
Suggests: bookdown, callr, covr, datasets, ggraph, kernelshap, knitr,
        lintr, magick, MASS, pkgdown, pkgload, quarto, ragg,
        RColorBrewer, rmarkdown, testthat, vdiffr
VignetteBuilder: quarto
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 14:21:39 UTC; ehrlinj
Author: John Ehrlinger [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5340-5154>)
Maintainer: John Ehrlinger <john.ehrlinger@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-04 17:00:07 UTC
Built: R 4.6.0; ; 2026-08-04 20:02:16 UTC; unix

Package: ggRandomForests
Type: Package
Title: Visually Exploring Random Forests
Version: 3.1.2
Date: 2026-06-13
Authors@R: person("John", "Ehrlinger",
  role = c("aut", "cre"),
  email = "john.ehrlinger@gmail.com")
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/ehrlinger/ggRandomForests,
        https://ehrlinger.github.io/ggRandomForests/
BugReports: https://github.com/ehrlinger/ggRandomForests/issues
Description: Graphic elements for exploring Random Forests using the
    'randomForest' or 'randomForestSRC' package for survival, regression
    and classification forests and 'ggplot2' package plotting. Implements
    visualisations of the methods described in Breiman (2001)
    <doi:10.1023/A:1010933404324> and Ishwaran, Kogalur, Blackstone, and
    Lauer (2008) <doi:10.1214/08-AOAS169>.
Depends: R (>= 4.4.0)
Imports: randomForestSRC (>= 3.4.0), randomForest, varPro, igraph,
        survival, tidyr, dplyr, ggplot2, patchwork, stringr
Suggests: testthat, bookdown, RColorBrewer, MASS, lintr, covr, vdiffr,
        datasets, rmarkdown, quarto, pkgdown, pkgload, knitr, ggraph,
        callr
VignetteBuilder: quarto
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-13 13:49:05 UTC; ehrlinj
Author: John Ehrlinger [aut, cre]
Maintainer: John Ehrlinger <john.ehrlinger@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-13 16:10:24 UTC

# Generated by roxygen2: do not edit by hand

S3method(autoplot,gg_brier)
S3method(autoplot,gg_error)
S3method(autoplot,gg_partial)
S3method(autoplot,gg_partial_rfsrc)
S3method(autoplot,gg_partialpro)
S3method(autoplot,gg_rfsrc)
S3method(autoplot,gg_roc)
S3method(autoplot,gg_survival)
S3method(autoplot,gg_variable)
S3method(autoplot,gg_vimp)
S3method(calc_roc,randomForest)
S3method(calc_roc,rfsrc)
S3method(gg_brier,rfsrc)
S3method(gg_error,randomForest)
S3method(gg_error,randomForest.formula)
S3method(gg_error,rfsrc)
S3method(gg_rfsrc,randomForest)
S3method(gg_rfsrc,rfsrc)
S3method(gg_roc,default)
S3method(gg_roc,randomForest)
S3method(gg_roc,rfsrc)
S3method(gg_survival,default)
S3method(gg_survival,rfsrc)
S3method(gg_variable,randomForest)
S3method(gg_variable,rfsrc)
S3method(gg_vimp,randomForest)
S3method(gg_vimp,rfsrc)
S3method(plot,gg_brier)
S3method(plot,gg_error)
S3method(plot,gg_partial)
S3method(plot,gg_partial_rfsrc)
S3method(plot,gg_partialpro)
S3method(plot,gg_rfsrc)
S3method(plot,gg_roc)
S3method(plot,gg_survival)
S3method(plot,gg_variable)
S3method(plot,gg_vimp)
S3method(print,gg_brier)
S3method(print,gg_error)
S3method(print,gg_partial)
S3method(print,gg_partial_rfsrc)
S3method(print,gg_partialpro)
S3method(print,gg_rfsrc)
S3method(print,gg_roc)
S3method(print,gg_survival)
S3method(print,gg_variable)
S3method(print,gg_vimp)
S3method(print,summary.gg)
S3method(summary,gg_brier)
S3method(summary,gg_error)
S3method(summary,gg_partial)
S3method(summary,gg_partial_rfsrc)
S3method(summary,gg_partialpro)
S3method(summary,gg_rfsrc)
S3method(summary,gg_roc)
S3method(summary,gg_survival)
S3method(summary,gg_variable)
S3method(summary,gg_vimp)
export(calc_auc)
export(calc_roc)
export(gg_brier)
export(gg_error)
export(gg_partial)
export(gg_partial_rfsrc)
export(gg_partialpro)
export(gg_rfsrc)
export(gg_roc)
export(gg_survival)
export(gg_variable)
export(gg_vimp)
export(kaplan)
export(nelson)
export(quantile_pts)
export(surv_partial.rfsrc)
export(varpro_feature_names)
importFrom(dplyr,all_of)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(parallel,mclapply)
importFrom(patchwork,wrap_plots)
importFrom(randomForest,randomForest)
importFrom(randomForestSRC,partial.rfsrc)
importFrom(randomForestSRC,vimp)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,xtabs)
importFrom(stringr,str_sub)
importFrom(survival,Surv)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(tidyr,pivot_longer)
importFrom(utils,head)
importFrom(utils,tail)

# ggRandomForests: Visually Exploring Random Forests

<!-- badges: start -->
[![cranlogs](https://cranlogs.r-pkg.org:443/badges/ggRandomForests)](https://cranlogs.r-pkg.org:443/badges/ggRandomForests)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ggRandomForests)](https://cran.r-project.org/package=ggRandomForests)
[![R package version](https://img.shields.io/github/r-package/v/ehrlinger/ggRandomForests)](https://github.com/ehrlinger/ggRandomForests)

[![active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/badges/latest/active.svg)

[![R-CMD-check](https://github.com/ehrlinger/ggRandomForests/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/R-CMD-check.yaml)
[![lint](https://github.com/ehrlinger/ggRandomForests/actions/workflows/lint.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/lint.yaml)
[![pkgdown](https://github.com/ehrlinger/ggRandomForests/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/pkgdown.yaml)
[![Codecov test coverage](https://codecov.io/gh/ehrlinger/ggRandomForests/graph/badge.svg)](https://app.codecov.io/gh/ehrlinger/ggRandomForests)

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.11526.svg)](https://doi.org/10.5281/zenodo.11526)
<!-- badges: end -->

<img src="man/figures/README-overview.png" alt="ggRandomForests overview: predicted survival, variable importance, OOB error, and partial dependence" width="100%" />

`ggRandomForests` provides `ggplot2`-based diagnostic and exploration plots for random forests fit with
[randomForestSRC](https://cran.r-project.org/package=randomForestSRC) (>= 3.4.0) or
[randomForest](https://cran.r-project.org/package=randomForest).
It separates data extraction from plotting so the intermediate tidy objects can be inspected, saved, or used
for custom analyses.
Listed in the [ggplot2 extensions gallery](https://exts.ggplot2.tidyverse.org/).

## Installation

```r
# CRAN (stable)
install.packages("ggRandomForests")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("ehrlinger/ggRandomForests")
```

## Quick start

```r
library(randomForestSRC)
library(ggRandomForests)

# 1. Fit a forest (regression)
rf <- rfsrc(medv ~ ., data = MASS::Boston, importance = TRUE)

# 2. Check convergence: did the forest grow enough trees?
plot(gg_error(rf))

# 3. Rank predictors by importance
plot(gg_vimp(rf))

# 4. Marginal dependence for top variables
gg_v <- gg_variable(rf)
plot(gg_v, xvar = "lstat")
plot(gg_v, xvar = rf$xvar.names, panel = TRUE, se = FALSE)

# 5. Partial dependence for a single predictor
pv <- plot.variable(rf, xvar.names = "lstat", partial = TRUE, show.plots = FALSE)
pd <- gg_partial(pv)
plot(pd)
```

For survival forests, see the package vignette:
```r
vignette("ggRandomForests")
```

For variable importance with varPro — partial dependence, importance
z-scores, beta importance, individual/local importance, and isolation
forests — see the dedicated vignette:
```r
vignette("varpro", package = "ggRandomForests")
```

## Function reference

| Function | Input | What you get |
|---|---|---|
| `gg_error()` | `rfsrc` / `randomForest` | OOB error vs. number of trees |
| `gg_vimp()` | `rfsrc` / `randomForest` | Variable importance ranking |
| `gg_rfsrc()` | `rfsrc` / `randomForest` | Predicted vs. observed values |
| `gg_variable()` | `rfsrc` / `randomForest` | Marginal dependence data frame |
| `gg_partial()` | `plot.variable` output | Partial dependence (continuous + categorical) |
| `gg_partial_rfsrc()` | `rfsrc` model | Partial dependence via `partial.rfsrc` |
| `gg_survival()` | `rfsrc` survival forest | Kaplan–Meier / Nelson–Aalen estimates |
| `gg_roc()` | `rfsrc` / `randomForest` (class) | ROC curve data |
| `gg_brier()` | `rfsrc` (survival) | Time-resolved Brier score and CRPS |

Each `gg_*` function has a matching `plot()` S3 method that hands back a single plottable object — a `ggplot`,
or a `patchwork` composite when the method lays out multiple panels — so you can keep adding layers, scales, or a theme. Every `gg_*` object also has `print()` and `summary()` methods: `print()`
shows a short header at the REPL rather than dumping every row (use `head()` when you want the rows), and
`summary()` gives you a diagnostics object you can print or keep.

## Why ggRandomForests?

The package is built on one decision: keep the data step and the figure step apart. The `gg_*`
functions pull a tidy data object out of the forest; the `plot()` methods turn that object into a
`ggplot2` figure. Two things follow from that split.

First, the data object stands on its own. It carries everything its plot needs, so you can save it,
inspect it, or come back to it later without keeping the original forest — which can be large —
in memory.

Second, you are never locked into the default figure. Because a `plot()` method returns a single
plottable object (a `ggplot`, or a `patchwork` composite for the multi-panel methods), you can add
layers, swap scales, or apply a theme; and if the default is not what you want, you can ignore it
entirely and build the figure from the tidy data yourself.

## Recent changes

See [NEWS.md](NEWS.md) for the full changelog. Highlights since v2.4.0:

- **v2.6.1** Fix factor-level assignment in `gg_partial` for categorical variables.
- **v2.6.0** New plotting functions exported; test coverage raised to 83%; removed internal dependency on `hvtiRutilities`.
- **v2.5.0** New `gg_partial_rfsrc()` computes partial dependence directly from an `rfsrc` model without a separate `plot.variable` call; supports a grouping variable via `xvar2.name`.

## References

Breiman, L. (2001). Random forests, *Machine Learning*, 45:5–32.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival, Regression and
Classification. R package version >= 3.4.0. <https://cran.r-project.org/package=randomForestSRC>

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. *R News* 7(2), 25–31.

Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random survival forests.
*Ann. Appl. Statist.* 2(3), 841–860.

Liaw A. and Wiener M. (2002). Classification and Regression by randomForest. *R News* 2(3), 18–22.

Wickham H. (2009). *ggplot2: Elegant Graphics for Data Analysis*. Springer New York.

# ggRandomForests: Visually Exploring Random Forests

<!-- badges: start -->
[![cranlogs](https://cranlogs.r-pkg.org:443/badges/ggRandomForests)](https://cranlogs.r-pkg.org:443/badges/ggRandomForests)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ggRandomForests)](https://cran.r-project.org/package=ggRandomForests)
[![R package version](https://img.shields.io/github/r-package/v/ehrlinger/ggRandomForests)](https://github.com/ehrlinger/ggRandomForests)

[![active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/badges/latest/active.svg)

[![R-CMD-check](https://github.com/ehrlinger/ggRandomForests/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/R-CMD-check.yaml)
[![lint](https://github.com/ehrlinger/ggRandomForests/actions/workflows/lint.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/lint.yaml)
[![pkgdown](https://github.com/ehrlinger/ggRandomForests/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/ehrlinger/ggRandomForests/actions/workflows/pkgdown.yaml)
[![Codecov test coverage](https://codecov.io/gh/ehrlinger/ggRandomForests/graph/badge.svg)](https://app.codecov.io/gh/ehrlinger/ggRandomForests)

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.11526.svg)](https://doi.org/10.5281/zenodo.11526)
<!-- badges: end -->

<img src="man/figures/README-overview.png" alt="ggRandomForests overview: predicted survival, variable importance, OOB error, and partial dependence" width="100%" />

`ggRandomForests` provides `ggplot2`-based diagnostic and exploration plots for random forests fit with
[randomForestSRC](https://cran.r-project.org/package=randomForestSRC) (>= 3.4.0) or
[randomForest](https://cran.r-project.org/package=randomForest).
It keeps the data step apart from the figure step, so you can inspect, save, or reuse the tidy object on its own.
Listed in the [ggplot2 extensions gallery](https://exts.ggplot2.tidyverse.org/).

## Installation

```r
# CRAN (stable)
install.packages("ggRandomForests")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("ehrlinger/ggRandomForests")
```

## Quick start

```r
library(randomForestSRC)
library(ggRandomForests)

# 1. Fit a forest (regression)
rf <- rfsrc(medv ~ ., data = MASS::Boston, importance = TRUE)

# 2. Check convergence: did the forest grow enough trees?
plot(gg_error(rf))

# 3. Rank predictors by importance
plot(gg_vimp(rf))

# 4. Marginal dependence for top variables
gg_v <- gg_variable(rf)
plot(gg_v, xvar = "lstat")
plot(gg_v, xvar = rf$xvar.names, panel = TRUE, se = FALSE)

# 5. Partial dependence for a single predictor
pv <- plot.variable(rf, xvar.names = "lstat", partial = TRUE, show.plots = FALSE)
pd <- gg_partial(pv)
plot(pd)
```

For survival forests, see the package vignette:
```r
vignette("ggRandomForests")
```

For variable importance with varPro — partial dependence, importance
z-scores, beta importance, individual/local importance, and isolation
forests — see the dedicated vignette:
```r
vignette("varpro", package = "ggRandomForests")
```

The unsupervised varPro tools — `gg_udependent()`, `gg_beta_uvarpro()`, and
`gg_sdependent()`, which read structure off a `uvarpro()` fit with no outcome —
have their own short vignette:
```r
vignette("uvarpro", package = "ggRandomForests")
```

## Function reference

| Function | Input | What you get |
|---|---|---|
| `gg_error()` | `rfsrc` / `randomForest` | OOB error vs. number of trees |
| `gg_vimp()` | `rfsrc` / `randomForest` | Variable importance ranking |
| `gg_rfsrc()` | `rfsrc` / `randomForest` | Predicted vs. observed values |
| `gg_variable()` | `rfsrc` / `randomForest` | Marginal dependence data frame |
| `gg_partial()` | `plot.variable` output | Partial dependence (continuous + categorical) |
| `gg_partial_rfsrc()` | `rfsrc` model | Partial dependence via `partial.rfsrc` |
| `gg_survival()` | `rfsrc` survival forest | Kaplan–Meier / Nelson–Aalen estimates |
| `gg_roc()` | `rfsrc` / `randomForest` (class) | ROC curve data |
| `gg_brier()` | `rfsrc` (survival) | Time-resolved Brier score and CRPS |

Each `gg_*` function has a matching `plot()` S3 method that hands back a single plottable object: a `ggplot`
you extend with `+`, or a `patchwork` composite for the multi-panel methods. Every `gg_*` object also has `print()` and `summary()` methods: `print()`
shows a short header at the REPL rather than dumping every row (use `head()` when you want the rows), and
`summary()` gives you a diagnostics object you can print or keep.

## Why ggRandomForests?

The package is built on one decision: keep the data step and the figure step apart. The `gg_*`
functions pull a tidy data object out of the forest; the `plot()` methods turn that object into a
`ggplot2` figure. Two things follow from that split.

First, the data object stands on its own. It carries everything its plot needs, so you can save it,
inspect it, or come back to it later without keeping the original forest — which can be large —
in memory.

Second, you are never locked into the default figure. Because a `plot()` method returns a single
plottable object (a `ggplot`, or a `patchwork` composite for the multi-panel methods), you can add
layers, swap scales, or apply a theme; and if the default is not what you want, you can ignore it
entirely and build the figure from the tidy data yourself.

## Recent changes

See [NEWS.md](NEWS.md) for the full changelog. Recent highlights:

- **v3.4.0** Unsupervised varPro wrappers (`gg_beta_uvarpro()`, `gg_sdependent()`) with their own vignette; `gg_partial_rfsrc()` now handles factor predictors correctly.
- **v3.3.0** varPro partial plots default to interpretable scales — probability for classification, survival S(&tau;) for survival.
- **v3.1.0** varPro integration: release-rule importance, partial dependence, local importance, anomaly scores, and the dependency graph.

## References

Breiman, L. (2001). Random forests, *Machine Learning*, 45:5–32.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival, Regression and
Classification. R package version >= 3.4.0. <https://cran.r-project.org/package=randomForestSRC>

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. *R News* 7(2), 25–31.

Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random survival forests.
*Ann. Appl. Statist.* 2(3), 841–860.

Liaw A. and Wiener M. (2002). Classification and Regression by randomForest. *R News* 2(3), 18–22.

Wickham H. (2009). *ggplot2: Elegant Graphics for Data Analysis*. Springer New York.

## ----setup, include=FALSE-----------------------------------------------------
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
knitr::opts_chunk$set(
  message    = FALSE,
  warning    = FALSE,
  fig.width  = 7,
  fig.height = 4.5,
  cache      = TRUE,
  cache.path = "uvarpro_cache/"
)
options(mc.cores = 1, rf.cores = 1)


## ----libs---------------------------------------------------------------------
library(ggplot2)

# Match the pattern used by the other vignettes: try the installed
# package first, fall back to pkgload::load_all() for the R CMD check
# vignette rebuild where the package isn't yet on .libPaths(). All varPro
# calls below are ::-qualified, so no library(varPro) is needed.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}


## ----fit----------------------------------------------------------------------
set.seed(1)
u <- varPro::uvarpro(mtcars, ntree = 50)


## ----beta_fit-----------------------------------------------------------------
beta_fit <- varPro::get.beta.entropy(u)


## ----udep, eval = requireNamespace("ggraph", quietly = TRUE)------------------
plot(gg_udependent(u))


## ----beta---------------------------------------------------------------------
plot(gg_beta_uvarpro(u, beta_fit = beta_fit))


## ----sdep---------------------------------------------------------------------
plot(gg_sdependent(u, beta_fit = beta_fit))

## ----setup, include=FALSE-----------------------------------------------------
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)


## ----packages-----------------------------------------------------------------
library(ggplot2)
library(dplyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())


## ----data-load----------------------------------------------------------------
data(iris)


## ----eda-petal, fig.height=4, fig.cap="Petal length vs. petal width, colored by species."----
ggplot(iris, aes(x = Petal.Length, y = Petal.Width, color = Species)) +
  geom_point(size = 2, alpha = 0.7) +
  scale_color_brewer(palette = "Set2")


## ----eda-sepal, fig.height=4, fig.cap="Sepal length vs. sepal width, colored by species."----
ggplot(iris, aes(x = Sepal.Length, y = Sepal.Width, color = Species)) +
  geom_point(size = 2, alpha = 0.7) +
  scale_color_brewer(palette = "Set2")


## ----rfsrc-fit----------------------------------------------------------------
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris,
                    ntree = 200, importance = TRUE, err.block = 5)
rfsrc_iris


## ----error-plot, fig.height=4, fig.cap="OOB misclassification rate vs. number of trees, overall and per class."----
plot(gg_error(rfsrc_iris))


## ----rfsrc-predicted, fig.height=5, fig.cap="OOB predicted class probabilities, faceted by the flower's true species."----
plot(gg_rfsrc(rfsrc_iris))


## ----vimp-plot, fig.cap="VIMP ranking, overall and per class."----------------
plot(gg_vimp(rfsrc_iris))


## ----varsel-------------------------------------------------------------------
md_iris <- max.subtree(rfsrc_iris)


## ----shap-fit-----------------------------------------------------------------
set.seed(43)
gg_shp <- gg_shap(rfsrc_iris, bg_n = 50, which.class = 3)


## ----shap-importance-plot, fig.cap="Mean absolute SHAP value per predictor, virginica probability."----
plot(gg_shp, type = "importance")


## ----shap-beeswarm-plot, fig.cap="SHAP beeswarm for virginica probability: every dot is one flower's contribution for one predictor."----
plot(gg_shp, type = "beeswarm")


## ----shap-dependence-plot, fig.cap="SHAP dependence for Petal.Width, virginica probability."----
plot(gg_shp, type = "dependence", xvar = "Petal.Width")


## ----vardep-panel, fig.cap="Variable dependence for Petal.Width, faceted by predicted-class probability."----
gg_v <- gg_variable(rfsrc_iris)

plot(gg_v, xvar = "Petal.Width")


## ----partial-dep, fig.cap="Partial dependence for petal measurements, setosa probability."----
pd <- gg_partial_rfsrc(rfsrc_iris, xvar.names = c("Petal.Length", "Petal.Width"))
plot(pd)


## ----roc-plot, fig.height=4, fig.cap="ROC curve for virginica vs. the rest."----
roc_virginica <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(roc_virginica)


## ----auc-calc-----------------------------------------------------------------
calc_auc(roc_virginica)

## ----setup, include=FALSE-----------------------------------------------------
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
knitr::opts_chunk$set(
  message    = FALSE,
  warning    = FALSE,
  fig.width  = 7,
  fig.height = 4.5,
  cache      = TRUE,
  cache.path = "varpro_cache/"
)
options(mc.cores = 1, rf.cores = 1)


## ----libs---------------------------------------------------------------------
library(ggplot2)

# Match the pattern used by the regression/survival vignettes: try the
# installed package first, fall back to pkgload::load_all() for the
# R CMD check vignette rebuild where the package isn't yet on
# .libPaths(). All varPro calls below are ::-qualified, so no
# library(varPro) is needed.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}


## ----precomputed, include=FALSE, cache=FALSE----------------------------------
# The three gg_partial_varpro() calls and the Boston beta.varpro() fit are
# the only expensive parts of this vignette (~34s combined). They are
# precomputed by precompute_varpro.R and loaded here so the R CMD check
# vignette rebuild stays fast. If the file is absent (e.g. a fresh clone
# that hasn't run the script), each chunk falls back to live computation,
# so the vignette is reproducible either way.
.vp <- tryCatch({
  # R CMD check rebuilds the vignette in a copy whose working directory is the
  # vignette dir; an interactive knit from the package root sees vignettes/
  # instead. Try both locations, and fall back to live computation below if
  # neither is present or readRDS() fails.
  .cand <- c("varpro_precomputed.rds",
             file.path("vignettes", "varpro_precomputed.rds"))
  .hit <- .cand[file.exists(.cand)]
  if (length(.hit)) readRDS(.hit[[1]]) else list()
}, error = function(e) list())   # missing or unreadable -> live fallback below


## ----boston-fit---------------------------------------------------------------
data("Boston", package = "MASS")
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_boston <- if (is.null(.vp$v_boston)) {
  varPro::varpro(medv ~ ., data = Boston, ntree = 50)
} else {
  .vp$v_boston
}
# A varpro fit is a large list; a bare `v_boston` would render every
# component, including the full per-rule `$results` table. Show its
# structure instead: one line per component.
str(v_boston, max.level = 1)


## ----boston-gg-varpro---------------------------------------------------------
plot(gg_varpro(v_boston))


## ----boston-gg-partial-varpro-------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd <- if (is.null(.vp$pd_boston)) {
  gg_partial_varpro(object = v_boston)
} else {
  .vp$pd_boston
}
plot(gg_pd)


## ----boston-beta-fit, cache=TRUE----------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_boston <- if (is.null(.vp$b_boston)) {
  varPro::beta.varpro(v_boston)
} else {
  .vp$b_boston
}


## ----boston-gg-beta-varpro----------------------------------------------------
plot(gg_beta_varpro(v_boston, beta_fit = b_boston))


## ----boston-isopro, cache=TRUE------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_boston <- if (is.null(.vp$iso_boston)) {
  varPro::isopro(data = Boston[, setdiff(names(Boston), "medv")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_boston
}


## ----boston-gg-isopro---------------------------------------------------------
plot(gg_isopro(iso_boston))


## ----boston-ivarpro-fit, cache=TRUE-------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iv_boston <- if (is.null(.vp$iv_boston)) {
  varPro::ivarpro(v_boston)
} else {
  .vp$iv_boston
}


## ----boston-gg-ivarpro-distribution-------------------------------------------
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston))


## ----boston-gg-ivarpro-which-obs----------------------------------------------
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston, which_obs = 1L))


## ----iris-fit-binary----------------------------------------------------------
iris_binary <- iris[iris$Species != "setosa", ]
iris_binary$Species <- droplevels(iris_binary$Species)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_binary <- if (is.null(.vp$v_iris_binary)) {
  varPro::varpro(Species ~ ., data = iris_binary, ntree = 50)
} else {
  .vp$v_iris_binary
}


## ----iris-fit-multi-----------------------------------------------------------
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_multi <- if (is.null(.vp$v_iris_multi)) {
  varPro::varpro(Species ~ ., data = iris, ntree = 50)
} else {
  .vp$v_iris_multi
}


## ----iris-multi-gg-varpro-conditional-----------------------------------------
plot(gg_varpro(v_iris_multi, conditional = TRUE))


## ----iris-multi-gg-partial-varpro---------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_iris <- if (is.null(.vp$pd_iris_multi)) {
  gg_partial_varpro(object = v_iris_multi)
} else {
  .vp$pd_iris_multi
}
plot(gg_pd_iris)


## ----iris-binary-beta-fit, cache=TRUE-----------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_binary <- if (is.null(.vp$b_iris_binary)) {
  varPro::beta.varpro(v_iris_binary)
} else {
  .vp$b_iris_binary
}


## ----iris-binary-gg-beta-varpro-----------------------------------------------
plot(gg_beta_varpro(v_iris_binary, beta_fit = b_iris_binary))


## ----iris-multi-beta-fit, cache=TRUE------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_multi <- if (is.null(.vp$b_iris_multi)) {
  varPro::beta.varpro(v_iris_multi)
} else {
  .vp$b_iris_multi
}


## ----iris-multi-gg-beta-varpro------------------------------------------------
plot(gg_beta_varpro(v_iris_multi, beta_fit = b_iris_multi))


## ----pbc-data-----------------------------------------------------------------
library(survival)
data(pbc, package = "randomForestSRC")
pbc_small <- pbc[, c("days", "status", "age", "albumin", "bili",
                     "edema", "platelet")]
pbc_small <- na.omit(pbc_small)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_pbc <- if (is.null(.vp$v_pbc)) {
  varPro::varpro(Surv(days, status) ~ ., data = pbc_small, ntree = 50)
} else {
  .vp$v_pbc
}


## ----pbc-gg-varpro------------------------------------------------------------
plot(gg_varpro(v_pbc))


## ----pbc-gg-partial-varpro----------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_pbc <- if (is.null(.vp$pd_pbc)) {
  gg_partial_varpro(object = v_pbc)
} else {
  .vp$pd_pbc
}
plot(gg_pd_pbc)


## ----pbc-isopro, cache=TRUE---------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_pbc <- if (is.null(.vp$iso_pbc)) {
  varPro::isopro(data = pbc_small[, c("age", "albumin", "bili", "platelet")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_pbc
}
plot(gg_isopro(iso_pbc))


## ----varpro-counts------------------------------------------------------------
c(handed_in = ncol(v_boston$x),
  reachable = length(v_boston$xvar.names),
  reported  = length(varPro::get.topvars(v_boston)))


## ----topvars-widen, eval=FALSE------------------------------------------------
# varPro::partialpro(v_boston, xvar.names = c("lstat", "age", "zn"))


## ----varpro-counts-iris-------------------------------------------------------
c(handed_in = ncol(v_iris_binary$x),
  reachable = length(v_iris_binary$xvar.names))


## ----partialpro-check, eval=FALSE---------------------------------------------
# pd <- varPro::partialpro(v_boston, xvar.names = my_vars)
# setdiff(my_vars, names(pd))   # anything here was dropped


## ----both-off, eval=FALSE-----------------------------------------------------
# varPro::varpro(medv ~ ., data = Boston, sparse = FALSE, split.weight = FALSE)

##=============================================================================
#' Split partial dependence data into continuous or categorical datasets
#'
#' Takes the list returned by \code{rfsrc::plot.variable(partial = TRUE)} and
#' separates the variables into two data frames: one for continuous predictors
#' and one for categorical (factor-like) predictors.  The split is controlled
#' by \code{cat_limit}: variables with more unique x-values than this threshold
#' are treated as continuous; all others are categorical.
#'
#' @param part_dta partial plot data from \code{rfsrc::plot.variable}
#' @param nvars how many of the partial plot variables to calculate
#' @param cat_limit Categorical features are built when there are fewer than
#'  \code{cat_limit} unique feature values.
#' @param model a label name applied to all features. Useful when combining
#'  multiple partial plot objects in figures.
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{continuous}{data.frame with columns \code{x}, \code{yhat},
#'       \code{name} (and optionally \code{model}) for continuous variables}
#'     \item{categorical}{data.frame with the same columns but with \code{x}
#'       as a factor, for low-cardinality / categorical variables}
#'   }
#'
#' @seealso \code{\link{gg_partial_rfsrc}} \code{\link{gg_partialpro}}
#'
#' @examples
#' ## Build a small regression forest on the airquality dataset
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- rfsrc(Ozone ~ ., data = airq, ntree = 50)
#'
#' ## Compute partial dependence via plot.variable (show.plots = FALSE to
#' ## suppress the base-graphics output — we only want the data)
#' pv <- randomForestSRC::plot.variable(rf, partial = TRUE,
#'                                       show.plots = FALSE)
#'
#' ## Split into continuous and categorical data frames
#' result <- gg_partial(pv)
#' head(result$continuous)
#'
#' ## Label this model for later comparison with a second forest
#' result_labelled <- gg_partial(pv, model = "airq_model")
#' unique(result_labelled$continuous$model)
#'
#' @export
gg_partial <- function(part_dta,
                       nvars = NULL,
                       cat_limit = 10,
                       model = NULL) {
  ## Default: process all variables returned by plot.variable
  if (is.null(nvars)) {
    nvars <- length(part_dta$plotthis)
  }

  # Accumulate per-variable data frames before binding
  cont_list <- list()
  cat_list <- list()

  for (feature in seq(nvars)) {
    x_vals <- part_dta$plotthis[[feature]]$x

    ## ---- Continuous variable: more unique x values than cat_limit -------
    if (length(unique(x_vals)) > cat_limit) {
      plt.df <- dplyr::bind_cols(
        x    = x_vals,
        yhat = part_dta$plotthis[[feature]]$yhat
      )
      # Tag each row with the variable name for downstream faceting
      plt.df$name <- names(part_dta$plotthis)[[feature]]

      cont_list[[feature]] <- plt.df

    } else {
      ## ---- Categorical variable: few unique x values -------------------
      ## VarPro works with logical or continuous only; factors are
      ## one-hot encoded internally in the varPro call.
      ## Normalize to character so bind_rows sees a consistent type; we'll
      ## re-factor within each feature after stacking.
      x_chr <- as.character(x_vals)

      plt.df <- dplyr::bind_cols(
        x    = x_chr,
        yhat = part_dta$plotthis[[feature]]$yhat
      )
      plt.df$name <- names(part_dta$plotthis)[[feature]]

      cat_list[[feature]] <- plt.df
    }
  }

  # Combine per-variable lists into single data frames (NULL entries dropped)
  continuous  <- dplyr::bind_rows(cont_list)
  if (length(cat_list) == 0) {
    categorical <- data.frame(x = character(0), yhat = numeric(0), name = character(0))
  } else {
    categorical <- dplyr::bind_rows(cat_list)
    # Set within-group factor levels to order-of-appearance (base-R, no .data pronoun)
    split_grps <- split(seq_len(nrow(categorical)), categorical$name)
    for (grp_idx in split_grps) {
      vals <- categorical$x[grp_idx]
      categorical$x[grp_idx] <- as.character(factor(vals, levels = unique(vals)))
    }
    categorical$x <- factor(categorical$x)
  }
  ## Optionally attach a model label (useful when overlaying multiple forests)
  if (!is.null(model)) {
    continuous$model <- model
    if (!is.null(categorical) && nrow(categorical) > 0) {
      categorical$model <- model
    }
  }

  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- "gg_partial"
  return(result)
}

# Tests for gg_partial and gg_partial_rfsrc

# Helper: create mock partial plot data (matching rfsrc::plot.variable structure)
make_mock_partial_data <- function() {
  set.seed(42)
  # Wind: continuous (15 unique values > cat_limit of 10)
  wind_x <- seq(1.7, 20.7, length.out = 15)
  # Month: categorical (5 unique values < cat_limit of 10)
  month_x <- rep(c(5, 6, 7, 8, 9), 3)

  list(
    plotthis = list(
      Wind = list(
        x = wind_x,
        yhat = rnorm(15, mean = 50, sd = 10)
      ),
      Month = list(
        x = month_x,
        yhat = rnorm(15, mean = 50, sd = 10)
      )
    )
  )
}

# ---- gg_partial unit tests -----------------------------------------------

test_that("gg_partial returns a gg_partial object with continuous and categorical", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  expect_s3_class(result, "gg_partial")
  expect_named(result, c("continuous", "categorical"))
  expect_s3_class(result$continuous, "data.frame")
  expect_s3_class(result$categorical, "data.frame")
})

test_that("gg_partial correctly separates continuous and categorical", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  # Wind has 15 unique values > cat_limit=10, so it is continuous
  expect_true("Wind" %in% result$continuous$name)
  expect_false("Wind" %in% result$categorical$name)

  # Month has 5 unique values <= cat_limit=10, so it is categorical
  expect_true("Month" %in% result$categorical$name)
  expect_false("Month" %in% result$continuous$name)
})

test_that("gg_partial continuous output has correct columns", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  expect_true(all(c("x", "yhat", "name") %in% colnames(result$continuous)))
})

test_that("gg_partial categorical output has correct columns", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  expect_true(all(c("x", "yhat", "name") %in% colnames(result$categorical)))
  # Categorical x should be a factor
  expect_s3_class(result$categorical$x, "factor")
})

test_that("gg_partial respects nvars argument", {
  mock_dta <- make_mock_partial_data()

  # nvars=1 processes only Wind (first variable, continuous)
  result <- gg_partial(mock_dta, nvars = 1)

  expect_true("Wind" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

test_that("gg_partial adds model column when model argument is provided", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta, model = "my_model")

  expect_true("model" %in% colnames(result$continuous))
  expect_true("model" %in% colnames(result$categorical))
  expect_equal(unique(result$continuous$model), "my_model")
  expect_equal(unique(result$categorical$model), "my_model")
})

test_that("gg_partial without model argument has no model column", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  expect_false("model" %in% colnames(result$continuous))
  expect_false("model" %in% colnames(result$categorical))
})

test_that("gg_partial with high cat_limit classifies Wind as categorical", {
  mock_dta <- make_mock_partial_data()
  # cat_limit=100 means all variables become categorical
  result <- gg_partial(mock_dta, cat_limit = 100)

  expect_equal(nrow(result$continuous), 0)
  expect_true("Wind" %in% result$categorical$name)
  expect_true("Month" %in% result$categorical$name)
})

test_that("gg_partial with low cat_limit classifies Month as continuous", {
  mock_dta <- make_mock_partial_data()
  # cat_limit=1 means Month (5 unique) is also continuous
  result <- gg_partial(mock_dta, cat_limit = 1)

  expect_true("Wind" %in% result$continuous$name)
  expect_true("Month" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

# ---- gg_partial_rfsrc integration tests -----------------------------------

test_that("gg_partial_rfsrc regression returns a gg_partial_rfsrc object", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  result <- gg_partial_rfsrc(rf, xvar.names = c("Wind"))

  expect_s3_class(result, "gg_partial_rfsrc")
  expect_named(result, c("continuous", "categorical"))
  expect_s3_class(result$continuous, "data.frame")
  expect_s3_class(result$categorical, "data.frame")
})

test_that("plot.gg_partial returns a ggplot for continuous-only data", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta, nvars = 1)   # only Wind (continuous)

  gg_plt <- plot(result)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_partial returns a single ggplot when both types present", {
  mock_dta <- make_mock_partial_data()
  result <- gg_partial(mock_dta)

  out <- plot(result)
  # patchwork combines the two panels into a single ggplot-inheriting object.
  expect_s3_class(out, "ggplot")
})

test_that("plot.gg_partial_rfsrc returns a ggplot", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)
  result <- gg_partial_rfsrc(rf, xvar.names = "Wind")

  gg_plt <- plot(result)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_partial_rfsrc Wind is continuous in airquality model", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  result <- gg_partial_rfsrc(rf, xvar.names = c("Wind"))

  # Wind has many unique values, should land in continuous
  expect_gt(nrow(result$continuous), 0)
  expect_true("Wind" %in% result$continuous$name)
})

test_that("gg_partial_rfsrc error on invalid xvar.names", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  expect_error(
    gg_partial_rfsrc(rf, xvar.names = c("NotAColumn")),
    "xvar.names contains column names not found"
  )
})

test_that("gg_partial_rfsrc error on invalid newx columns", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  bad_newx <- data.frame(foo = 1:10, bar = 1:10)
  expect_error(
    gg_partial_rfsrc(rf, xvar.names = c("Wind"), newx = bad_newx),
    "newx must be a dataframe"
  )
})

test_that("gg_partial_rfsrc uses supplied newx data", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  newx <- rf$xvar[1:20, ]
  result <- gg_partial_rfsrc(rf, xvar.names = c("Wind"), newx = newx)

  expect_type(result, "list")
  expect_named(result, c("continuous", "categorical"))
})

test_that("gg_partial_rfsrc works with multiple xvar.names", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  result <- gg_partial_rfsrc(rf, xvar.names = c("Wind", "Temp"))

  expect_gt(nrow(result$continuous), 0)
})

test_that("gg_partial_rfsrc with xvar2.name returns grouped result", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  result <- gg_partial_rfsrc(rf,
    xvar.names = c("Wind"),
    xvar2.name = "Month"
  )

  expect_type(result, "list")
  expect_named(result, c("continuous", "categorical"))
  # grouped output should have a grp column
  if (nrow(result$continuous) > 0) {
    expect_true("grp" %in% colnames(result$continuous))
  }
})

# ---- n_eval and cat_limit ---------------------------------------------------

test_that("n_eval limits evaluation grid size for continuous variables", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf   <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  # Wind has 31 unique values in the full airquality dataset; n_eval = 8
  # should produce at most 8 evaluation points in the output
  result <- gg_partial_rfsrc(rf, xvar.names = "Wind", n_eval = 8)

  expect_gt(nrow(result$continuous), 0)
  expect_lte(nrow(result$continuous), 8)
})

test_that("default n_eval = 25 caps large continuous grids", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf   <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  result <- gg_partial_rfsrc(rf, xvar.names = "Wind")  # default n_eval = 25

  expect_lte(nrow(result$continuous), 25)
})

test_that("cat_limit controls continuous vs categorical dispatch", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf   <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50, nsplit = 5)

  # Month has 5 unique values.
  # With default cat_limit = 10: Month → categorical
  result_cat <- gg_partial_rfsrc(rf, xvar.names = "Month", cat_limit = 10)
  expect_gt(nrow(result_cat$categorical), 0)
  expect_true("Month" %in% result_cat$categorical$name)

  # With cat_limit = 3: Month (5 unique values >= 3) → continuous
  result_cont <- gg_partial_rfsrc(rf, xvar.names = "Month", cat_limit = 3)
  expect_gt(nrow(result_cont$continuous), 0)
  expect_true("Month" %in% result_cont$continuous$name)
})

# ---- survival forest path ---------------------------------------------------

# Helper: fit a small survival forest on veteran data (n = 137, fast to build)
make_veteran_rf <- function() {
  skip_if_not_installed("randomForestSRC")
  skip_if_not_installed("survival")
  veteran <- survival::veteran        # direct access avoids .GlobalEnv loading issue
  Surv    <- survival::Surv           # parseFormula rejects namespace::Surv(...) syntax # nolint: object_name_linter
  set.seed(42)
  randomForestSRC::rfsrc(
    Surv(time, status) ~ trt + karno + diagtime + age + prior,
    data   = veteran,
    ntree  = 100,
    nsplit = 5
  )
}

test_that("survival forest is detected as is_surv = TRUE", {
  rf <- make_veteran_rf()
  expect_true(grepl("surv", rf$family, ignore.case = TRUE))
  expect_false(is.null(rf$time.interest))
  expect_gt(length(rf$time.interest), 0)
})

test_that("partial.time values are snapped to time.interest grid", {
  rf <- make_veteran_rf()
  ti <- rf$time.interest

  # Pick a target time that almost certainly is not exactly in time.interest
  target <- mean(range(ti))  # midpoint of observed event times
  snapped <- ti[which.min(abs(ti - target))]

  # snapped must be an exact member of time.interest
  expect_true(snapped %in% ti)
  # and it should be closer to target than any other grid point
  diffs <- abs(ti - target)
  expect_equal(snapped, ti[which.min(diffs)])
})

test_that("gg_partial_rfsrc survival: default partial.time uses quartiles", {
  rf <- make_veteran_rf()
  ti <- rf$time.interest

  # When partial.time = NULL, three quartile-snapped times are used.
  # We can verify this by inspecting how many distinct time values appear
  # in the output (should be 3 unless quartile snapping collapses duplicates).
  result <- tryCatch(
    gg_partial_rfsrc(rf, xvar.names = "karno"),
    error = function(e) {
      skip(paste(
        "partial.rfsrc() failed for survival forest (upstream bug):",
        conditionMessage(e)
      ))
    }
  )

  expect_type(result, "list")
  expect_named(result, c("continuous", "categorical"))
  expect_gt(nrow(result$continuous), 0)
  # time column should be present in survival output
  expect_true("time" %in% colnames(result$continuous))
  # at most 3 distinct time values (one per quartile)
  expect_lte(length(unique(result$continuous$time)), 3)
})

test_that("gg_partial_rfsrc survival: explicit partial.time is snapped and used", {
  rf <- make_veteran_rf()
  ti <- rf$time.interest
  # Target the median event time
  t_med <- ti[which.min(abs(ti - median(ti)))]

  result <- tryCatch(
    gg_partial_rfsrc(rf, xvar.names = "karno", partial.time = t_med),
    error = function(e) {
      skip(paste(
        "partial.rfsrc() failed for survival forest (upstream bug):",
        conditionMessage(e)
      ))
    }
  )

  expect_type(result, "list")
  expect_gt(nrow(result$continuous), 0)
  expect_true("time" %in% colnames(result$continuous))
  # The only time value in the output should be (close to) t_med
  expect_true(all(abs(result$continuous$time - t_med) < 1e-9))
})

test_that("gg_partial_rfsrc survival: multiple partial.time values produce one row-set per time", {
  rf <- make_veteran_rf()
  ti <- rf$time.interest
  t1 <- ti[which.min(abs(ti - quantile(ti, 0.25)))]
  t2 <- ti[which.min(abs(ti - quantile(ti, 0.75)))]
  # Ensure we actually have two distinct snapped times
  if (t1 == t2) skip("quartile times collapsed to same grid point")

  result <- tryCatch(
    gg_partial_rfsrc(rf, xvar.names = "karno", partial.time = c(t1, t2)),
    error = function(e) {
      skip(paste(
        "partial.rfsrc() failed for survival forest (upstream bug):",
        conditionMessage(e)
      ))
    }
  )

  expect_type(result, "list")
  n_times <- length(unique(result$continuous$time))
  expect_equal(n_times, 2L)
})

test_that("gg_partial_rfsrc survival: returns correct column names", {
  rf <- make_veteran_rf()
  ti <- rf$time.interest
  t_med <- ti[which.min(abs(ti - median(ti)))]

  result <- tryCatch(
    gg_partial_rfsrc(rf, xvar.names = "karno", partial.time = t_med),
    error = function(e) {
      skip(paste(
        "partial.rfsrc() failed for survival forest (upstream bug):",
        conditionMessage(e)
      ))
    }
  )

  expect_true(all(c("x", "yhat", "name", "time") %in% colnames(result$continuous)))
})

# Smoke tests for autoplot.gg_* methods.

setup_forests <- function() {
  data(airquality, package = "datasets")
  data(iris,       package = "datasets")
  data(pbc,        package = "randomForestSRC")
  set.seed(1)
  list(
    reg = randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                                 ntree = 50, importance = TRUE,
                                 tree.err = TRUE),
    cls = randomForestSRC::rfsrc(Species ~ ., data = iris,
                                 ntree = 50, importance = TRUE,
                                 tree.err = TRUE),
    srv = randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
                                 ntree = 50, nsplit = 10,
                                 importance = TRUE, tree.err = TRUE)
  )
}

test_that("autoplot returns a ggplot for gg_error", {
  f <- setup_forests()
  expect_s3_class(ggplot2::autoplot(gg_error(f$reg)), "ggplot")
})

test_that("autoplot returns a ggplot for gg_vimp", {
  f <- setup_forests()
  expect_s3_class(ggplot2::autoplot(gg_vimp(f$reg)), "ggplot")
})

test_that("autoplot returns a ggplot for gg_rfsrc", {
  f <- setup_forests()
  expect_s3_class(ggplot2::autoplot(gg_rfsrc(f$reg)), "ggplot")
})

test_that("autoplot returns a ggplot for gg_variable", {
  f <- setup_forests()
  expect_s3_class(
    ggplot2::autoplot(gg_variable(f$reg), xvar = "Temp"),
    "ggplot"
  )
})

test_that("autoplot returns a ggplot for gg_partial", {
  f <- setup_forests()
  pv <- randomForestSRC::plot.variable(f$reg, partial = TRUE,
                                        show.plots = FALSE)
  expect_s3_class(ggplot2::autoplot(gg_partial(pv)), "ggplot")
})

test_that("autoplot returns a ggplot for gg_partial_rfsrc", {
  f <- setup_forests()
  expect_s3_class(
    ggplot2::autoplot(gg_partial_rfsrc(f$reg, xvar.names = "Temp")),
    "ggplot"
  )
})

test_that("autoplot returns a ggplot for gg_roc", {
  f <- setup_forests()
  expect_s3_class(
    ggplot2::autoplot(gg_roc(f$cls, which_outcome = 1)),
    "ggplot"
  )
})

test_that("autoplot returns a ggplot for gg_survival", {
  data(pbc, package = "randomForestSRC")
  expect_s3_class(
    ggplot2::autoplot(
      gg_survival(interval = "days", censor = "status", data = pbc)
    ),
    "ggplot"
  )
})

test_that("autoplot returns a ggplot for gg_brier", {
  f <- setup_forests()
  expect_s3_class(ggplot2::autoplot(gg_brier(f$srv)), "ggplot")
})

# Smoke tests for print.gg_* / summary.gg_* methods.
# We fit one forest per family and exercise every method's happy path.

setup_forests <- function() {
  data(iris,     package = "datasets")
  data(airquality, package = "datasets")
  data(pbc,      package = "randomForestSRC")
  set.seed(1)
  list(
    cls = randomForestSRC::rfsrc(Species ~ ., data = iris,
                                 ntree = 50, importance = TRUE,
                                 tree.err = TRUE),
    reg = randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                                 ntree = 50, na.action = "na.impute",
                                 importance = TRUE, tree.err = TRUE),
    srv = randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
                                 ntree = 50, nsplit = 10,
                                 importance = TRUE, tree.err = TRUE)
  )
}

test_that("provenance attribute is attached by every constructor", {
  f <- setup_forests()
  expect_equal(attr(gg_error(f$cls),    "provenance")$source, "randomForestSRC")
  expect_equal(attr(gg_vimp(f$reg),     "provenance")$source, "randomForestSRC")
  expect_equal(attr(gg_rfsrc(f$srv),    "provenance")$source, "randomForestSRC")
  expect_equal(attr(gg_variable(f$reg), "provenance")$source, "randomForestSRC")
  expect_equal(attr(gg_roc(f$cls, which_outcome = 1),
                    "provenance")$source, "randomForestSRC")
  expect_equal(attr(gg_brier(f$srv),    "provenance")$source, "randomForestSRC")
  # gg_survival from a forest object should carry provenance.
  expect_equal(attr(gg_survival(f$srv), "provenance")$source, "randomForestSRC")
  # gg_partial objects come from plot.variable output, not a forest directly;
  # they do not carry provenance — confirm the attribute is absent (not NA).
  part_dta <- randomForestSRC::plot.variable(f$reg, partial = TRUE,
                                              xvar.names = "Wind")
  gp <- gg_partial(part_dta)
  expect_null(attr(gp, "provenance"))
})

test_that("print methods return their input invisibly and emit a header", {
  f <- setup_forests()

  # Header-only contract: exactly one newline-terminated line.
  expect_print_header <- function(obj) {
    out <- capture.output(print(obj))
    expect_length(out, 1L)
    expect_true(nchar(out) > 0)
  }

  expect_print_header(gg_error(f$cls))
  expect_print_header(gg_vimp(f$cls))
  expect_print_header(gg_rfsrc(f$reg))
  expect_print_header(gg_variable(f$reg))
  expect_print_header(gg_roc(f$cls, which_outcome = 1))
  data(pbc, package = "randomForestSRC")
  expect_print_header(gg_survival(interval = "days", censor = "status",
                                  data = pbc))
  expect_print_header(gg_brier(f$srv))

  # Partial classes — header includes variable counts.
  part_dta <- randomForestSRC::plot.variable(f$reg, partial = TRUE,
                                              xvar.names = "Wind")
  expect_print_header(gg_partial(part_dta))
  expect_print_header(gg_partial_rfsrc(f$reg, xvar.names = "Wind"))
})

test_that("print.gg_partial uses 'name' column (not 'variable')", {
  f <- setup_forests()
  part_dta <- randomForestSRC::plot.variable(f$reg, partial = TRUE,
                                              xvar.names = c("Wind", "Temp"))
  gp <- gg_partial(part_dta)
  out <- capture.output(print(gp))
  # Should report 2 continuous predictors (Wind and Temp both numeric in airquality).
  expect_match(out[1], "continuous: [12]")
})

test_that("summary methods return summary.gg objects that print cleanly", {
  f <- setup_forests()

  part_dta <- randomForestSRC::plot.variable(f$reg, partial = TRUE,
                                              xvar.names = "Wind")

  for (obj in list(
    gg_error(f$cls),
    gg_vimp(f$cls),
    gg_rfsrc(f$reg),
    gg_variable(f$reg),
    gg_roc(f$cls, which_outcome = 1),
    {
      data(pbc, package = "randomForestSRC", envir = environment())
      gg_survival(interval = "days", censor = "status", data = pbc)
    },
    gg_brier(f$srv),
    gg_partial(part_dta),
    gg_partial_rfsrc(f$reg, xvar.names = "Wind")
  )) {
    s <- summary(obj)
    expect_s3_class(s, "summary.gg")
    out <- capture.output(print(s))
    expect_true(length(out) >= 1L)
  }
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' ROC (Receiver Operating Characteristic) curve data from a classification forest.
#'
#' Computes sensitivity (true positive rate) and specificity (1 - false positive
#' rate) across all prediction thresholds for one class of a classification
#' \code{\link[randomForestSRC]{rfsrc}} or
#' \code{\link[randomForest]{randomForest}} object.
#'
#' @param object A classification \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object. Only forests with
#'   \code{family == "class"} (rfsrc) or \code{type == "classification"}
#'   (randomForest) are supported.
#' @param which_outcome Integer index or character name of the class for which
#'   the ROC curve is computed. For binary forests this is typically \code{1}
#'   or \code{2}; for multi-class forests any valid class index. Use
#'   \code{which_outcome = 0} to obtain the overall (averaged) ROC.
#' @param oob Logical; if \code{TRUE} (default) use out-of-bag predicted
#'   probabilities for the curve. Set to \code{FALSE} to use full in-bag
#'   predictions.
#' @param ... Extra arguments (currently unused).
#'
#' @return A \code{gg_roc} \code{data.frame} with one row per unique prediction
#'   threshold and columns:
#'   \describe{
#'     \item{sens}{Sensitivity (true positive rate) at each threshold.}
#'     \item{spec}{Specificity (true negative rate) at each threshold.}
#'     \item{yvar}{The observed class label for each observation.}
#'   }
#'   Pass to \code{\link{calc_auc}} for the area under the curve.
#'
#' @seealso \code{\link{plot.gg_roc}}, \code{\link{calc_roc}},
#'   \code{\link{calc_auc}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForest]{randomForest}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris)
#'
#' # ROC for setosa
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' ## -------- iris data
#' rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
#'
#' # ROC for setosa
#' gg_dta <- gg_roc(rf_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor
#' gg_dta <- gg_roc(rf_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica
#' gg_dta <- gg_roc(rf_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' @aliases gg_roc gg_roc.rfsrc gg_roc.randomForest

#' @export
gg_roc.rfsrc <- function(object, which_outcome, oob = TRUE, ...) {
  # Validate that the object was grown with randomForestSRC (grow or predict)
  # or is a randomForest object — the two supported class signatures.
  if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &&
    sum(inherits(object, c("rfsrc", "predict"), TRUE) == c(1, 2)) != 2 &&
    !inherits(object, "randomForest")) {
    stop(
      "This function only works for objects of class '(rfsrc, grow)', ",
      "'(rfsrc, predict)', or 'randomForest'."
    )
  }
  # ROC curves only make sense for classification; reject other families early.
  if (!inherits(object, "class")) {
    stop("gg_roc only works with classification forests")
  }

  # Default to "all" so the caller can later loop over every class level;
  # the caller may pass a specific integer index to get a single-class curve.
  if (missing(which_outcome)) {
    which_outcome <- "all"
  }

  # Redundant guard: rfsrc sets family = "class" for classification forests.
  # Kept here to surface a clearer error message if the object is somehow
  # mis-labelled.
  if (object$family != "class") {
    stop("gg_roc is intended for classification forests only.")
  }

  # Delegate the threshold-sweep computation to calc_roc, passing the
  # observed response vector and the chosen outcome column index.
  gg_dta <- # nolint: object_usage_linter
    calc_roc(object,
      object$yvar,
      which_outcome = which_outcome,
      oob = oob
    )
  class(gg_dta) <- c("gg_roc", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)

  invisible(gg_dta)
}
#' @export
gg_roc <- function(object, which_outcome, oob = TRUE, ...) {
  UseMethod("gg_roc", object)
}

#' @export
gg_roc.randomForest <- function(object, which_outcome, oob, ...) {
  # Validate that the object is a genuine randomForest instance.
  if (!inherits(object, "randomForest")) {
    stop(
      "gg_roc.randomForest only works for objects of class 'randomForest'."
    )
  }

  # Default to computing the ROC curve for all outcome classes.
  if (missing(which_outcome)) {
    which_outcome <- "all"
  }

  if (!(object$type == "classification")) {
    stop("gg_roc only works with classification forests")
  }

  # For randomForest objects the response is stored in $y (not $yvar).
  gg_dta <- # nolint: object_usage_linter
    calc_roc(object,
      object$y,
      which_outcome = which_outcome
    )
  class(gg_dta) <- c("gg_roc", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)

  invisible(gg_dta)
}



#' @export
gg_roc.default <- gg_roc.rfsrc

# Visual regression tests using vdiffr.
#
# These tests generate reference SVGs on the first run and compare on
# subsequent runs. To regenerate snapshots (e.g. after an intentional
# visual change) run: vdiffr::manage_cases()
#
# All models are built with set.seed() to ensure reproducible plots.

# Guard: only register snapshot tests when explicitly opted in AND vdiffr is
# available. Set VDIFFR_RUN_TESTS=true to generate or compare visual baselines.
# This avoids failures on fresh checkouts (no _snaps/ directory) and in CI,
# and avoids a top-level skip() that would appear as an empty test in reports.
#
# To generate baselines locally:
#   1. Run Sys.setenv(VDIFFR_RUN_TESTS = "true")
#   2. Run devtools::test(filter = "snapshots")
#   3. Call testthat::snapshot_accept()
#   4. Commit tests/testthat/_snaps/ to the repo
if (requireNamespace("vdiffr", quietly = TRUE) &&
    identical(Sys.getenv("VDIFFR_RUN_TESTS"), "true")) {

## ---- Shared fixtures -------------------------------------------------------

# Classification — iris
local({
  set.seed(42L)
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp classification", {
    gg_dta <- gg_vimp(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_vimp classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error classification", {
    gg_dta <- gg_error(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_error classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_roc classification rfsrc", {
    gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1L)
    vdiffr::expect_doppelganger("gg_roc classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc classification", {
    gg_dta <- gg_rfsrc(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_rfsrc classification rfsrc", plot(gg_dta))
  })
})

# Regression — Boston housing
local({
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  set.seed(42L)
  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp regression", {
    gg_dta <- gg_vimp(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_vimp regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error regression", {
    gg_dta <- gg_error(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_error regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc regression", {
    gg_dta <- gg_rfsrc(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_rfsrc regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_variable regression single xvar", {
    gg_rfsrc_dta <- gg_rfsrc(rfsrc_boston)
    xvar <- "lstat"
    gg_dta <- gg_variable(rfsrc_boston, time = NULL)
    vdiffr::expect_doppelganger(
      "gg_variable regression lstat",
      plot(gg_dta, xvar = xvar)
    )
  })
})

# Survival — pbc
local({
  data(pbc, package = "randomForestSRC")
  pbc$time <- pbc$days / 364.25
  pbc_sub <- pbc[, c("time", "status", "treatment", "age", "bili", "albumin")]

  set.seed(42L)
  rfsrc_pbc <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = pbc_sub,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp survival", {
    gg_dta <- gg_vimp(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_vimp survival rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error survival", {
    gg_dta <- gg_error(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_error survival rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc survival no CI", {
    gg_dta <- gg_rfsrc(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_rfsrc survival no ci", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc survival with bootstrap CI", {
    set.seed(1L)
    gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = 0.95, bs_samples = 50L)
    vdiffr::expect_doppelganger("gg_rfsrc survival bootstrap ci", plot(gg_dta))
  })

  test_that("snapshot: gg_brier overall line", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival overall", plot(gg_dta))
  })

  test_that("snapshot: gg_brier with envelope ribbon", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival envelope",
                                plot(gg_dta, envelope = TRUE))
  })

  test_that("snapshot: gg_brier CRPS", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival crps",
                                plot(gg_dta, type = "crps"))
  })
})

# randomForest classification — iris
local({
  set.seed(42L)
  rf_iris <- randomForest::randomForest(Species ~ ., data = iris)

  test_that("snapshot: gg_vimp randomForest classification", {
    gg_dta <- gg_vimp(rf_iris)
    vdiffr::expect_doppelganger("gg_vimp classification rf", plot(gg_dta))
  })

  test_that("snapshot: gg_error randomForest classification", {
    gg_dta <- gg_error(rf_iris)
    vdiffr::expect_doppelganger("gg_error classification rf", plot(gg_dta))
  })

  test_that("snapshot: gg_roc randomForest classification", {
    gg_dta <- gg_roc(rf_iris, which_outcome = 1L)
    vdiffr::expect_doppelganger("gg_roc classification rf", plot(gg_dta))
  })
})

} # end CI guard

# testthat for gg_roc function

test_that("gg_roc classifications", {
  ## Load the cached forest
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_iris$family, "class")

  ## Create the correct gg_roc object
  which_outcome <- 1
  gg_dta <- gg_roc(rfsrc_iris, which_outcome)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  # Test data is correctly pulled from randomForest obect.
  unts <- sort(unique(rfsrc_iris$predicted.oob[, which_outcome]))

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  # Try test set prediction.
  gg_dta <- gg_roc(rfsrc_iris, which_outcome, oob = FALSE)
  # Try test set prediction.
  gg_plt <- plot.gg_roc(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  # Test data is correctly pulled from randomForest obect.
  unts <- sort(unique(rfsrc_iris$predicted[, which_outcome]))

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  expect_s3_class(plot.gg_roc(rfsrc_iris), "ggplot")
  expect_error(gg_roc.randomForest(rfsrc_iris))
  expect_error(gg_roc.rfsrc(rf_iris))
})

test_that("gg_roc randomForest classifications", {
  ## Load the cached forest
  rf_iris <- randomForest(Species ~ ., data = iris)

  # Test the cached forest type
  expect_s3_class(rf_iris, "randomForest")

  # Test the forest family
  expect_match(rf_iris$type, "classification")

  ## Create the correct gg_roc object
  which_outcome <- 1
  gg_dta <- gg_roc(rf_iris, which_outcome)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  # Try test set prediction.
  gg_dta <- gg_roc(rf_iris, which_outcome, oob = FALSE)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")
  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  expect_s3_class(plot.gg_roc(rf_iris), "ggplot")

  expect_error(gg_roc.rfsrc(rf_iris))
})

test_that("gg_roc default oob=TRUE works without explicit argument", {
  # Regression test: gg_roc() was crashing with
  # "argument 'oob' is missing, with no default" when oob was not supplied.
  set.seed(42)
  rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)

  # All three outcomes must work without passing oob
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 1), "gg_roc")
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 2), "gg_roc")
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 3), "gg_roc")

  # OOB default should equal oob = TRUE explicitly
  gg_default <- gg_roc(rfsrc_iris, which_outcome = 1)
  gg_explicit <- gg_roc(rfsrc_iris, which_outcome = 1, oob = TRUE)
  expect_equal(gg_default, gg_explicit)
})

test_that("gg_roc regression", {
  data(Boston, package = "MASS")
  boston <- Boston

  boston$chas <- as.logical(boston$chas)

  ## Load the cached forest
  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_boston$family, "regr")

  ## Create the correct gg_roc object
  expect_error(gg_roc(rfsrc_boston))
  expect_error(plot.gg_roc(rfsrc_boston))

})

test_that("calc_roc", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_iris$family, "class")

  gg_dta <- calc_roc.rfsrc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 1,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # Test oob=FALSE
  gg_dta <- calc_roc.rfsrc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 1,
                           oob = FALSE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
  # The second outcome.
  gg_dta <- ggRandomForests::calc_roc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 2,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
  # and the third...
  gg_dta <- calc_roc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 3,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
})

# testthat for gg_roc function

test_that("gg_roc classifications", {
  ## Load the cached forest
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_iris$family, "class")

  ## Create the correct gg_roc object
  which_outcome <- 1
  gg_dta <- gg_roc(rfsrc_iris, which_outcome)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  # Test data is correctly pulled from randomForest obect.
  unts <- sort(unique(rfsrc_iris$predicted.oob[, which_outcome]))

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  # Try test set prediction.
  gg_dta <- gg_roc(rfsrc_iris, which_outcome, oob = FALSE)
  # Try test set prediction.
  gg_plt <- plot.gg_roc(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  # Test data is correctly pulled from randomForest obect.
  unts <- sort(unique(rfsrc_iris$predicted[, which_outcome]))

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  expect_s3_class(plot.gg_roc(rfsrc_iris), "ggplot")
  expect_error(gg_roc.randomForest(rfsrc_iris))
  expect_error(gg_roc.rfsrc(rf_iris))
})

test_that("gg_roc randomForest classifications", {
  ## Load the cached forest
  rf_iris <- randomForest(Species ~ ., data = iris)

  # Test the cached forest type
  expect_s3_class(rf_iris, "randomForest")

  # Test the forest family
  expect_match(rf_iris$type, "classification")

  ## Create the correct gg_roc object
  which_outcome <- 1
  gg_dta <- gg_roc(rf_iris, which_outcome)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  # Try test set prediction.
  gg_dta <- gg_roc(rf_iris, which_outcome, oob = FALSE)

  # Test object type
  expect_s3_class(gg_dta, "gg_roc")
  # Test classification dimensions
  expect_equal(ncol(gg_dta), 3)

  ## Test plotting the gg_roc object
  gg_obj <- plot.gg_roc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_obj, "ggplot")

  expect_s3_class(plot.gg_roc(rf_iris), "ggplot")

  expect_error(gg_roc.rfsrc(rf_iris))
})

test_that("gg_roc default oob=TRUE works without explicit argument", {
  # Regression test: gg_roc() was crashing with
  # "argument 'oob' is missing, with no default" when oob was not supplied.
  set.seed(42)
  rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)

  # All three outcomes must work without passing oob
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 1), "gg_roc")
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 2), "gg_roc")
  expect_s3_class(gg_roc(rfsrc_iris, which_outcome = 3), "gg_roc")

  # OOB default should equal oob = TRUE explicitly
  gg_default <- gg_roc(rfsrc_iris, which_outcome = 1)
  gg_explicit <- gg_roc(rfsrc_iris, which_outcome = 1, oob = TRUE)
  expect_equal(gg_default, gg_explicit)
})

test_that("gg_roc regression", {
  data(Boston, package = "MASS")
  boston <- Boston

  boston$chas <- as.logical(boston$chas)

  ## Load the cached forest
  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_boston$family, "regr")

  ## Create the correct gg_roc object
  expect_error(gg_roc(rfsrc_boston))
  expect_error(plot.gg_roc(rfsrc_boston))

})

test_that("calc_roc", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_iris$family, "class")

  gg_dta <- calc_roc.rfsrc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 1,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # Test oob=FALSE
  gg_dta <- calc_roc.rfsrc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 1,
                           oob = FALSE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
  # The second outcome.
  gg_dta <- ggRandomForests::calc_roc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 2,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
  # and the third...
  gg_dta <- calc_roc(rfsrc_iris,
                           rfsrc_iris$yvar,
                           which_outcome = 3,
                           oob = TRUE)

  # Test the cached forest type
  expect_s3_class(gg_dta, "data.frame")

  expect_equal(ncol(gg_dta), 3)

  # test the auc calculator
  auc <- calc_auc(gg_dta)
  expect_true(auc > .9)
  expect_true(auc <= 1)
})

# ---------------------------------------------------------------------------
# randomForest ROC correctness (#81) + rfsrc no-change characterization
# ---------------------------------------------------------------------------

test_that("gg_roc randomForest: separable class AUC ~ 1 and many thresholds (#81)", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  setosa <- which(levels(iris$Species) == "setosa")
  g <- gg_roc(rf, which_outcome = setosa)
  expect_s3_class(g, "gg_roc")
  expect_gt(nrow(g), nlevels(iris$Species))      # not the degenerate ~3-row curve
  expect_gt(calc_auc(g), 0.98)                    # setosa is separable
})

test_that("gg_roc randomForest default is macro-average, many points, no warning (#81)", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  expect_no_warning(g <- gg_roc(rf))              # was: 'Must specify which_outcome'
  expect_s3_class(g, "gg_roc")
  expect_gt(nrow(g), nlevels(iris$Species))
  expect_gt(calc_auc(g), 0.5)
})

test_that("calc_roc.rfsrc output is unchanged for an explicit which_outcome (guard)", {
  set.seed(42)
  rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
  g <- gg_roc(rfsrc_iris, which_outcome = 1)
  expect_s3_class(g, "gg_roc")
  expect_equal(ncol(g), 3L)                       # sens, spec, pct (existing contract)
  expect_true(all(c("sens", "spec", "pct") %in% colnames(g)))
  expect_gte(calc_auc(g), 0.9)                    # rfsrc iris setosa-vs-rest stays strong
})

## ── per_class = TRUE (PR #88) ──────────────────────────────────────────────

test_that("gg_roc per_class=TRUE: long format with class column", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, per_class = TRUE)
  expect_true("class" %in% names(gg))
  expect_true(all(c("sens", "spec", "pct") %in% names(gg)))  # pct = threshold; same 3-col contract as calc_roc
  expect_s3_class(gg$class, "factor")
  expect_equal(nlevels(gg$class), 3L)
})

test_that("gg_roc per_class=TRUE: auc attr is named numeric vector length 3", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf  <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg  <- gg_roc(rf, per_class = TRUE)
  auc <- attr(gg, "auc")
  expect_length(auc, 3L)
  expect_named(auc)
  # setosa is linearly separable in iris — AUC should be near-perfect
  expect_gt(auc[["setosa"]], 0.99)
  # AUC values must be sorted descending
  expect_true(all(diff(auc) <= 0))
})

test_that("gg_roc per_class=TRUE: class factor levels ordered by descending AUC", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf  <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg  <- gg_roc(rf, per_class = TRUE)
  auc <- attr(gg, "auc")
  expect_equal(levels(gg$class), names(auc))
})

test_that("gg_roc per_class=TRUE on binary forest: no class column (no-op)", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  bin_data         <- iris[iris$Species != "virginica", ]
  bin_data$Species <- droplevels(bin_data$Species)
  rf  <- randomForest::randomForest(Species ~ ., data = bin_data, ntree = 100L)
  gg  <- gg_roc(rf, per_class = TRUE)
  # Binary forest: per_class is a no-op — no class column, scalar AUC
  expect_false("class" %in% names(gg))
  expect_length(attr(gg, "auc"), 1L)
})

test_that("gg_roc per_class=TRUE + which_outcome integer: message then per_class wins", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  expect_message(
    gg <- gg_roc(rf, per_class = TRUE, which_outcome = 1L),
    "which_outcome.*ignored.*per_class"
  )
  expect_true("class" %in% names(gg))
})

test_that("gg_roc which_outcome='all' still returns macro-average (no class column)", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, which_outcome = "all")
  expect_false("class" %in% names(gg))
  # Macro-average returns a single data frame, not a class-faceted one
  expect_true(all(c("sens", "spec", "pct") %in% names(gg)))  # pct = threshold; same 3-col contract as calc_roc
})

## ── plot.gg_roc per_class paths (PR #88) ─────────────────────────────────

test_that("plot.gg_roc per_class=TRUE: overlay returns ggplot", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, per_class = TRUE)
  p  <- plot(gg, panel = "overlay")
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_roc per_class=TRUE: facet returns ggplot", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, per_class = TRUE)
  p  <- plot(gg, panel = "facet")
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_roc per_class=TRUE: layer_data smokeable for overlay", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, per_class = TRUE)
  p  <- plot(gg, panel = "overlay")
  expect_no_error(ggplot2::layer_data(p, 1L))
})

test_that("plot.gg_roc existing single-class path unchanged", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, which_outcome = 1L)
  p  <- plot(gg)
  expect_s3_class(p, "ggplot")
  expect_no_error(ggplot2::layer_data(p, 1L))
})

test_that("summary.gg_roc per_class=TRUE: prints named AUC, no error", {
  skip_if_not_installed("randomForest")
  set.seed(1L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
  gg <- gg_roc(rf, per_class = TRUE)
  s  <- summary(gg)
  expect_s3_class(s, "summary.gg")
  # Body should mention all three class names
  expect_true(any(grepl("setosa", s$body)))
  expect_true(any(grepl("versicolor", s$body)))
  expect_true(any(grepl("virginica", s$body)))
})

# Regression tests for the empty-figure bug class.
#
# A `plot.gg_*()` method can return a ggplot whose data frame has zero rows,
# or whose layer data computes to zero rows after aesthetic mapping. That
# produces a figure with axes but no points or lines — which is what users
# saw in the survival vignette when the data transform's column names did
# not match what the plot method expected.
#
# These tests use `ggplot2::layer_data()` to inspect what each plot would
# actually draw. layer_data() runs the full ggplot build pipeline and returns
# the post-mapping data frame, so a zero-row result here means the figure
# would render empty.

Surv <- survival::Surv  # nolint: object_name_linter

# Helper: number of rows ggplot2 would actually render for a given layer.
# Accepts either a `ggplot` or a `patchwork` composite — for a patchwork,
# introspect the first sub-plot (patchwork supports `[[` to extract panels).
# Needed since #80: `plot.gg_variable()` returns a patchwork for multi-xvar.
expect_layer_nonempty <- function(p, layer = 1L, label = NULL) {
  testthat::expect_s3_class(p, c("ggplot", "patchwork"))
  if (inherits(p, "patchwork")) {
    # Patchworks also inherit from "ggplot" but layer_data() fails on the
    # composite — extract the first sub-plot for introspection.
    sub <- tryCatch(p[[1]], error = function(e) NULL)
    if (inherits(sub, "ggplot") && !inherits(sub, "patchwork")) p <- sub
  }
  ld <- ggplot2::layer_data(p, layer)
  testthat::expect_true(
    nrow(ld) > 0,
    info = sprintf(
      "%slayer %d data should have rows (got %d). cols: %s",
      if (is.null(label)) "" else paste0(label, ": "),
      layer, nrow(ld), paste(colnames(ld), collapse = ",")
    )
  )
  invisible(ld)
}

# Helper: every aesthetic in `mapping_keys` should resolve to a column with
# more than one unique value somewhere in the layer data. The empty-figure
# bug presented as columns mapped to axis labels because the data frame had
# only one literal value per "variable"/"value"/etc. column.
expect_layer_has_variation <- function(p, layer = 1L, mapping_keys) {
  ld <- ggplot2::layer_data(p, layer)
  for (k in mapping_keys) {
    testthat::expect_true(
      k %in% colnames(ld),
      info = sprintf("layer %d missing aesthetic '%s'", layer, k)
    )
    testthat::expect_gt(
      length(unique(ld[[k]])), 1,
      label = sprintf("variation in layer %d aes '%s'", layer, k)
    )
  }
  invisible(ld)
}

# ----------------------------------------------------------------------------
# gg_rfsrc — survival
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc survival (no CI) renders many step curves", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)

  gg <- gg_rfsrc(rf)
  expect_true(all(c("variable", "value", "obs_id", "event") %in% colnames(gg)))
  expect_type(gg$variable, "double")
  expect_type(gg$value, "double")

  p <- plot(gg)
  expect_layer_nonempty(p)
  expect_layer_has_variation(p, mapping_keys = c("x", "y"))
})

test_that("plot.gg_rfsrc survival (CI) renders ribbon + median curve", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_rfsrc(rf, conf.int = .95)
  expect_true(all(c("value", "lower", "upper", "median", "mean") %in% colnames(gg)))
  p <- plot(gg)
  # Layer 1 is the ribbon, layer 2 is the median step line.
  expect_layer_nonempty(p, layer = 1L)
  expect_layer_nonempty(p, layer = 2L)
})

test_that("plot.gg_rfsrc survival (by group) renders per-group curves", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_rfsrc(rf, by = "trt")
  expect_true("group" %in% colnames(gg))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  expect_gt(length(unique(ld$group)), 1)
})

# ----------------------------------------------------------------------------
# gg_rfsrc — regression
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc regression renders jitter + boxplot with data", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50)
  gg <- gg_rfsrc(rf)
  expect_true(all(c("yhat", "Ozone") %in% colnames(gg)))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), nrow(gg))
  expect_gt(length(unique(ld$y)), 1)
})

# ----------------------------------------------------------------------------
# gg_rfsrc — classification
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc multi-class renders one row per (obs, class)", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
  gg <- gg_rfsrc(rf)
  # Multi-class: one column per class plus "y"
  expect_true(all(levels(iris$Species) %in% colnames(gg)))
  expect_true("y" %in% colnames(gg))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  # Three classes × 150 observations => 450 rows after pivot
  expect_equal(nrow(ld), nrow(iris) * nlevels(iris$Species))
})

# ----------------------------------------------------------------------------
# gg_partial_rfsrc — survival regression test for the partial.type fix
# ----------------------------------------------------------------------------
test_that("gg_partial_rfsrc survival passes partial.type and produces data", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t90 <- ti[which.min(abs(ti - 90))]

  # Single time point.
  expect_no_error({
    pd <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, n_eval = 8)
  })
  expect_s3_class(pd, "gg_partial_rfsrc")
  expect_true(nrow(pd$continuous) > 0)
  expect_true(all(c("x", "yhat", "name", "time") %in% colnames(pd$continuous)))
  # Survival plot path expects a "time" column.
  p <- plot(pd)
  expect_layer_nonempty(p)
})

test_that("gg_partial_rfsrc survival multi-time expands to long form", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t30 <- ti[which.min(abs(ti - 30))]
  t90 <- ti[which.min(abs(ti - 90))]

  pd <- gg_partial_rfsrc(rf, xvar.names = "age",
                         partial.time = c(t30, t90), n_eval = 8)
  # Two time points × evaluation grid: long form, each (x, time) one row.
  expect_equal(length(unique(pd$continuous$time)), 2L)
  per_time <- table(pd$continuous$time)
  expect_true(all(per_time == per_time[[1]]))

  p <- plot(pd)
  ld <- expect_layer_nonempty(p)
  # The fixed plot maps time → colour, so we should see two groups.
  expect_gte(length(unique(ld$colour)), 2)
})

test_that("gg_partial_rfsrc partial.type rejects bad values", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  expect_error(
    gg_partial_rfsrc(rf, xvar.names = "age", partial.type = "bogus"),
    regexp = "should be one of"
  )
})

test_that("plot.gg_partial_rfsrc y-axis adapts to partial.type", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t90 <- ti[which.min(abs(ti - 90))]

  # Default ("surv") => "Predicted Survival"
  pd_s <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, n_eval = 6)
  expect_equal(attr(pd_s, "partial.type"), "surv")
  p_s <- plot(pd_s)
  expect_equal(p_s$labels$y, "Predicted Survival")

  # "chf" => "Predicted CHF"
  pd_c <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, partial.type = "chf",
                           n_eval = 6)
  expect_equal(attr(pd_c, "partial.type"), "chf")
  p_c <- plot(pd_c)
  expect_equal(p_c$labels$y, "Predicted CHF")

  # "mort" => "Predicted Mortality"; mort returns one value per x (no time dim),
  # so the survival "time" branch will not engage — we only assert the
  # attribute round-trips.
  pd_m <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.type = "mort",
                           n_eval = 6)
  expect_equal(attr(pd_m, "partial.type"), "mort")
})

test_that("plot.gg_partial_rfsrc preserves full-precision time grouping", {
  # Distinct times that round to the same 2-dp value must not collapse into a
  # single line. Build a synthetic gg_partial_rfsrc object with two such times.
  cont <- data.frame(
    x    = rep(c(1, 2, 3), times = 2),
    yhat = c(0.9, 0.8, 0.7, 0.5, 0.4, 0.3),
    name = "x",
    time = rep(c(1.001, 1.002), each = 3)
  )
  obj <- structure(
    list(
      continuous  = cont,
      categorical = data.frame(x = character(0), yhat = numeric(0),
                               name = character(0), time = integer(0))
    ),
    class = "gg_partial_rfsrc",
    partial.type = "surv"
  )
  p  <- plot(obj)
  ld <- ggplot2::layer_data(p, 1L)
  # Two distinct full-precision time horizons => two distinct groups, even
  # though both round to "1" at 2-dp precision.
  expect_equal(length(unique(ld$group)), 2L)
  expect_equal(length(unique(ld$colour)), 2L)
})

# ----------------------------------------------------------------------------
# gg_partial_rfsrc — regression
# ----------------------------------------------------------------------------
test_that("plot.gg_partial_rfsrc regression renders a non-empty line plot", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                               ntree = 50)
  pd <- gg_partial_rfsrc(rf, xvar.names = "Wind", n_eval = 8)
  p <- plot(pd)
  expect_layer_nonempty(p)
})

# ----------------------------------------------------------------------------
# gg_error
# ----------------------------------------------------------------------------
test_that("plot.gg_error single-outcome renders points or a line", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute",
                               ntree = 50, tree.err = TRUE,
                               block.size = 5)
  gg <- gg_error(rf)
  expect_true(all(c("ntree", "error") %in% colnames(gg)))
  p <- plot(gg)
  expect_layer_nonempty(p)
})

test_that("plot.gg_error multi-class pivots and colours by class", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Species ~ ., data = iris,
                               ntree = 50, tree.err = TRUE,
                               block.size = 5)
  p <- plot(gg_error(rf))
  ld <- expect_layer_nonempty(p)
  expect_gte(length(unique(ld$colour)), 2)
})

# ----------------------------------------------------------------------------
# gg_vimp
# ----------------------------------------------------------------------------
test_that("plot.gg_vimp renders one bar per variable", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50,
                               importance = TRUE)
  p <- plot(gg_vimp(rf))
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), length(rf$xvar.names))
})

test_that("plot.gg_vimp produces a single merged 'VIMP > 0' legend", {
  # Regression: previously the bar geom mapped both `fill` and `color` to
  # `positive` but only the fill legend got a custom title via labs(). ggplot
  # rendered two legends -- one labelled "VIMP > 0" (fill) and one labelled
  # "positive" (column name). Setting matching titles on both aesthetics
  # merges them into a single legend.
  #
  # We build a synthetic gg_vimp object rather than fit a real forest so the
  # test is deterministic across platforms (CI runners regularly produced
  # all-positive VIMP for the PBC dataset, which would skip the fill branch
  # we want to exercise here).
  gg <- structure(
    data.frame(
      vars     = c("a", "b", "c", "d"),
      vimp     = c(0.20, 0.05, -0.01, -0.04),
      rel_vimp = c(1.00, 0.25, 0.05,  0.20),
      positive = c(TRUE, TRUE, FALSE, FALSE)
    ),
    class = c("gg_vimp", "data.frame")
  )

  # Sanity: both signs present.
  expect_true(length(unique(gg$positive)) > 1)

  p <- plot(gg)
  expect_equal(p$labels$fill,   "VIMP > 0")
  expect_equal(p$labels$colour, "VIMP > 0")
  # The two aesthetic legend titles must match for ggplot to merge them.
  expect_identical(p$labels$fill, p$labels$colour)
})

test_that("plot.gg_vimp produces filled bars even when all VIMP are positive", {
  # Regression: previously the all-positive branch mapped only `color`,
  # leaving the bars hollow / outline-only and emitting "Ignoring unknown
  # labels: fill : 'VIMP > 0'" because the function-level labs(fill = ...)
  # had nothing to bind to. The fix maps fill + color unconditionally so the
  # bars are always filled and the legend title applies cleanly.
  gg <- structure(
    data.frame(
      vars     = c("a", "b", "c"),
      vimp     = c(0.30, 0.10, 0.05),
      rel_vimp = c(1.00, 0.33, 0.17),
      positive = c(TRUE, TRUE, TRUE)
    ),
    class = c("gg_vimp", "data.frame")
  )
  expect_equal(length(unique(gg$positive)), 1L)

  warns <- character()
  p <- withCallingHandlers(
    plot(gg),
    warning = function(w) {
      warns <<- c(warns, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )
  expect_false(any(grepl("Ignoring unknown labels", warns)),
               info = sprintf("got warnings: %s", paste(warns, collapse = " | ")))

  ld <- ggplot2::layer_data(p, 1L)
  # Bars must be filled, not hollow.
  expect_true("fill" %in% colnames(ld))
  expect_true(all(!is.na(ld$fill)))
})

# ----------------------------------------------------------------------------
# gg_variable
# ----------------------------------------------------------------------------
test_that("plot.gg_variable survival single xvar has a point per observation", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_variable(rf, time = 90)
  expect_true(all(c("event", "yhat", "time") %in% colnames(gg)))
  p <- plot(gg, xvar = "age")
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), nrow(veteran))
})

test_that("plot.gg_variable regression renders a scatter with smooth", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50)
  gg <- gg_variable(rf)
  p <- plot(gg, xvar = "Wind", smooth = FALSE)
  expect_layer_nonempty(p)
})

# ---------------------------------------------------------------------------
# randomForest engine matrix (#82 / #80 / #81)
# ---------------------------------------------------------------------------

test_that("plot.gg_rfsrc randomForest classification renders", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  p <- plot(gg_rfsrc(rf))
  expect_s3_class(p, "ggplot")
  expect_layer_nonempty(p)
})

test_that("plot.gg_rfsrc randomForest regression renders", {
  set.seed(42)
  rf <- randomForest::randomForest(mpg ~ ., data = mtcars)
  p <- plot(gg_rfsrc(rf))
  expect_s3_class(p, "ggplot")
  expect_layer_nonempty(p)
})

test_that("plot.gg_error randomForest classification renders", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  p <- plot(gg_error(rf))
  expect_s3_class(p, "ggplot")
  expect_layer_nonempty(p)
})

test_that("plot.gg_error randomForest regression has no stray colour label (#82 wart)", {
  set.seed(42)
  rf <- randomForest::randomForest(mpg ~ ., data = mtcars)
  p <- plot(gg_error(rf))
  expect_s3_class(p, "ggplot")
  expect_layer_nonempty(p)
  # #82 wart: plot.gg_error.R:244,248 apply labs(color="Outcome") even on
  # the regression / single-outcome path where no colour aesthetic is
  # mapped — which produces "Ignoring unknown labels: colour Outcome" at
  # build time. ggplot2 emits that warning once-per-session (cli .freq =
  # "once"), so warning-based detection is unreliable inside a long
  # test_file. Assert STRUCTURALLY: if a colour label is set, a colour
  # aesthetic must be mapped somewhere in the plot.
  has_colour_aes <- "colour" %in% names(p$mapping) ||
    any(vapply(p$layers, function(l) "colour" %in% names(l$mapping),
               logical(1)))
  if (!is.null(p$labels$colour)) {
    expect_true(has_colour_aes,
                info = "plot.gg_error set labs(colour=) without a mapped colour aesthetic")
  }
})

test_that("plot.gg_vimp randomForest classification renders (formula + non-formula)", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris, importance = TRUE)
  expect_layer_nonempty(plot(gg_vimp(rf)))
  rf2 <- randomForest::randomForest(iris[, 1:4], iris$Species, importance = TRUE)
  expect_layer_nonempty(plot(gg_vimp(rf2)))
})

test_that("plot.gg_vimp randomForest regression renders", {
  set.seed(42)
  rf <- randomForest::randomForest(mpg ~ ., data = mtcars, importance = TRUE)
  expect_layer_nonempty(plot(gg_vimp(rf)))
})

test_that("plot.gg_variable randomForest default multi-xvar is a single object (#80)", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  p <- plot(gg_variable(rf))                 # default xvar = all predictors
  # #80: post-fix this is a single ggplot/patchwork, not a bare list
  expect_true(inherits(p, "ggplot") || inherits(p, "patchwork"))
  # Structural check only — layer_data on the iris-classification sub-plots
  # fails because per-variable `geom_smooth` over multi-class `yhat.*` can't
  # compute `stat_smooth` (separate pre-existing classification quirk,
  # logged as a follow-up under #82). The #80 invariant is the contract:
  # a single composable object, with >1 sub-plot for default multi-xvar.
  if (inherits(p, "patchwork")) {
    expect_gt(length(p), 1L)
  }
})

test_that("plot.gg_variable randomForest regression default multi-xvar is a single object (#80)", {
  set.seed(42)
  rf <- randomForest::randomForest(mpg ~ ., data = mtcars)
  p <- plot(gg_variable(rf))
  expect_true(inherits(p, "ggplot") || inherits(p, "patchwork"))
  expect_layer_nonempty(p)
})

test_that("plot.gg_roc randomForest classification renders (#81)", {
  set.seed(42)
  rf <- randomForest::randomForest(Species ~ ., data = iris)
  p <- plot(gg_roc(rf, which_outcome = 1))
  expect_s3_class(p, "ggplot")
  expect_layer_nonempty(p)
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Predicted response plot from a \code{\link{gg_rfsrc}} object.
#'
#' Plot the predicted response from a \code{\link{gg_rfsrc}} object, the
#' \code{\link[randomForestSRC]{rfsrc}} prediction, using the OOB prediction
#' from the forest.  The plot type adapts automatically to the forest family:
#' jitter + boxplot for regression and classification, step curves for
#' survival.
#'
#' @param x A \code{\link{gg_rfsrc}} object, or a raw
#'   \code{\link[randomForestSRC]{rfsrc}} object (which will be passed through
#'   \code{\link{gg_rfsrc}} automatically before plotting).
#' @param notch Logical; whether to draw notched boxplots for regression and
#'   classification forests (default \code{TRUE}).  Set \code{notch = FALSE}
#'   to suppress notches when sample sizes are too small for reliable
#'   confidence intervals on the median.
#' @param ... Additional arguments forwarded to the underlying
#'   \code{ggplot2} geometry calls.  Commonly useful arguments include:
#'   \describe{
#'     \item{\code{alpha}}{Numeric in \eqn{[0,1]}; point/ribbon transparency.
#'       For survival plots with confidence bands the ribbon alpha is
#'       automatically halved relative to the value supplied here.}
#'     \item{\code{size}}{Point or line size passed to \code{geom_jitter},
#'       \code{geom_step}, etc.}
#'   }
#'   Arguments that control \code{\link{gg_rfsrc}} (e.g. \code{conf.int},
#'   \code{surv_type}, \code{by}) should be applied when constructing the
#'   \code{gg_rfsrc} object before calling \code{plot()}.
#'
#' @return A \code{ggplot} object.  The plot appearance depends on the forest
#'   family stored in \code{x}:
#'   \describe{
#'     \item{Regression (\code{"regr"})}{Jitter + notched boxplot of OOB
#'       predicted values.  If a \code{group} column is present the x-axis
#'       shows each group label; otherwise observations are collapsed to a
#'       single x-position.}
#'     \item{Classification (\code{"class"})}{Binary: jitter + notched
#'       boxplot of the predicted class probability.  Multi-class: jitter
#'       plot with one panel per class (class probabilities in long form).}
#'     \item{Survival (\code{"surv"})}{Step curves of the ensemble survival
#'       function.  When \code{gg_rfsrc} was called with \code{conf.int},
#'       a shaded ribbon is added.  When called with \code{by}, curves are
#'       coloured by group.}
#'   }
#'
#' @seealso \code{\link{gg_rfsrc}} \code{\link[randomForestSRC]{rfsrc}}
#'   \code{\link[randomForest]{randomForest}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
#' R, Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' # Build a small classification forest (ntree=50 keeps example fast)
#' set.seed(42)
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_iris)
#'
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' # na.action = "na.impute" handles missing Ozone / Solar.R values
#' set.seed(42)
#' rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_airq)
#'
#' plot(gg_dta)
#'
#' ## -------- mtcars data
#' set.seed(42)
#' rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_mtcars)
#'
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## -------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_veteran)
#' plot(gg_dta)
#'
#' # With 95% pointwise bootstrap confidence bands
#' gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = .95)
#' plot(gg_dta)
#'
#' # Stratified by treatment arm
#' gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt")
#' plot(gg_dta)
#'
#' ## -------- pbc data (larger dataset -- skipped on CRAN)
#' \donttest{
#' data(pbc, package = "randomForestSRC")
#' # For whatever reason, the age variable is in days; convert to years
#' for (ind in seq_len(dim(pbc)[2])) {
#'   if (!is.factor(pbc[, ind])) {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   } else {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'       if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   }
#'   if (!is.logical(pbc[, ind]) &
#'     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'     pbc[, ind] <- factor(pbc[, ind])
#'   }
#' }
#' # Convert age from days to years
#' pbc$age <- pbc$age / 364.24
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' # Remove test-set patients (those with no assigned treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#'
#' set.seed(42)
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'   dta_train,
#'   nsplit = 10,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   save.memory = TRUE
#' )
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc)
#' plot(gg_dta)
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = .95)
#' plot(gg_dta)
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc, by = "treatment")
#' plot(gg_dta)
#' }
#'
#' @export
plot.gg_rfsrc <- function(x, notch = TRUE, ...) {
  gg_dta <- x

  # Capture extra passthrough args (e.g. alpha for survival ribbons).
  # notch is a named formal above so it is never present in arg_set.
  arg_set <- list(...)

  ## If the user passed a raw rfsrc object, extract predictions first
  if (inherits(gg_dta, "rfsrc")) {
    gg_dta <- gg_rfsrc(gg_dta)
  }

  ## ---- Classification forest branch ------------------------------------
  if (inherits(gg_dta, "class") ||
    inherits(gg_dta, "classification")) {
    # Identify probability columns (everything except "y" and "group")
    non_prob <- c("y", "group")
    prob_cols <- setdiff(colnames(gg_dta), non_prob)
    prob_col <- prob_cols[1]
    obs_col  <- "y"
    if (length(prob_cols) < 2) {
      # Binary classification: single probability column + observed class
      gg_plt <- ggplot2::ggplot(gg_dta) +
        ggplot2::geom_jitter(
          ggplot2::aes(
            x = 1,
            y = .data[[prob_col]],
            color = .data[[obs_col]],
            shape = .data[[obs_col]]
          ),
          ...
        ) +
        ggplot2::geom_boxplot(
          ggplot2::aes(x = 1, y = .data[[prob_col]]),
          outlier.colour = "transparent",
          fill = "transparent",
          notch = notch
        ) +
        ggplot2::theme(
          axis.ticks = ggplot2::element_blank(),
          axis.text.x = ggplot2::element_blank()
        )
    } else {
      # Multi-class: gather all class probability columns into long form
      pivot_cols <- setdiff(colnames(gg_dta), c("y", "group"))
      gg_dta_mlt <-
        tidyr::pivot_longer(gg_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")

      gg_plt <-
        ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(x = .data$variable, y = .data$value)
        ) +
        ggplot2::geom_jitter(
          ggplot2::aes(color = .data$y, shape = .data$y),
          alpha = .5
        )
    }
    gg_plt <- gg_plt + ggplot2::labs(y = "Predicted (%)", x = "")

  ## ---- Survival forest branch ------------------------------------------
  } else if (inherits(gg_dta, "surv")) {
    # Detect whether bootstrap confidence bands have been computed
    if ("lower" %in% colnames(gg_dta)) {
      # Split ... into ribbon alpha (halved) and step dots (full alpha).
      # Prevents duplicate-formal error when user passes alpha via ...
      spl  <- .gg_split_alpha(arg_set)
      alph <- spl$ribbon_alpha

      if ("group" %in% colnames(gg_dta)) {
        # Stratified survival curves with CI ribbon, coloured by group
        gg_plt <- ggplot2::ggplot(gg_dta) +
          do.call(
            ggplot2::geom_ribbon,
            c(list(
              ggplot2::aes(
                x    = .data$value,
                ymin = .data$lower,
                ymax = .data$upper,
                fill = .data$group
              ),
              alpha = alph
            ), spl$ribbon_dots)
          ) +
          do.call(
            ggplot2::geom_step,
            c(list(ggplot2::aes(
              x     = .data$value,
              y     = .data$median,
              color = .data$group
            )), spl$step_dots)
          )
      } else {
        # Single-group survival curve with CI ribbon
        gg_plt <- ggplot2::ggplot(gg_dta) +
          do.call(
            ggplot2::geom_ribbon,
            c(list(
              ggplot2::aes(
                x    = .data$value,
                ymin = .data$lower,
                ymax = .data$upper
              ),
              alpha = alph,
              fill  = .gg_ribbon_fill
            ), spl$ribbon_dots)
          ) +
          do.call(
            ggplot2::geom_step,
            c(list(ggplot2::aes(
              x = .data$value,
              y = .data$median
            )), spl$step_dots)
          )
      }
    } else {
      # No confidence bands: draw one step line per observation
      gg_plt <- ggplot2::ggplot(
        gg_dta,
        ggplot2::aes(
          x = .data$variable,
          y = .data$value,
          col = .data$event,
          group = .data$obs_id
        )
      ) +
        ggplot2::geom_step(...)
    }

    gg_plt <- gg_plt +
      ggplot2::labs(x = "time (years)", y = "Survival (%)")

  ## ---- Regression forest branch ----------------------------------------
  } else if (inherits(gg_dta, "regr") ||
    inherits(gg_dta, "regression")) {
    if ("group" %in% colnames(gg_dta)) {
      # Grouped regression: x-axis shows each group label
      gg_plt <- ggplot2::ggplot(gg_dta, ggplot2::aes(x = .data$group, y = .data$yhat))
    } else {
      # Single-group regression: collapse to a single x position
      gg_plt <- ggplot2::ggplot(gg_dta, ggplot2::aes(x = 1, y = .data$yhat))
    }

    gg_plt <- gg_plt +
      ggplot2::geom_jitter(...) +
      ggplot2::geom_boxplot(
        outlier.colour = "transparent",
        fill = "transparent",
        notch = notch
      ) +
      ggplot2::labs(
        y = "Predicted Value",
        x = if ("group" %in% colnames(gg_dta)) "Group" else ""
      ) +
      ggplot2::theme(
        axis.ticks = ggplot2::element_blank(),
        axis.text.x = if ("group" %in% colnames(gg_dta)) {
          ggplot2::element_text()
        } else {
          ggplot2::element_blank()
        }
      )
  } else {
    # Unknown forest type — not yet implemented
    stop(paste(
      "Plotting for ",
      class(gg_dta)[2],
      " randomForestSRC is not yet implemented."
    ))
  }
  return(gg_plt)
}

# testthat for gg_rfsrc function

# Bring survival::Surv into the file-scope environment so that rfsrc() survival
# formulas (e.g. Surv(time, status) ~ .) can resolve the symbol without
# requiring library(survival) to be called explicitly in every test block.
Surv <- survival::Surv  # nolint: object_name_linter

test_that("gg_rfsrc classifications", {
  ## Load the cached forest
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_s3_class(rfsrc_iris, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_rfsrc(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_rfsrc")

  # Test classification dimensions
  expect_equal(nrow(gg_dta), nrow(rfsrc_iris$predicted.oob))
  expect_equal(ncol(gg_dta), ncol(rfsrc_iris$predicted.oob) + 1)

  # Test data is correctly pulled from randomForest obect.
  expect_equal(as.matrix(gg_dta[, -which(colnames(gg_dta) == "y")], ignore_attr = TRUE),
                    rfsrc_iris$predicted.oob)

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_rfsrc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")


  ## Create the correct gg_error object
  gg_dta <- gg_rfsrc(rfsrc_iris, oob = FALSE)

  # Test object type
  expect_s3_class(gg_dta, "gg_rfsrc")

  # Test classification dimensions
  expect_equal(nrow(gg_dta), nrow(rfsrc_iris$predicted))
  expect_equal(ncol(gg_dta), ncol(rfsrc_iris$predicted) + 1)

  # Test data is correctly pulled from randomForest obect.
  expect_equal(as.matrix(gg_dta[, -which(colnames(gg_dta) == "y")], ignore_attr = TRUE),
                    rfsrc_iris$predicted)

  rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
  gg_dta <- gg_rfsrc(rf_iris)

})

test_that("gg_rfsrc regression", {
  data(Boston, package = "MASS")
  boston <- Boston

  boston$chas <- as.logical(boston$chas)

  ## Load the cached forest
  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_boston$family, "regr")

  ## Create the correct gg_error object
  gg_dta <- gg_rfsrc(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "regr")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_rfsrc(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Create the correct gg_error object
  gg_dta <- gg_rfsrc(rfsrc_boston, oob = FALSE)

  # Test object type
  expect_s3_class(gg_dta, "gg_rfsrc")

  # Test classification dimensions
  ## Create the correct gg_error object
  gg_dta <- gg_rfsrc(rfsrc_boston, by = "chas")

  # Test object type
  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "regr")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_rfsrc(gg_dta)

  # Test data is correctly pulled from randomForest obect.
  # Predicted values
  rfsrc_boston$family <- "test"
  expect_error(gg_rfsrc(rfsrc_boston))

  # Test exceptions
  # Is it an rfsrc object?
  expect_error(gg_rfsrc(gg_plt))

  # Does it contain the forest?
  rfsrc_boston$forest <- NULL
  expect_error(gg_rfsrc(rfsrc_boston))

  data(Boston, package = "MASS")
  rf_boston <- randomForest(medv ~ ., data = Boston)
  plot(gg_rfsrc(rf_boston))

})

test_that("gg_rfsrc survival: per-observation curves (no conf.int, no by)", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  expect_s3_class(rfsrc_veteran, "rfsrc")
  expect_match(rfsrc_veteran$family, "surv")

  gg_dta <- gg_rfsrc(rfsrc_veteran)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "surv")
  # Per-observation long format: should have variable, value, obs_id, event
  expect_true("variable" %in% colnames(gg_dta))
  expect_true("value" %in% colnames(gg_dta))
  expect_true("obs_id" %in% colnames(gg_dta))
  expect_true("event" %in% colnames(gg_dta))
})

test_that("gg_rfsrc survival: plot of per-observation curves", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran)
  gg_plt <- plot.gg_rfsrc(gg_dta)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_rfsrc survival: confidence interval calculation", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = 0.95)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "surv")
  # conf.int output has lower, upper, median columns
  expect_true("lower" %in% colnames(gg_dta))
  expect_true("upper" %in% colnames(gg_dta))
  expect_true("median" %in% colnames(gg_dta))

  # Plot the conf.int result
  gg_plt <- plot.gg_rfsrc(gg_dta)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_rfsrc survival: confidence interval with percentage > 1", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  # conf.int > 1 should be divided by 100
  gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = 95)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("lower" %in% colnames(gg_dta))
})

test_that("gg_rfsrc survival: by argument groups observations", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt")

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "surv")
  expect_true("group" %in% colnames(gg_dta))

  # Plot the grouped result
  gg_plt <- plot.gg_rfsrc(gg_dta)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_rfsrc survival: by + conf.int produces grouped confidence bands", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt", conf.int = 0.95)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("group" %in% colnames(gg_dta))
  expect_true("lower" %in% colnames(gg_dta))

  # Plot
  gg_plt <- plot.gg_rfsrc(gg_dta)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_rfsrc survival: surv_type = 'chf' (cumulative hazard)", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, surv_type = "chf")
  expect_s3_class(gg_dta, "gg_rfsrc")
})

test_that("gg_rfsrc survival: surv_type = 'mortality'", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, surv_type = "mortality")
  expect_s3_class(gg_dta, "gg_rfsrc")
})

test_that("gg_rfsrc survival: unknown surv_type throws error", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  expect_error(gg_rfsrc(rfsrc_veteran, surv_type = "unknown_type"))
})

test_that("gg_rfsrc survival: by vector (not column name) works", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  by_vec <- veteran$trt
  gg_dta <- gg_rfsrc(rfsrc_veteran, by = by_vec)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("group" %in% colnames(gg_dta))
})

test_that("gg_rfsrc survival: by vector wrong length throws error", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  expect_error(gg_rfsrc(rfsrc_veteran, by = c(1, 2, 3)))
})

test_that("gg_rfsrc survival: oob = FALSE uses in-bag predictions", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, oob = FALSE)
  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_s3_class(gg_dta, "surv")
})

test_that("gg_rfsrc survival: conf.int with custom bs.sample", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = 0.95, bs.sample = 20)
  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("lower" %in% colnames(gg_dta))
})

test_that("gg_rfsrc survival: by + conf.int with custom bs.sample", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt", conf.int = 0.95, bs.sample = 20)
  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("group" %in% colnames(gg_dta))
  expect_true("lower" %in% colnames(gg_dta))
})

test_that("gg_rfsrc survival: conf.int with two-element level_set", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5,
    forest = TRUE,
    save.memory = TRUE
  )

  # Two-element conf.int (lower and upper directly)
  gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = c(0.025, 0.975))
  expect_s3_class(gg_dta, "gg_rfsrc")
})

test_that("gg_rfsrc classification: by argument adds group column", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    save.memory = TRUE
  )

  # Use Petal.Width as a logical grouping vector (not character to avoid column-name lookup)
  by_vec <- iris$Petal.Width > median(iris$Petal.Width)
  gg_dta <- gg_rfsrc(rfsrc_iris, by = by_vec)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("group" %in% colnames(gg_dta))
})

test_that("gg_rfsrc regression: by = vector works", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  by_vec <- boston$chas
  gg_dta <- gg_rfsrc(rfsrc_boston, by = by_vec)

  expect_s3_class(gg_dta, "gg_rfsrc")
  expect_true("group" %in% colnames(gg_dta))
})

test_that("plot.gg_rfsrc notch=FALSE suppresses notch in boxplot", {
  # The notch argument used to be hardcoded to TRUE; callers had no way to
  # suppress it.  Passing notch = FALSE must not error and must return a ggplot.
  set.seed(42)
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ ., data = iris, ntree = 50, save.memory = TRUE
  )
  gg_dta <- gg_rfsrc(rfsrc_iris)
  gg_plt <- plot.gg_rfsrc(gg_dta, notch = FALSE)
  expect_s3_class(gg_plt, "ggplot")

  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
  set.seed(42)
  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ ., data = Boston, ntree = 50, save.memory = TRUE
  )
  gg_dta_regr <- gg_rfsrc(rfsrc_boston)
  gg_plt_regr <- plot.gg_rfsrc(gg_dta_regr, notch = FALSE)
  expect_s3_class(gg_plt_regr, "ggplot")
})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)


## ----packages-----------------------------------------------------------------
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)
library(survival)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())

# Plotting constants
event_marks  <- c(1, 4)
event_labels <- c("Censored", "Death")
event_colors <- c("steelblue", "firebrick")


## ----data-load----------------------------------------------------------------
data("pbc", package = "randomForestSRC")


## ----data-clean---------------------------------------------------------------
pbc <- pbc |>
  mutate(
    years     = days / 365.25,
    age       = age / 365.25,
    treatment = factor(
      ifelse(treatment == 1, "DPCA",
             ifelse(treatment == 2, "Placebo", NA)),
      levels = c("DPCA", "Placebo")
    )
  ) |>
  select(-days)

# Low-cardinality numerics (including binary 0/1) to factor.
# NOTE: do NOT convert to logical — partial.rfsrc() fails with logical
# predictors in survival forests (randomForestSRC <= 3.5.1).
# Exclude the response columns (status, years) from conversion.
resp_cols <- c("status", "years")
for (nm in setdiff(names(pbc), resp_cols)) {
  v <- pbc[[nm]]
  if (is.numeric(v) && !is.factor(v) && length(unique(v[!is.na(v)])) <= 5) {
    pbc[[nm]] <- factor(v)
  }
}


## ----variable-labels----------------------------------------------------------
# Human-readable labels for plot axes
st_labs <- c(
  status       = "Death Event",
  treatment    = "Treatment",
  age          = "Age (years)",
  sex          = "Female",
  ascites      = "Ascites",
  hepato       = "Hepatomegaly",
  spiders      = "Spiders",
  edema        = "Edema (0, 0.5, 1)",
  bili         = "Serum Bilirubin (mg/dl)",
  chol         = "Serum Cholesterol (mg/dl)",
  albumin      = "Albumin (gm/dl)",
  copper       = "Urine Copper (ug/day)",
  alk.phos     = "Alkaline Phosphatase (U/liter)",
  ast          = "SGOT (U/ml)",
  trig         = "Triglycerides (mg/dl)",
  platelet     = "Platelets (per cubic ml/1000)",
  prothrombin  = "Prothrombin Time (sec)",
  stage        = "Histologic Stage",
  years        = "Follow-up Time (years)"
)


## ----eda-categorical, fig.height=4, fig.cap="EDA for categorical variables. Bar color indicates factor level; white = missing."----
cls <- sapply(pbc, class)
cnt_idx <- which(cls %in% c("numeric", "integer"))
fct_idx <- setdiff(seq_along(pbc), cnt_idx)
fct_idx <- union(fct_idx, which(names(pbc) == "years"))

dta_cat <- suppressWarnings(
  pbc[, fct_idx] |>
    pivot_longer(-years, names_to = "variable", values_to = "value",
                 values_transform = list(value = as.character))
)

ggplot(dta_cat, aes(x = years, fill = value)) +
  geom_histogram(color = "black", binwidth = 1) +
  labs(y = "", x = st_labs["years"]) +
  scale_fill_brewer(palette = "RdBu", na.value = "white") +
  facet_wrap(~variable, scales = "free_y", nrow = 2) +
  theme(legend.position = "none")


## ----eda-continuous, fig.height=4, fig.cap="EDA for continuous variables. Points colored by death event; rug marks indicate missing values."----
cnt_idx <- union(cnt_idx, which(names(pbc) == "status"))
dta_cont <- pbc[, cnt_idx] |>
  pivot_longer(c(-years, -status),
               names_to = "variable", values_to = "value")

ggplot(dta_cont |> filter(!is.na(value)),
       aes(x = years, y = value, color = factor(status), shape = factor(status))) +
  geom_point(alpha = 0.4) +
  geom_rug(data = dta_cont |> filter(is.na(value)), color = "grey50") +
  labs(y = "", x = st_labs["years"], color = "Death", shape = "Death") +
  scale_color_manual(values = event_colors) +
  scale_shape_manual(values = event_marks) +
  facet_wrap(~variable, scales = "free_y", ncol = 4) +
  theme(legend.position = c(0.8, 0.2))


## ----km-survival, fig.cap="KM survival estimates by treatment group with 95% confidence bands."----
pbc_trial <- pbc |> filter(!is.na(treatment))
pbc_test  <- pbc |> filter(is.na(treatment))

gg_km <- gg_survival(interval = "years", censor = "status",
                     by = "treatment", data = pbc_trial,
                     conf.int = 0.95)

plot(gg_km) +
  labs(y = "Survival Probability", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.2)) +
  coord_cartesian(ylim = c(0, 1.01))


## ----km-cumhaz, fig.cap="KM cumulative hazard estimates by treatment group."----
plot(gg_km, type = "cum_haz") +
  labs(y = "Cumulative Hazard", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.8)) +
  coord_cartesian(ylim = c(-0.02, 1.22))


## ----km-bili, fig.width=6, fig.cap="KM survival stratified by bilirubin groups."----
pbc_bili <- pbc_trial |>
  mutate(bili_grp = cut(bili, breaks = c(0, 0.8, 1.3, 3.4, 29)))

plot(gg_survival(interval = "years", censor = "status",
                 by = "bili_grp", data = pbc_bili),
     error = "none") +
  labs(y = "Survival Probability", x = "Time (years)", color = "Bilirubin")


## ----rfsrc-fit----------------------------------------------------------------
# Step 1: impute missing values via random forest proximity
pbc_imputed <- impute.rfsrc(Surv(years, status) ~ .,
                             data    = pbc_trial,
                             ntree   = 500,
                             nimpute = 2)

# Step 2: grow the survival forest on the complete imputed data
rfsrc_pbc <- rfsrc(Surv(years, status) ~ .,
                   data      = pbc_imputed,
                   nsplit    = 10,
                   tree.err  = TRUE,
                   importance = TRUE)


## ----error-plot, fig.height=4, fig.cap="OOB prediction error vs. number of trees."----
plot(gg_error(rfsrc_pbc))


## ----rfsrc-predicted, fig.cap="OOB predicted survival curves. Blue = censored, red = death events."----
gg_rsf <- plot(gg_rfsrc(rfsrc_pbc), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
gg_rsf


## ----rfsrc-by-treatment, fig.cap="Median predicted survival by treatment group with 95% confidence bands."----
plot(gg_rfsrc(rfsrc_pbc, by = "treatment")) +
  theme(legend.position = c(0.2, 0.2)) +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----predict-test, fig.cap="Predicted survival for non-trial patients (test set)."----
rfsrc_pbc_test <- predict(rfsrc_pbc, newdata = pbc_test,
                          na.action = "na.impute",
                          importance = TRUE)

plot(gg_rfsrc(rfsrc_pbc_test), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----vimp-plot, fig.width=6, fig.cap="Variable importance ranking. Blue = positive VIMP, red = negative."----
plot(gg_vimp(rfsrc_pbc), lbls = st_labs) +
  theme(legend.position = c(0.8, 0.2)) +
  labs(fill = "VIMP > 0")


## ----varsel-------------------------------------------------------------------
md_pbc <- max.subtree(rfsrc_pbc)


## ----select-vars--------------------------------------------------------------
xvar     <- c("bili", "albumin", "copper", "prothrombin", "age")
xvar_cat <- "edema"
xvar_all <- c(xvar, xvar_cat)


## ----vardep-bili, fig.height=4, fig.cap="Variable dependence of survival at 1 and 3 years on bilirubin."----
gg_v <- gg_variable(rfsrc_pbc, time = c(1, 3),
                    time.labels = c("1 Year", "3 Years"))

plot(gg_v, xvar = "bili", alpha = 0.4) +
  labs(y = "Survival", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----vardep-panel, fig.height=5, fig.cap="Variable dependence at 1 and 3 years for continuous predictors."----
plot(gg_v, xvar = xvar[-1], panel = TRUE, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.05, 1.05))


## ----vardep-categorical, fig.height=4, fig.cap="Variable dependence on edema (categorical)."----
plot(gg_v, xvar = xvar_cat, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.02))


## ----partial-dep, error=TRUE, fig.height=5, fig.cap="Partial dependence of survival at approximately 1 and 3 years on continuous predictors."----
try({
# partial.rfsrc() requires times that match the model's time.interest grid;
# gg_partial_rfsrc() snaps the requested values to the nearest observed times.
ti   <- rfsrc_pbc$time.interest
t1yr <- ti[which.min(abs(ti - 1))]
t3yr <- ti[which.min(abs(ti - 3))]

pd <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = xvar,
                       partial.time = c(t1yr, t3yr))

# Quick S3 plot — survival forests produce time-series curves per predictor value
plot(pd)
})


## ----partial-dep-custom, error=TRUE, fig.height=5, fig.cap="Partial dependence (custom styling)."----
try({
ggplot(pd$continuous, aes(x = x, y = yhat,
                          color = factor(round(time, 2)),
                          group = factor(time))) +
  geom_line(linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = "Predicted Survival", x = "", color = "Year") +
  scale_color_manual(values = setNames(c("steelblue", "firebrick"),
                                       as.character(round(c(t1yr, t3yr), 2)))) +
  theme_bw()
})


## ----coplot-edema, fig.width=7, fig.height=4, fig.cap="Variable dependence on bilirubin, conditional on edema group."----
gg_v1 <- gg_variable(rfsrc_pbc, time = 1)
gg_v1$edema <- paste("edema =", gg_v1$edema)

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_grid(~edema) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----coplot-albumin, fig.width=7, fig.height=5, fig.cap="Variable dependence on bilirubin, conditional on albumin groups."----
albumin_cts <- quantile_pts(gg_v1$albumin, groups = 6, intervals = TRUE)
gg_v1$albumin_grp <- cut(gg_v1$albumin, breaks = albumin_cts)
levels(gg_v1$albumin_grp) <- paste("albumin =",
                                    levels(gg_v1$albumin_grp))

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_wrap(~albumin_grp) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----surface-data, error=TRUE, cache=FALSE------------------------------------
try({
# Create grid of albumin values
alb_grid <- quantile_pts(pbc_trial$albumin, groups = 25)

# For each albumin value, compute partial dependence on bili at ~1 year
surface_list <- lapply(alb_grid, function(alb_val) {
  newx <- pbc_trial[, rfsrc_pbc$xvar.names, drop = FALSE]
  newx$albumin <- alb_val
  pd_alb <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = "bili",
                              newx = newx, partial.time = t1yr)
  df <- pd_alb$continuous
  df$albumin <- alb_val
  df
})

surface_df <- bind_rows(surface_list)
})


## ----plotly-surface, error=TRUE, fig.cap="Interactive partial dependence surface: survival as a function of bilirubin and albumin."----
try({
if (!exists("surface_df")) {
  message("surface_df not available — skipping plotly surface (see surface-data chunk error above).")
} else if (requireNamespace("plotly", quietly = TRUE)) {
  # Reshape for surface
  library(plotly)

  surface_wide <- surface_df |>
    select(bili = x, albumin, survival = yhat) |>
    arrange(albumin, bili)

  # Create matrix form
  bili_vals <- sort(unique(surface_wide$bili))
  alb_vals  <- sort(unique(surface_wide$albumin))
  z_matrix  <- matrix(surface_wide$survival,
                       nrow = length(alb_vals),
                       ncol = length(bili_vals),
                       byrow = TRUE)

  plot_ly(x = bili_vals, y = alb_vals, z = z_matrix) |>
    add_surface(colorscale = "Viridis", showscale = TRUE) |>
    layout(
      scene = list(
        xaxis = list(title = "Bilirubin"),
        yaxis = list(title = "Albumin"),
        zaxis = list(title = "Survival")
      )
    )
} else {
  message("Install the plotly package for interactive 3D surfaces.")
  # Fallback: contour plot with ggplot2
  ggplot(surface_df, aes(x = x, y = albumin, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Survival") +
    labs(x = "Bilirubin", y = "Albumin") +
    theme_bw()
}
})


## ----brier-overall, fig.width=7, fig.height=3.5, fig.cap="Time-resolved Brier score for the PBC survival forest."----
gg_bs <- gg_brier(rfsrc_pbc)
plot(gg_bs)


## ----brier-envelope, fig.width=7, fig.height=3.5, fig.cap="Brier score with 15--85% per-subject envelope."----
plot(gg_bs, envelope = TRUE)


## ----brier-crps, fig.width=7, fig.height=3.5, fig.cap="Running CRPS for the PBC survival forest."----
plot(gg_bs, type = "crps")


## ----brier-crps-scalar--------------------------------------------------------
attr(gg_bs, "crps_integrated")

.onAttach <- function(libname, pkgname) {
  ggRandomForests.version <- read.dcf(file=system.file("DESCRIPTION", package=pkgname), 
                            fields="Version")
  packageStartupMessage(paste("\n",
                              pkgname,
                              ggRandomForests.version,
                              "\n",
                              "\n",
                              "Type ggrandomforests.news() to see new features, changes, and bug fixes.",
                              "\n",
                              "\n"))
}

####**********************************************************************
####  print.gg_* methods.
####
####  All print methods follow the same shape: emit a one-line header
####  (class label + provenance) and return the object invisibly. They do
####  NOT print rows — gg_* objects inherit data.frame (or list), so
####  head() and as.data.frame() remain available for inspecting contents.
####**********************************************************************

#' Print methods for gg_* data objects
#'
#' Each \code{print.gg_*()} method emits a single-line header containing the
#' class label and, when available, forest provenance metadata (source package,
#' family, ntree, n). The object is returned invisibly so \code{print()} calls
#' chain cleanly in pipes.
#'
#' To inspect rows use \code{head()}.  To retrieve per-class diagnostics use
#' \code{\link{summary.gg}}.
#'
#' @param x A \code{gg_*} data object.
#' @param ... Not currently used.
#'
#' @return The object \code{x}, invisibly.
#'
#' @seealso \code{\link{summary.gg}}, \code{\link{autoplot.gg}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' print(gg_error(rf))
#' print(gg_vimp(rf))
#'
#' @name print.gg
NULL

#' @rdname print.gg
#' @export
print.gg_error <- function(x, ...) {
  cat(.gg_header(x, "gg_error"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_vimp <- function(x, ...) {
  cat(.gg_header(x, "gg_vimp"),
      sprintf("  |  variables: %d", nrow(x)),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_rfsrc <- function(x, ...) {
  cat(.gg_header(x, "gg_rfsrc"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_variable <- function(x, ...) {
  cat(.gg_header(x, "gg_variable"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partial <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  cat(.gg_header(x, "gg_partial"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partial_rfsrc <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  cat(.gg_header(x, "gg_partial_rfsrc"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partialpro <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  cat(.gg_header(x, "gg_partialpro"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_roc <- function(x, ...) {
  cat(.gg_header(x, "gg_roc"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_survival <- function(x, ...) {
  cat(.gg_header(x, "gg_survival"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_brier <- function(x, ...) {
  crps <- attr(x, "crps_integrated")
  suffix <- if (!is.null(crps)) {
    sprintf("  |  integrated CRPS: %.4g", crps)
  } else {
    ""
  }
  cat(.gg_header(x, "gg_brier"), suffix, "\n", sep = "")
  invisible(x)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Variable Importance (VIMP) data object
#'
#' \code{gg_vimp} Extracts the variable importance (VIMP) information from a
#' \code{\link[randomForestSRC]{rfsrc}} or \code{\link[randomForest]{randomForest}}
#' object and reshapes it into a tidy data set.
#'
#' @param object A \code{\link[randomForestSRC]{rfsrc}} object, the output from
#' \code{\link[randomForestSRC]{vimp}}, or a fitted
#' \code{\link[randomForest]{randomForest}}.
#' @param nvar argument to control the number of variables included in the
#' output.
#' @param ... arguments passed to the \code{\link[randomForestSRC]{vimp.rfsrc}}
#' function if the \code{\link[randomForestSRC]{rfsrc}} object does not contain
#' importance information.
#'
#' @return \code{gg_vimp} object. A \code{data.frame} of VIMP measures, in rank
#'   order, optionally containing class-specific scores and a relative importance
#'   column. When \code{randomForest} objects lack stored importance values a
#'   warning is issued and \code{NA} placeholders are returned so plots remain
#'   reproducible.
#'
#' @seealso \code{\link{plot.gg_vimp}} \code{\link[randomForestSRC]{rfsrc}}
#' @seealso \code{\link[randomForestSRC]{vimp}}
#'
#' @references
#' Ishwaran H. (2007). Variable importance in binary regression trees and
#' forests, \emph{Electronic J. Statist.}, 1:519-537.
#'
#' @importFrom randomForestSRC vimp
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- rfsrc(Species ~ .,
#'   data = iris,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_iris)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## regression example
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data
#' rfsrc_airq <- rfsrc(Ozone ~ ., airquality,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_airq)
#' plot(gg_dta)
#'
#'
#' ## -------- Boston data
#' data(Boston, package = "MASS")
#' rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_boston)
#' plot(gg_dta)
#'
#' ## -------- Boston data
#' rf_boston <- randomForest::randomForest(medv ~ ., Boston)
#' gg_dta <- gg_vimp(rf_boston)
#' plot(gg_dta)
#'
#'
#' ## -------- mtcars data
#' rfsrc_mtcars <- rfsrc(mpg ~ .,
#'   data = mtcars,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_mtcars)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## survival example
#' ## ------------------------------------------------------------
#'
#' ## -------- veteran data
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ .,
#'   data = veteran,
#'   ntree = 100,
#'   importance = TRUE
#' )
#'
#' gg_dta <- gg_vimp(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## -------- pbc data
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC", )
#' # For whatever reason, the age variable is in days...
#' # makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'   if (!is.factor(pbc[, ind])) {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   } else {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'       if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   }
#'   if (!is.logical(pbc[, ind]) &
#'     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'     pbc[, ind] <- factor(pbc[, ind])
#'   }
#' }
#' # Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' # ========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'   dta_train,
#'   nsplit = 10,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   save.memory = TRUE
#' )
#'
#' gg_dta <- gg_vimp(rfsrc_pbc)
#' plot(gg_dta)
#'
#' # Restrict to only the top 10.
#' gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10)
#' plot(gg_dta)
#'
#' @aliases gg_vimp gg_vimp.rfsrc gg_vimp.randomForest
#' @aliases gg_vimp.randomForest.formula
#' @export
gg_vimp <- function(object, nvar, ...) {
  UseMethod("gg_vimp", object)
}
#' @export
gg_vimp.rfsrc <- function(object, nvar, ...) {
  # Validate that the object is an rfsrc grow or predict result.
  if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &&
    sum(inherits(object, c("rfsrc", "predict"), TRUE) == c(1, 2)) != 2) {
    stop("This function only works for objects of class `(rfsrc, grow)' or
         '(rfsrc, predict)'.")
  }

  # If the forest was grown without importance = TRUE, compute VIMP now via
  # a separate vimp() call (slower, but always safe).
  if (is.null(object$importance)) {
    warning("rfsrc object does not contain VIMP information. Calculating...")
    gg_dta <-
      data.frame(sort(randomForestSRC::vimp(object)$importance,
        decreasing = TRUE
      ))
  } else {
    gg_dta <- data.frame(object$importance)
  }

  # Single-outcome forests: $importance is a named vector → one-column df.
  # Rename the column and add a "vars" column with the variable names.
  if (ncol(gg_dta) == 1) {
    colnames(gg_dta) <- "VIMP"
    gg_dta$vars <- rownames(gg_dta)
    gg_dta <- gg_dta[order(gg_dta$VIMP, decreasing = TRUE), ]
  }

  # Clamp nvar to the number of available variables.
  if (missing(nvar)) {
    nvar <- nrow(gg_dta)
  }
  if (nvar > nrow(gg_dta)) {
    nvar <- nrow(gg_dta)
  }

  # Multi-class forests: $importance is a matrix (vars × classes).
  # Pivot to long form so each row is one (variable, class) combination.
  if (ncol(gg_dta) > 1) {
    arg_set <- list(...)

    if (!is.null(arg_set$which.outcome)) {
      # Caller requested importance for one specific class only.
      if (!is.numeric(arg_set$which.outcome)) {
        # Look up by class name (column name).
        if (arg_set$which.outcome %in% colnames(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste(
              "which.outcome naming is incorrect.",
              arg_set$which.outcome,
              "\nis not in",
              colnames(gg_dta)
            )
          )
        }
      } else {
        # Look up by integer index.
        # which.outcome = 0  → overall (across-class) importance, column 1
        # which.outcome = k  → importance for class k, column k+1
        if (!is.numeric(arg_set$which.outcome) || arg_set$which.outcome < 0) {
          stop("which.outcome must be a non-negative integer or a class name.")
        }
        if (arg_set$which.outcome < ncol(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome + 1],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome + 1],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste0(
              "which.outcome (", arg_set$which.outcome, ") is out of range. ",
              "Valid values are 0 (overall) to ", ncol(gg_dta) - 1,
              " (number of classes)."
            )
          )
        }
      }
      gg_dta <- gg_v
    } else {
      # No specific class requested: attach variable names and pivot.
      gg_dta$vars <- rownames(gg_dta)
    }

    gg_dta <- gg_dta[seq_len(nvar), ]
    pivot_cols <-
      colnames(gg_dta)[-which(colnames(gg_dta) == "vars")]
    # Pivot from wide (one column per class) to long (one row per class-var pair).
    gg_dta <- tidyr::pivot_longer(
      gg_dta,
      tidyr::all_of(pivot_cols),
      names_to = "set",
      values_to = "vimp"
    )
    gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    gg_dta$vars <- factor(gg_dta$vars)
  } else {
    # Single-outcome: compute relative VIMP (each value as a fraction of the
    # top-ranked variable's importance).
    cnms <- colnames(gg_dta)
    gg_dta <- cbind(gg_dta, gg_dta / gg_dta[1, 1])
    colnames(gg_dta) <- c(cnms, "rel_vimp")
    # Patch any NA entries in the vars column that slipped through.
    gg_dta$vars[which(is.na(gg_dta$vars))] <-
      rownames(gg_dta)[which(is.na(gg_dta$vars))]

    gg_dta <- gg_dta[seq_len(nvar), ]
  }

  # Convert vars to an ordered factor (reversed so the most important variable
  # plots at the top of a horizontal bar chart after coord_flip).
  gg_dta$vars <-
    factor(gg_dta$vars, levels = rev(unique(gg_dta$vars)))

  # Flag variables with non-positive VIMP so the plot can colour them
  # differently to indicate they do not improve (or actively hurt) predictions.
  gg_dta$positive <- TRUE
  gg_dta$positive[which(gg_dta$vimp <= 0)] <- FALSE

  class(gg_dta) <- c("gg_vimp", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_vimp.randomForest <- function(object, nvar, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForest package."
      )
    )
  }

  ### set importance to NA if it is NULL
  if (is.null(object$importance)) {
    importance_est <- tryCatch(
      randomForest::importance(object, scale = FALSE),
      error = function(e) NULL
    )
    if (is.null(importance_est)) {
      vars <- object$forest$xvar.names
      if (is.null(vars)) {
        training_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
        if (!is.null(training_info)) {
          vars <- setdiff(colnames(training_info$model_frame), training_info$response_name)
        }
      }
      if (is.null(vars)) {
        stop(
          "Variable importance is unavailable for this randomForest object and ",
          "training predictors cannot be recovered."
        )
      }
      warning(
        "randomForest object does not contain importance information. Returning NA values."
      )
      placeholder <- data.frame(
        vimp = rep(NA_real_, length(vars)),
        vars = vars,
        rel_vimp = NA_real_
      )
      placeholder$vars <- factor(placeholder$vars, levels = rev(unique(placeholder$vars)))
      placeholder$positive <- FALSE
      class(placeholder) <- c("gg_vimp", class(placeholder))
      return(placeholder)
    }
    gg_dta <- data.frame(importance_est)
  } else {
    gg_dta <- data.frame(object$importance)
  }
  if (ncol(gg_dta) < 3) {
    gg_dta$vars <- rownames(gg_dta)
    colnames(gg_dta)[which(colnames(gg_dta) == "X.IncMSE")] <-
      "vimp"
    if ("vimp" %in% colnames(gg_dta)) {
      gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    } else {
      cn <- colnames(gg_dta)[1]
      gg_dta <-
        gg_dta[order(gg_dta[, cn], decreasing = TRUE), ]
      # Ensure a canonical "vimp" column exists so the positive-flag logic and
      # plot.gg_vimp both work regardless of what randomForest named the column.
      colnames(gg_dta)[1] <- "vimp"
    }
  }
  if (missing(nvar)) {
    nvar <- nrow(gg_dta)
  }
  if (nvar > nrow(gg_dta)) {
    nvar <- nrow(gg_dta)
  }

  # Handle multiclass importance
  if (ncol(gg_dta) > 1) {
    # Classification...
    arg_set <- list(...)

    if (!is.null(arg_set$which.outcome)) {
      # test which.outcome specification
      if (!is.numeric(arg_set$which.outcome)) {
        if (arg_set$which.outcome %in% colnames(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste(
              "which.outcome naming is incorrect.",
              arg_set$which.outcome,
              "\nis not in",
              colnames(gg_dta)
            )
          )
        }
      } else {
        # which.outcome = 0 → overall importance (column 1)
        # which.outcome = k → class k importance (column k+1)
        if (!is.numeric(arg_set$which.outcome) || arg_set$which.outcome < 0) {
          stop("which.outcome must be a non-negative integer or a class name.")
        }
        if (arg_set$which.outcome < ncol(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome + 1],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome + 1],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste0(
              "which.outcome (", arg_set$which.outcome, ") is out of range. ",
              "Valid values are 0 (overall) to ", ncol(gg_dta) - 1,
              " (number of classes)."
            )
          )
        }
      }
      gg_dta <- gg_v
    } else {
      gg_dta$vars <- rownames(gg_dta)
    }

    pivot_cols <-
      colnames(gg_dta)[-which(colnames(gg_dta) == "vars")]
    gg_dta <- tidyr::pivot_longer(
      gg_dta,
      tidyr::all_of(pivot_cols),
      names_to = "set",
      values_to = "vimp"
    )
    gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    gg_dta$vars <- factor(gg_dta$vars)
  } else {
    gg_dta$vars[which(is.na(gg_dta$vars))] <-
      rownames(gg_dta)[which(is.na(gg_dta$vars))]
  }
  gg_dta <- gg_dta[seq_len(nvar), ]

  gg_dta$vars <-
    factor(gg_dta$vars, levels = rev(unique(gg_dta$vars)))
  gg_dta$positive <- TRUE
  gg_dta$positive[which(gg_dta$vimp <= 0)] <- FALSE

  class(gg_dta) <- c("gg_vimp", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

# testthat for gg_vimp function

test_that("gg_vimp classifications", {
  ## Load the cached forest
  data(iris, package = "datasets")
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE
  )
  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_equal(rfsrc_iris$family, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Grab only one class... by number.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = 2)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by number - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = 0)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by name - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = "all")

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by name - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = "setosa")

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Grab only one class... by name - that doesn't exist.
  expect_error(gg_vimp(rfsrc_iris, which.outcome = "nothing special"))

  # Grab only one class... by number - that doesn't exist.
  expect_error(gg_vimp(rfsrc_iris, which.outcome = 200))

  ## Single class/
  iris2 <- iris
  iris2$spec <- factor(as.character(iris2$Species) == "setosa")
  iris2 <- iris2[, -which(colnames(iris2) == "Species")]

  rf <- rfsrc(spec ~ ., iris2, importance = TRUE)

  gg_dta <- gg_vimp(rf)

  expect_s3_class(gg_dta, "gg_vimp")

  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))

  ## RandomForest case
  rf_iris <- randomForest::randomForest(Species ~ .,
                                        data = iris)

  gg_dta <- gg_vimp(rf_iris)

  expect_s3_class(gg_dta, "gg_vimp")

  rf_iris_noimp <- randomForest::randomForest(Species ~ .,
                                              data = iris,
                                              importance = FALSE)
  rf_iris_noimp$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rf_iris_noimp))
  expect_s3_class(gg_dta, "gg_vimp")

  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))


  gg_dta <- gg_vimp(rf_iris, which.outcome = 1)

  expect_s3_class(gg_dta, "gg_vimp")


  expect_s3_class(gg_dta, "gg_vimp")
  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))
  gg_dta <- gg_vimp(rf_iris, which.outcome = NULL)
})


test_that("gg_vimp survival", {
  dta <- new.env()
  data(pbc, package = "randomForestSRC",
       envir = dta)
  pbc <- dta$pbc
  # For whatever reason, the age variable is in days... makes no sense to me
  for (ind in seq_len(dim(pbc)[2])) {
    if (!is.factor(pbc[, ind])) {
      if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
        if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
      }
    } else {
      if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
        if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
        if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
      }
    }
    if (!is.logical(pbc[, ind]) &
        length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
      pbc[, ind] <- factor(pbc[, ind])
    }
  }
  # Convert age to years
  pbc$age <- pbc$age / 364.24

  pbc$years <- pbc$days / 364.24
  pbc <- pbc[, -which(colnames(pbc) == "days")]
  pbc$treatment <- as.numeric(pbc$treatment)
  pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
  pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
  pbc$treatment <- factor(pbc$treatment)

  cat("pbc: rfsrc\n")
  dta_train <- pbc[-which(is.na(pbc$treatment)), ]
  # Create a test set from the remaining patients
  pbc_test <- pbc[which(is.na(pbc$treatment)), ]

  rfsrc_pbc <- randomForestSRC::rfsrc(
    Surv(years, status) ~ .,
    dta_train,
    nsplit = 10,
    na.action = "na.impute",
    importance = TRUE,
    tree.err = TRUE
  )
  # Test the cached forest type
  expect_s3_class(rfsrc_pbc, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_pbc)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rfsrc_pbc$importance, decreasing =
                                             TRUE)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, nvar = 5)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  expect_s3_class(plot(gg_dta, relative = TRUE), "ggplot")

  # Test cutting the size down
  expect_s3_class(gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10), "gg_vimp")
  expect_equal(nrow(gg_dta), 10)
  expect_s3_class(plot(gg_dta), "ggplot")

  # Test the relative vimp output and plotting
  expect_s3_class(
    gg_dta <- gg_vimp(rfsrc_pbc, relative = TRUE),
    "gg_vimp"
  )
  expect_s3_class(plot(gg_dta), "ggplot")

  expect_s3_class(
    gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10, relative = TRUE),
    "gg_vimp"
  )
  expect_s3_class(plot(gg_dta), "ggplot")

  # Test importance calculations.
  # If the forest does not have importance
  rfsrc_pbc$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rfsrc_pbc))
  expect_s3_class(gg_dta, "gg_vimp")
  expect_s3_class(plot(gg_dta), "ggplot")

})

test_that("gg_vimp regression", {
  ## Load the cached forest
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = boston,
                                         importance = TRUE)
  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rfsrc_boston$importance, decreasing =
                                             TRUE)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, relative = TRUE)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")


  cls <- sapply(Boston, class)
  #
  lbls <-
    #crim
    c(
      "Crime rate by town.",
      # zn
      "Proportion of residential land zoned for lots over 25,000 sq.ft.",
      # indus
      "Proportion of non-retail business acres per town.",
      # chas
      "Charles River (tract bounds river).",
      # nox
      "Nitrogen oxides concentration (10 ppm).",
      # rm
      "Number of rooms per dwelling.",
      # age
      "Proportion of units built prior to 1940.",
      # dis
      "Distances to Boston employment center.",
      # rad
      "Accessibility to highways.",
      # tax
      "Property-tax rate per $10,000.",
      # ptratio
      "Pupil-teacher ratio by town.",
      # black
      "Proportion of blacks by town.",
      # lstat
      "Lower status of the population (percent).",
      # medv
      "Median value of homes ($1000s)."
    )

  # Build a table for data description
  dta_labs <-
    data.frame(cbind(
      Variable = names(cls),
      Description = lbls,
      type = cls
    ))

  # Build a named vector for labeling figures later/
  st_labs <- as.character(dta_labs$Description)
  names(st_labs) <- names(cls)

  ## Test plotting the rfsrc object
  gg_plt <- plot.gg_vimp(
    rfsrc_boston,
    lbls = st_labs,
    relative = TRUE,
    bars = rfsrc_boston$xvar.names
  )
  expect_s3_class(gg_plt, "ggplot")

  rf_boston <- randomForest::randomForest(medv ~ ., Boston)
  gg_dta <- gg_vimp(rf_boston)
  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rf_boston$importance, decreasing =
                                             TRUE)))

  gg_plt <- plot(gg_dta)
  expect_s3_class(gg_plt, "ggplot")

  rf_boston_noimp <- randomForest::randomForest(medv ~ ., Boston, importance = FALSE)
  rf_boston_noimp$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rf_boston_noimp))
  expect_s3_class(gg_dta, "gg_vimp")

})

test_that("gg_vimp.randomForest regression: vimp column present even when importance is IncNodePurity", {
  # Guard test: when randomForest stores importance as IncNodePurity (not X.IncMSE),
  # gg_vimp must still produce a 'vimp' column so plot.gg_vimp and the positive
  # flag work correctly.
  data(Boston, package = "MASS")
  # importance = FALSE (default) → $importance has only IncNodePurity
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
                                          importance = FALSE)
  gg_dta <- gg_vimp(rf_boston)

  expect_s3_class(gg_dta, "gg_vimp")
  expect_true("vimp" %in% colnames(gg_dta),
              info = "vimp column must exist regardless of original column name")
  expect_false(any(is.na(gg_dta$vimp)),
               info = "vimp values must not be NA")
  expect_true("positive" %in% colnames(gg_dta))
  expect_s3_class(plot(gg_dta), "ggplot")
})

# testthat for gg_vimp function

test_that("gg_vimp classifications", {
  ## Load the cached forest
  data(iris, package = "datasets")
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE
  )
  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_equal(rfsrc_iris$family, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Grab only one class... by number.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = 2)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by number - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = 0)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by name - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = "all")

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  # Grab only one class... by name - for the overall model.
  gg_dta <- gg_vimp(rfsrc_iris, which.outcome = "setosa")

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Grab only one class... by name - that doesn't exist.
  expect_error(gg_vimp(rfsrc_iris, which.outcome = "nothing special"))

  # Grab only one class... by number - that doesn't exist.
  expect_error(gg_vimp(rfsrc_iris, which.outcome = 200))

  ## Single class/
  iris2 <- iris
  iris2$spec <- factor(as.character(iris2$Species) == "setosa")
  iris2 <- iris2[, -which(colnames(iris2) == "Species")]

  rf <- randomForestSRC::rfsrc(spec ~ ., iris2, importance = TRUE)

  gg_dta <- gg_vimp(rf)

  expect_s3_class(gg_dta, "gg_vimp")

  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))

  ## RandomForest case
  rf_iris <- randomForest::randomForest(Species ~ .,
                                        data = iris)

  gg_dta <- gg_vimp(rf_iris)

  expect_s3_class(gg_dta, "gg_vimp")

  rf_iris_noimp <- randomForest::randomForest(Species ~ .,
                                              data = iris,
                                              importance = FALSE)
  rf_iris_noimp$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rf_iris_noimp))
  expect_s3_class(gg_dta, "gg_vimp")

  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))


  gg_dta <- gg_vimp(rf_iris, which.outcome = 1)

  expect_s3_class(gg_dta, "gg_vimp")


  expect_s3_class(gg_dta, "gg_vimp")
  # Test passing in the wrong object
  expect_error(gg_vimp(gg_dta))
  expect_error(gg_vimp.rfsrc(gg_dta))
  gg_dta <- gg_vimp(rf_iris, which.outcome = NULL)
})


test_that("gg_vimp survival", {
  dta <- new.env()
  data(pbc, package = "randomForestSRC",
       envir = dta)
  pbc <- dta$pbc
  # For whatever reason, the age variable is in days... makes no sense to me
  for (ind in seq_len(dim(pbc)[2])) {
    if (!is.factor(pbc[, ind])) {
      if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
        if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
      }
    } else {
      if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
        if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
        if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
          pbc[, ind] <- as.logical(pbc[, ind])
        }
      }
    }
    if (!is.logical(pbc[, ind]) &
        length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
      pbc[, ind] <- factor(pbc[, ind])
    }
  }
  # Convert age to years
  pbc$age <- pbc$age / 364.24

  pbc$years <- pbc$days / 364.24
  pbc <- pbc[, -which(colnames(pbc) == "days")]
  pbc$treatment <- as.numeric(pbc$treatment)
  pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
  pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
  pbc$treatment <- factor(pbc$treatment)

  cat("pbc: rfsrc\n")
  dta_train <- pbc[-which(is.na(pbc$treatment)), ]
  # Create a test set from the remaining patients
  pbc_test <- pbc[which(is.na(pbc$treatment)), ]

  rfsrc_pbc <- randomForestSRC::rfsrc(
    Surv(years, status) ~ .,
    dta_train,
    nsplit = 10,
    na.action = "na.impute",
    importance = TRUE,
    tree.err = TRUE
  )
  # Test the cached forest type
  expect_s3_class(rfsrc_pbc, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_pbc)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rfsrc_pbc$importance, decreasing =
                                             TRUE)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, nvar = 5)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  expect_s3_class(plot(gg_dta, relative = TRUE), "ggplot")

  # Test cutting the size down
  expect_s3_class(gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10), "gg_vimp")
  expect_equal(nrow(gg_dta), 10)
  expect_s3_class(plot(gg_dta), "ggplot")

  # Test the relative vimp output and plotting
  expect_s3_class(
    gg_dta <- gg_vimp(rfsrc_pbc, relative = TRUE),
    "gg_vimp"
  )
  expect_s3_class(plot(gg_dta), "ggplot")

  expect_s3_class(
    gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10, relative = TRUE),
    "gg_vimp"
  )
  expect_s3_class(plot(gg_dta), "ggplot")

  # Test importance calculations.
  # If the forest does not have importance
  rfsrc_pbc$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rfsrc_pbc))
  expect_s3_class(gg_dta, "gg_vimp")
  expect_s3_class(plot(gg_dta), "ggplot")

})

test_that("gg_vimp regression", {
  ## Load the cached forest
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = boston,
                                         importance = TRUE)
  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_vimp(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_vimp")

  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rfsrc_boston$importance, decreasing =
                                             TRUE)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, relative = TRUE)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")


  cls <- sapply(Boston, class)
  #
  lbls <-
    #crim
    c(
      "Crime rate by town.",
      # zn
      "Proportion of residential land zoned for lots over 25,000 sq.ft.",
      # indus
      "Proportion of non-retail business acres per town.",
      # chas
      "Charles River (tract bounds river).",
      # nox
      "Nitrogen oxides concentration (10 ppm).",
      # rm
      "Number of rooms per dwelling.",
      # age
      "Proportion of units built prior to 1940.",
      # dis
      "Distances to Boston employment center.",
      # rad
      "Accessibility to highways.",
      # tax
      "Property-tax rate per $10,000.",
      # ptratio
      "Pupil-teacher ratio by town.",
      # black
      "Proportion of blacks by town.",
      # lstat
      "Lower status of the population (percent).",
      # medv
      "Median value of homes ($1000s)."
    )

  # Build a table for data description
  dta_labs <-
    data.frame(cbind(
      Variable = names(cls),
      Description = lbls,
      type = cls
    ))

  # Build a named vector for labeling figures later/
  st_labs <- as.character(dta_labs$Description)
  names(st_labs) <- names(cls)

  ## Test plotting the rfsrc object
  gg_plt <- plot.gg_vimp(
    rfsrc_boston,
    lbls = st_labs,
    relative = TRUE,
    bars = rfsrc_boston$xvar.names
  )
  expect_s3_class(gg_plt, "ggplot")

  rf_boston <- randomForest::randomForest(medv ~ ., Boston)
  gg_dta <- gg_vimp(rf_boston)
  # Test varselect is the same
  expect_equal(gg_dta$vimp, as.vector(sort(rf_boston$importance, decreasing =
                                             TRUE)))

  gg_plt <- plot(gg_dta)
  expect_s3_class(gg_plt, "ggplot")

  rf_boston_noimp <- randomForest::randomForest(medv ~ ., Boston, importance = FALSE)
  rf_boston_noimp$importance <- NULL
  expect_warning(gg_dta <- gg_vimp(rf_boston_noimp))
  expect_s3_class(gg_dta, "gg_vimp")

})

test_that("gg_vimp.rfsrc single-outcome: positive flag correctly uses the VIMP column", {
  # Regression test for the bug where gg_dta$vimp was accessed but the column
  # is named "VIMP" (uppercase) in single-outcome rfsrc fits, leaving positive
  # always TRUE even for variables with non-positive VIMP.
  data(airquality)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                               ntree = 50, importance = TRUE)
  gg_dta <- gg_vimp(rf)

  expect_s3_class(gg_dta, "gg_vimp")
  expect_true("positive" %in% colnames(gg_dta))
  # The positive flag should correctly reflect the sign of VIMP:
  # variables with vimp <= 0 must have positive == FALSE.
  vimp_col <- intersect(c("vimp", "VIMP"), colnames(gg_dta))[1]
  neg_mask <- gg_dta[[vimp_col]] <= 0
  if (any(neg_mask, na.rm = TRUE)) {
    expect_true(all(!gg_dta$positive[which(neg_mask)]),
                info = "Variables with non-positive VIMP must have positive == FALSE")
  }
  # Even if all VIMP happen to be positive here, the column must exist.
  expect_true(length(unique(is.na(gg_dta$positive))) <= 1)
  expect_s3_class(plot(gg_dta), "ggplot")
})

test_that("gg_vimp.randomForest regression: vimp column present even when importance is IncNodePurity", {
  # Guard test: when randomForest stores importance as IncNodePurity (not X.IncMSE),
  # gg_vimp must still produce a 'vimp' column so plot.gg_vimp and the positive
  # flag work correctly.
  data(Boston, package = "MASS")
  # importance = FALSE (default) → $importance has only IncNodePurity
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
                                          importance = FALSE)
  gg_dta <- gg_vimp(rf_boston)

  expect_s3_class(gg_dta, "gg_vimp")
  expect_true("vimp" %in% colnames(gg_dta),
              info = "vimp column must exist regardless of original column name")
  expect_false(any(is.na(gg_dta$vimp)),
               info = "vimp values must not be NA")
  expect_true("positive" %in% colnames(gg_dta))
  expect_s3_class(plot(gg_dta), "ggplot")
})

data(Boston, package = "MASS")
data(iris, package = "datasets")

set.seed(100)

iris_subset <- droplevels(subset(iris, Species != "setosa"))


test_that(".rf_recover_model_frame rebuilds subsetted data", {
  skip_if_not_installed("randomForest")
  rf_subset <- randomForest::randomForest(
    Species ~ .,
    data = iris_subset,
    ntree = 50,
    keep.forest = TRUE
  )
  info <- ggRandomForests:::`.rf_recover_model_frame`(rf_subset)
  expect_type(info, "list")
  expect_true(all(c("Sepal.Length", "Sepal.Width") %in% names(info$model_frame)))
  expect_equal(info$response_name, "Species")
})


test_that(".rf_training_curve returns trajectories for both families", {
  skip_if_not_installed("randomForest")
  rf_cls <- randomForest::randomForest(
    Species ~ .,
    data = iris,
    ntree = 40,
    keep.forest = TRUE
  )
  rf_reg <- randomForest::randomForest(
    medv ~ .,
    data = Boston,
    ntree = 40,
    keep.forest = TRUE
  )

  cls_curve <- ggRandomForests:::`.rf_training_curve`(rf_cls)
  reg_curve <- ggRandomForests:::`.rf_training_curve`(rf_reg)

  expect_equal(length(cls_curve), rf_cls$ntree)
  expect_equal(length(reg_curve), rf_reg$ntree)
  expect_true(all(is.finite(reg_curve)))
  expect_true(all(cls_curve >= 0 & cls_curve <= 1))
})


test_that("training curves are skipped when forests are discarded", {
  skip_if_not_installed("randomForest")
  rf_plain <- randomForest::randomForest(
    Species ~ .,
    data = iris,
    ntree = 25,
    keep.forest = FALSE
  )
  expect_warning(
    gg_plain <- gg_error(rf_plain, training = TRUE),
    "Training error curve is unavailable"
  )
  expect_false("train" %in% names(gg_plain))
})


test_that("gg_vimp falls back to placeholder when importance is unavailable", {
  skip_if_not_installed("randomForest")
  rf_noimp <- randomForest::randomForest(
    medv ~ .,
    data = Boston,
    ntree = 60,
    keep.forest = FALSE,
    importance = FALSE
  )
  rf_noimp$importance <- NULL
  expect_warning(gg_na <- gg_vimp(rf_noimp), "Returning NA values")
  expect_true(all(is.na(gg_na$vimp)))
  expect_true(all(is.na(gg_na$rel_vimp)))
})

# Regression tests for the empty-figure bug class.
#
# A `plot.gg_*()` method can return a ggplot whose data frame has zero rows,
# or whose layer data computes to zero rows after aesthetic mapping. That
# produces a figure with axes but no points or lines — which is what users
# saw in the survival vignette when the data transform's column names did
# not match what the plot method expected.
#
# These tests use `ggplot2::layer_data()` to inspect what each plot would
# actually draw. layer_data() runs the full ggplot build pipeline and returns
# the post-mapping data frame, so a zero-row result here means the figure
# would render empty.

Surv <- survival::Surv  # nolint: object_name_linter

# Helper: number of rows ggplot2 would actually render for a given layer.
expect_layer_nonempty <- function(p, layer = 1L, label = NULL) {
  testthat::expect_s3_class(p, "ggplot")
  ld <- ggplot2::layer_data(p, layer)
  testthat::expect_true(
    nrow(ld) > 0,
    info = sprintf(
      "%slayer %d data should have rows (got %d). cols: %s",
      if (is.null(label)) "" else paste0(label, ": "),
      layer, nrow(ld), paste(colnames(ld), collapse = ",")
    )
  )
  invisible(ld)
}

# Helper: every aesthetic in `mapping_keys` should resolve to a column with
# more than one unique value somewhere in the layer data. The empty-figure
# bug presented as columns mapped to axis labels because the data frame had
# only one literal value per "variable"/"value"/etc. column.
expect_layer_has_variation <- function(p, layer = 1L, mapping_keys) {
  ld <- ggplot2::layer_data(p, layer)
  for (k in mapping_keys) {
    testthat::expect_true(
      k %in% colnames(ld),
      info = sprintf("layer %d missing aesthetic '%s'", layer, k)
    )
    testthat::expect_gt(
      length(unique(ld[[k]])), 1,
      label = sprintf("variation in layer %d aes '%s'", layer, k)
    )
  }
  invisible(ld)
}

# ----------------------------------------------------------------------------
# gg_rfsrc — survival
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc survival (no CI) renders many step curves", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)

  gg <- gg_rfsrc(rf)
  expect_true(all(c("variable", "value", "obs_id", "event") %in% colnames(gg)))
  expect_type(gg$variable, "double")
  expect_type(gg$value, "double")

  p <- plot(gg)
  expect_layer_nonempty(p)
  expect_layer_has_variation(p, mapping_keys = c("x", "y"))
})

test_that("plot.gg_rfsrc survival (CI) renders ribbon + median curve", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_rfsrc(rf, conf.int = .95)
  expect_true(all(c("value", "lower", "upper", "median", "mean") %in% colnames(gg)))
  p <- plot(gg)
  # Layer 1 is the ribbon, layer 2 is the median step line.
  expect_layer_nonempty(p, layer = 1L)
  expect_layer_nonempty(p, layer = 2L)
})

test_that("plot.gg_rfsrc survival (by group) renders per-group curves", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_rfsrc(rf, by = "trt")
  expect_true("group" %in% colnames(gg))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  expect_gt(length(unique(ld$group)), 1)
})

# ----------------------------------------------------------------------------
# gg_rfsrc — regression
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc regression renders jitter + boxplot with data", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50)
  gg <- gg_rfsrc(rf)
  expect_true(all(c("yhat", "Ozone") %in% colnames(gg)))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), nrow(gg))
  expect_gt(length(unique(ld$y)), 1)
})

# ----------------------------------------------------------------------------
# gg_rfsrc — classification
# ----------------------------------------------------------------------------
test_that("plot.gg_rfsrc multi-class renders one row per (obs, class)", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
  gg <- gg_rfsrc(rf)
  # Multi-class: one column per class plus "y"
  expect_true(all(levels(iris$Species) %in% colnames(gg)))
  expect_true("y" %in% colnames(gg))
  p <- plot(gg)
  ld <- expect_layer_nonempty(p)
  # Three classes × 150 observations => 450 rows after pivot
  expect_equal(nrow(ld), nrow(iris) * nlevels(iris$Species))
})

# ----------------------------------------------------------------------------
# gg_partial_rfsrc — survival regression test for the partial.type fix
# ----------------------------------------------------------------------------
test_that("gg_partial_rfsrc survival passes partial.type and produces data", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t90 <- ti[which.min(abs(ti - 90))]

  # Single time point.
  expect_no_error({
    pd <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, n_eval = 8)
  })
  expect_s3_class(pd, "gg_partial_rfsrc")
  expect_true(nrow(pd$continuous) > 0)
  expect_true(all(c("x", "yhat", "name", "time") %in% colnames(pd$continuous)))
  # Survival plot path expects a "time" column.
  p <- plot(pd)
  expect_layer_nonempty(p)
})

test_that("gg_partial_rfsrc survival multi-time expands to long form", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t30 <- ti[which.min(abs(ti - 30))]
  t90 <- ti[which.min(abs(ti - 90))]

  pd <- gg_partial_rfsrc(rf, xvar.names = "age",
                         partial.time = c(t30, t90), n_eval = 8)
  # Two time points × evaluation grid: long form, each (x, time) one row.
  expect_equal(length(unique(pd$continuous$time)), 2L)
  per_time <- table(pd$continuous$time)
  expect_true(all(per_time == per_time[[1]]))

  p <- plot(pd)
  ld <- expect_layer_nonempty(p)
  # The fixed plot maps time → colour, so we should see two groups.
  expect_gte(length(unique(ld$colour)), 2)
})

test_that("gg_partial_rfsrc partial.type rejects bad values", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  expect_error(
    gg_partial_rfsrc(rf, xvar.names = "age", partial.type = "bogus"),
    regexp = "should be one of"
  )
})

test_that("plot.gg_partial_rfsrc y-axis adapts to partial.type", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  ti  <- rf$time.interest
  t90 <- ti[which.min(abs(ti - 90))]

  # Default ("surv") => "Predicted Survival"
  pd_s <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, n_eval = 6)
  expect_equal(attr(pd_s, "partial.type"), "surv")
  p_s <- plot(pd_s)
  expect_equal(p_s$labels$y, "Predicted Survival")

  # "chf" => "Predicted CHF"
  pd_c <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.time = t90, partial.type = "chf",
                           n_eval = 6)
  expect_equal(attr(pd_c, "partial.type"), "chf")
  p_c <- plot(pd_c)
  expect_equal(p_c$labels$y, "Predicted CHF")

  # "mort" => "Predicted Mortality"; mort returns one value per x (no time dim),
  # so the survival "time" branch will not engage — we only assert the
  # attribute round-trips.
  pd_m <- gg_partial_rfsrc(rf, xvar.names = "age",
                           partial.type = "mort",
                           n_eval = 6)
  expect_equal(attr(pd_m, "partial.type"), "mort")
})

test_that("plot.gg_partial_rfsrc preserves full-precision time grouping", {
  # Distinct times that round to the same 2-dp value must not collapse into a
  # single line. Build a synthetic gg_partial_rfsrc object with two such times.
  cont <- data.frame(
    x    = rep(c(1, 2, 3), times = 2),
    yhat = c(0.9, 0.8, 0.7, 0.5, 0.4, 0.3),
    name = "x",
    time = rep(c(1.001, 1.002), each = 3)
  )
  obj <- structure(
    list(
      continuous  = cont,
      categorical = data.frame(x = character(0), yhat = numeric(0),
                               name = character(0), time = integer(0))
    ),
    class = "gg_partial_rfsrc",
    partial.type = "surv"
  )
  p  <- plot(obj)
  ld <- ggplot2::layer_data(p, 1L)
  # Two distinct full-precision time horizons => two distinct groups, even
  # though both round to "1" at 2-dp precision.
  expect_equal(length(unique(ld$group)), 2L)
  expect_equal(length(unique(ld$colour)), 2L)
})

# ----------------------------------------------------------------------------
# gg_partial_rfsrc — regression
# ----------------------------------------------------------------------------
test_that("plot.gg_partial_rfsrc regression renders a non-empty line plot", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                               ntree = 50)
  pd <- gg_partial_rfsrc(rf, xvar.names = "Wind", n_eval = 8)
  p <- plot(pd)
  expect_layer_nonempty(p)
})

# ----------------------------------------------------------------------------
# gg_error
# ----------------------------------------------------------------------------
test_that("plot.gg_error single-outcome renders points or a line", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute",
                               ntree = 50, tree.err = TRUE,
                               block.size = 5)
  gg <- gg_error(rf)
  expect_true(all(c("ntree", "error") %in% colnames(gg)))
  p <- plot(gg)
  expect_layer_nonempty(p)
})

test_that("plot.gg_error multi-class pivots and colours by class", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Species ~ ., data = iris,
                               ntree = 50, tree.err = TRUE,
                               block.size = 5)
  p <- plot(gg_error(rf))
  ld <- expect_layer_nonempty(p)
  expect_gte(length(unique(ld$colour)), 2)
})

# ----------------------------------------------------------------------------
# gg_vimp
# ----------------------------------------------------------------------------
test_that("plot.gg_vimp renders one bar per variable", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50,
                               importance = TRUE)
  p <- plot(gg_vimp(rf))
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), length(rf$xvar.names))
})

test_that("plot.gg_vimp produces a single merged 'VIMP > 0' legend", {
  # Regression: previously the bar geom mapped both `fill` and `color` to
  # `positive` but only the fill legend got a custom title via labs(). ggplot
  # rendered two legends -- one labelled "VIMP > 0" (fill) and one labelled
  # "positive" (column name). Setting matching titles on both aesthetics
  # merges them into a single legend.
  #
  # We build a synthetic gg_vimp object rather than fit a real forest so the
  # test is deterministic across platforms (CI runners regularly produced
  # all-positive VIMP for the PBC dataset, which would skip the fill branch
  # we want to exercise here).
  gg <- structure(
    data.frame(
      vars     = c("a", "b", "c", "d"),
      vimp     = c(0.20, 0.05, -0.01, -0.04),
      rel_vimp = c(1.00, 0.25, 0.05,  0.20),
      positive = c(TRUE, TRUE, FALSE, FALSE)
    ),
    class = c("gg_vimp", "data.frame")
  )

  # Sanity: both signs present.
  expect_true(length(unique(gg$positive)) > 1)

  p <- plot(gg)
  expect_equal(p$labels$fill,   "VIMP > 0")
  expect_equal(p$labels$colour, "VIMP > 0")
  # The two aesthetic legend titles must match for ggplot to merge them.
  expect_identical(p$labels$fill, p$labels$colour)
})

test_that("plot.gg_vimp produces filled bars even when all VIMP are positive", {
  # Regression: previously the all-positive branch mapped only `color`,
  # leaving the bars hollow / outline-only and emitting "Ignoring unknown
  # labels: fill : 'VIMP > 0'" because the function-level labs(fill = ...)
  # had nothing to bind to. The fix maps fill + color unconditionally so the
  # bars are always filled and the legend title applies cleanly.
  gg <- structure(
    data.frame(
      vars     = c("a", "b", "c"),
      vimp     = c(0.30, 0.10, 0.05),
      rel_vimp = c(1.00, 0.33, 0.17),
      positive = c(TRUE, TRUE, TRUE)
    ),
    class = c("gg_vimp", "data.frame")
  )
  expect_equal(length(unique(gg$positive)), 1L)

  warns <- character()
  p <- withCallingHandlers(
    plot(gg),
    warning = function(w) {
      warns <<- c(warns, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )
  expect_false(any(grepl("Ignoring unknown labels", warns)),
               info = sprintf("got warnings: %s", paste(warns, collapse = " | ")))

  ld <- ggplot2::layer_data(p, 1L)
  # Bars must be filled, not hollow.
  expect_true("fill" %in% colnames(ld))
  expect_true(all(!is.na(ld$fill)))
})

# ----------------------------------------------------------------------------
# gg_variable
# ----------------------------------------------------------------------------
test_that("plot.gg_variable survival single xvar has a point per observation", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                               ntree = 50)
  gg <- gg_variable(rf, time = 90)
  expect_true(all(c("event", "yhat", "time") %in% colnames(gg)))
  p <- plot(gg, xvar = "age")
  ld <- expect_layer_nonempty(p)
  expect_equal(nrow(ld), nrow(veteran))
})

test_that("plot.gg_variable regression renders a scatter with smooth", {
  set.seed(42)
  rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                               na.action = "na.impute", ntree = 50)
  gg <- gg_variable(rf)
  p <- plot(gg, xvar = "Wind", smooth = FALSE)
  expect_layer_nonempty(p)
})

##=============================================================================
#' Recover original variable names from varpro one-hot encoded feature names
#'
#' \code{varpro} one-hot encodes factor variables, appending a numeric suffix
#' for each level (e.g., \code{sex} becomes \code{sex0} and \code{sex1}).
#' This function strips those suffixes iteratively until every name in
#' \code{varpro_names} can be matched back to a column in \code{dataset}.
#'
#' @param varpro_names character vector of names as output by varpro (may
#'   include one-hot encoded suffixed names such as \code{"sex0"}, \code{"sex1"})
#' @param dataset the original data frame passed to varpro, used to look up
#'   valid column names
#'
#' @return character vector of unique original variable names (no suffixes)
#'
#' @seealso \code{\link{gg_partialpro}}
#'
#' @importFrom stringr str_sub
#'
#' @examples
#' ## ------------------------------------------------------------------
#' ## Simple case: one continuous variable + one binary factor
#' ## ------------------------------------------------------------------
#' ds <- data.frame(age = c(25, 30, 45), sex = c("M", "F", "M"))
#'
#' # varpro one-hot encodes 'sex' into 'sex0' and 'sex1'
#' varpro_names <- c("age", "sex0", "sex1")
#' varpro_feature_names(varpro_names, ds)
#' # Returns: c("age", "sex")
#'
#' ## ------------------------------------------------------------------
#' ## Multi-level factor: three-level 'group' variable
#' ## ------------------------------------------------------------------
#' ds2 <- data.frame(score = 1:6,
#'                   group = factor(rep(c("A", "B", "C"), 2)))
#'
#' # varpro appends 0/1/2 for each level
#' vn2 <- c("score", "group0", "group1", "group2")
#' varpro_feature_names(vn2, ds2)
#' # Returns: c("score", "group")
#'
#' ## ------------------------------------------------------------------
#' ## Already-clean names pass through unchanged
#' ## ------------------------------------------------------------------
#' ds3 <- data.frame(x = 1:5, y = 1:5)
#' varpro_feature_names(c("x", "y"), ds3)
#' # Returns: c("x", "y")
#'
#' @export
varpro_feature_names <- function(varpro_names, dataset) {
  # Names that already match a column in dataset — keep as-is
  inc_set <- varpro_names[which(varpro_names %in% colnames(dataset))]

  # Names that do not yet match any column — need suffix stripping
  one_set <- varpro_names[which(!varpro_names %in% colnames(dataset))]

  ## Iteratively strip the last character of each unmatched name until every
  ## name either resolves to a column or the set is exhausted.
  while (length(one_set) > 0) {
    # Drop the trailing character from each remaining unmatched name
    orig <- unlist(lapply(one_set, stringr::str_sub, 1, -2))

    # Accept names that now match a dataset column
    inc_set <- union(inc_set, orig[which(orig %in% colnames(dataset))])

    # Continue stripping names that still don't match
    one_set <- orig[which(!orig %in% colnames(dataset))]
  }
  return(inc_set)
}

# Tests for varpro_feature_names

test_that("varpro_feature_names returns exact matches unchanged", {
  dataset <- data.frame(age = 1:5, sex = 1:5, weight = 1:5)
  varpro_names <- c("age", "sex", "weight")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_true("age" %in% result)
  expect_true("sex" %in% result)
  expect_true("weight" %in% result)
})

test_that("varpro_feature_names resolves single-suffix one-hot names", {
  dataset <- data.frame(
    age = 1:5,
    sex = c("M", "F", "M", "F", "M"),
    weight = 1:5
  )
  # sex was one-hot encoded to sex0 and sex1
  varpro_names <- c("age", "sex0", "sex1")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_true("age" %in% result)
  expect_true("sex" %in% result)
  # Encoded versions should NOT appear
  expect_false("sex0" %in% result)
  expect_false("sex1" %in% result)
})

test_that("varpro_feature_names resolves multi-level one-hot names", {
  # grade0, grade1, grade2 → grade
  dataset <- data.frame(
    age = 1:10,
    grade = letters[1:10]
  )
  varpro_names <- c("age", "grade0", "grade1", "grade2")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_true("age" %in% result)
  expect_true("grade" %in% result)
  expect_false("grade0" %in% result)
  expect_false("grade1" %in% result)
  expect_false("grade2" %in% result)
})

test_that("varpro_feature_names handles deeply nested suffixes", {
  # groupABC → group (three characters to strip)
  dataset <- data.frame(group = 1:5, value = 1:5)
  varpro_names <- c("value", "groupABC")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_true("value" %in% result)
  expect_true("group" %in% result)
  expect_false("groupABC" %in% result)
})

test_that("varpro_feature_names returns only unique names", {
  dataset <- data.frame(sex = 1:5, age = 1:5)
  # Both sex0 and sex1 resolve to sex — should appear once
  varpro_names <- c("sex0", "sex1", "age")

  result <- varpro_feature_names(varpro_names, dataset)

  # sex appears exactly once
  expect_equal(sum(result == "sex"), 1)
  expect_true("age" %in% result)
})

test_that("varpro_feature_names with all direct matches returns them", {
  dataset <- data.frame(a = 1:3, b = 1:3, c = 1:3)
  varpro_names <- c("a", "b", "c")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_equal(sort(result), sort(varpro_names))
})

test_that("varpro_feature_names with empty input returns empty", {
  dataset <- data.frame(a = 1:3)
  result <- varpro_feature_names(character(0), dataset)

  expect_length(result, 0)
})

test_that("varpro_feature_names mix of direct and encoded names", {
  dataset <- data.frame(
    age = 1:20,
    smoking = 1:20,
    region = 1:20
  )
  # smoking is encoded, age and region are direct
  varpro_names <- c("age", "smoking0", "smoking1", "region")

  result <- varpro_feature_names(varpro_names, dataset)

  expect_true("age" %in% result)
  expect_true("smoking" %in% result)
  expect_true("region" %in% result)
  expect_false("smoking0" %in% result)
  expect_false("smoking1" %in% result)
})

#' Display the ggRandomForests NEWS file
#'
#' Opens the package change log in the system pager so users can read the
#' version history without leaving their R session. The function reads
#' \code{NEWS.md} from the installed package root (the canonical change log
#' that R also surfaces via \code{utils::news()}). \code{inst/NEWS} was
#' removed in v2.7.1 to eliminate the duplicate-source-of-truth maintenance
#' hole that left the legacy file frozen at v2.4.0; if any installation
#' still ships an \code{inst/NEWS}, this function falls back to it.
#'
#' @param ... Currently unused; reserved for future arguments.
#'
#' @return Called for its side-effect of opening the NEWS file in the system
#'   pager (\code{file.show}). Returns \code{invisible(NULL)}.
#'
#' @keywords internal
ggrandomforests.news <- function(...) {
  newsfile <- system.file("NEWS.md", package = "ggRandomForests")
  if (!nzchar(newsfile)) {
    # Fallback for legacy installs that still bundle inst/NEWS.
    newsfile <- system.file("NEWS", package = "ggRandomForests")
  }
  if (!nzchar(newsfile)) {
    warning("Could not locate a NEWS file for ggRandomForests.",
            call. = FALSE)
    return(invisible(NULL))
  }
  file.show(newsfile)
}

# Tests for ggrandomforests.news()
#
# v2.7.1 consolidated the helper to read NEWS.md (the canonical change log
# R also exposes via utils::news()) instead of the legacy inst/NEWS plain
# text file. These tests guard against silent drift back to a separate
# hand-maintained source.

test_that("NEWS.md is bundled with the installed package", {
  nf <- system.file("NEWS.md", package = "ggRandomForests")
  expect_true(nzchar(nf))
  expect_true(file.exists(nf))
})

test_that("Bundled NEWS.md reflects the installed package version", {
  # Reading NEWS.md and finding the DESCRIPTION version in it ensures the
  # change-log entry was written before the version bump landed — i.e. that
  # users running ggrandomforests.news() see the correct release.
  ver <- as.character(utils::packageVersion("ggRandomForests"))
  nf  <- system.file("NEWS.md", package = "ggRandomForests")
  txt <- readLines(nf, warn = FALSE)
  expect_true(any(grepl(ver, txt, fixed = TRUE)),
              info = sprintf("Installed version %s not found in %s", ver, nf))
})

test_that("ggrandomforests.news is callable without error", {
  expect_error(ggrandomforests.news(), NA)
})

test_that("inst/NEWS is no longer the source of truth", {
  # Defensive: if a future change re-introduces inst/NEWS, this test won't
  # fail (the helper still falls back to it) -- but we explicitly assert
  # NEWS.md takes priority when both exist.
  nf_md <- system.file("NEWS.md", package = "ggRandomForests")
  expect_true(nzchar(nf_md))
})

##=============================================================================
##=============================================================================
#' Split varpro partial dependence data into continuous or categorical datasets
#'
#' Takes the list returned by \code{varpro::partialpro} and separates variables
#' into two data frames: one for continuous predictors (parametric, non-
#' parametric, and causal effect curves) and one for categorical predictors
#' (one row per observation per category level).
#'
#' The split is governed by \code{cat_limit}: a variable is treated as
#' continuous when \code{length(xvirtual) > cat_limit}; otherwise it is
#' treated as categorical and the per-category rows are stacked.
#'
#' @param part_dta partial plot data from \code{varpro::partialpro}.  Each
#'   element of the list must contain fields \code{xvirtual}, \code{xorg},
#'   \code{yhat.par}, \code{yhat.nonpar}, and \code{yhat.causal}.
#' @param nvars how many variables (list elements) to process.  Defaults to
#'   all variables in \code{part_dta}.
#' @param cat_limit Variables with \code{length(xvirtual)} \eqn{\le}
#'   \code{cat_limit} are treated as categorical.  Default \code{10}.
#' @param model a label applied to all rows.  Useful when combining results
#'   from multiple models in a single figure.
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{continuous}{data.frame with columns \code{variable},
#'       \code{parametric}, \code{nonparametric}, \code{causal}, \code{name}
#'       (and optionally \code{model})}
#'     \item{categorical}{data.frame with the same columns but one row per
#'       observation per category level}
#'   }
#'
#' @seealso \code{\link{gg_partial}} \code{\link{varpro_feature_names}}
#'
#' @examples
#' ## Construct mock varpro partialpro output:
#' ##   - "age": a continuous predictor (xvirtual has > 10 points)
#' ##   - "sex": a categorical predictor (xvirtual has 2 points)
#' set.seed(42)
#' n_obs <- 30   # number of observations (rows in yhat matrices)
#' n_pts <- 15   # number of evaluation points for continuous variables
#'
#' mock_data <- list(
#'   age = list(
#'     # xvirtual: evaluation grid for the marginal effect curve
#'     xvirtual   = seq(30, 80, length.out = n_pts),
#'     # xorg: original observed values (used only for categorical detection)
#'     xorg       = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
#'     # yhat matrices: n_obs rows x n_pts columns (predictions at each grid pt)
#'     yhat.par   = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
#'   ),
#'   sex = list(
#'     # Two categories: the xvirtual grid has only 2 points
#'     xvirtual   = c(0, 1),
#'     xorg       = sample(c(0, 1), n_obs, replace = TRUE),
#'     # Two-column yhat matrices (one column per category)
#'     yhat.par   = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
#'   )
#' )
#'
#' result <- gg_partialpro(mock_data)
#'
#' ## Continuous result: one row per evaluation grid point
#' head(result$continuous)
#'
#' ## Categorical result: n_obs rows per category level
#' head(result$categorical)
#'
#' @export
#'
gg_partialpro <- function(part_dta,
                          nvars = NULL,
                          cat_limit = 10,
                          model = NULL) {
  ## Default: process all variables in the partialpro output list
  if (is.null(nvars)) {
    nvars <- length(part_dta)
  }

  # Accumulate per-variable data frames before binding
  cont_list <- list()
  cat_list  <- list()

  for (feature in seq(nvars)) {
    ## ---- Continuous variable: xvirtual grid has more points than cat_limit --
    if (length(part_dta[[feature]]$xvirtual) > cat_limit) {
      # Summarise each evaluation grid point by averaging across observations
      plt.df <- dplyr::bind_cols(
        variable     = part_dta[[feature]]$xvirtual,
        parametric   = colMeans(part_dta[[feature]]$yhat.par,    na.rm = TRUE),
        nonparametric = colMeans(part_dta[[feature]]$yhat.nonpar, na.rm = TRUE),
        causal       = colMeans(part_dta[[feature]]$yhat.causal,  na.rm = TRUE)
      )

      # Tag rows with variable name for downstream faceting
      plt.df$name <- names(part_dta)[[feature]]

      cont_list[[feature]] <- plt.df

    } else {
      ## ---- Categorical variable: stack per-observation rows per category ----
      ## VarPro works with logical/continuous only; factors are one-hot encoded
      ## internally.  We reconstruct the per-category rows here.
      cat_feat <- list()

      # Each column of yhat.par corresponds to one unique original category
      n_cats <- length(unique(part_dta[[feature]]$xorg))
      for (ind in seq(n_cats)) {
        # One row per observation for this category
        cat_feat[[ind]] <- dplyr::bind_cols(
          parametric    = part_dta[[feature]]$yhat.par[, ind],
          nonparametric = part_dta[[feature]]$yhat.nonpar[, ind],
          causal        = part_dta[[feature]]$yhat.causal[, ind]
        )
        # Record the category label this subset belongs to
        cat_feat[[ind]]$variable <- unique(part_dta[[feature]]$xorg)[ind]

        # Stack iteratively (first iteration initialises plt.df)
        if (ind == 1) {
          plt.df <- cat_feat[[ind]]
        } else {
          plt.df <- dplyr::bind_rows(plt.df, cat_feat[[ind]])
        }
      }

      # Tag rows with variable name for downstream faceting
      plt.df$name <- names(part_dta)[[feature]]

      cat_list[[feature]] <- plt.df
    }
  }

  # Combine per-variable lists into single data frames (NULL entries dropped)
  continuous  <- dplyr::bind_rows(cont_list)
  categorical <- dplyr::bind_rows(cat_list)

  ## Optionally attach a model label (useful when overlaying multiple models)
  if (!is.null(model)) {
    continuous$model <- categorical$model <- model
  }

  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- "gg_partialpro"
  return(result)
}

# Tests for gg_partialpro

# Helper: create mock VarPro partialpro data
# - Continuous: length(xvirtual) > cat_limit (10)
# - Categorical: length(xvirtual) <= cat_limit (10)
make_mock_partialpro_data <- function(n_obs = 50, n_pts = 20) {
  set.seed(42)

  cont_var <- list(
    xvirtual = seq(20, 70, length.out = n_pts),  # 20 unique points > 10
    xorg = rnorm(n_obs, 45, 15),
    yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts)
  )

  cat_var <- list(
    xvirtual = c(0, 1),  # 2 unique points <= 10
    xorg = sample(c(0, 1), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2)
  )

  result <- list(age = cont_var, sex = cat_var)
  return(result)
}

# ---- basic structure -------------------------------------------------------

test_that("gg_partialpro returns a gg_partialpro object with continuous and categorical", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  expect_s3_class(result, "gg_partialpro")
  expect_named(result, c("continuous", "categorical"))
  expect_s3_class(result$continuous, "data.frame")
  expect_s3_class(result$categorical, "data.frame")
})

test_that("gg_partialpro separates continuous (age) and categorical (sex)", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  # age has 20 unique points > cat_limit=10
  expect_true("age" %in% result$continuous$name)
  expect_false("age" %in% result$categorical$name)

  # sex has 2 unique points <= cat_limit=10
  expect_true("sex" %in% result$categorical$name)
  expect_false("sex" %in% result$continuous$name)
})

test_that("gg_partialpro continuous output has correct columns", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  expect_true(all(c("variable", "parametric", "nonparametric", "causal", "name")
                  %in% colnames(result$continuous)))
})

test_that("gg_partialpro categorical output has correct columns", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  expect_true(all(c("parametric", "nonparametric", "causal", "variable", "name")
                  %in% colnames(result$categorical)))
})

test_that("gg_partialpro continuous has one row per xvirtual point", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  # age has 20 xvirtual points
  age_rows <- result$continuous[result$continuous$name == "age", ]
  expect_equal(nrow(age_rows), 20)
})

# ---- nvars argument --------------------------------------------------------

test_that("gg_partialpro respects nvars = 1 (only age processed)", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, nvars = 1)

  # Only age (continuous) should appear
  expect_true("age" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

# ---- model argument --------------------------------------------------------

test_that("gg_partialpro adds model column when model is provided", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, model = "my_forest")

  expect_true("model" %in% colnames(result$continuous))
  expect_true("model" %in% colnames(result$categorical))
  expect_equal(unique(result$continuous$model), "my_forest")
  expect_equal(unique(result$categorical$model), "my_forest")
})

test_that("gg_partialpro without model has no model column", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  expect_false("model" %in% colnames(result$continuous))
  expect_false("model" %in% colnames(result$categorical))
})

# ---- cat_limit edge cases --------------------------------------------------

# Note: cat_limit edge case for high cat_limit is tested using data specifically
# designed for categorical use: xvirtual <= cat_limit points, and yhat columns
# match the number of unique xorg values.

test_that("gg_partialpro with low cat_limit makes sex continuous", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, cat_limit = 1)

  expect_true("age" %in% result$continuous$name)
  expect_true("sex" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

# ---- multi-category variable -----------------------------------------------

test_that("gg_partialpro handles variable with 3 categories", {
  set.seed(42)
  n_obs <- 30

  # Each category gets one yhat column; xorg has 3 unique values
  three_cat <- list(
    grp = list(
      xvirtual = c(1, 2, 3),  # 3 unique points <= cat_limit=10
      xorg = sample(c(1, 2, 3), n_obs, replace = TRUE),
      yhat.par    = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3),
      yhat.nonpar = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3),
      yhat.causal = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3)
    )
  )

  result <- gg_partialpro(three_cat)

  expect_equal(nrow(result$continuous), 0)
  expect_true("grp" %in% result$categorical$name)
  # Each category produces n_obs rows (one per observation), total = n_obs * 3
  grp_rows <- result$categorical[result$categorical$name == "grp", ]
  expect_equal(nrow(grp_rows), n_obs * 3)
  # All 3 category labels should appear
  expect_equal(length(unique(grp_rows$variable)), 3)
})

# ---- plot.gg_partialpro ----------------------------------------------------

test_that("plot.gg_partialpro returns a ggplot for continuous data", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, nvars = 1)   # only age (continuous)

  gg_plt <- plot(result)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_partialpro returns a single ggplot when both types present", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  out <- plot(result)
  # patchwork combines the two panels into a single ggplot-inheriting object.
  expect_s3_class(out, "ggplot")
})

test_that("plot.gg_partialpro type argument subsets effect columns", {
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, nvars = 1)

  gg_plt <- plot(result, type = "parametric")
  expect_s3_class(gg_plt, "ggplot")
})

# Tests for gg_partialpro

# Helper: create mock VarPro partialpro data
# - Continuous: length(xvirtual) > cat_limit (10)
# - Categorical: length(xvirtual) <= cat_limit (10)
make_mock_partialpro_data <- function(n_obs = 50, n_pts = 20) {
  set.seed(42)

  cont_var <- list(
    xvirtual = seq(20, 70, length.out = n_pts),  # 20 unique points > 10
    xorg = rnorm(n_obs, 45, 15),
    yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs, ncol = n_pts)
  )

  cat_var <- list(
    xvirtual = c(0, 1),  # 2 unique points <= 10
    xorg = sample(c(0, 1), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs, ncol = 2)
  )

  result <- list(age = cont_var, sex = cat_var)
  return(result)
}

# ---- basic structure -------------------------------------------------------

test_that("gg_partialpro returns a gg_partial_varpro object with continuous and categorical", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  # Shim now returns gg_partial_varpro (the new implementation)
  expect_s3_class(result, "gg_partial_varpro")
  expect_named(result, c("continuous", "categorical"))
  expect_s3_class(result$continuous, "data.frame")
  expect_s3_class(result$categorical, "data.frame")
})

test_that("gg_partialpro separates continuous (age) and categorical (sex)", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  # age has 20 unique points > cat_limit=10
  expect_true("age" %in% result$continuous$name)
  expect_false("age" %in% result$categorical$name)

  # sex has 2 unique points <= cat_limit=10
  expect_true("sex" %in% result$categorical$name)
  expect_false("sex" %in% result$continuous$name)
})

test_that("gg_partialpro continuous output has correct columns", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  expect_true(all(c("variable", "parametric", "nonparametric", "causal", "name")
                  %in% colnames(result$continuous)))
})

test_that("gg_partialpro categorical output has correct columns", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  expect_true(all(c("parametric", "nonparametric", "causal", "variable", "name")
                  %in% colnames(result$categorical)))
})

test_that("gg_partialpro continuous has one row per xvirtual point", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  # age has 20 xvirtual points
  age_rows <- result$continuous[result$continuous$name == "age", ]
  expect_equal(nrow(age_rows), 20)
})

# ---- nvars argument --------------------------------------------------------

test_that("gg_partialpro respects nvars = 1 (only age processed)", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta, nvars = 1))

  # Only age (continuous) should appear
  expect_true("age" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

# ---- model argument --------------------------------------------------------

test_that("gg_partialpro adds model column when model is provided", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta, model = "my_forest"))

  expect_true("model" %in% colnames(result$continuous))
  expect_true("model" %in% colnames(result$categorical))
  expect_equal(unique(result$continuous$model), "my_forest")
  expect_equal(unique(result$categorical$model), "my_forest")
})

test_that("gg_partialpro without model has no model column", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta))

  expect_false("model" %in% colnames(result$continuous))
  expect_false("model" %in% colnames(result$categorical))
})

# ---- cat_limit edge cases --------------------------------------------------

# Note: cat_limit edge case for high cat_limit is tested using data specifically
# designed for categorical use: xvirtual <= cat_limit points, and yhat columns
# match the number of unique xorg values.

test_that("gg_partialpro with low cat_limit makes sex continuous", {
  mock_dta <- make_mock_partialpro_data()
  result <- suppressWarnings(gg_partialpro(mock_dta, cat_limit = 1))

  expect_true("age" %in% result$continuous$name)
  expect_true("sex" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0)
})

# ---- multi-category variable -----------------------------------------------

test_that("gg_partialpro handles variable with 3 categories", {
  set.seed(42)
  n_obs <- 30

  # Each category gets one yhat column; xorg has 3 unique values
  three_cat <- list(
    grp = list(
      xvirtual = c(1, 2, 3),  # 3 unique points <= cat_limit=10
      xorg = sample(c(1, 2, 3), n_obs, replace = TRUE),
      yhat.par    = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3),
      yhat.nonpar = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3),
      yhat.causal = matrix(rnorm(n_obs * 3), nrow = n_obs, ncol = 3)
    )
  )

  result <- suppressWarnings(gg_partialpro(three_cat))

  expect_equal(nrow(result$continuous), 0)
  expect_true("grp" %in% result$categorical$name)
  # Each category produces n_obs rows (one per observation), total = n_obs * 3
  grp_rows <- result$categorical[result$categorical$name == "grp", ]
  expect_equal(nrow(grp_rows), n_obs * 3)
  # All 3 category labels should appear
  expect_equal(length(unique(grp_rows$variable)), 3)
})

# ---- plot.gg_partialpro (deprecated - superseded by plot.gg_partial_varpro) --

test_that("plot.gg_partialpro returns a ggplot for continuous data", {
  skip("plot.gg_partialpro tests superseded by gg_partial_varpro shim; see test_gg_partial_varpro.R")
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, nvars = 1)   # only age (continuous)

  gg_plt <- plot(result)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_partialpro returns a single ggplot when both types present", {
  skip("plot.gg_partialpro tests superseded by gg_partial_varpro shim; see test_gg_partial_varpro.R")
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta)

  out <- plot(result)
  # patchwork combines the two panels into a single ggplot-inheriting object.
  expect_s3_class(out, "ggplot")
})

test_that("plot.gg_partialpro type argument subsets effect columns", {
  skip("plot.gg_partialpro tests superseded by gg_partial_varpro shim; see test_gg_partial_varpro.R")
  mock_dta <- make_mock_partialpro_data()
  result <- gg_partialpro(mock_dta, nvars = 1)

  gg_plt <- plot(result, type = "parametric")
  expect_s3_class(gg_plt, "ggplot")
})

# ---- deprecation shim regression guards ------------------------------------

test_that("gg_partialpro shim emits a deprecation warning", {
  mock_dta <- make_mock_partialpro_data()
  expect_warning(
    gg_partialpro(mock_dta),
    regexp = "deprecated.*gg_partial_varpro",
    ignore.case = TRUE
  )
})

test_that("gg_partialpro shim output is a gg_partial_varpro object", {
  mock_dta <- make_mock_partialpro_data()
  result   <- suppressWarnings(gg_partialpro(mock_dta))
  expect_s3_class(result, "gg_partial_varpro")
})

test_that("gg_partialpro shim: continuous/categorical structure preserved", {
  mock_dta <- make_mock_partialpro_data()
  result   <- suppressWarnings(gg_partialpro(mock_dta))
  expect_named(result, c("continuous", "categorical"))
  expect_true("age" %in% result$continuous$name)
  expect_true("sex" %in% result$categorical$name)
})

##=============================================================================
#' Split partial dependence data into continuous or categorical datasets
#'
#' A partial dependence curve answers a what-if question about a forest: hold
#' every other predictor at its observed value, sweep one of them across its
#' range, and watch how the ensemble prediction moves.  Marginalized over the
#' joint distribution of the other variables, the resulting curve isolates the
#' average effect of the swept predictor alone.
#'
#' \code{gg_partial} handles the bookkeeping step after you've already called
#' \code{rfsrc::plot.variable(partial = TRUE)}: it takes the list that function
#' returns and separates the variables into two tidy data frames -- one for
#' continuous predictors (plotted as lines) and one for categorical predictors
#' (plotted as bar charts).  The split is controlled by \code{cat_limit}:
#' variables with more unique x-values than this threshold are treated as
#' continuous; all others are categorical.
#'
#' If you'd rather skip the \code{plot.variable} step and pass the fitted
#' forest directly, see \code{\link{gg_partial_rfsrc}}, which calls
#' \code{partial.rfsrc} for you.
#'
#' @param part_dta partial plot data from \code{rfsrc::plot.variable}
#' @param nvars how many of the partial plot variables to calculate
#' @param cat_limit Categorical features are built when there are fewer than
#'  \code{cat_limit} unique feature values.
#' @param model a label name applied to all features. Useful when combining
#'  multiple partial plot objects in figures.
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{continuous}{data.frame with columns \code{x}, \code{yhat},
#'       \code{name} (and optionally \code{model}) for continuous variables}
#'     \item{categorical}{data.frame with the same columns but with \code{x}
#'       as a factor, for low-cardinality / categorical variables}
#'   }
#'
#' @seealso \code{\link{gg_partial_rfsrc}} \code{\link{gg_partialpro}}
#'
#' @examples
#' ## Build a small regression forest on the airquality dataset
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#'
#' ## Compute partial dependence via plot.variable (show.plots = FALSE to
#' ## suppress the base-graphics output, we only want the data)
#' pv <- randomForestSRC::plot.variable(rf, partial = TRUE,
#'                                       show.plots = FALSE)
#'
#' ## Split into continuous and categorical data frames
#' result <- gg_partial(pv)
#' head(result$continuous)
#'
#' ## Label this model for later comparison with a second forest
#' result_labelled <- gg_partial(pv, model = "airq_model")
#' unique(result_labelled$continuous$model)
#'
#' @note Partial-dependence extraction is `randomForestSRC`-only;
#'   there is no `randomForest` method (the `randomForest` package
#'   provides no comparable partial-dependence interface).
#' @export
gg_partial <- function(part_dta,
                       nvars = NULL,
                       cat_limit = 10,
                       model = NULL) {
  ## Default: process all variables returned by plot.variable
  if (is.null(nvars)) {
    nvars <- length(part_dta$plotthis)
  }

  # Accumulate per-variable data frames before binding
  cont_list <- list()
  cat_list <- list()

  for (feature in seq(nvars)) {
    x_vals <- part_dta$plotthis[[feature]]$x

    ## ---- Continuous variable: more unique x values than cat_limit -------
    if (length(unique(x_vals)) > cat_limit) {
      plt.df <- dplyr::bind_cols(
        x    = x_vals,
        yhat = part_dta$plotthis[[feature]]$yhat
      )
      # Tag each row with the variable name for downstream faceting
      plt.df$name <- names(part_dta$plotthis)[[feature]]

      cont_list[[feature]] <- plt.df

    } else {
      ## ---- Categorical variable: few unique x values -------------------
      ## Normalize to character so bind_rows sees a consistent type; we'll
      ## re-factor within each feature after stacking.
      x_chr <- as.character(x_vals)

      plt.df <- dplyr::bind_cols(
        x    = x_chr,
        yhat = part_dta$plotthis[[feature]]$yhat
      )
      plt.df$name <- names(part_dta$plotthis)[[feature]]

      cat_list[[feature]] <- plt.df
    }
  }

  # Combine per-variable lists into single data frames (NULL entries dropped)
  continuous  <- dplyr::bind_rows(cont_list)
  if (length(cat_list) == 0) {
    categorical <- data.frame(x = character(0), yhat = numeric(0), name = character(0))
  } else {
    categorical <- dplyr::bind_rows(cat_list)
    # Set within-group factor levels to order-of-appearance (base-R, no .data pronoun)
    split_grps <- split(seq_len(nrow(categorical)), categorical$name)
    for (grp_idx in split_grps) {
      vals <- categorical$x[grp_idx]
      categorical$x[grp_idx] <- as.character(factor(vals, levels = unique(vals)))
    }
    categorical$x <- factor(categorical$x)
  }
  ## Optionally attach a model label (useful when overlaying multiple forests)
  if (!is.null(model)) {
    continuous$model <- model
    if (!is.null(categorical) && nrow(categorical) > 0) {
      categorical$model <- model
    }
  }

  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- "gg_partial"
  return(result)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Nonparametric survival estimates.
#'
#' @details \code{gg_survival} is an S3 generic for generating nonparametric
#' survival estimates.  It dispatches on the class of its first argument:
#'
#' \describe{
#'   \item{\code{rfsrc}}{Extracts the response data from the fitted forest and
#'     delegates to \code{\link{kaplan}}.  Use the \code{by} argument to
#'     stratify on a predictor stored in the model's \code{xvar} slot.}
#'   \item{default}{Accepts raw survival data columns via the \code{interval},
#'     \code{censor}, \code{by}, and \code{data} arguments, delegating to
#'     either \code{\link{kaplan}} (default) or \code{\link{nelson}}.}
#' }
#'
#' @param object For the \code{rfsrc} method: a fitted
#'   \code{\link[randomForestSRC]{rfsrc}} survival forest.  For the default
#'   method: pass \code{NULL} (or omit) and supply \code{interval},
#'   \code{censor}, and \code{data} instead.
#' @param interval Character; name of the time-to-event column in \code{data}
#'   (default method only).
#' @param censor Character; name of the event-indicator column in \code{data}
#'   (1 = event, 0 = censored; default method only).
#' @param by Optional character; name of a grouping column for stratified
#'   estimates.  For the \code{rfsrc} method, \code{by} must be a column in
#'   \code{object$xvar}.
#' @param data A \code{data.frame} containing survival data (default method
#'   only).
#' @param type One of \code{"kaplan"} (Kaplan-Meier, default) or
#'   \code{"nelson"} (Nelson-Aalen cumulative hazard).  Default method only.
#' @param ... Additional arguments passed to \code{\link{kaplan}} or
#'   \code{\link{nelson}}.
#'
#' @return A \code{gg_survival} \code{data.frame} with columns \code{time},
#'   \code{surv}, \code{cum_haz}, \code{lower}, \code{upper}, \code{n.risk},
#'   and optionally \code{groups} when \code{by} is supplied.
#'
#' @seealso \code{\link{kaplan}} \code{\link{nelson}}
#' @seealso \code{\link{plot.gg_survival}}
#'
#' @examples
#' ## -------- pbc data (default method — raw data columns)
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' gg_dta <- gg_survival(interval = "time", censor = "status", data = pbc)
#' plot(gg_dta, error = "none")
#'
#' # Stratified
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#' plot(gg_dta)
#'
#' @export
gg_survival <- function(object = NULL,
                        interval = NULL,
                        censor = NULL,
                        by = NULL,
                        data = NULL,
                        type = c("kaplan", "nelson"),
                        ...) {
  UseMethod("gg_survival")
}

#' @rdname gg_survival
#' @export
gg_survival.rfsrc <- function(object,
                              interval = NULL,
                              censor   = NULL,
                              by       = NULL,
                              data     = NULL,
                              type     = c("kaplan", "nelson"),
                              ...) {
  ## interval, censor, data, and type are accepted for consistency with the
  ## generic signature but are ignored: this method extracts everything it
  ## needs from the fitted forest's $yvar and $xvar slots.
  ##
  ## randomForestSRC stores the outcome as a two-column data frame: time (col 1)
  ## and status/censor (col 2) in object$yvar.
  yvar <- object$yvar
  if (is.null(yvar) || !is.data.frame(yvar) || ncol(yvar) < 2L) {
    stop(
      "gg_survival requires a survival forest; this rfsrc object ",
      "does not appear to be a survival forest.",
      call. = FALSE
    )
  }

  interval_col <- colnames(yvar)[1L]
  censor_col   <- colnames(yvar)[2L]
  surv_data    <- yvar

  if (!is.null(by)) {
    if (!by %in% colnames(object$xvar)) {
      stop(sprintf(
        "'by' column '%s' not found in the forest's xvar slot.", by
      ), call. = FALSE)
    }
    surv_data[[by]] <- object$xvar[[by]]
  }

  gg_dta <- kaplan(interval = interval_col, censor = censor_col, by = by,
                   data = surv_data, ...)
  .set_provenance(gg_dta, object)
}

#' @rdname gg_survival
#' @export
gg_survival.default <- function(object = NULL,
                                interval = NULL,
                                censor = NULL,
                                by = NULL,
                                data = NULL,
                                type = c("kaplan", "nelson"),
                                ...) {
  ## Allow data to be passed as the first positional argument for convenience.
  if (is.data.frame(object) && is.null(data)) {
    data <- object
  }

  type <- match.arg(type)

  gg_dta <- switch(type, # nolint: object_usage_linter
    kaplan = kaplan(
      interval = interval,
      censor   = censor,
      by       = by,
      data     = data,
      ...
    ),
    nelson = nelson(
      interval = interval,
      censor   = censor,
      by       = by,
      data     = data,
      ...
    )
  )

  return(gg_dta)
}

# testthat for gg_survival function

test_that("gg_survival classifications", {
  expect_error(gg_survival(data = iris))
})


test_that("gg_survival survival", {
  #   ## Load the cached forest
  data(pbc, package = "randomForestSRC")

  # Test the cached forest type
  expect_s3_class(pbc, "data.frame")

  # Test object type
  gg_dta <- gg_survival(
    interval = "days",
    censor = "status",
    by = "treatment",
    data = pbc,
    conf.int = .95
  )

  expect_s3_class(gg_dta, "gg_survival")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_survival(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  expect_s3_class(plot(gg_dta, error = "bars"), "ggplot")
  expect_s3_class(plot(gg_dta, error = "none"), "ggplot")
  expect_s3_class(plot(gg_dta, error = "lines"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "surv"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "cum_haz"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "density"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "mid_int"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "life"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "hazard"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "proplife"), "ggplot")
  # Test object type
  gg_dta <- gg_survival(
    interval = "days",
    censor = "status",
    by = "treatment",
    data = pbc,
    conf.int = .95,
    type = "nelson"
  )

  expect_s3_class(gg_dta, "gg_survival")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_survival(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")


  # Test object type
  gg_dta <- gg_survival(
    interval = "days",
    censor = "status",
    data = pbc,
    conf.int = .95
  )

  expect_s3_class(gg_dta, "gg_survival")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_survival(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  expect_s3_class(plot(gg_dta, error = "bars"), "ggplot")
  expect_s3_class(plot(gg_dta, error = "none"), "ggplot")
  expect_s3_class(plot(gg_dta, error = "lines"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "surv"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "cum_haz"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "density"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "mid_int"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "life"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "hazard"), "ggplot")
  expect_s3_class(plot(gg_dta, type = "proplife"), "ggplot")

})

test_that("gg_survival regression", {
  ## Load the data
  data(Boston, package = "MASS")

  ## Create the correct gg_error object
  expect_error(gg_survival(data = Boston))
})

test_that("gg_survival.rfsrc extracts KM from a survival forest", {
  skip_if_not_installed("randomForestSRC")

  veteran  <- survival::veteran
  Surv     <- survival::Surv  # nolint: object_name_linter
  set.seed(42)
  rf <- randomForestSRC::rfsrc(
    Surv(time, status) ~ trt + karno + diagtime + age + prior,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  gg_dta <- gg_survival(rf)

  expect_s3_class(gg_dta, "gg_survival")
  expect_true(all(c("time", "surv", "lower", "upper") %in% colnames(gg_dta)))
  expect_s3_class(plot(gg_dta, error = "none"), "ggplot")
})

test_that("gg_survival.rfsrc supports stratification via by", {
  skip_if_not_installed("randomForestSRC")

  veteran  <- survival::veteran
  Surv     <- survival::Surv  # nolint: object_name_linter
  set.seed(42)
  rf <- randomForestSRC::rfsrc(
    Surv(time, status) ~ trt + karno + diagtime + age + prior,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  gg_dta <- gg_survival(rf, by = "trt")

  expect_s3_class(gg_dta, "gg_survival")
  expect_true("groups" %in% colnames(gg_dta))
  expect_s3_class(plot(gg_dta), "ggplot")
})

test_that("gg_survival.rfsrc errors on non-survival forest", {
  skip_if_not_installed("randomForestSRC")

  set.seed(42)
  airq <- na.omit(airquality)
  rf_reg <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 30)

  # Regression forests have no $yvar with two survival columns
  expect_error(gg_survival(rf_reg), regexp = "survival forest")
})

##=============================================================================
#' Plot a `gg_ivarpro` object
#'
#' Branches on the presence of `which_obs` provenance and the `class`
#' column. Distribution view: jittered points showing per-observation
#' local importances per variable. Per-observation view: horizontal bar
#' chart of one observation's local importances across variables.
#' Classification: faceted by class unless `which_class` collapses to
#' a single class.
#'
#' @section Reading the chart:
#' Each point in the distribution view is one observation's local
#' importance for that variable. Variables are sorted by descending
#' `mean(|local_imp|)`. The cutoff line picks the variables whose local
#' importance is, on average, large enough to flag. For a classification
#' fit, every facet shares the same row order so you can read across.
#'
#' For a classification fit, variables are sorted by descending
#' `mean(|local_imp|)` across all (obs, class) rows and that ordering
#' is shared across every facet, so rows line up between classes for
#' visual comparison. Each facet has its own cutoff line.
#'
#' The per-observation view (`which_obs`) is a horizontal bar chart of
#' one observation's local importances; bars below the cutoff are grey,
#' above are blue. The visual resembles a SHAP waterfall, but the values
#' are release-rule contributions: scaled per-rule contrasts on observed
#' data, not Shapley values and not permutation-based.
#'
#' @param x A `gg_ivarpro` object from [gg_ivarpro()].
#' @param ... Not currently used.
#'
#' @return A `ggplot` object.
#'
#' @seealso [gg_ivarpro()].
#'
#' @examples
#' \donttest{
#' if (requireNamespace("varPro", quietly = TRUE) &&
#'     requireNamespace("MASS", quietly = TRUE)) {
#'   set.seed(1)
#'   v <- varPro::varpro(medv ~ ., data = MASS::Boston, ntree = 50)
#'   plot(gg_ivarpro(v))
#' }
#' }
#'
#' @name plot.gg_ivarpro
#' @importFrom ggplot2 ggplot aes geom_col geom_jitter geom_hline coord_flip
#' @importFrom ggplot2 facet_wrap scale_fill_manual scale_color_manual labs theme_minimal
#' @export
plot.gg_ivarpro <- function(x, ...) {
  if (nrow(x) == 0L) {
    stop("plot.gg_ivarpro: nothing to plot (gg_ivarpro has 0 rows).",
         call. = FALSE)
  }
  prov           <- attr(x, "provenance")
  has_class      <- "class" %in% names(x)
  which_obs_set  <- !is.null(prov) && !is.null(prov$which_obs)

  cutoff_vec <- if (!is.null(prov) && !is.null(prov$cutoff)) {
    prov$cutoff
  } else if (has_class) {
    stats::setNames(vapply(split(abs(x$local_imp), x$class), mean, numeric(1)),
                    levels(x$class))
  } else {
    stats::setNames(mean(abs(x$local_imp)), "regr")
  }

  base <- ggplot2::ggplot(
    x,
    ggplot2::aes(x = .data[["variable"]],
                 y = .data[["local_imp"]])
  )

  if (which_obs_set) {
    p <- base +
      ggplot2::geom_col(
        ggplot2::aes(fill = factor(.data[["selected"]]))
      ) +
      ggplot2::scale_fill_manual(
        values = c("TRUE" = "#4e8fcd", "FALSE" = "#888888"),
        guide  = "none"
      )
  } else {
    p <- base +
      ggplot2::geom_jitter(
        ggplot2::aes(color = factor(.data[["selected"]])),
        width = 0.2, height = 0, alpha = 0.5
      ) +
      ggplot2::scale_color_manual(
        values = c("TRUE" = "#4e8fcd", "FALSE" = "#888888"),
        guide  = "none"
      )
  }

  p <- p + ggplot2::coord_flip()

  if (has_class && length(unique(x$class)) > 1L) {
    hline_df <- data.frame(
      class  = factor(names(cutoff_vec), levels = levels(x$class)),
      cutoff = unname(cutoff_vec),
      stringsAsFactors = FALSE
    )
    p <- p +
      ggplot2::facet_wrap(~ class, nrow = 1L) +
      ggplot2::geom_hline(
        data        = hline_df,
        ggplot2::aes(yintercept = .data[["cutoff"]]),
        linetype    = "dashed",
        color       = "#e74c3c",
        linewidth   = 0.7,
        inherit.aes = FALSE
      )
  } else {
    cutoff_scalar <- if (has_class) cutoff_vec[[as.character(x$class[1])]] else cutoff_vec[[1]]
    p <- p + ggplot2::geom_hline(
      yintercept = cutoff_scalar,
      linetype   = "dashed",
      color      = "#e74c3c",
      linewidth  = 0.7
    )
  }

  caption_txt <- sprintf(
    "Local importance over %d obs x %d variables.%s",
    prov$n_obs %||% NA_integer_,
    prov$n_var %||% NA_integer_,
    if (which_obs_set) sprintf(" obs = %d.", prov$which_obs) else ""
  )

  p + ggplot2::labs(
    x = NULL,
    y = "Local importance",
    caption = caption_txt
  ) + ggplot2::theme_minimal()
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' ROC (Receiver Operating Characteristic) curve data from a classification forest.
#'
#' A classifier does not hand you a class; it hands you a predicted probability,
#' and you pick a threshold. Slide that threshold from 0 to 1 and the trade-off
#' between catching the positives and crying wolf shifts the whole way. The ROC
#' curve traces that trade-off. For one class of a classification
#' \code{\link[randomForestSRC]{rfsrc}} or
#' \code{\link[randomForest]{randomForest}} forest, \code{gg_roc} walks every
#' threshold and records sensitivity (the true positive rate) against
#' specificity (1 minus the false positive rate).
#'
#' @param object A classification \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object. Only forests with
#'   \code{family == "class"} (rfsrc) or \code{type == "classification"}
#'   (randomForest) are supported.
#' @param which_outcome Integer index or character name of the class to score.
#'   For binary forests this is usually \code{1} or \code{2}; for multi-class
#'   forests, any valid class index or level name. \code{which_outcome = "all"}
#'   or \code{0} behaves differently by engine:
#'   \describe{
#'     \item{\code{randomForest} method}{Returns a macro-averaged
#'       one-vs-rest ROC computed over the per-class probabilities.}
#'     \item{\code{rfsrc} method}{Warns and falls back to class 1. The
#'       macro-average and per-class faceting for the \code{rfsrc} path
#'       are tracked separately under issue #72.}
#'   }
#' @param oob Logical; if \code{TRUE} (default), build the curve from
#'   out-of-bag predicted probabilities, otherwise from full in-bag
#'   predictions. For \code{randomForest}, \code{TRUE} uses the out-of-bag
#'   vote probabilities in \code{object$votes}; \code{FALSE} uses in-bag
#'   \code{predict(type = "prob")}.
#' @param per_class Logical; if \code{TRUE} and the forest has more than two
#'   classes, return one ROC curve per class, each class scored against all
#'   the others. The result is a long-format \code{data.frame} with a
#'   \code{class} factor column and a named AUC vector attribute, ordered by
#'   descending AUC. Binary forests treat \code{per_class = TRUE} as a no-op.
#'   Honoured by the \code{randomForest} method only.
#' @param ... Extra arguments (currently unused).
#'
#' @return A \code{gg_roc} \code{data.frame}, one row per unique prediction
#'   threshold, with columns:
#'   \describe{
#'     \item{sens}{Sensitivity (true positive rate) at the threshold.}
#'     \item{spec}{Specificity (true negative rate) at the threshold.}
#'     \item{pct}{The probability threshold used for that row.}
#'   }
#'   Pass it to \code{\link{calc_auc}} for the area under the curve.
#'
#' @seealso \code{\link{plot.gg_roc}}, \code{\link{calc_roc}},
#'   \code{\link{calc_auc}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForest]{randomForest}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)
#'
#' # ROC for setosa
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' ## -------- iris data
#' rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
#'
#' # ROC for setosa
#' gg_dta <- gg_roc(rf_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor
#' gg_dta <- gg_roc(rf_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica
#' gg_dta <- gg_roc(rf_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' @aliases gg_roc gg_roc.rfsrc gg_roc.randomForest

#' @export
gg_roc.rfsrc <- function(object, which_outcome, oob = TRUE,
                         per_class = FALSE, ...) {
  # Validate that the object was grown with randomForestSRC (grow or predict)
  # or is a randomForest object: the two supported class signatures.
  if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &&
    sum(inherits(object, c("rfsrc", "predict"), TRUE) == c(1, 2)) != 2 &&
    !inherits(object, "randomForest")) {
    stop(
      "This function only works for objects of class '(rfsrc, grow)', ",
      "'(rfsrc, predict)', or 'randomForest'."
    )
  }
  # ROC curves only make sense for classification; reject other families early.
  if (!inherits(object, "class")) {
    stop("gg_roc only works with classification forests")
  }

  # Default to "all" so the caller can later loop over every class level;
  # the caller may pass a specific integer index to get a single-class curve.
  if (missing(which_outcome)) {
    which_outcome <- "all"
  }

  # Redundant guard: rfsrc sets family = "class" for classification forests.
  # Kept here to surface a clearer error message if the object is somehow
  # mis-labelled.
  if (object$family != "class") {
    stop("gg_roc is intended for classification forests only.")
  }

  # Delegate the threshold-sweep computation to calc_roc, passing the
  # observed response vector and the chosen outcome column index.
  gg_dta <- # nolint: object_usage_linter
    calc_roc(object,
      object$yvar,
      which_outcome = which_outcome,
      oob = oob
    )
  class(gg_dta) <- c("gg_roc", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)

  invisible(gg_dta)
}
#' @export
gg_roc <- function(object, which_outcome, oob = TRUE, per_class = FALSE, ...) {
  UseMethod("gg_roc", object)
}

#' @export
gg_roc.randomForest <- function(object, which_outcome, oob = TRUE,
                                per_class = FALSE, ...) {
  if (!inherits(object, "randomForest")) {
    stop("gg_roc.randomForest only works for objects of class 'randomForest'.")
  }
  if (missing(which_outcome)) {
    which_outcome <- "all"
  }
  if (!(object$type == "classification")) {
    stop("gg_roc only works with classification forests")
  }

  lvls    <- levels(object$y)
  n_class <- length(lvls)

  # ── per_class = TRUE path (multi-class only) ─────────────────────────────
  if (isTRUE(per_class) && n_class > 2L) {
    if (!missing(which_outcome) && !identical(which_outcome, "all")) {
      message("which_outcome is ignored when per_class = TRUE.")
    }
    prob   <- .rf_prob_matrix(object, oob, lvls)
    dta    <- object$y
    curves <- lapply(seq_along(lvls), function(k) {
      cv       <- .rf_one_class_roc(dta, prob, k, lvls)
      cv$class <- lvls[k]
      cv
    })
    auc_vals        <- vapply(curves, calc_auc, numeric(1L))
    names(auc_vals) <- lvls
    auc_ord         <- order(auc_vals, decreasing = TRUE)
    auc_vals        <- auc_vals[auc_ord]
    gg_dta          <- do.call(rbind, curves)
    gg_dta$class    <- factor(gg_dta$class, levels = lvls[auc_ord])
    class(gg_dta)   <- c("gg_roc", class(gg_dta))
    attr(gg_dta, "auc") <- auc_vals
    gg_dta <- .set_provenance(gg_dta, object)
    return(invisible(gg_dta))
  }

  # ── Standard path (binary, or per_class not requested) ──────────────────
  # For randomForest objects the response is stored in $y (not $yvar).
  gg_dta <- # nolint: object_usage_linter
    calc_roc(object, object$y, which_outcome = which_outcome, oob = oob)
  class(gg_dta)       <- c("gg_roc", class(gg_dta))
  attr(gg_dta, "auc") <- calc_auc(gg_dta)
  gg_dta              <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}



#' @export
gg_roc.default <- gg_roc.rfsrc

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Plot a \code{\link{gg_variable}} object,
#'
#' @param x \code{\link{gg_variable}} object created from a
#' \code{\link[randomForestSRC]{rfsrc}} object
#' @param xvar variable (or list of variables) of interest.
#' @param time For survival, one or more times of interest
#' @param time_labels string labels for times
#' @param panel Should plots be faceted along multiple xvar?
#' @param oob oob estimates (boolean)
#' @param points plot the raw data points (boolean)
#' @param smooth include a smooth curve (boolean)
#' @param ... arguments passed to the \code{ggplot2} functions.
#'
#' @return A single \code{ggplot} object when \code{length(xvar) == 1} or
#'   \code{panel = TRUE}. Otherwise a named list of \code{ggplot} objects, one
#'   per variable in \code{xvar}.
#'
#' @seealso \code{\link{gg_variable}}, \code{\link{gg_partial}},
#'   \code{\link[randomForestSRC]{plot.variable}}
#'
#' @references Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
#' 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#'
#' @importFrom ggplot2 .data
#' @examples
#' ## ------------------------------------------------------------
#' ## classification
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_iris)
#' plot(gg_dta, xvar = "Sepal.Width")
#' plot(gg_dta, xvar = "Sepal.Length")
#'
#' ## Panel plot across all predictors
#' plot(gg_dta,
#'   xvar = rfsrc_iris$xvar.names,
#'   panel = TRUE, se = FALSE
#' )
#'
#' ## ------------------------------------------------------------
#' ## regression
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' # na.action = "na.impute" handles missing Ozone / Solar.R values
#' set.seed(42)
#' rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' gg_dta <- gg_variable(rfsrc_airq)
#'
#' # Treat Month as an ordinal factor for better visualisation
#' gg_dta[, "Month"] <- factor(gg_dta[, "Month"])
#'
#' plot(gg_dta, xvar = "Wind")
#' plot(gg_dta, xvar = "Temp")
#' plot(gg_dta, xvar = "Solar.R")
#'
#' # Panel plot across continuous predictors
#' plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
#'
#' # Factor variable uses notched boxplots
#' plot(gg_dta, xvar = "Month", notch = TRUE)
#'
#' ## ------------------------------------------------------------
#' ## survival examples
#' ## ------------------------------------------------------------
#' ## -------- veteran data
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., veteran,
#'   nsplit = 10,
#'   ntree = 50
#' )
#'
#' # Marginal survival at 90 days
#' gg_dta <- gg_variable(rfsrc_veteran, time = 90)
#'
#' # Single-variable dependence plots
#' plot(gg_dta, xvar = "age")
#' plot(gg_dta, xvar = "diagtime")
#'
#' # Panel coplot for two predictors at a single time
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
#'
#' # Compare survival at 30, 90, and 365 days simultaneously
#' gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
#'
#' # Single-variable plot (one facet per time point)
#' plot(gg_dta, xvar = "age")
#'
#' # Panel coplot across two predictors and three time points
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
#'
#' @export
plot.gg_variable <- function(x, # nolint: cyclocomp_linter
                             xvar,
                             time,
                             time_labels,
                             panel = FALSE,
                             oob = TRUE,
                             points = TRUE,
                             smooth = TRUE,
                             ...) {
  gg_dta <- x

  # I don't think this will work with latest S3 models.
  if (inherits(x, "rfsrc")) {
    gg_dta <- gg_variable(x, ...)
  }

  ## ---- Detect forest family from gg_variable class attributes ----------
  # Default to classification; override if survival or regression flags found
  family <- "class"
  if (inherits(gg_dta, "surv")) {
    family <- "surv"
  } else if (inherits(gg_dta, "regr")) {
    family <- "regr"
  } else if (inherits(gg_dta, "regression")) {
    family <- "regr"
  }

  # Fallback detection: presence of an "event" column signals survival
  if (sum(colnames(gg_dta) == "event") != 0) {
    family <- "surv"
  }

  ## ---- Reshape multi-class classification data -------------------------
  # Multiple yhat.* columns indicate a multi-class forest
  if (length(grep("yhat.", colnames(gg_dta))) > 0) {
    if (length(grep("yhat.", colnames(gg_dta))) == 2) {
      # Binary: drop the first class column and rename the second to "yhat"
      gg_dta <- gg_dta[, -grep("yhat.", colnames(gg_dta))[1]]
      colnames(gg_dta)[grep("yhat.", colnames(gg_dta))] <- "yhat"
    } else {
      # Multi-class: pivot to long format so each class becomes a row group
      gg_dta_x <- gg_dta[, -grep("yhat.", colnames(gg_dta))]
      gg_dta_y <- gg_dta[, grep("yhat.", colnames(gg_dta))]
      lng <- ncol(gg_dta_y)
      gg2 <- parallel::mclapply(seq_len(ncol(gg_dta_y)), function(ind) {
        cbind(gg_dta_x, yhat = gg_dta_y[, ind], outcome = ind)
      })
      gg3 <- do.call(rbind, gg2)
      gg3$outcome <- factor(gg3$outcome)
      gg_dta <- gg3
    }
  }

  ## ---- Default xvar: all predictor columns -----------------------------
  if (missing(xvar)) {
    # Remove response-side columns (yhat, event, time) to isolate predictors
    cls <- c(
      grep("yhat", colnames(gg_dta)),
      grep("event", colnames(gg_dta)),
      grep("time", colnames(gg_dta))
    )
    xvar <- colnames(gg_dta)[-cls]
  }

  lng <- length(xvar)

  # Column indices corresponding to the requested predictor(s)
  wch_x_var <- which(colnames(gg_dta) %in% xvar)

  ## ---- Coerce 0/1 indicator columns to logical -------------------------
  # Columns with exactly two values in {0, 1} are treated as boolean flags
  for (ind in seq_len(ncol(gg_dta))) {
    if (!is.factor(gg_dta[, ind])) {
      if (length(unique(gg_dta[which(!is.na(gg_dta[, ind])), ind])) <= 2) {
        if (sum(range(gg_dta[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
      }
    } else {
      if (length(unique(gg_dta[which(!is.na(gg_dta[, ind])), ind])) <= 2) {
        if (sum(sort(unique(gg_dta[, ind])) == c(0, 1)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
        if (sum(sort(unique(gg_dta[, ind])) == c(FALSE, TRUE)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
      }
    }
  }

  ## ---- Record each column's storage class for plot dispatch -----------
  # "integer" is treated as numeric for plot branching purposes
  ccls <- sapply(gg_dta, class)
  ccls[which(ccls == "integer")] <- "numeric"

  ## =========================================================
  ## PANEL PLOT branch — facet multiple predictors in one figure
  ## =========================================================
  if (panel) {
    ## ---- Survival panel plot ----------------------------------------
    if (family == "surv") {
      ## Indices of response columns (time, event, and yhat)
      wch_y_var <-
        which(colnames(gg_dta) %in% c("event", "yhat", "time"))

      # Subset to response + requested predictors, then pivot to long form
      tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
      pivot_cols <-
        colnames(tmp_dta)[-which(colnames(tmp_dta) %in%
          c("time", "event", "yhat"))]
      gg_dta_mlt <-
        tidyr::pivot_longer(tmp_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")

      # Preserve user-supplied xvar ordering in the facet strips
      gg_dta_mlt$variable <-
        factor(gg_dta_mlt$variable, levels = xvar)
      if (points) {
        gg_plt <- ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(
            x = .data$value,
            y = .data$yhat,
            color = .data$event,
            shape = .data$event
          )
        )
      } else {
        gg_plt <- ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(x = .data$value, y = .data$yhat)
        )
      }
      # If these are all continuous...
      if (sum(ccls[wch_x_var] == "numeric") == length(wch_x_var)) {
        gg_plt <- gg_plt +
          ggplot2::labs(y = "Survival")
        if (points) {
          gg_plt <- gg_plt +
            ggplot2::geom_point(...)
        } else {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
        if (smooth) {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
      } else {
        # Mixed or all-factor predictors: fall back to boxplot+jitter
        if (sum(ccls[wch_x_var] == "numeric") > 0) {
          warning(
            "Mismatched variable types for panel plots...
            assuming these are all factor variables."
          )
        }

        gg_plt <- gg_plt +
          ggplot2::geom_boxplot(
            ggplot2::aes(x = .data$value, y = .data$yhat),
            color = "grey",
            ...,
            outlier.shape = NA
          ) +
          ggplot2::geom_jitter(
            ggplot2::aes(
              x = .data$value,
              y = .data$yhat,
              color = .data$event,
              shape = .data$event
            ),
            ...
          )
      }

      # Multiple time points: grid of (time × variable); single time: wrap
      if (length(levels(gg_dta$time)) > 1) {
        gg_plt <- gg_plt +
          ggplot2::facet_grid(stats::reformulate("variable", "time"),
            scales = "free_x"
          ) +
          labs(x = "")
      } else {
        gg_plt <- gg_plt +
          ggplot2::facet_wrap(~variable, scales = "free_x") +
          labs(
            x = "",
            y = paste("Survival at", gg_dta$time[1], "year")
          )
      }

    } else {
      ## ---- Regression / classification panel plot ---------------------
      wch_y_var <- which(colnames(gg_dta) %in% c("yhat"))

      if (family == "class") {
        # Include the observed class label column for colouring
        wch_y_var <- c(wch_y_var, which(colnames(gg_dta) == "yvar"))
        tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
        pivot_cols <-
          colnames(tmp_dta)[-which(colnames(tmp_dta) %in% c("yvar", "yhat"))]
        gg_dta_mlt <-
          tidyr::pivot_longer(
            tmp_dta,
            tidyr::all_of(pivot_cols),
            names_to = "variable",
            values_to = "value"
          )
      } else {
        # Regression: keep yhat and the optional yvar reference column
        wch_y_var <- c(wch_y_var, which(colnames(gg_dta) == "yvar"))
        tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
        pivot_cols <-
          colnames(tmp_dta)[-which(colnames(tmp_dta) == "yhat")]
        gg_dta_mlt <-
          tidyr::pivot_longer(
            tmp_dta,
            tidyr::all_of(pivot_cols),
            names_to = "variable",
            values_to = "value"
          )
      }
      # Preserve user-supplied xvar ordering in the facet strips
      gg_dta_mlt$variable <-
        factor(gg_dta_mlt$variable, levels = xvar)

      # All continuous predictors → scatter; any factor → boxplot
      if (sum(ccls[wch_x_var] == "numeric") == length(wch_x_var)) {
        if (family == "class") {
          gg_plt <-
            ggplot2::ggplot(
              gg_dta_mlt,
              ggplot2::aes(
                x = .data$value,
                y = .data$yhat,
                color = .data$yvar,
                shape = .data$yvar
              )
            ) +
            ggplot2::geom_point(...)
        } else {
          gg_plt <- ggplot2::ggplot(
            gg_dta_mlt,
            ggplot2::aes(x = .data$value, y = .data$yhat)
          ) +
            ggplot2::geom_point(...)
        }
      } else {
        # Warn if numeric and factor predictors are mixed in the same panel
        if (sum(ccls[wch_x_var] == "numeric") > 0) {
          warning("Mismatched variable types...
                  assuming these are all factor variables.")
        }

        if (family == "class") {
          gg_plt <-
            ggplot2::ggplot(
              gg_dta_mlt,
              ggplot2::aes(
                x = .data$value,
                y = .data$yhat,
                color = .data$yvar
              )
            ) +
            ggplot2::geom_boxplot(...)
        } else {
          gg_plt <- ggplot2::ggplot(
            gg_dta_mlt,
            ggplot2::aes(x = .data$value, y = .data$yhat)
          ) +
            ggplot2::geom_boxplot(...)
        }
      }
      # Add point/smooth layers for non-classification forests
      if (family != "class") {
        if (points) {
          gg_plt <- gg_plt +
            ggplot2::geom_point(...)
        }
        if (smooth) {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
      }

      gg_plt <- gg_plt +
        ggplot2::facet_wrap(~variable, scales = "free_x") +
        ggplot2::labs(x = "")
    }

  ## =========================================================
  ## INDIVIDUAL PLOT branch — one ggplot per predictor variable
  ## =========================================================
  } else {
    # Pre-allocate a list; collapsed to a single object when lng == 1
    gg_plt <- vector("list", length = lng)

    for (ind in seq_len(lng)) {
      # Temporarily rename the target predictor column to "var" for aes()
      ch_indx <- which(colnames(gg_dta) == xvar[ind])
      h_name <- colnames(gg_dta)[ch_indx]
      colnames(gg_dta)[ch_indx] <- "var"
      # Use only the primary class (class() can return multiple strings, e.g.
      # c("POSIXct", "POSIXt")); a multi-element vector in if() triggers a warning.
      ccls_var <- class(gg_dta[, "var"])[1L]
      if (ccls_var == "integer") ccls_var <- "numeric"

      gg_plt[[ind]] <- ggplot2::ggplot(gg_dta)

      ## ---- Survival individual plot -----------------------------------
      if (family == "surv") {
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Survival")

        if (ccls_var == "numeric") {
          # Continuous predictor: scatter (and optional smooth)
          if (points) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$event,
                  shape = .data$event
                ),
                ...
              )
          } else {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
          if (smooth) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
        } else {
          # Factor predictor: boxplot + jittered points coloured by event
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::geom_boxplot(
              ggplot2::aes(x = .data$var, y = .data$yhat),
              color = "black",
              ...,
              outlier.shape = NA
            ) +
            ggplot2::geom_jitter(
              ggplot2::aes(
                x = .data$var,
                y = .data$yhat,
                color = .data$event,
                shape = .data$event
              ),
              ...
            )
        }
        # Multiple time points: facet vertically by time
        if (length(levels(gg_dta$time)) > 1) {
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::facet_wrap(~time, ncol = 1)
        } else {
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::labs(
              x = h_name,
              y = paste("Survival at", gg_dta$time[1], "year")
            )
        }

      ## ---- Classification individual plot ----------------------------
      } else if (family == "class") {
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Predicted")

        if (sum(colnames(gg_dta) == "outcome") == 0) {
          # Single-outcome (binary) classification
          if (ccls_var == "numeric") {
            if (points) {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_point(
                  ggplot2::aes(
                    x = .data$var,
                    y = .data$yhat,
                    color = .data$yvar,
                    shape = .data$yvar
                  ),
                  ...
                )
            } else {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_smooth(
                  ggplot2::aes(x = .data$var, y = .data$yhat),
                  color = "black",
                  linetype = 2,
                  ...
                )
            }
            if (smooth) {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_smooth(...)
            }
          } else {
            # Factor predictor: jitter + boxplot coloured by observed class
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_jitter(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              ) +
              ggplot2::geom_boxplot(
                ggplot2::aes(x = .data$var, y = .data$yhat),
                color = "grey",
                ...,
                outlier.shape = NA
              )
          }
        } else {
          # Multi-class: facet by outcome class
          if (ccls_var == "numeric") {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              )
          } else {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_boxplot(
                ggplot2::aes(x = .data$var, y = .data$yhat),
                color = "grey",
                ...,
                outlier.shape = NA
              ) +
              ggplot2::geom_jitter(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              )
          }

          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::facet_grid(~outcome)
        }

      ## ---- Regression individual plot --------------------------------
      } else {
        # assume regression
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Predicted")
        if (ccls_var == "numeric") {
          if (points) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          } else {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
          if (smooth) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
        } else {
          # Factor predictor: boxplot + jitter
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::geom_boxplot(
              ggplot2::aes(x = .data$var, y = .data$yhat),
              color = "grey",
              ...,
              outlier.shape = NA
            ) +
            ggplot2::geom_jitter(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
        }
      }

      # Restore the original column name before the next iteration
      colnames(gg_dta)[ch_indx] <- h_name
    }
    # Return a single ggplot when only one variable was requested
    if (lng == 1) {
      gg_plt <- gg_plt[[1]]
    }
  }
  return(gg_plt)
}

# testthat for gg_variable function

# Survival formula helper (rfsrc requires Surv to be in local scope)
Surv <- survival::Surv # nolint: object_name_linter

test_that("gg_variable classifications", {
  ## Load the cached forest
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_equal(rfsrc_iris$family, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Ensure non-OOB predictions keep predictor columns
  gg_full <- gg_variable(rfsrc_iris, oob = FALSE)
  expect_true(all(rfsrc_iris$xvar.names %in% names(gg_full)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, xvar = "Petal.Width")

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta, xvar = rfsrc_iris$xvar.names)

  # Test return is s ggplot object
  expect_type(gg_plt, "list")
  expect_equal(length(gg_plt), length(rfsrc_iris$xvar.names))
  for (ind in seq_along(rfsrc_iris$xvar.names))
    expect_s3_class(gg_plt[[ind]], "ggplot")
  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta, xvar = rfsrc_iris$xvar.names,
                             panel = TRUE)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  rf_iris <- randomForest::randomForest(Species ~ .,
                                        data = iris)

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rf_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, xvar = "Petal.Width")

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  gg_plt <- plot(gg_dta)

  ## Ensure we can rebuild training data for subset() calls
  iris_two <- droplevels(subset(iris, Species != "setosa"))
  rf_subset <- randomForest::randomForest(Species ~ .,
                                          data = iris_two,
                                          ntree = 60)
  gg_subset <- gg_variable(rf_subset)
  expect_s3_class(gg_subset, "gg_variable")
  expect_true(all(c("Sepal.Length", "Sepal.Width") %in% names(gg_subset)))

})


test_that("gg_variable regression", {
  data(Boston, package = "MASS")
  boston <- Boston

  boston$chas <- as.logical(boston$chas)

  ## Load the cached forest
  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta)

  # Test return is s ggplot object
  expect_type(gg_plt, "list")
  expect_equal(length(gg_plt), length(rfsrc_boston$xvar.names))
  for (ind in seq_along(rfsrc_boston$xvar.names))
    expect_s3_class(gg_plt[[ind]], "ggplot")


  ## Test plotting the gg_error object
  expect_warning(gg_plt <- plot.gg_variable(gg_dta, panel = TRUE))
  expect_s3_class(gg_plt, "ggplot")


  data(Boston, package = "MASS")
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
  gg_dta <- gg_variable(rf_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  expect_warning(gg_plt <- plot(gg_dta, panel = TRUE))
  expect_s3_class(gg_plt, "ggplot")

})

test_that("gg_variable survival handles late time requests", {
  data(veteran, package = "randomForestSRC")
  Surv <- survival::Surv # nolint: object_name_linter
  rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
                                          data = veteran,
                                          ntree = 50,
                                          nsplit = 5)
  late_time <- max(rfsrc_veteran$time.interest) + 50
  expect_silent(gg_dta <- gg_variable(rfsrc_veteran, time = late_time))
  expect_s3_class(gg_dta, "gg_variable")
})

test_that("gg_variable survival: single time, single continuous variable plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)
  expect_s3_class(gg_dta, "gg_variable")

  gg_plt <- plot(gg_dta, xvar = "age")
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: single time, panel plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)
  gg_plt <- plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: multiple times, single variable facets", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
  expect_s3_class(gg_dta, "gg_variable")

  # Single xvar → plot.gg_variable returns a single ggplot (not a list)
  gg_plt <- plot(gg_dta, xvar = "age")
  expect_s3_class(gg_plt, "ggplot")

  # Multiple xvars → returns a list
  gg_plt2 <- plot(gg_dta, xvar = c("age", "diagtime"))
  expect_type(gg_plt2, "list")
  expect_s3_class(gg_plt2[[1]], "ggplot")
})

test_that("gg_variable survival: multiple times, panel plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90))
  gg_plt <- plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: points=FALSE smooth=TRUE options", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)

  gg_plt <- plot(gg_dta, xvar = "age", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")

  gg_plt2 <- plot(gg_dta, xvar = "age", points = TRUE, smooth = FALSE)
  expect_s3_class(gg_plt2, "ggplot")
})

test_that("gg_variable survival: oob=FALSE uses in-bag predictions", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90, oob = FALSE)
  expect_s3_class(gg_dta, "gg_variable")
})

test_that("plot.gg_variable regression: points=FALSE smooth=TRUE", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)

  gg_plt <- plot(gg_dta, xvar = "lstat", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")

  gg_plt2 <- plot(gg_dta, xvar = "lstat", points = FALSE, smooth = FALSE)
  expect_s3_class(gg_plt2, "ggplot")
})

test_that("plot.gg_variable regression: factor x variable triggers boxplot", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_dta$chas <- factor(gg_dta$chas)

  gg_plt <- plot(gg_dta, xvar = "chas")
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable regression: panel with two continuous variables", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta, xvar = c("lstat", "rm"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable regression: panel points=FALSE smooth=TRUE", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta, xvar = c("lstat", "rm"),
                 panel = TRUE, points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable classification: panel with multiple continuous vars", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_iris)
  gg_plt <- plot(gg_dta, xvar = c("Petal.Width", "Sepal.Width"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable classification: smooth and no-points path", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_iris)
  gg_plt <- plot(gg_dta, xvar = "Petal.Width", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable: missing xvar returns list for all predictors", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta)
  expect_type(gg_plt, "list")
  expect_gt(length(gg_plt), 0)
  expect_s3_class(gg_plt[[1]], "ggplot")
})

# testthat for gg_variable function

# Survival formula helper (rfsrc requires Surv to be in local scope)
Surv <- survival::Surv # nolint: object_name_linter

test_that("gg_variable classifications", {
  ## Load the cached forest
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    importance = TRUE,
    save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_equal(rfsrc_iris$family, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Ensure non-OOB predictions keep predictor columns
  gg_full <- gg_variable(rfsrc_iris, oob = FALSE)
  expect_true(all(rfsrc_iris$xvar.names %in% names(gg_full)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, xvar = "Petal.Width")

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta, xvar = rfsrc_iris$xvar.names)

  # Test return is a single plottable object (patchwork composite or ggplot)
  expect_true(inherits(gg_plt, "patchwork") || inherits(gg_plt, "ggplot"))
  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta, xvar = rfsrc_iris$xvar.names,
                             panel = TRUE)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  rf_iris <- randomForest::randomForest(Species ~ .,
                                        data = iris)

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rf_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta, xvar = "Petal.Width")

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
  gg_plt <- plot(gg_dta)

  ## Ensure we can rebuild training data for subset() calls
  iris_two <- droplevels(subset(iris, Species != "setosa"))
  rf_subset <- randomForest::randomForest(Species ~ .,
                                          data = iris_two,
                                          ntree = 60)
  gg_subset <- gg_variable(rf_subset)
  expect_s3_class(gg_subset, "gg_variable")
  expect_true(all(c("Sepal.Length", "Sepal.Width") %in% names(gg_subset)))

})


test_that("gg_variable regression", {
  data(Boston, package = "MASS")
  boston <- Boston

  boston$chas <- as.logical(boston$chas)

  ## Load the cached forest
  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)

  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  ## Create the correct gg_error object
  gg_dta <- gg_variable(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  ## Test plotting the gg_error object
  gg_plt <- plot.gg_variable(gg_dta)

  # Test return is a single plottable object (patchwork composite or ggplot)
  expect_true(inherits(gg_plt, "patchwork") || inherits(gg_plt, "ggplot"))


  ## Test plotting the gg_error object
  expect_warning(gg_plt <- plot.gg_variable(gg_dta, panel = TRUE))
  expect_s3_class(gg_plt, "ggplot")


  data(Boston, package = "MASS")
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
  gg_dta <- gg_variable(rf_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_variable")

  expect_warning(gg_plt <- plot(gg_dta, panel = TRUE))
  expect_s3_class(gg_plt, "ggplot")

})

test_that("gg_variable survival handles late time requests", {
  data(veteran, package = "randomForestSRC")
  Surv <- survival::Surv # nolint: object_name_linter
  rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
                                          data = veteran,
                                          ntree = 50,
                                          nsplit = 5)
  late_time <- max(rfsrc_veteran$time.interest) + 50
  expect_silent(gg_dta <- gg_variable(rfsrc_veteran, time = late_time))
  expect_s3_class(gg_dta, "gg_variable")
})

test_that("gg_variable survival: single time, single continuous variable plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)
  expect_s3_class(gg_dta, "gg_variable")

  gg_plt <- plot(gg_dta, xvar = "age")
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: single time, panel plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)
  gg_plt <- plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: multiple times, single variable facets", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
  expect_s3_class(gg_dta, "gg_variable")

  # Single xvar → plot.gg_variable returns a single ggplot (not a list)
  gg_plt <- plot(gg_dta, xvar = "age")
  expect_s3_class(gg_plt, "ggplot")

  # Multiple xvars → returns a single plottable object (patchwork composite)
  gg_plt2 <- plot(gg_dta, xvar = c("age", "diagtime"))
  expect_true(inherits(gg_plt2, "patchwork") || inherits(gg_plt2, "ggplot"))
})

test_that("gg_variable survival: multiple times, panel plot", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90))
  gg_plt <- plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("gg_variable survival: points=FALSE smooth=TRUE options", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90)

  gg_plt <- plot(gg_dta, xvar = "age", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")

  gg_plt2 <- plot(gg_dta, xvar = "age", points = TRUE, smooth = FALSE)
  expect_s3_class(gg_plt2, "ggplot")
})

test_that("gg_variable survival: oob=FALSE uses in-bag predictions", {
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  rfsrc_veteran <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran,
    ntree = 50,
    nsplit = 5
  )

  gg_dta <- gg_variable(rfsrc_veteran, time = 90, oob = FALSE)
  expect_s3_class(gg_dta, "gg_variable")
})

test_that("plot.gg_variable regression: points=FALSE smooth=TRUE", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)

  gg_plt <- plot(gg_dta, xvar = "lstat", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")

  gg_plt2 <- plot(gg_dta, xvar = "lstat", points = FALSE, smooth = FALSE)
  expect_s3_class(gg_plt2, "ggplot")
})

test_that("plot.gg_variable regression: factor x variable triggers boxplot", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_dta$chas <- factor(gg_dta$chas)

  gg_plt <- plot(gg_dta, xvar = "chas")
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable regression: panel with two continuous variables", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta, xvar = c("lstat", "rm"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable regression: panel points=FALSE smooth=TRUE", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta, xvar = c("lstat", "rm"),
                 panel = TRUE, points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable classification: panel with multiple continuous vars", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_iris)
  gg_plt <- plot(gg_dta, xvar = c("Petal.Width", "Sepal.Width"), panel = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable classification: smooth and no-points path", {
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_iris)
  gg_plt <- plot(gg_dta, xvar = "Petal.Width", points = FALSE, smooth = TRUE)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_variable: missing xvar returns list for all predictors", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    forest = TRUE,
    save.memory = TRUE
  )

  gg_dta <- gg_variable(rfsrc_boston)
  gg_plt <- plot(gg_dta)
  # Returns a single plottable object (patchwork composite for multiple predictors)
  expect_true(inherits(gg_plt, "patchwork") || inherits(gg_plt, "ggplot"))
})

test_that("gg_variable.randomForest classification: class attr uses 'class' not 'classification'", {
  # randomForest stores the family in $type as "classification", but
  # plot.gg_variable and the rfsrc path both dispatch on "class".
  # Verify the mapping is applied so callers see a consistent class attribute.
  set.seed(42)
  rf_iris <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg_dta  <- gg_variable(rf_iris)

  expect_true("class"          %in% class(gg_dta))
  expect_false("classification" %in% class(gg_dta))
  expect_s3_class(gg_dta, "gg_variable")
})

## ── randomForest classification (PR #87) ─────────────────────────────────────

test_that("gg_variable.randomForest classification: produces yhat.* columns not yhat", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  # Must have one column per class
  expect_true(all(c("yhat.setosa", "yhat.versicolor", "yhat.virginica")
                  %in% names(gg)))
  # Must NOT have a bare yhat column for multi-class
  expect_false("yhat" %in% names(gg))
  # Observed-class column must be present
  expect_true("yvar" %in% names(gg))
  # Vote fractions must be in [0, 1] and row-sum to ~1
  vote_cols <- c("yhat.setosa", "yhat.versicolor", "yhat.virginica")
  expect_true(all(gg[, vote_cols] >= 0))
  expect_true(all(gg[, vote_cols] <= 1))
  expect_true(all(abs(rowSums(gg[, vote_cols]) - 1) < 1e-6))
})

test_that("gg_variable.randomForest classification: plot returns patchwork for all xvar", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  p  <- plot(gg)
  # iris has 4 predictors so the no-xvar default assembles a multi-panel
  # patchwork; assert patchwork specifically to catch regressions to a bare
  # list (#80).
  expect_s3_class(p, "patchwork")
})

test_that("gg_variable.randomForest classification: layer_data works on single-xvar plot", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  p  <- plot(gg, xvar = "Sepal.Length")
  expect_no_error(ggplot2::layer_data(p, 1L))
})

test_that("gg_variable.randomForest classification: default plot renders every panel", {
  # yvar / outcome must not be treated as predictors by the default-xvar pick.
  skip_if_not_installed("randomForest")
  # Regression test: the default-xvar selection in plot.gg_variable used to
  # include yvar and outcome (the response factor and the multi-class pivot's
  # facet column), pivoting them into `var` and dropping them from the
  # panel data. The geom_jitter aes then errored at render time with
  # "Column `yvar` not found". This test exercises a real build of every
  # patchwork sub-plot, which is what catches the regression — checking the
  # patchwork class alone does not, because patchwork is lazy.
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  p  <- plot(gg)
  expect_s3_class(p, "patchwork")
  # Force every sub-plot to actually build.
  for (sub in p$patches$plots) {
    expect_no_error(ggplot2::ggplot_build(sub))
  }
})

test_that("plot.gg_variable default xvar matches column names exactly, not substring", {
  # Regression: the pre-existing default-xvar exclusion used grep("time", ...)
  # and grep("event", ...), which silently dropped any predictor whose name
  # *contained* those substrings -- e.g. the documented veteran-data survival
  # predictor `diagtime`. Switch to exact matching for event/time/yvar/outcome
  # and a prefix match for yhat (to catch yhat.<class> columns).
  data(veteran, package = "randomForestSRC")
  set.seed(42L)
  Surv <- survival::Surv # nolint: object_name_linter
  rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                                          ntree = 25, nsplit = 3)
  gg <- gg_variable(rfsrc_veteran, time = 90)
  expect_true("diagtime" %in% names(gg))
  # The default xvar list must include diagtime (not dropped by substring match).
  p <- plot(gg)
  # The plot should be a patchwork (multiple predictors) -- and diagtime should
  # appear as one of the panels. Force every sub-plot to build cleanly.
  expect_s3_class(p, "patchwork")
  for (sub in p$patches$plots) {
    expect_no_error(ggplot2::ggplot_build(sub))
  }
  # Spot-check: at least one panel's data has `var` sourced from diagtime by
  # confirming diagtime is NOT in the residual data columns of every panel
  # (it was pivoted into `var` for exactly one panel).
  var_panel_count <- sum(vapply(p$patches$plots, function(sub) {
    !("diagtime" %in% names(sub$data))
  }, logical(1L)))
  expect_gte(var_panel_count, 1L)
})

test_that("gg_variable.randomForest classification: norm.votes=FALSE still gives [0,1] fractions", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L,
                                   norm.votes = FALSE)
  gg <- gg_variable(rf)
  vote_cols <- c("yhat.setosa", "yhat.versicolor", "yhat.virginica")
  expect_true(all(c("yhat.setosa", "yhat.versicolor", "yhat.virginica") %in% names(gg)))
  expect_true(all(gg[, vote_cols] >= 0))
  expect_true(all(gg[, vote_cols] <= 1))
  expect_true(all(abs(rowSums(gg[, vote_cols]) - 1) < 1e-6))
})

test_that("plot.gg_variable RF classification: smooth=TRUE layer_data smokeable (binary smooth aes bug)", {
  skip_if_not_installed("randomForest")
  # Two-class subset to exercise the *binary* classification path
  set.seed(42L)
  bin_data        <- iris[iris$Species != "virginica", ]
  bin_data$Species <- droplevels(bin_data$Species)
  rf  <- randomForest::randomForest(Species ~ ., data = bin_data, ntree = 50L)
  gg  <- gg_variable(rf)
  p   <- plot(gg, xvar = "Sepal.Length", smooth = TRUE)
  # Before the fix, geom_smooth(...)  has no aes and layer_data errors with
  # "stat_smooth() requires the following missing aesthetics: x and y"
  expect_no_error(ggplot2::layer_data(p, 2L))
})

test_that("plot.gg_variable RF classification: smooth=TRUE works for multi-class (missing block)", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  # Before the fix the multi-class numeric path silently skips smooth=TRUE
  # but does not error; after the fix a smooth layer is present (layer 2).
  p  <- plot(gg, xvar = "Sepal.Length", smooth = TRUE)
  expect_s3_class(p, "ggplot")
  ld <- ggplot2::layer_data(p, 2L)   # layer 2 = geom_smooth
  expect_gt(nrow(ld), 0L)
})

test_that("plot.gg_variable RF classification multi-class: outcome column is class names not integers", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  p  <- plot(gg, xvar = "Sepal.Length")
  expect_s3_class(p, "ggplot")
  # The 'outcome' column in the plot data drives facet labels.
  # It must contain class names, not integer indices.
  # ggplot2 >= 3.5 uses S7 slots; fall back to $ accessor for older versions.
  pd <- tryCatch(p@data, error = function(e) p$data)
  expect_false(is.numeric(pd$outcome))
  expect_true(all(c("setosa", "versicolor", "virginica") %in% as.character(pd$outcome)))
})

test_that("plot.gg_variable RF classification multi-class: outcome factor levels match column order", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  gg <- gg_variable(rf)
  p  <- plot(gg, xvar = "Sepal.Length")
  pd <- tryCatch(p@data, error = function(e) p$data)
  # Levels must follow the yhat.* column order in gg_variable output,
  # not alphabetical order (which factor() would impose by default).
  expected_levels <- sub("^yhat\\.", "", grep("^yhat\\.", names(gg), value = TRUE))
  expect_equal(levels(pd$outcome), expected_levels)
})

test_that("gg_variable.randomForest: oob=FALSE triggers a warning", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  rf <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
  # oob=FALSE is not supported for randomForest; a warning must be emitted
  # and OOB vote fractions are still returned.
  expect_warning(
    gg <- gg_variable(rf, oob = FALSE),
    regexp = "oob = FALSE is not supported"
  )
  expect_s3_class(gg, "gg_variable")
  expect_true("yhat.setosa" %in% names(gg))
})

## ── smooth=TRUE is a no-op for factor predictors (all families) ──────────────

# geom_smooth requires a continuous x-axis, so smooth=TRUE has no meaning for a
# factor predictor.  The contract these tests lock in: no GeomSmooth layer is
# added for a factor x.  Assert that directly (rather than a brittle layer
# count) so benign plot-composition changes do not break the suite.
has_smooth_layer <- function(p) {
  any(vapply(p$layers, function(l) inherits(l$geom, "GeomSmooth"), logical(1)))
}

test_that("plot.gg_variable binary classification + factor predictor: smooth=TRUE adds no smooth layer", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  bin_data         <- iris[iris$Species != "virginica", ]
  bin_data$Species <- droplevels(bin_data$Species)
  bin_data$size    <- cut(bin_data$Petal.Length, 2L, labels = c("small", "large"))
  rf  <- randomForest::randomForest(Species ~ size + Sepal.Width, data = bin_data,
                                    ntree = 25L)
  gg  <- gg_variable(rf)
  # smooth=TRUE with a factor predictor must not error
  expect_no_error(p <- plot(gg, xvar = "size", smooth = TRUE))
  expect_s3_class(p, "ggplot")
  expect_false(has_smooth_layer(p))
})

test_that("plot.gg_variable multi-class classification + factor predictor: smooth=TRUE adds no smooth layer", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  iris2      <- iris
  iris2$size <- cut(iris2$Petal.Length, 3L, labels = c("small", "medium", "large"))
  rf <- randomForest::randomForest(Species ~ size + Sepal.Width, data = iris2,
                                   ntree = 25L)
  gg <- gg_variable(rf)
  # smooth=TRUE with a factor predictor must not error
  expect_no_error(p <- plot(gg, xvar = "size", smooth = TRUE))
  expect_s3_class(p, "ggplot")
  expect_false(has_smooth_layer(p))
})

test_that("plot.gg_variable regression + factor predictor: smooth=TRUE adds no smooth layer", {
  skip_if_not_installed("randomForest")
  set.seed(42L)
  iris2      <- iris
  iris2$size <- cut(iris2$Petal.Length, 3L, labels = c("small", "medium", "large"))
  rf <- randomForest::randomForest(Sepal.Length ~ size + Sepal.Width, data = iris2,
                                   ntree = 25L)
  gg <- gg_variable(rf)
  # smooth=TRUE with a factor predictor must not error
  expect_no_error(p <- plot(gg, xvar = "size", smooth = TRUE))
  expect_s3_class(p, "ggplot")
  expect_false(has_smooth_layer(p))
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
# Internal helper: normalise the which_outcome argument.
# "all" is not yet fully supported; fall back to class 1 with a warning.
.validate_which_outcome <- function(which_outcome) {
  if (identical(which_outcome, "all")) {
    warning("Must specify which_outcome for now.")
    return(1L)
  }
  which_outcome
}

#' Receiver Operator Characteristic calculator
#'
#' @details For a randomForestSRC prediction and the actual
#' response value, calculate the specificity (1-False Positive Rate) and
#' sensitivity (True Positive Rate) of a predictor.
#'
#' This is a helper function for the \code{\link{gg_roc}} functions, and
#' not intended for use by the end user.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForestSRC]{predict.rfsrc}}, or
#'   \code{\link[randomForest]{randomForest}} classification object containing
#'   predicted class probabilities.
#' @param dta A factor (or coercible to factor) of the true observed class
#'   labels, one per observation. Typically \code{object$yvar} for rfsrc or
#'   \code{object$y} for randomForest.
#' @param which_outcome Integer index of the class for which the ROC curve is
#'   computed (e.g. \code{1} for the first class, \code{2} for the second).
#'   Use \code{"all"} to request all classes (currently falls back to class 1
#'   with a warning).
#' @param oob Logical; if \code{TRUE} (default for rfsrc) use OOB predicted
#'   probabilities. Forced to \code{FALSE} for \code{randomForest} objects.
#' @param ... Extra arguments passed to helper functions (currently unused).
#'
#' @return A \code{gg_roc} \code{data.frame} with columns \code{sens}
#'   (sensitivity), \code{spec} (specificity), and \code{pct} (the probability
#'   threshold), with one row per unique prediction value. Suitable for passing
#'   to \code{\link{calc_auc}} or \code{\link{plot.gg_roc}}.
#'
#' @aliases calc_roc.rfsrc calc_roc.randomForest calc_roc
#'
#' @seealso \code{\link{calc_auc}} \code{\link{gg_roc}}
#' @seealso \code{\link{plot.gg_roc}}
#'
#' @importFrom parallel mclapply
#' @importFrom stats xtabs
#' @importFrom utils head tail
#' @importFrom randomForest randomForest
#'
#' @examples
#' ## Taken from the gg_roc example
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris)
#'
#' gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
#'   which_outcome = 1, oob = TRUE
#' )
#' gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
#'   which_outcome = 1, oob = FALSE
#' )
#'
#' rf_iris <- randomForest(Species ~ ., data = iris)
#' gg_dta <- calc_roc(rf_iris, rf_iris$yvar,
#'   which_outcome = 1
#' )
#' gg_dta <- calc_roc(rf_iris, rf_iris$yvar,
#'   which_outcome = 2
#' )
#'
#' @export
calc_roc.rfsrc <-
  function(object,
           dta,
           which_outcome = "all",
           oob = TRUE,
           ...) {
    # Ensure response is a factor so levels() is well-defined
    if (!is.factor(dta)) {
      dta <- factor(dta)
    }

    # "all" outcomes not yet supported; fall back to the first class
    which_outcome <- .validate_which_outcome(which_outcome)
    # Build (binary indicator, full-forest prediction, OOB prediction) triplet
    dta_roc <-
      data.frame(cbind(
        res = (dta == levels(dta)[which_outcome]),
        prd = object$predicted[, which_outcome],
        oob_prd = object$predicted.oob[, which_outcome]
      ))

    # Collect the unique predicted probability thresholds for the ROC sweep
    if (oob) {
      pct <- sort(unique(object$predicted.oob[, which_outcome]))
    } else {
      pct <- sort(unique(object$predicted[, which_outcome]))
    }

    last <- length(pct)
    # Remove the maximum threshold (the cutpoint where nothing is classified
    # as positive), which produces the (sens=0, spec=1) anchor point
    pct <- pct[-last]

    # Cap at 200 threshold points to keep multi-class ROC plots manageable
    if (last > 200) {
      pct <- pct[seq(1, length(pct), length.out = 200)]
    }

    # For each threshold, build the 2×2 confusion table and extract TPR/TNR
    gg_dta <- parallel::mclapply(pct, function(crit) {
      if (oob) {
        tbl <- xtabs(~ res + (oob_prd > crit), dta_roc)
      } else {
        tbl <- xtabs(~ res + (prd > crit), dta_roc)
      }

      spec <- tbl[2, 2] / rowSums(tbl)[2]
      sens <- tbl[1, 1] / rowSums(tbl)[1]
      cbind(sens = sens, spec = spec)
    })

    gg_dta <- do.call(rbind, gg_dta)
    # Anchor curve at perfect specificity (0, 1) and perfect sensitivity (1, 0)
    gg_dta <- rbind(c(0, 1), gg_dta, c(1, 0))

    gg_dta <- data.frame(gg_dta, row.names = seq_len(nrow(gg_dta)))
    gg_dta$pct <- c(0, pct, 1)
    invisible(gg_dta)
  }

#' @export
calc_roc <- function(object,
                     dta,
                     which_outcome = "all",
                     oob = TRUE,
                     ...) {
  UseMethod("calc_roc", object)
}

## This is in development still.
#' @export
calc_roc.randomForest <-
  function(object,
           dta,
           which_outcome = 1,
           oob = FALSE,
           ...) {
    prd <- predict(object, type = "prob")

    which_outcome <- .validate_which_outcome(which_outcome)
    dta_roc <-
      data.frame(cbind(res = (dta == levels(dta)[which_outcome]), prd = prd))

    pct <- sort(unique(prd[[which_outcome]]))

    # Make sure we don't have to many points... if the training set was large,
    # This may break plotting all ROC curves in multiclass settings.
    # Arbitrarily reduce this to only include 200 points along the curve
    if (length(pct) > 200) {
      pct <- pct[seq(1, length(pct), length.out = 200)]
    }

    gg_dta <- parallel::mclapply(pct, function(crit) {
      tmp <- dta_roc[, c(1, 1 + which_outcome)]
      colnames(tmp) <- c("res", "prd")
      tbl <- xtabs(~ res + (prd > crit), tmp)

      if (dim(tbl)[2] < 2) {
        tbl <- cbind(tbl, c(0, 0))
        colnames(tbl) <- c("FALSE", "TRUE")
      }
      spec <- tbl[2, 2] / rowSums(tbl)[2]
      sens <- tbl[1, 1] / rowSums(tbl)[1]

      cbind(sens = sens, spec = spec)
    })

    gg_dta <- do.call(rbind, gg_dta)
    gg_dta <- rbind(c(0, 1), gg_dta, c(1, 0))

    gg_dta <- data.frame(gg_dta, row.names = seq_len(nrow(gg_dta)))
    gg_dta$pct <- c(0, pct, 1)
    invisible(gg_dta)
  }

#'
#' Area Under the ROC Curve calculator
#'
#' @details calc_auc uses the trapezoidal rule to calculate the area under
#' the ROC curve.
#'
#'  This is a helper function for the \code{\link{gg_roc}} functions.
#'
#' @param x \code{\link{gg_roc}} object
#'
#' @return AUC. 50\% is random guessing, higher is better.
#'
# @importFrom dplyr lead
#'
#' @seealso \code{\link{calc_roc}} \code{\link{gg_roc}}
#' @seealso \code{\link{plot.gg_roc}}
#'
#' @examples
#' ##
#' ## Taken from the gg_roc example
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris)
#'
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#'
#' calc_auc(gg_dta)
#'
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#'
#' calc_auc(gg_dta)
#'
#' ## randomForest tests
#' rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#'
#' calc_auc(gg_dta)
#'
#' @aliases calc_auc calc_auc.gg_roc
#' @export
calc_auc <- function(x) {
  ## Trapezoidal rule:  AUC = Σ (f(x_i) + f(x_{i+1})) / 2 * |Δx|
  ## Here f(x) is sensitivity (TPR) and x is 1 - specificity (FPR).
  ## Sort so that specificity decreases (FPR increases) left-to-right,
  ## then each step moves one FPR increment to the right.

  # Sort in decreasing specificity so FPR = 1-spec increases monotonically
  x <- x[order(x$spec, decreasing = TRUE), ]

  # Δ(FPR) = -(Δspec)  — spec decreases, so (spec[i] - spec[i+1]) > 0
  # Average height of trapezoid = (sens[i] + sens[i+1]) / 2
  auc <- (x$sens + shift(x$sens)) / 2 * (x$spec - shift(x$spec)) # nolint: object_usage_linter
  sum(auc, na.rm = TRUE)
}

calc_auc.gg_roc <- calc_auc # nolint: object_name_linter

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Predicted response plot from a \code{\link{gg_rfsrc}} object.
#'
#' Visualizes the ensemble predictions extracted by \code{\link{gg_rfsrc}}.
#' By default those are out-of-bag (OOB) predictions -- the forest's built-in
#' cross-validation estimate, averaging only over the trees that left a given
#' observation out of their bootstrap sample.
#'
#' The geometry adapts to the forest family.  For regression or
#' classification, you get a jitter-and-boxplot: every observation is a dot
#' placed at its OOB predicted value, coloured by observed response, with a
#' notched boxplot overlaid to show the central tendency and spread.  For a
#' survival forest, each observation contributes one ensemble survival curve
#' (or CHF / mortality, whichever \code{surv_type} was chosen in
#' \code{gg_rfsrc}); the bundle of step functions shows the spread of
#' predicted risk across the cohort.  Pass \code{by} to \code{gg_rfsrc}
#' beforehand to colour curves by a predictor group, or \code{conf.int} to
#' replace the individual curves with a mean curve and bootstrap ribbon.
#'
#' @param x A \code{\link{gg_rfsrc}} object, or a raw
#'   \code{\link[randomForestSRC]{rfsrc}} object (which will be passed through
#'   \code{\link{gg_rfsrc}} automatically before plotting).
#' @param notch Logical; whether to draw notched boxplots for regression and
#'   classification forests (default \code{TRUE}).  Set \code{notch = FALSE}
#'   to suppress notches when sample sizes are too small for reliable
#'   confidence intervals on the median.
#' @param ... Additional arguments forwarded to the underlying
#'   \code{ggplot2} geometry calls.  Commonly useful arguments include:
#'   \describe{
#'     \item{\code{alpha}}{Numeric in \eqn{[0,1]}; point/ribbon transparency.
#'       For survival plots with confidence bands the ribbon alpha is
#'       automatically halved relative to the value supplied here.}
#'     \item{\code{size}}{Point or line size passed to \code{geom_jitter},
#'       \code{geom_step}, etc.}
#'   }
#'   Arguments that control \code{\link{gg_rfsrc}} (e.g. \code{conf.int},
#'   \code{surv_type}, \code{by}) should be applied when constructing the
#'   \code{gg_rfsrc} object before calling \code{plot()}.
#'
#' @return A \code{ggplot} object.  The plot appearance depends on the forest
#'   family stored in \code{x}:
#'   \describe{
#'     \item{Regression (\code{"regr"})}{Jitter + notched boxplot of OOB
#'       predicted values.  If a \code{group} column is present the x-axis
#'       shows each group label; otherwise observations are collapsed to a
#'       single x-position.}
#'     \item{Classification (\code{"class"})}{Binary: jitter + notched
#'       boxplot of the predicted class probability.  Multi-class: jitter
#'       plot with one panel per class (class probabilities in long form).}
#'     \item{Survival (\code{"surv"})}{Step curves of the ensemble survival
#'       function.  When \code{gg_rfsrc} was called with \code{conf.int},
#'       a shaded ribbon is added.  When called with \code{by}, curves are
#'       coloured by group.}
#'   }
#'
#' @seealso \code{\link{gg_rfsrc}} \code{\link[randomForestSRC]{rfsrc}}
#'   \code{\link[randomForest]{randomForest}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
#' R, Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' # Build a small classification forest (ntree=50 keeps example fast)
#' set.seed(42)
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_iris)
#'
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' # na.action = "na.impute" handles missing Ozone / Solar.R values
#' set.seed(42)
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_airq)
#'
#' plot(gg_dta)
#'
#' ## -------- mtcars data
#' set.seed(42)
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_mtcars)
#'
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## -------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_veteran)
#' plot(gg_dta)
#'
#' # With 95% pointwise bootstrap confidence bands
#' gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = .95)
#' plot(gg_dta)
#'
#' # Stratified by treatment arm
#' gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt")
#' plot(gg_dta)
#'
#' ## -------- pbc data (larger dataset -- skipped on CRAN)
#' \donttest{
#' data(pbc, package = "randomForestSRC")
#' # For whatever reason, the age variable is in days; convert to years
#' for (ind in seq_len(dim(pbc)[2])) {
#'   if (!is.factor(pbc[, ind])) {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   } else {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'       if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   }
#'   if (!is.logical(pbc[, ind]) &
#'     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'     pbc[, ind] <- factor(pbc[, ind])
#'   }
#' }
#' # Convert age from days to years
#' pbc$age <- pbc$age / 364.24
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' # Remove test-set patients (those with no assigned treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#'
#' set.seed(42)
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'   dta_train,
#'   nsplit = 10,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   save.memory = TRUE
#' )
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc)
#' plot(gg_dta)
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = .95)
#' plot(gg_dta)
#'
#' gg_dta <- gg_rfsrc(rfsrc_pbc, by = "treatment")
#' plot(gg_dta)
#' }
#'
#' @export
plot.gg_rfsrc <- function(x, notch = TRUE, ...) {
  gg_dta <- x

  # Capture extra passthrough args (e.g. alpha for survival ribbons).
  # notch is a named formal above so it is never present in arg_set.
  arg_set <- list(...)

  ## If the user passed a raw rfsrc object, extract predictions first
  if (inherits(gg_dta, "rfsrc")) {
    gg_dta <- gg_rfsrc(gg_dta)
  }

  ## ---- Classification forest branch ------------------------------------
  if (inherits(gg_dta, "class") ||
    inherits(gg_dta, "classification")) {
    # Identify probability columns (everything except "y" and "group")
    non_prob <- c("y", "group")
    prob_cols <- setdiff(colnames(gg_dta), non_prob)
    prob_col <- prob_cols[1]
    obs_col  <- "y"
    if (length(prob_cols) < 2) {
      # Binary classification: single probability column + observed class
      gg_plt <- ggplot2::ggplot(gg_dta) +
        ggplot2::geom_jitter(
          ggplot2::aes(
            x = 1,
            y = .data[[prob_col]],
            color = .data[[obs_col]],
            shape = .data[[obs_col]]
          ),
          ...
        ) +
        ggplot2::geom_boxplot(
          ggplot2::aes(x = 1, y = .data[[prob_col]]),
          outlier.colour = "transparent",
          fill = "transparent",
          notch = notch
        ) +
        ggplot2::theme(
          axis.ticks = ggplot2::element_blank(),
          axis.text.x = ggplot2::element_blank()
        )
    } else {
      # Multi-class: gather all class probability columns into long form
      pivot_cols <- setdiff(colnames(gg_dta), c("y", "group"))
      gg_dta_mlt <-
        tidyr::pivot_longer(gg_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")

      gg_plt <-
        ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(x = .data$variable, y = .data$value)
        ) +
        ggplot2::geom_jitter(
          ggplot2::aes(color = .data$y, shape = .data$y),
          alpha = .5
        )
    }
    gg_plt <- gg_plt + ggplot2::labs(y = "Predicted (%)", x = "")

  ## ---- Survival forest branch ------------------------------------------
  } else if (inherits(gg_dta, "surv")) {
    # Detect whether bootstrap confidence bands have been computed
    if ("lower" %in% colnames(gg_dta)) {
      # Split ... into ribbon alpha (halved) and step dots (full alpha).
      # Prevents duplicate-formal error when user passes alpha via ...
      spl  <- .gg_split_alpha(arg_set)
      alph <- spl$ribbon_alpha

      if ("group" %in% colnames(gg_dta)) {
        # Stratified survival curves with CI ribbon, coloured by group
        gg_plt <- ggplot2::ggplot(gg_dta) +
          do.call(
            ggplot2::geom_ribbon,
            c(list(
              ggplot2::aes(
                x    = .data$value,
                ymin = .data$lower,
                ymax = .data$upper,
                fill = .data$group
              ),
              alpha = alph
            ), spl$ribbon_dots)
          ) +
          do.call(
            ggplot2::geom_step,
            c(list(ggplot2::aes(
              x     = .data$value,
              y     = .data$median,
              color = .data$group
            )), spl$step_dots)
          )
      } else {
        # Single-group survival curve with CI ribbon
        gg_plt <- ggplot2::ggplot(gg_dta) +
          do.call(
            ggplot2::geom_ribbon,
            c(list(
              ggplot2::aes(
                x    = .data$value,
                ymin = .data$lower,
                ymax = .data$upper
              ),
              alpha = alph,
              fill  = .gg_ribbon_fill
            ), spl$ribbon_dots)
          ) +
          do.call(
            ggplot2::geom_step,
            c(list(ggplot2::aes(
              x = .data$value,
              y = .data$median
            )), spl$step_dots)
          )
      }
    } else {
      # No confidence bands: draw one step line per observation
      gg_plt <- ggplot2::ggplot(
        gg_dta,
        ggplot2::aes(
          x = .data$variable,
          y = .data$value,
          col = .data$event,
          group = .data$obs_id
        )
      ) +
        ggplot2::geom_step(...)
    }

    gg_plt <- gg_plt +
      ggplot2::labs(x = "time (years)", y = "Survival (%)")

  ## ---- Regression forest branch ----------------------------------------
  } else if (inherits(gg_dta, "regr") ||
    inherits(gg_dta, "regression")) {
    if ("group" %in% colnames(gg_dta)) {
      # Grouped regression: x-axis shows each group label
      gg_plt <- ggplot2::ggplot(gg_dta, ggplot2::aes(x = .data$group, y = .data$yhat))
    } else {
      # Single-group regression: collapse to a single x position
      gg_plt <- ggplot2::ggplot(gg_dta, ggplot2::aes(x = 1, y = .data$yhat))
    }

    gg_plt <- gg_plt +
      ggplot2::geom_jitter(...) +
      ggplot2::geom_boxplot(
        outlier.colour = "transparent",
        fill = "transparent",
        notch = notch
      ) +
      ggplot2::labs(
        y = "Predicted Value",
        x = if ("group" %in% colnames(gg_dta)) "Group" else ""
      ) +
      ggplot2::theme(
        axis.ticks = ggplot2::element_blank(),
        axis.text.x = if ("group" %in% colnames(gg_dta)) {
          ggplot2::element_text()
        } else {
          ggplot2::element_blank()
        }
      )
  } else {
    # Unknown forest type: not yet implemented
    stop(paste(
      "Plotting for ",
      class(gg_dta)[2],
      " randomForestSRC is not yet implemented."
    ))
  }
  return(gg_plt)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)


## ----packages-----------------------------------------------------------------
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)
library(survival)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())

# Plotting constants
event_marks  <- c(1, 4)
event_labels <- c("Censored", "Death")
event_colors <- c("steelblue", "firebrick")


## ----data-load----------------------------------------------------------------
data("pbc", package = "randomForestSRC")


## ----data-clean---------------------------------------------------------------
pbc <- pbc |>
  mutate(
    years     = days / 365.25,
    age       = age / 365.25,
    treatment = factor(
      ifelse(treatment == 1, "DPCA",
             ifelse(treatment == 2, "Placebo", NA)),
      levels = c("DPCA", "Placebo")
    )
  ) |>
  select(-days)

# Low-cardinality numerics (including binary 0/1) to factor.
# NOTE: do NOT convert to logical — partial.rfsrc() fails with logical
# predictors in survival forests (randomForestSRC <= 3.5.1).
# Exclude the response columns (status, years) from conversion.
resp_cols <- c("status", "years")
for (nm in setdiff(names(pbc), resp_cols)) {
  v <- pbc[[nm]]
  if (is.numeric(v) && !is.factor(v) && length(unique(v[!is.na(v)])) <= 5) {
    pbc[[nm]] <- factor(v)
  }
}


## ----variable-labels----------------------------------------------------------
# Human-readable labels for plot axes
st_labs <- c(
  status       = "Death Event",
  treatment    = "Treatment",
  age          = "Age (years)",
  sex          = "Female",
  ascites      = "Ascites",
  hepato       = "Hepatomegaly",
  spiders      = "Spiders",
  edema        = "Edema (0, 0.5, 1)",
  bili         = "Serum Bilirubin (mg/dl)",
  chol         = "Serum Cholesterol (mg/dl)",
  albumin      = "Albumin (gm/dl)",
  copper       = "Urine Copper (ug/day)",
  alk.phos     = "Alkaline Phosphatase (U/liter)",
  ast          = "SGOT (U/ml)",
  trig         = "Triglycerides (mg/dl)",
  platelet     = "Platelets (per cubic ml/1000)",
  prothrombin  = "Prothrombin Time (sec)",
  stage        = "Histologic Stage",
  years        = "Follow-up Time (years)"
)


## ----eda-categorical, fig.height=4, fig.cap="EDA for categorical variables. Bar color indicates factor level; white = missing."----
cls <- sapply(pbc, class)
cnt_idx <- which(cls %in% c("numeric", "integer"))
fct_idx <- setdiff(seq_along(pbc), cnt_idx)
fct_idx <- union(fct_idx, which(names(pbc) == "years"))

dta_cat <- suppressWarnings(
  pbc[, fct_idx] |>
    pivot_longer(-years, names_to = "variable", values_to = "value",
                 values_transform = list(value = as.character))
)

ggplot(dta_cat, aes(x = years, fill = value)) +
  geom_histogram(color = "black", binwidth = 1) +
  labs(y = "", x = st_labs["years"]) +
  scale_fill_brewer(palette = "RdBu", na.value = "white") +
  facet_wrap(~variable, scales = "free_y", nrow = 2) +
  theme(legend.position = "none")


## ----eda-continuous, fig.height=4, fig.cap="EDA for continuous variables. Points colored by death event; rug marks indicate missing values."----
cnt_idx <- union(cnt_idx, which(names(pbc) == "status"))
dta_cont <- pbc[, cnt_idx] |>
  pivot_longer(c(-years, -status),
               names_to = "variable", values_to = "value")

ggplot(dta_cont |> filter(!is.na(value)),
       aes(x = years, y = value, color = factor(status), shape = factor(status))) +
  geom_point(alpha = 0.4) +
  geom_rug(data = dta_cont |> filter(is.na(value)), color = "grey50") +
  labs(y = "", x = st_labs["years"], color = "Death", shape = "Death") +
  scale_color_manual(values = event_colors) +
  scale_shape_manual(values = event_marks) +
  facet_wrap(~variable, scales = "free_y", ncol = 4) +
  theme(legend.position = c(0.8, 0.2))


## ----km-survival, fig.cap="KM survival estimates by treatment group with 95% confidence bands."----
pbc_trial <- pbc |> filter(!is.na(treatment))
pbc_test  <- pbc |> filter(is.na(treatment))

gg_km <- gg_survival(interval = "years", censor = "status",
                     by = "treatment", data = pbc_trial,
                     conf.int = 0.95)

plot(gg_km) +
  labs(y = "Survival Probability", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.2)) +
  coord_cartesian(ylim = c(0, 1.01))


## ----km-cumhaz, fig.cap="KM cumulative hazard estimates by treatment group."----
plot(gg_km, type = "cum_haz") +
  labs(y = "Cumulative Hazard", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.8)) +
  coord_cartesian(ylim = c(-0.02, 1.22))


## ----km-bili, fig.width=6, fig.cap="KM survival stratified by bilirubin groups."----
pbc_bili <- pbc_trial |>
  mutate(bili_grp = cut(bili, breaks = c(0, 0.8, 1.3, 3.4, 29)))

plot(gg_survival(interval = "years", censor = "status",
                 by = "bili_grp", data = pbc_bili),
     error = "none") +
  labs(y = "Survival Probability", x = "Time (years)", color = "Bilirubin")


## ----rfsrc-fit----------------------------------------------------------------
# Step 1: impute missing values via random forest proximity
pbc_imputed <- impute.rfsrc(Surv(years, status) ~ .,
                             data    = pbc_trial,
                             ntree   = 100,
                             nimpute = 2)

# Step 2: grow the survival forest on the complete imputed data
rfsrc_pbc <- rfsrc(Surv(years, status) ~ .,
                   data      = pbc_imputed,
                   ntree     = 150,
                   nsplit    = 10,
                   tree.err  = TRUE,
                   importance = TRUE)


## ----error-plot, fig.height=4, fig.cap="OOB prediction error vs. number of trees."----
plot(gg_error(rfsrc_pbc))


## ----rfsrc-predicted, fig.cap="OOB predicted survival curves. Blue = censored, red = death events."----
gg_rsf <- plot(gg_rfsrc(rfsrc_pbc), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
gg_rsf


## ----rfsrc-by-treatment, fig.cap="Median predicted survival by treatment group with 95% confidence bands."----
plot(gg_rfsrc(rfsrc_pbc, by = "treatment")) +
  theme(legend.position = c(0.2, 0.2)) +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----predict-test, fig.cap="Predicted survival for non-trial patients (test set)."----
rfsrc_pbc_test <- predict(rfsrc_pbc, newdata = pbc_test,
                          na.action = "na.impute",
                          importance = TRUE)

plot(gg_rfsrc(rfsrc_pbc_test), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----vimp-plot, fig.width=6, fig.cap="Variable importance ranking. Blue = positive VIMP, red = negative."----
plot(gg_vimp(rfsrc_pbc), lbls = st_labs) +
  theme(legend.position = c(0.8, 0.2)) +
  labs(fill = "VIMP > 0")


## ----varsel-------------------------------------------------------------------
md_pbc <- max.subtree(rfsrc_pbc)


## ----select-vars--------------------------------------------------------------
xvar     <- c("bili", "albumin", "copper", "prothrombin", "age")
xvar_cat <- "edema"
xvar_all <- c(xvar, xvar_cat)


## ----vardep-bili, fig.height=4, fig.cap="Variable dependence of survival at 1 and 3 years on bilirubin."----
gg_v <- gg_variable(rfsrc_pbc, time = c(1, 3),
                    time.labels = c("1 Year", "3 Years"))

plot(gg_v, xvar = "bili", alpha = 0.4) +
  labs(y = "Survival", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----vardep-panel, fig.height=5, fig.cap="Variable dependence at 1 and 3 years for continuous predictors."----
plot(gg_v, xvar = xvar[-1], panel = TRUE, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.05, 1.05))


## ----vardep-categorical, fig.height=4, fig.cap="Variable dependence on edema (categorical)."----
plot(gg_v, xvar = xvar_cat, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.02))


## ----partial-dep, error=TRUE, fig.height=5, fig.cap="Partial dependence of survival at approximately 1 and 3 years on continuous predictors."----
try({
# partial.rfsrc() requires times that match the model's time.interest grid;
# gg_partial_rfsrc() snaps the requested values to the nearest observed times.
ti   <- rfsrc_pbc$time.interest
t1yr <- ti[which.min(abs(ti - 1))]
t3yr <- ti[which.min(abs(ti - 3))]

pd <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = xvar,
                       partial.time = c(t1yr, t3yr))

# Quick S3 plot — survival forests produce time-series curves per predictor value
plot(pd)
})


## ----partial-dep-custom, error=TRUE, fig.height=5, fig.cap="Partial dependence (custom styling)."----
try({
ggplot(pd$continuous, aes(x = x, y = yhat,
                          color = factor(round(time, 2)),
                          group = factor(time))) +
  geom_line(linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = "Predicted Survival", x = "", color = "Year") +
  scale_color_manual(values = setNames(c("steelblue", "firebrick"),
                                       as.character(round(c(t1yr, t3yr), 2)))) +
  theme_bw()
})


## ----coplot-edema, fig.width=7, fig.height=4, fig.cap="Variable dependence on bilirubin, conditional on edema group."----
gg_v1 <- gg_variable(rfsrc_pbc, time = 1)
gg_v1$edema <- paste("edema =", gg_v1$edema)

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_grid(~edema) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----coplot-albumin, fig.width=7, fig.height=5, fig.cap="Variable dependence on bilirubin, conditional on albumin groups."----
albumin_cts <- quantile_pts(gg_v1$albumin, groups = 6, intervals = TRUE)
gg_v1$albumin_grp <- cut(gg_v1$albumin, breaks = albumin_cts)
levels(gg_v1$albumin_grp) <- paste("albumin =",
                                    levels(gg_v1$albumin_grp))

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_wrap(~albumin_grp) +
  coord_cartesian(ylim = c(-0.01, 1.01))


## ----surface-data, error=TRUE, cache=FALSE------------------------------------
try({
# Create grid of albumin values
alb_grid <- quantile_pts(pbc_trial$albumin, groups = 8)

# For each albumin value, compute partial dependence on bili at ~1 year
surface_list <- lapply(alb_grid, function(alb_val) {
  newx <- pbc_trial[, rfsrc_pbc$xvar.names, drop = FALSE]
  newx$albumin <- alb_val
  pd_alb <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = "bili",
                              newx = newx, partial.time = t1yr)
  df <- pd_alb$continuous
  df$albumin <- alb_val
  df
})

surface_df <- bind_rows(surface_list)
})


## ----pd-surface, error=TRUE, fig.height=5, fig.cap="Partial dependence surface: survival at 1 year as a function of bilirubin and albumin. Fill colour is the predicted survival probability."----
try({
if (!exists("surface_df")) {
  message("surface_df not available --- skipping surface (see surface-data chunk error above).")
} else {
  ggplot(surface_df, aes(x = x, y = albumin, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Survival") +
    labs(x = "Bilirubin", y = "Albumin") +
    theme_bw()
}
})


## ----brier-overall, fig.width=7, fig.height=3.5, fig.cap="Time-resolved Brier score for the PBC survival forest."----
gg_bs <- gg_brier(rfsrc_pbc)
plot(gg_bs)


## ----brier-envelope, fig.width=7, fig.height=3.5, fig.cap="Brier score with 15--85% per-subject envelope."----
plot(gg_bs, envelope = TRUE)


## ----brier-crps, fig.width=7, fig.height=3.5, fig.cap="Running CRPS for the PBC survival forest."----
plot(gg_bs, type = "crps")


## ----brier-crps-scalar--------------------------------------------------------
attr(gg_bs, "crps_integrated")

##=============================================================================
#' Per-variable lasso-beta importance from a varPro fit
#'
#' Tidy wrapper around [varPro::beta.varpro()] for the regression or
#' classification family.
#' Aggregates the per-rule lasso coefficient \eqn{\hat{\beta}}{beta hat} by
#' variable into the mean absolute value
#' \eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)} and flags variables
#' above a scalar cutoff. Optional
#' `beta_fit` argument lets callers compute the expensive
#' `beta.varpro()` step once and reuse the result.
#'
#' @section What this is doing:
#' Think of the varPro release-rule mechanism as asking: "given a region of
#' the feature space that the forest carved out, what changes when I remove
#' the constraint on this one variable and let observations leave?" The
#' standard importance answer (from [gg_varpro()]) measures that change as a
#' z-scored contrast between local estimators: no synthetic data, no
#' permutation. \code{beta.varpro()} asks the same question with a different
#' ruler: for each rule (a tree-branch pair), it fits a one-predictor lasso
#' regression of the response on the released variable's values, restricted
#' to the OOB observations inside the rule's region. The wrapper aggregates
#' those per-rule coefficients into one number per variable.
#'
#' The key distinction from [gg_vimp()], which measures Breiman-Cutler
#' permutation importance by perturbing a variable's values and watching OOB
#' error climb, is that neither [gg_varpro()] nor \code{gg_beta_varpro()}
#' touches the data synthetically: all contrasts are between real subsets
#' defined by the forest's rules.
#'
#' @section What `imp` actually is (pedantic, because the column name is misleading):
#' The `imp` column on `beta.varpro()`'s `$results` is **not** a
#' variable-importance score in the conventional sense. It is a regularised
#' regression coefficient. Specifically:
#'
#' - Per rule, `glmnet` fits a one-predictor lasso of the response on
#'   the released variable inside the rule's OOB region. `use.cv = TRUE`
#'   selects \eqn{\lambda}{lambda} by 10-fold CV (default `nfolds = 10`);
#'   `use.1se = TRUE` (default) picks `lambda.1se`. `use.cv = FALSE` uses the
#'   full \eqn{\lambda}{lambda} path.
#' - `imp` is the **fitted coefficient** \eqn{\hat{\beta}}{beta hat} at the
#'   chosen \eqn{\lambda}{lambda}. **Sign is
#'   real** (direction of local association). **Magnitude depends on
#'   the predictor's units** (raw `x`, no standardisation); a predictor
#'   in millimetres has a smaller \eqn{|\hat{\beta}|}{|beta hat|} than the
#'   same predictor in metres.
#' - Lasso shrinkage can drive \eqn{\hat{\beta}}{beta hat} to **exactly zero**.
#'   Those zeros are
#'   data, not missingness, and are kept in the aggregation. Convergence
#'   failures land as `NA_real_` and are dropped.
#' - The per-variable aggregate is `beta_mean`,
#'   \eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)}, across the
#'   rules where this variable was released. It is **not** a permutation
#'   importance, **not** a split-strength importance, and **not** directly
#'   comparable on the same numeric axis to `gg_varpro()`'s z-scores.
#'   Disagreement with `gg_varpro` is often diagnostic, not a bug.
#'
#' In code form: `imp_r` is the glmnet coefficient \eqn{\hat{\beta}}{beta hat}
#' fit on `y ~ x_v` restricted to rule r, with \eqn{\lambda}{lambda} chosen by
#' `use.cv` / `use.1se`.
#'
#' @section What's in the output:
#' One row per released variable. Columns:
#' - `variable`: predictor name.
#' - `beta_mean`: mean of \eqn{|\hat{\beta}|}{|beta hat|} across that
#'   variable's rules.
#' - `n_rules`: count of rules contributing (zero-beta rules included; only
#'   `NA` failures excluded).
#' - `selected`: `beta_mean >= cutoff`.
#'
#' Provenance attribute carries `source`, `family`, `ntree`, `cutoff`,
#' `cutoff_default`, `use.cv`, `n_rules_total`, `n_rules_nonzero`,
#' `precomputed`, and `xvar.names`.
#'
#' @section What you use this for:
#' Picking variables when local effects matter more than aggregate
#' split-strength contribution. Compare side-by-side with [gg_varpro()]:
#' a variable that scores high here but low in `gg_varpro` is one whose
#' local linear effect inside many rules is real even though its
#' release-rule contrast is modest.
#'
#' @section Caching:
#' `beta.varpro()` is the expensive call (per-rule `glmnet` / `cv.glmnet`,
#' often minutes on real data). Compute it once and reuse:
#'
#' ```r
#' v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 200)
#' b <- varPro::beta.varpro(v, use.cv = TRUE)        # expensive, once
#' gg_a <- gg_beta_varpro(v, beta_fit = b)            # cheap
#' gg_b <- gg_beta_varpro(v, beta_fit = b, cutoff = 0.5)
#' ```
#'
#' Provenance carries `precomputed = TRUE` when `beta_fit` was supplied.
#'
#' @section Classification:
#' For a varpro classification fit (`object$family == "class"`,
#' binary or multi-class), the returned frame is long-format with an
#' extra `class` column: one row per (variable, class) pair. The
#' `beta_mean` column aggregates the **per-class lasso coefficient**
#' \eqn{\hat{\beta}}{beta hat} stored in
#' `beta.varpro()`'s `imp.<k>` columns (one per class level). Same
#' pedantic-beta semantics as regression, applied independently to each
#' class.
#'
#' **Binary default**: `which_class = NULL` resolves to the *last*
#' factor level of the response, the positive-class convention used
#' by `glm` and `gg_roc`. For a 30-day-mortality outcome with levels
#' `c("no", "yes")`, that means the wrapper shows you `"yes"` (the
#' event) by default.
#'
#' **Multi-class default**: `which_class = NULL` returns all K
#' classes; the plot method renders `facet_wrap(~ class)` with one
#' cutoff line per facet.
#'
#' **`which_class = "<name>"`** filters to a single class regardless
#' of K. Errors if the name isn't in the response levels.
#'
#' **Per-class cutoffs**: `cutoff = NULL` resolves to each class's
#' `mean(beta_mean)`. A scalar broadcasts. A named numeric vector
#' overrides per class; missing names fall back to that class's mean.
#'
#' Example (30-day mortality, binary):
#' ```r
#' fit <- varPro::varpro(event_30d ~ ., data = clinical, ntree = 200)
#' gg  <- gg_beta_varpro(fit)   # default: "yes" panel
#' plot(gg)
#' ```
#'
#' @section Reproducibility:
#' Byte-for-byte agreement between cached (`beta_fit = b`) and uncached
#' (`beta_fit = NULL`) outputs requires that `b` was computed by
#' `beta.varpro(object, ...)` on the same `object`; `set.seed()` alone is
#' not sufficient, because `beta.varpro`'s internal `cv.glmnet` fits can
#' pick slightly different folds across separate calls. Reuse `beta_fit`
#' when reproducibility matters.
#'
#' @note Multivariate regression (`regr+`) and survival families are out
#'   of scope for this release and tracked for v3.1.0. The
#'   unsupported-family path errors with a message pointing at that work.
#'
#' @param object A `varpro` fit from [varPro::varpro()] (regression or
#'   classification family).
#' @param ... Forwarded to [varPro::beta.varpro()] when `beta_fit = NULL`;
#'   ignored otherwise (with a warning). Documented forwardables: `use.cv`,
#'   `use.1se`, `nfolds`, `maxit`, `thresh`, `max.rules.tree`, `max.tree`.
#' @param cutoff Selection threshold on `beta_mean`. `NULL` (default) means
#'   `mean(beta_mean)` across released variables. Numeric scalar otherwise.
#' @param beta_fit Optional pre-computed [varPro::beta.varpro()] result for
#'   the same `object`. `NULL` (default) means the wrapper runs `beta.varpro()`
#'   itself. When supplied, must be a `varpro`-class object whose `$results`
#'   has columns `tree / branch / variable / n.oob / imp`.
#' @param which_class For a classification fit, name of a single response
#'   level to subset on. `NULL` (default) returns all classes (binary fits
#'   resolve to the *last* factor level, the positive-class convention
#'   used by `glm` and `gg_roc`). Ignored with a warning on regression
#'   fits.
#'
#' @return A `data.frame` of class `c("gg_beta_varpro", "data.frame")`.
#'   For a regression fit: one row per released variable, sorted by
#'   `beta_mean` descending. For a classification fit: long-format with
#'   an extra `class` column, one row per (variable, class) pair;
#'   `variable` is a factor whose levels are set by
#'   `mean(|sum-of-class-beta|)` descending so every facet / panel shares
#'   the same row order. `which_class` (or the binary default
#'   last-factor-level) collapses the output to a single class.
#'
#' @seealso [gg_varpro()], [gg_vimp()], [plot.gg_beta_varpro()], [varPro::beta.varpro()].
#'
#' @examples
#' \donttest{
#' if (requireNamespace("varPro", quietly = TRUE)) {
#'   set.seed(1)
#'   v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
#'   b <- varPro::beta.varpro(v)
#'   gg <- gg_beta_varpro(v, beta_fit = b)
#'   plot(gg)
#' }
#' }
#'
#' @export
gg_beta_varpro <- function(object, ..., cutoff = NULL, beta_fit = NULL,
                           which_class = NULL) {
  UseMethod("gg_beta_varpro", object)
}

#' @export
gg_beta_varpro.varpro <- function(object, ..., cutoff = NULL,
                                  beta_fit = NULL, which_class = NULL) {
  if (!inherits(object, "varpro")) {
    stop("gg_beta_varpro: expected a 'varpro' object from varPro::varpro().",
         call. = FALSE)
  }
  fam <- object$family
  if (!fam %in% c("regr", "class")) {
    stop(sprintf(
      paste0("gg_beta_varpro currently supports varpro regression and ",
             "classification forests only; got family = '%s'. regr+ and ",
             "survival are tracked for v3.1.0 (see vignette / NEWS)."),
      fam
    ), call. = FALSE)
  }

  # Resolve beta_fit (cache path)
  if (is.null(beta_fit)) {
    b <- varPro::beta.varpro(object, ...)
  } else {
    .validate_beta_fit(beta_fit, object, fam)
    if (length(list(...)) > 0L) {
      warning("gg_beta_varpro: arguments in '...' ignored because beta_fit is supplied.",
              call. = FALSE)
    }
    b <- beta_fit
  }

  # Warn on which_class with regression
  if (fam == "regr" && !is.null(which_class)) {
    warning("gg_beta_varpro: which_class ignored for regression family.",
            call. = FALSE)
    which_class <- NULL
  }

  # Capture use.cv from `...` here (NOT inside the internals; the dots
  # don't pass through to the internal frame).
  dots_use_cv <- if (is.null(beta_fit)) isTRUE(list(...)$use.cv) else NA

  # Empty fast-path
  if (is.null(b) || nrow(b$results) == 0L) {
    return(.gg_beta_varpro_empty(object, fam, which_class, beta_fit, cutoff))
  }

  if (fam == "regr") {
    return(.gg_beta_varpro_regr(object, b, cutoff, beta_fit, use_cv = dots_use_cv))
  }
  .gg_beta_varpro_class(object, b, cutoff, which_class, beta_fit,
                        use_cv = dots_use_cv)
}

#' @noRd
.gg_beta_varpro_regr <- function(object, b, cutoff, beta_fit, use_cv) {
  res <- b$results
  res <- res[is.finite(res$imp), , drop = FALSE]
  n_rules_total   <- nrow(b$results)
  n_rules_nonzero <- sum(abs(res$imp) > 0)

  var_name <- b$xvar.names[res$variable]
  beta_mean_v <- vapply(split(abs(res$imp), var_name), mean, numeric(1))
  n_rules_v   <- vapply(split(res$imp,        var_name), length, integer(1))

  # Rows are most-important-first (descending); the variable factor levels
  # are REVERSED so that, after coord_flip() in the plot method, the
  # most-important variable lands at the TOP. This matches the
  # gg_vimp / gg_varpro convention.
  ord_names <- names(sort(beta_mean_v, decreasing = TRUE))

  resolved_cutoff <- if (is.null(cutoff)) mean(beta_mean_v) else as.numeric(cutoff)

  out <- data.frame(
    variable  = factor(ord_names, levels = rev(ord_names)),
    beta_mean = unname(beta_mean_v[ord_names]),
    n_rules   = as.integer(unname(n_rules_v[ord_names])),
    stringsAsFactors = FALSE
  )
  out$selected <- out$beta_mean >= resolved_cutoff
  rownames(out) <- NULL

  class(out) <- c("gg_beta_varpro", "data.frame")
  attr(out, "provenance") <- list(
    source          = "varPro::beta.varpro",
    family          = "regr",
    ntree           = if (!is.null(object$ntree)) as.integer(object$ntree) else NA_integer_,
    cutoff          = stats::setNames(resolved_cutoff, "regr"),
    cutoff_default  = is.null(cutoff),
    use.cv          = use_cv,
    n_rules_total   = n_rules_total,
    n_rules_nonzero = n_rules_nonzero,
    precomputed     = !is.null(beta_fit),
    xvar.names      = b$xvar.names
  )
  out
}

#' @noRd
.validate_beta_fit <- function(beta_fit, object, fam) {
  if (!inherits(beta_fit, "varpro") || !is.data.frame(beta_fit$results)) {
    stop("gg_beta_varpro: beta_fit does not look like a varPro::beta.varpro() result. ",
         "Expected a varpro-class object with a data.frame in $results.",
         call. = FALSE)
  }
  required_cols <- c("tree", "branch", "variable", "n.oob", "imp")
  missing_cols <- setdiff(required_cols, names(beta_fit$results))
  if (length(missing_cols) > 0L) {
    stop("gg_beta_varpro: beta_fit does not look like a varPro::beta.varpro() result. ",
         "Missing column(s): ", paste(missing_cols, collapse = ", "), ".",
         call. = FALSE)
  }
  if (fam == "class") {
    cls <- .class_levels_from_varpro(object)
    cls_cols <- paste0("imp.", seq_along(cls))
    missing_cls_cols <- setdiff(cls_cols, names(beta_fit$results))
    if (length(missing_cls_cols) > 0L) {
      stop("gg_beta_varpro: beta_fit does not look like a classification ",
           "varPro::beta.varpro() result. Missing per-class column(s): ",
           paste(missing_cls_cols, collapse = ", "), ". ",
           "Expected one imp.<k> column per response level (",
           length(cls), " levels: ", paste(cls, collapse = ", "), ").",
           call. = FALSE)
    }
  }
  invisible(NULL)
}

#' @noRd
.class_levels_from_varpro <- function(object) {
  # varPro preserves original factor names in y.org; object$y may have been
  # internally relabelled to 0/1 for binary fits.
  if (!is.null(object$y.org) && is.factor(object$y.org)) {
    return(levels(object$y.org))
  }
  if (is.factor(object$y)) {
    return(levels(object$y))
  }
  if (!is.null(attr(object$y, "levels"))) {
    return(attr(object$y, "levels"))
  }
  sort(unique(as.character(object$y)))
}

#' @noRd
.resolve_class_cutoff <- function(cutoff, per_class_mean, class_levels) {
  n_classes <- length(class_levels)
  if (is.null(cutoff)) {
    return(per_class_mean)
  }
  if (is.null(names(cutoff))) {
    if (length(cutoff) != 1L) {
      stop("gg_beta_varpro: cutoff must be NULL, scalar, or a named vector with names in class levels.",
           call. = FALSE)
    }
    return(stats::setNames(rep(as.numeric(cutoff), n_classes), class_levels))
  }
  bad <- setdiff(names(cutoff), class_levels)
  if (length(bad) > 0L) {
    stop(sprintf(
      "gg_beta_varpro: cutoff name(s) '%s' is not a level of the response. Levels: %s.",
      paste(bad, collapse = ", "),
      paste(class_levels, collapse = ", ")
    ), call. = FALSE)
  }
  cv <- per_class_mean
  cv[names(cutoff)] <- as.numeric(cutoff)
  cv
}

#' @noRd
.gg_beta_varpro_class <- function(object, b, cutoff, which_class, beta_fit,
                                  use_cv) {
  class_levels <- .class_levels_from_varpro(object)
  n_classes    <- length(class_levels)
  imp_cols     <- paste0("imp.", seq_len(n_classes))

  # Validate which_class
  if (!is.null(which_class)) {
    if (!which_class %in% class_levels) {
      stop(sprintf(
        "gg_beta_varpro: which_class = '%s' is not a level of the response. Levels: %s.",
        which_class, paste(class_levels, collapse = ", ")
      ), call. = FALSE)
    }
  } else if (n_classes == 2L) {
    which_class <- class_levels[n_classes]   # binary default = last (positive class)
  }

  res <- b$results
  var_name <- b$xvar.names[res$variable]

  # Total |imp| per variable. ord_names is most-important-first (drives row
  # sort + summary); lvl is the REVERSED order used for the shared factor
  # levels so the most-important variable lands at the TOP after coord_flip,
  # consistently across every class facet.
  res_total <- res[is.finite(res$imp), , drop = FALSE]
  total_var <- b$xvar.names[res_total$variable]
  beta_mean_total <- vapply(split(abs(res_total$imp), total_var), mean, numeric(1))
  ord_names <- names(sort(beta_mean_total, decreasing = TRUE))
  lvl <- rev(ord_names)

  # Per-class aggregation: long format
  rows <- list()
  for (k in seq_len(n_classes)) {
    col <- imp_cols[k]
    imp_k <- res[[col]]
    keep  <- is.finite(imp_k)
    if (!any(keep)) next
    sub_var <- var_name[keep]
    sub_imp <- imp_k[keep]
    beta_mean_k <- vapply(split(abs(sub_imp), sub_var), mean, numeric(1))
    n_rules_k   <- vapply(split(sub_imp,        sub_var), length, integer(1))
    vars_present <- intersect(ord_names, names(beta_mean_k))
    rows[[k]] <- data.frame(
      variable  = factor(vars_present, levels = lvl),
      class     = class_levels[k],
      beta_mean = unname(beta_mean_k[vars_present]),
      n_rules   = as.integer(unname(n_rules_k[vars_present])),
      stringsAsFactors = FALSE
    )
  }
  long <- do.call(rbind, rows)

  # Resolve cutoff
  per_class_mean <- vapply(class_levels, function(cls) {
    vals <- long$beta_mean[long$class == cls]
    if (length(vals) == 0L) NA_real_ else mean(vals)
  }, numeric(1))
  names(per_class_mean) <- class_levels

  resolved_cutoff <- .resolve_class_cutoff(cutoff, per_class_mean, class_levels)

  long$selected <- mapply(function(bm, cls) bm >= resolved_cutoff[[cls]],
                          long$beta_mean, long$class)

  # Filter to single class if requested
  if (!is.null(which_class)) {
    long <- long[long$class == which_class, , drop = FALSE]
  }

  # Sort: class factor order, then most-important-first within class. The
  # reversed variable factor levels only drive the plot's vertical order.
  long$class <- factor(long$class, levels = class_levels)
  long <- long[order(long$class, -as.integer(long$variable)), , drop = FALSE]
  rownames(long) <- NULL

  class(long) <- c("gg_beta_varpro", "data.frame")
  attr(long, "provenance") <- list(
    source          = "varPro::beta.varpro",
    family          = "class",
    ntree           = if (!is.null(object$ntree)) as.integer(object$ntree) else NA_integer_,
    cutoff          = resolved_cutoff,
    cutoff_default  = is.null(cutoff),
    use.cv          = use_cv,
    n_rules_total   = nrow(b$results),
    n_rules_nonzero = sum(abs(res_total$imp) > 0, na.rm = TRUE),
    precomputed     = !is.null(beta_fit),
    xvar.names      = b$xvar.names,
    class_levels    = class_levels,
    which_class     = which_class
  )
  long
}

#' @noRd
.gg_beta_varpro_empty <- function(object, fam, which_class, beta_fit, cutoff) {
  base <- data.frame(
    variable  = factor(character(0)),
    beta_mean = numeric(0),
    n_rules   = integer(0),
    selected  = logical(0),
    stringsAsFactors = FALSE
  )
  if (fam == "class") {
    base <- cbind(base[, "variable", drop = FALSE],
                  class    = character(0),
                  base[, c("beta_mean", "n_rules", "selected"), drop = FALSE])
  }
  class(base) <- c("gg_beta_varpro", "data.frame")

  # Build provenance with shape-stable cutoff:
  # regr  gives c("regr" = NA_real_)
  # class gives named NA_real_ vector, one entry per class level
  if (fam == "class") {
    class_levels <- .class_levels_from_varpro(object)
    cutoff_empty <- stats::setNames(rep(NA_real_, length(class_levels)),
                                    class_levels)
    prov <- list(
      source = "varPro::beta.varpro", family = "class",
      n_rules_total = 0L, n_rules_nonzero = 0L,
      cutoff = cutoff_empty,
      cutoff_default = is.null(cutoff),
      use.cv = NA,
      precomputed = !is.null(beta_fit),
      class_levels = class_levels,
      which_class = which_class
    )
  } else {
    prov <- list(
      source = "varPro::beta.varpro", family = "regr",
      n_rules_total = 0L, n_rules_nonzero = 0L,
      cutoff = stats::setNames(NA_real_, "regr"),
      cutoff_default = is.null(cutoff),
      use.cv = NA,
      precomputed = !is.null(beta_fit),
      which_class = which_class
    )
  }
  attr(base, "provenance") <- prov
  base
}

#' @importFrom ggplot2 autoplot
#' @export
autoplot.gg_beta_varpro <- function(object, ...) {
  plot.gg_beta_varpro(object, ...)
}

#' Survival partial dependence data for one or more predictors
#'
#' \strong{Deprecated.} Use \code{\link{gg_partial_rfsrc}} instead, which
#' returns a classed \code{gg_partial_rfsrc} object with a dedicated
#' \code{plot()} method.
#'
#' Computes partial dependence curves for a survival or competing-risk
#' \code{\link[randomForestSRC]{rfsrc}} forest by calling
#' \code{\link[randomForestSRC]{partial.rfsrc}} at \code{npts} evenly-spaced
#' unique values of each predictor across all stored event times.
#'
#' @param rforest A fitted \code{\link[randomForestSRC]{rfsrc}} survival or
#'   competing-risk forest object.
#' @param var_list Character vector of predictor names for which partial
#'   dependence should be computed. Each must appear in
#'   \code{rforest$xvar.names}.
#' @param npts Integer; the number of predictor grid points to evaluate
#'   (default 25). Evenly-spaced unique values are sampled from each predictor.
#' @param partial.type The prediction type to return. For survival forests one
#'   of \code{"surv"} (default), \code{"mort"}, or \code{"chf"}. For competing
#'   risk forests one of \code{"years.lost"}, \code{"cif"}, or \code{"chf"}.
#'   See \code{\link[randomForestSRC]{partial.rfsrc}} for full details.
#'
#' @return A named list with one element per variable in \code{var_list}. Each
#'   element is itself a list with:
#'   \describe{
#'     \item{name}{The predictor variable name (character).}
#'     \item{dta}{The raw output of
#'       \code{\link[randomForestSRC]{get.partial.plot.data}}, a list containing
#'       at minimum \code{x} (predictor values) and \code{yhat} (partial
#'       predictions), and for survival/competing risk, \code{partial.time}.}
#'   }
#'
#' @seealso \code{\link{gg_partial_rfsrc}},
#'   \code{\link[randomForestSRC]{partial.rfsrc}},
#'   \code{\link[randomForestSRC]{get.partial.plot.data}}
#'
#' @importFrom randomForestSRC partial.rfsrc
#' @examples
#' ## ------------------------------------------------------------
#' ## survival
#' ## ------------------------------------------------------------
#'
#' data(veteran, package = "randomForestSRC")
#' v.obj <- randomForestSRC::rfsrc(Surv(time,status)~.,
#'   veteran, nsplit = 10, ntree = 100)
#'
#' spart <- surv_partial.rfsrc(v.obj, var_list="age", partial.type = "mort")
#'
#' ## partial effect of age on mortality
#' partial.obj <- partial(v.obj,
#'                        partial.type = "mort",
#'                        partial.xvar = "age",
#'                        partial.values = v.obj$xvar$age,
#'                        partial.time = v.obj$time.interest)
#' pdta <- get.partial.plot.data(partial.obj)
#'
#' plot(lowess(pdta$x, pdta$yhat, f = 1/3),
#'      type = "l", xlab = "age", ylab = "adjusted mortality")
#'
#' ## example where x is discrete - partial effect of age on mortality
#' ## we use the granule=TRUE option
#' partial.obj <- partial(v.obj,
#'                        partial.type = "mort",
#'                        partial.xvar = "trt",
#'                        partial.values = v.obj$xvar$trt,
#'                        partial.time = v.obj$time.interest)
#' pdta <- get.partial.plot.data(partial.obj, granule = TRUE)
#' boxplot(pdta$yhat ~ pdta$x, xlab = "treatment", ylab = "partial effect")
#'
#'
#' ## partial effects of karnofsky score on survival
#' karno <- quantile(v.obj$xvar$karno)
#' partial.obj <- partial(v.obj,
#'                        partial.type = "surv",
#'                        partial.xvar = "karno",
#'                        partial.values = karno,
#'                        partial.time = v.obj$time.interest)
#' pdta <- get.partial.plot.data(partial.obj)
#'
#' matplot(pdta$partial.time, t(pdta$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "karnofsky adjusted survival")
#' legend("topright", legend = paste0("karnofsky = ", karno), fill = 1:5)
#'
#'
#' ## ------------------------------------------------------------
#' ## competing risk
#' ## ------------------------------------------------------------
#'
#' data(follic, package = "randomForestSRC")
#' follic.obj <- rfsrc(Surv(time, status) ~ ., follic, nsplit = 3, ntree = 100)
#'
#' ## partial effect of age on years lost
#' partial.obj <- partial(follic.obj,
#'                        partial.type = "years.lost",
#'                        partial.xvar = "age",
#'                        partial.values = follic.obj$xvar$age,
#'                        partial.time = follic.obj$time.interest)
#' pdta1 <- get.partial.plot.data(partial.obj, target = 1)
#' pdta2 <- get.partial.plot.data(partial.obj, target = 2)
#'
#' # Save and restore the user's graphical parameters per CRAN policy.
#' oldpar <- par(no.readonly = TRUE)
#' on.exit(par(oldpar))
#' par(mfrow = c(2, 2))
#' plot(lowess(pdta1$x, pdta1$yhat),
#'      type = "l", xlab = "age", ylab = "adjusted years lost relapse")
#' plot(lowess(pdta2$x, pdta2$yhat),
#'      type = "l", xlab = "age", ylab = "adjusted years lost death")
#'
#' ## partial effect of age on cif
#' partial.obj <- partial(follic.obj,
#'                        partial.type = "cif",
#'                        partial.xvar = "age",
#'                        partial.values = quantile(follic.obj$xvar$age),
#'                        partial.time = follic.obj$time.interest)
#' pdta1 <- get.partial.plot.data(partial.obj, target = 1)
#' pdta2 <- get.partial.plot.data(partial.obj, target = 2)
#'
#' matplot(pdta1$partial.time, t(pdta1$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "age adjusted cif for relapse")
#' matplot(pdta2$partial.time, t(pdta2$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "age adjusted cif for death")
#'
#' @export surv_partial.rfsrc
surv_partial.rfsrc <- function(rforest, var_list, npts = 25, partial.type = "surv") { # nolint: object_name_linter
  .Deprecated(
    new     = "gg_partial_rfsrc",
    package = "ggRandomForests",
    msg     = paste0(
      "'surv_partial.rfsrc' is deprecated and will be removed in a future release.\n",
      "Use 'gg_partial_rfsrc()' instead, which returns a classed object with a\n",
      "dedicated plot() method."
    )
  )
  ###----------Partial dependency estimation, for each variable, at each time point ----
  surv.lst <- lapply(var_list, function(xvar) {
    ## extract the key variable. Use message() (suppressible) instead of
    ## cat() so the function plays nicely inside notebooks/Shiny/quarto.
    message("partial plot for: ", xvar)

    ## determine the partial plot data
    xv <- sort(unique(rforest$xvar[, xvar]))
    xv <- unique(xv[seq(1, length(xv), length = npts)])

    ## Get the partial.plot.data
    partial.dta    <- randomForestSRC::get.partial.plot.data(
      randomForestSRC::partial.rfsrc(
        rforest,
        partial.type = partial.type,
        partial.xvar = xvar,
        partial.values = xv,
        partial.time = rforest$time.interest
      )
    )

    list(name = xvar,
         dta = partial.dta)

  })
  return(surv.lst)
}

# Tests for surv_partial.rfsrc
#
# surv_partial.rfsrc() is deprecated in favour of gg_partial_rfsrc().
# All calls are wrapped in suppressWarnings() so the deprecation message does
# not produce noise in the test output.  The deprecation warning itself is
# verified in the first test_that block below.

# Survival formula helper (rfsrc requires Surv to be in local scope)
Surv <- survival::Surv # nolint: object_name_linter

test_that("surv_partial.rfsrc emits a deprecation warning", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  expect_warning(
    suppressMessages(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    ),
    regexp = "deprecated"
  )
})

test_that("surv_partial.rfsrc returns list with one element per variable", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
  )

  expect_type(result, "list")
  expect_length(result, 1)
  expect_equal(result[[1]]$name, "age")
})

test_that("surv_partial.rfsrc result element has name and dta fields", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
  )

  expect_named(result[[1]], c("name", "dta"))
  expect_true(!is.null(result[[1]]$dta))
})

test_that("surv_partial.rfsrc processes multiple variables", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = c("age", "karno"), partial.type = "mort")
  )

  expect_length(result, 2)
  names_out <- sapply(result, function(x) x$name)
  expect_true("age" %in% names_out)
  expect_true("karno" %in% names_out)
})

test_that("surv_partial.rfsrc works with surv partial.type", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "surv")
  )

  expect_type(result, "list")
  expect_length(result, 1)
  expect_equal(result[[1]]$name, "age")
})

test_that("surv_partial.rfsrc npts argument limits unique x values", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort", npts = 5)
  )

  expect_type(result, "list")
  expect_length(result, 1)
})

## ---- Shared fixture (built once, reused below) ----------------------------

local({
  skip_if_not_installed("randomForestSRC")
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  test_that("surv_partial.rfsrc dta element has x and yhat columns", {
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    )
    dta <- result[[1]]$dta
    expect_true(!is.null(dta))
    # get.partial.plot.data returns a list with $x (predictor values) and
    # $yhat (matrix of partial predictions, one column per time point)
    expect_true("x" %in% names(dta))
    expect_true("yhat" %in% names(dta))
  })

  test_that("surv_partial.rfsrc npts limits evaluation points", {
    npts_requested <- 5L
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort", npts = npts_requested)
    )
    dta <- result[[1]]$dta
    # The number of evaluation points should be <= npts_requested
    expect_true(length(dta$x) <= npts_requested)
  })

  test_that("surv_partial.rfsrc mort and surv partial.types return different yhat scales", {
    res_mort <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    )
    res_surv <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "surv")
    )

    yhat_mort <- res_mort[[1]]$dta$yhat
    yhat_surv <- res_surv[[1]]$dta$yhat

    # Mortality (years lost) and survival (probability) are on different scales;
    # survival probabilities are bounded [0, 1]; mortality values are unbounded
    if (is.matrix(yhat_surv)) {
      expect_true(all(yhat_surv >= 0 & yhat_surv <= 1 + 1e-8))
    }
    # The two types should not produce identical predictions
    expect_false(identical(yhat_mort, yhat_surv))
  })

  test_that("surv_partial.rfsrc verbose: emits a message with the variable name", {
    # v2.7.2: cat() -> message() so output is suppressible per CRAN cookbook.
    expect_message(
      suppressWarnings(
        surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
      ),
      regexp = "age"
    )
  })

  test_that("surv_partial.rfsrc errors on invalid variable name", {
    expect_error(
      suppressWarnings(
        surv_partial.rfsrc(v.obj, var_list = "nonexistent_var", partial.type = "mort")
      )
    )
  })

  test_that("surv_partial.rfsrc result names match requested var_list order", {
    vars <- c("karno", "age", "diagtime")
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = vars, partial.type = "mort")
    )
    names_out <- vapply(result, function(x) x$name, character(1L))
    expect_equal(names_out, vars)
  })
})

# Tests for surv_partial.rfsrc
#
# surv_partial.rfsrc() is deprecated in favour of gg_partial_rfsrc().
# All calls are wrapped in suppressWarnings() so the deprecation message does
# not produce noise in the test output.  The deprecation warning itself is
# verified in the first test_that block below.

# Survival formula helper (rfsrc requires Surv to be in local scope)
Surv <- survival::Surv # nolint: object_name_linter

test_that("surv_partial.rfsrc emits a deprecation warning", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  expect_warning(
    suppressMessages(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    ),
    regexp = "deprecated"
  )
})

test_that("surv_partial.rfsrc returns list with one element per variable", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
  )

  expect_type(result, "list")
  expect_length(result, 1)
  expect_equal(result[[1]]$name, "age")
})

test_that("surv_partial.rfsrc result element has name and dta fields", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
  )

  expect_named(result[[1]], c("name", "dta"))
  expect_true(!is.null(result[[1]]$dta))
})

test_that("surv_partial.rfsrc processes multiple variables", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = c("age", "karno"), partial.type = "mort")
  )

  expect_length(result, 2)
  names_out <- sapply(result, function(x) x$name)
  expect_true("age" %in% names_out)
  expect_true("karno" %in% names_out)
})

test_that("surv_partial.rfsrc works with surv partial.type", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "surv")
  )

  expect_type(result, "list")
  expect_length(result, 1)
  expect_equal(result[[1]]$name, "age")
})

test_that("surv_partial.rfsrc npts argument limits unique x values", {
  skip_if_not_installed("randomForestSRC")

  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  result <- suppressWarnings(
    surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort", npts = 5)
  )

  expect_type(result, "list")
  expect_length(result, 1)
})

## ---- Shared fixture (built once, reused below) ----------------------------

local({
  skip_if_not_installed("randomForestSRC")
  data(veteran, package = "randomForestSRC")
  set.seed(42)
  v.obj <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data   = veteran,
    ntree  = 50,
    nsplit = 5
  )

  test_that("surv_partial.rfsrc dta element has x and yhat columns", {
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    )
    dta <- result[[1]]$dta
    expect_true(!is.null(dta))
    # get.partial.plot.data returns a list with $x (predictor values) and
    # $yhat (matrix of partial predictions, one column per time point)
    expect_true("x" %in% names(dta))
    expect_true("yhat" %in% names(dta))
  })

  test_that("surv_partial.rfsrc npts limits evaluation points", {
    npts_requested <- 5L
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort", npts = npts_requested)
    )
    dta <- result[[1]]$dta
    # The number of evaluation points should be <= npts_requested
    expect_true(length(dta$x) <= npts_requested)
  })

  test_that("surv_partial.rfsrc mort and surv partial.types return different yhat scales", {
    res_mort <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
    )
    res_surv <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "surv")
    )

    yhat_mort <- res_mort[[1]]$dta$yhat
    yhat_surv <- res_surv[[1]]$dta$yhat

    # Mortality (years lost) and survival (probability) are on different scales;
    # survival probabilities are bounded [0, 1]; mortality values are unbounded
    if (is.matrix(yhat_surv)) {
      expect_true(all(yhat_surv >= 0 & yhat_surv <= 1 + 1e-8))
    }
    # The two types should not produce identical predictions
    expect_false(identical(yhat_mort, yhat_surv))
  })

  test_that("surv_partial.rfsrc runs quietly (no per-variable message)", {
    # v3.0.0: the chatty per-variable message() was removed (CRAN audit).
    # Only the deprecation warning remains; suppress it and assert silence.
    expect_no_message(
      suppressWarnings(
        surv_partial.rfsrc(v.obj, var_list = "age", partial.type = "mort")
      )
    )
  })

  test_that("surv_partial.rfsrc errors on invalid variable name", {
    expect_error(
      suppressWarnings(
        surv_partial.rfsrc(v.obj, var_list = "nonexistent_var", partial.type = "mort")
      )
    )
  })

  test_that("surv_partial.rfsrc result names match requested var_list order", {
    vars <- c("karno", "age", "diagtime")
    result <- suppressWarnings(
      surv_partial.rfsrc(v.obj, var_list = vars, partial.type = "mort")
    )
    names_out <- vapply(result, function(x) x$name, character(1L))
    expect_equal(names_out, vars)
  })
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
###############################################################################
# Package documentation
###############################################################################
#' @title ggRandomForests: Visually Exploring Random Forests
#'
#' @description \code{ggRandomForests} is a utility package for
#' \code{randomForestSRC} (Ishwaran and Kogalur) for survival,
#' regression and classification forests and uses the \code{ggplot2}
#' (Wickham 2009) package for plotting results. \code{ggRandomForests} is
#' structured to extract data objects from the random forest and provides S3
#' functions for printing and plotting these objects.
#' Requires \code{randomForestSRC} >= 3.4.0.
#'
#' The \code{randomForestSRC} package provides a unified treatment of
#' Breiman's (2001) random forests for a variety of data settings. Regression
#' and classification forests are grown when the response is numeric or
#' categorical (factor) while survival and competing risk forests
#' (Ishwaran et al. 2008, 2012) are grown for right-censored survival data.
#'
#' Many of the figures created by the \code{ggRandomForests} package are also
#' available directly from within the \code{randomForestSRC} package. However,
#' \code{ggRandomForests} offers the following advantages:
#' \itemize{
#' \item Separation of data and figures: \code{ggRandomForest} contains
#' functions that operate on either the \code{\link[randomForestSRC]{rfsrc}}
#' forest object directly, or on the output from \code{randomForestSRC} post
#' processing functions (i.e. \code{plot.variable}) to generate intermediate
#' \code{ggRandomForests}
#' data objects. S3 functions are provide to further process these objects and
#' plot results using the \code{ggplot2} graphics package. Alternatively,
#' users can use these data objects for additional custom plotting or
#' analysis operations.
#'
#' \item Each data object/figure is a single, self contained object. This
#' allows simple modification and manipulation of the data or \code{ggplot2}
#' objects to meet users specific needs and requirements.
#'
#' \item The use of \code{ggplot2} for plotting. We chose to use the
#' \code{ggplot2} package for our figures to allow users flexibility in
#' modifying the figures to their liking. Each S3 plot function returns either
#' a single \code{ggplot2} object, or a \code{list} of \code{ggplot2} objects,
#' allowing users to use additional \code{ggplot2} functions or themes to
#' modify and customize the figures to their liking.
#' }
#'
#' The \code{ggRandomForests} package contains the following data functions:
#' \itemize{
#' \item \code{\link{gg_rfsrc}}: randomForest[SRC] predictions.
#' \item \code{\link{gg_error}}: randomForest[SRC] convergence rate based on
#' the OOB error rate.
#' \item \code{\link{gg_roc}}: ROC curves for randomForest classification
#' models.
#' \item \code{\link{gg_vimp}}: Variable Importance ranking for variable
#' selection.
#' (Ishwaran et.al. 2010).
#' \item \code{\link{gg_variable}}: Marginal variable dependence.
#'
#' \item \code{\link{gg_survival}}: Kaplan-Meier/Nelson-Aalen hazard analysis.
#' }
#'
#' Each of these data functions has an associated S3 plot function that
#' returns \code{ggplot2} objects, either individually or as a list, which can
#' be further customized using standard \code{ggplot2} commands.
#'
#' @references
#' Breiman, L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. R News
#' 7(2), 25--31.
#'
#' Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random
#' survival forests. Ann. Appl. Statist. 2(3), 841--860.
#'
#' Ishwaran, H., U. B. Kogalur, E. Z. Gorodeski, A. J. Minn, and M. S. Lauer
#' (2010). High-dimensional variable selection for survival data. J. Amer.
#' Statist. Assoc. 105, 205-217.
#'
#' Ishwaran, H. (2007). Variable importance in binary regression trees and
#' forests. Electronic J. Statist., 1, 519-537.
#'
#' Wickham, H. ggplot2: elegant graphics for data analysis. Springer New York,
#' 2009.
#'
#' @name ggRandomForests-package
#'
################
NULL

####**********************************************************************
####  print.gg_* methods.
####
####  All print methods follow the same shape: emit a one-line header
####  (class label + provenance) and return the object invisibly. They do
####  NOT print rows. gg_* objects inherit data.frame (or list), so
####  head() and as.data.frame() remain available for inspecting contents.
####**********************************************************************

#' Print methods for gg_* data objects
#'
#' Each \code{print.gg_*()} method prints a one-line header: the class label
#' and, where the forest recorded it, provenance (source package, family,
#' ntree, n). It returns the object invisibly, so \code{print()} sits cleanly
#' in a pipe.
#'
#' To see the rows themselves, use \code{head()}; for per-class diagnostics,
#' use \code{\link{summary.gg}}.
#'
#' @param x A \code{gg_*} data object.
#' @param ... Not currently used.
#'
#' @return The object \code{x}, invisibly.
#'
#' @seealso \code{\link{summary.gg}}, \code{\link{autoplot.gg}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' print(gg_error(rf))
#' print(gg_vimp(rf))
#'
#' @name print.gg
NULL

#' @rdname print.gg
#' @export
print.gg_error <- function(x, ...) {
  cat(.gg_header(x, "gg_error"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_vimp <- function(x, ...) {
  cat(.gg_header(x, "gg_vimp"),
      sprintf("  |  variables: %d", nrow(x)),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_rfsrc <- function(x, ...) {
  cat(.gg_header(x, "gg_rfsrc"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_variable <- function(x, ...) {
  cat(.gg_header(x, "gg_variable"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partial <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  cat(.gg_header(x, "gg_partial"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partial_rfsrc <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  cat(.gg_header(x, "gg_partial_rfsrc"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_partialpro <- function(x, ...) {
  ## Deprecated-class shim: re-dispatch to print.gg_partial_varpro.
  class(x) <- c("gg_partial_varpro", setdiff(class(x), "gg_partialpro"))
  print.gg_partial_varpro(x, ...)
}

#' @rdname print.gg
#' @export
print.gg_partial_varpro <- function(x, ...) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  prov  <- attr(x, "provenance")
  scale <- if (!is.null(prov)) prov$scale %||% NA_character_ else NA_character_
  cat(.gg_header(x, "gg_partial_varpro"),
      sprintf("  |  continuous: %d, categorical: %d", nvar_cont, nvar_cat),
      if (!is.na(scale)) sprintf("  |  scale: %s", scale) else "",
      "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_roc <- function(x, ...) {
  cat(.gg_header(x, "gg_roc"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_survival <- function(x, ...) {
  cat(.gg_header(x, "gg_survival"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_brier <- function(x, ...) {
  crps <- attr(x, "crps_integrated")
  suffix <- if (!is.null(crps)) {
    sprintf("  |  integrated CRPS: %.4g", crps)
  } else {
    ""
  }
  cat(.gg_header(x, "gg_brier"), suffix, "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_udependent <- function(x, ...) {
  prov <- attr(x, "provenance")
  n    <- if (!is.null(prov)) prov$n else "?"
  p    <- if (!is.null(prov)) length(prov$xvar.names) else "?"
  thr  <- if (!is.null(prov)) prov$threshold else "?"
  cat(sprintf("<gg_udependent>  n=%s  p=%s  threshold=%s\n", n, p, thr))
  cat(sprintf(
    "  Edges: %d  Nodes in graph: %d  Selected: %d/%d\n",
    nrow(x$edges), nrow(x$nodes),
    sum(x$nodes$selected, na.rm = TRUE), nrow(x$nodes)
  ))
  invisible(x)
}

#' @rdname print.gg
#' @export
print.summary.gg_udependent <- function(x, ...) {
  ## summary.gg_udependent returns a summary.gg skeleton; delegate to
  ## print.summary.gg which renders header + body lines.
  NextMethod()
}

#' @rdname print.gg
#' @export
print.gg_varpro <- function(x, ...) {
  prov     <- attr(x, "provenance")
  cutoff   <- if (!is.null(prov)) prov$cutoff   %||% 0.79 else 0.79
  faithful <- if (!is.null(prov)) prov$faithful  %||% FALSE else FALSE
  family   <- if (!is.null(prov)) prov$family    %||% NA_character_ else NA_character_
  n_total  <- nrow(x$imp)
  n_sel    <- sum(x$imp$selected, na.rm = TRUE)
  cat(.gg_header(x, "gg_varpro"),
      sprintf("  |  family: %s", family),
      sprintf("  |  cutoff: %.2g", cutoff),
      sprintf("  |  faithful: %s", faithful),
      "\n",
      sprintf("  %d of %d variables selected (z > %.2g)\n",
              n_sel, n_total, cutoff),
      sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_isopro <- function(x, ...) {
  cat(.gg_header(x, "gg_isopro"), "\n", sep = "")
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_beta_varpro <- function(x, ...) {
  prov           <- attr(x, "provenance")
  family         <- if (!is.null(prov)) prov$family %||% NA_character_ else NA_character_
  precomputed    <- isTRUE(if (!is.null(prov)) prov$precomputed else FALSE)
  n_total        <- if (!is.null(prov)) prov$n_rules_total  %||% NA_integer_ else NA_integer_
  n_nonzero      <- if (!is.null(prov)) prov$n_rules_nonzero %||% NA_integer_ else NA_integer_
  n_sel          <- sum(x$selected, na.rm = TRUE)

  if (identical(family, "class") && "class" %in% names(x)) {
    n_classes   <- length(unique(x$class))
    which_class <- if (!is.null(prov)) prov$which_class else NULL
    view_str <- if (is.null(which_class)) "faceted" else sprintf("which_class: %s", which_class)
    cat(.gg_header(x, "gg_beta_varpro"),
        sprintf("  |  n_classes: %d  |  view: %s", n_classes, view_str),
        sprintf("  |  precomputed: %s", precomputed),
        "\n",
        sprintf("  %d of %d (variable, class) pairs selected; %d / %d rules with non-zero beta\n",
                n_sel, nrow(x), n_nonzero, n_total),
        sep = "")
  } else {
    cutoff         <- if (!is.null(prov)) prov$cutoff %||% NA_real_ else NA_real_
    cutoff_val     <- if (length(cutoff) >= 1L) cutoff[[1]] else NA_real_
    cutoff_default <- isTRUE(if (!is.null(prov)) prov$cutoff_default else FALSE)
    cat(.gg_header(x, "gg_beta_varpro"),
        sprintf("  |  cutoff: %.4g%s", cutoff_val,
                if (cutoff_default) " (default)" else ""),
        sprintf("  |  precomputed: %s", precomputed),
        "\n",
        sprintf("  %d of %d variables selected; %d / %d rules with non-zero beta\n",
                n_sel, nrow(x), n_nonzero, n_total),
        sep = "")
  }
  invisible(x)
}

#' @export
print.summary.gg_beta_varpro <- function(x, ...) {
  if (is.list(unclass(x)) && !is.numeric(unclass(x))) {
    # Classification: list of per-class summaries
    for (cls in names(x)) {
      cat(sprintf("Class '%s' -- mean |beta| per variable (descending):\n", cls))
      print(unclass(x[[cls]]))
      cat("\nRule counts:\n")
      print(attr(x[[cls]], "n_rules"))
      cat("\n")
    }
  } else {
    cat("Mean |beta| per variable (descending):\n")
    print(unclass(x))
    cat("\nRule counts:\n")
    print(attr(x, "n_rules"))
  }
  invisible(x)
}

#' @rdname print.gg
#' @export
print.gg_ivarpro <- function(x, ...) {
  prov        <- attr(x, "provenance")
  precomputed <- isTRUE(if (!is.null(prov)) prov$precomputed else FALSE)
  n_obs       <- if (!is.null(prov)) prov$n_obs %||% NA_integer_ else NA_integer_
  n_var       <- if (!is.null(prov)) prov$n_var %||% NA_integer_ else NA_integer_
  which_obs   <- if (!is.null(prov)) prov$which_obs else NULL
  which_cls   <- if (!is.null(prov)) prov$which_class else NULL
  has_class   <- "class" %in% names(x)

  view <- if (!is.null(which_obs)) sprintf("obs %d", which_obs) else "aggregate"
  cls_part <- if (has_class) {
    n_cls <- length(unique(x$class))
    if (!is.null(which_cls)) {
      sprintf("  |  class: %s", which_cls)
    } else {
      sprintf("  |  %d classes (faceted)", n_cls)
    }
  } else {
    ""
  }

  cat(.gg_header(x, "gg_ivarpro"),
      sprintf("  |  view: %s", view),
      cls_part,
      sprintf("  |  precomputed: %s", precomputed),
      "\n",
      sprintf("  %d (obs x variable) cells; %d unique obs across %d variables\n",
              nrow(x), n_obs, n_var),
      sep = "")
  invisible(x)
}

#' @export
print.summary.gg_ivarpro <- function(x, ...) {
  if (is.list(unclass(x)) && !is.numeric(unclass(x))) {
    for (cls in names(x)) {
      cat(sprintf("Class '%s' - mean |local_imp| per variable (descending):\n", cls))
      print(unclass(x[[cls]]))
      cat("\nObservation counts:\n")
      print(attr(x[[cls]], "n_obs"))
      cat("\n")
    }
  } else {
    cat("Mean |local_imp| per variable (descending):\n")
    print(unclass(x))
    cat("\nObservation counts:\n")
    print(attr(x, "n_obs"))
  }
  invisible(x)
}

##=============================================================================
#' Partial dependence data from an rfsrc model
#'
#' Computes partial dependence for one or more predictors by calling
#' \code{\link[randomForestSRC]{partial.rfsrc}} internally, then splits the
#' results into separate data frames for continuous and categorical variables.
#' Unlike \code{\link{gg_partial}}, no separate \code{plot.variable} call is
#' required — supply the fitted \code{rfsrc} object directly.
#'
#' @section Survival forests and \code{partial.time}:
#' \code{\link[randomForestSRC]{partial.rfsrc}} requires that every value in
#' \code{partial.time} be an exact member of the model's \code{time.interest}
#' vector (the unique observed event times stored in the fitted object).
#' Passing arbitrary time values — even plausible ones such as \code{c(1, 3)}
#' for a study measured in years — causes a C-level prediction error inside
#' \code{partial.rfsrc}.
#'
#' \code{gg_partial_rfsrc} handles this automatically: every element of
#' \code{partial.time} is silently snapped to its nearest \code{time.interest}
#' value before the call is made.  To target a specific follow-up horizon,
#' find the closest grid point yourself and pass it explicitly:
#'
#' \preformatted{
#' ti  <- rf_model$time.interest
#' t1  <- ti[which.min(abs(ti - 1))]   # nearest to 1 year
#' pd  <- gg_partial_rfsrc(rf_model, xvar.names = "x", partial.time = t1)
#' }
#'
#' @section Logical predictor columns:
#' \code{\link[randomForestSRC]{partial.rfsrc}} does not handle
#' \code{logical} predictor columns correctly in survival forests
#' (randomForestSRC <= 3.5.1).  If your training data contains binary 0/1
#' columns, convert them to \code{\link{factor}} rather than \code{logical}
#' before fitting the model.
#'
#' @param rf_model A fitted \code{\link[randomForestSRC]{rfsrc}} object.
#' @param xvar.names Character vector of predictor names for which partial
#'   dependence should be computed. Must be a subset of \code{rf_model$xvar.names}.
#' @param xvar2.name Optional single character name of a grouping variable in
#'   \code{newx}. When supplied, partial dependence is computed separately for
#'   each unique level of this variable and a \code{grp} column is appended.
#' @param newx Optional \code{data.frame} of predictor values to evaluate
#'   partial effects at. Defaults to the training data stored in
#'   \code{rf_model$xvar}. All column names must match \code{rf_model$xvar.names}.
#' @param partial.time Numeric vector of desired time points for survival
#'   forests (ignored for regression/classification).  Values are automatically
#'   snapped to the nearest entry in \code{rf_model$time.interest} — see the
#'   \strong{Survival forests} section below.  When \code{NULL} (default),
#'   three quartile points of \code{time.interest} are used.
#' @param partial.type Character; type of predicted value for survival
#'   forests, passed through to \code{\link[randomForestSRC]{partial.rfsrc}}.
#'   One of \code{"surv"} (default), \code{"chf"}, or \code{"mort"}. Ignored
#'   for non-survival forests. \code{partial.rfsrc()} requires a non-\code{NULL}
#'   value for survival families; supplying it here avoids a cryptic
#'   \dQuote{argument is of length zero} error from the underlying C code.
#' @param cat_limit Variables with fewer than \code{cat_limit} unique values in
#'   \code{newx} are treated as categorical; all others are continuous.
#'   Defaults to 10.
#' @param n_eval Number of evaluation points for continuous variables. Instead
#'   of passing all observed values (which can be slow, especially for survival
#'   forests), continuous predictors are evaluated on a quantile grid of this
#'   many points. Categorical variables always use all unique levels.
#'   Defaults to 25.
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{continuous}{A \code{data.frame} with columns \code{x} (numeric),
#'       \code{yhat}, \code{name} (variable name), and optionally \code{grp}
#'       (the level of \code{xvar2.name}) and \code{time} (survival forests
#'       only) for all continuous predictors.}
#'     \item{categorical}{A \code{data.frame} with the same columns but
#'       \code{x} kept as character, for low-cardinality predictors.}
#'   }
#'
#' @seealso \code{\link{gg_partial}}, \code{\link[randomForestSRC]{partial.rfsrc}},
#'   \code{\link[randomForestSRC]{get.partial.plot.data}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ##
#' ## regression
#' ##
#' ## ------------------------------------------------------------
#'
#' airq.obj <- rfsrc(Ozone ~ ., data = airquality)
#'
#' ## partial effect for wind
#' prt_dta <- gg_partial_rfsrc(airq.obj,
#'                        xvar.names = c("Wind"))
#'
#' @importFrom dplyr mutate filter select all_of
#' @export
gg_partial_rfsrc <- function(rf_model,
                             xvar.names = NULL,
                             xvar2.name = NULL,
                             newx = NULL,
                             partial.time = NULL,
                             partial.type = c("surv", "chf", "mort"),
                             cat_limit = 10,
                             n_eval = 25) {
  if (is.null(newx)) {
    newx <- rf_model$xvar
  }

  if (sum(colnames(newx) %in% rf_model$xvar.names) != ncol(newx)) {
    stop("newx must be a dataframe with the same columns used to train the rfsrc object")
  }

  if (!is.null(xvar.names)) {
    if (sum(xvar.names %in% colnames(newx)) != length(xvar.names)) {
      stop("xvar.names contains column names not found in the rfsrc object")
    }
  }

  v         <- validate_partial_args(n_eval, cat_limit)
  n_eval    <- v$n_eval
  cat_limit <- v$cat_limit

  is_surv <- !is.null(rf_model$family) && grepl("surv", rf_model$family)
  if (is_surv) {
    partial.time <- snap_partial_time(rf_model, partial.time)
    # partial.rfsrc() requires a non-NULL partial.type for survival forests;
    # NULL triggers a zero-length comparison inside the C code.
    partial.type <- match.arg(partial.type)
  } else {
    partial.type <- NULL
  }

  if (is.null(xvar2.name)) {
    pdta <- partial_no_group(xvar.names, newx, rf_model,
                             cat_limit, n_eval, is_surv, partial.time,
                             partial.type)
  } else {
    pdta <- partial_with_group(xvar.names, xvar2.name, newx, rf_model,
                               cat_limit, n_eval, is_surv, partial.time,
                               partial.type)
  }

  result <- split_partial_result(do.call("rbind", pdta))
  # Carry partial.type so plot.gg_partial_rfsrc() can pick the correct
  # y-axis label (Survival / CHF / Mortality).
  attr(result, "partial.type") <- partial.type
  result <- .set_provenance(result, rf_model)
  result
}

## ---- unexported helpers -------------------------------------------------------

## Check that x is a single non-NA numeric >= min_val; return as integer.
validate_scalar_int <- function(x, name, min_val = 2L) {
  ok <- is.numeric(x) && length(x) == 1L && !is.na(x) && x >= min_val
  if (!ok) {
    stop(sprintf("'%s' must be a single integer >= %d.", name, min_val),
         call. = FALSE)
  }
  as.integer(x)
}

## Validate and coerce n_eval / cat_limit.
validate_partial_args <- function(n_eval, cat_limit) {
  list(
    n_eval    = validate_scalar_int(n_eval,    "n_eval",    2L),
    cat_limit = validate_scalar_int(cat_limit, "cat_limit", 2L)
  )
}

## Snap requested time points to the nearest values in time.interest.
snap_partial_time <- function(rf_model, partial.time) {
  ti <- rf_model$time.interest
  if (is.null(partial.time)) {
    partial.time <- quantile(ti, probs = c(0.25, 0.5, 0.75), names = FALSE)
  }
  snapped <- sapply(partial.time,
                    function(t) ti[which.min(abs(ti - t))],
                    USE.NAMES = FALSE)
  unique(snapped)
}

## Build the evaluation grid (xval vector + categorical flag) for one variable.
make_eval_grid <- function(xname, newx, cat_limit, n_eval) {
  # Use `[[` to preserve the column's class (factor, character, numeric, …).
  # unlist(dplyr::select(...)) would coerce factors to integer codes, breaking
  # both the cat_limit check and the partial.values passed to partial.rfsrc().
  xval <- newx[[xname]]
  xval <- xval[!is.na(xval)]
  if (length(xval) == 0L) {
    warning(sprintf(
      "Variable '%s' contains only NA values in 'newx'; skipping partial dependence.",
      xname
    ), call. = FALSE)
    return(NULL)
  }
  gr <- is.factor(xval) || is.character(xval) || length(unique(xval)) < cat_limit
  if (!gr && length(unique(xval)) > n_eval) {
    xval <- quantile_pts(xval, groups = n_eval)
  } else if (is.factor(xval)) {
    xval <- levels(droplevels(xval))   # preserve factor ordering; drop unused levels
  } else {
    xval <- sort(unique(xval))
  }
  list(xval = xval, categorical = gr)
}

## Thin wrapper around partial.rfsrc that builds the argument list.
call_partial_rfsrc <- function(rf_model, xname, xval,
                                is_surv, partial.time, partial.type,
                                xvar2.name = NULL, x2val = NULL) {
  args <- list(
    object         = rf_model,
    partial.xvar   = xname,
    partial.values = xval
  )
  if (!is.null(xvar2.name)) {
    args$partial.xvar2   <- xvar2.name
    args$partial.values2 <- x2val
  }
  if (is_surv) {
    args$partial.time <- partial.time
    args$partial.type <- partial.type
  }
  do.call(randomForestSRC::partial.rfsrc, args)
}

## Process a single predictor variable and return a tidy data.frame (or NULL).
partial_one_var <- function(xname, newx, rf_model,
                            cat_limit, n_eval, is_surv, partial.time,
                            partial.type,
                            xvar2.name = NULL, x2val = NULL) {
  eg <- make_eval_grid(xname, newx, cat_limit, n_eval)
  if (is.null(eg)) return(NULL)
  xval <- eg$xval
  gr   <- eg$categorical
  partial.obj <- call_partial_rfsrc(rf_model, xname, xval,
                                     is_surv, partial.time, partial.type,
                                     xvar2.name, x2val)
  pout    <- randomForestSRC::get.partial.plot.data(partial.obj, granule = gr)
  # Survival forests with >1 partial.time return yhat as an
  # [length(partial.values) x length(partial.time)] matrix; expand to long form
  # so each (x, time) pair is its own row. For non-survival or single-time
  # cases yhat is already a vector of length(partial.values).
  if (is.matrix(pout$yhat)) {
    pt <- if (!is.null(pout$partial.time)) pout$partial.time else seq_len(ncol(pout$yhat))
    out_dta <- data.frame(
      x    = rep(pout$x, times = length(pt)),
      yhat = as.numeric(pout$yhat),
      time = rep(pt, each = length(pout$x))
    )
  } else {
    out_dta <- data.frame(x = pout$x, yhat = pout$yhat)
    if (!is.null(pout$partial.time)) {
      out_dta$time <- pout$partial.time
    }
  }
  out_dta$name <- xname
  out_dta$type <- c("continuous", "categorical")[gr + 1L]
  out_dta
}

## Compute partial dependence across xvar.names (no grouping variable).
partial_no_group <- function(xvar.names, newx, rf_model,
                             cat_limit, n_eval, is_surv, partial.time,
                             partial.type) {
  pdta <- lapply(xvar.names, partial_one_var,
                 newx = newx, rf_model = rf_model,
                 cat_limit = cat_limit, n_eval = n_eval,
                 is_surv = is_surv, partial.time = partial.time,
                 partial.type = partial.type)
  Filter(Negate(is.null), pdta)
}

## Compute partial dependence across xvar.names for each level of xvar2.name.
partial_with_group <- function(xvar.names, xvar2.name, newx, rf_model,
                               cat_limit, n_eval, is_surv, partial.time,
                               partial.type) {
  xv2 <- unique(newx[[xvar2.name]])
  xv2 <- xv2[!is.na(xv2)]
  if (length(xv2) == 0L) {
    stop(sprintf(
      "Grouping variable '%s' contains only NA values in 'newx'; cannot compute surface partial dependence.",
      xvar2.name
    ), call. = FALSE)
  }
  pdta <- lapply(xv2, function(x2val) {
    p1dta <- lapply(xvar.names, partial_one_var,
                    newx = newx, rf_model = rf_model,
                    cat_limit = cat_limit, n_eval = n_eval,
                    is_surv = is_surv, partial.time = partial.time,
                    partial.type = partial.type,
                    xvar2.name = xvar2.name, x2val = x2val)
    p1dta <- Filter(Negate(is.null), p1dta)
    if (length(p1dta) == 0L) return(NULL)
    p1dta        <- do.call("rbind", p1dta)
    p1dta$grp    <- x2val
    p1dta
  })
  Filter(Negate(is.null), pdta)
}

## Split the combined data.frame into continuous / categorical and stamp class.
split_partial_result <- function(pdta) {
  cont_idx        <- pdta$type == "continuous"
  continuous      <- pdta[cont_idx, , drop = FALSE]
  continuous$x    <- as.numeric(continuous$x)
  continuous$type <- NULL
  categorical      <- pdta[!cont_idx, , drop = FALSE]
  categorical$type <- NULL
  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- "gg_partial_rfsrc"
  result
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Variable Importance (VIMP) data object
#'
#' \code{gg_vimp} Extracts the variable importance (VIMP) information from a
#' \code{\link[randomForestSRC]{rfsrc}} or \code{\link[randomForest]{randomForest}}
#' object and reshapes it into a tidy data set.
#'
#' @param object A \code{\link[randomForestSRC]{rfsrc}} object, the output from
#' \code{\link[randomForestSRC]{vimp}}, or a fitted
#' \code{\link[randomForest]{randomForest}}.
#' @param nvar argument to control the number of variables included in the
#' output.
#' @param ... arguments passed to the \code{\link[randomForestSRC]{vimp.rfsrc}}
#' function if the \code{\link[randomForestSRC]{rfsrc}} object does not contain
#' importance information.
#'
#' @details
#' \code{gg_vimp()} shows \strong{permutation (Breiman-Cutler) variable
#' importance}: the forest permutes a variable's observed values across the
#' out-of-bag (OOB) cases, runs those perturbed cases down the already-grown
#' trees, and measures how much the OOB prediction error climbs.  That
#' perturbation is synthetic (the variable's link to the response is broken
#' on purpose) so a large increase means the variable was carrying genuine
#' signal; near-zero or negative values mean it added noise or nothing at all.
#'
#' \code{\link{gg_varpro}()} takes the opposite route, comparing local
#' estimators on real observed data through varPro's release rules, with no
#' permutation and no synthetic features.  The two approaches answer "which
#' variables matter?" by opposite mechanisms, so a variable can rank
#' differently under each, and that disagreement is itself informative: it
#' often signals interaction structure or non-monotone effects that one
#' mechanism surfaces and the other obscures.
#'
#' For survival forests, VIMP is measured against the ensemble cumulative
#' hazard function (CHF); the error metric is one minus the concordance index
#' (C-statistic).  Variables with non-positive VIMP are flagged in the
#' \code{positive} column and colored differently by
#' \code{\link{plot.gg_vimp}}.
#'
#' @return \code{gg_vimp} object. A \code{data.frame} of VIMP measures, in rank
#'   order, optionally containing class-specific scores and a relative importance
#'   column. When \code{randomForest} objects lack stored importance values a
#'   warning is issued and \code{NA} placeholders are returned so plots remain
#'   reproducible.
#'
#' @seealso \code{\link{plot.gg_vimp}} \code{\link[randomForestSRC]{rfsrc}}
#' @seealso \code{\link[randomForestSRC]{vimp}} \code{\link{gg_varpro}}
#'
#' @references
#' Ishwaran H. (2007). Variable importance in binary regression trees and
#' forests, \emph{Electronic J. Statist.}, 1:519-537.
#'
#' @importFrom randomForestSRC vimp
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ .,
#'   data = iris,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_iris)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## regression example
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., airquality,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_airq)
#' plot(gg_dta)
#'
#'
#' ## -------- Boston data
#' data(Boston, package = "MASS")
#' rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_boston)
#' plot(gg_dta)
#'
#' ## -------- Boston data
#' rf_boston <- randomForest::randomForest(medv ~ ., Boston)
#' gg_dta <- gg_vimp(rf_boston)
#' plot(gg_dta)
#'
#'
#' ## -------- mtcars data
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ .,
#'   data = mtcars,
#'   importance = TRUE
#' )
#' gg_dta <- gg_vimp(rfsrc_mtcars)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## survival example
#' ## ------------------------------------------------------------
#'
#' ## -------- veteran data
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
#'   data = veteran,
#'   ntree = 100,
#'   importance = TRUE
#' )
#'
#' gg_dta <- gg_vimp(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## -------- pbc data
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC", )
#' # For whatever reason, the age variable is in days...
#' # makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'   if (!is.factor(pbc[, ind])) {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   } else {
#'     if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'       if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'       if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'         pbc[, ind] <- as.logical(pbc[, ind])
#'       }
#'     }
#'   }
#'   if (!is.logical(pbc[, ind]) &
#'     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'     pbc[, ind] <- factor(pbc[, ind])
#'   }
#' }
#' # Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' # ========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'   dta_train,
#'   nsplit = 10,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   save.memory = TRUE
#' )
#'
#' gg_dta <- gg_vimp(rfsrc_pbc)
#' plot(gg_dta)
#'
#' # Restrict to only the top 10.
#' gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10)
#' plot(gg_dta)
#'
#' @aliases gg_vimp gg_vimp.rfsrc gg_vimp.randomForest
#' @aliases gg_vimp.randomForest.formula
#' @export
gg_vimp <- function(object, nvar, ...) {
  UseMethod("gg_vimp", object)
}
#' @export
gg_vimp.rfsrc <- function(object, nvar, ...) {
  # Validate that the object is an rfsrc grow or predict result.
  if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &&
    sum(inherits(object, c("rfsrc", "predict"), TRUE) == c(1, 2)) != 2) {
    stop("This function only works for objects of class `(rfsrc, grow)' or
         '(rfsrc, predict)'.")
  }

  # If the forest was grown without importance = TRUE, compute VIMP now via
  # a separate vimp() call (slower, but always safe).
  if (is.null(object$importance)) {
    warning("rfsrc object does not contain VIMP information. Calculating...")
    gg_dta <-
      data.frame(sort(randomForestSRC::vimp(object)$importance,
        decreasing = TRUE
      ))
  } else {
    gg_dta <- data.frame(object$importance)
  }

  # Single-outcome forests: $importance is a named vector, one-column df.
  # Rename the column and add a "vars" column with the variable names.
  if (ncol(gg_dta) == 1) {
    colnames(gg_dta) <- "VIMP"
    gg_dta$vars <- rownames(gg_dta)
    gg_dta <- gg_dta[order(gg_dta$VIMP, decreasing = TRUE), ]
  }

  # Clamp nvar to the number of available variables.
  if (missing(nvar)) {
    nvar <- nrow(gg_dta)
  }
  if (nvar > nrow(gg_dta)) {
    nvar <- nrow(gg_dta)
  }

  # Multi-class forests: $importance is a matrix (vars × classes).
  # Pivot to long form so each row is one (variable, class) combination.
  if (ncol(gg_dta) > 1) {
    arg_set <- list(...)

    if (!is.null(arg_set$which.outcome)) {
      # Caller requested importance for one specific class only.
      if (!is.numeric(arg_set$which.outcome)) {
        # Look up by class name (column name).
        if (arg_set$which.outcome %in% colnames(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste(
              "which.outcome naming is incorrect.",
              arg_set$which.outcome,
              "\nis not in",
              colnames(gg_dta)
            )
          )
        }
      } else {
        # Look up by integer index.
        # which.outcome = 0  gives overall (across-class) importance, column 1
        # which.outcome = k  gives importance for class k, column k+1
        if (!is.numeric(arg_set$which.outcome) || arg_set$which.outcome < 0) {
          stop("which.outcome must be a non-negative integer or a class name.")
        }
        if (arg_set$which.outcome < ncol(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome + 1],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome + 1],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste0(
              "which.outcome (", arg_set$which.outcome, ") is out of range. ",
              "Valid values are 0 (overall) to ", ncol(gg_dta) - 1,
              " (number of classes)."
            )
          )
        }
      }
      gg_dta <- gg_v
    } else {
      # No specific class requested: attach variable names and pivot.
      gg_dta$vars <- rownames(gg_dta)
    }

    gg_dta <- gg_dta[seq_len(nvar), ]
    pivot_cols <-
      colnames(gg_dta)[-which(colnames(gg_dta) == "vars")]
    # Pivot from wide (one column per class) to long (one row per class-var pair).
    gg_dta <- tidyr::pivot_longer(
      gg_dta,
      tidyr::all_of(pivot_cols),
      names_to = "set",
      values_to = "vimp"
    )
    gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    gg_dta$vars <- factor(gg_dta$vars)
  } else {
    # Single-outcome: compute relative VIMP (each value as a fraction of the
    # top-ranked variable's importance).
    cnms <- colnames(gg_dta)
    gg_dta <- cbind(gg_dta, gg_dta / gg_dta[1, 1])
    colnames(gg_dta) <- c(cnms, "rel_vimp")
    # Patch any NA entries in the vars column that slipped through.
    gg_dta$vars[which(is.na(gg_dta$vars))] <-
      rownames(gg_dta)[which(is.na(gg_dta$vars))]

    gg_dta <- gg_dta[seq_len(nvar), ]
  }

  # Convert vars to an ordered factor (reversed so the most important variable
  # plots at the top of a horizontal bar chart after coord_flip).
  gg_dta$vars <-
    factor(gg_dta$vars, levels = rev(unique(gg_dta$vars)))

  # Flag variables with non-positive VIMP so the plot can colour them
  # differently to indicate they do not improve (or actively hurt) predictions.
  # Use the actual VIMP column name: "vimp" after a multi-outcome pivot,
  # "VIMP" (uppercase) for single-outcome fits.
  vimp_col <- intersect(c("vimp", "VIMP"), colnames(gg_dta))[1]
  gg_dta$positive <- TRUE
  if (!is.na(vimp_col)) {
    gg_dta$positive[which(gg_dta[[vimp_col]] <= 0)] <- FALSE
  }

  class(gg_dta) <- c("gg_vimp", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_vimp.randomForest <- function(object, nvar, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForest package."
      )
    )
  }

  ### set importance to NA if it is NULL
  if (is.null(object$importance)) {
    importance_est <- tryCatch(
      randomForest::importance(object, scale = FALSE),
      error = function(e) NULL
    )
    if (is.null(importance_est)) {
      vars <- object$forest$xvar.names
      if (is.null(vars)) {
        training_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
        if (!is.null(training_info)) {
          vars <- setdiff(colnames(training_info$model_frame), training_info$response_name)
        }
      }
      if (is.null(vars)) {
        stop(
          "Variable importance is unavailable for this randomForest object and ",
          "training predictors cannot be recovered."
        )
      }
      warning(
        "randomForest object does not contain importance information. Returning NA values."
      )
      placeholder <- data.frame(
        vimp = rep(NA_real_, length(vars)),
        vars = vars,
        rel_vimp = NA_real_
      )
      placeholder$vars <- factor(placeholder$vars, levels = rev(unique(placeholder$vars)))
      placeholder$positive <- FALSE
      class(placeholder) <- c("gg_vimp", class(placeholder))
      return(placeholder)
    }
    gg_dta <- data.frame(importance_est)
  } else {
    gg_dta <- data.frame(object$importance)
  }
  if (ncol(gg_dta) < 3) {
    gg_dta$vars <- rownames(gg_dta)
    colnames(gg_dta)[which(colnames(gg_dta) == "X.IncMSE")] <-
      "vimp"
    if ("vimp" %in% colnames(gg_dta)) {
      gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    } else {
      cn <- colnames(gg_dta)[1]
      gg_dta <-
        gg_dta[order(gg_dta[, cn], decreasing = TRUE), ]
      # Ensure a canonical "vimp" column exists so the positive-flag logic and
      # plot.gg_vimp both work regardless of what randomForest named the column.
      colnames(gg_dta)[1] <- "vimp"
    }
  }
  if (missing(nvar)) {
    nvar <- nrow(gg_dta)
  }
  if (nvar > nrow(gg_dta)) {
    nvar <- nrow(gg_dta)
  }

  # Handle multiclass importance
  if (ncol(gg_dta) > 1) {
    # Classification...
    arg_set <- list(...)

    if (!is.null(arg_set$which.outcome)) {
      # test which.outcome specification
      if (!is.numeric(arg_set$which.outcome)) {
        if (arg_set$which.outcome %in% colnames(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste(
              "which.outcome naming is incorrect.",
              arg_set$which.outcome,
              "\nis not in",
              colnames(gg_dta)
            )
          )
        }
      } else {
        # which.outcome = 0 gives overall importance (column 1)
        # which.outcome = k gives class k importance (column k+1)
        if (!is.numeric(arg_set$which.outcome) || arg_set$which.outcome < 0) {
          stop("which.outcome must be a non-negative integer or a class name.")
        }
        if (arg_set$which.outcome < ncol(gg_dta)) {
          gg_v <- data.frame(vimp = sort(gg_dta[, arg_set$which.outcome + 1],
            decreasing = TRUE
          ))
          gg_v$vars <-
            rownames(gg_dta)[order(gg_dta[, arg_set$which.outcome + 1],
              decreasing = TRUE
            )]
        } else {
          stop(
            paste0(
              "which.outcome (", arg_set$which.outcome, ") is out of range. ",
              "Valid values are 0 (overall) to ", ncol(gg_dta) - 1,
              " (number of classes)."
            )
          )
        }
      }
      gg_dta <- gg_v
    } else {
      gg_dta$vars <- rownames(gg_dta)
    }

    pivot_cols <-
      colnames(gg_dta)[-which(colnames(gg_dta) == "vars")]
    gg_dta <- tidyr::pivot_longer(
      gg_dta,
      tidyr::all_of(pivot_cols),
      names_to = "set",
      values_to = "vimp"
    )
    gg_dta <- gg_dta[order(gg_dta$vimp, decreasing = TRUE), ]
    gg_dta$vars <- factor(gg_dta$vars)
  } else {
    gg_dta$vars[which(is.na(gg_dta$vars))] <-
      rownames(gg_dta)[which(is.na(gg_dta$vars))]
  }
  gg_dta <- gg_dta[seq_len(nvar), ]

  gg_dta$vars <-
    factor(gg_dta$vars, levels = rev(unique(gg_dta$vars)))
  gg_dta$positive <- TRUE
  gg_dta$positive[which(gg_dta$vimp <= 0)] <- FALSE

  class(gg_dta) <- c("gg_vimp", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
## Internal utility functions shared across the package.
## None of these are exported to end-users.

# --------------------------------------------------------------------------- #
# Internal: lead / lag shift for numeric vectors.
#
# `x`        numeric vector of values.
# `shift_by` integer length 1 giving the number of positions to lead
#            (positive) or lag (negative) by; can also be a vector to
#            return a matrix of shifts.
#
# Removes the dplyr::lead dependency.  Adapted from
# http://ctszkin.com/2012/03/11/generating-a-laglead-variables/
#
# @noRd
shift <- function(x, shift_by = 1) {
  stopifnot(is.numeric(shift_by))
  stopifnot(is.numeric(x))

  if (length(shift_by) > 1) {
    return(sapply(shift_by, shift, x = x))
  }

  abs_shift_by <- abs(shift_by)
  if (shift_by > 0) {
    out <- c(tail(x, -abs_shift_by), rep(NA, abs_shift_by))
  } else if (shift_by < 0) {
    out <- c(rep(NA, abs_shift_by), head(x, -abs_shift_by))
  } else {
    out <- x
  }
  out
}

# --------------------------------------------------------------------------- #
# Internal helper: label a survfit tbl with stratum group names.
#
# survfit() concatenates strata end-to-end in ascending-time order. Stratum
# boundaries are detected by finding rows where the time column resets
# (i.e. time[i] < time[i-1]).
#
# @param tbl     data.frame produced from survfit output (must have $time col)
# @param data    original data.frame passed to kaplan()/nelson()
# @param by      character; name of the grouping column in data
#
# @return tbl with an additional $groups column containing the group label
#   for each row.
.label_strata <- function(tbl, data, by) {
  # Use levels() for factors to respect the existing ordering; fall back to
  # unique() (in order of first appearance) for character/numeric vectors.
  by_col <- data[[by]]
  lbls <- if (is.factor(by_col)) levels(by_col) else unique(by_col)

  # Single stratum or fewer than 2 rows: label everything with first group
  if (nrow(tbl) < 2L) {
    tbl$groups <- lbls[1L]
    return(tbl)
  }

  # Detect stratum boundaries where the time column resets
  tm_splits <- which(c(FALSE, sapply(seq(2L, nrow(tbl)), function(ind) {
    tbl$time[ind] < tbl$time[ind - 1L]
  })))

  tbl$groups <- lbls[1L]
  if (length(tm_splits) > 0L) {
    for (ind in seq_along(tm_splits)) {
      tbl$groups[tm_splits[ind]:nrow(tbl)] <- lbls[ind + 1L]
    }
  }
  tbl
}

# testthat for shift function

test_that("lead or lag a vector", {
  expect_identical(shift(1:10, 2),  c(3:10, NA, NA))
  expect_identical(shift(1:10, -2), c(NA, NA, 1:8))
  expect_identical(shift(1:10, 0),  1:10)
  expect_identical(
    shift(1:10, 1:2),
    cbind(c(2:10, NA), c(3:10, NA, NA))
  )
})

##=============================================================================
#' Plot a `gg_beta_varpro` object
#'
#' Horizontal bar chart of the mean absolute coefficient
#' \eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)} per variable, sorted
#' descending so
#' the eye lands on the top variable first. Bars filled blue when above the
#' selection cutoff, grey otherwise. Dashed red line marks the cutoff.
#'
#' @section Reading the chart:
#' Each bar is the average magnitude of a per-rule lasso coefficient for
#' that variable. **The numeric scale carries the predictor's units.**
#' If "age" is in years and "creatinine" is in mg/dL, a longer bar for
#' age does not mean age is "more important" in any unit-free sense.
#' Comparisons across data sets or across variables with very different
#' units require keeping the units context in mind. Within one data set,
#' bars are comparable up to that unit caveat.
#'
#' Variables above the cutoff are coloured blue and flagged `selected`;
#' variables below are grey. Lasso shrinkage can drive a rule's
#' \eqn{\hat{\beta}}{beta hat} to
#' exactly zero; those rules are kept in the average, so a variable
#' with many shrunk-to-zero rules will sit lower in the ranking than
#' one whose released coefficients are consistently non-zero.
#'
#' For a classification fit, variables are sorted by
#' `mean(|sum-of-class-beta|)` descending and that ordering is shared
#' across every facet, so rows line up between classes for visual
#' comparison. Each facet has its own cutoff line.
#'
#' @section What this tells you:
#' Use the bar chart as a selection ranking, not as an effect-size axis.
#' Pair it with [gg_varpro()] to see where split-strength importance and
#' local lasso-beta importance agree or disagree; disagreement is often the
#' interesting signal.
#'
#' @param x A `gg_beta_varpro` object from [gg_beta_varpro()].
#' @param ... Not currently used.
#'
#' @return A `ggplot` object.
#'
#' @seealso [gg_beta_varpro()].
#'
#' @examples
#' \donttest{
#' if (requireNamespace("varPro", quietly = TRUE)) {
#'   set.seed(1)
#'   v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
#'   plot(gg_beta_varpro(v))
#' }
#' }
#'
#' @name plot.gg_beta_varpro
#' @importFrom ggplot2 ggplot aes geom_col geom_hline coord_flip
#' @importFrom ggplot2 scale_fill_manual labs theme_minimal
#' @export
plot.gg_beta_varpro <- function(x, ...) {
  if (nrow(x) == 0L) {
    stop("plot.gg_beta_varpro: nothing to plot (gg_beta_varpro has 0 rows).",
         call. = FALSE)
  }
  prov          <- attr(x, "provenance")
  n_rules_total <- if (!is.null(prov)) prov$n_rules_total %||% NA_integer_ else NA_integer_
  cv_txt        <- .gg_beta_cv_txt(prov)
  has_class     <- "class" %in% names(x)
  cutoff_vec    <- .gg_beta_cutoff_vec(prov, x, has_class)
  caption_txt   <- if (has_class) {
    .gg_beta_caption_class(x, n_rules_total, cv_txt)
  } else {
    sprintf("Mean |beta| over %s rules. Lasso: %s. Cutoff: %.4g.",
            if (is.na(n_rules_total)) "NA" else format(n_rules_total),
            cv_txt, cutoff_vec[[1]])
  }

  p <- ggplot2::ggplot(
    x,
    ggplot2::aes(
      x    = .data[["variable"]],
      y    = .data[["beta_mean"]],
      fill = factor(.data[["selected"]])
    )
  ) +
    ggplot2::geom_col() +
    ggplot2::coord_flip() +
    ggplot2::scale_fill_manual(
      values = c("TRUE" = "#4e8fcd", "FALSE" = "#888888"),
      guide  = "none"
    ) +
    ggplot2::labs(
      x = NULL,
      y = "Mean |beta| (per-rule lasso)",
      caption = caption_txt
    ) +
    ggplot2::theme_minimal()

  if (has_class && length(unique(x$class)) > 1L) {
    # Per-class cutoff lines via data join
    hline_df <- data.frame(
      class  = factor(names(cutoff_vec), levels = levels(x$class)),
      cutoff = unname(cutoff_vec),
      stringsAsFactors = FALSE
    )
    p <- p +
      ggplot2::facet_wrap(~ class, nrow = 1L) +
      ggplot2::geom_hline(
        data        = hline_df,
        ggplot2::aes(yintercept = .data[["cutoff"]]),
        linetype    = "dashed",
        color       = "#e74c3c",
        linewidth   = 0.7,
        inherit.aes = FALSE
      )
  } else {
    # Single panel (regression or single-class) -- one horizontal line
    cutoff_scalar <- if (has_class) cutoff_vec[[as.character(x$class[1])]] else cutoff_vec[[1]]
    p <- p + ggplot2::geom_hline(
      yintercept = cutoff_scalar,
      linetype   = "dashed",
      color      = "#e74c3c",
      linewidth  = 0.7
    )
  }
  p
}

# ---- Internal helpers ------------------------------------------------------

#' @noRd
.gg_beta_cv_txt <- function(prov) {
  if (!is.null(prov) && isTRUE(prov$use.cv)) return("cv")
  if (!is.null(prov) && length(prov$use.cv) == 1L && is.na(prov$use.cv)) {
    return("unknown (precomputed)")
  }
  "fixed"
}

#' @noRd
.gg_beta_cutoff_vec <- function(prov, x, has_class) {
  if (!is.null(prov) && !is.null(prov$cutoff)) return(prov$cutoff)
  if (has_class) {
    return(stats::setNames(vapply(split(x$beta_mean, x$class), mean, numeric(1)),
                           levels(x$class)))
  }
  stats::setNames(mean(x$beta_mean), "regr")
}

#' @noRd
.gg_beta_caption_class <- function(x, n_rules_total, cv_txt) {
  n_panels <- length(unique(x$class))
  tail <- if (n_panels == 1L) {
    sprintf("Class: %s.", as.character(x$class[1]))
  } else {
    sprintf("%d classes (faceted).", n_panels)
  }
  sprintf("Mean |beta| over %s rules. Lasso: %s. %s",
          if (is.na(n_rules_total)) "NA" else format(n_rules_total),
          cv_txt, tail)
}

####**********************************************************************
####  Internal helpers shared by print.gg_* and summary.gg_* methods.
####
####  Every gg_* extractor calls .set_provenance() on the object it
####  returns, attaching a small list under attr(x, "provenance") that
####  captures the source package, forest family, ntree, and predictor
####  set. The print/summary methods consume that attribute to render a
####  uniform header.
####
####  Provenance is best-effort: missing fields are NA. Methods must
####  tolerate the attribute being absent (older saved objects).
####**********************************************************************

# Build the provenance list from a forest object.
.gg_provenance <- function(object) {
  if (inherits(object, "rfsrc")) {
    return(list(
      source     = "randomForestSRC",
      family     = object$family %||% NA_character_,
      ntree      = object$ntree %||% NA_integer_,
      n          = object$n %||% NA_integer_,
      xvar.names = object$xvar.names %||% character(0)
    ))
  }
  if (inherits(object, "randomForest")) {
    return(list(
      source     = "randomForest",
      family     = object$type %||% NA_character_,
      ntree      = object$ntree %||% NA_integer_,
      n          = if (!is.null(object$y)) length(object$y) else NA_integer_,
      xvar.names = if (!is.null(object$importance)) {
        rownames(object$importance)
      } else {
        character(0)
      }
    ))
  }
  # Unknown / non-forest object — return NULL so callers can skip cleanly.
  NULL
}

# Attach provenance to a gg_* object and return it unchanged.
# Skips silently when .gg_provenance() returns NULL (non-forest input).
.set_provenance <- function(x, object) {
  prov <- .gg_provenance(object)
  if (!is.null(prov)) attr(x, "provenance") <- prov
  x
}

# Format the standard one-line header used by print.gg_* methods.
# Returns a character scalar.
.gg_header <- function(x, class_label) {
  prov <- attr(x, "provenance")
  if (is.null(prov)) {
    return(sprintf("<%s>", class_label))
  }
  src    <- prov$source    %||% NA
  family <- prov$family    %||% NA
  ntree  <- prov$ntree     %||% NA
  n      <- prov$n         %||% NA

  bits <- c(
    if (!is.na(src))    sprintf("from %s", src),
    if (!is.na(family)) sprintf("family: %s", family),
    if (!is.na(ntree))  sprintf("ntree: %s", ntree),
    if (!is.na(n))      sprintf("n: %s", n)
  )
  if (length(bits) == 0L) {
    return(sprintf("<%s>", class_label))
  }
  sprintf("<%s>  %s", class_label, paste(bits, collapse = "  |  "))
}

# Null-coalescing helper (avoid rlang dependency).
`%||%` <- function(a, b) if (is.null(a)) b else a

##=============================================================================
#' Variable dependency graph from a uvarpro model
#'
#' A \code{uvarpro} fit records how strongly each variable depends on the
#' others.  This function pulls those cross-variable dependency scores from
#' the fit with \code{\link[varPro]{get.beta.entropy}} and
#' \code{\link[varPro]{sdependent}}, and returns them as a tidy list that
#' \code{plot.gg_udependent} can draw as a network.
#'
#' @section What cross-variable dependency is doing:
#' UVarPro (Zhou, Lu and Ishwaran, 2026) extends the varpro framework to
#' the unsupervised setting: grow a forest without a response, then use
#' the same region-release contrasts varpro uses for supervised
#' importance to ask, "which variables explain the structure in the
#' data?" The lasso-driven variant frames each region-release contrast
#' as a classification task (does an observation belong to the region
#' or to its release?) and fits a lasso logistic regression with the
#' other variables as predictors. The coefficient on variable \eqn{j}
#' in the model for variable \eqn{i}'s region-release contrast is the
#' entry \eqn{I[i, j]} of the matrix \code{varPro::get.beta.entropy()}
#' returns.
#'
#' Read that entry as "how much does knowing \eqn{j} help separate
#' \eqn{i}'s region from its release". A large \eqn{I[i, j]} says
#' \eqn{j} carries information about the structure varpro picked up in
#' \eqn{i}. \code{varPro::sdependent} thresholds that matrix at a
#' user-chosen cut and returns the set of "signal" variables: the
#' nodes with high enough out-degree to be worth keeping. We pass the
#' threshold through to \code{sdependent} and use the same matrix to
#' weight the edges of the resulting graph.
#'
#' The graph is directed by default because \eqn{I[i, j]} and
#' \eqn{I[j, i]} are separate lasso coefficients and need not agree;
#' setting \code{directed = FALSE} collapses each pair by taking the
#' larger of the two, which is appropriate when you only want to see
#' that two variables are dependent, not which way the dependency
#' reads.
#'
#' @section What's in the output:
#' \code{$edges} has one row per surviving edge with the raw weight
#' \code{I[i, j]} (or, for undirected graphs, the max of the two
#' directions). \code{$nodes} has one row per surviving variable with
#' its degree (out-degree for directed, total degree for undirected)
#' and a \code{selected} flag for membership in the \code{sdependent}
#' signal set. \code{$graph} is the same information packaged as an
#' \code{igraph} object, with \code{weight}, \code{degree}, and
#' \code{selected} attached so \code{plot.gg_udependent} can render it
#' without recomputing anything.
#'
#' @section What you use this for:
#' \itemize{
#'   \item screen a wide unsupervised dataset for the small set of
#'     variables UVarPro thinks are carrying the signal: the nodes
#'     with high degree, or those flagged \code{selected = TRUE};
#'   \item spot clusters of mutually dependent variables (hubs and the
#'     spokes around them) that may be measuring the same underlying
#'     construct;
#'   \item compare two datasets, or two preprocessing pipelines, by
#'     looking at how their dependency graphs change.
#' }
#' An edge in this graph is a statistical dependency in the unsupervised
#' decomposition of the data. It is not a causal arrow. A high
#' \eqn{I[i, j]} says \eqn{j} predicts \eqn{i}'s region membership,
#' not that \eqn{j} causes \eqn{i}.
#'
#' @references
#' Zhou, L., Lu, M. and Ishwaran, H. (2026). Variable priority for
#' unsupervised variable selection. \emph{Pattern Recognition},
#' 172:112727.
#'
#' @param object A fitted \code{uvarpro} object (required).
#' @param threshold Numeric; the positive dependency threshold passed on to
#'   \code{sdependent()}.  An edge \eqn{i \to j} is drawn when
#'   \code{I[i, j] >= threshold}.  Default \code{0.25}.
#' @param q.signal Quantile threshold (0--1) for picking out the signal
#'   variables; passed on to \code{sdependent()}.  Default \code{0.75}.
#' @param directed Logical; \code{TRUE} (default) builds a directed igraph.
#' @param min.degree Integer or \code{NULL}.  When set, only nodes with
#'   degree \eqn{\ge} \code{min.degree} are kept in \code{$nodes},
#'   \code{$edges}, and \code{$graph}.
#' @param ... Additional arguments forwarded to \code{varPro::sdependent()}.
#'
#' @return A named list of class \code{"gg_udependent"} with elements:
#' \describe{
#'   \item{\code{$edges}}{Data frame: \code{variable_from}, \code{variable_to},
#'     \code{weight} (raw cross-importance value).}
#'   \item{\code{$nodes}}{Data frame: \code{variable} (factor, levels by
#'     descending degree), \code{degree} (integer; out-degree when
#'     \code{directed = TRUE}, total degree when \code{directed = FALSE}),
#'     \code{selected} (logical, \code{TRUE} if in \code{sdependent}'s
#'     signal set).}
#'   \item{\code{$graph}}{igraph object. \code{NULL} if no dependencies
#'     detected.}
#' }
#' A \code{"provenance"} attribute carries \code{threshold}, \code{q.signal},
#' \code{directed}, \code{min.degree}, \code{xvar.names}, and \code{n}.
#'
#' @seealso \code{\link{plot.gg_udependent}}
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' uv <- varPro::uvarpro(iris[, -5], ntree = 50)
#' gg <- gg_udependent(uv)
#' print(gg)
#' }
#'
#' @importFrom varPro get.beta.entropy sdependent
#' @importFrom igraph graph_from_adjacency_matrix degree delete_vertices as_data_frame V
#' @export
gg_udependent <- function(object,
                           threshold  = 0.25,
                           q.signal   = 0.75,
                           directed   = TRUE,
                           min.degree = NULL,
                           ...) {
  .validate_udep_inputs(object, threshold, directed)

  ## ---- Compute cross-variable dependency matrix ----------------------------
  imp_mat <- varPro::get.beta.entropy(object)

  ## ---- Helper: build and return an empty gg_udependent result ---------------
  .empty_result <- function(msg) {
    warning("gg_udependent: ", msg,
            "\nReturning empty structure. Consider lowering threshold.",
            call. = FALSE)
    empty_edges <- data.frame(variable_from = character(0),
                               variable_to   = character(0),
                               weight        = numeric(0),
                               stringsAsFactors = FALSE)
    empty_nodes <- data.frame(variable = factor(character(0)),
                               degree   = integer(0),
                               selected = logical(0),
                               stringsAsFactors = FALSE)
    result <- structure(
      list(edges = empty_edges, nodes = empty_nodes, graph = NULL),
      class = c("gg_udependent", "list")
    )
    attr(result, "provenance") <- .udep_provenance(object, threshold, q.signal,
                                                     directed, min.degree)
    result
  }

  ## ---- Build adjacency from threshold; short-circuit if empty --------------
  adj_mat <- (imp_mat >= threshold) * 1
  diag(adj_mat) <- 0
  if (sum(adj_mat) == 0L) {
    return(.empty_result(
      paste0("no edges found at threshold=", threshold)
    ))
  }

  ## ---- Call sdependent for signal detection --------------------------------
  sdep <- varPro::sdependent(imp_mat, threshold = threshold,
                              q.signal = q.signal, directed = directed,
                              min.degree = min.degree, plot = FALSE, ...)

  ## ---- Handle empty graph (sdependent may also return character) -----------
  if (is.character(sdep)) {
    return(.empty_result(sdep))
  }

  ## ---- Build igraph from adjacency -----------------------------------------
  ## For undirected, symmetrise first so edge existence = max(I[i,j], I[j,i])
  ## and mode = "undirected" is valid (igraph >= 1.6.0 requires symmetry).
  if (!isTRUE(directed)) {
    adj_mat <- pmax(adj_mat, t(adj_mat))
  }
  g <- igraph::graph_from_adjacency_matrix(
    adj_mat,
    mode  = if (isTRUE(directed)) "directed" else "undirected",
    diag  = FALSE
  )
  isolated <- igraph::degree(g, mode = "all") == 0
  g <- igraph::delete_vertices(g, which(isolated))

  ## ---- Build tidy edge data frame with raw weights -------------------------
  edge_df <- igraph::as_data_frame(g, what = "edges")
  if (nrow(edge_df) > 0L) {
    if (isTRUE(directed)) {
      edge_df$weight <- mapply(function(i, j) imp_mat[i, j],
                                edge_df[[1L]], edge_df[[2L]])
    } else {
      ## Undirected: weight = max of both directions
      edge_df$weight <- mapply(
        function(i, j) max(imp_mat[i, j], imp_mat[j, i]),
        edge_df[[1L]], edge_df[[2L]])
    }
  } else {
    edge_df$weight <- numeric(0)
  }
  names(edge_df)[1:2] <- c("variable_from", "variable_to")

  ## ---- Build tidy node data frame ------------------------------------------
  vnames  <- igraph::V(g)$name
  ## degree: out-degree for directed (matches sdependent's signal.vars logic),
  ## total degree for undirected
  deg_vec <- if (isTRUE(directed)) {
    igraph::degree(g, mode = "out")[vnames]
  } else {
    igraph::degree(g)[vnames]
  }

  signal_set <- if (is.null(sdep$signal.vars)) character(0) else sdep$signal.vars
  node_df <- data.frame(
    variable = factor(vnames, levels = vnames[order(-deg_vec)]),
    degree   = as.integer(deg_vec),
    selected = vnames %in% signal_set,
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  ## ---- Apply min.degree node filtering (user-requested subsetting) ---------
  if (!is.null(min.degree)) {
    keep       <- node_df$degree >= min.degree
    keep_names <- as.character(node_df$variable)[keep]
    drop_names <- as.character(node_df$variable)[!keep]
    g          <- igraph::delete_vertices(g, drop_names)
    edge_df    <- edge_df[
      edge_df$variable_from %in% keep_names &
      edge_df$variable_to   %in% keep_names, , drop = FALSE]
    node_df    <- node_df[keep, , drop = FALSE]
    rownames(edge_df) <- NULL
    rownames(node_df) <- NULL
  }

  ## ---- Set igraph node attributes ------------------------------------------
  if (length(igraph::V(g)) > 0L) {
    igraph::V(g)$degree   <- node_df$degree[
      match(igraph::V(g)$name, as.character(node_df$variable))]
    igraph::V(g)$selected <- node_df$selected[
      match(igraph::V(g)$name, as.character(node_df$variable))]
  }

  ## ---- Set igraph edge weights (order-insensitive for undirected) -----------
  if (length(igraph::E(g)) > 0L && nrow(edge_df) > 0L) {
    el <- igraph::as_data_frame(g, what = "edges")
    if (isTRUE(directed)) {
      idx <- match(paste(el$from, el$to),
                   paste(edge_df$variable_from, edge_df$variable_to))
    } else {
      key_g <- paste(pmin(el$from, el$to),     pmax(el$from, el$to))
      key_e <- paste(pmin(edge_df$variable_from, edge_df$variable_to),
                     pmax(edge_df$variable_from, edge_df$variable_to))
      idx <- match(key_g, key_e)
    }
    igraph::E(g)$weight <- edge_df$weight[idx]
  }

  ## ---- Assemble result ------------------------------------------------------
  result <- structure(
    list(edges = edge_df, nodes = node_df, graph = g),
    class = c("gg_udependent", "list")
  )
  attr(result, "provenance") <- .udep_provenance(object, threshold, q.signal,
                                                   directed, min.degree)
  result
}

## ---- Internal helpers -------------------------------------------------------

#' @keywords internal
.validate_udep_inputs <- function(object, threshold, directed) {
  if (missing(object) || is.null(object)) {
    stop("'object' must be a fitted uvarpro object.", call. = FALSE)
  }
  if (!inherits(object, "uvarpro")) {
    stop("'object' must be a uvarpro fit (class \"uvarpro\").", call. = FALSE)
  }
  if (!is.numeric(threshold) || length(threshold) != 1L || threshold <= 0) {
    stop("'threshold' must be a single positive numeric value.", call. = FALSE)
  }
  if (!is.logical(directed) || length(directed) != 1L) {
    stop("'directed' must be a single logical value.", call. = FALSE)
  }
  invisible(NULL)
}

#' @keywords internal
.udep_provenance <- function(object, threshold, q.signal, directed, min.degree) {
  list(
    threshold  = threshold,
    q.signal   = q.signal,
    directed   = directed,
    min.degree = min.degree,
    xvar.names = object$xvar.names,
    n          = nrow(object$x)
  )
}

# Tests for gg_udependent (Phase 3)

## ── Helpers ──────────────────────────────────────────────────────────────────

make_uvp <- function(ntree = 25L) {
  # uvarpro() grows a *synthetic-supervised* forest (yxyz123 ~ ., random Y), so
  # yvar.wt is non-empty and it does NOT trip the entry.c:184 gcc-UBSAN report
  # (only the truly-unsupervised isopro grow does). Runs on CRAN.
  set.seed(42L)
  varPro::uvarpro(iris[, -5L], ntree = ntree)
}

# gg_udependent() recomputes varPro::get.beta.entropy() on every call (~1.5s,
# the only slow step, and a pure function of the fit). The tests below exercise
# the same fit under several argument combinations, so memoise the result per
# argument signature: identical coverage, but one entropy computation per
# distinct call instead of one per test (this file was ~24s of the suite).
.ggu_cache <- new.env(parent = emptyenv())

# .quiet = TRUE suppresses warnings only for callers that legitimately warn
# (the empty-graph threshold cases); every other call leaves warnings live so
# an unexpected new warning still surfaces as a test failure.
make_ggu <- function(..., .quiet = FALSE) {
  key <- paste(deparse(list(...)), collapse = "")
  if (is.null(.ggu_cache[[key]])) {
    .ggu_cache[[key]] <- if (.quiet) {
      suppressWarnings(gg_udependent(make_uvp(), ...))
    } else {
      gg_udependent(make_uvp(), ...)
    }
  }
  .ggu_cache[[key]]
}

## ── Input validation ─────────────────────────────────────────────────────────

test_that("gg_udependent: missing object -> stop", {
  expect_error(gg_udependent(), regexp = "object")
})

test_that("gg_udependent: non-uvarpro object -> stop", {
  expect_error(gg_udependent(list(x = 1)), regexp = "uvarpro")
})

test_that("gg_udependent: non-positive threshold -> stop", {
  uv <- make_uvp()
  expect_error(gg_udependent(uv, threshold = -0.1), regexp = "threshold")
  expect_error(gg_udependent(uv, threshold = 0),    regexp = "threshold")
})

## ── Class & structure ────────────────────────────────────────────────────────

test_that("gg_udependent returns gg_udependent class", {
  expect_s3_class(make_ggu(), "gg_udependent")
})

test_that("gg_udependent$edges has required columns", {
  gg <- make_ggu()
  expect_true(all(c("variable_from", "variable_to", "weight") %in% names(gg$edges)))
  expect_type(gg$edges$weight, "double")
})

test_that("gg_udependent$nodes has required columns", {
  gg <- make_ggu()
  expect_true(all(c("variable", "degree", "selected") %in% names(gg$nodes)))
  expect_s3_class(gg$nodes$variable, "factor")
  expect_type(gg$nodes$degree,   "integer")
  expect_type(gg$nodes$selected, "logical")
})

test_that("gg_udependent$graph is an igraph", {
  skip_if_not_installed("igraph")
  gg <- make_ggu()
  expect_true(igraph::is_igraph(gg$graph))
})

test_that("gg_udependent directed=TRUE returns directed igraph", {
  skip_if_not_installed("igraph")
  gg <- make_ggu(directed = TRUE)
  expect_true(igraph::is_directed(gg$graph))
})

test_that("gg_udependent directed=FALSE returns undirected igraph", {
  skip_if_not_installed("igraph")
  gg <- make_ggu(directed = FALSE)
  expect_false(igraph::is_directed(gg$graph))
})

test_that("gg_udependent$edges is empty data frame (not NULL) for empty graph", {
  # threshold=999 -> no edges -> empty graph
  gg <- make_ggu(threshold = 999, .quiet = TRUE)
  expect_false(is.null(gg$edges))
  expect_s3_class(gg$edges, "data.frame")
  expect_equal(nrow(gg$edges), 0L)
})

test_that("gg_udependent$nodes is empty data frame for empty graph", {
  gg <- make_ggu(threshold = 999, .quiet = TRUE)
  expect_false(is.null(gg$nodes))
  expect_equal(nrow(gg$nodes), 0L)
})

## ── Provenance ───────────────────────────────────────────────────────────────

test_that("gg_udependent provenance has all expected fields", {
  gg   <- make_ggu()
  prov <- attr(gg, "provenance")
  expect_type(prov, "list")
  expect_true(all(c("threshold", "q.signal", "directed", "min.degree",
                    "xvar.names", "n") %in% names(prov)))
})

test_that("gg_udependent provenance threshold matches argument", {
  gg <- make_ggu(threshold = 0.5)
  expect_equal(attr(gg, "provenance")$threshold, 0.5)
})

## ── S3 companions ────────────────────────────────────────────────────────────

test_that("print.gg_udependent returns object invisibly", {
  gg  <- make_ggu()
  out <- capture.output(ret <- print(gg))
  expect_identical(ret, gg)
  expect_true(any(grepl("gg_udependent", out)))
})

test_that("summary.gg_udependent returns summary.gg_udependent class", {
  gg <- make_ggu()
  s  <- summary(gg)
  expect_s3_class(s, "summary.gg_udependent")
})

test_that("autoplot.gg_udependent returns a ggplot", {
  skip_if_not_installed("ggraph")
  gg <- make_ggu()
  expect_s3_class(ggplot2::autoplot(gg), "ggplot")
})

## ── Plot smoke tests ─────────────────────────────────────────────────────────

test_that("plot.gg_udependent default returns a ggplot", {
  skip_if_not_installed("ggraph")
  gg <- make_ggu()
  p  <- plot(gg)
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_udependent layout='kk' returns a ggplot", {
  skip_if_not_installed("ggraph")
  gg <- make_ggu()
  p  <- plot(gg, layout = "kk")
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_udependent empty graph -> stop with informative message", {
  gg <- make_ggu(threshold = 999, .quiet = TRUE)
  expect_error(plot(gg), regexp = "no edges")
})

## ── vdiffr snapshots — see test_snapshots.R ──────────────────────────────────
## Visual regression tests for plot.gg_udependent are in test_snapshots.R
## (guarded by VDIFFR_RUN_TESTS=true), following the package convention.

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Quantile-based cut points for coplots
#'
#' @param object Numeric vector of predictor values.
#' @param groups Number of quantile points (or intervals) to compute.
#' @param intervals Logical indicating whether to return interval boundaries
#'   suitable for \code{cut()} (length \code{groups + 1}) or the interior
#'   quantile points (length \code{groups}).
#'
#'
#' @description
#' This helper wraps \code{\link[stats]{quantile}} to create well-spaced
#' cut points for conditioning plots. When \code{intervals = TRUE} the lower
#' boundary is nudged down so that \code{cut()} treats the minimum value as a
#' valid observation.
#'
#' The output can be passed directly into the breaks argument of the
#' \code{cut} function for creating groups for coplots.
#'
#' @return Numeric vector of quantile points. When \code{intervals = TRUE}
#'   the result is strictly increasing and can be supplied to \code{cut()} to
#'   produce \code{groups} balanced strata.
#'
#' @seealso \code{cut}
#' @importFrom stats quantile
#'
#' @examples
#' data(Boston, package = "MASS")
#' rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston)
#'
#' # To create 6 intervals, we want 7 points.
#' # quantile_pts will find balanced intervals
#' rm_pts <- quantile_pts(rfsrc_boston$xvar$rm, groups = 6, intervals = TRUE)
#'
#' # Use cut to create the intervals
#' rm_grp <- cut(rfsrc_boston$xvar$rm, breaks = rm_pts)
#'
#' summary(rm_grp)
#'
#' @export
quantile_pts <- function(object, groups, intervals = FALSE) {
  if (!is.numeric(groups) || groups < 1) {
    stop("`groups` must be a positive integer")
  }
  groups <- as.integer(groups)

  # Drop missing values before computing quantiles
  object <- stats::na.omit(object)
  if (!length(object)) {
    return(numeric())
  }

  # When intervals = TRUE we need groups + 1 boundary points (including both
  # endpoints) so that cut() produces exactly `groups` non-overlapping bins.
  # When intervals = FALSE we return `groups` interior quantile points.
  probs <- if (intervals) {
    seq(0, 1, length.out = groups + 1)
  } else {
    seq(0, 1, length.out = groups)
  }

  # type = 2 uses the "nearest even" convention, matching the behaviour of
  # SAS PROC UNIVARIATE and ensuring consistent results on small samples.
  pts <- as.numeric(stats::quantile(object,
    probs = probs,
    na.rm = TRUE,
    type = 2
  ))

  # Ensure breaks are strictly increasing for cut()
  if (intervals) {
    # Collapse any duplicated quantile values (can happen with low-cardinality
    # variables where many observations share the same value).
    pts <- unique(pts)
    if (length(pts) < 2) {
      # Degenerate case: all observations equal. Add a tiny offset so cut()
      # still produces a valid (if trivial) set of intervals.
      pts <- c(pts, pts + .Machine$double.eps)
    }
    # Nudge the lower boundary down so that the minimum value falls *inside*
    # the first interval (cut() uses open-on-the-left, closed-on-the-right
    # intervals by default, so the exact minimum would otherwise be excluded).
    pts[1] <- pts[1] - 1e-7
  }

  pts
}

# testthat for quantile_pts function

test_that("cutting a vector at evenly space points", {
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  rfsrc_boston <-
    randomForestSRC::rfsrc(
      medv ~ .,
      data = boston,
      forest = TRUE,
      importance = TRUE,
      tree.err = TRUE,
      save.memory = TRUE)
  # To create 6 intervals, we want 7 points.
  # quantile_pts will find balanced intervals
  rm_pts <-
    quantile_pts(rfsrc_boston$xvar$rm,
                 groups = 6,
                 intervals = TRUE)

  expect_true(is.numeric(rm_pts))
  expect_true(length(rm_pts) >= 2)
  expect_true(rm_pts[1] < min(rfsrc_boston$xvar$rm))
  expect_equal(max(rm_pts), max(rfsrc_boston$xvar$rm))
  expect_true(all(diff(rm_pts) > 0))

  # Use cut to create the intervals
  rm_grp <- cut(rfsrc_boston$xvar$rm, breaks = rm_pts)
  expect_s3_class(rm_grp, "factor")
  expect_equal(length(rm_grp), length(rfsrc_boston$xvar$rm))
  expect_equal(length(levels(rm_grp)), length(rm_pts) - 1)

  rm_pts <- quantile_pts(rfsrc_boston$xvar$rm, groups = 6)

  expect_true(is.numeric(rm_pts))
  expect_equal(length(rm_pts), 6)
  # First quantile coincides with the minimum value
  expect_equal(min(rfsrc_boston$xvar$rm), min(rm_pts), tolerance = 1.e-7)

  # Test the number of points for lots of groups.
  rm_pts <-
    quantile_pts(rfsrc_boston$xvar$rm, groups = nrow(rfsrc_boston$xvar) + 2)
  expect_equal(length(rm_pts), nrow(rfsrc_boston$xvar) + 2)
})

##=============================================================================
#' Recover original variable names from varpro one-hot encoded feature names
#'
#' \code{varpro} one-hot encodes factor variables, appending a numeric suffix
#' for each level -- \code{sex} becomes \code{sex0} and \code{sex1}.  To map
#' those back, this function strips the suffix one character at a time until
#' every name in \code{varpro_names} matches a column in \code{dataset}.
#'
#' @param varpro_names character vector of names as output by varpro (may
#'   include one-hot encoded suffixed names such as \code{"sex0"}, \code{"sex1"})
#' @param dataset the original data frame passed to varpro, used to look up
#'   valid column names
#'
#' @return character vector of unique original variable names (no suffixes)
#'
#' @seealso \code{\link{gg_partialpro}}
#'
#' @importFrom stringr str_sub
#'
#' @examples
#' ## ------------------------------------------------------------------
#' ## Simple case: one continuous variable + one binary factor
#' ## ------------------------------------------------------------------
#' ds <- data.frame(age = c(25, 30, 45), sex = c("M", "F", "M"))
#'
#' # varpro one-hot encodes 'sex' into 'sex0' and 'sex1'
#' varpro_names <- c("age", "sex0", "sex1")
#' varpro_feature_names(varpro_names, ds)
#' # Returns: c("age", "sex")
#'
#' ## ------------------------------------------------------------------
#' ## Multi-level factor: three-level 'group' variable
#' ## ------------------------------------------------------------------
#' ds2 <- data.frame(score = 1:6,
#'                   group = factor(rep(c("A", "B", "C"), 2)))
#'
#' # varpro appends 0/1/2 for each level
#' vn2 <- c("score", "group0", "group1", "group2")
#' varpro_feature_names(vn2, ds2)
#' # Returns: c("score", "group")
#'
#' ## ------------------------------------------------------------------
#' ## Already-clean names pass through unchanged
#' ## ------------------------------------------------------------------
#' ds3 <- data.frame(x = 1:5, y = 1:5)
#' varpro_feature_names(c("x", "y"), ds3)
#' # Returns: c("x", "y")
#'
#' @export
varpro_feature_names <- function(varpro_names, dataset) {
  # Names that already match a column in dataset: keep as-is
  inc_set <- varpro_names[which(varpro_names %in% colnames(dataset))]

  # Names that do not yet match any column: need suffix stripping
  one_set <- varpro_names[which(!varpro_names %in% colnames(dataset))]

  ## Iteratively strip the last character of each unmatched name until every
  ## name either resolves to a column or the set is exhausted.
  while (length(one_set) > 0) {
    # Drop the trailing character from each remaining unmatched name
    orig <- unlist(lapply(one_set, stringr::str_sub, 1, -2))

    # Accept names that now match a dataset column
    inc_set <- union(inc_set, orig[which(orig %in% colnames(dataset))])

    # Continue stripping names that still don't match
    one_set <- orig[which(!orig %in% colnames(dataset))]
  }
  return(inc_set)
}

####**********************************************************************
####  autoplot.gg_* S3 methods.
####
####  Thin wrappers that delegate to the corresponding plot.gg_*() method,
####  letting users call ggplot2::autoplot() in pipe-friendly workflows
####  and inside patchwork / cowplot compositions.
####
####  Each method accepts ... and passes it straight through to plot.gg_*
####  so all existing plot arguments remain available.
####**********************************************************************

#' @importFrom ggplot2 autoplot
NULL

#' \code{autoplot} methods for \pkg{ggRandomForests} data objects
#'
#' These methods let you use \code{ggplot2::autoplot()} on any \code{gg_*}
#' object returned by \pkg{ggRandomForests}.  They are thin wrappers around
#' the corresponding \code{plot.gg_*()} S3 methods, so all arguments
#' accepted by those methods are forwarded via \code{...}.
#'
#' @param object A \code{gg_*} data object (see Details).
#' @param ... Additional arguments forwarded to the underlying
#'   \code{plot.gg_*()} method.
#'
#' @return A \code{ggplot} object.
#'
#' @details
#' The following \code{gg_*} classes are supported:
#' \describe{
#'   \item{\code{gg_error}}{OOB error vs. number of trees}
#'   \item{\code{gg_vimp}}{Variable importance ranking}
#'   \item{\code{gg_rfsrc}}{Predicted vs. observed values}
#'   \item{\code{gg_variable}}{Marginal dependence}
#'   \item{\code{gg_partial}}{Partial dependence (via \code{plot.variable})}
#'   \item{\code{gg_partial_rfsrc}}{Partial dependence (via \code{partial.rfsrc})}
#'   \item{\code{gg_partialpro}}{Partial dependence (via \code{varPro})}
#'   \item{\code{gg_roc}}{ROC curve}
#'   \item{\code{gg_survival}}{Survival / cumulative hazard curves}
#'   \item{\code{gg_brier}}{Time-resolved Brier score and CRPS}
#' }
#'
#' @name autoplot.gg
#'
#' @examples
#' \donttest{
#' library(ggplot2)
#' set.seed(42)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
#'                               ntree = 50, importance = TRUE,
#'                               tree.err = TRUE)
#' autoplot(gg_error(rf))
#' autoplot(gg_vimp(rf))
#' }
NULL

#' @rdname autoplot.gg
#' @export
autoplot.gg_error <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_vimp <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_rfsrc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_variable <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partial <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partial_rfsrc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partialpro <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_roc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_survival <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_brier <- function(object, ...) {
  plot(object, ...)
}

##=============================================================================
#' Plot a \code{\link{gg_partial_varpro}} object
#'
#' Draws the partial dependence curves from the list that
#' \code{\link{gg_partial_varpro}} returns.  Continuous predictors get
#' overlaid line curves, one per effect type; categorical predictors get
#' side-by-side boxplots.  Survival path-C objects (the ones you get when
#' \code{scale \%in\% c("surv","chf")} was passed to the extractor) are
#' handed off to \code{\link{plot.gg_partial_rfsrc}} for drawing.
#'
#' @section Reading the partial dependence:
#' For a continuous variable the x-axis is the variable's grid of values
#' and the y-axis is the partial prediction; each of the three effect
#' types (\code{parametric}, \code{nonparametric}, \code{causal}) is
#' drawn as its own line. The shape of the line is the story: a clear
#' slope says the model uses the variable, a flat line says it
#' essentially does not, and a U-shape or a threshold says the effect
#' is nonlinear in a way a single coefficient would miss. For a
#' categorical variable the picture is a boxplot per level; here the
#' eye is looking at level-to-level shifts in the centre of each box.
#'
#' Where the three effect types track each other, the parametric story
#' is a fair summary of what the forest is doing. Where they fan
#' apart (typically the parametric curve smoother than the
#' nonparametric, or the causal curve flatter than either) the
#' variable is one to inspect more carefully before reading a single
#' effect off the plot.
#'
#' @section What this tells you:
#' Use these curves to describe how the model uses each variable, not
#' to claim how the world works. They are a window into the fitted
#' relationship; they do not by themselves establish that intervening
#' on the variable would move the outcome. For survival path-C
#' (\code{scale = "surv"} or \code{"chf"}), the y-axis is on the
#' probability or cumulative-hazard scale, which is usually the scale
#' you want to report to a clinical audience.
#'
#' @param x A \code{\link{gg_partial_varpro}} object.
#' @param type Character vector; one or more of \code{"parametric"},
#'   \code{"nonparametric"}, \code{"causal"}.  Defaults to all three.
#'   Ignored for path-C objects.
#' @param ... Unused for path-A objects; forwarded to
#'   \code{plot.gg_partial_rfsrc} for path-C objects.
#'
#' @details
#' **Ensemble mortality (scale = "mortality"):** when the provenance scale
#' is \code{"mortality"}, the y-axis is labelled
#' \emph{"Ensemble mortality (expected events)"}.  The wording is
#' deliberate: this is an \strong{unbounded relative-risk score}, not a
#' survival probability and not \eqn{1 - S(t)} (Ishwaran, Kogalur,
#' Blackstone & Lauer, 2008 <doi:10.1214/08-AOAS169>).
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.
#'
#' @references
#' Ishwaran H, Kogalur UB, Blackstone EH, Lauer MS (2008).
#' Random survival forests. \emph{The Annals of Applied Statistics},
#' \bold{2}(3), 841--860. \doi{10.1214/08-AOAS169}.
#'
#' @seealso \code{\link{gg_partial_varpro}}
#'
#' @examples
#' set.seed(42)
#' n_obs <- 30; n_pts <- 15
#' mock_data <- list(
#'   age = list(
#'     xvirtual    = seq(30, 80, length.out = n_pts),
#'     xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
#'   ),
#'   sex = list(
#'     xvirtual    = c(0, 1),
#'     xorg        = sample(c(0, 1), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
#'   )
#' )
#' pp <- gg_partial_varpro(mock_data)
#' plot(pp)
#' plot(pp, type = "parametric")
#'
#' @importFrom ggplot2 .data ggplot aes geom_line geom_boxplot facet_wrap labs
#' @importFrom tidyr pivot_longer all_of
#' @importFrom patchwork wrap_plots
#' @name plot.gg_partial_varpro
#' @export
plot.gg_partial_varpro <- function(x,
                                    type = c("parametric", "nonparametric",
                                             "causal"),
                                    ...) {
  ## C-path: delegate to plot.gg_partial_rfsrc via NextMethod().
  prov <- attr(x, "provenance")
  if (!is.null(prov) && identical(prov$path, "C")) {
    return(NextMethod())
  }

  ## A-path rendering.
  type   <- match.arg(type, several.ok = TRUE)
  ylabel <- .partial_varpro_ylabel(prov)

  gg_cont <- NULL
  if (!is.null(x$continuous) && nrow(x$continuous) > 0) {
    cont_long <- tidyr::pivot_longer(
      x$continuous,
      cols      = tidyr::all_of(type),
      names_to  = "effect_type",
      values_to = "yhat"
    )
    gg_cont <- ggplot2::ggplot(
      cont_long,
      ggplot2::aes(
        x        = .data$variable,
        y        = .data$yhat,
        color    = .data$effect_type,
        linetype = .data$effect_type
      )
    ) +
      ggplot2::geom_line() +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = ylabel,
                    color = "Effect type", linetype = "Effect type")
  }

  gg_cat <- NULL
  if (!is.null(x$categorical) && nrow(x$categorical) > 0) {
    cat_long <- tidyr::pivot_longer(
      x$categorical,
      cols      = tidyr::all_of(type),
      names_to  = "effect_type",
      values_to = "yhat"
    )
    gg_cat <- ggplot2::ggplot(
      cat_long,
      ggplot2::aes(
        x    = factor(.data$variable),
        y    = .data$yhat,
        fill = .data$effect_type
      )
    ) +
      ggplot2::geom_boxplot() +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = ylabel, fill = "Effect type")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    patchwork::wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

## ---------------------------------------------------------------------------
## Internal: build honest y-axis label from provenance.
#' @keywords internal
.partial_varpro_ylabel <- function(prov) {
  if (is.null(prov)) return("Partial Effect")
  scale <- prov$scale %||% "generic"
  switch(scale,
    mortality = "Ensemble mortality (expected events)",
    rmst      = {
      tau <- prov$rmst_tau
      if (!is.null(tau) && !is.na(tau)) {
        sprintf("RMST (\u03c4 = %g)", tau)
      } else {
        "RMST"
      }
    },
    surv      = {
      t <- prov$rmst_tau
      if (!is.null(t) && !is.na(t)) {
        sprintf("Survival probability at t = %g", t)
      } else {
        "Survival probability"
      }
    },
    chf       = {
      t <- prov$rmst_tau
      if (!is.null(t) && !is.na(t)) {
        sprintf("Cumulative hazard at t = %g", t)
      } else {
        "Cumulative hazard"
      }
    },
    "Partial Effect"   # generic / auto-regr / auto-class / unknown
  )
}

# Tests for gg_partial_varpro (Phase 1: A-path extractor)
# C-path tests are in Task 6.

## ── Helper: mock partialpro data ────────────────────────────────────────────
make_mock_vpro_data <- function(n_obs = 30, n_pts = 15) {
  set.seed(42)
  list(
    age = list(
      xvirtual    = seq(30, 80, length.out = n_pts),   # continuous: 15 > 10
      xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
      yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
      yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
      yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
    ),
    sex = list(
      xvirtual    = c(0, 1),                           # categorical: 2 <= 10
      xorg        = sample(c(0, 1), n_obs, replace = TRUE),
      yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
      yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
      yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
    )
  )
}

## ── Input validation ─────────────────────────────────────────────────────────
test_that("gg_partial_varpro: neither part_dta nor object → stop", {
  expect_error(gg_partial_varpro(), regexp = "at least one")
})

test_that("gg_partial_varpro: scale='surv' without object → stop", {
  expect_error(
    gg_partial_varpro(part_dta = make_mock_vpro_data(), scale = "surv"),
    regexp = "requires 'object'"
  )
})

test_that("gg_partial_varpro: scale='chf' without object → stop", {
  expect_error(
    gg_partial_varpro(part_dta = make_mock_vpro_data(), scale = "chf"),
    regexp = "requires 'object'"
  )
})

test_that("gg_partial_varpro: scale='rmst' without time → stop", {
  expect_error(
    gg_partial_varpro(part_dta = make_mock_vpro_data(), scale = "rmst"),
    regexp = "requires 'time'"
  )
})

## ── Class & structure ────────────────────────────────────────────────────────
test_that("gg_partial_varpro returns gg_partial_varpro class", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_s3_class(result, "gg_partial_varpro")
})

test_that("gg_partial_varpro has continuous and categorical elements", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_named(result, c("continuous", "categorical"))
  expect_s3_class(result$continuous, "data.frame")
  expect_s3_class(result$categorical, "data.frame")
})

test_that("gg_partial_varpro continuous has required columns", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_true(all(c("variable", "parametric", "nonparametric", "causal", "name")
                  %in% colnames(result$continuous)))
})

test_that("gg_partial_varpro categorical has required columns", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_true(all(c("parametric", "nonparametric", "causal", "variable", "name")
                  %in% colnames(result$categorical)))
})

test_that("gg_partial_varpro continuous: one row per xvirtual point (age)", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  age_rows <- result$continuous[result$continuous$name == "age", ]
  expect_equal(nrow(age_rows), 15L)
})

test_that("gg_partial_varpro: age is continuous, sex is categorical", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_true("age" %in% result$continuous$name)
  expect_false("age" %in% result$categorical$name)
  expect_true("sex" %in% result$categorical$name)
  expect_false("sex" %in% result$continuous$name)
})

## ── Provenance attribute ─────────────────────────────────────────────────────
test_that("gg_partial_varpro attaches provenance list attr", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  prov <- attr(result, "provenance")
  expect_type(prov, "list")
  expect_true(all(c("family", "scale", "rmst_tau", "xvar.names", "n", "path",
                    "source", "ntree")
                  %in% names(prov)))
})

test_that("gg_partial_varpro: provenance path = 'A' for A-path", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_equal(attr(result, "provenance")$path, "A")
})

## ── Scale resolution ─────────────────────────────────────────────────────────
test_that("gg_partial_varpro: scale='auto' no object → prov scale='generic'", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_equal(attr(result, "provenance")$scale, "generic")
})

test_that("gg_partial_varpro: scale='mortality' recorded in provenance", {
  result <- gg_partial_varpro(make_mock_vpro_data(), scale = "mortality")
  expect_equal(attr(result, "provenance")$scale, "mortality")
})

test_that("gg_partial_varpro: scale='rmst' with time stored in provenance", {
  result <- gg_partial_varpro(make_mock_vpro_data(), scale = "rmst", time = 365)
  prov <- attr(result, "provenance")
  expect_equal(prov$scale, "rmst")
  expect_equal(prov$rmst_tau, 365)
})

## ── nvars + model ────────────────────────────────────────────────────────────
test_that("gg_partial_varpro: nvars=1 processes only first variable", {
  result <- gg_partial_varpro(make_mock_vpro_data(), nvars = 1)
  expect_true("age" %in% result$continuous$name)
  expect_equal(nrow(result$categorical), 0L)
})

test_that("gg_partial_varpro: model arg adds column", {
  result <- gg_partial_varpro(make_mock_vpro_data(), model = "forest1")
  expect_true("model" %in% colnames(result$continuous))
  expect_true("model" %in% colnames(result$categorical))
  expect_equal(unique(result$continuous$model), "forest1")
})

test_that("gg_partial_varpro: no model arg → no model column", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_false("model" %in% colnames(result$continuous))
})

## ── Numeric/structural tests ──────────────────────────────────────────────────
test_that("gg_partial_varpro: continuous parametric equals colMeans(yhat.par)", {
  d      <- make_mock_vpro_data()
  result <- gg_partial_varpro(d)
  expected <- colMeans(d$age$yhat.par, na.rm = TRUE)
  expect_equal(result$continuous$parametric[result$continuous$name == "age"],
               expected)
})

test_that("gg_partial_varpro: categorical sex has n_obs * 2 rows", {
  result  <- gg_partial_varpro(make_mock_vpro_data())
  sex_rows <- result$categorical[result$categorical$name == "sex", ]
  expect_equal(nrow(sex_rows), 30L * 2L)
})

## ── plot.gg_partial_varpro (A-path) ──────────────────────────────────────────
test_that("plot.gg_partial_varpro: continuous-only returns ggplot", {
  result <- gg_partial_varpro(make_mock_vpro_data(), nvars = 1)
  gg <- plot(result)
  expect_s3_class(gg, "ggplot")
})

test_that("plot.gg_partial_varpro: both cont + cat returns ggplot", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  gg <- plot(result)
  expect_s3_class(gg, "ggplot")
})

test_that("plot.gg_partial_varpro: type arg selects effect columns", {
  result <- gg_partial_varpro(make_mock_vpro_data(), nvars = 1)
  gg <- plot(result, type = "parametric")
  expect_s3_class(gg, "ggplot")
})

test_that("plot.gg_partial_varpro: scale='mortality' → honest y-label", {
  result <- gg_partial_varpro(make_mock_vpro_data(), nvars = 1,
                               scale = "mortality")
  gg <- plot(result)
  expect_true(grepl("mortality|Ensemble|expected", gg$labels$y,
                    ignore.case = TRUE))
})

test_that("plot.gg_partial_varpro: scale='rmst' with time → RMST y-label", {
  result <- gg_partial_varpro(make_mock_vpro_data(), nvars = 1,
                               scale = "rmst", time = 365)
  gg <- plot(result)
  expect_true(grepl("RMST|365", gg$labels$y))
})

## ── autoplot / print / summary ───────────────────────────────────────────────
test_that("autoplot.gg_partial_varpro: returns ggplot", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  expect_s3_class(ggplot2::autoplot(result), "ggplot")
})

test_that("print.gg_partial_varpro: returns x invisibly", {
  result <- gg_partial_varpro(make_mock_vpro_data())
  out <- capture.output(ret <- print(result))
  expect_identical(ret, result)
  expect_true(any(grepl("gg_partial_varpro", out)))
})

test_that("summary.gg_partial_varpro: returns summary.gg", {
  result  <- gg_partial_varpro(make_mock_vpro_data())
  s       <- summary(result)
  expect_s3_class(s, "summary.gg")
})

## ── Helper: mock C-path varpro object ────────────────────────────────────────
make_mock_cpath <- function(xvar_subset = NULL) {
  skip_if_not_installed("randomForestSRC")
  set.seed(7)
  veteran_data <- survival::veteran
  rf <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = veteran_data, ntree = 30, importance = FALSE
  )
  xnames <- if (is.null(xvar_subset)) rf$xvar.names else rf$xvar.names[xvar_subset]
  x      <- if (is.null(xvar_subset)) rf$xvar else rf$xvar[, xnames, drop = FALSE]
  vp <- list(rf = rf, family = "surv", xvar.names = xnames,
             x = x, max.tree = 30L)
  class(vp) <- "varpro"
  list(vp = vp, rf = rf)
}

## ── C-path (scale = surv / chf) ──────────────────────────────────────────────
test_that("gg_partial_varpro: C-path returns gg_partial_varpro class", {
  skip_if_not_installed("randomForestSRC")
  m      <- make_mock_cpath()
  result <- suppressWarnings(
    gg_partial_varpro(object = m$vp, scale = "surv",
                      time = median(m$rf$time.interest))
  )
  expect_s3_class(result, "gg_partial_varpro")
})

test_that("gg_partial_varpro: C-path provenance path='C'", {
  skip_if_not_installed("randomForestSRC")
  m      <- make_mock_cpath()
  result <- suppressWarnings(
    gg_partial_varpro(object = m$vp, scale = "surv",
                      time = median(m$rf$time.interest))
  )
  expect_equal(attr(result, "provenance")$path, "C")
  expect_equal(attr(result, "provenance")$scale, "surv")
})

test_that("gg_partial_varpro: plot C-path returns ggplot", {
  skip_if_not_installed("randomForestSRC")
  m      <- make_mock_cpath(xvar_subset = 1L)
  result <- suppressWarnings(
    gg_partial_varpro(object = m$vp, scale = "surv",
                      time = median(m$rf$time.interest))
  )
  gg <- plot(result)
  expect_s3_class(gg, "ggplot")
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' ROC plot generic function for a \code{\link{gg_roc}} object.
#'
#' @param x A \code{\link{gg_roc}} object, or a raw
#'   \code{\link[randomForestSRC]{rfsrc}} classification forest or
#'   \code{\link[randomForest]{randomForest}} classification object.  When a
#'   forest is supplied, \code{\link{gg_roc}} is called automatically.
#' @param which_outcome Integer; for multi-class problems, the index of the
#'   class whose ROC curve should be plotted.  When \code{NULL} (default) and
#'   the forest has more than two classes, ROC curves for all classes are
#'   overlaid in a single plot.  For binary forests \code{NULL} defaults to
#'   class index 2.
#' @param ... Additional arguments forwarded to \code{\link{gg_roc}} when
#'   \code{x} is a raw forest object (e.g. \code{oob = FALSE}).
#'
#' @return A \code{ggplot} object.  The x-axis shows 1 - Specificity (FPR)
#'   and the y-axis shows Sensitivity (TPR).  A dashed red diagonal reference
#'   line marks the random-classifier baseline.  The AUC value is annotated
#'   on the plot for single-class curves.  Multi-class plots colour and style
#'   each class curve distinctly.
#'
#' @seealso \code{\link{gg_roc}} \code{\link{calc_roc}} \code{\link{calc_auc}}
#'   \code{\link[randomForestSRC]{rfsrc}}
#'   \code{\link[randomForest]{randomForest}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
#' Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' # Build a small classification forest (ntree=50 keeps example fast)
#' set.seed(42)
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' # ROC for setosa (outcome index 1)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor (outcome index 2)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica (outcome index 3)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' # Plot all three ROC curves in one call by iterating over outcome indices
#' n_cls <- ncol(rfsrc_iris$predicted)
#' for (i in seq_len(n_cls)) print(plot(gg_roc(rfsrc_iris, which_outcome = i)))
#'
#' @export
plot.gg_roc <- function(x, which_outcome = NULL, ...) {
  gg_dta <- x

  ## ---- Accept a raw rfsrc or randomForest object -----------------------
  # If we call this with a forest object instead of a gg_roc object,
  # automatically compute the ROC data for the appropriate class(es).
  if (inherits(gg_dta, "rfsrc")) {
    if (inherits(gg_dta, "class")) {
      # Determine the number of outcome classes
      crv <- dim(gg_dta$predicted)[2]

      if (crv > 2 && is.null(which_outcome)) {
        # Multi-class: compute ROC for every class in parallel
        gg_dta <- mclapply(seq_len(crv), function(ind) {
          gg_roc(gg_dta, which_outcome = ind, ...)
        })
      } else {
        # Binary (or user specified a class): default to second column
        if (is.null(which_outcome)) {
          which_outcome <- 2
        }
        gg_dta <- gg_roc(gg_dta, which_outcome, ...)
      }
    } else {
      stop("gg_roc expects a classification randomForest.")
    }
  } else if (inherits(gg_dta, "randomForest")) {
    if (gg_dta$type == "classification") {
      # Determine the number of outcome classes
      crv <- length(levels(gg_dta$predicted))
      if (crv > 2 && is.null(which_outcome)) {
        # Multi-class: compute ROC for every class in parallel
        gg_dta <- parallel::mclapply(seq_len(crv), function(ind) {
          gg_roc(gg_dta, which_outcome = ind, ...)
        })
      } else {
        # Binary (or user specified a class): default to second column
        if (is.null(which_outcome)) {
          which_outcome <- 2
        }
        gg_dta <- gg_roc(gg_dta, which_outcome, ...)
      }
    }
  }

  ## ---- Single-class ROC plot ------------------------------------------
  if (inherits(gg_dta, "gg_roc")) {
    # Sort by specificity so the ROC curve is drawn left-to-right
    gg_dta <- gg_dta[order(gg_dta$spec), ]
    # False positive rate = 1 - specificity
    gg_dta$fpr <- 1 - gg_dta$spec
    auc <- calc_auc(gg_dta)

    gg_plt <- ggplot2::ggplot(data = gg_dta) +
      ggplot2::geom_line(ggplot2::aes(x = .data$fpr, y = .data$sens)) +
      ggplot2::labs(x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)") +
      # Reference diagonal for a random classifier
      ggplot2::geom_abline(
        slope = 1,
        intercept = 0,
        col = "red",
        linetype = 2,
        linewidth = .5
      ) +
      ggplot2::coord_fixed()

    # Annotate the plot with the computed AUC value
    gg_plt <- gg_plt +
      ggplot2::annotate(
        x = .5,
        y = .2,
        geom = "text",
        label = paste("AUC = ", round(auc, digits = 3), sep = ""),
        hjust = 0
      )
  } else {
    ## ---- Multi-class ROC plot (list of gg_roc objects) ----------------
    # Sort each class's data by specificity
    gg_dta <- parallel::mclapply(gg_dta, function(st) {
      st <- st[order(st$spec), ]
      st
    })
    # Compute FPR for each class
    gg_dta <- parallel::mclapply(gg_dta, function(st) {
      st$fpr <- 1 - st$spec
      st
    })
    # Tag each subset with its outcome index for colour/linetype mapping
    gg_dta <- parallel::mclapply(seq_along(gg_dta), function(ind) {
      gg_dta[[ind]]$outcome <- ind
      gg_dta[[ind]]
    })

    # Compute AUC for each class
    auc <- parallel::mclapply(gg_dta, function(st) {
      calc_auc(st)
    })

    # Combine all classes into a single data frame for faceting/colouring
    o_dta <- do.call(rbind, gg_dta)
    o_dta$outcome <- factor(o_dta$outcome)

    gg_plt <- ggplot2::ggplot(data = o_dta) +
      ggplot2::geom_line(ggplot2::aes(
        x = .data$fpr,
        y = .data$sens,
        linetype = .data$outcome,
        col = .data$outcome
      )) +
      ggplot2::labs(x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)") +
      # Reference diagonal for a random classifier
      ggplot2::geom_abline(
        slope = 1,
        intercept = 0,
        col = "red",
        linetype = 2,
        linewidth = .5
      ) +
      ggplot2::coord_fixed()

    # Multi-class: do not annotate a single AUC value — each class has its own.
  }
  return(gg_plt)
}

##=============================================================================
#' Split varpro partial dependence data into continuous or categorical datasets
#' (deprecated)
#'
#' \emph{Deprecated.} \code{gg_partialpro()} has been superseded by
#' \code{\link{gg_partial_varpro}()} and is now a thin alias for it.  It will
#' be removed in the release after \pkg{ggRandomForests} v3.0.0; use
#' \code{\link{gg_partial_varpro}()} directly.
#'
#' @param part_dta Passed to \code{\link{gg_partial_varpro}}.
#' @param object Passed to \code{\link{gg_partial_varpro}}.
#' @param scale Passed to \code{\link{gg_partial_varpro}}.
#' @param time Passed to \code{\link{gg_partial_varpro}}.
#' @param nvars Passed to \code{\link{gg_partial_varpro}}.
#' @param cat_limit Passed to \code{\link{gg_partial_varpro}}.
#' @param model Passed to \code{\link{gg_partial_varpro}}.
#'
#' @return A \code{gg_partial_varpro} object (see
#'   \code{\link{gg_partial_varpro}}).
#'
#' @seealso \code{\link{gg_partial_varpro}}
#'
#' @rdname gg_partial_varpro
#' @export
gg_partialpro <- function(part_dta,
                          object    = NULL,
                          scale     = c("auto", "rmst", "mortality",
                                        "surv", "chf"),
                          time      = NULL,
                          nvars     = NULL,
                          cat_limit = 10,
                          model     = NULL) {
  .Deprecated(
    new     = "gg_partial_varpro",
    package = "ggRandomForests",
    msg     = paste0("'gg_partialpro()' is deprecated. ",
                     "Use 'gg_partial_varpro()' instead.")
  )
  gg_partial_varpro(part_dta  = part_dta,
                    object    = object,
                    scale     = scale,
                    time      = time,
                    nvars     = nvars,
                    cat_limit = cat_limit,
                    model     = model)
}

##=============================================================================
#' Individual (local) variable importance from a varPro fit
#'
#' Tidy wrapper around [varPro::ivarpro()] for the regression or
#' classification family. Returns one row per (observation, variable)
#' pair where the local-importance cell is non-NA; classification adds
#' a `class` column. `which_obs` collapses to a per-observation
#' profile; `which_class` collapses to a single class. Optional
#' `ivarpro_fit` argument lets callers cache the expensive
#' `ivarpro()` call.
#'
#' @section What this is doing:
#' The varPro framework builds importance from release rules: for a given
#' rule region, it compares a local estimator inside that region to what
#' the estimator becomes after the constraint on the tested variable is
#' removed ("released"). That contrast is summed over many rules and trees
#' to get a global z-score: the quantity [gg_varpro()] shows. What
#' `ivarpro()` adds is a per-observation view of the same mechanism.
#'
#' Concretely: `ivarpro()` walks the forest's rules and, for each
#' (observation, variable) pair, computes a scaled per-rule contribution
#' to predicting that observation. Per-rule LOO removes the observation
#' from its own rule before scoring, so the contribution is not inflated
#' by the observation having helped define the region. Per-region scaling
#' (`scale = "local"`, default) standardises the contribution by the
#' rule's local response standard deviation so values are comparable
#' across rules of different size. Aggregating those per-rule scores into
#' one number per (obs, variable) pair gives the `local_imp` cell.
#'
#' No permutation, no synthetic data: the contrast is always between real
#' subsets of the observed data, defined by the forest's own rules. This
#' is the same no-synthetic-features property that distinguishes
#' [gg_varpro()] from [gg_vimp()]'s Breiman-Cutler permutation importance.
#'
#' @section What `local_imp` actually is (pedantic):
#' `local_imp[i, v]` is the **scaled aggregated rule contribution** of
#' variable `v` to predicting observation `i`, NOT a permutation
#' importance and NOT a SHAP value. **Sign carries direction** of the
#' local response shift inside the rule's region. **Magnitude is on
#' the response scale** when `scale = "global"`, or unit-free when
#' `scale = "local"` (the default). The matrix is **heavily sparse** -
#' an observation contributes only to rules that retain it as OOB; on
#' real data, per-variable NA fractions of 50-95% are common.
#' Comparison with `gg_varpro()` (aggregate split-strength) and
#' `gg_beta_varpro()` (per-rule lasso beta) is diagnostic: a variable
#' that's important globally but has low per-observation contribution
#' for a specific case is interesting; the inverse - high local but
#' low global - flags a regime-specific signal.
#'
#' @section What's in the output:
#' Long-format tidy frame. Regression has columns `obs`, `variable`,
#' `local_imp`, `selected`. Classification adds a `class` column
#' (factor in response-level order). `variable` is a factor whose
#' levels are set by `mean(|local_imp|)` descending across all rows;
#' for classification that aggregate is across all (obs, class) so
#' every facet / panel shows variables in the same row order. NA
#' cells are filtered out - the source matrix is sparse, and the
#' tidy frame only carries the cells where local importance is
#' defined.
#'
#' Provenance attribute carries `source`, `family`, `ntree`, `cutoff`
#' (named numeric vector - length 1 named `"regr"` for regression,
#' length K named with class levels for classification),
#' `cutoff_default`, `use.loo`, `scale`, `n_train`, `n_obs`, `n_var`,
#' `precomputed`, `xvar.names`, `class_levels` (classification only),
#' `which_obs`, `which_class`.
#'
#' @section What you use this for:
#' Per-observation interpretation ("which variables drive *this*
#' prediction?"), variable-selection diagnostics via the aggregate
#' distribution view, and side-by-side comparison against
#' [gg_varpro()] / [gg_beta_varpro()] to spot variables that matter
#' locally but not globally (or vice versa).
#'
#' @section Caching:
#' `ivarpro()` is **the most expensive call in varPro** (per-rule
#' leave-one-out + per-region scaling, often minutes on real data).
#' Compute it once and reuse:
#'
#' ```r
#' v   <- varPro::varpro(medv ~ ., data = Boston, ntree = 200)
#' iv  <- varPro::ivarpro(v, scale = "local")              # expensive, once
#' gg_aggregate <- gg_ivarpro(v, ivarpro_fit = iv)          # cheap
#' gg_case1     <- gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L)
#' ```
#'
#' Provenance carries `precomputed = TRUE` when `ivarpro_fit` was supplied.
#'
#' @section Classification:
#' For a classification fit, `ivarpro()` returns a list of K matrices
#' (one per class) for multi-class, or a flat data.frame for binary
#' (positive-class importances only - the wrapper normalises this to
#' a single-element list under the last factor level). The wrapper
#' stacks per-class frames into a long-format frame with a `class`
#' column. `which_class = NULL` returns all classes (binary defaults
#' to the last factor level, the positive-class convention used by
#' `glm` and `gg_roc`); `which_class = "<name>"` filters to a single
#' class. `cutoff` polymorphism mirrors [gg_beta_varpro()] - `NULL`
#' is per-class mean(|local_imp|), a scalar broadcasts, a named
#' numeric vector overrides per class with fallback to that class's
#' mean.
#'
#' @section Reproducibility:
#' Byte-for-byte agreement between cached (`ivarpro_fit = iv`) and
#' uncached (`ivarpro_fit = NULL`) outputs requires reusing the same
#' `ivarpro()` result. `set.seed()` alone is not sufficient because
#' per-rule LOO subsampling can drift across separate calls. Reuse
#' `ivarpro_fit` when reproducibility matters.
#'
#' @note Multivariate regression (`regr+`) and survival families are
#'   out of scope for this release. The non-regression / non-class
#'   path errors with a message naming v3.1.0 as the tracker.
#'
#' @param object A `varpro` fit from [varPro::varpro()] (regression or
#'   classification family).
#' @param ... Forwarded to [varPro::ivarpro()] when `ivarpro_fit = NULL`;
#'   ignored otherwise (with a warning). Documented forwardables:
#'   `adaptive`, `cut`, `cut.max`, `ncut`, `nmin`, `nmax`, `noise.na`,
#'   `max.rules.tree`, `max.tree`, `use.loo`, `use.abs`, `scale`.
#' @param which_obs Optional integer scalar - 1-based row index into the
#'   training data. `NULL` (default) returns the aggregate view.
#' @param which_class Optional response level name. `NULL` default on a
#'   binary classification fit resolves to the last factor level
#'   (positive-class convention). Ignored with a warning on regression
#'   fits.
#' @param cutoff Selection threshold on `|local_imp|`. `NULL` (default)
#'   resolves to the per-class `mean(|local_imp|)` (or per-frame mean
#'   for regression). A numeric scalar broadcasts. A named numeric
#'   vector (names a subset of class levels) overrides per class with
#'   fallback to the per-class mean for missing names.
#' @param ivarpro_fit Optional pre-computed [varPro::ivarpro()] result
#'   for the same `object`. Shape-validated.
#'
#' @return A `data.frame` of class `c("gg_ivarpro", "data.frame")`.
#'   Regression: columns `obs / variable / local_imp / selected`.
#'   Classification: long-format with an extra `class` column.
#'   `variable` is a factor whose levels are set by
#'   `mean(|local_imp|)` descending across all rows (the unified
#'   ranking axis shared across facets / panels).
#'
#' @seealso [gg_varpro()], [gg_vimp()], [gg_beta_varpro()], [varPro::ivarpro()].
#'
#' @examples
#' \donttest{
#' if (requireNamespace("varPro", quietly = TRUE) &&
#'     requireNamespace("MASS", quietly = TRUE)) {
#'   set.seed(1)
#'   v <- varPro::varpro(medv ~ ., data = MASS::Boston, ntree = 50)
#'   iv <- varPro::ivarpro(v)
#'   gg <- gg_ivarpro(v, ivarpro_fit = iv)
#'   plot(gg)
#' }
#' }
#'
#' @export
gg_ivarpro <- function(object, ..., which_obs = NULL, which_class = NULL,
                       cutoff = NULL, ivarpro_fit = NULL) {
  UseMethod("gg_ivarpro", object)
}

#' @export
gg_ivarpro.varpro <- function(object, ..., which_obs = NULL,
                              which_class = NULL, cutoff = NULL,
                              ivarpro_fit = NULL) {
  if (!inherits(object, "varpro")) {
    stop("gg_ivarpro: expected a 'varpro' object from varPro::varpro().",
         call. = FALSE)
  }
  fam <- object$family
  if (!fam %in% c("regr", "class")) {
    stop(sprintf(
      paste0("gg_ivarpro currently supports varpro regression and ",
             "classification forests only; got family = '%s'. regr+ and ",
             "survival are tracked for v3.1.0 (see NEWS)."),
      fam
    ), call. = FALSE)
  }

  # Capture forwardable args before dispatch (the dots don't pass through
  # to the internal frames).
  dots_use_loo <- if (is.null(ivarpro_fit)) isTRUE(list(...)$use.loo) else NA
  dots_scale   <- if (is.null(ivarpro_fit)) {
    s <- list(...)$scale
    if (is.null(s)) NA_character_ else s[1L]
  } else {
    NA_character_
  }

  iv <- .resolve_ivarpro_fit(ivarpro_fit, object, fam, list(...))

  # Warn on which_class with regression
  if (fam == "regr" && !is.null(which_class)) {
    warning("gg_ivarpro: which_class ignored for regression family.",
            call. = FALSE)
    which_class <- NULL
  }

  n_train <- if (fam == "regr") nrow(iv) else nrow(iv[[1L]])
  which_obs <- .validate_which_obs(which_obs, n_train)

  if (fam == "regr") {
    return(.gg_ivarpro_regr(object, iv, cutoff, which_obs, ivarpro_fit,
                            dots_use_loo, dots_scale))
  }
  .gg_ivarpro_class(object, iv, cutoff, which_obs, which_class, ivarpro_fit,
                    dots_use_loo, dots_scale)
}

#' @noRd
.resolve_ivarpro_fit <- function(ivarpro_fit, object, fam, dots) {
  if (is.null(ivarpro_fit)) {
    iv <- do.call(varPro::ivarpro, c(list(object), dots))
  } else {
    .validate_ivarpro_fit(ivarpro_fit, object, fam)
    if (length(dots) > 0L) {
      warning(
        "gg_ivarpro: arguments in '...' ignored because ivarpro_fit is supplied.",
        call. = FALSE
      )
    }
    iv <- ivarpro_fit
  }
  # varPro::ivarpro() returns a flat data.frame for binary classification
  # (positive-class importances only). Normalise to the spec's list-of-K
  # shape by wrapping under the last factor level (positive class).
  if (fam == "class" && is.data.frame(iv)) {
    cls_levels <- .ivarpro_class_levels(object)
    iv <- stats::setNames(list(iv), cls_levels[length(cls_levels)])
  }
  iv
}

#' @noRd
.validate_which_obs <- function(which_obs, n_train) {
  if (is.null(which_obs)) return(NULL)
  if (!is.numeric(which_obs) || length(which_obs) != 1L ||
      which_obs != as.integer(which_obs) ||
      which_obs < 1L || which_obs > n_train) {
    stop(sprintf(
      "gg_ivarpro: which_obs = %s is out of range. Valid range: 1..%d.",
      format(which_obs), n_train
    ), call. = FALSE)
  }
  as.integer(which_obs)
}

#' @noRd
.validate_ivarpro_fit <- function(ivarpro_fit, object, fam) {
  xvars <- object$xvar.org.names
  n_train <- nrow(object$x)

  if (fam == "regr") {
    if (!is.data.frame(ivarpro_fit)) {
      stop("gg_ivarpro: ivarpro_fit does not look like a varPro::ivarpro() ",
           "result (expected a data.frame for the regression family).",
           call. = FALSE)
    }
    # varPro::ivarpro() may omit columns for variables it deems
    # uninformative; only require at least one xvar column to be present.
    if (length(intersect(xvars, names(ivarpro_fit))) == 0L) {
      stop("gg_ivarpro: ivarpro_fit has no columns matching the predictors.",
           call. = FALSE)
    }
    if (nrow(ivarpro_fit) != n_train) {
      stop(sprintf(
        paste0("gg_ivarpro: ivarpro_fit has %d rows but object trained ",
               "on %d observations."),
        nrow(ivarpro_fit), n_train), call. = FALSE)
    }
  } else {
    cls <- .ivarpro_class_levels(object)
    # varPro::ivarpro() returns a flat data.frame for binary classification
    # (single positive-class frame), and a named list of K frames for
    # multi-class. Accept either shape here.
    if (is.data.frame(ivarpro_fit)) {
      if (length(cls) != 2L) {
        stop("gg_ivarpro: ivarpro_fit does not look like a varPro::ivarpro() ",
             "result (expected a list of data.frames, one per class).",
             call. = FALSE)
      }
      # varPro::ivarpro() may omit columns for variables it deems
      # uninformative; only require at least one xvar column to be present.
      if (length(intersect(xvars, names(ivarpro_fit))) == 0L) {
        stop("gg_ivarpro: ivarpro_fit has no columns matching the predictors.",
             call. = FALSE)
      }
      if (nrow(ivarpro_fit) != n_train) {
        stop(sprintf(
          paste0("gg_ivarpro: ivarpro_fit has %d rows but object trained ",
                 "on %d observations."),
          nrow(ivarpro_fit), n_train), call. = FALSE)
      }
      return(invisible(NULL))
    }
    if (!is.list(ivarpro_fit)) {
      stop("gg_ivarpro: ivarpro_fit does not look like a varPro::ivarpro() ",
           "result (expected a list of data.frames, one per class).",
           call. = FALSE)
    }
    if (!identical(sort(names(ivarpro_fit)), sort(cls))) {
      stop("gg_ivarpro: ivarpro_fit class names mismatch. Expected: ",
           paste(cls, collapse = ", "), ". Got: ",
           paste(names(ivarpro_fit), collapse = ", "), ".",
           call. = FALSE)
    }
    # Each per-class element must be a data.frame with the right row count
    # and at least one predictor column.
    for (k in names(ivarpro_fit)) {
      el <- ivarpro_fit[[k]]
      if (!is.data.frame(el)) {
        stop(sprintf(
          paste0("gg_ivarpro: ivarpro_fit[['%s']] is not a data.frame ",
                 "(got class '%s')."),
          k, class(el)[1L]), call. = FALSE)
      }
      if (nrow(el) != n_train) {
        stop(sprintf(
          paste0("gg_ivarpro: ivarpro_fit[['%s']] has %d rows but object ",
                 "trained on %d observations."),
          k, nrow(el), n_train), call. = FALSE)
      }
      if (length(intersect(xvars, names(el))) == 0L) {
        stop(sprintf(
          "gg_ivarpro: ivarpro_fit[['%s']] has no columns matching the predictors.",
          k), call. = FALSE)
      }
    }
  }
  invisible(NULL)
}

#' @noRd
.ivarpro_class_levels <- function(object) {
  # Same priority chain as gg_beta_varpro classification - y.org first because
  # varPro internally recodes binary y to 0/1.
  if (!is.null(object$y.org) && is.factor(object$y.org)) return(levels(object$y.org))
  if (is.factor(object$y)) return(levels(object$y))
  if (!is.null(attr(object$y, "levels"))) return(attr(object$y, "levels"))
  sort(unique(as.character(object$y)))
}

#' @noRd
.ivarpro_long <- function(mat, xvars) {
  # mat: nrow = n_train, ncol = p (xvars). Returns long-format
  # data.frame (obs, variable, local_imp) with NA cells dropped.
  rows <- list()
  for (j in seq_along(xvars)) {
    col_vals <- mat[[xvars[j]]]
    keep <- which(is.finite(col_vals))
    if (length(keep) == 0L) next
    rows[[j]] <- data.frame(
      obs       = keep,
      variable  = xvars[j],
      local_imp = col_vals[keep],
      stringsAsFactors = FALSE
    )
  }
  if (length(rows) == 0L) {
    return(data.frame(obs = integer(0), variable = character(0),
                      local_imp = numeric(0), stringsAsFactors = FALSE))
  }
  do.call(rbind, rows)
}

#' @noRd
.gg_ivarpro_regr <- function(object, iv, cutoff, which_obs, ivarpro_fit,
                             use_loo, scale_arg) {
  xvars <- object$xvar.org.names
  long  <- .ivarpro_long(iv, xvars)

  # Factor levels are REVERSED descending mean(|local_imp|) so the
  # most-important variable lands at the TOP after coord_flip, matching
  # the gg_vimp / gg_varpro convention.
  agg <- tapply(abs(long$local_imp), long$variable, mean, na.rm = TRUE)
  ord_names <- names(sort(agg, decreasing = TRUE))
  long$variable <- factor(long$variable, levels = rev(ord_names))

  resolved_cutoff <- if (is.null(cutoff)) mean(abs(long$local_imp)) else as.numeric(cutoff)
  long$selected <- abs(long$local_imp) >= resolved_cutoff

  if (!is.null(which_obs)) long <- long[long$obs == which_obs, , drop = FALSE]
  ## Rows stay most-important-first (obs ascending within variable); the
  ## reversed factor levels only drive the plot's vertical order.
  long <- long[order(-as.integer(long$variable), long$obs), , drop = FALSE]
  rownames(long) <- NULL

  class(long) <- c("gg_ivarpro", "data.frame")
  attr(long, "provenance") <- list(
    source         = "varPro::ivarpro",
    family         = "regr",
    ntree          = if (!is.null(object$ntree)) as.integer(object$ntree) else NA_integer_,
    cutoff         = stats::setNames(resolved_cutoff, "regr"),
    cutoff_default = is.null(cutoff),
    use.loo        = use_loo,
    scale          = scale_arg,
    n_train        = nrow(object$x),
    n_obs          = length(unique(long$obs)),
    n_var          = nlevels(long$variable),
    precomputed    = !is.null(ivarpro_fit),
    xvar.names     = xvars,
    which_obs      = which_obs,
    which_class    = NULL
  )
  long
}

#' @noRd
.gg_ivarpro_class <- function(object, iv, cutoff, which_obs, which_class,
                              ivarpro_fit, use_loo, scale_arg) {
  cls   <- .ivarpro_class_levels(object)
  xvars <- object$xvar.org.names
  n_cls <- length(cls)

  # Per-class long-format frames
  per_class_long <- lapply(cls, function(c_name) {
    mat <- iv[[c_name]]
    if (is.null(mat)) return(NULL)
    df <- .ivarpro_long(mat, xvars)
    df$class <- c_name
    df
  })
  long <- do.call(rbind, per_class_long)

  if (nrow(long) == 0L) {
    return(.gg_ivarpro_empty(object, "class", which_obs, which_class,
                             ivarpro_fit, cutoff))
  }

  # Unified factor-level ordering across all (obs, class), REVERSED so the
  # most-important variable lands at the TOP after coord_flip; shared
  # across every class facet for alignment.
  agg <- tapply(abs(long$local_imp), long$variable, mean, na.rm = TRUE)
  ord_names <- names(sort(agg, decreasing = TRUE))
  long$variable <- factor(long$variable, levels = rev(ord_names))

  # Validate which_class
  if (!is.null(which_class)) {
    if (!which_class %in% cls) {
      stop(sprintf(
        "gg_ivarpro: which_class = '%s' is not a level of the response. Levels: %s.",
        which_class, paste(cls, collapse = ", ")
      ), call. = FALSE)
    }
  } else if (n_cls == 2L) {
    which_class <- cls[n_cls]   # binary default = last factor level
  }

  # Per-class mean(|local_imp|) for cutoff resolution
  per_class_mean <- vapply(cls, function(c_name) {
    vals <- long$local_imp[long$class == c_name]
    if (length(vals) == 0L) NA_real_ else mean(abs(vals))
  }, numeric(1))
  names(per_class_mean) <- cls

  resolved_cutoff <- .resolve_class_cutoff(cutoff, per_class_mean, cls)

  long$selected <- mapply(function(li, cl) abs(li) >= resolved_cutoff[[cl]],
                          long$local_imp, long$class)

  if (!is.null(which_class)) long <- long[long$class == which_class, , drop = FALSE]
  if (!is.null(which_obs))   long <- long[long$obs   == which_obs,   , drop = FALSE]

  long$class <- factor(long$class, levels = cls)
  ## Within each class, rows stay most-important-first (obs ascending); the
  ## reversed variable factor levels only drive the plot's vertical order.
  long <- long[order(long$class, -as.integer(long$variable), long$obs), ,
               drop = FALSE]
  rownames(long) <- NULL

  class(long) <- c("gg_ivarpro", "data.frame")
  attr(long, "provenance") <- list(
    source         = "varPro::ivarpro",
    family         = "class",
    ntree          = if (!is.null(object$ntree)) as.integer(object$ntree) else NA_integer_,
    cutoff         = resolved_cutoff,
    cutoff_default = is.null(cutoff),
    use.loo        = use_loo,
    scale          = scale_arg,
    n_train        = nrow(object$x),
    n_obs          = length(unique(long$obs)),
    n_var          = nlevels(long$variable),
    precomputed    = !is.null(ivarpro_fit),
    xvar.names     = xvars,
    class_levels   = cls,
    which_obs      = which_obs,
    which_class    = which_class
  )
  long
}

#' @noRd
.gg_ivarpro_empty <- function(object, fam, which_obs, which_class,
                              ivarpro_fit, cutoff) {
  cols <- if (fam == "class") {
    list(obs = integer(0), variable = factor(character(0)),
         class = factor(character(0)),
         local_imp = numeric(0), selected = logical(0))
  } else {
    list(obs = integer(0), variable = factor(character(0)),
         local_imp = numeric(0), selected = logical(0))
  }
  base <- as.data.frame(cols, stringsAsFactors = FALSE)
  class(base) <- c("gg_ivarpro", "data.frame")
  prov <- if (fam == "class") {
    cls <- .ivarpro_class_levels(object)
    list(source = "varPro::ivarpro", family = "class",
         cutoff = stats::setNames(rep(NA_real_, length(cls)), cls),
         cutoff_default = is.null(cutoff),
         n_train = nrow(object$x), n_obs = 0L,
         precomputed = !is.null(ivarpro_fit),
         class_levels = cls,
         which_obs = which_obs, which_class = which_class)
  } else {
    list(source = "varPro::ivarpro", family = "regr",
         cutoff = stats::setNames(NA_real_, "regr"),
         cutoff_default = is.null(cutoff),
         n_train = nrow(object$x), n_obs = 0L,
         precomputed = !is.null(ivarpro_fit),
         which_obs = which_obs, which_class = NULL)
  }
  attr(base, "provenance") <- prov
  base
}

#' @importFrom ggplot2 autoplot
#' @export
autoplot.gg_ivarpro <- function(object, ...) {
  plot(object, ...)
}

####**********************************************************************
####  gg_isopro: tidy extractor for varPro::isopro anomaly scores.
####
####  varPro::isopro returns a list with $howbad (per-observation anomaly
####  score in [0,1]) and $case.depth (average isolation depth, lower =
####  more anomalous). gg_isopro() reshapes these into a tidy data.frame
####  the plot/print/summary methods can consume.
####**********************************************************************

#' Tidy data from a varPro isolation-forest fit
#'
#' Pulls per-observation anomaly scores out of a \code{\link[varPro]{isopro}}
#' fit so you can plot them, sort them, or write them to disk without having
#' to know the internal shape of the fit.
#'
#' @section What isopro is doing:
#' An isolation forest (Liu, Ting and Zhou 2008) is a random forest grown
#' on very small subsamples of the data and asked to split until each
#' observation lands in its own terminal node. The intuition is geometric:
#' a typical observation sits in the dense middle of the feature cloud and
#' takes many splits to isolate, while an unusual observation sits out
#' near an edge and gets cut off after only a few. So \strong{the depth at
#' which an observation is isolated is a proxy for how typical it is}:
#' shallow depth means anomalous, deep depth means ordinary. Average a
#' single observation's depth across many trees and the noise washes out,
#' leaving a stable per-observation rank.
#'
#' \code{\link[varPro]{isopro}} supports three flavours of isolation
#' forest, which differ in how the splits are chosen:
#' \describe{
#'   \item{\code{"rnd"}}{The original Liu/Ting/Zhou method: each tree node
#'     picks a variable at random and a split point uniformly at random
#'     in the variable's range. Fast, no model, surprisingly effective.}
#'   \item{\code{"unsupv"}}{Unsupervised splitting from
#'     \code{randomForestSRC}: splits are chosen to separate the data
#'     along the directions of highest variance. More structured than
#'     \code{"rnd"}; sometimes more accurate, especially when the
#'     anomalies follow a coherent direction.}
#'   \item{\code{"auto"}}{An auto-encoder formulation that grows a
#'     multivariate forest predicting each feature from the others. Most
#'     expressive, slowest, best suited to low-dimensional data.}
#' }
#' No method is universally best. The varPro authors recommend trying at
#' least two and comparing the score distributions; the plot method here
#' colours per-method curves automatically when you stack the results.
#'
#' @section What's in the output:
#' The fit gives back two parallel per-observation vectors:
#' \code{case.depth} is the raw mean isolation depth (units of "splits",
#' lower = more anomalous) and \code{howbad} is the same information
#' transformed onto a \code{[0, 1]} scale via the empirical CDF of
#' \code{case.depth} (higher = more anomalous). Both columns are kept so
#' you can plot in either space and have the raw depth on hand for
#' diagnostics; \code{howbad} is the canonical score and is what the plot
#' method uses by default.
#'
#' @section What you use this for:
#' This is screening, not inference. Reach for it when you want to:
#' \itemize{
#'   \item flag observations that may be data-entry errors, out-of-range
#'     measurements, or distinct subpopulations before fitting a primary
#'     model;
#'   \item check whether a held-out cohort sits inside the training
#'     distribution before scoring with a model trained elsewhere;
#'   \item give the analyst a ranked list of "look at these first" cases
#'     for a manual review;
#'   \item score a held-out cohort or a fresh batch of incoming data
#'     against a fitted model and compare the test scores to the training
#'     distribution.
#' }
#' The score is a \emph{rank}, not a probability of being an outlier: two
#' observations with \code{howbad = 0.92} are both unusual, not "92\%
#' likely to be anomalous". Pick a cutoff by looking at where the elbow
#' rises; \code{\link{plot.gg_isopro}} can annotate either a score
#' (\code{threshold}) or a top-percent (\code{top_n_pct}) for you.
#'
#' @section Scoring new data:
#' Pass a \code{data.frame} as \code{newdata} and the extractor calls
#' \code{\link[varPro]{predict.isopro}} twice: once with
#' \code{quantiles = FALSE} to get the raw mean case depth per row, and once
#' with \code{quantiles = TRUE} to get the per-row quantile of that depth
#' against the training-data depth distribution.
#'
#' varPro's \code{predict.isopro} returns quantiles where \emph{smaller is
#' more anomalous}, which is the opposite polarity of the wrapper's
#' \code{howbad} (where \emph{higher} is more anomalous). The wrapper
#' exposes both conventions so nothing is hidden:
#' \itemize{
#'   \item \code{case.depth} carries varPro's native polarity, \emph{lower
#'     = more anomalous}. This is the unmodified output of
#'     \code{predict(object, newdata, quantiles = FALSE)}. Use it to
#'     cross-reference against raw varPro output.
#'   \item \code{howbad} is the flipped, wrapper-convention version. The
#'     relationship is \code{howbad = 1 - predict(object, newdata, quantiles = TRUE)}.
#' }
#'
#' To overlay training and test scores in one plot, bind the two extractor
#' calls with a \code{method} label column (the same column
#' \code{\link{plot.gg_isopro}} uses to colour rnd / unsupv / auto
#' comparisons):
#'
#' \preformatted{
#' gg_train <- gg_isopro(fit)
#' gg_test  <- gg_isopro(fit, newdata = test_df)
#' gg_both  <- rbind(cbind(gg_train, method = "train"),
#'                   cbind(gg_test,  method = "test"))
#' class(gg_both) <- c("gg_isopro", "data.frame")
#' plot(gg_both)
#' }
#'
#' @param object An \code{isopro} fit returned by
#'   \code{\link[varPro]{isopro}}.
#' @param ... Currently unused. Present before \code{newdata} so that
#'   \code{newdata} is only matched by name, preserving backward
#'   compatibility with callers of the PR #94 signature
#'   \code{gg_isopro(object, ...)}.
#' @param newdata Optional \code{data.frame} of new observations to score
#'   against the fit. Must be passed by name. When \code{NULL} (default)
#'   the extractor returns the in-sample tidy frame from the fit's stored
#'   \code{$case.depth} and \code{$howbad}. When supplied, each row is
#'   scored via \code{\link[varPro]{predict.isopro}} and the same tidy
#'   shape is returned for the test data.
#'
#' @return A \code{data.frame} of class \code{c("gg_isopro", "data.frame")},
#'   one row per observation. Columns:
#'   \describe{
#'     \item{obs}{Integer; observation index \code{1..n}, in the same
#'       order as the rows of the data passed to
#'       \code{\link[varPro]{isopro}}.}
#'     \item{case.depth}{Numeric; mean isolation depth across the forest.
#'       Lower means the observation was isolated quickly, so more
#'       anomalous.}
#'     \item{howbad}{Numeric in \code{[0, 1]}; the \code{case.depth}
#'       values pushed through their own empirical CDF and flipped so
#'       higher means more anomalous. This is the score the plot method
#'       draws by default.}
#'   }
#'   A \code{provenance} attribute records
#'   \code{source = "varPro::isopro"}, the observation count \code{n}, and
#'   the number of trees \code{ntree}.
#'
#' @section Comparing methods:
#' To compare methods (\code{"rnd"}, \code{"unsupv"}, \code{"auto"}), call
#' \code{\link{gg_isopro}} on each fit and \code{dplyr::bind_rows()} the
#' results with a \code{method} label column. The plot method auto-detects
#' \code{method} and colours the curves.
#'
#' @references
#' Liu, F. T., Ting, K. M., and Zhou, Z. H. (2008). Isolation Forest.
#' \emph{Eighth IEEE International Conference on Data Mining}, 413-422.
#'
#' Ishwaran, H., Mantero, A., and Lu, M. (2025). varPro: Model-Independent
#' Variable Selection via the Rule-Based Variable Priority Framework.
#' \emph{R package version 3.x}.
#'
#' @seealso \code{\link{plot.gg_isopro}}, \code{\link[varPro]{isopro}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("varPro", quietly = TRUE)) {
#'   set.seed(1)
#'   fit <- varPro::isopro(data = iris[, 1:4], method = "rnd",
#'                         sampsize = 32, ntree = 50)
#'   gg <- gg_isopro(fit)
#'   plot(gg)
#' }
#' }
#'
#' @export
gg_isopro <- function(object, ..., newdata = NULL) {
  UseMethod("gg_isopro", object)
}

#' @export
gg_isopro.isopro <- function(object, ..., newdata = NULL) {
  if (!inherits(object, "isopro")) {
    stop("gg_isopro expects a 'isopro' object from varPro::isopro().",
         call. = FALSE)
  }

  ntree <- tryCatch(
    as.integer(object$isoforest$ntree),
    error = function(e) NA_integer_
  )
  ntree <- if (length(ntree) == 1L && !is.na(ntree)) ntree else NA_integer_

  ## ---- Training path (newdata = NULL) ------------------------------------
  if (is.null(newdata)) {
    # varPro's $howbad uses "lower = more anomalous" polarity (it is the
    # quantile of case.depth, low depth = anomalous). The wrapper convention
    # is "higher = more anomalous", so flip the polarity here the same way
    # the prediction path does (howbad = 1 - quantile).
    howbad <- 1 - as.numeric(object$howbad)
    depth  <- as.numeric(object$case.depth)
    n      <- length(howbad)

    gg_dta <- data.frame(
      obs        = seq_len(n),
      case.depth = depth,
      howbad     = howbad
    )
    class(gg_dta) <- c("gg_isopro", class(gg_dta))
    attr(gg_dta, "provenance") <- list(
      source     = "varPro::isopro",
      n          = n,
      ntree      = ntree,
      prediction = FALSE
    )
    return(invisible(gg_dta))
  }

  ## ---- Prediction path (newdata supplied) -------------------------------
  if (!is.data.frame(newdata)) {
    stop("newdata must be a data.frame.", call. = FALSE)
  }

  # Two calls to predict.isopro: raw depth and quantile-against-training.
  # The wrapper polarity is "higher = more anomalous", so we flip the quantile:
  #   howbad = 1 - predict(object, newdata, quantiles = TRUE)
  # case.depth keeps varPro's native scale (lower = more anomalous), giving
  # the user a varPro-polarity number for cross-reference.
  depth <- as.numeric(stats::predict(object, newdata = newdata,
                                     quantiles = FALSE))
  q     <- as.numeric(stats::predict(object, newdata = newdata,
                                     quantiles = TRUE))
  howbad <- 1 - q
  n      <- nrow(newdata)

  gg_dta <- data.frame(
    obs        = seq_len(n),
    case.depth = depth,
    howbad     = howbad
  )
  class(gg_dta) <- c("gg_isopro", class(gg_dta))
  attr(gg_dta, "provenance") <- list(
    source     = "varPro::isopro",
    n          = n,
    ntree      = ntree,
    prediction = TRUE
  )
  invisible(gg_dta)
}

# Tests for gg_isopro() — varPro::isopro tidy-data wrapper.
# Phase 4 of the v2.8.0 varPro integration.

make_iso_fit <- function(seed = 1L, method = "rnd", ntree = 25, sampsize = 16) {
  # Only the *unsupervised* isopro grow (method = "unsupv") trips
  # randomForestSRC's gcc-UBSAN report at entry.c:184 (length-0 yvar.wt
  # decremented to an out-of-bounds pointer; upstream bug, ggRandomForests is
  # pure R). The default method = "rnd" builds a synthetic-supervised forest and
  # is UBSAN-clean — verified under -fsanitize=undefined. So skip_on_cran() only
  # the unsupervised path; the rnd tests run on CRAN. Both run in CI and locally.
  if (identical(method, "unsupv")) testthat::skip_on_cran()
  testthat::skip_if_not_installed("varPro")
  set.seed(seed)
  varPro::isopro(
    data     = iris[, 1:4],
    method   = method,
    sampsize = sampsize,
    ntree    = ntree
  )
}

test_that("gg_isopro: returns gg_isopro data.frame with correct columns", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_s3_class(gg, "gg_isopro")
  expect_s3_class(gg, "data.frame")
  expect_named(gg, c("obs", "case.depth", "howbad"), ignore.order = TRUE)
})

test_that("gg_isopro: row count equals n; howbad in [0,1]; obs is 1..n", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_equal(nrow(gg), nrow(iris))
  expect_true(all(gg$howbad >= 0 & gg$howbad <= 1))
  expect_equal(gg$obs, seq_len(nrow(iris)))
})

test_that("gg_isopro: provenance attribute attached", {
  fit  <- make_iso_fit(ntree = 25)
  gg   <- gg_isopro(fit)
  prov <- attr(gg, "provenance")
  expect_type(prov, "list")
  expect_equal(prov$source, "varPro::isopro")
  expect_equal(prov$n, nrow(iris))
  expect_equal(prov$ntree, 25)
})

test_that("plot.gg_isopro: panel='both' returns patchwork with 2 sub-plots that build", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  p   <- plot(gg, panel = "both")
  expect_s3_class(p, "patchwork")
  expect_length(p$patches$plots, 1L) # left panel is `p` itself; right is in $patches$plots
  for (sub in c(list(p), p$patches$plots)) {
    expect_no_error(ggplot2::ggplot_build(sub))
  }
})

test_that("plot.gg_isopro: panel='elbow' returns a single ggplot", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  p   <- plot(gg, panel = "elbow")
  expect_s3_class(p, "ggplot")
  expect_false(inherits(p, "patchwork"))
  expect_no_error(ggplot2::ggplot_build(p))
})

test_that("plot.gg_isopro: panel='density' returns a single ggplot", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  p   <- plot(gg, panel = "density")
  expect_s3_class(p, "ggplot")
  expect_false(inherits(p, "patchwork"))
  expect_no_error(ggplot2::ggplot_build(p))
})

test_that("plot.gg_isopro: rejects bad panel values via match.arg", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_error(plot(gg, panel = "nope"))
})

# Helper: count geom_hline / geom_vline layers in a ggplot or patchwork.
# Note: a patchwork object is itself the "left" plot (still inherits "ggplot"),
# while siblings live in $patches$plots. Strip the patchwork class before
# counting layers on the top-level object to avoid infinite recursion.
.count_ref_lines <- function(p) {
  count_one <- function(plt) {
    sum(vapply(plt$layers, function(l) {
      inherits(l$geom, "GeomHline") || inherits(l$geom, "GeomVline")
    }, logical(1L)))
  }
  if (inherits(p, "patchwork")) {
    siblings <- p$patches$plots
    top <- p
    class(top) <- setdiff(class(top), "patchwork")
    return(count_one(top) + sum(vapply(siblings, count_one, integer(1L))))
  }
  count_one(p)
}

test_that("plot.gg_isopro: threshold adds a reference line on each panel", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  p_none <- plot(gg, panel = "both")
  p_thr  <- plot(gg, panel = "both", threshold = 0.8)
  # panel = both has elbow + density => one hline + one vline = 2 ref lines.
  expect_equal(.count_ref_lines(p_none), 0L)
  expect_equal(.count_ref_lines(p_thr), 2L)
})

test_that("plot.gg_isopro: top_n_pct resolves to the matching quantile", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  q95 <- as.numeric(stats::quantile(gg$howbad, 0.95))
  p   <- plot(gg, panel = "elbow", top_n_pct = 5)
  # Find the hline; check its yintercept equals q95.
  yints <- vapply(p$layers, function(l) {
    if (inherits(l$geom, "GeomHline")) l$data$yintercept else NA_real_
  }, numeric(1L))
  yints <- yints[!is.na(yints)]
  expect_length(yints, 1L)
  expect_equal(yints[[1]], q95, tolerance = 1e-9)
})

test_that("plot.gg_isopro: threshold + top_n_pct both set => message, threshold wins", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_message(
    p <- plot(gg, panel = "elbow", threshold = 0.7, top_n_pct = 5),
    "Both .* using `threshold`"
  )
  yints <- vapply(p$layers, function(l) {
    if (inherits(l$geom, "GeomHline")) l$data$yintercept else NA_real_
  }, numeric(1L))
  yints <- yints[!is.na(yints)]
  expect_equal(yints[[1]], 0.7, tolerance = 1e-9)
})

test_that("plot.gg_isopro: a method column triggers colour grouping in the elbow", {
  fit_rnd <- make_iso_fit(seed = 1L, method = "rnd")
  fit_uns <- make_iso_fit(seed = 1L, method = "unsupv")
  gg <- rbind(
    cbind(gg_isopro(fit_rnd), method = "rnd"),
    cbind(gg_isopro(fit_uns), method = "unsupv")
  )
  class(gg) <- c("gg_isopro", "data.frame")

  p <- plot(gg, panel = "elbow")
  expect_s3_class(p, "ggplot")
  # Built scales should include a colour scale because `method` is mapped.
  built <- ggplot2::ggplot_build(p)
  scales <- built$plot$scales$scales
  has_colour <- any(vapply(scales, function(s) "colour" %in% s$aesthetics, logical(1L)))
  # Even when ggplot2 hasn't materialised a discrete colour scale into
  # $scales, the layer's aes mapping must include `colour = method`.
  layer_aes <- p$layers[[1]]$mapping
  expect_true("colour" %in% names(layer_aes) || has_colour)
})

test_that("print.gg_isopro: prints a one-line header, invisibly", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  txt <- utils::capture.output(out <- print(gg))
  expect_identical(out, gg)               # returns invisibly
  expect_length(txt, 1L)                  # exactly one line
  expect_match(txt, "gg_isopro")
})

test_that("summary.gg_isopro: returns summary.gg with quantile snapshot in body", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  s   <- summary(gg)
  expect_s3_class(s, "summary.gg")
  expect_true(any(grepl("rows", s$body)))
  expect_true(any(grepl("howbad", s$body)))
})

test_that("autoplot.gg_isopro: same return as plot.gg_isopro for the default args", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_s3_class(autoplot(gg), "patchwork")
})

test_that("plot.gg_isopro: threshold must be numeric scalar in [0,1]", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_error(plot(gg, threshold = "high"),  "single numeric value in \\[0, 1\\]")
  expect_error(plot(gg, threshold = c(0.5, 0.8)), "single numeric value in \\[0, 1\\]")
  expect_error(plot(gg, threshold = 1.5), "single numeric value in \\[0, 1\\]")
})

test_that("plot.gg_isopro: top_n_pct must be numeric scalar in (0,100)", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit)
  expect_error(plot(gg, top_n_pct = "5"),  "single numeric value in \\(0, 100\\)")
  expect_error(plot(gg, top_n_pct = 0),    "single numeric value in \\(0, 100\\)")
  expect_error(plot(gg, top_n_pct = 100),  "single numeric value in \\(0, 100\\)")
  expect_error(plot(gg, top_n_pct = 150),  "single numeric value in \\(0, 100\\)")
})

## ── predict.isopro path (PR for Phase 4b) ──────────────────────────────────

test_that("gg_isopro: newdata returns gg_isopro data.frame with correct columns", {
  fit <- make_iso_fit()
  gg  <- gg_isopro(fit, newdata = iris[, 1:4])
  expect_s3_class(gg, "gg_isopro")
  expect_s3_class(gg, "data.frame")
  expect_named(gg, c("obs", "case.depth", "howbad"), ignore.order = TRUE)
  expect_equal(nrow(gg), nrow(iris))
  expect_equal(gg$obs, seq_len(nrow(iris)))
  expect_true(all(gg$howbad >= 0 & gg$howbad <= 1))
})

test_that("gg_isopro: newdata provenance carries prediction = TRUE and right n", {
  fit  <- make_iso_fit(ntree = 25)
  test_df <- iris[c(1:10, 51:60, 101:110), 1:4]  # 30 rows
  gg   <- gg_isopro(fit, newdata = test_df)
  prov <- attr(gg, "provenance")
  expect_type(prov, "list")
  expect_equal(prov$source, "varPro::isopro")
  expect_equal(prov$n, nrow(test_df))
  expect_equal(prov$ntree, 25)
  expect_true(isTRUE(prov$prediction))
})

test_that("gg_isopro: training-path provenance has no prediction field set TRUE", {
  fit  <- make_iso_fit()
  gg   <- gg_isopro(fit)
  prov <- attr(gg, "provenance")
  expect_false(isTRUE(prov$prediction))
})

test_that("gg_isopro: newdata must be a data.frame", {
  fit <- make_iso_fit()
  expect_error(gg_isopro(fit, newdata = "not a df"),
               "newdata must be a data.frame")
  expect_error(gg_isopro(fit, newdata = 1:10),
               "newdata must be a data.frame")
  expect_error(gg_isopro(fit, newdata = list(a = 1)),
               "newdata must be a data.frame")
})

test_that("gg_isopro: scoring the training set as newdata matches training howbad in range and top-5 ordering", {
  fit       <- make_iso_fit()
  train_df  <- iris[, 1:4]
  gg_train  <- gg_isopro(fit)
  gg_pred   <- gg_isopro(fit, newdata = train_df)
  # The two code paths inside varPro are slightly different so byte equality
  # is too strong, but the score range and the most-anomalous rows should agree.
  expect_equal(nrow(gg_pred), nrow(gg_train))
  expect_true(all(gg_pred$howbad  >= 0 & gg_pred$howbad  <= 1))
  expect_true(all(gg_train$howbad >= 0 & gg_train$howbad <= 1))
  top_train <- head(order(-gg_train$howbad), 5)
  top_pred  <- head(order(-gg_pred$howbad),  5)
  # At least 3 of the top-5 anomalous rows should overlap between the two paths.
  expect_gte(length(intersect(top_train, top_pred)), 3L)
})

test_that("gg_isopro: howbad == 1 - predict(quantiles = TRUE)", {
  fit     <- make_iso_fit()
  test_df <- iris[1:30, 1:4]
  gg      <- gg_isopro(fit, newdata = test_df)
  q_raw   <- as.numeric(stats::predict(fit, newdata = test_df, quantiles = TRUE))
  expect_equal(gg$howbad, 1 - q_raw, tolerance = 1e-12)
})

test_that("gg_isopro: bind_rows(train, test) plots without error via the method-column path", {
  fit      <- make_iso_fit()
  test_df  <- iris[seq(1, nrow(iris), by = 3), 1:4]  # 50 rows
  gg_train <- gg_isopro(fit)
  gg_test  <- gg_isopro(fit, newdata = test_df)
  gg_both  <- rbind(
    cbind(as.data.frame(gg_train), method = "train"),
    cbind(as.data.frame(gg_test),  method = "test")
  )
  class(gg_both) <- c("gg_isopro", "data.frame")
  p <- plot(gg_both, panel = "both")
  expect_s3_class(p, "patchwork")
  # Each sub-plot must actually build (catches aes/data mismatches the way the
  # original plot.gg_variable regression test did).
  built_top <- ggplot2::ggplot_build(p)
  expect_true(inherits(built_top, "ggplot_built"))
  for (sub in p$patches$plots) {
    expect_no_error(ggplot2::ggplot_build(sub))
  }
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_vimp}} object, extracted variable importance of a
#' \code{\link[randomForestSRC]{rfsrc}} object
#'
#' @param x \code{\link{gg_vimp}} object created from a
#' \code{\link[randomForestSRC]{rfsrc}} object
#' @param relative should we plot vimp or relative vimp. Defaults to vimp.
#' @param lbls A vector of alternative variable labels. Item names should be
#' the same as the variable names.
#' @param ... optional arguments passed to gg_vimp if necessary
#'
#' @return \code{ggplot} object
#'
#' @seealso \code{\link{gg_vimp}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
#' R, Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris)
#' gg_dta <- gg_vimp(rfsrc_iris)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## regression example
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' rfsrc_airq <- rfsrc(Ozone ~ ., airquality)
#' gg_dta <- gg_vimp(rfsrc_airq)
#' plot(gg_dta)
#'
#'
#' @export
plot.gg_vimp <- function(x, relative, lbls, ...) {
  gg_dta <- x

  # Accept raw rfsrc / randomForest objects and compute VIMP on the fly
  if (!inherits(gg_dta, "gg_vimp")) {
    gg_dta <- gg_vimp(gg_dta, ...)
  }

  # Capture extra args so we can inspect nvar.
  arg_set <- list(...)

  # Optionally restrict to the top-nvar most important variables (gg_vimp
  # already sorts by descending VIMP, so we just trim the tail).
  nvar <- nrow(gg_dta)
  if (!is.null(arg_set$nvar)) {
    if (is.numeric(arg_set$nvar) && arg_set$nvar > 1) {
      if (arg_set$nvar < nrow(gg_dta)) {
        nvar <- arg_set$nvar
        gg_dta <- gg_dta[seq_len(nvar), ]
      }
    }
  }

  gg_plt <- ggplot2::ggplot(gg_dta)

  # Use "vimp" as the bar-height column when it exists; fall back to the
  # first column name for objects that store a renamed importance measure.
  msr <- "vimp"
  if (!msr %in% colnames(gg_dta)) {
    msr <- colnames(gg_dta)[1]
  }

  # Always map both `fill` and `color` to `positive` -- this gives filled bars
  # (rather than hollow outlines) and ensures the function-level
  # `labs(fill = ..., color = ...)` below applies cleanly. When `positive` has
  # only one level (all VIMPs positive, the common case for well-behaved
  # forests), the bars simply render in a single colour and ggplot collapses
  # the fill+color legend into a one-row legend; when both signs are present,
  # the two-row legend distinguishes positive from negative VIMP.
  #
  # Both aesthetics share the same legend title ("VIMP > 0") so ggplot
  # collapses what would otherwise be two legends -- one for fill and one
  # titled with the column name "positive" -- into a single merged legend.
  legend_title <- "VIMP > 0"
  gg_plt <- gg_plt +
    ggplot2::geom_bar(
      ggplot2::aes(
        y    = .data[[msr]],
        x    = .data$vars,
        fill = .data$positive,
        color = .data$positive
      ),
      stat = "identity",
      width = .5,
    )
  # Set both legends' titles to the same string so ggplot merges them.
  # Users can override with their own labs() call after the fact.
  gg_plt <- gg_plt +
    ggplot2::labs(x = "", y = msr,
                  fill = legend_title, color = legend_title)

  if (!missing(lbls)) {
    # Map internal variable names to human-readable labels.  lbls should be a
    # named character vector; any unmatched variables keep their original name.
    if (length(lbls) >= length(gg_dta$vars)) {
      st_lbls <- lbls[as.character(gg_dta$vars)]
      names(st_lbls) <- as.character(gg_dta$vars)
      # Fall back to the raw variable name when no label was supplied
      st_lbls[which(is.na(st_lbls))] <-
        names(st_lbls[which(is.na(st_lbls))])

      gg_plt <- gg_plt +
        ggplot2::scale_x_discrete(labels = st_lbls)
    }
  }

  # Flip coordinates so variable names appear on the y-axis (horizontal bars
  # are easier to read when there are many variables).  If gg_vimp contains
  # a "set" column (comparison VIMP across two forests), facet by set.
  if (is.null(gg_dta$set) || length(unique(gg_dta$set)) < 2) {
    gg_plt <- gg_plt +
      ggplot2::coord_flip()
  } else {
    gg_plt <- gg_plt +
      ggplot2::coord_flip() +
      ggplot2::facet_grid(~set)
  }

  return(gg_plt)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Nonparametric survival estimates.
#'
#' @details
#' Comparing the forest's ensemble survival curve to the marginal
#' Kaplan-Meier baseline is a quick sanity check: if they diverge the forest
#' has found structure the predictors carry; if they track each other closely
#' the predictors may add little.  \code{gg_survival} computes
#' the nonparametric baseline -- the Kaplan-Meier or Nelson-Aalen estimate --
#' so you can place it on the same canvas as the forest predictions from
#' \code{\link{gg_rfsrc}}.
#'
#' \code{gg_survival} is an S3 generic that dispatches on the class of its
#' first argument:
#'
#' \describe{
#'   \item{\code{rfsrc}}{Extracts the outcome columns from the fitted
#'     forest's \code{$yvar} slot (time in column 1, event indicator in
#'     column 2) and delegates to \code{\link{kaplan}}.  Use \code{by} to
#'     stratify on a predictor from \code{$xvar}: you get one
#'     Kaplan-Meier curve per group, ready to compare against the forest's
#'     group-specific ensemble curves.}
#'   \item{default}{Accepts raw survival columns directly via
#'     \code{interval}, \code{censor}, and \code{data}.  Delegates to
#'     \code{\link{kaplan}} (the default) or \code{\link{nelson}} depending
#'     on \code{type}.}
#' }
#'
#' @param object For the \code{rfsrc} method: a fitted
#'   \code{\link[randomForestSRC]{rfsrc}} survival forest.  For the default
#'   method: pass \code{NULL} (or omit) and supply \code{interval},
#'   \code{censor}, and \code{data} instead.
#' @param interval Character; name of the time-to-event column in \code{data}
#'   (default method only).
#' @param censor Character; name of the event-indicator column in \code{data}
#'   (1 = event, 0 = censored; default method only).
#' @param by Optional character; name of a grouping column for stratified
#'   estimates.  For the \code{rfsrc} method, \code{by} must be a column in
#'   \code{object$xvar}.
#' @param data A \code{data.frame} containing survival data (default method
#'   only).
#' @param type One of \code{"kaplan"} (Kaplan-Meier, default) or
#'   \code{"nelson"} (Nelson-Aalen cumulative hazard).  Default method only.
#' @param ... Additional arguments passed to \code{\link{kaplan}} or
#'   \code{\link{nelson}}.
#'
#' @return A \code{gg_survival} \code{data.frame} with columns \code{time},
#'   \code{surv}, \code{cum_haz}, \code{lower}, \code{upper}, \code{n.risk},
#'   and optionally \code{groups} when \code{by} is supplied.
#'
#' @seealso \code{\link{kaplan}} \code{\link{nelson}}
#' @seealso \code{\link{plot.gg_survival}}
#'
#' @examples
#' ## -------- pbc data (default method, raw data columns)
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' gg_dta <- gg_survival(interval = "time", censor = "status", data = pbc)
#' plot(gg_dta, error = "none")
#'
#' # Stratified
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#' plot(gg_dta)
#'
#' @note Survival estimation is `randomForestSRC`-only; `randomForest`
#'   has no survival forest, so no `randomForest` method exists.
#' @export
gg_survival <- function(object = NULL,
                        interval = NULL,
                        censor = NULL,
                        by = NULL,
                        data = NULL,
                        type = c("kaplan", "nelson"),
                        ...) {
  UseMethod("gg_survival")
}

#' @rdname gg_survival
#' @export
gg_survival.rfsrc <- function(object,
                              interval = NULL,
                              censor   = NULL,
                              by       = NULL,
                              data     = NULL,
                              type     = c("kaplan", "nelson"),
                              ...) {
  ## interval, censor, data, and type are accepted for consistency with the
  ## generic signature but are ignored: this method extracts everything it
  ## needs from the fitted forest's $yvar and $xvar slots.
  ##
  ## randomForestSRC stores the outcome as a two-column data frame: time (col 1)
  ## and status/censor (col 2) in object$yvar.
  yvar <- object$yvar
  if (is.null(yvar) || !is.data.frame(yvar) || ncol(yvar) < 2L) {
    stop(
      "gg_survival requires a survival forest; this rfsrc object ",
      "does not appear to be a survival forest.",
      call. = FALSE
    )
  }

  interval_col <- colnames(yvar)[1L]
  censor_col   <- colnames(yvar)[2L]
  surv_data    <- yvar

  if (!is.null(by)) {
    if (!by %in% colnames(object$xvar)) {
      stop(sprintf(
        "'by' column '%s' not found in the forest's xvar slot.", by
      ), call. = FALSE)
    }
    surv_data[[by]] <- object$xvar[[by]]
  }

  gg_dta <- kaplan(interval = interval_col, censor = censor_col, by = by,
                   data = surv_data, ...)
  .set_provenance(gg_dta, object)
}

#' @rdname gg_survival
#' @export
gg_survival.default <- function(object = NULL,
                                interval = NULL,
                                censor = NULL,
                                by = NULL,
                                data = NULL,
                                type = c("kaplan", "nelson"),
                                ...) {
  ## Allow data to be passed as the first positional argument for convenience.
  if (is.data.frame(object) && is.null(data)) {
    data <- object
  }

  type <- match.arg(type)

  gg_dta <- switch(type, # nolint: object_usage_linter
    kaplan = kaplan(
      interval = interval,
      censor   = censor,
      by       = by,
      data     = data,
      ...
    ),
    nelson = nelson(
      interval = interval,
      censor   = censor,
      by       = by,
      data     = data,
      ...
    )
  )

  return(gg_dta)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Plot a \code{\link{gg_survival}}  object.
#'
#' @param x \code{\link{gg_survival}} or a survival \code{\link{gg_rfsrc}}
#' object created from a \code{\link[randomForestSRC]{rfsrc}} object
#'
#' @param error "shade", "bars", "lines" or "none"
#' @param type "surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"
#' @param label Modify the legend label when gg_survival has stratified samples
#' @param ... Additional arguments forwarded to \code{geom_step()}. When
#'   \code{alpha} is supplied it is passed to the step geom and a halved
#'   value is used for the ribbon overlay.
#'
#' @return A \code{ggplot} object. The y-axis shows the chosen \code{type}
#'   (e.g. survival probability for \code{"surv"}) and the x-axis shows time.
#'   Confidence shading, bars, or lines are added when the input object carries
#'   confidence-interval columns.
#'
#' @seealso \code{\link{gg_survival}}, \code{\link{kaplan}},
#'   \code{\link{nelson}}, \code{\link{gg_rfsrc}}
#'
#' @examples
#' ## -------- pbc data
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' # This is the same as kaplan
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' # Stratified on treatment variable.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#' plot(gg_dta, label = "treatment")
#'
#' # ...with smaller confidence limits.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment", conf.int = .68
#' )
#'
#' plot(gg_dta, error = "lines")
#' plot(gg_dta, label = "treatment", error = "lines")
#'
#' # ...with smaller confidence limits.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "sex", conf.int = .68
#' )
#'
#' plot(gg_dta, error = "lines")
#' plot(gg_dta, label = "sex", error = "lines")
#'
#' @export
### Survival plots
plot.gg_survival <- function(x,
                             type = c("surv",
                                      "cum_haz",
                                      "hazard",
                                      "density",
                                      "mid_int",
                                      "life",
                                      "proplife"),
                             error = c("shade", "bars", "lines", "none"),
                             label = NULL,
                             ...) {
  # Auto-coerce raw rfsrc objects so plot(rfsrc_obj) works directly.
  # gg_survival.rfsrc extracts yvar and computes Kaplan-Meier estimates.
  gg_dta <- if (inherits(x, "rfsrc")) gg_survival(x) else x

  error <- match.arg(error)
  type  <- match.arg(type)

  # Split ... so alpha applies to step geoms while ribbons use a softer value.
  spl          <- .gg_split_alpha(list(...))
  ribbon_alpha <- spl$ribbon_alpha

  # Now order matters, so we want to place the forest predictions on the bottom
  # Create the figure skeleton,
  if (is.null(gg_dta$groups)) {
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_step(ggplot2::aes(x = .data$time, y = .data[[type]]), ...)
  } else {
    gg_dta$groups <- factor(gg_dta$groups)
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_step(ggplot2::aes(
        x = .data$time,
        y = .data[[type]],
        color = .data$groups
      ),
      ...)
    if (!is.null(label)) {
      gg_plt <- gg_plt +
        ggplot2::labs(color = label, fill = label)
    }
  }
  # Do we want to show confidence limits?
  if (type == "surv") {
    if (is.null(gg_dta$groups)) {
      gg_plt <- switch(
        error,
        # Shading the standard errors
        shade = gg_plt +
          ggplot2::geom_ribbon(
            ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower
            ),
            alpha = ribbon_alpha,
            fill  = .gg_ribbon_fill
          ),
        # Or showing error bars
        bars = {
          # Need to figure out how to remove some of these points when
          # requesting error bars, or this will get really messy.
          gg_plt +
            ggplot2::geom_errorbar(ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower
            ))
        },
        lines = gg_plt +
          ggplot2::geom_step(ggplot2::aes(
            x = .data$time, y = .data$upper
          ), linetype = 2) +
          ggplot2::geom_step(ggplot2::aes(
            x = .data$time, y = .data$lower
          ), linetype = 2),
        none = gg_plt
      )
    } else {
      gg_plt <- switch(
        error,
        # Shading the standard errors
        shade = gg_plt +
          ggplot2::geom_ribbon(
            ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower,
              fill = .data$groups,
              color = .data$groups
            ),
            alpha = ribbon_alpha
          ),
        # Or showing error bars
        bars = {
          # Need to figure out how to remove some of these points when
          # requesting error bars, or this will get really messy.
          gg_plt +
            ggplot2::geom_errorbar(
              ggplot2::aes(
                x = .data$time,
                ymax = .data$upper,
                ymin = .data$lower,
                color = .data$groups
              )
            )
        },
        lines = gg_plt +
          ggplot2::geom_step(
            ggplot2::aes(
              x = .data$time,
              y = .data$upper,
              color = .data$groups
            ),
            linetype = 2
          ) +
          ggplot2::geom_step(
            ggplot2::aes(
              x = .data$time,
              y = .data$lower,
              color = .data$groups
            ),
            linetype = 2
          ),
        none = gg_plt
      )
    }
  }
  return(gg_plt)
}

####**********************************************************************
####  Internal: shared styling constants and helpers for ribbon overlays.
####
####  Every plot.gg_* method that draws a ribbon (CI band on KM/NA curves,
####  bootstrap CI band on gg_rfsrc survival curves, per-subject envelope
####  on gg_brier) uses these constants so the family renders uniformly.
####  This is styling only — the underlying probability/coverage of each
####  ribbon is computed elsewhere and is intentionally heterogeneous
####  (95% inferential CIs vs. 15-85 percentile descriptive envelopes).
####**********************************************************************

# Canonical ribbon transparency. Low enough that overlapping bands stay
# readable when several models are layered, high enough to read on white.
.gg_ribbon_alpha <- 0.2

# Default fill for single-series ribbons (no group aesthetic). Group-coloured
# ribbons override via aes(fill = group) and ignore this constant.
.gg_ribbon_fill <- "steelblue"

# Split a captured list(...) into ribbon and step components.
#
# When the user passes alpha = x to a plot.gg_* method, that alpha is
# meant for the main line/step geom.  The ribbon should use a softer
# value (half of the user alpha, or the package default if none supplied).
# This helper centralises that logic and prevents the duplicate-formal
# error that arises when alpha is both in ... and set explicitly on geom_ribbon.
#
# Returns a list with:
#   $ribbon_alpha  — numeric alpha to pass explicitly to geom_ribbon()
#   $step_dots     — original dots (forwarded to geom_step / geom_line)
#   $ribbon_dots   — dots with alpha removed (ribbon alpha is explicit)
.gg_split_alpha <- function(dots) {
  user_alpha <- dots[["alpha"]]
  list(
    ribbon_alpha = if (is.null(user_alpha)) .gg_ribbon_alpha
                   else user_alpha * 0.5,
    step_dots    = dots,
    ribbon_dots  = dots[setdiff(names(dots), "alpha")]
  )
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Plot a \code{\link{gg_variable}} object,
#'
#' @param x \code{\link{gg_variable}} object created from a
#' \code{\link[randomForestSRC]{rfsrc}} object
#' @param xvar variable (or list of variables) of interest.
#' @param time For survival, one or more times of interest
#' @param time_labels string labels for times
#' @param panel Should plots be faceted along multiple xvar?
#' @param oob oob estimates (boolean)
#' @param points plot the raw data points (boolean)
#' @param smooth include a smooth curve (boolean)
#' @param ... arguments passed to the \code{ggplot2} functions.
#'
#' @return A single \code{ggplot} object when \code{length(xvar) == 1} or
#'   \code{panel = TRUE}; otherwise a \code{patchwork} composite that stacks
#'   one panel per variable in \code{xvar}. Either way the result is one
#'   plottable object, never a bare list, so it composes with
#'   \code{patchwork} and dispatches through \code{ggplot2::autoplot()}.
#'   For the patchwork case, to inspect one panel with
#'   \code{ggplot2::layer_data()} pull that panel out first
#'   (e.g. \code{ggplot2::layer_data(p[[1]])}).
#'
#' @seealso \code{\link{gg_variable}}, \code{\link{gg_partial}},
#'   \code{\link[randomForestSRC]{plot.variable}}
#'
#' @references Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
#' 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#'
#' @importFrom ggplot2 .data
#' @examples
#' ## ------------------------------------------------------------
#' ## classification
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_iris)
#' plot(gg_dta, xvar = "Sepal.Width")
#' plot(gg_dta, xvar = "Sepal.Length")
#'
#' ## Panel plot across all predictors
#' plot(gg_dta,
#'   xvar = rfsrc_iris$xvar.names,
#'   panel = TRUE, se = FALSE
#' )
#'
#' ## ------------------------------------------------------------
#' ## regression
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' # na.action = "na.impute" handles missing Ozone / Solar.R values
#' set.seed(42)
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' gg_dta <- gg_variable(rfsrc_airq)
#'
#' # Treat Month as an ordinal factor for better visualisation
#' gg_dta[, "Month"] <- factor(gg_dta[, "Month"])
#'
#' plot(gg_dta, xvar = "Wind")
#' plot(gg_dta, xvar = "Temp")
#' plot(gg_dta, xvar = "Solar.R")
#'
#' # Factor variable uses notched boxplots
#' plot(gg_dta, xvar = "Month", notch = TRUE)
#'
#' \donttest{
#' # Panel plot across continuous predictors (loess smooths; slower)
#' plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
#' }
#'
#' ## ------------------------------------------------------------
#' ## survival examples
#' ## ------------------------------------------------------------
#' ## -------- veteran data
#' \donttest{
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., veteran,
#'   nsplit = 10,
#'   ntree = 50
#' )
#'
#' # Marginal survival at 90 days
#' gg_dta <- gg_variable(rfsrc_veteran, time = 90)
#'
#' # Single-variable dependence plots
#' plot(gg_dta, xvar = "age")
#' plot(gg_dta, xvar = "diagtime")
#'
#' # Panel coplot for two predictors at a single time
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
#'
#' # Compare survival at 30, 90, and 365 days simultaneously
#' gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
#'
#' # Single-variable plot (one facet per time point)
#' plot(gg_dta, xvar = "age")
#'
#' # Panel coplot across two predictors and three time points
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
#' }
#'
#' @export
plot.gg_variable <- function(x, # nolint: cyclocomp_linter
                             xvar,
                             time,
                             time_labels,
                             panel = FALSE,
                             oob = TRUE,
                             points = TRUE,
                             smooth = TRUE,
                             ...) {
  gg_dta <- x

  # I don't think this will work with latest S3 models.
  if (inherits(x, "rfsrc")) {
    gg_dta <- gg_variable(x, ...)
  }

  ## ---- Detect forest family from gg_variable class attributes ----------
  # Default to classification; override if survival or regression flags found
  family <- "class"
  if (inherits(gg_dta, "surv")) {
    family <- "surv"
  } else if (inherits(gg_dta, "regr")) {
    family <- "regr"
  } else if (inherits(gg_dta, "regression")) {
    family <- "regr"
  }

  # Fallback detection: presence of an "event" column signals survival
  if (sum(colnames(gg_dta) == "event") != 0) {
    family <- "surv"
  }

  ## ---- Reshape multi-class classification data -------------------------
  # Multiple yhat.* columns indicate a multi-class forest
  if (length(grep("yhat.", colnames(gg_dta))) > 0) {
    if (length(grep("yhat.", colnames(gg_dta))) == 2) {
      # Binary: drop the first class column and rename the second to "yhat"
      gg_dta <- gg_dta[, -grep("yhat.", colnames(gg_dta))[1]]
      colnames(gg_dta)[grep("yhat.", colnames(gg_dta))] <- "yhat"
    } else {
      # Multi-class: pivot to long format so each class becomes a row group
      gg_dta_x <- gg_dta[, -grep("yhat.", colnames(gg_dta))]
      gg_dta_y <- gg_dta[, grep("yhat.", colnames(gg_dta))]
      lng <- ncol(gg_dta_y)
      gg2 <- lapply(seq_len(ncol(gg_dta_y)), function(ind) {
        cbind(gg_dta_x, yhat = gg_dta_y[, ind],
              outcome = sub("^yhat\\.", "", colnames(gg_dta_y)[ind]))
      })
      gg3 <- do.call(rbind, gg2)
      # Use column order from gg_dta_y (not alphabetical) so facet panels
      # appear in the same order as the model's class levels.
      outcome_levels <- sub("^yhat\\.", "", colnames(gg_dta_y))
      gg3$outcome <- factor(gg3$outcome, levels = outcome_levels)
      gg_dta <- gg3
    }
  }

  ## ---- Default xvar: all predictor columns -----------------------------
  if (missing(xvar)) {
    # Remove response-side and structural columns to isolate predictors.
    # `event`, `time`, `yvar`, `outcome` are matched by exact name -- a
    # substring match (the prior code's `grep("time", ...)`) would silently
    # drop a documented predictor like the veteran data's `diagtime`. `yhat`
    # matches the bare `yhat` column (regression / single-class survival) and
    # also `yhat.<classname>` columns produced by classification gg_variable,
    # so the regex anchors at the start and either ends or transitions on a
    # literal dot.
    cls <- c(
      grep("^yhat(\\.|$)", colnames(gg_dta)),
      which(colnames(gg_dta) %in% c("event", "time", "yvar", "outcome"))
    )
    xvar <- colnames(gg_dta)[-cls]
  }

  lng <- length(xvar)

  # Column indices corresponding to the requested predictor(s)
  wch_x_var <- which(colnames(gg_dta) %in% xvar)

  ## ---- Coerce 0/1 indicator columns to logical -------------------------
  # Columns with exactly two values in {0, 1} are treated as boolean flags
  for (ind in seq_len(ncol(gg_dta))) {
    if (!is.factor(gg_dta[, ind])) {
      if (length(unique(gg_dta[which(!is.na(gg_dta[, ind])), ind])) <= 2) {
        if (sum(range(gg_dta[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
      }
    } else {
      if (length(unique(gg_dta[which(!is.na(gg_dta[, ind])), ind])) <= 2) {
        if (sum(sort(unique(gg_dta[, ind])) == c(0, 1)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
        if (sum(sort(unique(gg_dta[, ind])) == c(FALSE, TRUE)) == 2) {
          gg_dta[, ind] <- as.logical(gg_dta[, ind])
        }
      }
    }
  }

  ## ---- Record each column's storage class for plot dispatch -----------
  # "integer" is treated as numeric for plot branching purposes
  ccls <- sapply(gg_dta, class)
  ccls[which(ccls == "integer")] <- "numeric"

  ## =========================================================
  ## PANEL PLOT branch: facet multiple predictors in one figure
  ## =========================================================
  if (panel) {
    ## ---- Survival panel plot ----------------------------------------
    if (family == "surv") {
      ## Indices of response columns (time, event, and yhat)
      wch_y_var <-
        which(colnames(gg_dta) %in% c("event", "yhat", "time"))

      # Subset to response + requested predictors, then pivot to long form
      tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
      pivot_cols <-
        colnames(tmp_dta)[-which(colnames(tmp_dta) %in%
          c("time", "event", "yhat"))]
      gg_dta_mlt <-
        tidyr::pivot_longer(tmp_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")

      # Preserve user-supplied xvar ordering in the facet strips
      gg_dta_mlt$variable <-
        factor(gg_dta_mlt$variable, levels = xvar)
      if (points) {
        gg_plt <- ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(
            x = .data$value,
            y = .data$yhat,
            color = .data$event,
            shape = .data$event
          )
        )
      } else {
        gg_plt <- ggplot2::ggplot(
          gg_dta_mlt,
          ggplot2::aes(x = .data$value, y = .data$yhat)
        )
      }
      # If these are all continuous...
      if (sum(ccls[wch_x_var] == "numeric") == length(wch_x_var)) {
        gg_plt <- gg_plt +
          ggplot2::labs(y = "Survival")
        if (points) {
          gg_plt <- gg_plt +
            ggplot2::geom_point(...)
        } else {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
        if (smooth) {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
      } else {
        # Mixed or all-factor predictors: fall back to boxplot+jitter
        if (sum(ccls[wch_x_var] == "numeric") > 0) {
          warning(
            "Mismatched variable types for panel plots...
            assuming these are all factor variables."
          )
        }

        gg_plt <- gg_plt +
          ggplot2::geom_boxplot(
            ggplot2::aes(x = .data$value, y = .data$yhat),
            color = "grey",
            ...,
            outlier.shape = NA
          ) +
          ggplot2::geom_jitter(
            ggplot2::aes(
              x = .data$value,
              y = .data$yhat,
              color = .data$event,
              shape = .data$event
            ),
            ...
          )
      }

      # Multiple time points: grid of (time × variable); single time: wrap
      if (length(levels(gg_dta$time)) > 1) {
        gg_plt <- gg_plt +
          ggplot2::facet_grid(stats::reformulate("variable", "time"),
            scales = "free_x"
          ) +
          labs(x = "")
      } else {
        gg_plt <- gg_plt +
          ggplot2::facet_wrap(~variable, scales = "free_x") +
          labs(
            x = "",
            y = paste("Survival at", gg_dta$time[1], "year")
          )
      }

    } else {
      ## ---- Regression / classification panel plot ---------------------
      wch_y_var <- which(colnames(gg_dta) %in% c("yhat"))

      if (family == "class") {
        # Include the observed class label column for colouring
        wch_y_var <- c(wch_y_var, which(colnames(gg_dta) == "yvar"))
        tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
        pivot_cols <-
          colnames(tmp_dta)[-which(colnames(tmp_dta) %in% c("yvar", "yhat"))]
        gg_dta_mlt <-
          tidyr::pivot_longer(
            tmp_dta,
            tidyr::all_of(pivot_cols),
            names_to = "variable",
            values_to = "value"
          )
      } else {
        # Regression: keep yhat and the optional yvar reference column
        wch_y_var <- c(wch_y_var, which(colnames(gg_dta) == "yvar"))
        tmp_dta <- gg_dta[, c(wch_y_var, wch_x_var)]
        pivot_cols <-
          colnames(tmp_dta)[-which(colnames(tmp_dta) == "yhat")]
        gg_dta_mlt <-
          tidyr::pivot_longer(
            tmp_dta,
            tidyr::all_of(pivot_cols),
            names_to = "variable",
            values_to = "value"
          )
      }
      # Preserve user-supplied xvar ordering in the facet strips
      gg_dta_mlt$variable <-
        factor(gg_dta_mlt$variable, levels = xvar)

      # All continuous predictors give scatter; any factor gives boxplot
      if (sum(ccls[wch_x_var] == "numeric") == length(wch_x_var)) {
        if (family == "class") {
          gg_plt <-
            ggplot2::ggplot(
              gg_dta_mlt,
              ggplot2::aes(
                x = .data$value,
                y = .data$yhat,
                color = .data$yvar,
                shape = .data$yvar
              )
            ) +
            ggplot2::geom_point(...)
        } else {
          gg_plt <- ggplot2::ggplot(
            gg_dta_mlt,
            ggplot2::aes(x = .data$value, y = .data$yhat)
          ) +
            ggplot2::geom_point(...)
        }
      } else {
        # Warn if numeric and factor predictors are mixed in the same panel
        if (sum(ccls[wch_x_var] == "numeric") > 0) {
          warning("Mismatched variable types...
                  assuming these are all factor variables.")
        }

        if (family == "class") {
          gg_plt <-
            ggplot2::ggplot(
              gg_dta_mlt,
              ggplot2::aes(
                x = .data$value,
                y = .data$yhat,
                color = .data$yvar
              )
            ) +
            ggplot2::geom_boxplot(...)
        } else {
          gg_plt <- ggplot2::ggplot(
            gg_dta_mlt,
            ggplot2::aes(x = .data$value, y = .data$yhat)
          ) +
            ggplot2::geom_boxplot(...)
        }
      }
      # Add point/smooth layers for non-classification forests
      if (family != "class") {
        if (points) {
          gg_plt <- gg_plt +
            ggplot2::geom_point(...)
        }
        if (smooth) {
          gg_plt <- gg_plt +
            ggplot2::geom_smooth(...)
        }
      }

      gg_plt <- gg_plt +
        ggplot2::facet_wrap(~variable, scales = "free_x") +
        ggplot2::labs(x = "")
    }

  ## =========================================================
  ## INDIVIDUAL PLOT branch: one ggplot per predictor variable
  ## =========================================================
  } else {
    # Pre-allocate a list; collapsed to a single object when lng == 1
    gg_plt <- vector("list", length = lng)

    for (ind in seq_len(lng)) {
      # Temporarily rename the target predictor column to "var" for aes()
      ch_indx <- which(colnames(gg_dta) == xvar[ind])
      h_name <- colnames(gg_dta)[ch_indx]
      colnames(gg_dta)[ch_indx] <- "var"
      # Use only the primary class (class() can return multiple strings, e.g.
      # c("POSIXct", "POSIXt")); a multi-element vector in if() triggers a warning.
      ccls_var <- class(gg_dta[, "var"])[1L]
      if (ccls_var == "integer") ccls_var <- "numeric"

      gg_plt[[ind]] <- ggplot2::ggplot(gg_dta)

      ## ---- Survival individual plot -----------------------------------
      if (family == "surv") {
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Survival")

        if (ccls_var == "numeric") {
          # Continuous predictor: scatter (and optional smooth)
          if (points) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$event,
                  shape = .data$event
                ),
                ...
              )
          } else {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
          if (smooth) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
        } else {
          # Factor predictor: boxplot + jittered points coloured by event
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::geom_boxplot(
              ggplot2::aes(x = .data$var, y = .data$yhat),
              color = "black",
              ...,
              outlier.shape = NA
            ) +
            ggplot2::geom_jitter(
              ggplot2::aes(
                x = .data$var,
                y = .data$yhat,
                color = .data$event,
                shape = .data$event
              ),
              ...
            )
        }
        # Multiple time points: facet vertically by time
        if (length(levels(gg_dta$time)) > 1) {
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::facet_wrap(~time, ncol = 1)
        } else {
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::labs(
              x = h_name,
              y = paste("Survival at", gg_dta$time[1], "year")
            )
        }

      ## ---- Classification individual plot ----------------------------
      } else if (family == "class") {
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Predicted")

        if (sum(colnames(gg_dta) == "outcome") == 0) {
          # Single-outcome (binary) classification
          if (ccls_var == "numeric") {
            if (points) {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_point(
                  ggplot2::aes(
                    x = .data$var,
                    y = .data$yhat,
                    color = .data$yvar,
                    shape = .data$yvar
                  ),
                  ...
                )
            } else {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_smooth(
                  ggplot2::aes(x = .data$var, y = .data$yhat),
                  color = "black",
                  linetype = 2,
                  ...
                )
            }
            if (smooth) {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_smooth(
                  ggplot2::aes(x = .data$var, y = .data$yhat),
                  ...
                )
            }
          } else {
            # Factor predictor: jitter + boxplot coloured by observed class.
            # smooth=TRUE is intentionally a no-op here: geom_smooth requires
            # a continuous x-axis and has no meaningful interpretation for
            # discrete factor levels.  The boxplot IQR serves as the spread
            # summary.
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_jitter(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              ) +
              ggplot2::geom_boxplot(
                ggplot2::aes(x = .data$var, y = .data$yhat),
                color = "grey",
                ...,
                outlier.shape = NA
              )
          }
        } else {
          # Multi-class: facet by outcome class
          if (ccls_var == "numeric") {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              )
            if (smooth) {
              gg_plt[[ind]] <- gg_plt[[ind]] +
                ggplot2::geom_smooth(
                  ggplot2::aes(x = .data$var, y = .data$yhat),
                  ...
                )
            }
          } else {
            # Factor predictor (multi-class): boxplot + jitter per facet.
            # smooth=TRUE is intentionally a no-op here for the same reason
            # as the binary factor path above: geom_smooth requires a
            # continuous x-axis.
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_boxplot(
                ggplot2::aes(x = .data$var, y = .data$yhat),
                color = "grey",
                ...,
                outlier.shape = NA
              ) +
              ggplot2::geom_jitter(
                ggplot2::aes(
                  x = .data$var,
                  y = .data$yhat,
                  color = .data$yvar,
                  shape = .data$yvar
                ),
                ...
              )
          }

          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::facet_grid(~outcome)
        }

      ## ---- Regression individual plot --------------------------------
      } else {
        # assume regression
        gg_plt[[ind]] <- gg_plt[[ind]] +
          ggplot2::labs(x = h_name, y = "Predicted")
        if (ccls_var == "numeric") {
          if (points) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_point(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          } else {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
          if (smooth) {
            gg_plt[[ind]] <- gg_plt[[ind]] +
              ggplot2::geom_smooth(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
          }
        } else {
          # Factor predictor (regression): boxplot + jitter.
          # smooth=TRUE is intentionally a no-op here: geom_smooth requires a
          # continuous x-axis and has no meaningful interpretation for discrete
          # factor levels.  The boxplot IQR serves as the spread summary.
          gg_plt[[ind]] <- gg_plt[[ind]] +
            ggplot2::geom_boxplot(
              ggplot2::aes(x = .data$var, y = .data$yhat),
              color = "grey",
              ...,
              outlier.shape = NA
            ) +
            ggplot2::geom_jitter(ggplot2::aes(x = .data$var, y = .data$yhat), ...)
        }
      }

      # Restore the original column name before the next iteration
      colnames(gg_dta)[ch_indx] <- h_name
    }
    # Return a single object: one ggplot for a single variable, otherwise a
    # patchwork composite (one panel per variable). Never a bare list; see
    # #80 / NEWS; mirrors the v2.7.3 #77/#78 plot.gg_partial* unification.
    if (lng == 1) {
      gg_plt <- gg_plt[[1]]
    } else {
      gg_plt <- patchwork::wrap_plots(gg_plt, ncol = 1)
    }
  }
  return(gg_plt)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Random forest error trajectory data object
#'
#' Extract the cumulative out-of-bag (OOB) or in-bag training error rate from
#' \code{randomForestSRC} and \code{randomForest} fits as a function of the
#' number of grown trees.
#'
#' @details For \code{randomForestSRC} objects the function reshapes the
#' \code{\link[randomForestSRC]{rfsrc}$err.rate} matrix and annotates it with
#' the tree index required by \code{\link{plot.gg_error}}. When supplied a
#' \code{\link[randomForest]{randomForest}} object, the method inspects either
#' the \code{$mse} or \code{$err.rate} component and, when
#' \code{training = TRUE} is requested, reconstructs the original training set
#' via the model call to compute an in-bag error curve using per-tree
#' predictions. Training curves are only available when the forest was stored
#' (\code{keep.forest = TRUE}) and the original data can be recovered.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object.
#' @param ... Optional arguments passed to the methods. Set
#'   \code{training = TRUE} to append the in-bag error trajectory when
#'   supported.
#'
#' @return A \code{gg_error} \code{data.frame} containing at least the
#'   cumulative OOB error columns and an \code{ntree} counter. When
#'   \code{training = TRUE} is honored an additional \code{train} column is
#'   included.
#'
#' @seealso \code{\link{plot.gg_error}}, \code{\link{gg_vimp}},
#'   \code{\link{gg_variable}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForest]{randomForest}},
#'   \code{\link[randomForestSRC]{plot.rfsrc}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
#' Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @aliases gg_error gg_error.rfsrc gg_error.randomForest
#' @aliases gg_error.randomForest.formula
#'
#' @examples
#' ## Examples from RFSRC package...
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## ------------- iris data
#' ## You can build a randomForest
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, tree.err = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_iris)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## RandomForest example
#' rf_iris <- randomForest::randomForest(Species ~ .,
#'   data = iris,
#'   tree.err = TRUE,
#' )
#' gg_dta <- gg_error(rf_iris)
#' plot(gg_dta)
#'
#' gg_dta <- gg_error(rf_iris, training = TRUE)
#' plot(gg_dta)
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#'
#' ## ------------- airq data
#' rfsrc_airq <- rfsrc(Ozone ~ .,
#'   data = airquality,
#'   na.action = "na.impute", tree.err = TRUE,
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_airq)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- Boston data
#' data(Boston, package = "MASS")
#' Boston$chas <- as.logical(Boston$chas)
#' rfsrc_boston <- rfsrc(medv ~ .,
#'   data = Boston,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_boston)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- mtcars data
#' rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, tree.err = TRUE)
#'

#' # Get a data.frame containing error rates
#' gg_dta<- gg_error(rfsrc_mtcars)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## ------------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        tree.err = TRUE)
#'
#' gg_dta <- gg_error(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## ------------- pbc data
#' # Load a cached randomForestSRC object
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC",)
#' # For whatever reason, the age variable is in days... makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'  if (!is.factor(pbc[, ind])) {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  } else {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  }
#'  if (!is.logical(pbc[, ind]) &
#'      length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'    pbc[, ind] <- factor(pbc[, ind])
#'  }
#' }
#' #Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' #========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'  dta_train,
#'  nsplit = 10,
#'  na.action = "na.impute",
#'  tree.err = TRUE,
#'  forest = TRUE,
#'  importance = TRUE,
#'  save.memory = TRUE
#' )
#'
#'
#' gg_dta <- gg_error(rfsrc_pbc)
#' plot(gg_dta)
#'
#' @importFrom stats as.formula model.frame model.response na.omit predict qnorm
#'
#' @export
gg_error <- function(object, ...) {
  UseMethod("gg_error", object)
}
#' @export
gg_error.rfsrc <- function(object, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "rfsrc")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForestSRC package."
      )
    )
  }
  # The forest must have been grown with tree.err = TRUE so that per-tree
  # OOB error rates are recorded in $err.rate.
  if (is.null(object$err.rate)) {
    stop("Performance values are not available for this forest.")
  }

  # Convert the err.rate matrix (ntree × n_outcomes) to a data frame.
  gg_dta <- data.frame(object$err.rate)

  # rfsrc wraps single-column matrices with a column name derived from the
  # object name; rename it to the neutral label "error" for downstream use.
  if ("object.err.rate" %in% colnames(gg_dta)) {
    colnames(gg_dta)[which(colnames(gg_dta) == "object.err.rate")] <-
      "error"
  }

  # Add a sequential tree counter required by the x-axis of plot.gg_error.
  gg_dta$ntree <- seq_len(dim(gg_dta)[1])

  # Optional in-bag training error: re-predict on the full training set using
  # the stored forest and record the resulting per-tree error trajectory.
  arg_list <- list(...)
  training <- isTRUE(arg_list$training)
  if (training) {
    trn <- data.frame(cbind(object$xvar, object$yvar))
    colnames(trn) <- c(object$xvar.names, object$yvar.names)
    gg_prd <- predict(
      object,
      newdata = trn,
      importance = "none",
      membership = FALSE
    )
    gg_dta$train <- gg_prd$err.rate
  }
  gg_dta <- na.omit(gg_dta)
  class(gg_dta) <- c("gg_error", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_error.randomForest <- function(object, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForest package."
      )
    )
  }

  if (!is.null(object$mse)) {
    # Regression forests store the cumulative OOB mean squared error in $mse.
    gg_dta <- data.frame(object$mse)

    # Normalise the auto-generated column name to "error".
    if ("object.mse" %in% colnames(gg_dta)) {
      colnames(gg_dta)[which(colnames(gg_dta) == "object.mse")] <-
        "error"
    }

    gg_dta$ntree <- seq_len(nrow(gg_dta))

    arg_list <- list(...)
    training <- isTRUE(arg_list$training)

    # Optionally compute and append the per-tree in-bag training error curve.
    if (training) {
      train_curve <- .rf_training_curve(object)
      if (!is.null(train_curve)) {
        gg_dta$train <- train_curve
      }
    }
  } else if (!is.null(object$err.rate)) {
    # Classification forests store the cumulative OOB error matrix in
    # $err.rate (rows = trees, columns = overall + per-class error rates).
    gg_dta <- data.frame(object$err.rate)

    gg_dta$ntree <- seq_len(nrow(gg_dta))

    arg_list <- list(...)
    training <- isTRUE(arg_list$training)
    if (training) {
      train_curve <- .rf_training_curve(object)
      if (!is.null(train_curve)) {
        gg_dta$train <- train_curve
      }
    }
  } else {
    stop("Performance values are not available for this forest.")
  }
  class(gg_dta) <- c("gg_error", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_error.randomForest.formula <- gg_error.randomForest

.rf_training_curve <- function(object) {
  if (is.null(object$forest)) {
    warning(
      "Training error curve is unavailable because the forest was not saved. ",
      "Refit with keep.forest = TRUE to enable training=TRUE."
    )
    return(NULL)
  }
  training_info <- .rf_recover_model_frame(object)
  if (is.null(training_info)) {
    warning(
      "Unable to reconstruct the training data for this randomForest object;",
      " training=TRUE is ignored."
    )
    return(NULL)
  }

  training_frame <- training_info$model_frame
  response <- stats::model.response(training_frame)
  resp_name <- training_info$response_name
  predictors <- training_frame
  if (!is.null(resp_name) && resp_name %in% colnames(predictors)) {
    predictors[[resp_name]] <- NULL
  }
  special_cols <- grep("^\\(", colnames(predictors), value = TRUE)
  if (length(special_cols) > 0) {
    predictors[special_cols] <- NULL
  }
  predictors <- as.data.frame(predictors)
  if (ncol(predictors) == 0) {
    warning("No predictor columns available to compute training curve.")
    return(NULL)
  }

  pred_all <- predict(object,
    newdata = predictors,
    predict.all = TRUE
  )

  if (is.null(pred_all$individual)) {
    warning("Unable to extract per-tree predictions; training=TRUE ignored.")
    return(NULL)
  }

  individual <- pred_all$individual
  if (object$type == "regression") {
    return(.rf_training_curve_regression(individual, response))
  }
  if (object$type == "classification") {
    return(.rf_training_curve_classification(individual, response, object$classes))
  }
  warning("Training error curves are not supported for this forest type.")
  NULL
}

.rf_training_curve_regression <- function(individual, response) {
  cum_preds <- t(apply(individual, 1, cumsum))
  ntree <- ncol(individual)
  pred_by_tree <- sweep(cum_preds, 2, seq_len(ntree), "/")
  mse <- colMeans((pred_by_tree - response)^2)
  as.numeric(mse)
}

.rf_training_curve_classification <- function(individual, response, classes) {
  response <- factor(response, levels = classes)
  ntree <- ncol(individual)
  nobs <- nrow(individual)
  votes <- matrix(0, nrow = nobs, ncol = length(classes))
  colnames(votes) <- classes
  err <- numeric(ntree)

  for (tree in seq_len(ntree)) {
    preds <- factor(individual[, tree], levels = classes)
    pred_index <- as.integer(preds)
    valid <- !is.na(pred_index)
    idx <- cbind(seq_len(nobs)[valid], pred_index[valid])
    if (nrow(idx) > 0) {
      votes[idx] <- votes[idx] + 1
    }
    agg <- factor(classes[max.col(votes, ties.method = "first")],
      levels = classes
    )
    err[tree] <- mean(agg != response)
  }
  err
}

.rf_recover_model_frame <- function(object) {
  if (is.null(object$call$formula)) {
    return(NULL)
  }
  formula <- stats::as.formula(object$call$formula)
  data_env <- environment(formula)
  if (is.null(data_env)) {
    data_env <- parent.frame()
  }
  data_expr <- object$call$data
  if (is.null(data_expr)) {
    return(NULL)
  }
  mf_list <- c(
    list(quote(stats::model.frame)),
    list(formula = formula, data = data_expr)
  )
  optional_args <- c("subset", "weights", "na.action", "offset")
  for (arg in optional_args) {
    arg_value <- object$call[[arg]]
    if (!is.null(arg_value)) {
      mf_list[[arg]] <- arg_value
    }
  }
  mf_call <- as.call(mf_list)
  env_candidates <- c(list(data_env), rev(sys.frames()), list(.GlobalEnv))
  mf <- NULL
  for (env in env_candidates) {
    mf <- tryCatch(
      eval(mf_call, envir = env),
      error = function(e) NULL
    )
    if (!is.null(mf)) {
      break
    }
  }
  if (is.null(mf)) {
    return(NULL)
  }
  terms_obj <- attr(mf, "terms")
  resp_idx <- attr(terms_obj, "response")
  resp_name <- NULL
  if (!is.null(resp_idx) && resp_idx > 0) {
    resp_name <- colnames(mf)[resp_idx]
  }
  list(
    model_frame = mf,
    response_name = resp_name
  )
}

# testthat for gg_error function

test_that("gg_error.rfsrc classifications", {
  ## Load the cached forest
  data(iris, package = "datasets")
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE
  )
  # Test the cached forest type
  expect_s3_class(rfsrc_iris, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_iris$family, "class")

  ## Create the correct gg_error object
  gg_dta <- gg_error(rfsrc_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  # Test classification dimensions
  expect_equal(dim(gg_dta)[1], dim(na.omit(rfsrc_iris$err.rate))[1])
  expect_equal(dim(gg_dta)[2], dim(rfsrc_iris$err.rate)[2] + 1)

  # Test data is correctly pulled from randomForest obect.
  # expect_equal(as.matrix(gg_dta[, -which(colnames(gg_dta) == "ntree")], ignore_attr = TRUE),
  #                   rfsrc_iris$err.rate)

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # "Incorrect object type: Expects a gg_error object"
  expect_error(gg_error(gg_plt))
  expect_error(gg_error.rfsrc(gg_plt))
  rfsrc_iris$err.rate <- NULL
  expect_error(gg_error(rfsrc_iris))

  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE
  )
  ## Create the correct gg_error object
  gg_dta <- gg_error(rfsrc_iris, training = TRUE)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  ## Test plotting the gg_error object (no warnings expected since pivot_longer migration)
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")
})


test_that("gg_error.randomForest classifications", {
  ## Load the cached forest
  data(iris, package = "datasets")
  ## Load the cached forest
  rf_iris <- randomForest::randomForest(Species ~ .,
                                        data = iris,
                                        ntree = 75)

  # Test the cached forest type
  expect_s3_class(rf_iris, "randomForest")

  # Test the forest family
  expect_match(rf_iris$type, "classification")

  ## Create the correct gg_error object
  gg_dta <- gg_error(rf_iris)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  # Test classification dimensions
  expect_equal(dim(gg_dta)[1], dim(rf_iris$err.rate)[1])
  expect_equal(dim(gg_dta)[2], dim(rf_iris$err.rate)[2] + 1)

  # Test data is correctly pulled from randomForest obect.
  expect_equal(as.matrix(gg_dta[, -which(colnames(gg_dta) == "ntree")], ignore_attr = TRUE),
                    rf_iris$err.rate)

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  ## Ensure training curve can be requested
  gg_train <- gg_error(rf_iris, training = TRUE)
  expect_true("train" %in% names(gg_train))
  expect_length(gg_train$train, nrow(gg_train))
  expect_true(min(gg_train$train, na.rm = TRUE) >= 0)
  expect_true(max(gg_train$train, na.rm = TRUE) <= 1)

  # "Incorrect object type: Expects a gg_error object"
  expect_error(gg_error(gg_plt))
  expect_error(gg_error.randomForest(gg_plt))
  rf_iris$err.rate <- NULL
  expect_error(gg_error(rf_iris))


})



test_that("gg_error regression rfsrc", {
  ## Load the cached forest
  data(Boston, package = "MASS")

  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter

  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston)
  # Test the cached forest type
  expect_s3_class(rfsrc_boston, "rfsrc")

  # Test the forest family
  expect_match(rfsrc_boston$family, "regr")

  ## Create the correct gg_error object
  gg_dta <- gg_error(rfsrc_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  # Test classification dimensions
  expect_equal(nrow(gg_dta), length(na.omit(rfsrc_boston$err.rate)))
  expect_equal(ncol(gg_dta), 2)

  # Test data is correctly pulled from randomForest object.
  expect_equal(as.numeric(gg_dta[[1]]), as.numeric(na.omit(rfsrc_boston$err.rate)))

  ## Test plotting the gg_error object
  gg_plt <- plot(gg_dta)

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Test return is s ggplot object
  expect_s3_class(gg_plt, "ggplot")

  # Test the exception for input
  expect_error(gg_error(gg_plt))


  gg_dta <- gg_error(rfsrc_boston, training = TRUE)
  expect_s3_class(gg_dta, "gg_error")


})


test_that("gg_error regression randomForest", {
  ## Load the cached forest
  data(Boston, package = "MASS")

  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter

  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston, ntree = 100)
  # Test the cached forest type
  expect_s3_class(rf_boston, "randomForest")

  # Test the forest family
  expect_match(rf_boston$type, "regression")

  ## Create the correct gg_error object
  gg_dta <- gg_error(rf_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  # Test classification dimensions
  expect_equal(nrow(gg_dta), length(na.omit(rf_boston$mse)))
  expect_equal(ncol(gg_dta), 2)

  # Test data is correctly pulled from randomForest obect.
  expect_equal(c(gg_dta[, 1]), rf_boston$mse, ignore_attr = TRUE)

  ## Create the correct gg_error object
  gg_dta <- gg_error(rf_boston)

  # Test object type
  expect_s3_class(gg_dta, "gg_error")

  ## Training curve is available
  gg_train <- gg_error(rf_boston, training = TRUE)
  expect_true("train" %in% names(gg_train))
  expect_length(gg_train$train, nrow(gg_train))
  expect_true(all(is.finite(gg_train$train)))

})

## ---- Direct plot.gg_error() tests -----------------------------------------
# The tests above exercise plot() via S3 dispatch. The tests below call
# plot.gg_error() by name to ensure the function itself is exercised directly
# and all branches are covered explicitly.

test_that("plot.gg_error direct: regression rfsrc (single-outcome path, no legend)", {
  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston, ntree = 50L)
  gg_dta <- gg_error(rfsrc_boston)

  # Call by name — not via generic dispatch
  gg_plt <- plot.gg_error(gg_dta)

  expect_s3_class(gg_plt, "ggplot")
  # Single-outcome: gg_dta has exactly 2 columns (error, ntree)
  expect_equal(ncol(gg_dta), 2L)
  # Legend should be suppressed — theme has legend.position = "none"
  expect_equal(gg_plt$theme$legend.position, "none")
})

test_that("plot.gg_error direct: survival rfsrc (single-outcome path)", {
  data(pbc, package = "randomForestSRC")
  pbc$time <- pbc$days / 364.25
  pbc_sub <- pbc[, c("time", "status", "age", "bili")]
  Surv <- survival::Surv  # nolint: object_name_linter
  rfsrc_pbc <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = pbc_sub,
    ntree = 50L
  )
  gg_dta <- gg_error(rfsrc_pbc)
  gg_plt <- plot.gg_error(gg_dta)

  expect_s3_class(gg_plt, "ggplot")
  expect_equal(ncol(gg_dta), 2L)
})

test_that("plot.gg_error direct: classification rfsrc (multi-outcome path, legend shown)", {
  data(iris, package = "datasets")
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ ., data = iris, importance = TRUE, tree.err = TRUE
  )
  gg_dta <- gg_error(rfsrc_iris)
  gg_plt <- plot.gg_error(gg_dta)

  expect_s3_class(gg_plt, "ggplot")
  # Multi-outcome: more than 2 columns before pivot
  expect_true(ncol(gg_dta) > 2L)
  # Legend should NOT be suppressed — multiple outcomes need the colour key
  expect_false(identical(gg_plt$theme$legend.position, "none"))
})

test_that("plot.gg_error direct: accepts raw rfsrc object (auto-extract path)", {
  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston, ntree = 50L)

  # Pass the rfsrc object directly — plot.gg_error should call gg_error() internally
  gg_plt <- plot.gg_error(rfsrc_boston)
  expect_s3_class(gg_plt, "ggplot")
})

test_that("plot.gg_error direct: errors on non-gg_error non-rfsrc input", {
  expect_error(plot.gg_error(list(a = 1)), "Incorrect object type")
  expect_error(plot.gg_error("not a forest"), "Incorrect object type")
})

test_that("plot.gg_error direct: point geometry used when only one valid row", {
  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston, ntree = 50L)
  gg_dta <- gg_error(rfsrc_boston)

  # Manufacture a single-row gg_error to trigger the point branch
  single_row <- gg_dta[1L, ]
  class(single_row) <- class(gg_dta)
  gg_plt <- plot.gg_error(single_row)

  expect_s3_class(gg_plt, "ggplot")
  # Confirm geom_point was used (not geom_line)
  geom_classes <- sapply(gg_plt$layers, function(l) class(l$geom)[1])
  expect_true(any(grepl("GeomPoint", geom_classes)))
})

# Regression guard for the v2.8.0 Phase 0 dependency modernization.
# Asserts the Depends->Imports migration took effect and stays effective.

test_that("DESCRIPTION Depends declares only the R version constraint", {
  # system.file locates DESCRIPTION in installed contexts (R CMD check,
  # installed-from-source) and via pkgload's shim under devtools::test.
  # Fall back to the relative path for bare testthat::test_file on the
  # source tree, where the package is not installed (system.file -> "").
  desc_path <- system.file("DESCRIPTION", package = "ggRandomForests")
  if (!nzchar(desc_path)) desc_path <- testthat::test_path("..", "..", "DESCRIPTION")
  deps <- read.dcf(desc_path, fields = "Depends")[1, 1]
  deps_pkgs <- trimws(strsplit(deps, ",")[[1]])
  # Every entry must be the R(>= x) constraint, not a package.
  expect_true(all(grepl("^R([[:space:]]|$)", deps_pkgs)),
              info = paste("Depends still attaches packages:", deps))
})

test_that("core extractors run without randomForestSRC/randomForest attached", {
  skip_if_not_installed("callr")
  skip_if_not_installed("randomForestSRC")
  skip_if_not_installed("randomForest")
  # Run ONLY under R CMD check, where the package under test is freshly
  # installed into the check library. `_R_CHECK_PACKAGE_NAME_` is set by
  # R CMD check during its test phase and is absent under devtools::test()
  # / bare testthat::test_file(), so this skips there instead of loading a
  # possibly-stale separately-installed ggRandomForests in the callr child.
  skip_if_not(
    nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_")),
    "clean-session test runs only under R CMD check"
  )

  # Pass the test process's library paths into the child so it resolves
  # ggRandomForests from the SAME library as the check run (the freshly
  # built copy), not an unrelated installed version.
  out <- callr::r(
    function() {
      # Load ggRandomForests WITHOUT attaching its former Depends.
      suppressMessages(library(ggRandomForests))
      attached <- search()
      stopifnot(!("package:randomForestSRC" %in% attached))
      stopifnot(!("package:randomForest" %in% attached))

      # A regression forest extractor must work with rfsrc only namespaced.
      rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                                   ntree = 30, na.action = "na.impute")
      g <- gg_error(rf)
      inherits(g, "gg_error")
    },
    libpath = .libPaths()
  )
  expect_true(out)
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
# Internal helper: normalise the which_outcome argument.
# "all" is not yet fully supported; fall back to class 1 with a warning.
.validate_which_outcome <- function(which_outcome) {
  if (identical(which_outcome, "all")) {
    warning("Must specify which_outcome for now.")
    return(1L)
  }
  which_outcome
}

#' Receiver Operator Characteristic calculator
#'
#' @details For a randomForestSRC prediction and the actual
#' response value, calculate the specificity (1-False Positive Rate) and
#' sensitivity (True Positive Rate) of a predictor.
#'
#' This is a helper function for the \code{\link{gg_roc}} functions, and
#' not intended for use by the end user.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForestSRC]{predict.rfsrc}}, or
#'   \code{\link[randomForest]{randomForest}} classification object containing
#'   predicted class probabilities.
#' @param dta A factor (or coercible to factor) of the true observed class
#'   labels, one per observation. Typically \code{object$yvar} for rfsrc or
#'   \code{object$y} for randomForest.
#' @param which_outcome Integer index of the class for which the ROC curve is
#'   computed (e.g. \code{1} for the first class, \code{2} for the second).
#'   Use \code{"all"} to request all classes (currently falls back to class 1
#'   with a warning).
#' @param oob Logical; if \code{TRUE} (default for rfsrc) use OOB predicted
#'   probabilities. Forced to \code{FALSE} for \code{randomForest} objects.
#' @param ... Extra arguments passed to helper functions (currently unused).
#'
#' @return A \code{gg_roc} \code{data.frame} with columns \code{sens}
#'   (sensitivity), \code{spec} (specificity), and \code{pct} (the probability
#'   threshold), with one row per unique prediction value. Suitable for passing
#'   to \code{\link{calc_auc}} or \code{\link{plot.gg_roc}}.
#'
#' @aliases calc_roc.rfsrc calc_roc.randomForest calc_roc
#'
#' @seealso \code{\link{calc_auc}} \code{\link{gg_roc}}
#' @seealso \code{\link{plot.gg_roc}}
#'
#' @importFrom stats xtabs
#' @importFrom utils head tail
#' @importFrom randomForest randomForest
#'
#' @examples
#' ## Taken from the gg_roc example
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)
#'
#' gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
#'   which_outcome = 1, oob = TRUE
#' )
#' gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
#'   which_outcome = 1, oob = FALSE
#' )
#'
#' rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
#' # randomForest stores the response in $y (rfsrc uses $yvar); pass the
#' # original training factor so calc_roc has the class labels.
#' gg_dta <- calc_roc(rf_iris, iris$Species,
#'   which_outcome = 1
#' )
#' gg_dta <- calc_roc(rf_iris, iris$Species,
#'   which_outcome = 2
#' )
#'
#' @export
calc_roc.rfsrc <-
  function(object,
           dta,
           which_outcome = "all",
           oob = TRUE,
           ...) {
    # Ensure response is a factor so levels() is well-defined
    if (!is.factor(dta)) {
      dta <- factor(dta)
    }

    # "all" outcomes not yet supported; fall back to the first class
    which_outcome <- .validate_which_outcome(which_outcome)
    # Build (binary indicator, full-forest prediction, OOB prediction) triplet
    dta_roc <-
      data.frame(cbind(
        res = (dta == levels(dta)[which_outcome]),
        prd = object$predicted[, which_outcome],
        oob_prd = object$predicted.oob[, which_outcome]
      ))

    # Collect the unique predicted probability thresholds for the ROC sweep
    if (oob) {
      pct <- sort(unique(object$predicted.oob[, which_outcome]))
    } else {
      pct <- sort(unique(object$predicted[, which_outcome]))
    }

    last <- length(pct)
    # Remove the maximum threshold (the cutpoint where nothing is classified
    # as positive), which produces the (sens=0, spec=1) anchor point
    pct <- pct[-last]

    # Cap at 200 threshold points to keep multi-class ROC plots manageable
    if (last > 200) {
      pct <- pct[seq(1, length(pct), length.out = 200)]
    }

    # For each threshold, build the 2×2 confusion table and extract TPR/TNR
    gg_dta <- lapply(pct, function(crit) {
      if (oob) {
        tbl <- xtabs(~ res + (oob_prd > crit), dta_roc)
      } else {
        tbl <- xtabs(~ res + (prd > crit), dta_roc)
      }

      spec <- tbl[2, 2] / rowSums(tbl)[2]
      sens <- tbl[1, 1] / rowSums(tbl)[1]
      cbind(sens = sens, spec = spec)
    })

    gg_dta <- do.call(rbind, gg_dta)
    # Anchor curve at perfect specificity (0, 1) and perfect sensitivity (1, 0)
    gg_dta <- rbind(c(0, 1), gg_dta, c(1, 0))

    gg_dta <- data.frame(gg_dta, row.names = seq_len(nrow(gg_dta)))
    gg_dta$pct <- c(0, pct, 1)
    invisible(gg_dta)
  }

#' @export
calc_roc <- function(object,
                     dta,
                     which_outcome = "all",
                     oob = TRUE,
                     ...) {
  UseMethod("calc_roc", object)
}

## randomForest ROC: OOB vote probabilities by default; macro-average for
## the overall ("all"/0) case. Does NOT use the shared
## .validate_which_outcome (rfsrc path unchanged). See #81.
##
## The body is intentionally thin: heavy logic lives in unexported
## helpers (.rf_prob_matrix, .normalize_which_outcome, .rf_one_class_roc,
## .rf_macro_average_roc) to keep cyclomatic complexity under the
## lintr cyclocomp_linter threshold and to make each step testable in
## isolation.

# Build the per-class probability matrix used by the ROC sweep:
#   oob = TRUE  -> object$votes (OOB class proportions; honest)
#   else        -> predict(object, type = "prob") (in-bag)
# randomForest votes can be raw counts; row-normalise to probabilities.
.rf_prob_matrix <- function(object, oob, lvls) {
  prob <- if (isTRUE(oob) && !is.null(object$votes)) {
    as.matrix(object$votes)
  } else {
    stats::predict(object, type = "prob")
  }
  rs <- rowSums(prob)
  if (any(rs > 1 + 1e-8, na.rm = TRUE)) {
    prob <- prob / rs
  }
  colnames(prob) <- lvls
  prob
}

# Coerce/validate which_outcome. Returns either the string "all" (for
# the overall macro-average case) or a single integer class index.
# Accepts: "all", numeric == 0 (handles 0 and 0L), character class
# names (matched against lvls), integer/double indices in 1:nlvls.
# Errors on unknown class names or out-of-range indices.
.normalize_which_outcome <- function(which_outcome, lvls) {
  if (is.character(which_outcome) && !identical(which_outcome, "all")) {
    idx <- match(which_outcome, lvls)
    if (anyNA(idx)) {
      stop("Unknown class name(s) in which_outcome: ",
           paste(which_outcome[is.na(idx)], collapse = ", "),
           ". Must be one of: ", paste(lvls, collapse = ", "))
    }
    return(idx)
  }
  if (is.numeric(which_outcome) && length(which_outcome) == 1L &&
      which_outcome == 0) {
    return("all")
  }
  if (is.numeric(which_outcome) &&
      (any(which_outcome < 1) || any(which_outcome > length(lvls)))) {
    stop("which_outcome out of range; must be in 1:", length(lvls),
         ' or "all" / 0.')
  }
  which_outcome
}

# Build the sensitivity/specificity table for a single class index k.
# Plain lapply (not mclapply): per-threshold work is a single table()
# + a few arithmetic ops (microseconds); fork overhead would dominate,
# and the closure-scope fragility caused the earlier xtabs/Windows
# failure. Returns a data.frame with columns sens, spec, pct.
.rf_one_class_roc <- function(dta, prob, k, lvls) {
  res <- dta == lvls[k]
  score <- prob[, k]
  pct <- sort(unique(score))
  if (length(pct) > 1) pct <- pct[-length(pct)]
  if (length(pct) > 200) {
    pct <- pct[seq(1, length(pct), length.out = 200)]
  }
  rc <- lapply(pct, function(crit) {
    tbl <- table(res, score > crit)
    if (ncol(tbl) < 2) {
      tbl <- cbind(tbl, c(0, 0))
      colnames(tbl) <- c("FALSE", "TRUE")
    }
    spec <- tbl[2, 2] / rowSums(tbl)[2]
    sens <- tbl[1, 1] / rowSums(tbl)[1]
    cbind(sens = sens, spec = spec)
  })
  rc <- do.call(rbind, rc)
  rc <- rbind(c(0, 1), rc, c(1, 0))
  data.frame(rc, pct = c(0, pct, 1), row.names = seq_len(nrow(rc)))
}

# Macro-average one-vs-rest ROC on a shared FPR grid.
.rf_macro_average_roc <- function(dta, prob, lvls) {
  curves <- lapply(seq_along(lvls),
                   function(k) .rf_one_class_roc(dta, prob, k, lvls))
  grid <- seq(0, 1, length.out = 200)
  interp <- vapply(curves, function(cv) {
    cv <- cv[order(cv$spec, decreasing = TRUE), ]
    stats::approx(1 - cv$spec, cv$sens, xout = grid,
                  ties = "ordered", rule = 2)$y
  }, numeric(length(grid)))
  data.frame(
    sens = rowMeans(interp, na.rm = TRUE),
    spec = 1 - grid,
    pct  = grid,
    row.names = seq_along(grid)
  )
}

#' @export
calc_roc.randomForest <-
  function(object,
           dta,
           which_outcome = "all",
           oob = TRUE,
           ...) {
    if (!is.factor(dta)) {
      dta <- factor(dta)
    }
    lvls <- levels(dta)
    prob <- .rf_prob_matrix(object, oob, lvls)
    which_outcome <- .normalize_which_outcome(which_outcome, lvls)

    gg_dta <- if (identical(which_outcome, "all")) {
      .rf_macro_average_roc(dta, prob, lvls)
    } else {
      .rf_one_class_roc(dta, prob, which_outcome, lvls)
    }
    invisible(gg_dta)
  }

#'
#' Area Under the ROC Curve calculator
#'
#' @details calc_auc uses the trapezoidal rule to calculate the area under
#' the ROC curve.
#'
#'  This is a helper function for the \code{\link{gg_roc}} functions.
#'
#' @param x \code{\link{gg_roc}} object
#'
#' @return AUC. 50\% is random guessing, higher is better.
#'
# @importFrom dplyr lead
#'
#' @seealso \code{\link{calc_roc}} \code{\link{gg_roc}}
#' @seealso \code{\link{plot.gg_roc}}
#'
#' @examples
#' ##
#' ## Taken from the gg_roc example
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)
#'
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#'
#' calc_auc(gg_dta)
#'
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#'
#' calc_auc(gg_dta)
#'
#' ## randomForest tests
#' rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#'
#' calc_auc(gg_dta)
#'
#' @aliases calc_auc calc_auc.gg_roc
#' @export
calc_auc <- function(x) {
  ## Trapezoidal rule:  AUC = Σ (f(x_i) + f(x_{i+1})) / 2 * |Δx|
  ## Here f(x) is sensitivity (TPR) and x is 1 - specificity (FPR).
  ## Sort so that specificity decreases (FPR increases) left-to-right,
  ## then each step moves one FPR increment to the right.

  # Sort in decreasing specificity so FPR = 1-spec increases monotonically
  x <- x[order(x$spec, decreasing = TRUE), ]

  # Trapezoid area = sens_avg * Δspec, where Δspec = spec[i] - spec[i+1] > 0
  # (spec decreases left-to-right). This equals sens_avg * Δ(1-FPR), which
  # gives the standard AUC = ∫ sens d(FPR) with a positive sign.
  auc <- (x$sens + shift(x$sens)) / 2 * (x$spec - shift(x$spec)) # nolint: object_usage_linter
  sum(auc, na.rm = TRUE)
}

calc_auc.gg_roc <- calc_auc # nolint: object_name_linter

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************

# Map partial.type ("surv" / "chf" / "mort") to a human y-axis label.
# Falls back to "Predicted Survival" when the attribute is absent (e.g. an
# object built before this attribute was introduced).
partial_surv_y_label <- function(partial.type) {
  if (is.null(partial.type)) return("Predicted Survival")
  switch(partial.type,
         surv = "Predicted Survival",
         chf  = "Predicted CHF",
         mort = "Predicted Mortality",
         "Predicted Survival")
}

#' Plot a \code{\link{gg_partial}} object
#'
#' Produces ggplot2 partial dependence curves from the named list returned by
#' \code{\link{gg_partial}}.  Continuous predictors are shown as line plots;
#' categorical predictors are shown as bar charts.  Both panels are faceted by
#' variable name so multiple predictors can be compared at a glance.
#'
#' @param x A \code{\link{gg_partial}} object (output of \code{\link{gg_partial}}).
#' @param ... Not currently used; reserved for future arguments.
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.  When only one
#'   variable type is present a single \code{ggplot} is returned.  When both
#'   continuous and categorical variables are present the two panels are
#'   combined vertically via \code{patchwork::wrap_plots()}, which also
#'   satisfies \code{inherits(p, "ggplot")}.
#'
#' @seealso \code{\link{gg_partial}}, \code{\link{plot.gg_variable}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' pv <- randomForestSRC::plot.variable(rf, partial = TRUE, show.plots = FALSE)
#' pd <- gg_partial(pv)
#' plot(pd)
#'
#' @importFrom ggplot2 .data
#' @importFrom patchwork wrap_plots
#' @export
plot.gg_partial <- function(x, ...) {
  gg_dta <- x

  gg_cont <- NULL
  if (!is.null(gg_dta$continuous) && nrow(gg_dta$continuous) > 0) {
    cont <- gg_dta$continuous
    gg_cont <- ggplot2::ggplot(cont,
                               ggplot2::aes(x = .data$x, y = .data$yhat)) +
      ggplot2::geom_line()

    if ("model" %in% colnames(cont)) {
      gg_cont <- gg_cont +
        ggplot2::aes(color = .data$model, group = .data$model)
    }

    gg_cont <- gg_cont +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  gg_cat <- NULL
  if (!is.null(gg_dta$categorical) && nrow(gg_dta$categorical) > 0) {
    cat_dta <- gg_dta$categorical
    gg_cat <- ggplot2::ggplot(cat_dta,
                              ggplot2::aes(x = .data$x, y = .data$yhat)) +
      ggplot2::geom_bar(stat = "identity", width = 0.5) +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

#' Plot a \code{\link{gg_partial_rfsrc}} object
#'
#' Produces ggplot2 partial dependence curves from the named list returned by
#' \code{\link{gg_partial_rfsrc}}.
#'
#' For standard (non-survival) forests: continuous predictors are line plots,
#' categorical predictors are bar charts, both faceted by variable name.
#'
#' For survival forests (when a \code{time} column is present): each evaluation
#' time point is a separate curve over the predictor's value, faceted by
#' variable name. The y-axis label adapts to the \code{partial.type} stored on
#' the object (\dQuote{Predicted Survival}, \dQuote{Predicted CHF}, or
#' \dQuote{Predicted Mortality}).
#'
#' For two-variable surface plots (when a \code{grp} column is present):
#' each group level is a separate line, faceted by primary predictor name.
#'
#' @param x A \code{\link{gg_partial_rfsrc}} object.
#' @param ... Not currently used.
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.  When both continuous
#'   and categorical variables are present the two panels are combined
#'   vertically via \code{patchwork::wrap_plots()}.
#'
#' @seealso \code{\link{gg_partial_rfsrc}}, \code{\link{plot.gg_partial}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## Regression forest -- one continuous curve per variable
#' ## ------------------------------------------------------------
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#'
#' pd <- gg_partial_rfsrc(rfsrc_airq, xvar.names = c("Wind", "Temp"),
#'                        n_eval = 10)
#' plot(pd)
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Survival forest -- one curve per requested time horizon,
#' ## faceted by variable. Y-axis label tracks `partial.type`.
#' ## ------------------------------------------------------------
#' # randomForestSRC's formula parser requires the unqualified Surv() symbol;
#' # it Depends on `survival`, so Surv is on the search path once
#' # randomForestSRC is loaded.
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_v <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
#'                                   data = veteran, ntree = 50)
#' ti  <- rfsrc_v$time.interest
#' t30 <- ti[which.min(abs(ti - 30))]
#' t90 <- ti[which.min(abs(ti - 90))]
#'
#' # Default partial.type = "surv" -> y-axis "Predicted Survival"
#' pd_s <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
#'                          partial.time = c(t30, t90), n_eval = 8)
#' plot(pd_s)
#'
#' # partial.type = "chf" -> y-axis "Predicted CHF"
#' pd_c <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
#'                          partial.time = c(t30, t90),
#'                          partial.type = "chf", n_eval = 8)
#' plot(pd_c)
#' }
#'
#' @importFrom ggplot2 .data
#' @importFrom patchwork wrap_plots
#' @export
plot.gg_partial_rfsrc <- function(x, ...) {
  gg_dta <- x

  gg_cont <- NULL
  if (!is.null(gg_dta$continuous) && nrow(gg_dta$continuous) > 0) {
    cont <- gg_dta$continuous

    if (!is.null(cont$time)) {
      ## Survival forest: predictor value on x-axis, one curve per time point.
      ## Group/colour by the *full-precision* time so distinct horizons that
      ## happen to round to the same value are not silently merged. The legend
      ## is relabelled with rounded values for readability.
      time_levels <- sort(unique(cont$time))
      cont$.time_factor <- factor(cont$time, levels = time_levels)
      legend_labels <- format(round(time_levels, 2), trim = TRUE)
      y_lab <- partial_surv_y_label(attr(gg_dta, "partial.type"))
      gg_cont <- ggplot2::ggplot(
        cont,
        ggplot2::aes(
          x     = .data$x,
          y     = .data$yhat,
          color = .data$.time_factor,
          group = .data$.time_factor
        )
      ) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::scale_color_discrete(labels = legend_labels) +
        ggplot2::labs(x = NULL, y = y_lab, color = "Time")

    } else if (!is.null(cont$grp)) {
      ## Two-variable surface: group is xvar2; x-axis is the primary predictor
      gg_cont <- ggplot2::ggplot(
        cont,
        ggplot2::aes(
          x     = .data$x,
          y     = .data$yhat,
          color = factor(.data$grp),
          group = factor(.data$grp)
        )
      ) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::labs(x = NULL, y = "Partial Effect", color = "Group")

    } else {
      ## Standard: one curve per variable
      gg_cont <- ggplot2::ggplot(cont,
                                 ggplot2::aes(x = .data$x, y = .data$yhat)) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::labs(x = NULL, y = "Partial Effect")
    }
  }

  gg_cat <- NULL
  if (!is.null(gg_dta$categorical) && nrow(gg_dta$categorical) > 0) {
    cat_dta <- gg_dta$categorical
    gg_cat <- ggplot2::ggplot(
      cat_dta,
      ggplot2::aes(x = factor(.data$x), y = .data$yhat)
    ) +
      ggplot2::geom_bar(stat = "identity", width = 0.5) +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

#' Plot a \code{\link{gg_partialpro}} object
#'
#' Produces ggplot2 partial dependence curves from the named list returned by
#' \code{\link{gg_partialpro}}, which wraps \code{varpro::partialpro} output.
#'
#' Each variable produces up to three effect curves: parametric, nonparametric,
#' and causal.  The \code{type} argument controls which are shown.
#'
#' @param x A \code{\link{gg_partialpro}} object.
#' @param type Character vector; one or more of \code{"parametric"},
#'   \code{"nonparametric"}, \code{"causal"}.  Defaults to all three.
#' @param ... Not currently used.
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.  When both continuous
#'   and categorical variables are present the two panels are combined
#'   vertically via \code{patchwork::wrap_plots()}.
#'
#' @seealso \code{\link{gg_partialpro}}
#'
#' @examples
#' ## ggRandomForests does not depend on the varpro package; we construct a
#' ## minimal mock of the partialpro() output so the example runs everywhere.
#' set.seed(42)
#' n_obs <- 30
#' n_pts <- 15
#' mock_data <- list(
#'   age = list(
#'     xvirtual    = seq(30, 80, length.out = n_pts),
#'     xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
#'   ),
#'   sex = list(
#'     xvirtual    = c(0, 1),
#'     xorg        = sample(c(0, 1), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
#'   )
#' )
#'
#' pp <- gg_partialpro(mock_data)
#'
#' # Returns a single ggplot/patchwork figure combining continuous and
#' # categorical panels when both variable types are present.
#' plot(pp)
#'
#' # Restrict to one or two effect types
#' plot(pp, type = "nonparametric")
#' plot(pp, type = c("parametric", "causal"))
#'
#' @importFrom ggplot2 .data
#' @importFrom patchwork wrap_plots
#' @export
plot.gg_partialpro <- function(x,
                               type = c("parametric", "nonparametric", "causal"),
                               ...) {
  gg_dta <- x
  type <- match.arg(type, several.ok = TRUE)

  gg_cont <- NULL
  if (!is.null(gg_dta$continuous) && nrow(gg_dta$continuous) > 0) {
    cont <- gg_dta$continuous
    ## Pivot to long form so all requested effect types appear as one colour/line
    cont_long <- tidyr::pivot_longer(
      cont,
      cols      = tidyr::all_of(type),
      names_to  = "effect_type",
      values_to = "yhat"
    )
    gg_cont <- ggplot2::ggplot(
      cont_long,
      ggplot2::aes(
        x        = .data$variable,
        y        = .data$yhat,
        color    = .data$effect_type,
        linetype = .data$effect_type
      )
    ) +
      ggplot2::geom_line() +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect",
                    color = "Effect type", linetype = "Effect type")
  }

  gg_cat <- NULL
  if (!is.null(gg_dta$categorical) && nrow(gg_dta$categorical) > 0) {
    cat_dta <- gg_dta$categorical
    cat_long <- tidyr::pivot_longer(
      cat_dta,
      cols      = tidyr::all_of(type),
      names_to  = "effect_type",
      values_to = "yhat"
    )
    gg_cat <- ggplot2::ggplot(
      cat_long,
      ggplot2::aes(
        x    = factor(.data$variable),
        y    = .data$yhat,
        fill = .data$effect_type
      )
    ) +
      ggplot2::geom_boxplot() +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect", fill = "Effect type")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

#' Survival partial dependence data for one or more predictors
#'
#' \strong{Deprecated.} Use \code{\link{gg_partial_rfsrc}} instead, which
#' returns a classed \code{gg_partial_rfsrc} object with a dedicated
#' \code{plot()} method.
#'
#' Computes partial dependence curves for a survival or competing-risk
#' \code{\link[randomForestSRC]{rfsrc}} forest.  For each predictor it calls
#' \code{\link[randomForestSRC]{partial.rfsrc}} at \code{npts} evenly-spaced
#' unique values, across every stored event time.
#'
#' @param rforest A fitted \code{\link[randomForestSRC]{rfsrc}} survival or
#'   competing-risk forest object.
#' @param var_list Character vector of predictor names for which partial
#'   dependence should be computed. Each must appear in
#'   \code{rforest$xvar.names}.
#' @param npts Integer; the number of predictor grid points to evaluate
#'   (default 25). Evenly-spaced unique values are sampled from each predictor.
#' @param partial.type The prediction type to return. For survival forests one
#'   of \code{"surv"} (default), \code{"mort"}, or \code{"chf"}. For competing
#'   risk forests one of \code{"years.lost"}, \code{"cif"}, or \code{"chf"}.
#'   See \code{\link[randomForestSRC]{partial.rfsrc}} for full details.
#'
#' @return A named list with one element per variable in \code{var_list}. Each
#'   element is itself a list with:
#'   \describe{
#'     \item{name}{The predictor variable name (character).}
#'     \item{dta}{The raw output of
#'       \code{\link[randomForestSRC]{get.partial.plot.data}}, a list containing
#'       at minimum \code{x} (predictor values) and \code{yhat} (partial
#'       predictions), and for survival/competing risk, \code{partial.time}.}
#'   }
#'
#' @seealso \code{\link{gg_partial_rfsrc}},
#'   \code{\link[randomForestSRC]{partial.rfsrc}},
#'   \code{\link[randomForestSRC]{get.partial.plot.data}}
#'
#' @importFrom randomForestSRC partial.rfsrc
#' @examples
#' ## ------------------------------------------------------------
#' ## survival
#' ## ------------------------------------------------------------
#'
#' data(veteran, package = "randomForestSRC")
#' v.obj <- randomForestSRC::rfsrc(Surv(time,status)~.,
#'   veteran, nsplit = 10, ntree = 100)
#'
#' spart <- surv_partial.rfsrc(v.obj, var_list="age", partial.type = "mort")
#'
#' ## partial effect of age on mortality
#' partial.obj <- randomForestSRC::partial(v.obj,
#'                        partial.type = "mort",
#'                        partial.xvar = "age",
#'                        partial.values = v.obj$xvar$age,
#'                        partial.time = v.obj$time.interest)
#' pdta <- randomForestSRC::get.partial.plot.data(partial.obj)
#'
#' plot(lowess(pdta$x, pdta$yhat, f = 1/3),
#'      type = "l", xlab = "age", ylab = "adjusted mortality")
#'
#' ## example where x is discrete - partial effect of age on mortality
#' ## we use the granule=TRUE option
#' partial.obj <- randomForestSRC::partial(v.obj,
#'                        partial.type = "mort",
#'                        partial.xvar = "trt",
#'                        partial.values = v.obj$xvar$trt,
#'                        partial.time = v.obj$time.interest)
#' pdta <- randomForestSRC::get.partial.plot.data(partial.obj, granule = TRUE)
#' boxplot(pdta$yhat ~ pdta$x, xlab = "treatment", ylab = "partial effect")
#'
#'
#' ## partial effects of karnofsky score on survival
#' karno <- quantile(v.obj$xvar$karno)
#' partial.obj <- randomForestSRC::partial(v.obj,
#'                        partial.type = "surv",
#'                        partial.xvar = "karno",
#'                        partial.values = karno,
#'                        partial.time = v.obj$time.interest)
#' pdta <- randomForestSRC::get.partial.plot.data(partial.obj)
#'
#' matplot(pdta$partial.time, t(pdta$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "karnofsky adjusted survival")
#' legend("topright", legend = paste0("karnofsky = ", karno), fill = 1:5)
#'
#'
#' ## ------------------------------------------------------------
#' ## competing risk
#' ## ------------------------------------------------------------
#'
#' data(follic, package = "randomForestSRC")
#' follic.obj <- randomForestSRC::rfsrc(Surv(time, status) ~ ., follic, nsplit = 3, ntree = 100)
#'
#' ## partial effect of age on years lost
#' partial.obj <- randomForestSRC::partial(follic.obj,
#'                        partial.type = "years.lost",
#'                        partial.xvar = "age",
#'                        partial.values = follic.obj$xvar$age,
#'                        partial.time = follic.obj$time.interest)
#' pdta1 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 1)
#' pdta2 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 2)
#'
#' # Save and restore the user's graphical parameters per CRAN policy.
#' oldpar <- par(no.readonly = TRUE)
#' on.exit(par(oldpar))
#' par(mfrow = c(2, 2))
#' plot(lowess(pdta1$x, pdta1$yhat),
#'      type = "l", xlab = "age", ylab = "adjusted years lost relapse")
#' plot(lowess(pdta2$x, pdta2$yhat),
#'      type = "l", xlab = "age", ylab = "adjusted years lost death")
#'
#' ## partial effect of age on cif
#' partial.obj <- randomForestSRC::partial(follic.obj,
#'                        partial.type = "cif",
#'                        partial.xvar = "age",
#'                        partial.values = quantile(follic.obj$xvar$age),
#'                        partial.time = follic.obj$time.interest)
#' pdta1 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 1)
#' pdta2 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 2)
#'
#' matplot(pdta1$partial.time, t(pdta1$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "age adjusted cif for relapse")
#' matplot(pdta2$partial.time, t(pdta2$yhat), type = "l", lty = 1,
#'         xlab = "time", ylab = "age adjusted cif for death")
#'
#' @export surv_partial.rfsrc
surv_partial.rfsrc <- function(rforest, var_list, npts = 25, partial.type = "surv") { # nolint: object_name_linter
  .Deprecated(
    new     = "gg_partial_rfsrc",
    package = "ggRandomForests",
    msg     = paste0(
      "'surv_partial.rfsrc' is deprecated and will be removed in a future release.\n",
      "Use 'gg_partial_rfsrc()' instead, which returns a classed object with a\n",
      "dedicated plot() method."
    )
  )
  ###----------Partial dependency estimation, for each variable, at each time point ----
  surv.lst <- lapply(var_list, function(xvar) {
    ## determine the partial plot data
    xv <- sort(unique(rforest$xvar[, xvar]))
    xv <- unique(xv[seq(1, length(xv), length = npts)])

    ## Get the partial.plot.data
    partial.dta    <- randomForestSRC::get.partial.plot.data(
      randomForestSRC::partial.rfsrc(
        rforest,
        partial.type = partial.type,
        partial.xvar = xvar,
        partial.values = xv,
        partial.time = rforest$time.interest
      )
    )

    list(name = xvar,
         dta = partial.dta)

  })
  return(surv.lst)
}

## ----pkg-setup, include=FALSE-------------------------------------------------
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}


## ----error-demo---------------------------------------------------------------
library(randomForest)
set.seed(42)
rf_iris <- randomForest(Species ~ ., data = iris, ntree = 200, keep.forest = TRUE)
err_df <- ggRandomForests::gg_error(rf_iris, training = TRUE)
head(err_df)


## ----error-plot, fig.height=4-------------------------------------------------
plot(err_df)


## ----variable-demo------------------------------------------------------------
set.seed(99)
boston <- MASS::Boston
rf_boston <- randomForest(medv ~ ., data = boston, ntree = 150)
var_df <- ggRandomForests::gg_variable(rf_boston)
str(var_df[, c("lstat", "yhat")])


## ----variable-plot, fig.height=4----------------------------------------------
plot(var_df, xvar = "lstat")


## ----vimp-demo----------------------------------------------------------------
vimp_df <- ggRandomForests::gg_vimp(rf_boston)
head(vimp_df)
plot(vimp_df)


## ----quantile-demo------------------------------------------------------------
rm_breaks <- ggRandomForests::quantile_pts(boston$rm, groups = 6, intervals = TRUE)
rm_groups <- cut(boston$rm, breaks = rm_breaks)
table(rm_groups)

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_brier}} object
#'
#' Plot the time-resolved Brier score (default) or running CRPS for a
#' survival forest, optionally overlaid with a 15-85 percent per-subject
#' envelope.
#'
#' @param x A \code{\link{gg_brier}} object.
#' @param type Which series to plot: \code{"brier"} (default) or
#'   \code{"crps"}.
#' @param envelope Logical. When \code{TRUE}, overlays a ribbon spanning
#'   the 15th-85th percentile of per-subject Brier (or running CRPS)
#'   contributions at each time, around the overall line. When \code{FALSE}
#'   (default), draws the overall series only.
#' @param ... Extra arguments forwarded to \code{geom_line()}.
#'
#' @return A \code{ggplot} object.
#'
#' @seealso \code{\link{gg_brier}},
#'   \code{\link[randomForestSRC]{get.brier.survival}}
#'
#' @examples
#' \dontrun{
#' data(pbc, package = "randomForestSRC")
#' rf <- randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
#'                              nsplit = 10)
#' gg_dta <- gg_brier(rf)
#' plot(gg_dta)
#' plot(gg_dta, type = "crps")
#' plot(gg_dta, envelope = TRUE)   # adds 15-85% envelope
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon labs theme
#' @export
plot.gg_brier <- function(x,
                          type = c("brier", "crps"),
                          envelope = FALSE,
                          ...) {
  if (!inherits(x, "gg_brier")) {
    stop("Incorrect object type: Expects a gg_brier object")
  }
  type <- match.arg(type)

  y_label <- if (type == "brier") "Brier score" else "CRPS"
  y_col   <- if (type == "brier") "brier" else "crps"
  lo_col  <- if (type == "brier") "bs.lower" else "crps.lower"
  hi_col  <- if (type == "brier") "bs.upper" else "crps.upper"

  gg_plt <- ggplot2::ggplot(
    x,
    ggplot2::aes(x = .data[["time"]], y = .data[[y_col]])
  )

  if (envelope) {
    gg_plt <- gg_plt +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data[[lo_col]], ymax = .data[[hi_col]]),
        alpha = 0.2,
        fill  = "steelblue"
      )
  }

  gg_plt <- gg_plt +
    ggplot2::geom_line(...) +
    ggplot2::labs(x = "Time", y = y_label) +
    ggplot2::theme(legend.position = "none")

  return(gg_plt)
}

# testthat for gg_brier

test_that("gg_brier.rfsrc produces a tidy survival frame", {
  data(pbc, package = "randomForestSRC")
  set.seed(1)
  rf <- randomForestSRC::rfsrc(
    Surv(days, status) ~ .,
    data = pbc,
    nsplit = 10,
    ntree = 100
  )

  gg_dta <- gg_brier(rf)

  expect_s3_class(gg_dta, "gg_brier")
  expect_true(all(c("time", "brier", "bs.q25", "bs.q50", "bs.q75",
                    "bs.q100", "bs.lower", "bs.upper",
                    "crps", "crps.q25", "crps.q50", "crps.q75",
                    "crps.q100", "crps.lower", "crps.upper") %in%
                    names(gg_dta)))
  # Envelope sanity: lower <= upper at every time.
  expect_true(all(gg_dta$bs.lower <= gg_dta$bs.upper))
  expect_true(all(gg_dta$crps.lower <= gg_dta$crps.upper, na.rm = TRUE))
  expect_equal(nrow(gg_dta), length(rf$time.interest))
  expect_true(all(is.finite(gg_dta$brier)))
  expect_true(all(gg_dta$brier >= 0 & gg_dta$brier <= 1))

  # integrated CRPS attribute matches the upstream scalar.
  expect_true(!is.null(attr(gg_dta, "crps_integrated")))

  # plot method returns a ggplot for each supported display.
  expect_s3_class(plot(gg_dta), "ggplot")
  expect_s3_class(plot(gg_dta, type = "crps"), "ggplot")
  expect_s3_class(plot(gg_dta, envelope = TRUE), "ggplot")
  expect_s3_class(plot(gg_dta, type = "crps", envelope = TRUE), "ggplot")
})

test_that("gg_brier rejects non-survival forests", {
  data(iris, package = "datasets")
  rf <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
  expect_error(gg_brier(rf), "right-censored survival")
})

test_that("plot.gg_brier rejects non-gg_brier input", {
  data(iris, package = "datasets")
  expect_error(plot.gg_brier(iris), "gg_brier object")
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
###############################################################################
# Package documentation
###############################################################################
#' @title ggRandomForests: Visually Exploring Random Forests
#'
#' @description \code{ggRandomForests} is a utility package for
#' \code{randomForestSRC} (Ishwaran and Kogalur) for survival,
#' regression and classification forests and uses the \code{ggplot2}
#' (Wickham 2009) package for plotting results. \code{ggRandomForests} is
#' structured to extract data objects from the random forest and provides S3
#' functions for printing and plotting these objects.
#' Requires \code{randomForestSRC} >= 3.4.0.
#'
#' The \code{randomForestSRC} package provides a unified treatment of
#' Breiman's (2001) random forests for a variety of data settings. Regression
#' and classification forests are grown when the response is numeric or
#' categorical (factor) while survival and competing risk forests
#' (Ishwaran et al. 2008, 2012) are grown for right-censored survival data.
#'
#' Many of the figures created by the \code{ggRandomForests} package are also
#' available directly from within the \code{randomForestSRC} package. However,
#' \code{ggRandomForests} offers the following advantages:
#' \itemize{
#' \item Separation of data and figures: \code{ggRandomForest} contains
#' functions that operate on either the \code{\link[randomForestSRC]{rfsrc}}
#' forest object directly, or on the output from \code{randomForestSRC} post
#' processing functions (i.e. \code{plot.variable}) to generate intermediate
#' \code{ggRandomForests}
#' data objects. S3 functions are provide to further process these objects and
#' plot results using the \code{ggplot2} graphics package. Alternatively,
#' users can use these data objects for additional custom plotting or
#' analysis operations.
#'
#' \item Each data object/figure is a single, self contained object. This
#' allows simple modification and manipulation of the data or \code{ggplot2}
#' objects to meet users specific needs and requirements.
#'
#' \item The use of \code{ggplot2} for plotting. We chose to use the
#' \code{ggplot2} package for our figures to allow users flexibility in
#' modifying the figures to their liking. Each S3 plot function returns either
#' a single \code{ggplot2} object, or a \code{list} of \code{ggplot2} objects,
#' allowing users to use additional \code{ggplot2} functions or themes to
#' modify and customize the figures to their liking.
#' }
#'
#' The \code{ggRandomForests} package contains the following data functions:
#' \itemize{
#' \item \code{\link{gg_rfsrc}}: randomForestSRC predictions.
#' \item \code{\link{gg_error}}: randomForestSRC convergence rate based on
#' the OOB error rate.
#' \item \code{\link{gg_roc}}: ROC curves for randomForest classification
#' models.
#' \item \code{\link{gg_vimp}}: Variable Importance ranking for variable
#' selection.
#' (Ishwaran et.al. 2010).
#' \item \code{\link{gg_variable}}: Marginal variable dependence.
#'
#' \item \code{\link{gg_survival}}: Kaplan-Meier/Nelson-Aalen hazard analysis.
#' }
#'
#' Each of these data functions has an associated S3 plot function that
#' returns \code{ggplot2} objects, either individually or as a list, which can
#' be further customized using standard \code{ggplot2} commands.
#'
#' @references
#' Breiman, L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. R News
#' 7(2), 25--31.
#'
#' Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random
#' survival forests. Ann. Appl. Statist. 2(3), 841--860.
#'
#' Ishwaran, H., U. B. Kogalur, E. Z. Gorodeski, A. J. Minn, and M. S. Lauer
#' (2010). High-dimensional variable selection for survival data. J. Amer.
#' Statist. Assoc. 105, 205-217.
#'
#' Ishwaran, H. (2007). Variable importance in binary regression trees and
#' forests. Electronic J. Statist., 1, 519-537.
#'
#' Wickham, H. ggplot2: elegant graphics for data analysis. Springer New York,
#' 2009.
#'
#' @name ggRandomForests-package
#'
################
NULL

##=============================================================================
#' Partial dependence data from an rfsrc model
#'
#' A partial dependence curve marginalizes the forest's prediction over all
#' other predictors: for each evaluation point of the target variable, the
#' forest scores every training observation with that value substituted in,
#' then averages the result.  What you get is the average effect of the target
#' variable after "integrating out" the rest -- a curve that would be flat if
#' the variable carried no signal.
#'
#' This function builds those curves for one or more predictors by calling
#' \code{\link[randomForestSRC]{partial.rfsrc}} and then tidy-stacking the
#' results into separate data frames for continuous and categorical variables.
#' Unlike \code{\link{gg_partial}} (which wraps \code{plot.variable}), you
#' pass the fitted \code{rfsrc} object directly -- no intermediate
#' \code{plot.variable} step.
#'
#' For survival forests, the marginalized quantity depends on
#' \code{partial.type}: survival probability (\code{"surv"}), cumulative
#' hazard function (\code{"chf"}), or expected mortality (\code{"mort"}).
#' You can request the curve at one or more time horizons via
#' \code{partial.time}; the resulting data have a \code{time} column so the
#' plot layers them as separate coloured lines.
#'
#' @section Survival forests and \code{partial.time}:
#' \code{\link[randomForestSRC]{partial.rfsrc}} expects every value in
#' \code{partial.time} to be an exact member of the model's
#' \code{time.interest} vector, the unique observed event times stored in the
#' fitted object.  Pass an arbitrary time, even a plausible one such as
#' \code{c(1, 3)} for a study measured in years, and you get a C-level
#' prediction error from inside \code{partial.rfsrc}.
#'
#' \code{gg_partial_rfsrc} takes care of this: every element of
#' \code{partial.time} is silently snapped to its nearest
#' \code{time.interest} value before the call.  To target a specific
#' follow-up horizon, find the closest grid point yourself and pass it
#' explicitly:
#'
#' \preformatted{
#' ti  <- rf_model$time.interest
#' t1  <- ti[which.min(abs(ti - 1))]   # nearest to 1 year
#' pd  <- gg_partial_rfsrc(rf_model, xvar.names = "x", partial.time = t1)
#' }
#'
#' @section Logical predictor columns:
#' \code{\link[randomForestSRC]{partial.rfsrc}} does not handle
#' \code{logical} predictor columns correctly in survival forests
#' (randomForestSRC <= 3.5.1).  If your training data contains binary 0/1
#' columns, convert them to \code{\link{factor}} rather than \code{logical}
#' before fitting the model.
#'
#' @param rf_model A fitted \code{\link[randomForestSRC]{rfsrc}} object.
#' @param xvar.names Character vector of predictor names for which partial
#'   dependence should be computed. Must be a subset of \code{rf_model$xvar.names}.
#' @param xvar2.name Optional single character name of a grouping variable in
#'   \code{newx}. When supplied, partial dependence is computed separately for
#'   each unique level of this variable and a \code{grp} column is appended.
#' @param newx Optional \code{data.frame} of predictor values to evaluate
#'   partial effects at. Defaults to the training data stored in
#'   \code{rf_model$xvar}. All column names must match \code{rf_model$xvar.names}.
#' @param partial.time Numeric vector of desired time points for survival
#'   forests (ignored for regression/classification).  Values are automatically
#'   snapped to the nearest entry in \code{rf_model$time.interest}; see the
#'   \strong{Survival forests} section below.  When \code{NULL} (default),
#'   three quartile points of \code{time.interest} are used.
#' @param partial.type Character; type of predicted value for survival
#'   forests, passed through to \code{\link[randomForestSRC]{partial.rfsrc}}.
#'   One of \code{"surv"} (default), \code{"chf"}, or \code{"mort"}. Ignored
#'   for non-survival forests. \code{partial.rfsrc()} requires a non-\code{NULL}
#'   value for survival families; supplying it here avoids a cryptic
#'   \dQuote{argument is of length zero} error from the underlying C code.
#' @param cat_limit Variables with fewer than \code{cat_limit} unique values in
#'   \code{newx} are treated as categorical; all others are continuous.
#'   Defaults to 10.
#' @param n_eval Number of evaluation points for continuous variables. Instead
#'   of passing all observed values (which can be slow, especially for survival
#'   forests), continuous predictors are evaluated on a quantile grid of this
#'   many points. Categorical variables always use all unique levels.
#'   Defaults to 25.
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{continuous}{A \code{data.frame} with columns \code{x} (numeric),
#'       \code{yhat}, \code{name} (variable name), and optionally \code{grp}
#'       (the level of \code{xvar2.name}) and \code{time} (survival forests
#'       only) for all continuous predictors.}
#'     \item{categorical}{A \code{data.frame} with the same columns but
#'       \code{x} kept as character, for low-cardinality predictors.}
#'   }
#'
#' @seealso \code{\link{gg_partial}}, \code{\link[randomForestSRC]{partial.rfsrc}},
#'   \code{\link[randomForestSRC]{get.partial.plot.data}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ##
#' ## regression
#' ##
#' ## ------------------------------------------------------------
#'
#' airq.obj <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality)
#'
#' ## partial effect for wind
#' prt_dta <- gg_partial_rfsrc(airq.obj,
#'                        xvar.names = c("Wind"))
#'
#' @importFrom dplyr mutate filter select all_of
#' @export
gg_partial_rfsrc <- function(rf_model,
                             xvar.names = NULL,
                             xvar2.name = NULL,
                             newx = NULL,
                             partial.time = NULL,
                             partial.type = c("surv", "chf", "mort"),
                             cat_limit = 10,
                             n_eval = 25) {
  if (is.null(newx)) {
    newx <- rf_model$xvar
  }

  if (sum(colnames(newx) %in% rf_model$xvar.names) != ncol(newx)) {
    stop("newx must be a dataframe with the same columns used to train the rfsrc object")
  }

  if (!is.null(xvar.names)) {
    if (sum(xvar.names %in% colnames(newx)) != length(xvar.names)) {
      stop("xvar.names contains column names not found in the rfsrc object")
    }
  }

  v         <- validate_partial_args(n_eval, cat_limit)
  n_eval    <- v$n_eval
  cat_limit <- v$cat_limit

  is_surv <- !is.null(rf_model$family) && grepl("surv", rf_model$family)
  if (is_surv) {
    partial.time <- snap_partial_time(rf_model, partial.time)
    # partial.rfsrc() requires a non-NULL partial.type for survival forests;
    # NULL triggers a zero-length comparison inside the C code.
    partial.type <- match.arg(partial.type)
  } else {
    partial.type <- NULL
  }

  if (is.null(xvar2.name)) {
    pdta <- partial_no_group(xvar.names, newx, rf_model,
                             cat_limit, n_eval, is_surv, partial.time,
                             partial.type)
  } else {
    pdta <- partial_with_group(xvar.names, xvar2.name, newx, rf_model,
                               cat_limit, n_eval, is_surv, partial.time,
                               partial.type)
  }

  result <- split_partial_result(do.call("rbind", pdta))
  # Carry partial.type so plot.gg_partial_rfsrc() can pick the correct
  # y-axis label (Survival / CHF / Mortality).
  attr(result, "partial.type") <- partial.type
  result <- .set_provenance(result, rf_model)
  result
}

## ---- unexported helpers -------------------------------------------------------

## Check that x is a single non-NA numeric >= min_val; return as integer.
validate_scalar_int <- function(x, name, min_val = 2L) {
  ok <- is.numeric(x) && length(x) == 1L && !is.na(x) && x >= min_val
  if (!ok) {
    stop(sprintf("'%s' must be a single integer >= %d.", name, min_val),
         call. = FALSE)
  }
  as.integer(x)
}

## Validate and coerce n_eval / cat_limit.
validate_partial_args <- function(n_eval, cat_limit) {
  list(
    n_eval    = validate_scalar_int(n_eval,    "n_eval",    2L),
    cat_limit = validate_scalar_int(cat_limit, "cat_limit", 2L)
  )
}

## Snap requested time points to the nearest values in time.interest.
snap_partial_time <- function(rf_model, partial.time) {
  ti <- rf_model$time.interest
  if (is.null(partial.time)) {
    partial.time <- quantile(ti, probs = c(0.25, 0.5, 0.75), names = FALSE)
  }
  snapped <- sapply(partial.time,
                    function(t) ti[which.min(abs(ti - t))],
                    USE.NAMES = FALSE)
  unique(snapped)
}

## Build the evaluation grid (xval vector + categorical flag) for one variable.
make_eval_grid <- function(xname, newx, cat_limit, n_eval) {
  # Use `[[` to preserve the column's class (factor, character, numeric, …).
  # unlist(dplyr::select(...)) would coerce factors to integer codes, breaking
  # both the cat_limit check and the partial.values passed to partial.rfsrc().
  xval <- newx[[xname]]
  xval <- xval[!is.na(xval)]
  if (length(xval) == 0L) {
    warning(sprintf(
      "Variable '%s' contains only NA values in 'newx'; skipping partial dependence.",
      xname
    ), call. = FALSE)
    return(NULL)
  }
  gr <- is.factor(xval) || is.character(xval) || length(unique(xval)) < cat_limit
  if (!gr && length(unique(xval)) > n_eval) {
    xval <- quantile_pts(xval, groups = n_eval)
  } else if (is.factor(xval)) {
    xval <- levels(droplevels(xval))   # preserve factor ordering; drop unused levels
  } else {
    xval <- sort(unique(xval))
  }
  list(xval = xval, categorical = gr)
}

## Thin wrapper around partial.rfsrc that builds the argument list.
call_partial_rfsrc <- function(rf_model, xname, xval,
                                is_surv, partial.time, partial.type,
                                xvar2.name = NULL, x2val = NULL) {
  args <- list(
    object         = rf_model,
    partial.xvar   = xname,
    partial.values = xval
  )
  if (!is.null(xvar2.name)) {
    args$partial.xvar2   <- xvar2.name
    args$partial.values2 <- x2val
  }
  if (is_surv) {
    args$partial.time <- partial.time
    args$partial.type <- partial.type
  }
  do.call(randomForestSRC::partial.rfsrc, args)
}

## Process a single predictor variable and return a tidy data.frame (or NULL).
partial_one_var <- function(xname, newx, rf_model,
                            cat_limit, n_eval, is_surv, partial.time,
                            partial.type,
                            xvar2.name = NULL, x2val = NULL) {
  eg <- make_eval_grid(xname, newx, cat_limit, n_eval)
  if (is.null(eg)) return(NULL)
  xval <- eg$xval
  gr   <- eg$categorical
  partial.obj <- call_partial_rfsrc(rf_model, xname, xval,
                                     is_surv, partial.time, partial.type,
                                     xvar2.name, x2val)
  pout    <- randomForestSRC::get.partial.plot.data(partial.obj, granule = gr)
  # Survival forests with >1 partial.time return yhat as an
  # [length(partial.values) x length(partial.time)] matrix; expand to long form
  # so each (x, time) pair is its own row. For non-survival or single-time
  # cases yhat is already a vector of length(partial.values).
  if (is.matrix(pout$yhat)) {
    pt <- if (!is.null(pout$partial.time)) pout$partial.time else seq_len(ncol(pout$yhat))
    out_dta <- data.frame(
      x    = rep(pout$x, times = length(pt)),
      yhat = as.numeric(pout$yhat),
      time = rep(pt, each = length(pout$x))
    )
  } else {
    out_dta <- data.frame(x = pout$x, yhat = pout$yhat)
    if (!is.null(pout$partial.time)) {
      out_dta$time <- pout$partial.time
    }
  }
  out_dta$name <- xname
  out_dta$type <- c("continuous", "categorical")[gr + 1L]
  out_dta
}

## Compute partial dependence across xvar.names (no grouping variable).
partial_no_group <- function(xvar.names, newx, rf_model,
                             cat_limit, n_eval, is_surv, partial.time,
                             partial.type) {
  pdta <- lapply(xvar.names, partial_one_var,
                 newx = newx, rf_model = rf_model,
                 cat_limit = cat_limit, n_eval = n_eval,
                 is_surv = is_surv, partial.time = partial.time,
                 partial.type = partial.type)
  Filter(Negate(is.null), pdta)
}

## Compute partial dependence across xvar.names for each level of xvar2.name.
partial_with_group <- function(xvar.names, xvar2.name, newx, rf_model,
                               cat_limit, n_eval, is_surv, partial.time,
                               partial.type) {
  xv2 <- unique(newx[[xvar2.name]])
  xv2 <- xv2[!is.na(xv2)]
  if (length(xv2) == 0L) {
    stop(sprintf(
      "Grouping variable '%s' contains only NA values in 'newx'; cannot compute surface partial dependence.",
      xvar2.name
    ), call. = FALSE)
  }
  pdta <- lapply(xv2, function(x2val) {
    p1dta <- lapply(xvar.names, partial_one_var,
                    newx = newx, rf_model = rf_model,
                    cat_limit = cat_limit, n_eval = n_eval,
                    is_surv = is_surv, partial.time = partial.time,
                    partial.type = partial.type,
                    xvar2.name = xvar2.name, x2val = x2val)
    p1dta <- Filter(Negate(is.null), p1dta)
    if (length(p1dta) == 0L) return(NULL)
    p1dta        <- do.call("rbind", p1dta)
    p1dta$grp    <- x2val
    p1dta
  })
  Filter(Negate(is.null), pdta)
}

## Split the combined data.frame into continuous / categorical and stamp class.
split_partial_result <- function(pdta) {
  cont_idx        <- pdta$type == "continuous"
  continuous      <- pdta[cont_idx, , drop = FALSE]
  continuous$x    <- as.numeric(continuous$x)
  continuous$type <- NULL
  categorical      <- pdta[!cont_idx, , drop = FALSE]
  categorical$type <- NULL
  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- "gg_partial_rfsrc"
  result
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Predicted response data object
#'
#' Extracts the predicted response values from the
#' \code{\link[randomForestSRC]{rfsrc}} object, and formats data for plotting
#' the response using \code{\link{plot.gg_rfsrc}}.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object.
#' @param by Optional stratifying variable. Either a character column name
#'   present in the training data, or a vector/factor of the same length as
#'   the training set. When supplied, a \code{group} column is added to the
#'   returned data and bootstrap CI bands (survival) are computed per group.
#'   Omit or leave missing to return an unstratified result.
#' @param oob Logical; if \code{TRUE} (default) return out-of-bag predictions.
#'   Set to \code{FALSE} to use full in-bag (training) predictions. Forced to
#'   \code{FALSE} automatically for \code{predict.rfsrc} objects, which carry
#'   no OOB estimates.
#' @param ... Additional arguments controlling output for specific forest
#'   families:
#'   \describe{
#'     \item{surv_type}{Character; one of \code{"surv"} (default),
#'       \code{"chf"}, or \code{"mortality"} for survival forests.}
#'     \item{conf.int}{Numeric coverage probability (e.g. \code{0.95}) to
#'       request bootstrap pointwise confidence bands for survival forests.
#'       Triggers wide-format output with \code{lower}, \code{upper},
#'       \code{median}, and \code{mean} columns.}
#'     \item{bs.sample}{Integer; number of bootstrap resamples when
#'       \code{conf.int} is set. Defaults to the number of observations.}
#'   }
#'
#' @return A \code{gg_rfsrc} object (a classed \code{data.frame}) whose
#'   structure depends on the forest family:
#'   \describe{
#'     \item{regression}{Columns \code{yhat} and the response name; optionally
#'       a \code{group} column when \code{by} is supplied.}
#'     \item{classification}{One column per class with predicted probabilities;
#'       a \code{y} column with observed class labels; optionally \code{group}.}
#'     \item{survival (no CI / grouping)}{Long-format with columns
#'       \code{variable} (event time), \code{value} (survival probability),
#'       \code{obs_id}, and \code{event}.}
#'     \item{survival (with \code{conf.int} or \code{by})}{Wide-format with
#'       pointwise bootstrap CI columns (\code{lower}, \code{upper},
#'       \code{median}, \code{mean}) per time point; a \code{group} column
#'       when \code{by} is supplied.}
#'   }
#'   The object carries class attributes for the forest family so that
#'   \code{\link{plot.gg_rfsrc}} dispatches correctly.
#'
#' @details
#'   For survival forests, use the \code{surv_type} argument
#'   (\code{"surv"}, \code{"chf"}, or \code{"mortality"}) to select the
#'   predicted quantity. Bootstrap confidence bands are requested by passing
#'   \code{conf.int} (e.g. \code{conf.int = 0.95}); the number of resamples
#'   is controlled by \code{bs.sample}.
#'
#' @seealso \code{\link{plot.gg_rfsrc}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link{gg_survival}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example (small, runs on CRAN)
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_iris)
#' plot(gg_dta)
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Additional regression / survival examples are guarded with
#' ## \donttest because the cumulative example time exceeds the
#' ## 10-second CRAN budget. Run locally with `R CMD check --run-donttest`
#' ## (or `devtools::check(run_dont_test = TRUE)`) to exercise them.
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data (regression)
#' rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' plot(gg_rfsrc(rfsrc_airq))
#'
#' ## -------- Boston data (rfsrc + randomForest)
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'   data(Boston, package = "MASS")
#'   Boston$chas <- as.logical(Boston$chas)
#'   rfsrc_boston <- rfsrc(medv ~ ., data = Boston, ntree = 50,
#'                         forest = TRUE, importance = TRUE,
#'                         tree.err = TRUE, save.memory = TRUE)
#'   plot(gg_rfsrc(rfsrc_boston))
#'
#'   rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
#'                                           ntree = 50)
#'   plot(gg_rfsrc(rf_boston))
#' }
#'
#' ## -------- mtcars data
#' rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#' plot(gg_rfsrc(rfsrc_mtcars))
#'
#' ## -------- veteran data (survival; with CI and group-by)
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        ntree = 50)
#' plot(gg_rfsrc(rfsrc_veteran))
#' plot(gg_rfsrc(rfsrc_veteran, conf.int = .95))
#' plot(gg_rfsrc(rfsrc_veteran, by = "trt"))
#' }
#'
#' @aliases gg_rfsrc gg_rfsrc.rfsrc

#' @export
gg_rfsrc.rfsrc <- function(object, # nolint: cyclocomp_linter
                           oob = TRUE,
                           by,
                           ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "rfsrc")) {
    stop(
      paste(
        "This function only works for Forests grown with the",
        "randomForestSRC package."
      )
    )
  }
  # The forest object itself must be stored; rfsrc uses forest = TRUE for this.
  if (is.null(object$forest)) {
    stop(
      paste(
        "The function requires the \"forest = TRUE\"",
        "attribute when growing the randomForest"
      )
    )
  }

  # Collect optional arguments (conf.int, surv_type, bs.sample, etc.)
  arg_list <- list(...)

  # Prediction objects do not have OOB predictions; force full-forest preds.
  if (inherits(object, "predict")) {
    oob <- FALSE
  }

  ## ---- Resolve the stratifying variable ("by") ----------------------------
  if (!missing(by)) {
    grp <- by
    # Accept either a column name (character) or a pre-built vector/factor.
    if (is.character(grp)) {
      if (!grp %in% colnames(object$xvar)) {
        stop(paste("No column named", grp, "in forest training set."))
      }
      grp <- object$xvar[, grp]
    }

    if (is.vector(grp) || is.factor(grp)) {
      if (length(grp) != nrow(object$xvar)) {
        stop(paste(
          "By argument does not have the correct dimension ",
          nrow(object$xvar)
        ))
      }
    } else {
      stop(
        paste(
          "By argument should be either a vector, or colname",
          "of training data"
        )
      )
    }
    # Convert to factor with levels in order of first appearance to preserve
    # the natural ordering of groups in downstream plots.
    grp <- factor(grp, levels = unique(grp))
  }

  ## ---- Classification branch -----------------------------------------------
  if (object$family == "class") {
    # For binary classification rfsrc stores exactly two probability columns
    # (one per class); we drop the first (the "negative" class probability)
    # since it is redundant — prob(class 2) = 1 - prob(class 1).
    # Multi-class forests (3+ classes) keep all columns.
    if (oob) {
      gg_dta <-
        if (ncol(object$predicted.oob) == 2) {
          data.frame(cbind(object$predicted.oob[, -1]))
        } else {
          data.frame(cbind(object$predicted.oob))
        }
    } else {
      gg_dta <- if (ncol(object$predicted) == 2) {
        data.frame(cbind(object$predicted[, -1]))
      } else {
        data.frame(cbind(object$predicted))
      }
    }

    # Assign column names and append the observed class label column "y".
    if (ncol(gg_dta) == 1) {
      colnames(gg_dta) <- object$yvar.names
      # rfsrc internally stores binary outcomes as a two-level factor even
      # when the original response is logical; convert back to logical here.
      gg_dta$y <- as.logical(as.numeric(object$yvar) - 1)
    } else {
      colnames(gg_dta) <- levels(object$yvar)
      gg_dta$y <- object$yvar
    }

    if (!missing(by)) {
      gg_dta$group <- grp
    }

  ## ---- Survival branch -----------------------------------------------------
  } else if (object$family == "surv") {
    # Default to overall survival; callers can request CHF or mortality.
    surv_type <- "surv"

    if (!is.null(arg_list$surv_type)) {
      surv_type <- arg_list$surv_type
    }

    # Extract the appropriate prediction matrix (rows = observations,
    # columns = unique time points stored in $time.interest).
    if (oob) {
      rng <- switch(surv_type,
        surv = data.frame(object$survival.oob),
        chf = data.frame(object$chf.oob),
        mortality = data.frame(1 - object$survival.oob),
        stop(paste(
          surv_type, " not implemented at this time"
        ))
      )
    } else {
      rng <- switch(surv_type,
        surv = data.frame(object$survival),
        chf = data.frame(object$chf),
        mortality = data.frame(1 - object$survival),
        stop(paste(
          surv_type, " not implemented at this time"
        ))
      )
    }

    # Name each column with the corresponding observed event time.
    colnames(rng) <- object$time.interest

    rng$obs_id <- seq_len(nrow(rng))
    if (is.null(object$yvar)) {
      rng$event <- FALSE
    } else {
      # Second column of yvar is the event indicator (0 = censored, 1 = event).
      rng$event <- as.logical(object$yvar[, 2])
    }
    gg_dta <- rng

    if (is.null(arg_list$conf.int) && missing(by)) {
      # No grouping or CI requested: pivot to long form so plot.gg_rfsrc can
      # draw one survival step function per observation.
      pivot_cols <-
        colnames(gg_dta)[-which(colnames(gg_dta) %in% c("obs_id", "event"))]
      gg_dta_mlt <-
        tidyr::pivot_longer(gg_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")
      gg_dta_mlt$variable <-
        as.numeric(as.character(gg_dta_mlt$variable))
      gg_dta_mlt$obs_id <- factor(gg_dta_mlt$obs_id)

      gg_dta <- gg_dta_mlt
    } else {
      # Bootstrap confidence bands: compute pointwise quantiles of the
      # bootstrapped mean survival curve at each time point.
      level <- if (is.null(arg_list$conf.int)) {
        .95
      } else {
        arg_list$conf.int
      }

      # Accept a single coverage probability (e.g. 0.95 → two-sided tails
      # at 0.025 and 0.975) or a length-2 vector of explicit quantile probs.
      if (length(level) == 1) {
        if (level > 1) {
          level <- level / 100
        }

        level_set <- c((1 - level) / 2, 1 - (1 - level) / 2)
        level_set <- sort(level_set)
      } else {
        level_set <- sort(level)
      }

      # Number of bootstrap resamples — default to the number of observations.
      if (is.null(arg_list$bs.sample)) {
        bs_samples <- nrow(gg_dta)
      } else {
        bs_samples <- arg_list$bs.sample
      }

      if (!missing(by)) {
        # Stratified: compute separate bootstrap CI bands per group level.
        gg_dta$group <- grp
        grp_dta <- lapply(levels(grp), function(st) {
          if (is.null(arg_list$bs.sample)) {
            bs_samples <-
              nrow(gg_dta[which(as.character(gg_dta$group) == st), ])
          }

          obj <-
            bootstrap_survival(
              gg_dta[which(as.character(gg_dta$group) == st), ],
              bs_samples, level_set
            )
          obj$group <- st
          obj
        })
        gg_grp <- do.call(rbind, grp_dta)
        gg_grp$group <- factor(gg_grp$group,
          levels = unique(gg_grp$group)
        )
        gg_dta <- gg_grp
      } else {
        # Single-group CI band.
        gg_dta <- bootstrap_survival(gg_dta, bs_samples, level_set)
      }
    }

  ## ---- Regression branch ---------------------------------------------------
  } else if (object$family == "regr") {
    # Bind predicted values alongside the observed response for scatter plots.
    if (oob) {
      gg_dta <- data.frame(cbind(object$predicted.oob, object$yvar))
    } else {
      gg_dta <- data.frame(cbind(object$predicted, object$yvar))
    }

    colnames(gg_dta) <- c("yhat", object$yvar.names)

    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else {
    stop(
      paste(
        "Plotting for ",
        object$family,
        " randomForestSRC is not yet implemented.",
        sep = ""
      )
    )
  }

  # Tag the data frame with the forest family so plot.gg_rfsrc can dispatch
  # to the correct branch without re-inspecting the original forest object.
  class(gg_dta) <- c("gg_rfsrc", object$family, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}



#' Bootstrap pointwise confidence bands for a mean survival curve
#'
#' Draws \code{bs_samples} bootstrap resamples (with replacement) of the
#' per-observation survival curves stored in \code{gg_dta}, computes the column
#' means to obtain a bootstrapped mean curve per resample, then returns the
#' pointwise quantiles at \code{level_set} and the overall mean across
#' resamples.
#'
#' @param gg_dta A wide \code{data.frame} of survival probabilities as returned
#'   by the survival branch of \code{\link{gg_rfsrc.rfsrc}}, before the
#'   optional pivot to long form. Columns \code{obs_id}, \code{event}, and
#'   \code{group} (if present) are excluded from the resampling.
#' @param bs_samples Integer; number of bootstrap resamples.
#' @param level_set Numeric vector of length 2 giving the lower and upper
#'   quantile probabilities for the confidence band (e.g. \code{c(0.025, 0.975)}
#'   for a 95\% CI).
#'
#' @return A \code{data.frame} with one row per unique event time and columns
#'   \code{value} (time), \code{lower}, \code{upper}, \code{median}, and
#'   \code{mean}.
#'
#' @keywords internal
bootstrap_survival <- function(gg_dta, bs_samples, level_set) {
  ## Calculate the leave one out estimate of the mean survival
  gg_t <-
    gg_dta[, -which(colnames(gg_dta) %in% c("obs_id", "event", "group"))]
  mn_bs <- t(sapply(
    seq_len(bs_samples),
    function(pat) {
      st <- sample(seq_len(nrow(gg_t)),
        size = nrow(gg_t),
        replace = TRUE
      )
      colMeans(gg_t[st, ])
    }
  ))

  ## now get the confidence interval of the mean, and the median (.5)
  rng <- sapply(
    seq_len(ncol(mn_bs)),
    function(t_pt) {
      quantile(mn_bs[, t_pt], probs = c(level_set, .5))
    }
  )
  mn <- sapply(seq_len(ncol(rng)), function(t_pt) {
    mean(rng[, t_pt])
  })

  # gg_t already has obs_id/event/group stripped; rng and mn are indexed over
  # time points only, so no further exclusion is needed.
  time_interest <- as.numeric(colnames(gg_t))

  dta <- data.frame(cbind(
    time_interest,
    t(rng),
    mn
  ))

  # rng always has 3 rows: lower quantile, upper quantile, median (.5).
  # Name columns canonically so plot.gg_rfsrc can always find "lower"/"upper".
  colnames(dta) <- c("value", "lower", "upper", "median", "mean")
  dta
}

#' @export
gg_rfsrc <- function(object,
                     oob = TRUE,
                     by, ...) {
  UseMethod("gg_rfsrc", object)
}

#' @export
gg_rfsrc.randomForest <- function(object,
                                  oob = TRUE,
                                  by,
                                  ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown with the",
        "randomForest package."
      )
    )
  }

  if (inherits(object, "predict")) {
    oob <- FALSE
  }

  # Recover the training predictor frame once (needed for by-column lookup and
  # dimension checks).  randomForest stores predictors in $forest$xlevels keys
  # but not the actual data; use .rf_recover_model_frame() for that.
  rf_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
  rf_xvar <- if (!is.null(rf_info)) rf_info$model_frame[
    , setdiff(colnames(rf_info$model_frame), rf_info$response_name),
    drop = FALSE
  ] else NULL
  n_train <- length(object$predicted)

  if (!missing(by)) {
    grp <- by
    # Accept either a column name (character) or a pre-built vector/factor.
    if (is.character(grp)) {
      if (is.null(rf_xvar) || !grp %in% colnames(rf_xvar)) {
        stop(paste("No column named", grp, "in forest training set."))
      }
      grp <- rf_xvar[, grp]
    }

    if (is.vector(grp) || is.factor(grp)) {
      if (length(grp) != n_train) {
        stop(paste(
          "By argument does not have the correct dimension ",
          n_train
        ))
      }
    } else {
      stop(
        paste(
          "By argument should be either a vector, or colname",
          "of training data"
        )
      )
    }
    grp <- factor(grp, levels = unique(grp))
  }

  # Extract the response variable name from the formula for column naming below.
  # (Both branches below build gg_dta from the forest's stored predictions, so
  # there is no need to recover the original training data frame here.)
  rsp <- as.character(object$call$formula)[2]

  # Do the work...
  if (object$type == "classification") {
    ### Classification models...

    # Need to add multiclass methods
    prd <- predict(object, type = "prob")
    gg_dta <- if (ncol(prd) <= 2) {
      data.frame(cbind(prd[, -1]))
    } else {
      data.frame(cbind(prd))
    }
    colnames(gg_dta) <- object$classes

    # Switch between logical or categorical outcome
    if (ncol(gg_dta) == 1) {
      # Force this to logical return value...
      #
      # This may be a bug in rfsrc, as it converts all classification models
      # into factors.
      gg_dta$y <- as.logical(as.numeric(object$yvar) - 1)
    } else {
      gg_dta$y <- object$y
    }

    # Handle the "by" argument.
    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else if (object$type == "regression") {
    gg_dta <- data.frame(cbind(object$predicted, object$y))

    colnames(gg_dta) <- c("yhat", rsp)

    # Handle the "by" argument.
    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else {
    stop(
      paste(
        "Plotting for ",
        object$family,
        " randomForest is not yet implemented for ",
        object$type,
        sep = ""
      )
    )
  }

  class(gg_dta) <- c("gg_rfsrc", object$type, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

# Unit tests for kaplan(), nelson(), and bootstrap_survival()

## ---- Shared survival data --------------------------------------------------

data(pbc, package = "randomForestSRC")
pbc_dta <- pbc
pbc_dta$time <- pbc_dta$days / 364.25

## ---- kaplan() --------------------------------------------------------------

test_that("kaplan returns a gg_survival data frame", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_s3_class(gg_dta, "gg_survival")
  expect_s3_class(gg_dta, "data.frame")
})

test_that("kaplan output has required columns", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  required_cols <- c("time", "n", "cens", "dead", "surv", "se",
                     "lower", "upper", "cum_haz",
                     "hazard", "density", "mid_int", "life", "proplife")
  expect_true(all(required_cols %in% colnames(gg_dta)))
})

test_that("kaplan retains only event rows (dead > 0)", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(gg_dta$dead > 0))
})

test_that("kaplan survival is monotonically non-increasing", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(diff(gg_dta$surv) <= 0))
})

test_that("kaplan survival is bounded in [0, 1]", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(gg_dta$surv >= 0))
  expect_true(all(gg_dta$surv <= 1))
})

test_that("kaplan with stratification adds groups column", {
  pbc_strat <- pbc_dta
  pbc_strat$treatment <- factor(pbc_strat$treatment)
  gg_dta <- kaplan(interval = "time", censor = "status",
                   data = pbc_strat, by = "treatment")
  expect_true("groups" %in% colnames(gg_dta))
  # Both treatment levels should appear
  expect_true(length(unique(gg_dta$groups)) >= 2L)
})

test_that("kaplan life column is non-decreasing and proplife is in [0, 1]", {
  # Regression test: the trapezoidal formula L(t_i) = L(t_{i-1}) + (S(t_{i-1}) + S(t_i))/2 * Δt
  # must produce a monotonically non-decreasing cumulative integral.
  # The old Adams-Bashforth formula could produce this too, but values were
  # numerically wrong (over-estimated).
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(diff(gg_dta$life) >= 0),
              info = "life must be non-decreasing (cumulative area under S(t))")
  expect_true(all(gg_dta$proplife >= 0),
              info = "proplife must be >= 0")
  expect_true(all(gg_dta$proplife <= 1 + .Machine$double.eps^0.5),
              info = "proplife must be <= 1 (area under S(t) <= t * 1)")
})

test_that("kaplan with character (non-factor) by uses unique() labels", {
  # .label_strata() has two code paths: levels() for factors, unique() for
  # character/numeric.  This test exercises the unique() path.
  pbc_strat <- pbc_dta
  pbc_strat$trt_chr <- as.character(pbc_strat$treatment)
  pbc_strat <- pbc_strat[!is.na(pbc_strat$trt_chr), ]

  gg_dta <- kaplan(interval = "time", censor = "status",
                   data = pbc_strat, by = "trt_chr")
  expect_true("groups" %in% colnames(gg_dta))
  expect_true(length(unique(gg_dta$groups)) >= 2L)
})

test_that("kaplan plot returns a ggplot", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_s3_class(plot(gg_dta), "ggplot")
})

test_that("kaplan plot with error = 'none' returns a ggplot", {
  gg_dta <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  expect_s3_class(plot(gg_dta, error = "none"), "ggplot")
})

## ---- nelson() --------------------------------------------------------------

test_that("nelson returns a gg_survival data frame", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  expect_s3_class(gg_dta, "gg_survival")
  expect_s3_class(gg_dta, "data.frame")
})

test_that("nelson output has required columns", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  required_cols <- c("time", "n", "cens", "dead", "surv", "se",
                     "lower", "upper", "cum_haz",
                     "hazard", "density", "mid_int", "life", "proplife")
  expect_true(all(required_cols %in% colnames(gg_dta)))
})

test_that("nelson retains only event rows (dead > 0)", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(gg_dta$dead > 0))
})

test_that("nelson cum_haz is non-decreasing", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(diff(gg_dta$cum_haz) >= 0))
})

test_that("nelson with stratification adds groups column", {
  pbc_strat <- pbc_dta
  pbc_strat$treatment <- factor(pbc_strat$treatment)
  gg_dta <- nelson(interval = "time", censor = "status",
                   data = pbc_strat, by = "treatment")
  expect_true("groups" %in% colnames(gg_dta))
  expect_true(length(unique(gg_dta$groups)) >= 2L)
})

test_that("nelson and kaplan agree on survival estimates", {
  kap <- kaplan(interval = "time", censor = "status", data = pbc_dta)
  nel <- nelson(interval = "time", censor = "status", data = pbc_dta)
  # Both should produce estimates at the same time points
  common_times <- intersect(kap$time, nel$time)
  expect_true(length(common_times) > 0L)
  kap_surv <- kap$surv[kap$time %in% common_times]
  nel_surv <- nel$surv[nel$time %in% common_times]
  # KM and NA estimates are equivalent for the same data
  expect_equal(kap_surv, nel_surv, tolerance = 1e-6)
})

test_that("nelson life column is non-decreasing and proplife is in [0, 1]", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  expect_true(all(diff(gg_dta$life) >= 0),
              info = "life must be non-decreasing")
  expect_true(all(gg_dta$proplife >= 0))
  expect_true(all(gg_dta$proplife <= 1 + .Machine$double.eps^0.5))
})

test_that("nelson with character (non-factor) by uses unique() labels", {
  pbc_strat <- pbc_dta
  pbc_strat$trt_chr <- as.character(pbc_strat$treatment)
  pbc_strat <- pbc_strat[!is.na(pbc_strat$trt_chr), ]

  gg_dta <- nelson(interval = "time", censor = "status",
                   data = pbc_strat, by = "trt_chr")
  expect_true("groups" %in% colnames(gg_dta))
  expect_true(length(unique(gg_dta$groups)) >= 2L)
})

test_that("nelson plot returns a ggplot", {
  gg_dta <- nelson(interval = "time", censor = "status", data = pbc_dta)
  expect_s3_class(plot(gg_dta), "ggplot")
})

## ---- bootstrap_survival() --------------------------------------------------

# Helper: build wide-form survival data (obs x time-point columns) that
# bootstrap_survival() expects. gg_rfsrc() without conf.int pivots to long
# form, so we construct the wide matrix directly from the rfsrc internals.
.make_wide_surv <- function(rfsrc_obj) {
  wide <- data.frame(rfsrc_obj$survival.oob)
  colnames(wide) <- as.character(rfsrc_obj$time.interest)
  wide$obs_id <- seq_len(nrow(wide))
  wide$event  <- as.logical(rfsrc_obj$yvar[, 2L])
  wide
}

test_that("bootstrap_survival returns a data frame with correct columns (95% CI)", {
  data(pbc, package = "randomForestSRC")
  pbc_sub <- pbc[, c("days", "status", "treatment", "age", "bili", "albumin")]
  pbc_sub$time <- pbc_sub$days / 364.25

  set.seed(1L)
  rfsrc_pbc <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = pbc_sub, ntree = 50L)

  wide_dta   <- .make_wide_surv(rfsrc_pbc)
  n_timepts  <- length(rfsrc_pbc$time.interest)
  level_set  <- c(0.025, 0.975)

  set.seed(42L)
  result <- ggRandomForests:::bootstrap_survival(wide_dta, 50L, level_set)

  expect_s3_class(result, "data.frame")
  expect_equal(colnames(result), c("value", "lower", "upper", "median", "mean"))
  expect_equal(nrow(result), n_timepts)
  expect_true(all(result$lower <= result$mean + 1e-10))
  expect_true(all(result$mean  <= result$upper + 1e-10))
})

test_that("bootstrap_survival lower bound <= median <= upper bound", {
  data(pbc, package = "randomForestSRC")
  pbc_sub <- pbc[, c("days", "status", "treatment", "age", "bili", "albumin")]
  pbc_sub$time <- pbc_sub$days / 364.25

  set.seed(2L)
  rfsrc_pbc <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = pbc_sub, ntree = 50L)

  wide_dta  <- .make_wide_surv(rfsrc_pbc)
  level_set <- c(0.025, 0.975)

  set.seed(42L)
  result <- ggRandomForests:::bootstrap_survival(wide_dta, 50L, level_set)

  expect_true(all(result$lower  <= result$median + 1e-10))
  expect_true(all(result$median <= result$upper  + 1e-10))
})

test_that("bootstrap_survival time points match rfsrc$time.interest", {
  data(pbc, package = "randomForestSRC")
  pbc_sub <- pbc[, c("days", "status", "treatment", "age", "bili", "albumin")]
  pbc_sub$time <- pbc_sub$days / 364.25

  set.seed(3L)
  rfsrc_pbc <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = pbc_sub, ntree = 50L)

  wide_dta       <- .make_wide_surv(rfsrc_pbc)
  expected_times <- rfsrc_pbc$time.interest
  level_set      <- c(0.025, 0.975)

  set.seed(42L)
  result <- ggRandomForests:::bootstrap_survival(wide_dta, 50L, level_set)

  expect_equal(result$value, expected_times)
})

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Brier score and CRPS for survival forests
#'
#' Extract the time-resolved Brier score and continuous ranked probability
#' score (CRPS) for a survival forest grown with \code{randomForestSRC}. The
#' Brier score is computed at each time on \code{object$time.interest}, both
#' overall and stratified by mortality-risk quartile. CRPS is the running
#' trapezoidal integral of the Brier score, normalised by elapsed time, and
#' is computed within each quartile and overall.
#'
#' @details Wraps \code{\link[randomForestSRC]{get.brier.survival}} and
#' rebuilds the quartile decomposition + running CRPS from the returned
#' \code{brier.matx} and \code{mort} components, mirroring the computation
#' in the internal \code{plot.survival} function of \pkg{randomForestSRC}. The Brier score uses
#' inverse-probability-of-censoring weighting; the censoring distribution
#' is estimated either by Kaplan-Meier (\code{cens.model = "km"}, the
#' default) or by a separate censoring forest (\code{cens.model = "rfsrc"}).
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} survival
#'   forest (\code{object$family == "surv"}).
#' @param ... Currently unused; accepted for S3 dispatch compatibility.
#'
#' @return A \code{gg_brier} \code{data.frame} with columns
#'   \describe{
#'     \item{time}{event time grid (\code{object$time.interest}).}
#'     \item{brier}{overall Brier score at each time.}
#'     \item{bs.q25, bs.q50, bs.q75, bs.q100}{Brier score within each
#'       mortality-risk quartile (lowest to highest risk).}
#'     \item{bs.lower, bs.upper}{15th and 85th percentile of per-subject
#'       Brier contributions at each time. Used by
#'       \code{plot.gg_brier(by_quartile = TRUE)} to draw an envelope
#'       around the overall curve.}
#'     \item{crps}{running CRPS (overall) at each time, normalised by
#'       elapsed time.}
#'     \item{crps.q25, crps.q50, crps.q75, crps.q100}{running CRPS within
#'       each mortality-risk quartile.}
#'     \item{crps.lower, crps.upper}{running CRPS of the 15th / 85th
#'       per-subject Brier percentile, normalised by elapsed time.}
#'   }
#'   The integrated CRPS (a single scalar matching
#'   \code{get.brier.survival()$crps}) is attached as
#'   \code{attr(., "crps_integrated")}.
#'
#' @seealso \code{\link{plot.gg_brier}},
#'   \code{\link[randomForestSRC]{get.brier.survival}},
#'   \code{\link{gg_error}}
#'
#' @references
#' Graf E., Schmoor C., Sauerbrei W., Schumacher M. (1999). Assessment and
#' comparison of prognostic classification schemes for survival data.
#' Statistics in Medicine, 18(17-18):2529-2545.
#'
#' Gerds T.A., Schumacher M. (2006). Consistent estimation of the expected
#' Brier score in general survival models with right-censored event times.
#' Biometrical Journal, 48(6):1029-1040.
#'
#' @examples
#' \dontrun{
#' data(pbc, package = "randomForestSRC")
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(days, status) ~ ., data = pbc, nsplit = 10
#' )
#' gg_dta <- gg_brier(rfsrc_pbc)
#' plot(gg_dta)
#' plot(gg_dta, type = "crps")
#' plot(gg_dta, envelope = TRUE)   # overall line + 15-85% envelope
#'
#' # Multi-model comparison: stack gg_brier outputs and plot with ggplot2.
#' rf2 <- randomForestSRC::rfsrc(
#'   Surv(days, status) ~ ., data = pbc, nsplit = 10, mtry = 4
#' )
#' compare_dta <- dplyr::bind_rows(
#'   dplyr::mutate(gg_brier(rfsrc_pbc), model = "default"),
#'   dplyr::mutate(gg_brier(rf2),       model = "mtry=4")
#' )
#' ggplot2::ggplot(compare_dta,
#'   ggplot2::aes(x = time, y = brier, colour = model)) +
#'   ggplot2::geom_line()
#' }
#'
#' @importFrom stats quantile
#' @export
gg_brier <- function(object, ...) {
  UseMethod("gg_brier", object)
}

#' @export
gg_brier.rfsrc <- function(object,
                           subset = NULL,
                           cens.model = c("km", "rfsrc"),
                           ...) {
  if (!inherits(object, "rfsrc")) {
    stop("This function only works on rfsrc objects.")
  }
  if (is.null(object$family) || object$family != "surv") {
    stop("gg_brier only supports right-censored survival forests.")
  }

  cens.model <- match.arg(cens.model)

  brier_obj <- randomForestSRC::get.brier.survival(
    object,
    subset = subset,
    cens.model = cens.model
  )

  bs_df <- brier_obj$brier.score
  if (!all(c("time", "brier.score") %in% names(bs_df))) {
    stop("Unexpected output from get.brier.survival(); ",
         "ggRandomForests may need to be updated for this randomForestSRC version.")
  }

  brier_matx <- brier_obj$brier.matx
  mort       <- brier_obj$mort

  # Quartile breakpoints of per-subject mortality. The lowest bin uses an
  # open-left bound so the minimum-risk subject is captured.
  mort_breaks <- c(min(mort, na.rm = TRUE) - 1e-05,
                   stats::quantile(mort, (1:4) / 4, na.rm = TRUE))

  bs_quartile <- vapply(seq_len(4), function(k) {
    in_bin <- mort > mort_breaks[k] & mort <= mort_breaks[k + 1]
    if (!any(in_bin, na.rm = TRUE)) {
      # Empty bin — can occur when mort has ties at a quantile boundary.
      return(rep(NA_real_, nrow(bs_df)))
    }
    colMeans(brier_matx[in_bin, , drop = FALSE], na.rm = TRUE)
  }, numeric(nrow(bs_df)))

  # Per-time 15th and 85th percentile of per-subject Brier contributions.
  # Provides a non-parametric envelope around the overall curve for the
  # default by_quartile = TRUE ribbon display.
  bs_envelope <- apply(brier_matx, 2, stats::quantile,
                       probs = c(0.15, 0.85), na.rm = TRUE)

  time_grid <- bs_df$time
  bs_all    <- bs_df$brier.score

  # Running, time-normalised CRPS via single-pass cumulative trapezoid rule.
  # O(n) per series: accumulate area incrementally rather than recomputing
  # from scratch at every time point.
  crps_running <- function(times, scores) {
    n        <- length(times)
    out      <- numeric(n)
    out[1]   <- scores[1]
    cum_area <- 0
    for (j in seq_len(n)[-1]) {
      cum_area <- cum_area +
        (times[j] - times[j - 1]) * (scores[j] + scores[j - 1]) / 2
      span     <- times[j] - times[1]
      out[j]   <- if (span <= 0) NA_real_ else cum_area / span
    }
    out
  }

  crps_all   <- crps_running(time_grid, bs_all)
  crps_q     <- vapply(seq_len(4), function(k) {
    crps_running(time_grid, bs_quartile[, k])
  }, numeric(length(time_grid)))
  crps_lower <- crps_running(time_grid, bs_envelope[1, ])
  crps_upper <- crps_running(time_grid, bs_envelope[2, ])

  gg_dta <- data.frame(
    time        = time_grid,
    brier       = bs_all,
    bs.q25      = bs_quartile[, 1],
    bs.q50      = bs_quartile[, 2],
    bs.q75      = bs_quartile[, 3],
    bs.q100     = bs_quartile[, 4],
    bs.lower    = bs_envelope[1, ],
    bs.upper    = bs_envelope[2, ],
    crps        = crps_all,
    crps.q25    = crps_q[, 1],
    crps.q50    = crps_q[, 2],
    crps.q75    = crps_q[, 3],
    crps.q100   = crps_q[, 4],
    crps.lower  = crps_lower,
    crps.upper  = crps_upper
  )

  attr(gg_dta, "crps_integrated") <- brier_obj$crps
  attr(gg_dta, "cens.model")      <- cens.model
  class(gg_dta) <- c("gg_brier", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

# Internal trapezoidal integrator: sum_i (x[i+1]-x[i]) * (y[i]+y[i+1])/2.
.trapz <- function(x, y) {
  n <- length(x)
  if (n < 2) {
    return(0)
  }
  sum((x[-1] - x[-n]) * (y[-1] + y[-n])) / 2
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' nonparametric Nelson-Aalen estimates
#'
#' @param data name of the survival training data.frame
#' @param interval name of the interval variable in the training dataset.
#' @param censor name of the censoring variable in the training dataset.
#' @param by stratifying variable in the training dataset, defaults to NULL
#' @param weight for each observation (default=NULL)
#' @param ... arguments passed to the \code{survfit} function
#'
#' @return \code{\link{gg_survival}} object
#'
#' @importFrom survival Surv survfit strata
#'
#' @seealso \code{\link{gg_survival}} \code{\link{nelson}}
#' \code{\link{plot.gg_survival}}
#'
#' @examples
#' # These get run through the gg_survival examples.
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' # This is the same as gg_survival
#' gg_dta <- nelson(
#'   interval = "time", censor = "status",
#'   data = pbc
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' # Stratified on treatment variable.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta, error = "lines")
#' plot(gg_dta)
#'
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment",
#'   type = "nelson"
#' )
#'
#' plot(gg_dta, error = "bars")
#' plot(gg_dta)
#'
#' @export
nelson <-
  function(interval,
           censor,
           data,
           by = NULL,
           weight = NULL,
           ...) {
    # Incorporate observation weights: zero out weights for censored records
    # so they do not contribute to the weighted hazard computation.
    if (!is.null(weight)) {
      weight <- data[[censor]] * weight
    }

    # Build the Surv object and fit the (possibly stratified) estimator.
    srv <- # nolint: object_usage_linter
      survival::Surv(time = data[[interval]], event = data[[censor]])
    if (is.null(by)) {
      srv_tab <- survival::survfit(srv ~ 1, ...)
    } else {
      srv_tab <-
        survival::survfit(srv ~ survival::strata(data[[by]]), ...)
    }

    # Cumulative hazard H(t) = -log(S(t)), consistent with the KM estimator.
    cum_hazard <- -log(srv_tab$surv)

    # Collect per-time-point statistics into a flat data frame.
    tbl <- data.frame(
      cbind(
        time = srv_tab$time,
        n = srv_tab$n.risk,        # number at risk just before t
        cens = srv_tab$n.censor,   # number censored at t
        dead = srv_tab$n.event,    # number of events at t
        surv = srv_tab$surv,       # KM survival estimate S(t)
        se = srv_tab$std.err,      # standard error of S(t)
        lower = srv_tab$lower,     # lower confidence bound
        upper = srv_tab$upper,     # upper confidence bound
        cum_haz = cum_hazard
      )
    )

    # Detect stratum boundaries and label each row with its group name.
    if (!is.null(by)) tbl <- .label_strata(tbl, data, by) # nolint: object_usage_linter

    # Retain only rows with at least one event.
    gg_dta <- tbl[which(tbl[["dead"]] != 0), ]

    # Derived interval-based quantities (same as in kaplan.R).
    lag_surv <- c(1, gg_dta$surv)[-(dim(gg_dta)[1] + 1)]
    lag_time <- c(0, gg_dta$time)[-(dim(gg_dta)[1] + 1)]

    delta_t <- gg_dta$time - lag_time
    # h(t) ≈ -log(S(t)/S(t-)) / Δt
    hzrd <- log(lag_surv / gg_dta$surv) / delta_t

    # f(t) ≈ (S(t-) - S(t)) / Δt
    dnsty <- (lag_surv - gg_dta$surv) / delta_t
    mid_int <- (gg_dta$time + lag_time) / 2
    lag_l <- 0

    # Cumulative expected life in each interval (trapezoidal rule):
    # L(t_i) = L(t_{i-1}) + (S(t_{i-1}) + S(t_i)) / 2 * Δt_i
    life <- vector("numeric", length = dim(gg_dta)[1])
    for (ind in seq_len(dim(gg_dta)[1])) {
      life[ind] <-
        lag_l + (lag_surv[ind] + gg_dta[ind, "surv"]) / 2 * delta_t[ind]
      lag_l <- life[ind]
    }
    prp_life <- life / gg_dta$time
    gg_dta <- data.frame(
      cbind(
        gg_dta,
        hazard = hzrd,
        density = dnsty,
        mid_int = mid_int,
        life = life,
        proplife = prp_life
      )
    )

    class(gg_dta) <- c("gg_survival", class(gg_dta))
    invisible(gg_dta)
  }

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' nonparametric Kaplan-Meier estimates
#'
#' @param data name of the training set \code{data.frame}
#' @param interval name of the interval variable in the training dataset.
#' @param censor name of the censoring variable in the training dataset.
#' @param by stratifying variable in the training dataset, defaults to NULL
#' @param ... arguments passed to the \code{survfit} function
#'
#' @return \code{\link{gg_survival}} object
#'
#' @importFrom survival Surv survfit strata
#'
#' @seealso \code{\link{gg_survival}} \code{\link{nelson}}
#' \code{\link{plot.gg_survival}}
#'
#' @examples
#'
#' # These get run through the gg_survival examples.
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' # This is the same as gg_survival
#' gg_dta <- kaplan(
#'   interval = "time", censor = "status",
#'   data = pbc
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' # Stratified on treatment variable.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' @export
kaplan <- function(interval,
                   censor,
                   data,
                   by = NULL, ...) {
  # Build a Surv object from the named columns in the data frame.
  srv <- survival::Surv(time = data[[interval]], event = data[[censor]]) # nolint: object_usage_linter

  # Fit the Kaplan-Meier estimator; stratify on `by` when provided.
  if (is.null(by)) {
    srv_tab <- survival::survfit(srv ~ 1, ...)
  } else {
    srv_tab <-
      survival::survfit(srv ~ survival::strata(data[[by]]), ...)
  }

  # Cumulative hazard H(t) = -log(S(t)) via the Nelson-Aalen transform.
  cum_hazard <- -log(srv_tab$surv)

  # Collect per-time-point summary statistics into a flat data frame.
  tbl <- data.frame(
    cbind(
      time = srv_tab$time,
      n = srv_tab$n.risk,        # number at risk just before time t
      cens = srv_tab$n.censor,   # number censored at time t
      dead = srv_tab$n.event,    # number of events at time t
      surv = srv_tab$surv,       # KM survival estimate S(t)
      se = srv_tab$std.err,      # standard error of S(t)
      lower = srv_tab$lower,     # lower confidence bound
      upper = srv_tab$upper,     # upper confidence bound
      cum_haz = cum_hazard
    )
  )

  # When stratifying, stitch a "groups" label column onto the table.
  if (!is.null(by)) tbl <- .label_strata(tbl, data, by) # nolint: object_usage_linter

  # Keep only rows where at least one event occurred — censoring-only rows
  # do not contribute new KM estimates.
  gg_dta <- tbl[which(tbl[["dead"]] != 0), ]

  # Derived quantities computed from interval-based lagged differences.
  lag_s <- c(1, gg_dta$surv)[-(dim(gg_dta)[1] + 1)]
  lag_t <- c(0, gg_dta$time)[-(dim(gg_dta)[1] + 1)]

  delta_t <- gg_dta$time - lag_t
  # Conditional hazard rate approximation: h(t) ≈ -log(S(t)/S(t-)) / Δt
  hzrd <- log(lag_s / gg_dta$surv) / delta_t

  # Probability density: f(t) ≈ (S(t-) - S(t)) / Δt
  dnsty <- (lag_s - gg_dta$surv) / delta_t
  mid_int <- (gg_dta$time + lag_t) / 2
  lag_l <- 0

  # Cumulative expected life in each interval (trapezoidal rule):
  # L(t_i) = L(t_{i-1}) + (S(t_{i-1}) + S(t_i)) / 2 * Δt_i
  life <- vector("numeric", length = dim(gg_dta)[1])
  for (ind in seq_len(dim(gg_dta)[1])) {
    life[ind] <-
      lag_l + (lag_s[ind] + gg_dta[ind, "surv"]) / 2 * delta_t[ind]
    lag_l <- life[ind]
  }
  prp_life <- life / gg_dta$time
  gg_dta <- data.frame(
    cbind(
      gg_dta,
      hazard = hzrd,
      density = dnsty,
      mid_int = mid_int,
      life = life,
      proplife = prp_life
    )
  )

  class(gg_dta) <- c("gg_survival", class(gg_dta))
  invisible(gg_dta)
}

####**********************************************************************
####  Internal helpers shared by print.gg_* and summary.gg_* methods.
####
####  Every gg_* extractor calls .set_provenance() on the object it
####  returns, attaching a small list under attr(x, "provenance") that
####  captures the source package, forest family, ntree, and predictor
####  set. The print/summary methods consume that attribute to render a
####  uniform header.
####
####  Provenance is best-effort: missing fields are NA. Methods must
####  tolerate the attribute being absent (older saved objects).
####**********************************************************************

# Build the provenance list from a forest object.
.gg_provenance <- function(object) {
  if (inherits(object, "rfsrc")) {
    return(list(
      source     = "randomForestSRC",
      family     = object$family %||% NA_character_,
      ntree      = object$ntree %||% NA_integer_,
      n          = object$n %||% NA_integer_,
      xvar.names = object$xvar.names %||% character(0)
    ))
  }
  if (inherits(object, "randomForest")) {
    return(list(
      source     = "randomForest",
      family     = object$type %||% NA_character_,
      ntree      = object$ntree %||% NA_integer_,
      n          = if (!is.null(object$y)) length(object$y) else NA_integer_,
      xvar.names = if (!is.null(object$importance)) {
        rownames(object$importance)
      } else {
        character(0)
      }
    ))
  }
  # Unknown / non-forest object: return NULL so callers can skip cleanly.
  NULL
}

# Attach provenance to a gg_* object and return it unchanged.
# Skips silently when .gg_provenance() returns NULL (non-forest input).
.set_provenance <- function(x, object) {
  prov <- .gg_provenance(object)
  if (!is.null(prov)) attr(x, "provenance") <- prov
  x
}

# Format the standard one-line header used by print.gg_* methods.
# Returns a character scalar.
.gg_header <- function(x, class_label) {
  prov <- attr(x, "provenance")
  if (is.null(prov)) {
    return(sprintf("<%s>", class_label))
  }
  src    <- prov$source    %||% NA
  family <- prov$family    %||% NA
  ntree  <- prov$ntree     %||% NA
  n      <- prov$n         %||% NA

  bits <- c(
    if (!is.na(src))    sprintf("from %s", src),
    if (!is.na(family)) sprintf("family: %s", family),
    if (!is.na(ntree))  sprintf("ntree: %s", ntree),
    if (!is.na(n))      sprintf("n: %s", n)
  )
  if (length(bits) == 0L) {
    return(sprintf("<%s>", class_label))
  }
  sprintf("<%s>  %s", class_label, paste(bits, collapse = "  |  "))
}

# Null-coalescing helper (avoid rlang dependency).
`%||%` <- function(a, b) if (is.null(a)) b else a

####**********************************************************************
####**********************************************************************
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Marginal variable dependence data object.
#'
#' @details The marginal variable dependence is determined by comparing
#' relation between the predicted response from the randomForest and a
#' covariate of interest.
#'
#' The \code{gg_variable} function operates on a
#' \code{\link[randomForestSRC]{rfsrc}} object, the output from the
#' \code{\link[randomForestSRC]{plot.variable}} function, or on a fitted
#' \code{\link[randomForest]{randomForest}} object via the formula interface.
#'
#' @description \code{\link[randomForestSRC]{plot.variable}} generates a
#' \code{data.frame} containing the marginal variable dependence or the
#' partial variable dependence. The \code{gg_variable} function creates a
#' \code{data.frame} of containing the full set of covariate data (predictor
#' variables) and the predicted response for each observation. Marginal
#' dependence figures are created using the \code{\link{plot.gg_variable}}
#' function.
#' For \code{randomForest} fits the original model frame is rebuilt
#' from the stored call so that the same predictors can be paired with the
#' in-sample predictions.
#'
#' Optional arguments include \code{time} (scalar or vector of survival times
#' of interest), \code{time_labels} (labels for multiple survival horizons) and
#' \code{oob} which toggles between out-of-bag and in-bag predictions when the
#' forest stores both.
#'
#' @param object A \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object, or a
#'   \code{\link[randomForestSRC]{plot.variable}} result.
#' @param ... Optional arguments such as \code{time}, \code{time_labels}, and
#'   \code{oob} that tailor the marginal dependence extraction.
#'
#' @return A \code{gg_variable} object: a \code{data.frame} of all predictor
#'   columns from the training data paired with the OOB (or in-bag) predicted
#'   response. For survival forests each requested time horizon produces an
#'   additional column named by \code{time_labels}. The object carries a
#'   \code{"family"} class attribute (\code{"regr"}, \code{"class"}, or
#'   \code{"surv"}) used by \code{\link{plot.gg_variable}} for dispatch.
#'
#' @seealso \code{\link{plot.gg_variable}},
#'   \code{\link[randomForestSRC]{plot.variable}}
#'
#' @aliases gg_variable gg_variable.rfsrc
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification (small, runs on CRAN)
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_iris)
#' plot(gg_dta, xvar = "Sepal.Width")
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Additional classification / regression / survival examples are
#' ## guarded with \donttest because the cumulative example time exceeds
#' ## the 10-second CRAN budget. Run locally with `R CMD check
#' ## --run-donttest` (or `devtools::check(run_dont_test = TRUE)`) to
#' ## exercise them.
#' ## ------------------------------------------------------------
#' plot(gg_dta, xvar = "Sepal.Length")
#' plot(gg_dta, xvar = rfsrc_iris$xvar.names, panel = TRUE)
#'
#' ## ------------------------------------------------------------
#' ## regression
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data
#' rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality, ntree = 50)
#' gg_dta <- gg_variable(rfsrc_airq)
#'
#' # an ordinal variable
#' gg_dta[, "Month"] <- factor(gg_dta[, "Month"])
#'
#' plot(gg_dta, xvar = "Wind")
#' plot(gg_dta, xvar = "Temp")
#' plot(gg_dta, xvar = "Solar.R")
#' plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
#' plot(gg_dta, xvar = "Month", notch = TRUE)
#'
#' ## -------- motor trend cars data
#' rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_mtcars)
#'
#' # mtcars$cyl is an ordinal variable
#' gg_dta$cyl <- factor(gg_dta$cyl)
#' gg_dta$am <- factor(gg_dta$am)
#' gg_dta$vs <- factor(gg_dta$vs)
#' gg_dta$gear <- factor(gg_dta$gear)
#' gg_dta$carb <- factor(gg_dta$carb)
#'
#' plot(gg_dta, xvar = "cyl")
#' plot(gg_dta, xvar = "disp")
#' plot(gg_dta, xvar = "hp")
#' plot(gg_dta, xvar = "wt")
#' plot(gg_dta, xvar = c("disp", "hp", "drat", "wt", "qsec"), panel = TRUE)
#' plot(gg_dta,
#'   xvar = c("cyl", "vs", "am", "gear", "carb"), panel = TRUE,
#'   notch = TRUE
#' )
#'
#' ## -------- Boston data
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'   data(Boston, package = "MASS")
#'   rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
#'   gg_dta <- gg_variable(rf_boston)
#'   plot(gg_dta)
#'   plot(gg_dta, panel = TRUE)
#' }
#'
#' ## ------------------------------------------------------------
#' ## survival examples
#' ## ------------------------------------------------------------
#'
#' ## -------- veteran data
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., veteran,
#'   nsplit = 10,
#'   ntree = 50
#' )
#'
#' # get the 90-day survival time.
#' gg_dta <- gg_variable(rfsrc_veteran, time = 90)
#'
#' # Generate variable dependence plots for age and diagtime
#' plot(gg_dta, xvar = "age")
#' plot(gg_dta, xvar = "diagtime")
#'
#' # Generate coplots
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE, se = FALSE)
#'
#' # Compare survival at 30, 90, and 365 days simultaneously
#' gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
#' plot(gg_dta, xvar = "age")
#' }
#'
#' @aliases gg_variable gg_variable.rfsrc gg_variable.randomForest
#' @aliases gg_variable.random
#' @importFrom stats median
#'
#' @export
gg_variable <- function(object, ...) {
  UseMethod("gg_variable", object)
}
#' @export
gg_variable.rfsrc <- function(object,
                              ...) {
  # Collect optional arguments: time, time_labels, oob
  arg_list <- list(...)

  time <- arg_list$time
  time_labels <- arg_list$time_labels
  # Default to OOB predictions; callers can override with oob = FALSE to use
  # full-forest (in-bag) predictions instead.
  oob <- if (!is.null(arg_list$oob)) {
    arg_list$oob
  } else {
    TRUE
  }

  if (!inherits(object, "rfsrc")) {
    stop("gg_variable expects a rfsrc or plot.variable object.")
  }

  # Start with the predictor (xvar) data frame; we will append the predicted
  # response column(s) below depending on the forest family.
  gg_dta <- data.frame(object$xvar)

  ## ---- Regression branch ---------------------------------------------------
  if (object$family == "regr") {
    # Append a single "yhat" column containing the OOB (or full) predictions.
    if (oob) {
      gg_dta$yhat <- object$predicted.oob
    } else {
      gg_dta$yhat <- object$predicted
    }

  ## ---- Classification branch -----------------------------------------------
  } else if (object$family == "class") {
    # Append one "yhat.<class>" probability column per outcome class so the
    # caller can choose which class to display on the variable plot.
    preds <- if (oob) {
      object$predicted.oob
    } else {
      object$predicted
    }
    colnames(preds) <-
      paste("yhat.", colnames(preds),
        sep = ""
      )
    gg_dta <- cbind(gg_dta, preds)
    gg_dta$yvar <- object$yvar

  ## ---- Survival branch -----------------------------------------------------
  } else if (object$family == "surv") {
    # Survival forests predict a matrix (observations × time points); we need
    # to slice it at one or more user-specified time horizons.
    gg_dta$event <- as.logical(object$yvar[, 2])
    colnames(gg_dta) <- c(object$xvar.names, "event")

    # Default to the median observed event time when no time is specified.
    if (is.null(time)) {
      time <- median(object$time.interest)
    }
    lng <- length(time)

    for (ind in seq_len(lng)) {
      if (ind > 1) {
        gg_dta_t_old <- gg_dta_t
      }
      # Find the largest recorded time point that is <= the requested horizon.
      # Survival is a step function, so we read the value at the last eligible
      # recorded time point.
      gg_dta_t <- gg_dta
      eligible <- which(object$time.interest <= time[ind])
      if (!length(eligible)) {
        stop(
          "The time of interest is less than the first event time.\n",
          "  Make sure you are using the correct time units."
        )
      }
      in_time <- max(eligible)

      if (oob) {
        gg_dta_t$yhat <- object$survival.oob[, in_time]
      } else {
        gg_dta_t$yhat <- object$survival[, in_time]
      }

      # Attach either the numeric time value or a user-supplied label.
      if (is.null(time_labels)) {
        gg_dta_t$time <- time[ind]
      } else {
        gg_dta_t$time <- time_labels[ind]
      }

      # Stack time-slice data frames for multi-horizon plots.
      if (ind > 1) {
        gg_dta_t <- rbind(gg_dta_t_old, gg_dta_t)
      }
    }

    gg_dta <- gg_dta_t
    # Preserve the ordering of time levels in the factor for downstream plots.
    gg_dta$time <- factor(gg_dta$time, levels = unique(gg_dta$time))
  }
  class(gg_dta) <- c("gg_variable", object$family, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_variable.randomForest <- function(object,
                                     ...) {
  arg_list <- list(...)

  # randomForest objects do not store OOB predictions in a way that maps back
  # to the predictor space, so we always use in-bag (full-forest) predictions.
  if (!is.null(arg_list$oob)) {
    arg_list$oob <- FALSE
  }

  if (!inherits(object, "randomForest")) {
    stop("gg_variable expects a randomForest object.")
  }

  # Reconstruct the training data from the stored call so we can pair
  # predictions with the original predictors.
  training_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
  if (is.null(training_info)) {
    stop(
      "Unable to reconstruct the training data for this randomForest object.",
      " Make sure the model was fit with a formula interface and keep.forest = TRUE."
    )
  }

  model_frame <- training_info$model_frame
  response <- stats::model.response(model_frame)
  resp_name <- training_info$response_name

  # Drop the response column and any special model-frame columns (e.g. offset)
  # to leave only the predictor columns.
  predictors <- model_frame
  if (!is.null(resp_name) && resp_name %in% names(predictors)) {
    predictors[[resp_name]] <- NULL
  }
  special_cols <- grep("^\\(", colnames(predictors), value = TRUE)
  if (length(special_cols) > 0) {
    predictors[special_cols] <- NULL
  }

  gg_dta <- predictors
  # Append the forest's in-bag predicted values.
  gg_dta$yhat <- as.vector(object$predicted)
  if (object$type == "classification") {
    gg_dta$yvar <- response
  }

  class(gg_dta) <- c("gg_variable", object$type, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

##=============================================================================
#' Plot a \code{gg_udependent} variable dependency graph
#'
#' Draws the dependency graph held in a \code{gg_udependent} object as a
#' ggraph network.  Node colour marks whether a variable made the signal
#' set, and the width and opacity of an edge tell you how strong the
#' dependency between its two variables is.
#'
#' @section Reading the network:
#' Each node is a variable; each edge is a cross-variable dependency
#' that cleared the threshold passed to \code{gg_udependent}. The
#' Fruchterman-Reingold layout (the default) places mutually connected
#' variables near each other, so the picture tends to show hubs and
#' the clusters around them rather than a tidy ring. The eye usually
#' goes first to the largest blue node: a variable that is both in
#' the signal set and connects to many others is a hub of the
#' dependency structure. Edges with wider, more opaque strokes are
#' stronger dependencies; thin, faint edges sit near the threshold and
#' are the ones that would disappear first if you raised it.
#'
#' Grey, low-degree nodes are the ones UVarPro thinks are not
#' contributing much to the structure. (Truly isolated nodes are
#' dropped by `gg_udependent()` before the graph is drawn; what you
#' see is the connected component.) A cluster of mutually
#' connected variables is worth checking for redundancy; they may be
#' several views of the same underlying quantity.
#'
#' @section What this tells you:
#' Use the figure to pick a working set of variables: the hubs and
#' their immediate neighbours are the candidates UVarPro flags as
#' carrying structure. If a cluster of high-degree variables looks
#' like it might be measuring the same thing, that is a cue to look at
#' their pairwise correlations or fit them as a block rather than
#' individually. The threshold and layout are recorded in the caption
#' so a different choice is easy to spot in a later figure.
#'
#' @param x A \code{gg_udependent} object from \code{\link{gg_udependent}}.
#' @param layout Character; the igraph/ggraph layout algorithm.  Common
#'   choices are \code{"fr"} (Fruchterman-Reingold, the default),
#'   \code{"kk"} (Kamada-Kawai), \code{"stress"}, \code{"circle"}, and
#'   \code{"grid"}.
#' @param ... Not currently used.
#'
#' @details
#' This plot needs the \pkg{ggraph} package, which is in \code{Suggests}
#' rather than installed for you.  If it is missing, run
#' \code{install.packages("ggraph")}.
#'
#' A signal variable (\code{selected = TRUE}) gets a blue node
#' (\code{#4e8fcd}); the rest are grey (\code{#888888}).  Node size grows
#' with degree.  Edge width and opacity both grow with the raw dependency
#' weight \code{I[i,j]}.
#'
#' @return A \code{ggplot} object (built via ggraph).
#'
#' @seealso \code{\link{gg_udependent}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggraph", quietly = TRUE)) {
#'   set.seed(42)
#'   uv <- varPro::uvarpro(iris[, -5], ntree = 50)
#'   plot(gg_udependent(uv))
#' }
#' }
#'
#' @name plot.gg_udependent
#' @importFrom ggplot2 aes labs scale_color_manual theme_void
#' @importFrom igraph vertex_attr V edge_attr as_data_frame E
#' @export
plot.gg_udependent <- function(x, layout = "fr", ...) {
  prov <- attr(x, "provenance")

  if (is.null(x$graph) || nrow(x$edges) == 0L) {
    stop("gg_udependent: no edges to plot. ",
         "Lower threshold (currently ", prov$threshold,
         ") to detect dependencies.",
         call. = FALSE)
  }

  if (!requireNamespace("ggraph", quietly = TRUE)) {
    stop("Install the 'ggraph' package to use plot.gg_udependent(): ",
         "install.packages('ggraph')", call. = FALSE)
  }

  ## Ensure vertex attributes are present (guard against old objects) --------
  if (is.null(igraph::vertex_attr(x$graph, "selected"))) {
    sel <- x$nodes$selected[match(igraph::V(x$graph)$name,
                                   as.character(x$nodes$variable))]
    igraph::V(x$graph)$selected <- sel
  }
  if (is.null(igraph::vertex_attr(x$graph, "degree"))) {
    deg <- x$nodes$degree[match(igraph::V(x$graph)$name,
                                 as.character(x$nodes$variable))]
    igraph::V(x$graph)$degree <- deg
  }

  ## Edge-weight backfill: guard for legacy objects saved before edge weights
  ## were stored on the igraph.  Order-insensitive for undirected graphs.
  if (is.null(igraph::edge_attr(x$graph, "weight"))) {
    prov_d    <- isTRUE(attr(x, "provenance")$directed)
    edge_list <- igraph::as_data_frame(x$graph, what = "edges")
    if (prov_d) {
      idx <- match(paste(edge_list$from, edge_list$to),
                   paste(x$edges$variable_from, x$edges$variable_to))
    } else {
      key_g <- paste(pmin(edge_list$from, edge_list$to),
                     pmax(edge_list$from, edge_list$to))
      key_e <- paste(pmin(x$edges$variable_from, x$edges$variable_to),
                     pmax(x$edges$variable_from, x$edges$variable_to))
      idx <- match(key_g, key_e)
    }
    igraph::E(x$graph)$weight <- x$edges$weight[idx]
  }

  ggraph::ggraph(x$graph, layout = layout) +
    ggraph::geom_edge_link(
      ggplot2::aes(width = .data[["weight"]],
                   alpha = .data[["weight"]]),
      color = "#4e8fcd"
    ) +
    ggraph::geom_node_point(
      ggplot2::aes(color = factor(.data[["selected"]]),
                   size  = .data[["degree"]])
    ) +
    ggraph::geom_node_label(
      ggplot2::aes(label = .data[["name"]]),
      size = 3, show.legend = FALSE
    ) +
    ggplot2::scale_color_manual(
      values = c("TRUE" = "#4e8fcd", "FALSE" = "#888888"),
      guide  = "none"
    ) +
    ggraph::scale_edge_width(range = c(0.5, 2.5), guide = "none") +
    ggraph::scale_edge_alpha(range = c(0.3, 1.0),  guide = "none") +
    ggplot2::labs(
      size    = "Degree",
      caption = paste0("Dependency threshold: ", prov$threshold,
                       ". Layout: ", layout, ".")
    ) +
    ggplot2::theme_void()
}

####**********************************************************************
####  summary.gg_* methods + shared print.summary.gg_* printer.
####
####  Each summary method returns a list of pre-formatted lines plus a few
####  raw numeric fields, classed "summary.gg" so print.summary.gg can
####  format it uniformly.
####**********************************************************************

# Build the standard summary skeleton: header line + body lines.
.summary_skel <- function(x, class_label, body = character(0)) {
  out <- list(
    header = .gg_header(x, class_label),
    body   = body
  )
  class(out) <- c(sprintf("summary.%s", class_label), "summary.gg")
  out
}

#' Summary methods for gg_* data objects
#'
#' Each \code{summary.gg_*()} method returns a \code{summary.gg} object
#' containing a header line and per-class diagnostic statistics (OOB error
#' curve, top VIMP variables, time range, integrated CRPS, etc.).
#' \code{print.summary.gg()} renders the object to the console.
#'
#' @param object A \code{gg_*} data object.
#' @param x A \code{summary.gg} object (for \code{print.summary.gg}).
#' @param ... Not currently used.
#'
#' @return A \code{summary.gg} object (a list with \code{header} and
#'   \code{body} character vectors), returned invisibly from
#'   \code{print.summary.gg}.
#'
#' @seealso \code{\link{print.gg}}, \code{\link{autoplot.gg}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' summary(gg_error(rf))
#' summary(gg_vimp(rf))
#'
#' @name summary.gg
NULL

#' @rdname summary.gg
#' @export
print.summary.gg <- function(x, ...) {
  cat(x$header, "\n", sep = "")
  for (line in x$body) {
    cat("  ", line, "\n", sep = "")
  }
  invisible(x)
}

#' @rdname summary.gg
#' @export
summary.gg_error <- function(object, ...) {
  err_cols <- setdiff(names(object), c("ntree", "train"))
  final  <- vapply(err_cols, function(c) {
    v <- object[[c]]
    v[length(v)]
  }, numeric(1))
  min_err <- vapply(err_cols, function(c) min(object[[c]], na.rm = TRUE),
                    numeric(1))
  body <- c(
    sprintf("ntree: %d", max(object$ntree, na.rm = TRUE)),
    sprintf("final OOB error: %s",
            paste(sprintf("%s=%.4g", err_cols, final), collapse = ", ")),
    sprintf("min OOB error:   %s",
            paste(sprintf("%s=%.4g", err_cols, min_err), collapse = ", "))
  )
  .summary_skel(object, "gg_error", body)
}

#' @rdname summary.gg
#' @export
summary.gg_vimp <- function(object, ...) {
  top_n <- min(5L, nrow(object))
  top   <- utils::head(object[order(-object$vimp), , drop = FALSE], top_n)
  body  <- c(
    sprintf("variables: %d", nrow(object)),
    sprintf("positive VIMP: %d / negative: %d",
            sum(object$positive, na.rm = TRUE),
            sum(!object$positive, na.rm = TRUE)),
    sprintf("top %d: %s", top_n,
            paste(sprintf("%s (%.4g)", top$vars, top$vimp), collapse = ", "))
  )
  .summary_skel(object, "gg_vimp", body)
}

#' @rdname summary.gg
#' @export
summary.gg_rfsrc <- function(object, ...) {
  body <- c(sprintf("rows: %d, cols: %d", nrow(object), ncol(object)))
  .summary_skel(object, "gg_rfsrc", body)
}

#' @rdname summary.gg
#' @export
summary.gg_variable <- function(object, ...) {
  resp <- intersect(c("yhat", "yvar"), names(object))
  body <- c(
    sprintf("rows: %d", nrow(object)),
    sprintf("response columns: %s",
            if (length(resp) > 0) paste(resp, collapse = ", ") else "(none)")
  )
  .summary_skel(object, "gg_variable", body)
}

# Internal: shared partial-summary body builder.
.partial_body <- function(x) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  yhat_rng  <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0 &&
                   "yhat" %in% names(x$continuous)) {
    sprintf("yhat range: [%.4g, %.4g]",
            min(x$continuous$yhat, na.rm = TRUE),
            max(x$continuous$yhat, na.rm = TRUE))
  } else {
    NULL
  }
  c(sprintf("continuous: %d, categorical: %d", nvar_cont, nvar_cat),
    yhat_rng)
}

.partialpro_body <- function(x) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  # gg_partialpro has parametric / nonparametric / causal columns, not yhat.
  rng_lines <- NULL
  if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    for (col in c("parametric", "nonparametric", "causal")) {
      if (col %in% names(x$continuous)) {
        vals <- x$continuous[[col]]
        rng_lines <- c(rng_lines,
          sprintf("%s range: [%.4g, %.4g]",
                  col, min(vals, na.rm = TRUE), max(vals, na.rm = TRUE)))
      }
    }
  }
  c(sprintf("continuous: %d, categorical: %d", nvar_cont, nvar_cat),
    rng_lines)
}

#' @rdname summary.gg
#' @export
summary.gg_partial <- function(object, ...) {
  .summary_skel(object, "gg_partial", .partial_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_partial_rfsrc <- function(object, ...) {
  .summary_skel(object, "gg_partial_rfsrc", .partial_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_partialpro <- function(object, ...) {
  .summary_skel(object, "gg_partialpro", .partialpro_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_roc <- function(object, ...) {
  body <- c(
    sprintf("thresholds: %d", nrow(object)),
    sprintf("AUC: %.4g",
            attr(object, "auc") %||% .gg_auc_trap(object))
  )
  .summary_skel(object, "gg_roc", body)
}

# Trapezoidal AUC from a gg_roc data.frame (fpr / tpr).
.gg_auc_trap <- function(x) {
  if (!all(c("fpr", "tpr") %in% names(x))) return(NA_real_)
  ord <- order(x$fpr)
  fpr <- x$fpr[ord]
  tpr <- x$tpr[ord]
  sum((fpr[-1] - fpr[-length(fpr)]) * (tpr[-1] + tpr[-length(tpr)])) / 2
}

#' @rdname summary.gg
#' @export
summary.gg_survival <- function(object, ...) {
  body <- c(
    sprintf("time range: [%.4g, %.4g]",
            min(object$time, na.rm = TRUE),
            max(object$time, na.rm = TRUE)),
    sprintf("rows: %d", nrow(object))
  )
  .summary_skel(object, "gg_survival", body)
}

#' @rdname summary.gg
#' @export
summary.gg_brier <- function(object, ...) {
  crps <- attr(object, "crps_integrated")
  envelope_mean <- mean(object$bs.upper - object$bs.lower, na.rm = TRUE)
  body <- c(
    sprintf("time range: [%.4g, %.4g]",
            min(object$time, na.rm = TRUE),
            max(object$time, na.rm = TRUE)),
    sprintf("peak Brier: %.4g at time %.4g",
            max(object$brier, na.rm = TRUE),
            object$time[which.max(object$brier)]),
    sprintf("integrated CRPS: %s",
            if (is.null(crps)) "NA" else sprintf("%.4g", crps)),
    sprintf("mean 15-85%% envelope width: %.4g", envelope_mean)
  )
  .summary_skel(object, "gg_brier", body)
}

##=============================================================================
#' Variable importance data from a varPro model
#'
#' Pulls the per-tree importance scores out of a fitted \code{varpro} object
#' and summarises them into a data structure the plot method can draw as a
#' boxplot.  The box hinges are the 15th and 85th percentiles and the
#' whiskers run to the 5th and 95th -- not the usual Tukey 1.5 IQR whiskers.
#' For a classification forest you can also keep the class-conditional
#' importances.
#'
#' @section What varpro is doing:
#' Permutation importance asks "what happens to OOB accuracy when I scramble
#' this variable?" That works, but it leans on artificial data (the
#' permuted column) and the answer can be unstable when variables are
#' correlated. The varpro framework (Lu and Ishwaran, 2024) replaces
#' permutation with \emph{release rules}. The forest is grown with guided
#' splitting; from a subset of trees varpro samples a collection of
#' decision-rule branches; for each variable it then compares the
#' response inside the rule's region to the response after the rule's
#' constraint on that variable is "released". The size of that change,
#' aggregated over many rules and trees, is the variable's importance.
#' No synthetic covariates, no permutation: the contrast is between two
#' real subsets of the data.
#'
#' Because varpro builds importance from rules sampled over trees, every
#' tree contributes its own importance value for each variable. Those are
#' the per-tree scores we summarise here. With \code{local.std = TRUE}
#' (the default) the per-tree values are standardised by their column
#' standard deviation so the column mean equals the aggregate z-score
#' returned by \code{varPro::importance()}; that z-score is the canonical
#' "is this variable in or out?" statistic, and \code{cutoff = 0.79} is
#' varpro's default selection threshold.
#'
#' For a classification forest, varpro also returns a class-conditional
#' z table: the same importance computed restricting attention to rules
#' relevant to each class. \code{conditional = TRUE} keeps that table so
#' the plot method can show which variables matter for which class
#' rather than only in aggregate.
#'
#' @section What's in the output:
#' \code{$imp} is the one-row-per-variable summary: aggregate z from
#' \code{varPro::importance()}, plus a \code{selected} flag for
#' \code{z > cutoff}. \code{$stats} holds the box quantiles
#' (5/15/50/85/95 percentiles, plus the raw mean) computed from the
#' per-tree matrix; these are what the boxplot draws. \code{$imp.tree}
#' is the per-tree matrix itself, kept only when \code{faithful = TRUE}
#' so the plot method can scatter individual tree values over the box.
#' \code{$conditional} is the tidy class x variable z table, present
#' only when \code{conditional = TRUE} and the family is
#' classification.
#'
#' @section What you use this for:
#' \itemize{
#'   \item rank candidate variables by importance and pick a working set
#'     above varpro's z cutoff;
#'   \item see, via the boxplot's spread and the per-tree points
#'     (\code{faithful = TRUE}), how stable each variable's importance
#'     is across trees: a high median with a wide box is a different
#'     story from a high median with a tight box;
#'   \item for a classification forest, ask which variables drive which
#'     class (\code{conditional = TRUE}) rather than just which
#'     variables drive the model overall.
#' }
#' The z-score is a standardised ranking statistic, not a p-value or a
#' probability. Two variables with the same z are "similarly important
#' by this method", not "equally likely to be true signal". For a
#' data-driven cutoff rather than the 0.79 default, see
#' \code{varPro::cv.varpro}.
#'
#' @references
#' Lu, M. and Ishwaran, H. (2024). Model-independent variable selection
#' via the rule-based variable priority framework. \emph{arXiv preprint}
#' \href{https://arxiv.org/abs/2409.09003}{arXiv:2409.09003}.
#'
#' @param object A fitted \code{varpro} object (required).
#' @param local.std Logical; default \code{TRUE}.  When \code{TRUE} the
#'   per-tree importances are put on the z-scale before the box statistics
#'   are computed.  Set it \code{FALSE} to keep the raw importance scale,
#'   which is what \code{type = "raw"} in \code{plot.gg_varpro} needs.
#' @param cutoff Numeric; the z-score above which a variable is treated as
#'   selected.  Default \code{0.79}.  A variable with aggregate z above the
#'   cutoff is flagged \code{selected = TRUE} in \code{$imp}.
#' @param faithful Logical; default \code{FALSE}.  When \code{TRUE},
#'   \code{$imp.tree} is kept so \code{plot.gg_varpro} can scatter the
#'   per-tree points over the box.
#' @param conditional Logical; default \code{FALSE}.  When \code{TRUE}, and
#'   only for a classification forest, the \code{$conditional.z} matrix is
#'   extracted and stored as \code{$conditional}.
#' @param nvar Integer; keep only the top \code{nvar} variables, ranked by
#'   median per-tree z, after the cutoff filter has been applied.
#'   \code{NULL} keeps all of them.
#' @param ... Additional arguments passed to \code{varPro::importance()}.
#'
#' @return A named list of class \code{"gg_varpro"} with elements:
#' \describe{
#'   \item{\code{$imp}}{Summary data frame: \code{variable} (factor whose
#'     levels run least- to most-important by median per-tree z, so the
#'     most-important variable sits at the top of the plot after
#'     \code{coord_flip}), \code{z} (aggregate
#'     z-score from \code{importance()}), \code{selected} (logical,
#'     \code{z > cutoff}).}
#'   \item{\code{$imp.tree}}{\code{NULL} when \code{faithful = FALSE};
#'     otherwise an ntree x p matrix of per-tree importance values.}
#'   \item{\code{$stats}}{Per-variable summary: \code{variable},
#'     \code{median}, \code{q05}, \code{q15}, \code{q85}, \code{q95}
#'     (on z-scale when \code{local.std = TRUE}, raw when \code{FALSE}),
#'     plus \code{mean} (raw importance mean, always stored).}
#'   \item{\code{$conditional}}{\code{NULL} when \code{conditional = FALSE};
#'     otherwise a data frame with columns \code{variable}, \code{class},
#'     \code{z} (one row per variable x class combination).}
#' }
#' A \code{"provenance"} attribute carries \code{family}, \code{local.std},
#' \code{cutoff}, \code{faithful}, \code{conditional}, \code{xvar.names},
#' and \code{n}.
#'
#' @seealso \code{\link{plot.gg_varpro}}, \code{\link{gg_vimp}}
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' vp <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
#' gg <- gg_varpro(vp)
#' print(gg)
#' plot(gg)
#' }
#'
#' @importFrom varPro importance
#' @export
gg_varpro <- function(object,
                      local.std   = TRUE,
                      cutoff      = 0.79,
                      faithful    = FALSE,
                      conditional = FALSE,
                      nvar        = NULL,
                      ...) {

  ## ---- Validation + coercion ------------------------------------------------
  local.std <- .validate_varpro_imp_inputs(object, local.std, faithful, conditional)

  ## ---- Call importance (local.std=FALSE gives mean & std columns) -----------
  imp_out <- varPro::importance(object, local.std = FALSE, ...)

  ## ---- Build per-tree importance matrix from object$results -----------------
  imp_tree_mat <- .build_varpro_imp_tree(object)

  ## ---- Build tidy data structures ------------------------------------------
  dfs <- .build_varpro_imp_dfs(imp_out, imp_tree_mat, object$family,
                                cutoff, nvar, faithful, local.std, conditional)

  ## ---- Assemble result ------------------------------------------------------
  result <- structure(
    list(
      imp         = dfs$imp,
      imp.tree    = dfs$imp_tree,
      stats       = dfs$stats,
      conditional = dfs$conditional
    ),
    class = c("gg_varpro", "list")
  )

  attr(result, "provenance") <- list(
    family      = object$family,
    local.std   = local.std,  # resolved value (may differ from arg when faithful=TRUE)
    cutoff      = cutoff,
    faithful    = faithful,
    conditional = conditional,
    xvar.names  = object$xvar.names,
    n           = nrow(object$x)
  )

  result
}

## ---- Internal helpers -------------------------------------------------------

#' @keywords internal
.validate_varpro_imp_inputs <- function(object, local.std, faithful, conditional) {
  if (missing(object) || is.null(object)) {
    stop("'object' must be a fitted varpro object.", call. = FALSE)
  }
  if (!inherits(object, "varpro")) {
    stop("'object' must be a varpro fit (class \"varpro\").", call. = FALSE)
  }
  if (conditional && !identical(object$family, "class")) {
    stop("conditional=TRUE requires a classification forest ",
         "(object$family == \"class\").", call. = FALSE)
  }
  invisible(local.std)
}

#' Build per-tree importance matrix from varpro object results
#'
#' Reconstructs the ntree x p per-tree importance matrix from
#' \code{object$results}, replicating the aggregation performed internally
#' by \code{varPro:::.importance.varpro.workhorse}.
#' @keywords internal
.build_varpro_imp_tree <- function(object) {
  dta        <- object$results
  xvar.names <- object$xvar.names

  dta$variable <- factor(xvar.names[dta$variable])
  xvarused.names <- levels(dta$variable)
  p          <- length(xvarused.names)
  wt.vec     <- rep(1, p)
  names(wt.vec) <- xvarused.names

  trees       <- sort(unique(dta$tree))
  ntree.used  <- length(trees)
  tree.idx    <- match(dta$tree, trees)
  var.idx     <- match(as.character(dta$variable), xvarused.names)
  grp         <- tree.idx + (var.idx - 1L) * ntree.used

  w          <- dta$n.oob
  numer      <- dta$imp * w
  numer[is.na(numer)] <- 0
  numer.sum  <- rowsum(numer, grp, reorder = FALSE)
  denom.sum  <- rowsum(w,     grp, reorder = FALSE)
  ratio      <- as.numeric(numer.sum[, 1L] / denom.sum[, 1L])
  ratio[!is.finite(ratio)] <- 0

  imp_tree <- matrix(0, nrow = ntree.used, ncol = p,
    dimnames = list(as.character(trees), xvarused.names))
  ## g.rows are the grp keys (1..ntree.used*p); decode back to (tree, var) indices.
  ## reorder=FALSE preserves insertion order but rownames() always give the
  ## original grp key values, so the modular decode is stable regardless of order.
  g.rows   <- as.integer(rownames(numer.sum))
  imp_tree[cbind(((g.rows - 1L) %% ntree.used) + 1L,
                 ((g.rows - 1L) %/% ntree.used) + 1L)] <- ratio
  sweep(imp_tree, 2L, wt.vec, `*`)
}

#' Compute per-variable box statistics from a per-tree importance matrix
#'
#' When \code{local.std = TRUE} the columns of \code{mat} are standardised
#' to unit variance before computing quantiles so that the display scale
#' matches the aggregate z-score.  The \code{mean} column always stores raw
#' (unstandardised) column means.
#' @keywords internal
.varpro_imp_stats <- function(mat, local.std = TRUE) {
  if (local.std) {
    sd_j <- apply(mat, 2L, stats::sd, na.rm = TRUE)
    sd_j[sd_j < .Machine$double.eps] <- 1   # guard zero-variance columns
    # z_ij = imp_ij / sd_j so that mean(z_ij) == aggregate z_j from importance()
    display_mat <- sweep(mat, 2L, sd_j, FUN = "/")
  } else {
    display_mat <- mat
  }
  data.frame(
    variable = colnames(mat),
    median   = apply(display_mat, 2L, stats::quantile, probs = 0.50,
                     na.rm = TRUE),
    q05      = apply(display_mat, 2L, stats::quantile, probs = 0.05,
                     na.rm = TRUE),
    q15      = apply(display_mat, 2L, stats::quantile, probs = 0.15,
                     na.rm = TRUE),
    q85      = apply(display_mat, 2L, stats::quantile, probs = 0.85,
                     na.rm = TRUE),
    q95      = apply(display_mat, 2L, stats::quantile, probs = 0.95,
                     na.rm = TRUE),
    mean     = colMeans(mat, na.rm = TRUE),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' Build the tidy importance data structures from importance() output
#' @keywords internal
.build_varpro_imp_dfs <- function(imp_out, imp_tree_mat, family, cutoff, nvar,
                                   faithful, local.std, conditional) {
  is_class <- identical(family, "class")

  ## Unpack importance() data frame: different shapes per family
  if (is_class && is.list(imp_out) && !is.data.frame(imp_out)) {
    imp_df_raw <- imp_out$unconditional
    cond_mat   <- if (conditional) imp_out$conditional.z else NULL
  } else {
    imp_df_raw <- imp_out
    cond_mat   <- NULL
  }

  ## Replace NaN z with 0 for sorting/selection
  z_vec <- imp_df_raw$z
  z_vec[!is.finite(z_vec)] <- 0

  ## $imp: one row per variable
  imp_df <- data.frame(
    variable = rownames(imp_df_raw),
    z        = as.numeric(z_vec),
    selected = as.numeric(z_vec) > cutoff,
    stringsAsFactors = FALSE
  )

  ## Keep only columns present in both imp_df and imp_tree_mat
  keep_vars  <- imp_df$variable[imp_df$variable %in% colnames(imp_tree_mat)]
  imp_tree_k <- imp_tree_mat[, keep_vars, drop = FALSE]

  ## $stats: box quantiles per variable (full set before nvar truncation)
  stats_df <- .varpro_imp_stats(imp_tree_k, local.std = local.std)

  ## Apply nvar truncation by median z (spec: "top-nvar by median z")
  if (!is.null(nvar)) {
    top_vars <- stats_df$variable[order(-stats_df$median)][
      seq_len(min(nvar, nrow(stats_df)))]
    imp_df   <- imp_df[imp_df$variable %in% top_vars, , drop = FALSE]
    stats_df <- stats_df[stats_df$variable %in% top_vars, , drop = FALSE]
  }

  ## Factor levels run least-important-first (reversed descending median z)
  ## so the most-important variable lands at the TOP after coord_flip in the
  ## plot method, matching the gg_vimp convention. `var_order` itself stays
  ## most-important-first for row ordering and top-nvar selection.
  var_order         <- stats_df$variable[order(-stats_df$median)]
  lvl               <- rev(as.character(var_order))
  imp_df$variable   <- factor(imp_df$variable,   levels = lvl)
  stats_df$variable <- factor(stats_df$variable, levels = lvl)
  ## Rows stay most-important-first (descending median z); the reversed
  ## factor levels above only drive the plot's vertical order, not row order.
  imp_df   <- imp_df[order(imp_df$variable,   decreasing = TRUE), , drop = FALSE]
  stats_df <- stats_df[order(stats_df$variable, decreasing = TRUE), , drop = FALSE]
  rownames(imp_df)   <- NULL
  rownames(stats_df) <- NULL

  ## $conditional: tidy class-conditional z-scores
  cond_df <- NULL
  if (!is.null(cond_mat)) {
    cond_df <- data.frame(
      variable = rep(rownames(cond_mat), ncol(cond_mat)),
      class    = rep(colnames(cond_mat), each = nrow(cond_mat)),
      z        = as.vector(cond_mat),
      stringsAsFactors = FALSE
    )
    cond_df <- cond_df[cond_df$variable %in% keep_vars, , drop = FALSE]
    cond_df$variable <- factor(cond_df$variable,
                               levels = levels(imp_df$variable))
  }

  list(
    imp         = imp_df,
    imp_tree    = if (faithful) imp_tree_k else NULL,
    stats       = stats_df,
    conditional = cond_df
  )
}

# Tests for gg_varpro (Phase 2: importance extractor)

## ── Helpers ──────────────────────────────────────────────────────────────────

# Regression fit — fast, always available (mtcars is base R)
make_vp_regr <- function(ntree = 25L) {
  # Supervised varpro grow (real Y) — UBSAN-clean; runs on CRAN. See
  # helper-varpro-fixtures.R for why varPro grows are safe except isopro(unsupv).
  set.seed(42L)
  varPro::varpro(mpg ~ ., data = mtcars, ntree = ntree)
}

# Classification fit — iris, always available
make_vp_class <- function(ntree = 25L) {
  set.seed(42L)
  varPro::varpro(Species ~ ., data = iris, ntree = ntree)
}

## ── Input validation ─────────────────────────────────────────────────────────

test_that("gg_varpro: missing object -> stop", {
  expect_error(gg_varpro(), regexp = "object")
})

test_that("gg_varpro: non-varpro object -> stop", {
  expect_error(gg_varpro(list(x = 1)), regexp = "varpro")
})

test_that("gg_varpro: conditional=TRUE on regression -> stop", {
  vp <- make_vp_regr()
  expect_error(gg_varpro(vp, conditional = TRUE),
               regexp = "classification")
})

test_that("gg_varpro: faithful=TRUE + local.std=TRUE is valid, records local.std=TRUE", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, faithful = TRUE, local.std = TRUE)
  expect_true(is.matrix(gg$imp.tree))
  expect_true(attr(gg, "provenance")$local.std)
})

## ── Class & structure ────────────────────────────────────────────────────────

test_that("gg_varpro returns gg_varpro class", {
  vp <- make_vp_regr()
  expect_s3_class(gg_varpro(vp), "gg_varpro")
})

test_that("gg_varpro$imp has variable, z, selected columns", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  expect_named(gg$imp, c("variable", "z", "selected"))
  expect_s3_class(gg$imp$variable, "factor")
  expect_type(gg$imp$z, "double")
  expect_type(gg$imp$selected, "logical")
})

test_that("gg_varpro$stats has expected columns", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  expect_true(all(c("variable", "median", "q05", "q15", "q85", "q95", "mean")
                  %in% names(gg$stats)))
})

test_that("gg_varpro$imp.tree is NULL when faithful=FALSE", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, faithful = FALSE)
  expect_null(gg$imp.tree)
})

test_that("gg_varpro$imp.tree is a matrix when faithful=TRUE", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, faithful = TRUE)
  expect_true(is.matrix(gg$imp.tree))
})

test_that("gg_varpro$conditional is NULL when conditional=FALSE", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, conditional = FALSE)
  expect_null(gg$conditional)
})

test_that("gg_varpro$conditional has variable, class, z when conditional=TRUE", {
  vp <- make_vp_class()
  gg <- gg_varpro(vp, conditional = TRUE)
  expect_false(is.null(gg$conditional))
  expect_true(all(c("variable", "class", "z") %in% names(gg$conditional)))
})

## ── Provenance attribute ─────────────────────────────────────────────────────

test_that("gg_varpro provenance has all expected fields", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  prov <- attr(gg, "provenance")
  expect_type(prov, "list")
  expect_true(all(c("family", "local.std", "cutoff", "faithful",
                    "conditional", "xvar.names", "n") %in% names(prov)))
})

test_that("gg_varpro provenance cutoff matches argument", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, cutoff = 1.2)
  expect_equal(attr(gg, "provenance")$cutoff, 1.2)
})

## ── cutoff and nvar ──────────────────────────────────────────────────────────

test_that("gg_varpro$imp$selected reflects z > cutoff", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, cutoff = 0.79)
  expect_true(all(gg$imp$z[gg$imp$selected]  >  0.79))
  expect_true(all(gg$imp$z[!gg$imp$selected] <= 0.79))
})

test_that("gg_varpro nvar=3 returns exactly 3 variables", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, nvar = 3L)
  expect_equal(nrow(gg$imp), 3L)
  expect_equal(nrow(gg$stats), 3L)
})

test_that("gg_varpro variable factor runs least- to most-important median z", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  # Factor levels run least-important first so the most-important variable is
  # the last level (top of the plot after coord_flip). Median z therefore
  # increases along the factor levels.
  med_vals <- gg$stats$median[order(match(gg$stats$variable,
                                           levels(gg$imp$variable)))]
  expect_true(all(diff(med_vals) >= 0))
})

test_that("gg_varpro nvar larger than p returns all variables", {
  vp <- make_vp_regr()
  n_all <- nrow(gg_varpro(vp)$imp)
  gg <- gg_varpro(vp, nvar = 999L)
  expect_equal(nrow(gg$imp), n_all)
})

test_that("gg_varpro local.std=FALSE stats equal raw-column medians", {
  ## Mechanistic check: verify each path against the actual column computation.
  ## faithful=TRUE forces local.std=FALSE (coercion tested above); imp.tree
  ## gives the ground-truth raw importance matrix for both assertions.
  vp <- make_vp_regr(ntree = 50L)
  suppressMessages(gg_raw <- gg_varpro(vp, local.std = FALSE, faithful = TRUE))
  mat   <- gg_raw$imp.tree          # ntree x p raw importance matrix
  vname <- colnames(mat)[[1L]]      # pick any variable by name, not position

  ## local.std = FALSE: stats$median == median of raw column
  expect_equal(
    gg_raw$stats$median[as.character(gg_raw$stats$variable) == vname],
    stats::median(mat[, vname], na.rm = TRUE),
    tolerance = 1e-10
  )

  ## local.std = TRUE: stats$median == median of z-normalised column
  gg_z <- gg_varpro(vp, local.std = TRUE)
  sd_j <- apply(mat, 2L, stats::sd, na.rm = TRUE)
  sd_j[sd_j < .Machine$double.eps] <- 1
  z_med <- stats::median(mat[, vname] / sd_j[[vname]], na.rm = TRUE)
  expect_equal(
    gg_z$stats$median[as.character(gg_z$stats$variable) == vname],
    z_med,
    tolerance = 1e-10
  )
})

## ── Z-scale alignment ─────────────────────────────────────────────────────────

test_that("gg_varpro: z-normalisation mean(z_ij) == aggregate z_j", {
  # varPro's importance() computes z_j = mean(imp_ij) / sd_j (no sqrt(ntree)).
  # So .varpro_imp_stats uses z_ij = imp_ij / sd_j, giving mean(z_ij) == z_j.
  # Verify by recomputing from imp.tree (faithful=TRUE).
  vp <- make_vp_regr(ntree = 50L)
  gg <- gg_varpro(vp, faithful = TRUE)
  mat <- gg$imp.tree  # ntree x p raw importance matrix
  sd_j <- apply(mat, 2L, stats::sd, na.rm = TRUE)
  sd_j[sd_j < .Machine$double.eps] <- 1
  z_mat <- sweep(mat, 2L, sd_j, FUN = "/")  # z_ij = imp_ij / sd_j
  z_means <- colMeans(z_mat, na.rm = TRUE)
  # Column means should equal aggregate z from importance()
  agg_z <- setNames(gg$imp$z, as.character(gg$imp$variable))
  common <- intersect(names(z_means), names(agg_z))
  expect_gt(length(common), 0L)
  expect_equal(z_means[common], agg_z[common], tolerance = 0.1,
               info = "mean per-tree z should approximate aggregate z within 0.1")
})

## ── Plot smoke tests ─────────────────────────────────────────────────────────

test_that("plot.gg_varpro default returns a ggplot", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  p  <- plot(gg)
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_varpro type='z' returns a ggplot", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)  # local.std=TRUE default
  p  <- plot(gg, type = "z")
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_varpro type='raw' with local.std=FALSE returns a ggplot", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, local.std = FALSE)
  p  <- plot(gg, type = "raw")
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_varpro type='raw' with local.std=TRUE -> stop", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, local.std = TRUE)
  expect_error(plot(gg, type = "raw"), regexp = "local\\.std")
})

test_that("plot.gg_varpro type='z' with local.std=FALSE -> stop", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, local.std = FALSE)
  expect_error(plot(gg, type = "z"), regexp = "local\\.std")
})

test_that("plot.gg_varpro faithful=TRUE returns a ggplot", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp, faithful = TRUE)
  p  <- plot(gg)
  expect_s3_class(p, "ggplot")
})

test_that("plot.gg_varpro conditional=TRUE returns ggplot with FacetWrap", {
  vp <- make_vp_class()
  gg <- gg_varpro(vp, conditional = TRUE)
  p  <- plot(gg)
  expect_s3_class(p, "ggplot")
  expect_true(inherits(p$facet, "FacetWrap"))
})

## ── S3 companions ────────────────────────────────────────────────────────────

test_that("autoplot.gg_varpro returns a ggplot", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  expect_s3_class(ggplot2::autoplot(gg), "ggplot")
})

test_that("print.gg_varpro returns object invisibly", {
  vp  <- make_vp_regr()
  gg  <- gg_varpro(vp)
  out <- capture.output(ret <- print(gg))
  expect_identical(ret, gg)
  expect_true(any(grepl("gg_varpro", out)))
})

test_that("print.gg_varpro output contains selected/total counts", {
  vp  <- make_vp_regr()
  gg  <- gg_varpro(vp, cutoff = 0.79)
  out <- capture.output(print(gg))
  expect_true(any(grepl("selected", out, ignore.case = TRUE)))
})

test_that("summary.gg_varpro returns summary.gg_varpro class", {
  vp <- make_vp_regr()
  gg <- gg_varpro(vp)
  s  <- summary(gg)
  expect_s3_class(s, "summary.gg_varpro")
})

####**********************************************************************
####  autoplot.gg_* S3 methods.
####
####  Thin wrappers that delegate to the corresponding plot.gg_*() method,
####  letting users call ggplot2::autoplot() in pipe-friendly workflows
####  and inside patchwork / cowplot compositions.
####
####  Each method accepts ... and passes it straight through to plot.gg_*
####  so all existing plot arguments remain available.
####**********************************************************************

#' @importFrom ggplot2 autoplot
NULL

#' \code{autoplot} methods for \pkg{ggRandomForests} data objects
#'
#' These let you call \code{ggplot2::autoplot()} on any \code{gg_*} object
#' \pkg{ggRandomForests} returns. Each is a thin wrapper around the matching
#' \code{plot.gg_*()} S3 method, and \code{...} passes straight through, so
#' every argument those plot methods take is still available here.
#'
#' @param object A \code{gg_*} data object (see Details).
#' @param ... Additional arguments passed to the underlying
#'   \code{plot.gg_*()} method.
#'
#' @return A \code{ggplot} object.
#'
#' @details
#' The following \code{gg_*} classes are supported:
#' \describe{
#'   \item{\code{gg_error}}{OOB error vs. number of trees}
#'   \item{\code{gg_vimp}}{Variable importance ranking}
#'   \item{\code{gg_rfsrc}}{Predicted vs. observed values}
#'   \item{\code{gg_variable}}{Marginal dependence}
#'   \item{\code{gg_partial}}{Partial dependence (via \code{plot.variable})}
#'   \item{\code{gg_partial_rfsrc}}{Partial dependence (via \code{partial.rfsrc})}
#'   \item{\code{gg_partial_varpro}}{Partial dependence (via \code{varPro})}
#'   \item{\code{gg_partialpro}}{Partial dependence via \code{varPro} (deprecated alias)}
#'   \item{\code{gg_varpro}}{Variable importance from \code{varPro}}
#'   \item{\code{gg_roc}}{ROC curve}
#'   \item{\code{gg_survival}}{Survival / cumulative hazard curves}
#'   \item{\code{gg_brier}}{Time-resolved Brier score and CRPS}
#' }
#'
#' @name autoplot.gg
#'
#' @examples
#' \donttest{
#' library(ggplot2)
#' set.seed(42)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
#'                               ntree = 50, importance = TRUE,
#'                               tree.err = TRUE)
#' autoplot(gg_error(rf))
#' autoplot(gg_vimp(rf))
#' }
NULL

#' @rdname autoplot.gg
#' @export
autoplot.gg_error <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_vimp <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_rfsrc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_variable <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partial <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partial_rfsrc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partialpro <- function(object, ...) {
  ## Deprecated-class shim: re-dispatch to autoplot.gg_partial_varpro.
  class(object) <- c("gg_partial_varpro", setdiff(class(object), "gg_partialpro"))
  autoplot.gg_partial_varpro(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_partial_varpro <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_roc <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_survival <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_brier <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_varpro <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_udependent <- function(object, ...) {
  plot(object, ...)
}

#' @rdname autoplot.gg
#' @export
autoplot.gg_isopro <- function(object, ...) {
  plot(object, ...)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_error}} object
#'
#' A plot of the cumulative OOB error rates of the random forest as a
#' function of number of trees.
#'
#' @param x A \code{\link{gg_error}} object created from either a
#'   \code{\link[randomForestSRC]{rfsrc}} or a
#'   \code{\link[randomForest]{randomForest}} object.  A raw forest object
#'   may also be supplied and will be passed through \code{\link{gg_error}}
#'   automatically before plotting.
#' @param ... Extra arguments forwarded to the underlying \code{ggplot2}
#'   geometry calls (e.g. \code{size}, \code{linetype}).
#'
#' @return A \code{ggplot} object with \code{ntree} on the x-axis and
#'   OOB error rate on the y-axis.  Single-outcome forests (regression,
#'   survival) produce a single line; multi-outcome forests (classification)
#'   produce one coloured line per class.
#'
#' @details The gg_error plot is used to track the convergence of the
#' randomForest. This figure is a reproduction of the error plot
#' from the \code{\link[randomForestSRC]{plot.rfsrc}} function.
#'
#' @seealso \code{\link{gg_error}} \code{\link[randomForestSRC]{rfsrc}}
#'  \code{\link[randomForestSRC]{plot.rfsrc}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
#' 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## Examples from RFSRC package...
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## ------------- iris data
#' ## You can build a randomForest
#' rfsrc_iris <- rfsrc(Species ~ ., data = iris,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_iris)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## RandomForest example
#' rf_iris <- randomForest::randomForest(Species ~ .,
#'   data = iris,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#' gg_dta <- gg_error(rf_iris)
#' plot(gg_dta)
#'
#' gg_dta <- gg_error(rf_iris, training = TRUE)
#' plot(gg_dta)
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#' ## ------------- airq data
#' rfsrc_airq <- rfsrc(Ozone ~ .,
#'   data = airquality,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_airq)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- Boston data
#' data(Boston, package = "MASS")
#' Boston$chas <- as.logical(Boston$chas)
#' rfsrc_boston <- rfsrc(medv ~ .,
#'   data = Boston,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_boston)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## ------------- mtcars data
#' rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars,
#'   importance = TRUE,
#'   save.memory = TRUE,
#'   forest = TRUE,
#'   tree.err = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta<- gg_error(rfsrc_mtcars)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## ------------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        tree.err = TRUE)
#'
#' gg_dta <- gg_error(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## ------------- pbc data
#' # Load a cached randomForestSRC object
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC",)
#' # For whatever reason, the age variable is in days... makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'  if (!is.factor(pbc[, ind])) {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  } else {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  }
#'  if (!is.logical(pbc[, ind]) &
#'      length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'    pbc[, ind] <- factor(pbc[, ind])
#'  }
#' }
#' #Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' #========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'  dta_train,
#'  nsplit = 10,
#'  na.action = "na.impute",
#'  tree.err = TRUE,
#'  forest = TRUE,
#'  importance = TRUE,
#'  save.memory = TRUE
#' )
#'
#'
#' gg_dta <- gg_error(rfsrc_pbc)
#' plot(gg_dta)
#'
#' @importFrom ggplot2 ggplot geom_line theme labs
#' @importFrom tidyr pivot_longer
#' @export
plot.gg_error <- function(x, ...) {
  gg_dta <- x

  # Accept a raw rfsrc object and extract error rates on the fly
  if (inherits(gg_dta, "rfsrc")) {
    gg_dta <- gg_error(gg_dta)
  }

  if (!inherits(gg_dta, "gg_error")) {
    stop("Incorrect object type: Expects a gg_error object")
  }

  # Use points instead of lines when there is only one non-NA row (e.g. a
  # forest built with a single tree, or one where only ntree=1 has an error
  # rate recorded).  A line plot with one point renders nothing visible.
  point <- FALSE
  if (nrow(na.omit(gg_dta)) < 2) {
    point <- TRUE
  }

  if (ncol(gg_dta) > 2) {
    # Multi-outcome (classification): gg_error has one column per class plus
    # the "ntree" column.  Pivot to long form so we can colour by outcome.
    gg_dta <- tidyr::pivot_longer(gg_dta, -"ntree", names_to = "variable", values_to = "value")
    gg_plt <-
      ggplot2::ggplot(na.omit(gg_dta),
                      ggplot2::aes(x = .data[["ntree"]], y = .data[["value"]],
                                   col = .data[["variable"]]))
  } else {
    # Single-outcome (regression / survival): gg_error has columns "ntree"
    # and "error".  Map directly without reshaping.
    gg_plt <-
      ggplot2::ggplot(na.omit(gg_dta), ggplot2::aes(x = .data[["ntree"]],
                                                    y = .data[["error"]]))
  }

  if (point) {
    gg_plt <- gg_plt +
      ggplot2::geom_point() +
      ggplot2::labs(x = "Number of Trees", y = "OOB Error Rate", color = "Outcome")
  } else {
    gg_plt <- gg_plt +
      ggplot2::geom_line() +
      ggplot2::labs(x = "Number of Trees", y = "OOB Error Rate", color = "Outcome")
  }

  # Hide the legend when there is only a single outcome variable — the colour
  # key adds no information and clutters the plot.  For single-outcome forests
  # (regression / survival) the data is never gathered, so there is no
  # "variable" column; suppress the legend unconditionally in that case.
  if (!"variable" %in% names(gg_dta) ||
      length(unique(gg_dta$variable)) <= 1) {
    gg_plt <- gg_plt + ggplot2::theme(legend.position = "none")
  }
  return(gg_plt)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' ROC plot generic function for a \code{\link{gg_roc}} object.
#'
#' @param x A \code{\link{gg_roc}} object, or a raw
#'   \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} classification forest. Hand it a
#'   forest and \code{\link{gg_roc}} is called for you.
#' @param which_outcome Integer; for multi-class problems, the index of the
#'   class to plot. When \code{NULL} (default) and the forest has more than two
#'   classes, the curves for all classes are overlaid in one plot. For binary
#'   forests, \code{NULL} defaults to class index 2.
#' @param panel Character; layout for per-class ROC objects, the ones from
#'   \code{gg_roc(..., per_class = TRUE)}. \code{"overlay"} (default) draws
#'   every class curve in one panel, coloured by class; \code{"facet"} gives
#'   each class its own panel. Ignored for single-class \code{gg_roc} objects.
#' @param ... Additional arguments passed to \code{\link{gg_roc}} when
#'   \code{x} is a raw forest (e.g. \code{oob = FALSE}).
#'
#' @return A \code{ggplot} object. The x-axis is 1 - Specificity (FPR), the
#'   y-axis is Sensitivity (TPR), and a dashed red diagonal marks the
#'   random-classifier baseline. Single-class curves carry the AUC as an
#'   annotation; multi-class plots colour and style each class curve
#'   distinctly.
#'
#' @seealso \code{\link{gg_roc}} \code{\link{calc_roc}} \code{\link{calc_auc}}
#'   \code{\link[randomForestSRC]{rfsrc}}
#'   \code{\link[randomForest]{randomForest}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
#' Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' # Build a small classification forest (ntree=50 keeps example fast)
#' set.seed(42)
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' # ROC for setosa (outcome index 1)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
#' plot(gg_dta)
#'
#' # ROC for versicolor (outcome index 2)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
#' plot(gg_dta)
#'
#' # ROC for virginica (outcome index 3)
#' gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
#' plot(gg_dta)
#'
#' # Plot all three ROC curves in one call by iterating over outcome indices
#' n_cls <- ncol(rfsrc_iris$predicted)
#' for (i in seq_len(n_cls)) print(plot(gg_roc(rfsrc_iris, which_outcome = i)))
#'
#' @export
plot.gg_roc <- function(x, which_outcome = NULL, ...,
                        panel = c("overlay", "facet")) {
  # `panel` is placed after `...` so it is name-only: this preserves
  # positional back-compatibility for existing callers (e.g.
  # plot(x, 1, FALSE) still routes the 3rd positional arg into `...`).
  panel <- match.arg(panel)
  gg_dta <- x

  ## ---- Accept a raw rfsrc or randomForest object -----------------------
  # If we call this with a forest object instead of a gg_roc object,
  # automatically compute the ROC data for the appropriate class(es).
  if (inherits(gg_dta, "rfsrc")) {
    if (inherits(gg_dta, "class")) {
      # Determine the number of outcome classes
      crv <- dim(gg_dta$predicted)[2]

      if (crv > 2 && is.null(which_outcome)) {
        # Multi-class: compute ROC for every class in parallel
        gg_dta <- lapply(seq_len(crv), function(ind) {
          gg_roc(gg_dta, which_outcome = ind, ...)
        })
      } else {
        # Binary (or user specified a class): default to second column
        if (is.null(which_outcome)) {
          which_outcome <- 2
        }
        gg_dta <- gg_roc(gg_dta, which_outcome, ...)
      }
    } else {
      stop("gg_roc expects a classification randomForest.")
    }
  } else if (inherits(gg_dta, "randomForest")) {
    if (gg_dta$type == "classification") {
      # Determine the number of outcome classes
      crv <- length(levels(gg_dta$predicted))
      if (crv > 2 && is.null(which_outcome)) {
        # Multi-class: compute ROC for every class in parallel
        gg_dta <- lapply(seq_len(crv), function(ind) {
          gg_roc(gg_dta, which_outcome = ind, ...)
        })
      } else {
        # Binary (or user specified a class): default to second column
        if (is.null(which_outcome)) {
          which_outcome <- 2
        }
        gg_dta <- gg_roc(gg_dta, which_outcome, ...)
      }
    }
  }

  ## ---- Single-class ROC plot (or per_class long-format) ----------------
  if (inherits(gg_dta, "gg_roc")) {

    # Per-class detection: gg_roc produced by gg_roc(..., per_class = TRUE)
    # carries a 'class' column (factor) + a named AUC vector attribute.
    # Rendering is delegated to a helper to keep this method's cyclomatic
    # complexity within the project lint budget.
    if ("class" %in% names(gg_dta)) {
      return(.plot_gg_roc_per_class(gg_dta, attr(x, "auc"), panel))
    }

    # Sort by specificity so the ROC curve is drawn left-to-right
    gg_dta <- gg_dta[order(gg_dta$spec), ]
    # False positive rate = 1 - specificity
    gg_dta$fpr <- 1 - gg_dta$spec
    auc <- calc_auc(gg_dta)

    gg_plt <- ggplot2::ggplot(data = gg_dta) +
      ggplot2::geom_line(ggplot2::aes(x = .data$fpr, y = .data$sens)) +
      ggplot2::labs(x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)") +
      # Reference diagonal for a random classifier
      ggplot2::geom_abline(
        slope = 1,
        intercept = 0,
        col = "red",
        linetype = 2,
        linewidth = .5
      ) +
      ggplot2::coord_fixed()

    # Annotate the plot with the computed AUC value
    gg_plt <- gg_plt +
      ggplot2::annotate(
        x = .5,
        y = .2,
        geom = "text",
        label = paste("AUC = ", round(auc, digits = 3), sep = ""),
        hjust = 0
      )
  } else {
    ## ---- Multi-class ROC plot (list of gg_roc objects) ----------------
    # Sort each class's data by specificity
    gg_dta <- lapply(gg_dta, function(st) {
      st <- st[order(st$spec), ]
      st
    })
    # Compute FPR for each class
    gg_dta <- lapply(gg_dta, function(st) {
      st$fpr <- 1 - st$spec
      st
    })
    # Tag each subset with its outcome index for colour/linetype mapping
    gg_dta <- lapply(seq_along(gg_dta), function(ind) {
      gg_dta[[ind]]$outcome <- ind
      gg_dta[[ind]]
    })

    # Compute AUC for each class
    auc <- lapply(gg_dta, function(st) {
      calc_auc(st)
    })

    # Combine all classes into a single data frame for faceting/colouring
    o_dta <- do.call(rbind, gg_dta)
    o_dta$outcome <- factor(o_dta$outcome)

    gg_plt <- ggplot2::ggplot(data = o_dta) +
      ggplot2::geom_line(ggplot2::aes(
        x = .data$fpr,
        y = .data$sens,
        linetype = .data$outcome,
        col = .data$outcome
      )) +
      ggplot2::labs(x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)") +
      # Reference diagonal for a random classifier
      ggplot2::geom_abline(
        slope = 1,
        intercept = 0,
        col = "red",
        linetype = 2,
        linewidth = .5
      ) +
      ggplot2::coord_fixed()

    # Multi-class: do not annotate a single AUC value - each class has its own.
  }
  return(gg_plt)
}

# Render a per-class (one-vs-rest) ROC object produced by
# gg_roc(..., per_class = TRUE).  Split out of plot.gg_roc() so that method
# stays within the project's cyclomatic-complexity lint budget.
#
# gg_dta : long-format gg_roc data frame with a 'class' factor column
# auc    : named numeric AUC vector (one entry per class) or NULL
# panel  : "overlay" (curves coloured by class) or "facet" (one panel each)
.plot_gg_roc_per_class <- function(gg_dta, auc, panel) {
  gg_dta$fpr <- 1 - gg_dta$spec

  if (panel == "overlay") {
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_line(ggplot2::aes(
        x = .data$fpr, y = .data$sens, color = .data$class
      )) +
      ggplot2::labs(
        x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)",
        color = "Class"
      )
  } else {
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_line(ggplot2::aes(x = .data$fpr, y = .data$sens)) +
      ggplot2::labs(x = "1 - Specificity (FPR)", y = "Sensitivity (TPR)") +
      ggplot2::facet_wrap(~class)
  }

  gg_plt <- gg_plt +
    ggplot2::geom_abline(
      slope = 1, intercept = 0,
      col = "red", linetype = 2, linewidth = .5
    ) +
    ggplot2::coord_fixed()

  # AUC caption - top 5 classes by descending AUC (already sorted)
  if (!is.null(auc) && length(auc) > 0L) {
    top_n   <- min(5L, length(auc))
    auc_str <- paste(
      sprintf("%s=%.3g", names(auc)[seq_len(top_n)], auc[seq_len(top_n)]),
      collapse = ", "
    )
    if (length(auc) > 5L) auc_str <- paste0(auc_str, ", ...")
    gg_plt <- gg_plt +
      ggplot2::labs(caption = paste("OvR ROC, per_class=TRUE. AUC:", auc_str))
  }
  gg_plt
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)


## ----packages-----------------------------------------------------------------
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())


## ----data-load----------------------------------------------------------------
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter


## ----variable-labels----------------------------------------------------------
st_labs <- c(
  crim    = "Crime rate by town",
  zn      = "Residential land zoned > 25k sq ft (%)",
  indus   = "Non-retail business acres (%)",
  chas    = "Borders Charles River",
  nox     = "Nitrogen oxides (10 ppm)",
  rm      = "Rooms per dwelling",
  age     = "Units built before 1940 (%)",
  dis     = "Distance to employment centers",
  rad     = "Highway accessibility index",
  tax     = "Property tax rate per $10,000",
  ptratio = "Pupil-teacher ratio",
  black   = "Proportion of Black residents",
  lstat   = "Lower status population (%)",
  medv    = "Median home value ($1000s)"
)


## ----eda, fig.cap="EDA: each predictor vs. median home value, colored by Charles River indicator."----
dta <- Boston |>
  pivot_longer(c(-medv, -chas), names_to = "variable", values_to = "value")

ggplot(dta, aes(x = medv, y = value, color = chas)) +
  geom_point(alpha = 0.4) +
  geom_smooth(aes(x = medv, y = value), color = "grey30",
              inherit.aes = FALSE, se = FALSE) +
  labs(y = "", x = st_labs["medv"]) +
  scale_color_brewer(palette = "Set2") +
  facet_wrap(~variable, scales = "free_y", ncol = 3)


## ----rfsrc-fit----------------------------------------------------------------
rfsrc_Boston <- rfsrc(medv ~ ., data = Boston, # nolint: object_name_linter
                      importance = TRUE, err.block = 5)
rfsrc_Boston


## ----error-plot, fig.height=4, fig.cap="OOB mean squared error vs. number of trees."----
gg_e <- gg_error(rfsrc_Boston)
gg_e <- gg_e |> filter(!is.na(error))
class(gg_e) <- c("gg_error", class(gg_e))
plot(gg_e)


## ----rfsrc-predicted, fig.height=4, fig.cap="OOB predicted median home values. Points are jittered; boxplot shows the distribution."----
plot(gg_rfsrc(rfsrc_Boston), alpha = 0.5) +
  coord_cartesian(ylim = c(5, 49))


## ----vimp-plot, fig.cap="VIMP ranking. Longer blue bars indicate more important variables."----
plot(gg_vimp(rfsrc_Boston), lbls = st_labs)


## ----varsel-------------------------------------------------------------------
md_Boston <- max.subtree(rfsrc_Boston) # nolint: object_name_linter


## ----select-vars--------------------------------------------------------------
xvar <- md_Boston$topvars


## ----vardep-panel, fig.cap="Variable dependence for top predictors (minimal depth rank order)."----
gg_v <- gg_variable(rfsrc_Boston)

plot(gg_v, xvar = xvar, panel = TRUE, alpha = 0.5) +
  labs(y = st_labs["medv"], x = "")


## ----vardep-chas, fig.height=4, fig.cap="Variable dependence for Charles River (categorical)."----
plot(gg_v, xvar = "chas", alpha = 0.4) +
  labs(y = st_labs["medv"])


## ----partial-dep, fig.cap="Partial dependence for top predictors."------------
pd <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = xvar)

# Quick S3 plot — works out of the box for the standard regression case
plot(pd)


## ----partial-dep-custom, fig.cap="Partial dependence (custom styling)."-------
ggplot(pd$continuous, aes(x = x, y = yhat)) +
  geom_line(color = "steelblue", linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = st_labs["medv"], x = "") +
  theme_bw()


## ----coplot-chas, fig.height=4, fig.cap="Variable dependence on lstat, conditional on Charles River."----
gg_v$chas_label <- ifelse(gg_v$chas, "Borders Charles River",
                          "Does not border")

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  facet_wrap(~chas_label)


## ----coplot-rm, fig.cap="Predicted medv vs. lstat, conditional on rooms-per-dwelling groups."----
rm_pts <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 6, intervals = TRUE)
gg_v$rm_grp <- cut(rfsrc_Boston$xvar$rm, breaks = rm_pts)
levels(gg_v$rm_grp) <- paste("rm in", levels(gg_v$rm_grp))

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~rm_grp)


## ----coplot-lstat, fig.cap="Predicted medv vs. rooms, conditional on lower-status groups."----
lstat_pts <- quantile_pts(rfsrc_Boston$xvar$lstat, groups = 6,
                          intervals = TRUE)
gg_v$lstat_grp <- cut(rfsrc_Boston$xvar$lstat, breaks = lstat_pts)
levels(gg_v$lstat_grp) <- paste("lstat in", levels(gg_v$lstat_grp))

plot(gg_v, xvar = "rm", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["rm"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~lstat_grp)


## ----surface-data, cache=TRUE-------------------------------------------------
rm_grid <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 25)

surface_list <- lapply(rm_grid, function(rm_val) {
  newx <- rfsrc_Boston$xvar
  newx$rm <- rm_val
  pd_rm <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = "lstat", newx = newx)
  df <- pd_rm$continuous
  df$rm <- rm_val
  df
})

surface_df <- bind_rows(surface_list)


## ----plotly-surface, fig.cap="Interactive partial dependence surface: median home value as a function of lstat and rm."----
if (requireNamespace("plotly", quietly = TRUE)) {
  library(plotly)

  surface_wide <- surface_df |>
    select(lstat = x, rm, medv = yhat) |>
    arrange(rm, lstat)

  lstat_vals <- sort(unique(surface_wide$lstat))
  rm_vals    <- sort(unique(surface_wide$rm))
  z_matrix   <- matrix(surface_wide$medv,
                        nrow = length(rm_vals),
                        ncol = length(lstat_vals),
                        byrow = TRUE)

  plot_ly(x = lstat_vals, y = rm_vals, z = z_matrix) |>
    add_surface(colorscale = "Viridis", showscale = TRUE) |>
    layout(
      scene = list(
        xaxis = list(title = "Lower Status (%)"),
        yaxis = list(title = "Rooms per Dwelling"),
        zaxis = list(title = "Median Value ($1000s)")
      )
    )
} else {
  message("Install the plotly package for interactive 3D surfaces.")
  ggplot(surface_df, aes(x = x, y = rm, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Median Value") +
    labs(x = "Lower Status (%)", y = "Rooms per Dwelling") +
    theme_bw()
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_vimp}} object, extracted variable importance of a
#' \code{\link[randomForestSRC]{rfsrc}} object
#'
#' Draws a horizontal bar chart of the VIMP scores extracted by
#' \code{\link{gg_vimp}}.  Each bar represents one predictor; bar length is
#' proportional to its permutation VIMP -- the average rise in OOB prediction
#' error when that predictor's OOB values are randomly shuffled.  Predictors
#' are sorted in descending order of importance so the most influential
#' variables appear at the top.
#'
#' Bars are coloured by the \code{positive} flag: a bar at or below zero
#' (non-positive VIMP) is colour-coded differently to flag predictors that
#' \emph{hurt} OOB accuracy when their signal is removed -- usually a sign of
#' collinearity or a very noisy variable.  In a well-behaved forest most bars
#' are positive; the colour distinction matters when a handful are not.
#'
#' @param x \code{\link{gg_vimp}} object created from a
#' \code{\link[randomForestSRC]{rfsrc}} object
#' @param relative should we plot vimp or relative vimp. Defaults to vimp.
#' @param lbls A vector of alternative variable labels. Item names should be
#' the same as the variable names.
#' @param ... optional arguments passed to gg_vimp if necessary
#'
#' @return \code{ggplot} object
#'
#' @seealso \code{\link{gg_vimp}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
#' R, Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)
#' gg_dta <- gg_vimp(rfsrc_iris)
#' plot(gg_dta)
#'
#' ## ------------------------------------------------------------
#' ## regression example
#' ## ------------------------------------------------------------
#' ## -------- air quality data
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., airquality)
#' gg_dta <- gg_vimp(rfsrc_airq)
#' plot(gg_dta)
#'
#'
#' @export
plot.gg_vimp <- function(x, relative, lbls, ...) {
  gg_dta <- x

  # Accept raw rfsrc / randomForest objects and compute VIMP on the fly
  if (!inherits(gg_dta, "gg_vimp")) {
    gg_dta <- gg_vimp(gg_dta, ...)
  }

  # Capture extra args so we can inspect nvar.
  arg_set <- list(...)

  # Optionally restrict to the top-nvar most important variables (gg_vimp
  # already sorts by descending VIMP, so we just trim the tail).
  nvar <- nrow(gg_dta)
  if (!is.null(arg_set$nvar)) {
    if (is.numeric(arg_set$nvar) && arg_set$nvar > 1) {
      if (arg_set$nvar < nrow(gg_dta)) {
        nvar <- arg_set$nvar
        gg_dta <- gg_dta[seq_len(nvar), ]
      }
    }
  }

  gg_plt <- ggplot2::ggplot(gg_dta)

  # Use "vimp" as the bar-height column when it exists; fall back to the
  # first column name for objects that store a renamed importance measure.
  msr <- "vimp"
  if (!msr %in% colnames(gg_dta)) {
    msr <- colnames(gg_dta)[1]
  }

  # Always map both `fill` and `color` to `positive` -- this gives filled bars
  # (rather than hollow outlines) and ensures the function-level
  # `labs(fill = ..., color = ...)` below applies cleanly. When `positive` has
  # only one level (all VIMPs positive, the common case for well-behaved
  # forests), the bars simply render in a single colour and ggplot collapses
  # the fill+color legend into a one-row legend; when both signs are present,
  # the two-row legend distinguishes positive from negative VIMP.
  #
  # Both aesthetics share the same legend title ("VIMP > 0") so ggplot
  # collapses what would otherwise be two legends -- one for fill and one
  # titled with the column name "positive" -- into a single merged legend.
  legend_title <- "VIMP > 0"
  gg_plt <- gg_plt +
    ggplot2::geom_bar(
      ggplot2::aes(
        y    = .data[[msr]],
        x    = .data$vars,
        fill = .data$positive,
        color = .data$positive
      ),
      stat = "identity",
      width = .5,
    )
  # Set both legends' titles to the same string so ggplot merges them.
  # Users can override with their own labs() call after the fact.
  gg_plt <- gg_plt +
    ggplot2::labs(x = "", y = msr,
                  fill = legend_title, color = legend_title)

  if (!missing(lbls)) {
    # Map internal variable names to human-readable labels.  lbls should be a
    # named character vector; any unmatched variables keep their original name.
    if (length(lbls) >= length(gg_dta$vars)) {
      st_lbls <- lbls[as.character(gg_dta$vars)]
      names(st_lbls) <- as.character(gg_dta$vars)
      # Fall back to the raw variable name when no label was supplied
      st_lbls[which(is.na(st_lbls))] <-
        names(st_lbls[which(is.na(st_lbls))])

      gg_plt <- gg_plt +
        ggplot2::scale_x_discrete(labels = st_lbls)
    }
  }

  # Flip coordinates so variable names appear on the y-axis (horizontal bars
  # are easier to read when there are many variables).  If gg_vimp contains
  # a "set" column (comparison VIMP across two forests), facet by set.
  if (is.null(gg_dta$set) || length(unique(gg_dta$set)) < 2) {
    gg_plt <- gg_plt +
      ggplot2::coord_flip()
  } else {
    gg_plt <- gg_plt +
      ggplot2::coord_flip() +
      ggplot2::facet_grid(~set)
  }

  return(gg_plt)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Plot a \code{\link{gg_survival}} object.
#'
#' Draws a Kaplan-Meier (or Nelson-Aalen) survival curve from a
#' \code{\link{gg_survival}} object.  You can overlay a confidence envelope
#' around the curve using \code{error}: \code{"shade"} fills the area between
#' the pointwise confidence limits, \code{"lines"} draws them as dashed step
#' functions, and \code{"bars"} shows them as error bars.  When \code{gg_survival}
#' was called with a \code{by} argument, each group gets its own step function
#' and the \code{label} argument renames the legend.
#'
#' The \code{type} argument selects which quantity to plot on the y-axis --
#' survival probability (\code{"surv"}) is the default, but cumulative hazard,
#' density, and several transformed scales are available for the cases where a
#' linear scale reveals more about the tails.
#'
#' @param x \code{\link{gg_survival}} or a survival \code{\link{gg_rfsrc}}
#' object created from a \code{\link[randomForestSRC]{rfsrc}} object
#'
#' @param error "shade", "bars", "lines" or "none"
#' @param type "surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"
#' @param label Modify the legend label when gg_survival has stratified samples
#' @param ... Additional arguments forwarded to \code{geom_step()}. When
#'   \code{alpha} is supplied it is passed to the step geom and a halved
#'   value is used for the ribbon overlay.
#'
#' @return A \code{ggplot} object. The y-axis shows the chosen \code{type}
#'   (e.g. survival probability for \code{"surv"}) and the x-axis shows time.
#'   Confidence shading, bars, or lines are added when the input object carries
#'   confidence-interval columns.
#'
#' @seealso \code{\link{gg_survival}}, \code{\link{kaplan}},
#'   \code{\link{nelson}}, \code{\link{gg_rfsrc}}
#'
#' @examples
#' ## -------- pbc data
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' # This is the same as kaplan
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' # Stratified on treatment variable.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#' plot(gg_dta, label = "treatment")
#'
#' # ...with smaller confidence limits.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment", conf.int = .68
#' )
#'
#' plot(gg_dta, error = "lines")
#' plot(gg_dta, label = "treatment", error = "lines")
#'
#' # ...with smaller confidence limits.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "sex", conf.int = .68
#' )
#'
#' plot(gg_dta, error = "lines")
#' plot(gg_dta, label = "sex", error = "lines")
#'
#' @export
### Survival plots
plot.gg_survival <- function(x,
                             type = c("surv",
                                      "cum_haz",
                                      "hazard",
                                      "density",
                                      "mid_int",
                                      "life",
                                      "proplife"),
                             error = c("shade", "bars", "lines", "none"),
                             label = NULL,
                             ...) {
  # Auto-coerce raw rfsrc objects so plot(rfsrc_obj) works directly.
  # gg_survival.rfsrc extracts yvar and computes Kaplan-Meier estimates.
  gg_dta <- if (inherits(x, "rfsrc")) gg_survival(x) else x

  error <- match.arg(error)
  type  <- match.arg(type)

  # Split ... so alpha applies to step geoms while ribbons use a softer value.
  spl          <- .gg_split_alpha(list(...))
  ribbon_alpha <- spl$ribbon_alpha

  # Now order matters, so we want to place the forest predictions on the bottom
  # Create the figure skeleton,
  if (is.null(gg_dta$groups)) {
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_step(ggplot2::aes(x = .data$time, y = .data[[type]]), ...)
  } else {
    gg_dta$groups <- factor(gg_dta$groups)
    gg_plt <- ggplot2::ggplot(gg_dta) +
      ggplot2::geom_step(ggplot2::aes(
        x = .data$time,
        y = .data[[type]],
        color = .data$groups
      ),
      ...)
    if (!is.null(label)) {
      gg_plt <- gg_plt +
        ggplot2::labs(color = label, fill = label)
    }
  }
  # Do we want to show confidence limits?
  if (type == "surv") {
    if (is.null(gg_dta$groups)) {
      gg_plt <- switch(
        error,
        # Shading the standard errors
        shade = gg_plt +
          ggplot2::geom_ribbon(
            ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower
            ),
            alpha = ribbon_alpha,
            fill  = .gg_ribbon_fill
          ),
        # Or showing error bars
        bars = {
          # Need to figure out how to remove some of these points when
          # requesting error bars, or this will get really messy.
          gg_plt +
            ggplot2::geom_errorbar(ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower
            ))
        },
        lines = gg_plt +
          ggplot2::geom_step(ggplot2::aes(
            x = .data$time, y = .data$upper
          ), linetype = 2) +
          ggplot2::geom_step(ggplot2::aes(
            x = .data$time, y = .data$lower
          ), linetype = 2),
        none = gg_plt
      )
    } else {
      gg_plt <- switch(
        error,
        # Shading the standard errors
        shade = gg_plt +
          ggplot2::geom_ribbon(
            ggplot2::aes(
              x = .data$time,
              ymax = .data$upper,
              ymin = .data$lower,
              fill = .data$groups,
              color = .data$groups
            ),
            alpha = ribbon_alpha
          ),
        # Or showing error bars
        bars = {
          # Need to figure out how to remove some of these points when
          # requesting error bars, or this will get really messy.
          gg_plt +
            ggplot2::geom_errorbar(
              ggplot2::aes(
                x = .data$time,
                ymax = .data$upper,
                ymin = .data$lower,
                color = .data$groups
              )
            )
        },
        lines = gg_plt +
          ggplot2::geom_step(
            ggplot2::aes(
              x = .data$time,
              y = .data$upper,
              color = .data$groups
            ),
            linetype = 2
          ) +
          ggplot2::geom_step(
            ggplot2::aes(
              x = .data$time,
              y = .data$lower,
              color = .data$groups
            ),
            linetype = 2
          ),
        none = gg_plt
      )
    }
  }
  return(gg_plt)
}

####**********************************************************************
####  Internal: shared styling constants and helpers for ribbon overlays.
####
####  Every plot.gg_* method that draws a ribbon (CI band on KM/NA curves,
####  bootstrap CI band on gg_rfsrc survival curves, per-subject envelope
####  on gg_brier) uses these constants so the family renders uniformly.
####  This is styling only. The underlying probability/coverage of each
####  ribbon is computed elsewhere and is intentionally heterogeneous
####  (95% inferential CIs vs. 15-85 percentile descriptive envelopes).
####**********************************************************************

# Canonical ribbon transparency. Low enough that overlapping bands stay
# readable when several models are layered, high enough to read on white.
.gg_ribbon_alpha <- 0.2

# Default fill for single-series ribbons (no group aesthetic). Group-coloured
# ribbons override via aes(fill = group) and ignore this constant.
.gg_ribbon_fill <- "steelblue"

# Split a captured list(...) into ribbon and step components.
#
# When the user passes alpha = x to a plot.gg_* method, that alpha is
# meant for the main line/step geom.  The ribbon should use a softer
# value (half of the user alpha, or the package default if none supplied).
# This helper centralises that logic and prevents the duplicate-formal
# error that arises when alpha is both in ... and set explicitly on geom_ribbon.
#
# Returns a list with:
#   $ribbon_alpha  : numeric alpha to pass explicitly to geom_ribbon()
#   $step_dots     : original dots (forwarded to geom_step / geom_line)
#   $ribbon_dots   : dots with alpha removed (ribbon alpha is explicit)
.gg_split_alpha <- function(dots) {
  user_alpha <- dots[["alpha"]]
  list(
    ribbon_alpha = if (is.null(user_alpha)) .gg_ribbon_alpha
                   else user_alpha * 0.5,
    step_dots    = dots,
    ribbon_dots  = dots[setdiff(names(dots), "alpha")]
  )
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Random forest error trajectory data object
#'
#' Extract the cumulative out-of-bag (OOB) or in-bag training error rate from
#' \code{randomForestSRC} and \code{randomForest} fits as a function of the
#' number of grown trees.
#'
#' @details For \code{randomForestSRC} objects the function reshapes the
#' \code{\link[randomForestSRC]{rfsrc}$err.rate} matrix and annotates it with
#' the tree index required by \code{\link{plot.gg_error}}. When supplied a
#' \code{\link[randomForest]{randomForest}} object, the method inspects either
#' the \code{$mse} or \code{$err.rate} component and, when
#' \code{training = TRUE} is requested, reconstructs the original training set
#' via the model call to compute an in-bag error curve using per-tree
#' predictions. Training curves are only available when the forest was stored
#' (\code{keep.forest = TRUE}) and the original data can be recovered.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object.
#' @param ... Optional arguments passed to the methods. Set
#'   \code{training = TRUE} to append the in-bag error trajectory when
#'   supported.
#'
#' @return A \code{gg_error} \code{data.frame} containing at least the
#'   cumulative OOB error columns and an \code{ntree} counter. When
#'   \code{training = TRUE} is honored an additional \code{train} column is
#'   included.
#'
#' @seealso \code{\link{plot.gg_error}}, \code{\link{gg_vimp}},
#'   \code{\link{gg_variable}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link[randomForest]{randomForest}},
#'   \code{\link[randomForestSRC]{plot.rfsrc}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
#' Rnews, 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @aliases gg_error gg_error.rfsrc gg_error.randomForest
#' @aliases gg_error.randomForest.formula
#'
#' @examples
#' ## Examples from RFSRC package...
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## ------------- iris data
#' ## You can build a randomForest
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, tree.err = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_iris)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## RandomForest example
#' rf_iris <- randomForest::randomForest(Species ~ .,
#'   data = iris,
#'   tree.err = TRUE,
#' )
#' gg_dta <- gg_error(rf_iris)
#' plot(gg_dta)
#'
#' gg_dta <- gg_error(rf_iris, training = TRUE)
#' plot(gg_dta)
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#'
#' ## ------------- airq data
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ .,
#'   data = airquality,
#'   na.action = "na.impute", tree.err = TRUE,
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_airq)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- Boston data
#' data(Boston, package = "MASS")
#' Boston$chas <- as.logical(Boston$chas)
#' rfsrc_boston <- randomForestSRC::rfsrc(medv ~ .,
#'   data = Boston,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_boston)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- mtcars data
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, tree.err = TRUE)
#'

#' # Get a data.frame containing error rates
#' gg_dta<- gg_error(rfsrc_mtcars)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## ------------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        tree.err = TRUE)
#'
#' gg_dta <- gg_error(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## ------------- pbc data
#' # Load a cached randomForestSRC object
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC",)
#' # For whatever reason, the age variable is in days... makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'  if (!is.factor(pbc[, ind])) {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  } else {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  }
#'  if (!is.logical(pbc[, ind]) &
#'      length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'    pbc[, ind] <- factor(pbc[, ind])
#'  }
#' }
#' #Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' #========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'  dta_train,
#'  nsplit = 10,
#'  na.action = "na.impute",
#'  tree.err = TRUE,
#'  forest = TRUE,
#'  importance = TRUE,
#'  save.memory = TRUE
#' )
#'
#'
#' gg_dta <- gg_error(rfsrc_pbc)
#' plot(gg_dta)
#'
#' @importFrom stats as.formula model.frame model.response na.omit predict qnorm
#'
#' @export
gg_error <- function(object, ...) {
  UseMethod("gg_error", object)
}
#' @export
gg_error.rfsrc <- function(object, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "rfsrc")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForestSRC package."
      )
    )
  }
  # The forest must have been grown with tree.err = TRUE so that per-tree
  # OOB error rates are recorded in $err.rate.
  if (is.null(object$err.rate)) {
    stop("Performance values are not available for this forest.")
  }

  # Convert the err.rate matrix (ntree × n_outcomes) to a data frame.
  gg_dta <- data.frame(object$err.rate)

  # rfsrc wraps single-column matrices with a column name derived from the
  # object name; rename it to the neutral label "error" for downstream use.
  if ("object.err.rate" %in% colnames(gg_dta)) {
    colnames(gg_dta)[which(colnames(gg_dta) == "object.err.rate")] <-
      "error"
  }

  # Add a sequential tree counter required by the x-axis of plot.gg_error.
  gg_dta$ntree <- seq_len(dim(gg_dta)[1])

  # Optional in-bag training error: re-predict on the full training set using
  # the stored forest and record the resulting per-tree error trajectory.
  arg_list <- list(...)
  training <- isTRUE(arg_list$training)
  if (training) {
    trn <- data.frame(cbind(object$xvar, object$yvar))
    colnames(trn) <- c(object$xvar.names, object$yvar.names)
    gg_prd <- predict(
      object,
      newdata = trn,
      importance = "none",
      membership = FALSE
    )
    gg_dta$train <- gg_prd$err.rate
  }
  gg_dta <- na.omit(gg_dta)
  class(gg_dta) <- c("gg_error", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_error.randomForest <- function(object, ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown",
        "with the randomForest package."
      )
    )
  }

  if (!is.null(object$mse)) {
    # Regression forests store the cumulative OOB mean squared error in $mse.
    gg_dta <- data.frame(object$mse)

    # Normalise the auto-generated column name to "error".
    if ("object.mse" %in% colnames(gg_dta)) {
      colnames(gg_dta)[which(colnames(gg_dta) == "object.mse")] <-
        "error"
    }

    gg_dta$ntree <- seq_len(nrow(gg_dta))

    arg_list <- list(...)
    training <- isTRUE(arg_list$training)

    # Optionally compute and append the per-tree in-bag training error curve.
    if (training) {
      train_curve <- .rf_training_curve(object)
      if (!is.null(train_curve)) {
        gg_dta$train <- train_curve
      }
    }
  } else if (!is.null(object$err.rate)) {
    # Classification forests store the cumulative OOB error matrix in
    # $err.rate (rows = trees, columns = overall + per-class error rates).
    gg_dta <- data.frame(object$err.rate)

    gg_dta$ntree <- seq_len(nrow(gg_dta))

    arg_list <- list(...)
    training <- isTRUE(arg_list$training)
    if (training) {
      train_curve <- .rf_training_curve(object)
      if (!is.null(train_curve)) {
        gg_dta$train <- train_curve
      }
    }
  } else {
    stop("Performance values are not available for this forest.")
  }
  class(gg_dta) <- c("gg_error", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_error.randomForest.formula <- gg_error.randomForest

.rf_training_curve <- function(object) {
  if (is.null(object$forest)) {
    warning(
      "Training error curve is unavailable because the forest was not saved. ",
      "Refit with keep.forest = TRUE to enable training=TRUE."
    )
    return(NULL)
  }
  training_info <- .rf_recover_model_frame(object)
  if (is.null(training_info)) {
    warning(
      "Unable to reconstruct the training data for this randomForest object;",
      " training=TRUE is ignored."
    )
    return(NULL)
  }

  training_frame <- training_info$model_frame
  response <- stats::model.response(training_frame)
  resp_name <- training_info$response_name
  predictors <- training_frame
  if (!is.null(resp_name) && resp_name %in% colnames(predictors)) {
    predictors[[resp_name]] <- NULL
  }
  special_cols <- grep("^\\(", colnames(predictors), value = TRUE)
  if (length(special_cols) > 0) {
    predictors[special_cols] <- NULL
  }
  predictors <- as.data.frame(predictors)
  if (ncol(predictors) == 0) {
    warning("No predictor columns available to compute training curve.")
    return(NULL)
  }

  pred_all <- predict(object,
    newdata = predictors,
    predict.all = TRUE
  )

  if (is.null(pred_all$individual)) {
    warning("Unable to extract per-tree predictions; training=TRUE ignored.")
    return(NULL)
  }

  individual <- pred_all$individual
  if (object$type == "regression") {
    return(.rf_training_curve_regression(individual, response))
  }
  if (object$type == "classification") {
    return(.rf_training_curve_classification(individual, response, object$classes))
  }
  warning("Training error curves are not supported for this forest type.")
  NULL
}

.rf_training_curve_regression <- function(individual, response) {
  cum_preds <- t(apply(individual, 1, cumsum))
  ntree <- ncol(individual)
  pred_by_tree <- sweep(cum_preds, 2, seq_len(ntree), "/")
  mse <- colMeans((pred_by_tree - response)^2)
  as.numeric(mse)
}

.rf_training_curve_classification <- function(individual, response, classes) {
  response <- factor(response, levels = classes)
  ntree <- ncol(individual)
  nobs <- nrow(individual)
  votes <- matrix(0, nrow = nobs, ncol = length(classes))
  colnames(votes) <- classes
  err <- numeric(ntree)

  for (tree in seq_len(ntree)) {
    preds <- factor(individual[, tree], levels = classes)
    pred_index <- as.integer(preds)
    valid <- !is.na(pred_index)
    idx <- cbind(seq_len(nobs)[valid], pred_index[valid])
    if (nrow(idx) > 0) {
      votes[idx] <- votes[idx] + 1
    }
    agg <- factor(classes[max.col(votes, ties.method = "first")],
      levels = classes
    )
    err[tree] <- mean(agg != response)
  }
  err
}

.rf_recover_model_frame <- function(object) {
  if (is.null(object$call$formula)) {
    return(NULL)
  }
  formula <- stats::as.formula(object$call$formula)
  data_env <- environment(formula)
  if (is.null(data_env)) {
    data_env <- parent.frame()
  }
  data_expr <- object$call$data
  if (is.null(data_expr)) {
    return(NULL)
  }
  mf_list <- c(
    list(quote(stats::model.frame)),
    list(formula = formula, data = data_expr)
  )
  optional_args <- c("subset", "weights", "na.action", "offset")
  for (arg in optional_args) {
    arg_value <- object$call[[arg]]
    if (!is.null(arg_value)) {
      mf_list[[arg]] <- arg_value
    }
  }
  mf_call <- as.call(mf_list)
  env_candidates <- c(list(data_env), rev(sys.frames()), list(.GlobalEnv))
  mf <- NULL
  for (env in env_candidates) {
    mf <- tryCatch(
      eval(mf_call, envir = env),
      error = function(e) NULL
    )
    if (!is.null(mf)) {
      break
    }
  }
  if (is.null(mf)) {
    return(NULL)
  }
  terms_obj <- attr(mf, "terms")
  resp_idx <- attr(terms_obj, "response")
  resp_name <- NULL
  if (!is.null(resp_idx) && resp_idx > 0) {
    resp_name <- colnames(mf)[resp_idx]
  }
  list(
    model_frame = mf,
    response_name = resp_name
  )
}

# Precompute the varPro objects used by the varpro vignette.
#
# Two reasons this file exists:
#
#  1. Speed. The three gg_partial_varpro() calls (~31s) and the beta.varpro()
#     fits dominate the vignette rebuild time (CRAN flagged the overall check
#     time).
#  2. Safety. Every varPro grow (varpro(), uvarpro(), isopro(), ivarpro(),
#     beta.varpro()) reaches randomForestSRC's compiled rule-grow path, which
#     trips a gcc-UBSAN "0-length array" report (rfsrcGrow, entry.c:184). By
#     loading these objects from disk, the vignette performs NO live varPro
#     grow during R CMD check, so it cannot surface that upstream sanitizer
#     issue even if a CRAN flavour builds the vignette under UBSAN.
#
# Run from the package root after changing any of the varpro vignette fits:
#   Rscript vignettes/precompute_varpro.R
#
# Produces vignettes/varpro_precomputed.rds. The vignette falls back to live
# computation for any object missing from that file, so the result is
# reproducible either way.

suppressMessages({
  library(varPro)
  library(survival)
})
# Mirror the vignette's loader: prefer the installed package, fall back to
# pkgload::load_all() so this runs in a fresh clone before installation.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  suppressMessages(library(ggRandomForests))
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to run this script.")
}
options(mc.cores = 1, rf.cores = 1)

# --- Regression: Boston housing ------------------------------------------
data("Boston", package = "MASS")
boston_x <- Boston[, setdiff(names(Boston), "medv")]
set.seed(20260527L)
v_boston   <- varPro::varpro(medv ~ ., data = Boston, ntree = 50)
pd_boston  <- gg_partial_varpro(object = v_boston)
b_boston   <- varPro::beta.varpro(v_boston)
iv_boston  <- varPro::ivarpro(v_boston)
set.seed(20260527L)
u_boston   <- varPro::uvarpro(boston_x, ntree = 50)
set.seed(20260527L)
iso_boston <- varPro::isopro(data = boston_x, method = "rnd",
                             sampsize = 256, ntree = 50)

# --- Classification: iris ------------------------------------------------
iris_binary <- iris[iris$Species != "setosa", ]
iris_binary$Species <- droplevels(iris_binary$Species)
set.seed(20260527L)
v_iris_binary <- varPro::varpro(Species ~ ., data = iris_binary, ntree = 50)
b_iris_binary <- varPro::beta.varpro(v_iris_binary)
set.seed(20260527L)
v_iris_multi  <- varPro::varpro(Species ~ ., data = iris, ntree = 50)
pd_iris_multi <- gg_partial_varpro(object = v_iris_multi)
b_iris_multi  <- varPro::beta.varpro(v_iris_multi)

# --- Survival: PBC (C-path partial dependence) ---------------------------
data(pbc, package = "randomForestSRC")
pbc_small <- na.omit(pbc[, c("days", "status", "age", "albumin", "bili",
                             "edema", "platelet")])
set.seed(20260527L)
v_pbc   <- varPro::varpro(Surv(days, status) ~ ., data = pbc_small, ntree = 50)
pd_pbc  <- gg_partial_varpro(object = v_pbc)
set.seed(20260527L)
iso_pbc <- varPro::isopro(data = pbc_small[, c("age", "albumin", "bili",
                                               "platelet")],
                          method = "rnd", sampsize = 256, ntree = 50)

# --- Trim to keep the shipped .rds (and the source tarball) small --------
# The gg_* calls this vignette makes (on its cached path) read only
# importance/rule summaries, not the embedded forests. The survival C-path
# gg_partial_varpro() and gg_isopro(newdata=) DO use the forest, but the
# vignette never invokes those on a stripped object (pd_pbc is cached and
# gg_isopro() is training-path only). Dropping those heavy
# slots takes the file from ~1.6 MB to ~0.4 MB (validated: every vignette
# wrapper call returns output identical to the un-stripped object). Two
# exceptions keep their forest: v_boston is printed in the vignette
# (print.varpro reads $rf), and u_boston feeds gg_udependent(), which uses it.
.strip_varpro <- function(v) {              # $rf: unused on the cached path
  v$rf <- NULL
  v
}
.strip_isopro <- function(o) {              # training-path gg_isopro reads only $ntree
  o$isoforest <- list(ntree = o$isoforest$ntree)
  o
}
.strip_ivarpro <- function(iv) {            # drop redundant model/path attrs
  for (a in c("ivarpro.path", "model", "data")) attr(iv, a) <- NULL
  iv
}
b_boston      <- .strip_varpro(b_boston)
v_pbc         <- .strip_varpro(v_pbc)
v_iris_binary <- .strip_varpro(v_iris_binary)
v_iris_multi  <- .strip_varpro(v_iris_multi)
iso_boston    <- .strip_isopro(iso_boston)
iso_pbc       <- .strip_isopro(iso_pbc)
iv_boston     <- .strip_ivarpro(iv_boston)

saveRDS(
  list(
    # Boston (regr)
    v_boston      = v_boston,
    pd_boston     = pd_boston,
    b_boston      = b_boston,
    iv_boston     = iv_boston,
    u_boston      = u_boston,
    iso_boston    = iso_boston,
    # iris (class)
    v_iris_binary = v_iris_binary,
    b_iris_binary = b_iris_binary,
    v_iris_multi  = v_iris_multi,
    pd_iris_multi = pd_iris_multi,
    b_iris_multi  = b_iris_multi,
    # PBC (surv)
    v_pbc         = v_pbc,
    pd_pbc        = pd_pbc,
    iso_pbc       = iso_pbc
  ),
  file = "vignettes/varpro_precomputed.rds",
  version = 2,
  compress = "xz"
)

cat("Wrote vignettes/varpro_precomputed.rds (",
    round(file.size("vignettes/varpro_precomputed.rds") / 1024, 1), "KB )\n")

##=============================================================================
#' Partial dependence data from a varPro model
#'
#' \code{varPro::partialpro} returns one list, with continuous and
#' categorical predictors mixed together. This function splits that list into
#' two tidy data frames, one for each kind, and resolves the y-axis label the
#' plot method will use.
#'
#' @section What partialpro is doing:
#' A partial dependence curve answers the question, "if I hold a single
#' variable at a grid of values and average out everything else, how does
#' the model's prediction move?" That is the same question \code{rfsrc}
#' partial dependence answers. What \code{varPro::partialpro} adds is two
#' wrinkles that are worth understanding before you read the curves.
#'
#' First, \code{partialpro} filters the partial grid through an isolation
#' forest (Unlimited Virtual Twins, or UVT) so that unlikely combinations
#' of the focal variable with the rest of the data are downweighted. The
#' \code{rfsrc} version, by contrast, averages over the full marginal grid
#' regardless of plausibility. So when a covariate is highly correlated
#' with others, the two methods can disagree, and \code{partialpro}'s
#' curve is the one restricted to the data manifold.
#'
#' Second, \code{partialpro} fits a local polynomial model to the
#' predicted values rather than just plotting their mean. That gives
#' three parallel curves per variable, stored as \code{yhat.par},
#' \code{yhat.nonpar}, and \code{yhat.causal}, which the plot method
#' overlays so you can see whether a smooth parametric story and the
#' raw forest predictions are telling you the same thing.
#'
#' Interpretation of the y-axis depends on the outcome (per
#' \code{varPro::partialpro}): response scale for regression, log-odds of
#' the target class for classification, and either ensemble mortality
#' (default) or RMST (if the original \code{varpro} call set
#' \code{rmst}) for survival.
#'
#' @section What's in the output:
#' We split \code{partialpro}'s mixed list into two tidy data frames so
#' the plot method does not have to. A variable with more than
#' \code{cat_limit} distinct grid points goes into \code{$continuous},
#' one row per grid point with the column means of \code{yhat.par},
#' \code{yhat.nonpar}, and \code{yhat.causal} stored as
#' \code{parametric}, \code{nonparametric}, and \code{causal}. A
#' variable at or below \code{cat_limit} goes into \code{$categorical},
#' one row per observation per category level, carrying the same three
#' columns unaveraged so the plot method can draw boxplots. Path C
#' (\code{scale \%in\% c("surv","chf")}) takes a different route: we
#' hand the underlying \code{rfsrc} forest to \code{gg_partial_rfsrc} so
#' you get a survival-probability or cumulative-hazard curve on the
#' usual rfsrc scale instead.
#'
#' @section What you use this for:
#' \itemize{
#'   \item read the marginal shape of a relationship the varpro model
#'     found important: monotone, threshold, U-shape, flat;
#'   \item compare the three partialpro estimators on the same variable
#'     and flag the ones where parametric and nonparametric disagree,
#'     those are the candidates for closer inspection;
#'   \item report a survival partial dependence on the probability or
#'     cumulative-hazard scale (\code{scale = "surv"} or \code{"chf"})
#'     rather than the unbounded mortality scale.
#' }
#' A varpro partial dependence curve is a description of the model, not
#' a causal effect. The \code{causal} column is varpro's local
#' estimator, not a structural causal claim about the data-generating
#' process.
#'
#' @param part_dta Partial plot data from \code{varPro::partialpro}.  Each
#'   element must contain \code{xvirtual}, \code{xorg}, \code{yhat.par},
#'   \code{yhat.nonpar}, and \code{yhat.causal}.  Supply at least one of
#'   \code{part_dta} or \code{object}.
#' @param object A fitted \code{varpro} object, the forest the partial data
#'   came from.  When supplied it provides the provenance metadata, and when
#'   \code{part_dta} is \code{NULL} it is passed to
#'   \code{varPro::partialpro(object)} for you.  Required when
#'   \code{scale \%in\% c("surv","chf")}.
#' @param scale Character; sets the y-axis label and, for survival forests,
#'   the output type.  One of \code{"auto"} (default), \code{"mortality"},
#'   \code{"rmst"}, \code{"surv"}, or \code{"chf"}.
#' @param time Numeric; the evaluation time point.  Required when
#'   \code{scale = "rmst"} (the RMST horizon \eqn{\tau}).  Optional when
#'   \code{scale \%in\% c("surv","chf")}: if supplied it is snapped to the
#'   nearest value in \code{object\$rf\$time.interest} and used for both
#'   computation and axis labeling; if \code{NULL}, three quartile time
#'   points from \code{time.interest} are used (see
#'   \code{\link{gg_partial_rfsrc}}).
#' @param nvars Integer; how many variables (list elements) to process.
#'   Defaults to every variable in \code{part_dta}.
#' @param cat_limit Integer; a variable with
#'   \code{length(xvirtual) <= cat_limit} is treated as categorical.
#'   Default \code{10}.
#' @param model Character; a label tacked onto every row, handy when you are
#'   combining results from several models in one figure.
#'
#' @details
#' **Scale detection:** with \code{scale = "auto"} and an \code{object} in
#' hand, the scale resolves to \code{"mortality"} for a survival forest and
#' \code{"generic"} for a regression or classification forest.  The RMST
#' horizon \eqn{\tau} is \emph{not} stored in the \code{varpro} object
#' (varPro 3.1.0), so for RMST-labeled output you have to pass
#' \code{scale = "rmst", time = tau} yourself.
#'
#' **Ensemble mortality (scale = "mortality"):** here the y-axis is
#' \emph{ensemble mortality}, the expected number of events a subject would
#' see if they were exposed to the study-average cumulative hazard.  It is
#' the same quantity as the \code{rfsrc} \code{predicted} value for survival
#' forests (Ishwaran, Kogalur, Blackstone & Lauer, 2008
#' <doi:10.1214/08-AOAS169>).  This is an \strong{unbounded relative-risk
#' score}, \emph{not} a survival probability and not \eqn{1 - S(t)}; don't
#' read it as one.  If you want output on the probability scale, refit with
#' \code{varpro(..., rmst = tau)} and use \code{scale = "rmst"}.
#'
#' @return A named list of class \code{"gg_partial_varpro"} with elements:
#' \describe{
#'   \item{continuous}{data.frame with columns \code{variable},
#'     \code{parametric}, \code{nonparametric}, \code{causal}, \code{name}
#'     (and optionally \code{model}).}
#'   \item{categorical}{data.frame with the same columns, one row per
#'     observation per category level.}
#' }
#' A \code{"provenance"} attribute carries \code{source}, \code{family},
#' \code{ntree}, \code{n}, \code{scale}, \code{rmst_tau},
#' \code{xvar.names}, and \code{path}.
#'
#' @references
#' Ishwaran H, Kogalur UB, Blackstone EH, Lauer MS (2008).
#' Random survival forests. \emph{The Annals of Applied Statistics},
#' \bold{2}(3), 841--860. \doi{10.1214/08-AOAS169}.
#'
#' @seealso \code{\link{plot.gg_partial_varpro}},
#'   \code{\link{gg_varpro}}, \code{\link{gg_vimp}},
#'   \code{\link{gg_partialpro}} (deprecated),
#'   \code{\link{gg_partial_rfsrc}}, \code{\link{varpro_feature_names}}
#'
#' @examples
#' set.seed(42)
#' n_obs <- 30; n_pts <- 15
#' mock_data <- list(
#'   age = list(
#'     xvirtual    = seq(30, 80, length.out = n_pts),
#'     xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
#'   ),
#'   sex = list(
#'     xvirtual    = c(0, 1),
#'     xorg        = sample(c(0, 1), n_obs, replace = TRUE),
#'     yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
#'     yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
#'   )
#' )
#' result <- gg_partial_varpro(mock_data)
#' head(result$continuous)
#' head(result$categorical)
#'
#' @importFrom varPro partialpro
#' @export
gg_partial_varpro <- function(part_dta  = NULL,
                               object    = NULL,
                               scale     = c("auto", "rmst", "mortality",
                                             "surv", "chf"),
                               time      = NULL,
                               nvars     = NULL,
                               cat_limit = 10,
                               model     = NULL) {
  scale <- match.arg(scale)

  ## ---- Input validation --------------------------------------------------
  .validate_varpro_inputs(part_dta, object, scale, time)

  ## ---- C-path: route through gg_partial_rfsrc ----------------------------
  if (!is.null(object) && scale %in% c("surv", "chf")) {
    return(.gg_partial_varpro_cpath(object, scale, time, model))
  }

  ## ---- A-path: partialpro-based partial dependence -----------------------
  if (is.null(part_dta)) {
    part_dta <- varPro::partialpro(object)
  }
  if (is.null(nvars)) {
    nvars <- length(part_dta)
  }

  prov <- .varpro_provenance(object, scale, time, path = "A")

  dfs <- .build_varpro_dfs(part_dta, nvars, cat_limit)
  continuous  <- dfs$continuous
  categorical <- dfs$categorical

  if (!is.null(model)) {
    continuous$model  <- model
    categorical$model <- model
  }

  result <- list(continuous = continuous, categorical = categorical)
  class(result) <- c("gg_partial_varpro", "list")
  attr(result, "provenance") <- prov
  result
}

## ---------------------------------------------------------------------------
## Internal helpers

#' @keywords internal
.validate_varpro_inputs <- function(part_dta, object, scale, time) {
  if (is.null(part_dta) && is.null(object)) {
    stop("at least one of 'part_dta' or 'object' must be supplied",
         call. = FALSE)
  }
  if (scale %in% c("surv", "chf") && is.null(object)) {
    stop("scale = '", scale, "' requires 'object' (the varpro fit)",
         call. = FALSE)
  }
  if (scale == "rmst" && is.null(time)) {
    stop("scale = 'rmst' requires 'time' (the RMST horizon tau)",
         call. = FALSE)
  }
  invisible(NULL)
}

#' @keywords internal
.varpro_provenance <- function(object, scale, time, path = "A") {
  prov_family <- if (!is.null(object)) object$family     else NA_character_
  prov_xvars  <- if (!is.null(object)) object$xvar.names else NA_character_
  prov_n      <- if (!is.null(object)) nrow(object$x)    else NA_integer_
  prov_ntree  <- if (!is.null(object)) object$max.tree   else NA_integer_
  scale_used  <- .resolve_varpro_scale(scale, prov_family)
  list(
    source     = "varPro",
    family     = prov_family,
    ntree      = prov_ntree,
    n          = prov_n,
    scale      = scale_used,
    rmst_tau   = time,
    xvar.names = prov_xvars,
    path       = path
  )
}

#' @keywords internal
.build_varpro_dfs <- function(part_dta, nvars, cat_limit) {
  cont_list <- list()
  cat_list  <- list()
  for (feature in seq(nvars)) {
    feat      <- part_dta[[feature]]
    feat_name <- names(part_dta)[[feature]]
    if (length(feat$xvirtual) > cat_limit) {
      plt.df <- dplyr::bind_cols(
        variable      = feat$xvirtual,
        parametric    = colMeans(feat$yhat.par,    na.rm = TRUE),
        nonparametric = colMeans(feat$yhat.nonpar, na.rm = TRUE),
        causal        = colMeans(feat$yhat.causal, na.rm = TRUE)
      )
      plt.df$name <- feat_name
      cont_list[[feature]] <- plt.df
    } else {
      cat_list[[feature]] <- .process_cat_var(feat, feat_name)
    }
  }
  list(
    continuous  = dplyr::bind_rows(cont_list),
    categorical = dplyr::bind_rows(cat_list)
  )
}

#' @keywords internal
.resolve_varpro_scale <- function(scale, family) {
  if (scale != "auto") return(scale)
  if (is.na(family) || is.null(family)) return("generic")
  if (family == "surv")  return("mortality")
  "generic"   # regr, class, or unknown
}

#' @keywords internal
.process_cat_var <- function(feat, feat_name) {
  n_cats   <- length(unique(feat$xorg))
  cat_feat <- list()
  for (ind in seq(n_cats)) {
    cat_feat[[ind]] <- dplyr::bind_cols(
      parametric    = feat$yhat.par[, ind],
      nonparametric = feat$yhat.nonpar[, ind],
      causal        = feat$yhat.causal[, ind]
    )
    cat_feat[[ind]]$variable <- unique(feat$xorg)[ind]
    plt.df <- if (ind == 1L) cat_feat[[ind]] else
      dplyr::bind_rows(plt.df, cat_feat[[ind]])
  }
  plt.df$name <- feat_name
  plt.df
}

#' @keywords internal
.gg_partial_varpro_cpath <- function(object, scale, time, model) {
  rf           <- object$rf
  partial_type <- if (scale == "surv") "surv" else "chf"

  partial_time <- NULL
  if (!is.null(time)) {
    ti           <- rf$time.interest
    partial_time <- ti[which.min(abs(ti - time))]
  }

  pd <- gg_partial_rfsrc(rf,
                          xvar.names   = object$xvar.names,
                          partial.time = partial_time,
                          partial.type = partial_type)

  ## Prepend our class so S3 dispatch routes here first; NextMethod() falls
  ## through to plot.gg_partial_rfsrc for rendering.
  class(pd) <- c("gg_partial_varpro", class(pd))

  if (!is.null(model)) {
    if (is.data.frame(pd$continuous))  pd$continuous$model  <- model
    if (is.data.frame(pd$categorical)) pd$categorical$model <- model
  }

  attr(pd, "provenance") <- list(
    source     = "varPro",
    family     = object$family,
    ntree      = object$max.tree,
    n          = nrow(object$x),
    scale      = scale,
    rmst_tau   = time,
    xvar.names = object$xvar.names,
    path       = "C"
  )
  pd
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************

# Map partial.type ("surv" / "chf" / "mort") to a human y-axis label.
# Falls back to "Predicted Survival" when the attribute is absent (e.g. an
# object built before this attribute was introduced).
partial_surv_y_label <- function(partial.type) {
  if (is.null(partial.type)) return("Predicted Survival")
  switch(partial.type,
         surv = "Predicted Survival",
         chf  = "Predicted CHF",
         mort = "Predicted Mortality",
         "Predicted Survival")
}

#' Plot a \code{\link{gg_partial}} object
#'
#' Turns a \code{\link{gg_partial}} object into a ggplot2 figure.  Each curve
#' is a partial dependence trace -- the forest's average prediction as one
#' predictor is swept across its range while the rest are marginalized over the
#' training data.  Continuous predictors appear as line plots; categorical
#' predictors appear as bar charts.  Both panels are faceted by variable name
#' so you can compare the shape and scale of each variable's effect at a
#' glance.
#'
#' When a \code{model} label was attached in \code{gg_partial()}, lines are
#' coloured by model -- handy for overlaying results from two forests (e.g.,
#' one tuned, one default) in the same figure.
#'
#' @param x A \code{\link{gg_partial}} object (output of \code{\link{gg_partial}}).
#' @param ... Not currently used; reserved for future arguments.
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.  When only one
#'   variable type is present a single \code{ggplot} is returned.  When both
#'   continuous and categorical variables are present the two panels are
#'   combined vertically via \code{patchwork::wrap_plots()}, which also
#'   satisfies \code{inherits(p, "ggplot")}.
#'
#' @seealso \code{\link{gg_partial}}, \code{\link{plot.gg_variable}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' pv <- randomForestSRC::plot.variable(rf, partial = TRUE, show.plots = FALSE)
#' pd <- gg_partial(pv)
#' plot(pd)
#'
#' @importFrom ggplot2 .data
#' @importFrom patchwork wrap_plots
#' @export
plot.gg_partial <- function(x, ...) {
  gg_dta <- x

  gg_cont <- NULL
  if (!is.null(gg_dta$continuous) && nrow(gg_dta$continuous) > 0) {
    cont <- gg_dta$continuous
    gg_cont <- ggplot2::ggplot(cont,
                               ggplot2::aes(x = .data$x, y = .data$yhat)) +
      ggplot2::geom_line()

    if ("model" %in% colnames(cont)) {
      gg_cont <- gg_cont +
        ggplot2::aes(color = .data$model, group = .data$model)
    }

    gg_cont <- gg_cont +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  gg_cat <- NULL
  if (!is.null(gg_dta$categorical) && nrow(gg_dta$categorical) > 0) {
    cat_dta <- gg_dta$categorical
    gg_cat <- ggplot2::ggplot(cat_dta,
                              ggplot2::aes(x = .data$x, y = .data$yhat)) +
      ggplot2::geom_bar(stat = "identity", width = 0.5) +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

#' Plot a \code{\link{gg_partial_rfsrc}} object
#'
#' Renders the partial dependence curves from \code{\link{gg_partial_rfsrc}}
#' as a ggplot2 figure.  The layout adapts automatically to what the object
#' contains.
#'
#' For a standard regression or classification forest, continuous predictors
#' are drawn as line plots and categorical predictors as bar charts, both
#' faceted by variable name -- the same arrangement as
#' \code{\link{plot.gg_partial}}.
#'
#' For a survival forest, each call to \code{partial.rfsrc} returns a predicted
#' quantity (survival probability, cumulative hazard function, or mortality) at
#' one or more chosen time horizons.  When a \code{time} column is present in
#' the data, each horizon becomes a separate coloured curve over the predictor's
#' value, still faceted by variable.  The y-axis label (\dQuote{Predicted
#' Survival}, \dQuote{Predicted CHF}, or \dQuote{Predicted Mortality}) tracks
#' the \code{partial.type} attribute set by \code{gg_partial_rfsrc()}.
#'
#' For a two-variable interaction surface (when \code{xvar2.name} was supplied
#' to \code{gg_partial_rfsrc}), the secondary variable's levels become
#' separate coloured lines, faceted by the primary predictor.
#'
#' @param x A \code{\link{gg_partial_rfsrc}} object.
#' @param ... Not currently used.
#'
#' @return A \code{ggplot} (or \code{patchwork}) object.  When both continuous
#'   and categorical variables are present the two panels are combined
#'   vertically via \code{patchwork::wrap_plots()}.
#'
#' @seealso \code{\link{gg_partial_rfsrc}}, \code{\link{plot.gg_partial}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## Regression forest -- one continuous curve per variable
#' ## ------------------------------------------------------------
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#'
#' pd <- gg_partial_rfsrc(rfsrc_airq, xvar.names = c("Wind", "Temp"),
#'                        n_eval = 10)
#' plot(pd)
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Survival forest -- one curve per requested time horizon,
#' ## faceted by variable. Y-axis label tracks `partial.type`.
#' ## ------------------------------------------------------------
#' # randomForestSRC's formula parser requires the unqualified Surv() symbol;
#' # it Depends on `survival`, so Surv is on the search path once
#' # randomForestSRC is loaded.
#' data(veteran, package = "randomForestSRC")
#' set.seed(42)
#' rfsrc_v <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
#'                                   data = veteran, ntree = 50)
#' ti  <- rfsrc_v$time.interest
#' t30 <- ti[which.min(abs(ti - 30))]
#' t90 <- ti[which.min(abs(ti - 90))]
#'
#' # Default partial.type = "surv" -> y-axis "Predicted Survival"
#' pd_s <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
#'                          partial.time = c(t30, t90), n_eval = 8)
#' plot(pd_s)
#'
#' # partial.type = "chf" -> y-axis "Predicted CHF"
#' pd_c <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
#'                          partial.time = c(t30, t90),
#'                          partial.type = "chf", n_eval = 8)
#' plot(pd_c)
#' }
#'
#' @importFrom ggplot2 .data
#' @importFrom patchwork wrap_plots
#' @export
plot.gg_partial_rfsrc <- function(x, ...) {
  gg_dta <- x

  gg_cont <- NULL
  if (!is.null(gg_dta$continuous) && nrow(gg_dta$continuous) > 0) {
    cont <- gg_dta$continuous

    if (!is.null(cont$time)) {
      ## Survival forest: predictor value on x-axis, one curve per time point.
      ## Group/colour by the *full-precision* time so distinct horizons that
      ## happen to round to the same value are not silently merged. The legend
      ## is relabelled with rounded values for readability.
      time_levels <- sort(unique(cont$time))
      cont$.time_factor <- factor(cont$time, levels = time_levels)
      legend_labels <- format(round(time_levels, 2), trim = TRUE)
      y_lab <- partial_surv_y_label(attr(gg_dta, "partial.type"))
      gg_cont <- ggplot2::ggplot(
        cont,
        ggplot2::aes(
          x     = .data$x,
          y     = .data$yhat,
          color = .data$.time_factor,
          group = .data$.time_factor
        )
      ) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::scale_color_discrete(labels = legend_labels) +
        ggplot2::labs(x = NULL, y = y_lab, color = "Time")

    } else if (!is.null(cont$grp)) {
      ## Two-variable surface: group is xvar2; x-axis is the primary predictor
      gg_cont <- ggplot2::ggplot(
        cont,
        ggplot2::aes(
          x     = .data$x,
          y     = .data$yhat,
          color = factor(.data$grp),
          group = factor(.data$grp)
        )
      ) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::labs(x = NULL, y = "Partial Effect", color = "Group")

    } else {
      ## Standard: one curve per variable
      gg_cont <- ggplot2::ggplot(cont,
                                 ggplot2::aes(x = .data$x, y = .data$yhat)) +
        ggplot2::geom_line() +
        ggplot2::facet_wrap(~name, scales = "free_x") +
        ggplot2::labs(x = NULL, y = "Partial Effect")
    }
  }

  gg_cat <- NULL
  if (!is.null(gg_dta$categorical) && nrow(gg_dta$categorical) > 0) {
    cat_dta <- gg_dta$categorical
    gg_cat <- ggplot2::ggplot(
      cat_dta,
      ggplot2::aes(x = factor(.data$x), y = .data$yhat)
    ) +
      ggplot2::geom_bar(stat = "identity", width = 0.5) +
      ggplot2::facet_wrap(~name, scales = "free_x") +
      ggplot2::labs(x = NULL, y = "Partial Effect")
  }

  if (!is.null(gg_cont) && !is.null(gg_cat)) {
    wrap_plots(gg_cont, gg_cat, ncol = 1)
  } else if (!is.null(gg_cont)) {
    gg_cont
  } else {
    gg_cat
  }
}

#' @rdname plot.gg_partial_varpro
#' @name plot.gg_partial_varpro
#' @export
plot.gg_partialpro <- function(x, type = c("parametric", "nonparametric",
                                            "causal"), ...) {
  ## Deprecated class shim: re-dispatch to plot.gg_partial_varpro.
  class(x) <- c("gg_partial_varpro", setdiff(class(x), "gg_partialpro"))
  plot.gg_partial_varpro(x, type = type, ...)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_brier}} object
#'
#' Draws the time-resolved Brier score or the running CRPS from a
#' \code{\link{gg_brier}} object.  The curve moves across the event-time
#' grid on the x-axis; lower values mean the forest's predicted survival
#' probabilities are closer to what actually happened.  Think of
#' \code{0} as "perfect" and roughly \code{0.25} as "uninformative" -- a
#' forest that predicts \code{0.5} for every subject regardless of
#' prognosis would sit near that ceiling.
#'
#' Set \code{envelope = TRUE} to add a ribbon around the overall curve
#' spanning the 15th to 85th percentile of the per-subject Brier
#' contributions at each time.  The ribbon shows how heterogeneous the
#' scoring is across subjects: a narrow ribbon means most subjects are
#' predicted equally well (or equally poorly); a wide ribbon means a
#' minority of subjects are driving the average.
#'
#' @param x A \code{\link{gg_brier}} object.
#' @param type Which series to plot: \code{"brier"} (default) or
#'   \code{"crps"}.
#' @param envelope Logical. When \code{TRUE}, overlays a ribbon spanning
#'   the 15th-85th percentile of per-subject Brier (or running CRPS)
#'   contributions at each time, around the overall line. When \code{FALSE}
#'   (default), draws the overall series only.
#' @param ... Extra arguments forwarded to \code{geom_line()}.
#'
#' @return A \code{ggplot} object.
#'
#' @seealso \code{\link{gg_brier}},
#'   \code{\link[randomForestSRC]{get.brier.survival}}
#'
#' @examples
#' \donttest{
#' library(survival)   # Surv() must be on the search path for rfsrc()
#' data(pbc, package = "randomForestSRC")
#' rf <- randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
#'                              nsplit = 10)
#' gg_dta <- gg_brier(rf)
#' plot(gg_dta)
#' plot(gg_dta, type = "crps")
#' plot(gg_dta, envelope = TRUE)   # adds 15-85% envelope
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon labs theme
#' @export
plot.gg_brier <- function(x,
                          type = c("brier", "crps"),
                          envelope = FALSE,
                          ...) {
  if (!inherits(x, "gg_brier")) {
    stop("Incorrect object type: Expects a gg_brier object")
  }
  type <- match.arg(type)

  y_label <- if (type == "brier") "Brier score" else "CRPS"
  y_col   <- if (type == "brier") "brier" else "crps"
  lo_col  <- if (type == "brier") "bs.lower" else "crps.lower"
  hi_col  <- if (type == "brier") "bs.upper" else "crps.upper"

  gg_plt <- ggplot2::ggplot(
    x,
    ggplot2::aes(x = .data[["time"]], y = .data[[y_col]])
  )

  if (envelope) {
    gg_plt <- gg_plt +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data[[lo_col]], ymax = .data[[hi_col]]),
        alpha = 0.2,
        fill  = "steelblue"
      )
  }

  gg_plt <- gg_plt +
    ggplot2::geom_line(...) +
    ggplot2::labs(x = "Time", y = y_label) +
    ggplot2::theme(legend.position = "none")

  return(gg_plt)
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Predicted response data object
#'
#' Every tree in a random forest makes its own prediction, and the forest's
#' "ensemble" prediction is the average across all trees.  The out-of-bag
#' (OOB) variant averages only over the trees that did not include a given
#' observation in their bootstrap sample -- a built-in cross-validation
#' estimate that requires no held-out test set.  \code{gg_rfsrc} pulls those
#' ensemble predictions out of the fitted forest and arranges them for plotting
#' with \code{\link{plot.gg_rfsrc}}.
#'
#' The structure of the returned data depends on the forest family.  For a
#' regression forest you get a scatter of OOB predicted values against the
#' observed response.  For classification you get predicted class probabilities
#' alongside the observed class label.  For a survival forest you get the
#' ensemble survival function (or cumulative hazard, or mortality, controlled
#' by \code{surv_type}) at each unique event time -- one curve per
#' observation -- which together trace the range of predicted risk in the
#' cohort.  Pass \code{conf.int} to add pointwise bootstrap confidence bands
#' around the mean survival curve, or \code{by} to stratify all of the above
#' by a predictor group.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object.
#' @param by Optional stratifying variable. Either a character column name
#'   present in the training data, or a vector/factor of the same length as
#'   the training set. When supplied, a \code{group} column is added to the
#'   returned data and bootstrap CI bands (survival) are computed per group.
#'   Omit or leave missing to return an unstratified result.
#' @param oob Logical; if \code{TRUE} (default) return out-of-bag predictions.
#'   Set to \code{FALSE} to use full in-bag (training) predictions. Forced to
#'   \code{FALSE} automatically for \code{predict.rfsrc} objects, which carry
#'   no OOB estimates.
#' @param ... Additional arguments controlling output for specific forest
#'   families:
#'   \describe{
#'     \item{surv_type}{Character; one of \code{"surv"} (default),
#'       \code{"chf"}, or \code{"mortality"} for survival forests.}
#'     \item{conf.int}{Numeric coverage probability (e.g. \code{0.95}) to
#'       request bootstrap pointwise confidence bands for survival forests.
#'       Triggers wide-format output with \code{lower}, \code{upper},
#'       \code{median}, and \code{mean} columns.}
#'     \item{bs.sample}{Integer; number of bootstrap resamples when
#'       \code{conf.int} is set. Defaults to the number of observations.}
#'   }
#'
#' @return A \code{gg_rfsrc} object (a classed \code{data.frame}) whose
#'   structure depends on the forest family:
#'   \describe{
#'     \item{regression}{Columns \code{yhat} and the response name; optionally
#'       a \code{group} column when \code{by} is supplied.}
#'     \item{classification}{One column per class with predicted probabilities;
#'       a \code{y} column with observed class labels; optionally \code{group}.}
#'     \item{survival (no CI / grouping)}{Long-format with columns
#'       \code{variable} (event time), \code{value} (survival probability),
#'       \code{obs_id}, and \code{event}.}
#'     \item{survival (with \code{conf.int} or \code{by})}{Wide-format with
#'       pointwise bootstrap CI columns (\code{lower}, \code{upper},
#'       \code{median}, \code{mean}) per time point; a \code{group} column
#'       when \code{by} is supplied.}
#'   }
#'   The object carries class attributes for the forest family so that
#'   \code{\link{plot.gg_rfsrc}} dispatches correctly.
#'
#' @details
#'   For survival forests, use the \code{surv_type} argument
#'   (\code{"surv"}, \code{"chf"}, or \code{"mortality"}) to select the
#'   predicted quantity. Bootstrap confidence bands are requested by passing
#'   \code{conf.int} (e.g. \code{conf.int = 0.95}); the number of resamples
#'   is controlled by \code{bs.sample}.
#'
#' @seealso \code{\link{plot.gg_rfsrc}},
#'   \code{\link[randomForestSRC]{rfsrc}},
#'   \code{\link{gg_survival}}
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification example (small, runs on CRAN)
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
#' gg_dta <- gg_rfsrc(rfsrc_iris)
#' plot(gg_dta)
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Additional regression / survival examples are guarded with
#' ## \donttest because the cumulative example time exceeds the
#' ## 10-second CRAN budget. Run locally with `R CMD check --run-donttest`
#' ## (or `devtools::check(run_dont_test = TRUE)`) to exercise them.
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data (regression)
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
#'                     na.action = "na.impute", ntree = 50)
#' plot(gg_rfsrc(rfsrc_airq))
#'
#' ## -------- Boston data (rfsrc + randomForest)
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'   data(Boston, package = "MASS")
#'   Boston$chas <- as.logical(Boston$chas)
#'   rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston, ntree = 50,
#'                         forest = TRUE, importance = TRUE,
#'                         tree.err = TRUE, save.memory = TRUE)
#'   plot(gg_rfsrc(rfsrc_boston))
#'
#'   rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
#'                                           ntree = 50)
#'   plot(gg_rfsrc(rf_boston))
#' }
#'
#' ## -------- mtcars data
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#' plot(gg_rfsrc(rfsrc_mtcars))
#'
#' ## -------- veteran data (survival; with CI and group-by)
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        ntree = 50)
#' plot(gg_rfsrc(rfsrc_veteran))
#' plot(gg_rfsrc(rfsrc_veteran, conf.int = .95))
#' plot(gg_rfsrc(rfsrc_veteran, by = "trt"))
#' }
#'
#' @aliases gg_rfsrc gg_rfsrc.rfsrc

#' @export
gg_rfsrc.rfsrc <- function(object, # nolint: cyclocomp_linter
                           oob = TRUE,
                           by,
                           ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "rfsrc")) {
    stop(
      paste(
        "This function only works for Forests grown with the",
        "randomForestSRC package."
      )
    )
  }
  # The forest object itself must be stored; rfsrc uses forest = TRUE for this.
  if (is.null(object$forest)) {
    stop(
      paste(
        "The function requires the \"forest = TRUE\"",
        "attribute when growing the randomForest"
      )
    )
  }

  # Collect optional arguments (conf.int, surv_type, bs.sample, etc.)
  arg_list <- list(...)

  # Prediction objects do not have OOB predictions; force full-forest preds.
  if (inherits(object, "predict")) {
    oob <- FALSE
  }

  ## ---- Resolve the stratifying variable ("by") ----------------------------
  if (!missing(by)) {
    grp <- by
    # Accept either a column name (character) or a pre-built vector/factor.
    if (is.character(grp)) {
      if (!grp %in% colnames(object$xvar)) {
        stop(paste("No column named", grp, "in forest training set."))
      }
      grp <- object$xvar[, grp]
    }

    if (is.vector(grp) || is.factor(grp)) {
      if (length(grp) != nrow(object$xvar)) {
        stop(paste(
          "By argument does not have the correct dimension ",
          nrow(object$xvar)
        ))
      }
    } else {
      stop(
        paste(
          "By argument should be either a vector, or colname",
          "of training data"
        )
      )
    }
    # Convert to factor with levels in order of first appearance to preserve
    # the natural ordering of groups in downstream plots.
    grp <- factor(grp, levels = unique(grp))
  }

  ## ---- Classification branch -----------------------------------------------
  if (object$family == "class") {
    # For binary classification rfsrc stores exactly two probability columns
    # (one per class); we drop the first (the "negative" class probability)
    # since it is redundant: prob(class 2) = 1 - prob(class 1).
    # Multi-class forests (3+ classes) keep all columns.
    if (oob) {
      gg_dta <-
        if (ncol(object$predicted.oob) == 2) {
          data.frame(cbind(object$predicted.oob[, -1]))
        } else {
          data.frame(cbind(object$predicted.oob))
        }
    } else {
      gg_dta <- if (ncol(object$predicted) == 2) {
        data.frame(cbind(object$predicted[, -1]))
      } else {
        data.frame(cbind(object$predicted))
      }
    }

    # Assign column names and append the observed class label column "y".
    if (ncol(gg_dta) == 1) {
      colnames(gg_dta) <- object$yvar.names
      # rfsrc internally stores binary outcomes as a two-level factor even
      # when the original response is logical; convert back to logical here.
      gg_dta$y <- as.logical(as.numeric(object$yvar) - 1)
    } else {
      colnames(gg_dta) <- levels(object$yvar)
      gg_dta$y <- object$yvar
    }

    if (!missing(by)) {
      gg_dta$group <- grp
    }

  ## ---- Survival branch -----------------------------------------------------
  } else if (object$family == "surv") {
    # Default to overall survival; callers can request CHF or mortality.
    surv_type <- "surv"

    if (!is.null(arg_list$surv_type)) {
      surv_type <- arg_list$surv_type
    }

    # Extract the appropriate prediction matrix (rows = observations,
    # columns = unique time points stored in $time.interest).
    if (oob) {
      rng <- switch(surv_type,
        surv = data.frame(object$survival.oob),
        chf = data.frame(object$chf.oob),
        mortality = data.frame(1 - object$survival.oob),
        stop(paste(
          surv_type, " not implemented at this time"
        ))
      )
    } else {
      rng <- switch(surv_type,
        surv = data.frame(object$survival),
        chf = data.frame(object$chf),
        mortality = data.frame(1 - object$survival),
        stop(paste(
          surv_type, " not implemented at this time"
        ))
      )
    }

    # Name each column with the corresponding observed event time.
    colnames(rng) <- object$time.interest

    rng$obs_id <- seq_len(nrow(rng))
    if (is.null(object$yvar)) {
      rng$event <- FALSE
    } else {
      # Second column of yvar is the event indicator (0 = censored, 1 = event).
      rng$event <- as.logical(object$yvar[, 2])
    }
    gg_dta <- rng

    if (is.null(arg_list$conf.int) && missing(by)) {
      # No grouping or CI requested: pivot to long form so plot.gg_rfsrc can
      # draw one survival step function per observation.
      pivot_cols <-
        colnames(gg_dta)[-which(colnames(gg_dta) %in% c("obs_id", "event"))]
      gg_dta_mlt <-
        tidyr::pivot_longer(gg_dta, tidyr::all_of(pivot_cols), names_to = "variable", values_to = "value")
      gg_dta_mlt$variable <-
        as.numeric(as.character(gg_dta_mlt$variable))
      gg_dta_mlt$obs_id <- factor(gg_dta_mlt$obs_id)

      gg_dta <- gg_dta_mlt
    } else {
      # Bootstrap confidence bands: compute pointwise quantiles of the
      # bootstrapped mean survival curve at each time point.
      level <- if (is.null(arg_list$conf.int)) {
        .95
      } else {
        arg_list$conf.int
      }

      # Accept a single coverage probability (e.g. 0.95 gives two-sided tails
      # at 0.025 and 0.975) or a length-2 vector of explicit quantile probs.
      if (length(level) == 1) {
        if (level > 1) {
          level <- level / 100
        }

        level_set <- c((1 - level) / 2, 1 - (1 - level) / 2)
        level_set <- sort(level_set)
      } else {
        level_set <- sort(level)
      }

      # Number of bootstrap resamples: default to the number of observations.
      if (is.null(arg_list$bs.sample)) {
        bs_samples <- nrow(gg_dta)
      } else {
        bs_samples <- arg_list$bs.sample
      }

      if (!missing(by)) {
        # Stratified: compute separate bootstrap CI bands per group level.
        gg_dta$group <- grp
        grp_dta <- lapply(levels(grp), function(st) {
          if (is.null(arg_list$bs.sample)) {
            bs_samples <-
              nrow(gg_dta[which(as.character(gg_dta$group) == st), ])
          }

          obj <-
            bootstrap_survival(
              gg_dta[which(as.character(gg_dta$group) == st), ],
              bs_samples, level_set
            )
          obj$group <- st
          obj
        })
        gg_grp <- do.call(rbind, grp_dta)
        gg_grp$group <- factor(gg_grp$group,
          levels = unique(gg_grp$group)
        )
        gg_dta <- gg_grp
      } else {
        # Single-group CI band.
        gg_dta <- bootstrap_survival(gg_dta, bs_samples, level_set)
      }
    }

  ## ---- Regression branch ---------------------------------------------------
  } else if (object$family == "regr") {
    # Bind predicted values alongside the observed response for scatter plots.
    if (oob) {
      gg_dta <- data.frame(cbind(object$predicted.oob, object$yvar))
    } else {
      gg_dta <- data.frame(cbind(object$predicted, object$yvar))
    }

    colnames(gg_dta) <- c("yhat", object$yvar.names)

    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else {
    stop(
      paste(
        "Plotting for ",
        object$family,
        " randomForestSRC is not yet implemented.",
        sep = ""
      )
    )
  }

  # Tag the data frame with the forest family so plot.gg_rfsrc can dispatch
  # to the correct branch without re-inspecting the original forest object.
  class(gg_dta) <- c("gg_rfsrc", object$family, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}



#' Bootstrap pointwise confidence bands for a mean survival curve
#'
#' Draws \code{bs_samples} bootstrap resamples (with replacement) of the
#' per-observation survival curves stored in \code{gg_dta}, computes the column
#' means to obtain a bootstrapped mean curve per resample, then returns the
#' pointwise quantiles at \code{level_set} and the overall mean across
#' resamples.
#'
#' @param gg_dta A wide \code{data.frame} of survival probabilities as returned
#'   by the survival branch of \code{\link{gg_rfsrc.rfsrc}}, before the
#'   optional pivot to long form. Columns \code{obs_id}, \code{event}, and
#'   \code{group} (if present) are excluded from the resampling.
#' @param bs_samples Integer; number of bootstrap resamples.
#' @param level_set Numeric vector of length 2 giving the lower and upper
#'   quantile probabilities for the confidence band (e.g. \code{c(0.025, 0.975)}
#'   for a 95% CI).
#'
#' @return A \code{data.frame} with one row per unique event time and columns
#'   \code{value} (time), \code{lower}, \code{upper}, \code{median}, and
#'   \code{mean}.
#'
#' @keywords internal
bootstrap_survival <- function(gg_dta, bs_samples, level_set) {
  ## Calculate the leave one out estimate of the mean survival
  gg_t <-
    gg_dta[, -which(colnames(gg_dta) %in% c("obs_id", "event", "group"))]
  mn_bs <- t(sapply(
    seq_len(bs_samples),
    function(pat) {
      st <- sample(seq_len(nrow(gg_t)),
        size = nrow(gg_t),
        replace = TRUE
      )
      colMeans(gg_t[st, ])
    }
  ))

  ## now get the confidence interval of the mean, and the median (.5)
  rng <- sapply(
    seq_len(ncol(mn_bs)),
    function(t_pt) {
      quantile(mn_bs[, t_pt], probs = c(level_set, .5))
    }
  )
  mn <- sapply(seq_len(ncol(rng)), function(t_pt) {
    mean(rng[, t_pt])
  })

  # gg_t already has obs_id/event/group stripped; rng and mn are indexed over
  # time points only, so no further exclusion is needed.
  time_interest <- as.numeric(colnames(gg_t))

  dta <- data.frame(cbind(
    time_interest,
    t(rng),
    mn
  ))

  # rng always has 3 rows: lower quantile, upper quantile, median (.5).
  # Name columns canonically so plot.gg_rfsrc can always find "lower"/"upper".
  colnames(dta) <- c("value", "lower", "upper", "median", "mean")
  dta
}

#' @export
gg_rfsrc <- function(object,
                     oob = TRUE,
                     by, ...) {
  UseMethod("gg_rfsrc", object)
}

#' @export
gg_rfsrc.randomForest <- function(object,
                                  oob = TRUE,
                                  by,
                                  ...) {
  ## Check that the input object is of the correct type.
  if (!inherits(object, "randomForest")) {
    stop(
      paste(
        "This function only works for Forests grown with the",
        "randomForest package."
      )
    )
  }

  if (inherits(object, "predict")) {
    oob <- FALSE
  }

  # Recover the training predictor frame once (needed for by-column lookup and
  # dimension checks).  randomForest stores predictors in $forest$xlevels keys
  # but not the actual data; use .rf_recover_model_frame() for that.
  rf_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
  rf_xvar <- if (!is.null(rf_info)) rf_info$model_frame[
    , setdiff(colnames(rf_info$model_frame), rf_info$response_name),
    drop = FALSE
  ] else NULL
  n_train <- length(object$predicted)

  if (!missing(by)) {
    grp <- by
    # Accept either a column name (character) or a pre-built vector/factor.
    if (is.character(grp)) {
      if (is.null(rf_xvar) || !grp %in% colnames(rf_xvar)) {
        stop(paste("No column named", grp, "in forest training set."))
      }
      grp <- rf_xvar[, grp]
    }

    if (is.vector(grp) || is.factor(grp)) {
      if (length(grp) != n_train) {
        stop(paste(
          "By argument does not have the correct dimension ",
          n_train
        ))
      }
    } else {
      stop(
        paste(
          "By argument should be either a vector, or colname",
          "of training data"
        )
      )
    }
    grp <- factor(grp, levels = unique(grp))
  }

  # Extract the response variable name from the formula for column naming below.
  # (Both branches below build gg_dta from the forest's stored predictions, so
  # there is no need to recover the original training data frame here.)
  rsp <- as.character(object$call$formula)[2]

  # Do the work...
  if (object$type == "classification") {
    ### Classification models...

    # Need to add multiclass methods
    prd <- predict(object, type = "prob")
    gg_dta <- if (ncol(prd) <= 2) {
      data.frame(cbind(prd[, -1]))
    } else {
      data.frame(cbind(prd))
    }
    colnames(gg_dta) <- object$classes

    # Switch between logical or categorical outcome
    if (ncol(gg_dta) == 1) {
      # Force this to logical return value...
      #
      # This may be a bug in rfsrc, as it converts all classification models
      # into factors.
      gg_dta$y <- as.logical(as.numeric(object$yvar) - 1)
    } else {
      gg_dta$y <- object$y
    }

    # Handle the "by" argument.
    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else if (object$type == "regression") {
    gg_dta <- data.frame(cbind(object$predicted, object$y))

    colnames(gg_dta) <- c("yhat", rsp)

    # Handle the "by" argument.
    if (!missing(by)) {
      gg_dta$group <- grp
    }
  } else {
    stop(
      paste(
        "Plotting for ",
        object$family,
        " randomForest is not yet implemented for ",
        object$type,
        sep = ""
      )
    )
  }

  class(gg_dta) <- c("gg_rfsrc", object$type, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  message    = FALSE,
  warning    = FALSE,
  fig.width  = 7,
  fig.height = 4.5,
  cache      = TRUE,
  cache.path = "varpro_cache/"
)
options(mc.cores = 1, rf.cores = 1)


## ----libs---------------------------------------------------------------------
library(ggplot2)

# Match the pattern used by the regression/survival vignettes: try the
# installed package first, fall back to pkgload::load_all() for the
# R CMD check vignette rebuild where the package isn't yet on
# .libPaths(). All varPro calls below are ::-qualified, so no
# library(varPro) is needed.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}


## ----precomputed, include=FALSE, cache=FALSE----------------------------------
# The three gg_partial_varpro() calls and the Boston beta.varpro() fit are
# the only expensive parts of this vignette (~34s combined). They are
# precomputed by precompute_varpro.R and loaded here so the R CMD check
# vignette rebuild stays fast. If the file is absent (e.g. a fresh clone
# that hasn't run the script), each chunk falls back to live computation,
# so the vignette is reproducible either way.
.vp <- tryCatch({
  # R CMD check rebuilds the vignette in a copy whose working directory is the
  # vignette dir; an interactive knit from the package root sees vignettes/
  # instead. Try both locations, and fall back to live computation below if
  # neither is present or readRDS() fails.
  .cand <- c("varpro_precomputed.rds",
             file.path("vignettes", "varpro_precomputed.rds"))
  .hit <- .cand[file.exists(.cand)]
  if (length(.hit)) readRDS(.hit[[1]]) else list()
}, error = function(e) list())   # missing or unreadable -> live fallback below


## ----boston-fit---------------------------------------------------------------
data("Boston", package = "MASS")
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_boston <- if (is.null(.vp$v_boston)) {
  varPro::varpro(medv ~ ., data = Boston, ntree = 50)
} else {
  .vp$v_boston
}
v_boston


## ----boston-gg-varpro---------------------------------------------------------
plot(gg_varpro(v_boston))


## ----boston-gg-partial-varpro-------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd <- if (is.null(.vp$pd_boston)) {
  gg_partial_varpro(object = v_boston)
} else {
  .vp$pd_boston
}
plot(gg_pd)


## ----boston-beta-fit, cache=TRUE----------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_boston <- if (is.null(.vp$b_boston)) {
  varPro::beta.varpro(v_boston)
} else {
  .vp$b_boston
}


## ----boston-gg-beta-varpro----------------------------------------------------
plot(gg_beta_varpro(v_boston, beta_fit = b_boston))


## ----boston-uvarpro, cache=TRUE-----------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
u_boston <- if (is.null(.vp$u_boston)) {
  varPro::uvarpro(Boston[, setdiff(names(Boston), "medv")], ntree = 50)
} else {
  .vp$u_boston
}


## ----boston-gg-udependent, eval = requireNamespace("ggraph", quietly = TRUE)----
plot(gg_udependent(u_boston))


## ----boston-isopro, cache=TRUE------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_boston <- if (is.null(.vp$iso_boston)) {
  varPro::isopro(data = Boston[, setdiff(names(Boston), "medv")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_boston
}


## ----boston-gg-isopro---------------------------------------------------------
plot(gg_isopro(iso_boston))


## ----boston-ivarpro-fit, cache=TRUE-------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iv_boston <- if (is.null(.vp$iv_boston)) {
  varPro::ivarpro(v_boston)
} else {
  .vp$iv_boston
}


## ----boston-gg-ivarpro-distribution-------------------------------------------
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston))


## ----boston-gg-ivarpro-which-obs----------------------------------------------
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston, which_obs = 1L))


## ----iris-fit-binary----------------------------------------------------------
iris_binary <- iris[iris$Species != "setosa", ]
iris_binary$Species <- droplevels(iris_binary$Species)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_binary <- if (is.null(.vp$v_iris_binary)) {
  varPro::varpro(Species ~ ., data = iris_binary, ntree = 50)
} else {
  .vp$v_iris_binary
}


## ----iris-fit-multi-----------------------------------------------------------
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_multi <- if (is.null(.vp$v_iris_multi)) {
  varPro::varpro(Species ~ ., data = iris, ntree = 50)
} else {
  .vp$v_iris_multi
}


## ----iris-multi-gg-varpro-conditional-----------------------------------------
plot(gg_varpro(v_iris_multi, conditional = TRUE))


## ----iris-multi-gg-partial-varpro---------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_iris <- if (is.null(.vp$pd_iris_multi)) {
  gg_partial_varpro(object = v_iris_multi)
} else {
  .vp$pd_iris_multi
}
plot(gg_pd_iris)


## ----iris-binary-beta-fit, cache=TRUE-----------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_binary <- if (is.null(.vp$b_iris_binary)) {
  varPro::beta.varpro(v_iris_binary)
} else {
  .vp$b_iris_binary
}


## ----iris-binary-gg-beta-varpro-----------------------------------------------
plot(gg_beta_varpro(v_iris_binary, beta_fit = b_iris_binary))


## ----iris-multi-beta-fit, cache=TRUE------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_multi <- if (is.null(.vp$b_iris_multi)) {
  varPro::beta.varpro(v_iris_multi)
} else {
  .vp$b_iris_multi
}


## ----iris-multi-gg-beta-varpro------------------------------------------------
plot(gg_beta_varpro(v_iris_multi, beta_fit = b_iris_multi))


## ----pbc-data-----------------------------------------------------------------
library(survival)
data(pbc, package = "randomForestSRC")
pbc_small <- pbc[, c("days", "status", "age", "albumin", "bili",
                     "edema", "platelet")]
pbc_small <- na.omit(pbc_small)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_pbc <- if (is.null(.vp$v_pbc)) {
  varPro::varpro(Surv(days, status) ~ ., data = pbc_small, ntree = 50)
} else {
  .vp$v_pbc
}


## ----pbc-gg-varpro------------------------------------------------------------
plot(gg_varpro(v_pbc))


## ----pbc-gg-partial-varpro----------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_pbc <- if (is.null(.vp$pd_pbc)) {
  gg_partial_varpro(object = v_pbc)
} else {
  .vp$pd_pbc
}
plot(gg_pd_pbc)


## ----pbc-isopro, cache=TRUE---------------------------------------------------
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_pbc <- if (is.null(.vp$iso_pbc)) {
  varPro::isopro(data = pbc_small[, c("age", "albumin", "bili", "platelet")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_pbc
}
plot(gg_isopro(iso_pbc))

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#'
#' Brier score and CRPS for survival forests
#'
#' The Brier score asks a familiar question of any probabilistic forecast:
#' how far did the predicted probability sit from what actually happened?
#' For a survival forest the forecast is the predicted survival probability
#' at a given moment, and the "what happened" is whether the subject was
#' still alive at that moment.  The score is computed at every event time,
#' so you get a curve rather than a single number -- lower is better
#' everywhere.  A perfectly calibrated forest that predicts \code{0} for
#' every subject who died and \code{1} for every subject who survived would
#' score \code{0}; a forest that predicts \code{0.5} for everyone scores
#' roughly \code{0.25} regardless of the true outcome -- that is the
#' "uninformative" ceiling.
#'
#' This function extracts the time-resolved Brier score for a survival
#' forest grown with \code{randomForestSRC}, both overall and broken down
#' by mortality-risk quartile (lowest-risk to highest-risk subjects).  It
#' also returns the continuous ranked probability score (CRPS) -- the Brier
#' score integrated over time and divided by elapsed time, a running average
#' that summarises calibration up to each point on the time axis.
#'
#' @details
#' Because subjects are right-censored, a plain Brier score is biased:
#' censored subjects contribute no outcome information yet still inflate the
#' denominator.  The score here uses inverse-probability-of-censoring
#' weighting (IPCW), which up-weights uncensored observations to compensate.
#' The censoring distribution is estimated either by Kaplan-Meier
#' (\code{cens.model = "km"}, the default) or by a separate censoring
#' forest (\code{cens.model = "rfsrc"}) when the censoring mechanism is
#' itself covariate-dependent.
#'
#' Internally, this wraps \code{\link[randomForestSRC]{get.brier.survival}}
#' and rebuilds the quartile decomposition and running CRPS from the returned
#' \code{brier.matx} and \code{mort} components, following the approach in
#' the internal \code{plot.survival} of \pkg{randomForestSRC}.
#'
#' @param object A fitted \code{\link[randomForestSRC]{rfsrc}} survival
#'   forest (\code{object$family == "surv"}).
#' @param ... Currently unused; accepted for S3 dispatch compatibility.
#'
#' @return A \code{gg_brier} \code{data.frame} with columns
#'   \describe{
#'     \item{time}{event time grid (\code{object$time.interest}).}
#'     \item{brier}{overall Brier score at each time.}
#'     \item{bs.q25, bs.q50, bs.q75, bs.q100}{Brier score within each
#'       mortality-risk quartile (lowest to highest risk).}
#'     \item{bs.lower, bs.upper}{15th and 85th percentile of per-subject
#'       Brier contributions at each time. Used by
#'       \code{plot.gg_brier(by_quartile = TRUE)} to draw an envelope
#'       around the overall curve.}
#'     \item{crps}{running CRPS (overall) at each time, normalised by
#'       elapsed time.}
#'     \item{crps.q25, crps.q50, crps.q75, crps.q100}{running CRPS within
#'       each mortality-risk quartile.}
#'     \item{crps.lower, crps.upper}{running CRPS of the 15th / 85th
#'       per-subject Brier percentile, normalised by elapsed time.}
#'   }
#'   The integrated CRPS (a single scalar matching
#'   \code{get.brier.survival()$crps}) is attached as
#'   \code{attr(., "crps_integrated")}.
#'
#' @seealso \code{\link{plot.gg_brier}},
#'   \code{\link[randomForestSRC]{get.brier.survival}},
#'   \code{\link{gg_error}}
#'
#' @references
#' Graf E., Schmoor C., Sauerbrei W., Schumacher M. (1999). Assessment and
#' comparison of prognostic classification schemes for survival data.
#' Statistics in Medicine, 18(17-18):2529-2545.
#'
#' Gerds T.A., Schumacher M. (2006). Consistent estimation of the expected
#' Brier score in general survival models with right-censored event times.
#' Biometrical Journal, 48(6):1029-1040.
#'
#' @examples
#' \donttest{
#' library(survival)   # Surv() must be on the search path for rfsrc()
#' data(pbc, package = "randomForestSRC")
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(days, status) ~ ., data = pbc, nsplit = 10
#' )
#' gg_dta <- gg_brier(rfsrc_pbc)
#' plot(gg_dta)
#' plot(gg_dta, type = "crps")
#' plot(gg_dta, envelope = TRUE)   # overall line + 15-85% envelope
#'
#' # Multi-model comparison: stack gg_brier outputs and plot with ggplot2.
#' rf2 <- randomForestSRC::rfsrc(
#'   Surv(days, status) ~ ., data = pbc, nsplit = 10, mtry = 4
#' )
#' compare_dta <- dplyr::bind_rows(
#'   dplyr::mutate(gg_brier(rfsrc_pbc), model = "default"),
#'   dplyr::mutate(gg_brier(rf2),       model = "mtry=4")
#' )
#' ggplot2::ggplot(compare_dta,
#'   ggplot2::aes(x = time, y = brier, colour = model)) +
#'   ggplot2::geom_line()
#' }
#'
#' @note Brier score / CRPS is `randomForestSRC` survival-only; there
#'   is no `randomForest` method.
#' @importFrom stats quantile
#' @export
gg_brier <- function(object, ...) {
  UseMethod("gg_brier", object)
}

#' @export
gg_brier.rfsrc <- function(object,
                           subset = NULL,
                           cens.model = c("km", "rfsrc"),
                           ...) {
  if (!inherits(object, "rfsrc")) {
    stop("This function only works on rfsrc objects.")
  }
  if (is.null(object$family) || object$family != "surv") {
    stop("gg_brier only supports right-censored survival forests.")
  }

  cens.model <- match.arg(cens.model)

  brier_obj <- randomForestSRC::get.brier.survival(
    object,
    subset = subset,
    cens.model = cens.model
  )

  bs_df <- brier_obj$brier.score
  if (!all(c("time", "brier.score") %in% names(bs_df))) {
    stop("Unexpected output from get.brier.survival(); ",
         "ggRandomForests may need to be updated for this randomForestSRC version.")
  }

  brier_matx <- brier_obj$brier.matx
  mort       <- brier_obj$mort

  # Quartile breakpoints of per-subject mortality. The lowest bin uses an
  # open-left bound so the minimum-risk subject is captured.
  mort_breaks <- c(min(mort, na.rm = TRUE) - 1e-05,
                   stats::quantile(mort, (1:4) / 4, na.rm = TRUE))

  bs_quartile <- vapply(seq_len(4), function(k) {
    in_bin <- mort > mort_breaks[k] & mort <= mort_breaks[k + 1]
    if (!any(in_bin, na.rm = TRUE)) {
      # Empty bin: can occur when mort has ties at a quantile boundary.
      return(rep(NA_real_, nrow(bs_df)))
    }
    colMeans(brier_matx[in_bin, , drop = FALSE], na.rm = TRUE)
  }, numeric(nrow(bs_df)))

  # Per-time 15th and 85th percentile of per-subject Brier contributions.
  # Provides a non-parametric envelope around the overall curve for the
  # default by_quartile = TRUE ribbon display.
  bs_envelope <- apply(brier_matx, 2, stats::quantile,
                       probs = c(0.15, 0.85), na.rm = TRUE)

  time_grid <- bs_df$time
  bs_all    <- bs_df$brier.score

  # Running, time-normalised CRPS via single-pass cumulative trapezoid rule.
  # O(n) per series: accumulate area incrementally rather than recomputing
  # from scratch at every time point.
  crps_running <- function(times, scores) {
    n        <- length(times)
    out      <- numeric(n)
    out[1]   <- scores[1]
    cum_area <- 0
    for (j in seq_len(n)[-1]) {
      cum_area <- cum_area +
        (times[j] - times[j - 1]) * (scores[j] + scores[j - 1]) / 2
      span     <- times[j] - times[1]
      out[j]   <- if (span <= 0) NA_real_ else cum_area / span
    }
    out
  }

  crps_all   <- crps_running(time_grid, bs_all)
  crps_q     <- vapply(seq_len(4), function(k) {
    crps_running(time_grid, bs_quartile[, k])
  }, numeric(length(time_grid)))
  crps_lower <- crps_running(time_grid, bs_envelope[1, ])
  crps_upper <- crps_running(time_grid, bs_envelope[2, ])

  gg_dta <- data.frame(
    time        = time_grid,
    brier       = bs_all,
    bs.q25      = bs_quartile[, 1],
    bs.q50      = bs_quartile[, 2],
    bs.q75      = bs_quartile[, 3],
    bs.q100     = bs_quartile[, 4],
    bs.lower    = bs_envelope[1, ],
    bs.upper    = bs_envelope[2, ],
    crps        = crps_all,
    crps.q25    = crps_q[, 1],
    crps.q50    = crps_q[, 2],
    crps.q75    = crps_q[, 3],
    crps.q100   = crps_q[, 4],
    crps.lower  = crps_lower,
    crps.upper  = crps_upper
  )

  attr(gg_dta, "crps_integrated") <- brier_obj$crps
  attr(gg_dta, "cens.model")      <- cens.model
  class(gg_dta) <- c("gg_brier", class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

# Internal trapezoidal integrator: sum_i (x[i+1]-x[i]) * (y[i]+y[i+1])/2.
.trapz <- function(x, y) {
  n <- length(x)
  if (n < 2) {
    return(0)
  }
  sum((x[-1] - x[-n]) * (y[-1] + y[-n])) / 2
}

# v2.8.0 Phase 0: randomForestSRC/randomForest are Imports, not attached.
# Survival-formula tests use a bare Surv(...) in rfsrc() formulas, which
# resolve in the test environment only if `survival` is attached. Attach
# the exact dependency surface the tests assume, here, once.
library(survival)
library(randomForestSRC)
library(randomForest)

####**********************************************************************
####  plot.gg_isopro: ranked-elbow + density plot for isopro anomaly scores.
####**********************************************************************

#' Plot a varPro isolation-forest anomaly score
#'
#' Renders a \code{gg_isopro} object as a ranked elbow (observations
#' sorted by anomaly score), a density of scores, or both side-by-side.
#' Optionally annotates a threshold either in score-space
#' (\code{threshold}) or in quantile-space (\code{top_n_pct}).
#'
#' @section Reading the elbow:
#' The elbow plot is the canonical anomaly-detection picture. The x-axis
#' is observation rank (observations sorted from most ordinary to most
#' anomalous) and the y-axis is the \code{howbad} score. For a clean
#' population the curve sits flat near zero across the bulk of the data
#' and then bends sharply upward in the right tail; that bend is where
#' the anomalous observations live. The point of the plot is not to read
#' off a single score, it is to \emph{see where the curve breaks}. Pick a
#' cutoff there. Pass it back in as \code{threshold} (for a score) or
#' \code{top_n_pct} (for "the top 5\%") and the plot draws a dashed
#' reference line so you can record the choice you made.
#'
#' @section Reading the density:
#' The density panel is the same scores viewed as a distribution. A
#' single tight mode near zero with a long thin right tail is the
#' picture you hope for: bulk of the data ordinary, a few clear
#' anomalies. A bimodal density says you may have two populations rather
#' than one clean cluster plus outliers, and the cutoff question becomes
#' harder. Either way, this panel is a sanity check on what the elbow
#' suggests.
#'
#' @section Comparing methods:
#' When the input object carries a \code{method} column (because you
#' bound several \code{\link{gg_isopro}} calls together), both panels
#' colour by method automatically. The point of comparing \code{"rnd"},
#' \code{"unsupv"}, and \code{"auto"} is not to pick a winner from the
#' figure alone, it is to see whether the methods agree on which
#' observations are anomalous. Curves that overlap in the right tail and
#' elbow at roughly the same rank are telling you the same story three
#' ways. Curves that diverge are telling you the score is
#' method-sensitive, which is itself useful information.
#'
#' @param x A \code{gg_isopro} object from \code{\link{gg_isopro}}.
#' @param panel One of \code{"both"} (default: a \code{patchwork} of
#'   elbow + density), \code{"elbow"}, or \code{"density"} (each returns a
#'   single \code{ggplot}).
#' @param threshold Numeric in \code{[0, 1]}, or \code{NULL} (default). If
#'   set, draws a reference line at that \code{howbad} value on the elbow
#'   and density.
#' @param top_n_pct Numeric in \code{(0, 100)}, or \code{NULL} (default).
#'   If set, resolves to the matching \code{howbad} quantile and draws the
#'   same reference line. If both \code{threshold} and \code{top_n_pct}
#'   are supplied, \code{threshold} wins with a \code{message()}.
#' @param ... Currently unused.
#'
#' @return A \code{ggplot} (single panel) or a \code{patchwork}
#'   (\code{panel = "both"}).
#'
#' @seealso \code{\link{gg_isopro}}, \code{\link[varPro]{isopro}}
#' @export
plot.gg_isopro <- function(x,
                           panel     = c("both", "elbow", "density"),
                           threshold = NULL,
                           top_n_pct = NULL,
                           ...) {
  panel <- match.arg(panel)

  # Resolve threshold (score wins if both supplied).
  thr <- .resolve_isopro_threshold(x$howbad, threshold, top_n_pct)

  # Multi-method detection: a `method` column means the user bound several
  # fits together with bind_rows; colour/group by method.
  has_method <- "method" %in% names(x)

  elbow   <- .gg_isopro_elbow(x, thr, has_method)
  density <- .gg_isopro_density(x, thr, has_method)

  if (panel == "elbow")    return(elbow)
  if (panel == "density")  return(density)

  # panel == "both"
  patchwork::wrap_plots(elbow, density, nrow = 1)
}

# Validate a single threshold-style argument: numeric scalar, not NA, and
# within (lo, hi). `closed_lo` / `closed_hi` control whether each end is
# inclusive. stop()s with a clear message; returns the value as numeric.
.check_threshold_arg <- function(x, name, lo, hi,
                                 closed_lo = TRUE, closed_hi = TRUE) {
  ok_type <- is.numeric(x) && length(x) == 1L && !is.na(x)
  ok_lo   <- ok_type && (if (closed_lo) x >= lo else x > lo)
  ok_hi   <- ok_type && (if (closed_hi) x <= hi else x < hi)
  if (!(ok_type && ok_lo && ok_hi)) {
    bra_lo <- if (closed_lo) "[" else "("
    bra_hi <- if (closed_hi) "]" else ")"
    stop(sprintf("`%s` must be a single numeric value in %s%g, %g%s.",
                 name, bra_lo, lo, hi, bra_hi),
         call. = FALSE)
  }
  as.numeric(x)
}

# Returns the resolved threshold (numeric scalar) or NA_real_ if neither
# arg was supplied. Emits a message when both args are set.
.resolve_isopro_threshold <- function(howbad, threshold, top_n_pct) {
  if (!is.null(threshold) && !is.null(top_n_pct)) {
    message("Both `threshold` and `top_n_pct` supplied; using `threshold`.")
    top_n_pct <- NULL
  }
  if (!is.null(threshold)) {
    return(.check_threshold_arg(threshold, "threshold", 0, 1))
  }
  if (!is.null(top_n_pct)) {
    pct <- .check_threshold_arg(top_n_pct, "top_n_pct", 0, 100,
                                closed_lo = FALSE, closed_hi = FALSE)
    q   <- 1 - (pct / 100)
    return(as.numeric(stats::quantile(howbad, probs = q, na.rm = TRUE)))
  }
  NA_real_
}

# Rank-ordered elbow panel.
.gg_isopro_elbow <- function(x, thr, has_method) {
  if (has_method) {
    x$rank <- stats::ave(x$howbad, x$method, FUN = function(v) rank(v, ties.method = "first"))
  } else {
    x$rank <- rank(x$howbad, ties.method = "first")
  }
  x <- x[order(x$rank), , drop = FALSE]

  aes_line <- if (has_method) {
    ggplot2::aes(x = .data$rank, y = .data$howbad, colour = .data$method)
  } else {
    ggplot2::aes(x = .data$rank, y = .data$howbad)
  }

  p <- ggplot2::ggplot(x) +
    ggplot2::geom_line(aes_line) +
    ggplot2::labs(x = "Observation rank (by howbad)",
                  y = "howbad (anomaly score)")

  if (!is.na(thr)) {
    p <- p +
      ggplot2::geom_hline(yintercept = thr, linetype = "dashed",
                          colour = "red", linewidth = 0.4)
  }
  p
}

# Score-density panel.
.gg_isopro_density <- function(x, thr, has_method) {
  aes_dens <- if (has_method) {
    ggplot2::aes(x = .data$howbad, fill = .data$method)
  } else {
    ggplot2::aes(x = .data$howbad)
  }

  p <- ggplot2::ggplot(x) +
    ggplot2::geom_density(aes_dens, alpha = if (has_method) 0.4 else 1) +
    ggplot2::labs(x = "howbad (anomaly score)", y = "Density")

  if (!is.na(thr)) {
    p <- p +
      ggplot2::geom_vline(xintercept = thr, linetype = "dashed",
                          colour = "red", linewidth = 0.4)
  }
  p
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' nonparametric Kaplan-Meier estimates
#'
#' @param data name of the training set \code{data.frame}
#' @param interval name of the interval variable in the training dataset.
#' @param censor name of the censoring variable in the training dataset.
#' @param by stratifying variable in the training dataset, defaults to NULL
#' @param ... arguments passed to the \code{survfit} function
#'
#' @return \code{\link{gg_survival}} object
#'
#' @importFrom survival Surv survfit strata
#'
#' @seealso \code{\link{gg_survival}} \code{\link{nelson}}
#' \code{\link{plot.gg_survival}}
#'
#' @examples
#'
#' # These get run through the gg_survival examples.
#' data(pbc, package = "randomForestSRC")
#' pbc$time <- pbc$days / 364.25
#'
#' # This is the same as gg_survival
#' gg_dta <- kaplan(
#'   interval = "time", censor = "status",
#'   data = pbc
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' # Stratified on treatment variable.
#' gg_dta <- gg_survival(
#'   interval = "time", censor = "status",
#'   data = pbc, by = "treatment"
#' )
#'
#' plot(gg_dta, error = "none")
#' plot(gg_dta)
#'
#' @export
kaplan <- function(interval,
                   censor,
                   data,
                   by = NULL, ...) {
  # Build a Surv object from the named columns in the data frame.
  srv <- survival::Surv(time = data[[interval]], event = data[[censor]]) # nolint: object_usage_linter

  # Fit the Kaplan-Meier estimator; stratify on `by` when provided.
  if (is.null(by)) {
    srv_tab <- survival::survfit(srv ~ 1, ...)
  } else {
    srv_tab <-
      survival::survfit(srv ~ survival::strata(data[[by]]), ...)
  }

  # Cumulative hazard H(t) = -log(S(t)) via the Nelson-Aalen transform.
  cum_hazard <- -log(srv_tab$surv)

  # Collect per-time-point summary statistics into a flat data frame.
  tbl <- data.frame(
    cbind(
      time = srv_tab$time,
      n = srv_tab$n.risk,        # number at risk just before time t
      cens = srv_tab$n.censor,   # number censored at time t
      dead = srv_tab$n.event,    # number of events at time t
      surv = srv_tab$surv,       # KM survival estimate S(t)
      se = srv_tab$std.err,      # standard error of S(t)
      lower = srv_tab$lower,     # lower confidence bound
      upper = srv_tab$upper,     # upper confidence bound
      cum_haz = cum_hazard
    )
  )

  # When stratifying, stitch a "groups" label column onto the table.
  if (!is.null(by)) tbl <- .label_strata(tbl, data, by) # nolint: object_usage_linter

  # Keep only rows where at least one event occurred; censoring-only rows
  # do not contribute new KM estimates.
  gg_dta <- tbl[which(tbl[["dead"]] != 0), ]

  # Derived quantities computed from interval-based lagged differences.
  lag_s <- c(1, gg_dta$surv)[-(dim(gg_dta)[1] + 1)]
  lag_t <- c(0, gg_dta$time)[-(dim(gg_dta)[1] + 1)]

  delta_t <- gg_dta$time - lag_t
  # Conditional hazard rate approximation: h(t) ≈ -log(S(t)/S(t-)) / Δt
  hzrd <- log(lag_s / gg_dta$surv) / delta_t

  # Probability density: f(t) ≈ (S(t-) - S(t)) / Δt
  dnsty <- (lag_s - gg_dta$surv) / delta_t
  mid_int <- (gg_dta$time + lag_t) / 2
  lag_l <- 0

  # Cumulative expected life in each interval (trapezoidal rule):
  # L(t_i) = L(t_{i-1}) + (S(t_{i-1}) + S(t_i)) / 2 * Δt_i
  life <- vector("numeric", length = dim(gg_dta)[1])
  for (ind in seq_len(dim(gg_dta)[1])) {
    life[ind] <-
      lag_l + (lag_s[ind] + gg_dta[ind, "surv"]) / 2 * delta_t[ind]
    lag_l <- life[ind]
  }
  prp_life <- life / gg_dta$time
  gg_dta <- data.frame(
    cbind(
      gg_dta,
      hazard = hzrd,
      density = dnsty,
      mid_int = mid_int,
      life = life,
      proplife = prp_life
    )
  )

  class(gg_dta) <- c("gg_survival", class(gg_dta))
  invisible(gg_dta)
}

##=============================================================================
#' Plot a \code{gg_varpro} variable importance object
#'
#' Draws a horizontal boxplot of the per-tree importance z-scores, or of the
#' raw importances if you asked for those.  Set \code{faithful = TRUE} at
#' extract time and the per-tree points are scattered over the box; for a
#' classification forest, \code{conditional = TRUE} splits the plot into one
#' facet per class.
#'
#' @section Reading the boxplot:
#' Variables are sorted top to bottom by descending median per-tree
#' importance, so the eye lands on the most important variable first.
#' For each variable the box spans the 15th to 85th percentile of the
#' per-tree scores, the centre line is the median, and the whiskers run
#' out to the 5th and 95th percentile, not the usual Tukey 1.5 IQR
#' whiskers. The dashed vertical line is the selection \code{cutoff}
#' (default \code{0.79}). On the default z-score axis
#' (\code{local.std = TRUE}) that line is a z; on the raw-importance
#' axis (\code{local.std = FALSE}, \code{type = "raw"}) it is the same
#' numeric value but in raw-importance units. Boxes whose aggregate
#' value sits above the line are coloured blue and flagged
#' \code{selected = TRUE}, the rest are grey. A
#' selected variable with a tight, high box is a variable the forest
#' agrees on across trees. A selected variable with a wide box that
#' straddles the cutoff is one to look at twice before relying on it.
#'
#' With \code{faithful = TRUE} the box is drawn faint and the per-tree
#' values are jittered over it as semi-transparent points, on the same
#' scale as the box (z when \code{local.std = TRUE}, raw otherwise). A
#' white-outlined dot marks the mean. Use this view when you want to
#' see how individual trees voted rather than just the summary.
#'
#' For a classification forest with \code{conditional = TRUE} the plot
#' splits into one facet per class. Variables keep the unconditional
#' sort order, so the rows line up across facets and you can read
#' across to see which class a variable is informative for.
#'
#' @section What this tells you:
#' Take the variables above the cutoff as your candidate set. Use the
#' width of the box and the per-tree overlay to gauge confidence: a
#' narrow box well above the cutoff is a confident pick, a wide box
#' that crosses it is a coin flip you should not lean on. For
#' classification, conditional importance tells you which variables
#' drive which class; a variable that is unconditionally important but
#' only important for one class out of several is still useful, just
#' useful for a narrower question.
#'
#' @param x A \code{gg_varpro} object from \code{\link{gg_varpro}}.
#' @param type Character; the display scale.  Leave it off and it is read
#'   from \code{provenance$local.std}: \code{"z"} when \code{local.std =
#'   TRUE} (the default), \code{"raw"} when \code{local.std = FALSE}.
#'   Asking for a scale that the extract step did not prepare raises an
#'   error.
#' @param ... Not currently used.
#'
#' @details
#' **Boxplot geometry:** the hinges are the 15th and 85th percentiles of the
#' per-tree z-distribution, and the whiskers run to the 5th and 95th.  This
#' is \strong{not} a Tukey boxplot, and the plot carries a caption that says
#' so.
#'
#' **\code{faithful = TRUE}:** the per-tree values are jittered over the box
#' as semi-transparent points, on the same scale as the box itself (z when
#' \code{local.std = TRUE}, raw when \code{local.std = FALSE}).  The box is
#' drawn faint to let the points show through, and a white-outlined dot
#' marks the mean.
#'
#' **\code{conditional = TRUE}:** the class-conditional importances
#' (\code{$conditional}) are shown as a faceted bar chart
#' (\code{facet_wrap(~class, nrow = 1)}).  Variables keep the sort order set
#' by the unconditional median z in \code{$stats}, so the facets line up.
#'
#' @return A \code{ggplot} object.
#'
#' @seealso \code{\link{gg_varpro}}
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' vp <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
#' plot(gg_varpro(vp))
#' plot(gg_varpro(vp, faithful = TRUE))
#' }
#'
#' @name plot.gg_varpro
#' @importFrom ggplot2 ggplot aes geom_boxplot geom_jitter geom_point geom_col
#' @importFrom ggplot2 geom_vline geom_hline coord_flip facet_wrap
#' @importFrom ggplot2 scale_fill_manual labs theme_minimal
#' @importFrom tidyr pivot_longer
#' @export
plot.gg_varpro <- function(x, type, ...) {
  prov <- attr(x, "provenance")

  ## ---- Auto-detect type from provenance when not supplied ------------------
  if (missing(type)) {
    type <- if (isTRUE(prov$local.std)) "z" else "raw"
  } else {
    type <- match.arg(type, c("z", "raw"))
  }

  ## ---- Validate type vs. extract-time local.std ----------------------------
  if (identical(type, "raw") && isTRUE(prov$local.std)) {
    stop("type = 'raw' requires local.std = FALSE at gg_varpro() extract time.",
         call. = FALSE)
  }
  if (identical(type, "z") && !isTRUE(prov$local.std)) {
    stop("type = 'z' requires local.std = TRUE at gg_varpro() extract time.",
         call. = FALSE)
  }

  ## ---- Conditional view ---------------------------------------------------
  if (!is.null(x$conditional)) {
    if (!missing(type) && !identical(type, "z")) {
      message("Conditional plot ignores type = '", type,
              "'; showing class-conditional z scores.")
    }
    return(.plot_varpro_conditional(x, prov))
  }

  ## ---- Default / faithful view --------------------------------------------
  .plot_varpro_main(x, type, prov)
}

## ---- Internal renderers ----------------------------------------------------

#' @keywords internal
.plot_varpro_main <- function(x, type, prov) {
  stats_df <- x$stats
  # Merge selected flag
  sel_df <- unique(x$imp[, c("variable", "selected")])
  stats_df <- merge(stats_df, sel_df, by = "variable", all.x = TRUE)

  faithful  <- isTRUE(prov$faithful)
  cutoff    <- prov$cutoff %||% 0.79
  fill_vals <- c("TRUE" = "#4e8fcd", "FALSE" = "#888888")
  cap_text  <- paste0(
    "Hinges: 15th/85th percentiles; whiskers: 5th/95th. ",
    if (faithful) "Points show per-tree importance. " else "",
    "Not a Tukey boxplot."
  )

  p <- ggplot2::ggplot(stats_df,
         ggplot2::aes(x      = .data[["variable"]],
                      ymin   = .data[["q05"]],
                      lower  = .data[["q15"]],
                      middle = .data[["median"]],
                      upper  = .data[["q85"]],
                      ymax   = .data[["q95"]],
                      fill   = factor(.data[["selected"]]))) +
    ggplot2::geom_boxplot(stat = "identity",
                          alpha = if (faithful) 0.4 else 0.85) +
    ggplot2::coord_flip() +
    ggplot2::scale_fill_manual(values = fill_vals, guide = "none") +
    ggplot2::geom_hline(yintercept = cutoff,
                        linetype = "dashed", color = "#e74c3c", linewidth = 0.7) +
    ggplot2::labs(x = NULL,
                  y = .varpro_imp_ylabel(type, prov),
                  caption = cap_text) +
    ggplot2::theme_minimal()

  ## ---- faithful overlay ---------------------------------------------------
  if (faithful && !is.null(x$imp.tree)) {
    long_df <- as.data.frame(x$imp.tree)
    long_df$tree <- seq_len(nrow(x$imp.tree))
    long_df <- tidyr::pivot_longer(long_df, !"tree",
                                   names_to = "variable", values_to = "imp_raw")
    long_df$variable <- factor(long_df$variable, levels = levels(x$imp$variable))

    ## Overlay y-values match the box scale (type-aware) --------------------
    if (identical(type, "z")) {
      ## z_ij = imp_ij / sd_j  (same formula as .varpro_imp_stats)
      sd_j <- tapply(long_df$imp_raw, long_df$variable,
                     stats::sd, na.rm = TRUE)
      sd_j[sd_j < .Machine$double.eps] <- 1
      long_df$overlay  <- long_df$imp_raw / sd_j[long_df$variable]
      mean_df          <- stats_df
      mean_df$overlay_mean <- mean_df$mean / sd_j[as.character(mean_df$variable)]
    } else {
      ## raw scale: imp_raw as-is; mean column is already raw
      long_df$overlay      <- long_df$imp_raw
      mean_df              <- stats_df
      mean_df$overlay_mean <- mean_df$mean
    }

    p <- p +
      ggplot2::geom_jitter(data = long_df,
                           ggplot2::aes(x = .data[["variable"]],
                                        y = .data[["overlay"]]),
                           inherit.aes = FALSE,
                           width = 0.15, height = 0,  # width jitters in categorical band
                           alpha = 0.4, size = 1.2,
                           color = "#4e8fcd") +
      ggplot2::geom_point(data = mean_df,
                          ggplot2::aes(x = .data[["variable"]],
                                       y = .data[["overlay_mean"]]),
                          inherit.aes = FALSE,
                          shape = 21, fill = "white", color = "#4e8fcd",
                          size = 2.5)
  }

  p
}

#' @keywords internal
.plot_varpro_conditional <- function(x, prov) {
  ## Class-conditional z-scores as faceted bar chart.
  cond_df   <- x$conditional
  ## Replace NA/NaN z with 0 to suppress geom_col remove_missing warnings
  cond_df$z[!is.finite(cond_df$z)] <- 0
  cond_df$variable <- factor(cond_df$variable, levels = levels(x$imp$variable))
  cutoff    <- prov$cutoff %||% 0.79

  ggplot2::ggplot(cond_df,
                  ggplot2::aes(x    = .data[["variable"]],
                               y    = .data[["z"]],
                               fill = .data[["z"]] > cutoff)) +
    ggplot2::geom_col() +
    ggplot2::coord_flip() +
    ggplot2::facet_wrap(~ class, nrow = 1L) +
    ggplot2::scale_fill_manual(
      values = c("TRUE" = "#4e8fcd", "FALSE" = "#888888"),
      guide  = "none"
    ) +
    ggplot2::geom_hline(yintercept = cutoff,
                        linetype = "dashed", color = "#e74c3c",
                        linewidth = 0.7) +
    ggplot2::labs(x = NULL,
                  y = "Variable importance (z)",
                  caption = paste0("Dashed line at z = ", cutoff,
                                   ". Conditional class importance.")) +
    ggplot2::theme_minimal()
}

#' @keywords internal
.varpro_imp_ylabel <- function(type, prov) {
  if (identical(type, "raw")) {
    "Variable importance"
  } else {
    "Variable importance (z)"
  }
}

####**********************************************************************
####**********************************************************************
####  ----------------------------------------------------------------
####  Written by:
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Marginal variable dependence data object.
#'
#' @details The marginal variable dependence is determined by comparing
#' relation between the predicted response from the randomForest and a
#' covariate of interest.
#'
#' The \code{gg_variable} function operates on a
#' \code{\link[randomForestSRC]{rfsrc}} object, the output from the
#' \code{\link[randomForestSRC]{plot.variable}} function, or on a fitted
#' \code{\link[randomForest]{randomForest}} object via the formula interface.
#'
#' @description \code{\link[randomForestSRC]{plot.variable}} generates a
#' \code{data.frame} containing the marginal variable dependence or the
#' partial variable dependence. The \code{gg_variable} function creates a
#' \code{data.frame} of containing the full set of covariate data (predictor
#' variables) and the predicted response for each observation. Marginal
#' dependence figures are created using the \code{\link{plot.gg_variable}}
#' function.
#' A \code{randomForest} fit does not keep the model frame, so for those
#' objects \code{gg_variable} rebuilds it from the stored call. That lets the
#' same predictors be paired with the in-sample predictions.
#'
#' A few optional arguments tune the extraction: \code{time} (one survival
#' time, or a vector of them), \code{time_labels} (labels for multiple
#' survival horizons), and \code{oob}, which switches between out-of-bag and
#' in-bag predictions when the forest carries both.
#'
#' @param object A \code{\link[randomForestSRC]{rfsrc}} or
#'   \code{\link[randomForest]{randomForest}} object, or a
#'   \code{\link[randomForestSRC]{plot.variable}} result.
#' @param ... Optional arguments \code{time}, \code{time_labels}, and
#'   \code{oob} that tune the marginal dependence extraction.
#'
#' @return A \code{gg_variable} object: a \code{data.frame} pairing every
#'   training predictor column with the OOB (or in-bag) predicted response.
#'   For survival forests, each requested time horizon adds a column named by
#'   \code{time_labels}. The object carries a \code{"family"} class attribute
#'   (\code{"regr"}, \code{"class"}, or \code{"surv"}) that
#'   \code{\link{plot.gg_variable}} uses for dispatch.
#'
#' @seealso \code{\link{plot.gg_variable}},
#'   \code{\link[randomForestSRC]{plot.variable}}
#'
#' @aliases gg_variable gg_variable.rfsrc
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## classification (small, runs on CRAN)
#' ## ------------------------------------------------------------
#' ## -------- iris data
#' set.seed(42)
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_iris)
#' plot(gg_dta, xvar = "Sepal.Width")
#'
#' \donttest{
#' ## ------------------------------------------------------------
#' ## Additional classification / regression / survival examples are
#' ## guarded with \donttest because the cumulative example time exceeds
#' ## the 10-second CRAN budget. Run locally with `R CMD check
#' ## --run-donttest` (or `devtools::check(run_dont_test = TRUE)`) to
#' ## exercise them.
#' ## ------------------------------------------------------------
#' plot(gg_dta, xvar = "Sepal.Length")
#' plot(gg_dta, xvar = rfsrc_iris$xvar.names, panel = TRUE)
#'
#' ## ------------------------------------------------------------
#' ## regression
#' ## ------------------------------------------------------------
#'
#' ## -------- air quality data
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality, ntree = 50)
#' gg_dta <- gg_variable(rfsrc_airq)
#'
#' # an ordinal variable
#' gg_dta[, "Month"] <- factor(gg_dta[, "Month"])
#'
#' plot(gg_dta, xvar = "Wind")
#' plot(gg_dta, xvar = "Temp")
#' plot(gg_dta, xvar = "Solar.R")
#' plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
#' plot(gg_dta, xvar = "Month", notch = TRUE)
#'
#' ## -------- motor trend cars data
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)
#'
#' gg_dta <- gg_variable(rfsrc_mtcars)
#'
#' # mtcars$cyl is an ordinal variable
#' gg_dta$cyl <- factor(gg_dta$cyl)
#' gg_dta$am <- factor(gg_dta$am)
#' gg_dta$vs <- factor(gg_dta$vs)
#' gg_dta$gear <- factor(gg_dta$gear)
#' gg_dta$carb <- factor(gg_dta$carb)
#'
#' plot(gg_dta, xvar = "cyl")
#' plot(gg_dta, xvar = "disp")
#' plot(gg_dta, xvar = "hp")
#' plot(gg_dta, xvar = "wt")
#' plot(gg_dta, xvar = c("disp", "hp", "drat", "wt", "qsec"), panel = TRUE)
#' plot(gg_dta,
#'   xvar = c("cyl", "vs", "am", "gear", "carb"), panel = TRUE,
#'   notch = TRUE
#' )
#'
#' ## -------- Boston data
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'   data(Boston, package = "MASS")
#'   rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
#'   gg_dta <- gg_variable(rf_boston)
#'   plot(gg_dta)
#'   plot(gg_dta, panel = TRUE)
#' }
#'
#' ## ------------------------------------------------------------
#' ## survival examples
#' ## ------------------------------------------------------------
#'
#' ## -------- veteran data
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., veteran,
#'   nsplit = 10,
#'   ntree = 50
#' )
#'
#' # get the 90-day survival time.
#' gg_dta <- gg_variable(rfsrc_veteran, time = 90)
#'
#' # Generate variable dependence plots for age and diagtime
#' plot(gg_dta, xvar = "age")
#' plot(gg_dta, xvar = "diagtime")
#'
#' # Generate coplots
#' plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE, se = FALSE)
#'
#' # Compare survival at 30, 90, and 365 days simultaneously
#' gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
#' plot(gg_dta, xvar = "age")
#' }
#'
#' @aliases gg_variable gg_variable.rfsrc gg_variable.randomForest
#' @aliases gg_variable.random
#' @importFrom stats median
#'
#' @export
gg_variable <- function(object, ...) {
  UseMethod("gg_variable", object)
}
#' @export
gg_variable.rfsrc <- function(object,
                              ...) {
  # Collect optional arguments: time, time_labels, oob
  arg_list <- list(...)

  time <- arg_list$time
  time_labels <- arg_list$time_labels
  # Default to OOB predictions; callers can override with oob = FALSE to use
  # full-forest (in-bag) predictions instead.
  oob <- if (!is.null(arg_list$oob)) {
    arg_list$oob
  } else {
    TRUE
  }

  if (!inherits(object, "rfsrc")) {
    stop("gg_variable expects a rfsrc or plot.variable object.")
  }

  # Start with the predictor (xvar) data frame; we will append the predicted
  # response column(s) below depending on the forest family.
  gg_dta <- data.frame(object$xvar)

  ## ---- Regression branch ---------------------------------------------------
  if (object$family == "regr") {
    # Append a single "yhat" column containing the OOB (or full) predictions.
    if (oob) {
      gg_dta$yhat <- object$predicted.oob
    } else {
      gg_dta$yhat <- object$predicted
    }

  ## ---- Classification branch -----------------------------------------------
  } else if (object$family == "class") {
    # Append one "yhat.<class>" probability column per outcome class so the
    # caller can choose which class to display on the variable plot.
    preds <- if (oob) {
      object$predicted.oob
    } else {
      object$predicted
    }
    colnames(preds) <-
      paste("yhat.", colnames(preds),
        sep = ""
      )
    gg_dta <- cbind(gg_dta, preds)
    gg_dta$yvar <- object$yvar

  ## ---- Survival branch -----------------------------------------------------
  } else if (object$family == "surv") {
    # Survival forests predict a matrix (observations × time points); we need
    # to slice it at one or more user-specified time horizons.
    gg_dta$event <- as.logical(object$yvar[, 2])
    colnames(gg_dta) <- c(object$xvar.names, "event")

    # Default to the median observed event time when no time is specified.
    if (is.null(time)) {
      time <- median(object$time.interest)
    }
    lng <- length(time)

    for (ind in seq_len(lng)) {
      if (ind > 1) {
        gg_dta_t_old <- gg_dta_t
      }
      # Find the largest recorded time point that is <= the requested horizon.
      # Survival is a step function, so we read the value at the last eligible
      # recorded time point.
      gg_dta_t <- gg_dta
      eligible <- which(object$time.interest <= time[ind])
      if (!length(eligible)) {
        stop(
          "The time of interest is less than the first event time.\n",
          "  Make sure you are using the correct time units."
        )
      }
      in_time <- max(eligible)

      if (oob) {
        gg_dta_t$yhat <- object$survival.oob[, in_time]
      } else {
        gg_dta_t$yhat <- object$survival[, in_time]
      }

      # Attach either the numeric time value or a user-supplied label.
      if (is.null(time_labels)) {
        gg_dta_t$time <- time[ind]
      } else {
        gg_dta_t$time <- time_labels[ind]
      }

      # Stack time-slice data frames for multi-horizon plots.
      if (ind > 1) {
        gg_dta_t <- rbind(gg_dta_t_old, gg_dta_t)
      }
    }

    gg_dta <- gg_dta_t
    # Preserve the ordering of time levels in the factor for downstream plots.
    gg_dta$time <- factor(gg_dta$time, levels = unique(gg_dta$time))
  }
  class(gg_dta) <- c("gg_variable", object$family, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

#' @export
gg_variable.randomForest <- function(object,
                                     ...) {
  arg_list <- list(...)

  # randomForest uses object$votes (OOB vote matrix) unconditionally; it is the
  # only honest per-class probability estimate.  In-bag class probabilities are
  # not exposed through a consistent randomForest API, so oob=FALSE is not
  # supported.  Warn the caller rather than silently ignoring the argument.
  if (!is.null(arg_list$oob) && identical(arg_list$oob, FALSE)) {
    warning(
      "oob = FALSE is not supported for randomForest objects: ",
      "in-bag class probabilities are unavailable. ",
      "OOB vote fractions (object$votes) will be used instead."
    )
  }

  if (!inherits(object, "randomForest")) {
    stop("gg_variable expects a randomForest object.")
  }

  # Reconstruct the training data from the stored call so we can pair
  # predictions with the original predictors.
  training_info <- .rf_recover_model_frame(object) # nolint: object_usage_linter
  if (is.null(training_info)) {
    stop(
      "Unable to reconstruct the training data for this randomForest object.",
      " Make sure the model was fit with a formula interface and keep.forest = TRUE."
    )
  }

  model_frame <- training_info$model_frame
  response <- stats::model.response(model_frame)
  resp_name <- training_info$response_name

  # Drop the response column and any special model-frame columns (e.g. offset)
  # to leave only the predictor columns.
  predictors <- model_frame
  if (!is.null(resp_name) && resp_name %in% names(predictors)) {
    predictors[[resp_name]] <- NULL
  }
  special_cols <- grep("^\\(", colnames(predictors), value = TRUE)
  if (length(special_cols) > 0) {
    predictors[special_cols] <- NULL
  }

  gg_dta <- predictors
  # For classification forests use per-class OOB vote fractions (object$votes),
  # stored as yhat.<classname> columns: the same shape gg_variable.rfsrc
  # produces.  For regression a single numeric yhat column suffices.
  if (object$type == "classification") {
    preds    <- object$votes   # n × n_classes matrix; may be raw counts or fractions
    rs       <- rowSums(preds)
    if (any(rs > 1 + 1e-8, na.rm = TRUE)) {
      preds  <- preds / rs   # normalise raw vote counts to [0, 1]
    }
    colnames(preds) <- paste0("yhat.", colnames(preds))
    gg_dta          <- cbind(gg_dta, preds)
    gg_dta$yvar     <- response
  } else {
    gg_dta$yhat <- as.vector(object$predicted)
  }

  # randomForest uses object$type ("classification" / "regression"); the
  # plot.gg_variable dispatcher and the rfsrc path both use "class" for
  # classification forests.  Map here so the class attribute is consistent
  # and callers never need to special-case "classification".
  family_lbl <- if (object$type == "classification") "class" else object$type
  class(gg_dta) <- c("gg_variable", family_lbl, class(gg_dta))
  gg_dta <- .set_provenance(gg_dta, object)
  invisible(gg_dta)
}

# Session-memoised varpro + beta.varpro fixtures for the gg_beta_varpro tests.
# beta.varpro() is the expensive call (per-rule glmnet); compute once per R
# session and reuse. In-memory only — no disk cache.
#
# These all grow *supervised* varpro/ivarpro/beta.varpro forests (a real Y), so
# yvar.wt is non-empty and they do NOT trip randomForestSRC's gcc-UBSAN report
# at entry.c:184 (that fires only for the unsupervised family — verified under
# -fsanitize=undefined). They intentionally run on CRAN; do not skip_on_cran().
# Only the unsupervised isopro grow is skipped (see test_gg_isopro.R).

.varpro_cache <- new.env(parent = emptyenv())

.varpro_mtcars <- function() {
  if (is.null(.varpro_cache$v)) {
    if (!requireNamespace("varPro", quietly = TRUE)) {
      testthat::skip("varPro not installed")
    }
    set.seed(20260526L)
    .varpro_cache$v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
  }
  .varpro_cache$v
}

.beta_fit_mtcars <- function() {
  if (is.null(.varpro_cache$b)) {
    v <- .varpro_mtcars()
    set.seed(20260526L)
    .varpro_cache$b <- varPro::beta.varpro(v)
  }
  .varpro_cache$b
}

.varpro_iris_binary <- function() {
  if (is.null(.varpro_cache$vb)) {
    if (!requireNamespace("varPro", quietly = TRUE)) testthat::skip("varPro not installed")
    set.seed(20260526L)
    d <- iris[iris$Species != "setosa", ]
    d$Species <- droplevels(d$Species)
    .varpro_cache$vb <- varPro::varpro(Species ~ ., data = d, ntree = 30)
  }
  .varpro_cache$vb
}

.beta_fit_iris_binary <- function() {
  if (is.null(.varpro_cache$bb)) {
    set.seed(20260526L)
    .varpro_cache$bb <- varPro::beta.varpro(.varpro_iris_binary())
  }
  .varpro_cache$bb
}

.varpro_iris_multiclass <- function() {
  if (is.null(.varpro_cache$vm)) {
    if (!requireNamespace("varPro", quietly = TRUE)) testthat::skip("varPro not installed")
    set.seed(20260526L)
    .varpro_cache$vm <- varPro::varpro(Species ~ ., data = iris, ntree = 30)
  }
  .varpro_cache$vm
}

.beta_fit_iris_multiclass <- function() {
  if (is.null(.varpro_cache$bm)) {
    set.seed(20260526L)
    .varpro_cache$bm <- varPro::beta.varpro(.varpro_iris_multiclass())
  }
  .varpro_cache$bm
}

.ivarpro_boston <- function() {
  if (is.null(.varpro_cache$iv_boston)) {
    if (!requireNamespace("varPro", quietly = TRUE)) testthat::skip("varPro not installed")
    if (!requireNamespace("MASS", quietly = TRUE))   testthat::skip("MASS not installed")
    set.seed(20260526L)
    v <- varPro::varpro(medv ~ ., data = MASS::Boston, ntree = 50)
    .varpro_cache$v_boston <- v
    .varpro_cache$iv_boston <- varPro::ivarpro(v)
  }
  .varpro_cache$iv_boston
}

.varpro_boston <- function() {
  if (is.null(.varpro_cache$v_boston)) {
    invisible(.ivarpro_boston())   # populates v_boston as a side-effect
  }
  .varpro_cache$v_boston
}

.ivarpro_iris_binary <- function() {
  if (is.null(.varpro_cache$iv_iris_binary)) {
    if (!requireNamespace("varPro", quietly = TRUE)) testthat::skip("varPro not installed")
    set.seed(20260526L)
    d <- iris[iris$Species != "setosa", ]
    d$Species <- droplevels(d$Species)
    v <- varPro::varpro(Species ~ ., data = d, ntree = 50)
    .varpro_cache$v_iris_binary <- v
    .varpro_cache$iv_iris_binary <- varPro::ivarpro(v)
  }
  .varpro_cache$iv_iris_binary
}

.varpro_iris_binary_for_ivarpro <- function() {
  if (is.null(.varpro_cache$v_iris_binary)) invisible(.ivarpro_iris_binary())
  .varpro_cache$v_iris_binary
}

.ivarpro_iris_multiclass <- function() {
  if (is.null(.varpro_cache$iv_iris_multi)) {
    if (!requireNamespace("varPro", quietly = TRUE)) testthat::skip("varPro not installed")
    set.seed(20260526L)
    v <- varPro::varpro(Species ~ ., data = iris, ntree = 50)
    .varpro_cache$v_iris_multi <- v
    .varpro_cache$iv_iris_multi <- varPro::ivarpro(v)
  }
  .varpro_cache$iv_iris_multi
}

.varpro_iris_multiclass_for_ivarpro <- function() {
  if (is.null(.varpro_cache$v_iris_multi)) invisible(.ivarpro_iris_multiclass())
  .varpro_cache$v_iris_multi
}

test_that("gg_beta_varpro returns the expected tidy shape", {
  b <- .beta_fit_mtcars()
  v <- .varpro_mtcars()

  out <- gg_beta_varpro(v, beta_fit = b)

  expect_s3_class(out, "gg_beta_varpro")
  expect_s3_class(out, "data.frame")
  expect_setequal(names(out),
                  c("variable", "beta_mean", "n_rules", "selected"))
  released <- unique(b$results$variable[is.finite(b$results$imp)])
  expect_equal(nrow(out), length(released))
})

test_that("gg_beta_varpro aggregation equals mean(|imp|) per variable", {
  b <- .beta_fit_mtcars()
  v <- .varpro_mtcars()
  out <- gg_beta_varpro(v, beta_fit = b)

  res <- b$results
  res <- res[is.finite(res$imp), , drop = FALSE]
  for (i in seq_len(min(2L, nrow(out)))) {
    var_name <- as.character(out$variable[i])
    var_idx  <- match(var_name, b$xvar.names)
    expected <- mean(abs(res$imp[res$variable == var_idx]))
    expect_equal(out$beta_mean[i], expected, tolerance = 1e-10,
                 label = paste0("beta_mean for ", var_name))
  }
})

test_that("gg_beta_varpro now accepts classification family (regr + class supported)", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  expect_s3_class(out, "gg_beta_varpro")
  expect_equal(attr(out, "provenance")$family, "class")
})

test_that("gg_beta_varpro cutoff = 0 selects everything, Inf selects nothing", {
  b <- .beta_fit_mtcars()
  v <- .varpro_mtcars()

  zero_cut <- gg_beta_varpro(v, beta_fit = b, cutoff = 0)
  expect_true(all(zero_cut$selected))

  inf_cut <- gg_beta_varpro(v, beta_fit = b, cutoff = Inf)
  expect_true(!any(inf_cut$selected))
})

test_that("gg_beta_varpro default cutoff is mean(beta_mean), provenance flagged", {
  b <- .beta_fit_mtcars()
  v <- .varpro_mtcars()

  out <- gg_beta_varpro(v, beta_fit = b)
  prov <- attr(out, "provenance")

  expect_equal(prov$cutoff[["regr"]], mean(out$beta_mean), tolerance = 1e-10)
  expect_true(prov$cutoff_default)

  explicit <- gg_beta_varpro(v, beta_fit = b, cutoff = 0.123)
  expect_false(attr(explicit, "provenance")$cutoff_default)
  expect_equal(attr(explicit, "provenance")$cutoff[["regr"]], 0.123)
})

test_that("gg_beta_varpro rejects malformed beta_fit", {
  v <- .varpro_mtcars()
  expect_error(
    gg_beta_varpro(v, beta_fit = list()),
    "beta_fit does not look like a varPro::beta.varpro\\(\\) result"
  )

  b <- .beta_fit_mtcars()
  b_bad <- b
  b_bad$results <- b_bad$results[, setdiff(names(b_bad$results), "imp"), drop = FALSE]
  expect_error(
    gg_beta_varpro(v, beta_fit = b_bad),
    "imp"
  )
})

test_that("gg_beta_varpro warns when ... is supplied alongside beta_fit", {
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  expect_warning(
    out <- gg_beta_varpro(v, use.cv = TRUE, beta_fit = b),
    "ignored because beta_fit is supplied"
  )
  expect_s3_class(out, "gg_beta_varpro")
})

test_that("gg_beta_varpro returns empty frame when beta.varpro yields no rules", {
  v <- .varpro_mtcars()
  empty <- structure(
    list(results = data.frame(tree = integer(0), branch = integer(0),
                              variable = integer(0), n.oob = integer(0),
                              imp = numeric(0)),
         xvar.names = v$xvar.names),
    class = "varpro"
  )
  out <- gg_beta_varpro(v, beta_fit = empty)
  expect_s3_class(out, "gg_beta_varpro")
  expect_equal(nrow(out), 0L)
  expect_setequal(names(out), c("variable", "beta_mean", "n_rules", "selected"))
  expect_equal(attr(out, "provenance")$n_rules_total, 0L)
})

test_that("gg_beta_varpro counts lasso-shrunk-to-zero rules in n_rules", {
  v <- .varpro_mtcars()
  fake <- structure(
    list(
      results = data.frame(
        tree     = c(1L, 1L, 2L, 2L),
        branch   = c(1L, 2L, 1L, 2L),
        variable = c(1L, 1L, 1L, 1L),
        n.oob    = c(5L, 5L, 5L, 5L),
        imp      = c(0.0, 0.0, 1.5, -2.5)
      ),
      xvar.names = v$xvar.names
    ),
    class = "varpro"
  )
  out <- gg_beta_varpro(v, beta_fit = fake)
  expect_equal(nrow(out), 1L)
  expect_equal(out$n_rules, 4L)
  # mean(|0|, |0|, |1.5|, |-2.5|) = 1.0
  expect_equal(out$beta_mean, 1.0, tolerance = 1e-10)
})

test_that("gg_beta_varpro cached and uncached paths agree", {
  if (!identical(Sys.getenv("GG_BETA_VARPRO_SLOW_TESTS", "true"), "true")) {
    skip("Slow test — set GG_BETA_VARPRO_SLOW_TESTS=true to run")
  }
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()

  set.seed(20260526L)
  uncached <- gg_beta_varpro(v)
  cached   <- gg_beta_varpro(v, beta_fit = b)

  # Strip provenance before comparing the tidy-frame content — `precomputed`
  # differs (TRUE vs FALSE) between the two paths by design, and that flag is
  # checked separately below.
  strip <- function(x) {
    attr(x, "provenance") <- NULL
    as.data.frame(x)
  }
  expect_equal(
    strip(uncached),
    strip(cached),
    tolerance = 1e-10
  )
  expect_false(attr(uncached, "provenance")$precomputed)
  expect_true(attr(cached, "provenance")$precomputed)
})

test_that("plot.gg_beta_varpro returns a ggplot that builds", {
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  out <- gg_beta_varpro(v, beta_fit = b)

  p <- plot(out)
  expect_s3_class(p, "ggplot")
  built <- ggplot2::ggplot_build(p)
  expect_s3_class(built, "ggplot_built")
  expect_true(length(built$data) >= 1L)
})

test_that("gg_beta_varpro S3 companions return the expected shapes", {
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  out <- gg_beta_varpro(v, beta_fit = b)

  # print invisibly returns x
  pr <- withVisible(print(out))
  expect_false(pr$visible)
  expect_identical(pr$value, out)

  # summary returns a named numeric, sorted descending, attribute n_rules present
  s <- summary(out)
  expect_s3_class(s, "summary.gg_beta_varpro")
  expect_type(unclass(s), "double")
  # as.numeric() strips both names and the n_rules attr so the comparison is
  # value-only; sort() drops the n_rules attr and would otherwise cause a
  # spurious attribute-mismatch failure.
  vals <- as.numeric(unclass(s))
  expect_equal(vals, sort(vals, decreasing = TRUE))
  expect_equal(length(s), nrow(out))
  expect_true(!is.null(attr(s, "n_rules")))

  # autoplot delegates to plot
  p1 <- plot(out)
  p2 <- ggplot2::autoplot(out)
  expect_equal(
    ggplot2::ggplot_build(p1)$data,
    ggplot2::ggplot_build(p2)$data
  )
})

# ---- Classification: shape + factor ordering --------------------------------

test_that("gg_beta_varpro binary classification returns long-format with class", {
  vb <- .varpro_iris_binary()
  bb <- .beta_fit_iris_binary()
  out <- gg_beta_varpro(vb, beta_fit = bb)
  expect_s3_class(out, "gg_beta_varpro")
  expect_true("class" %in% names(out))
  # Binary default: which_class = last factor level (positive class)
  expect_equal(as.character(unique(out$class)),
               tail(levels(droplevels(iris$Species[iris$Species != "setosa"])), 1))
})

test_that("gg_beta_varpro multi-class returns all classes faceted by default", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  expect_true("class" %in% names(out))
  expect_setequal(as.character(unique(out$class)), levels(iris$Species))
})

test_that("gg_beta_varpro variable is a factor ordered by total-|imp| descending", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  expect_true(is.factor(out$variable))

  # Compute expected order independently
  res <- bm$results
  res <- res[is.finite(res$imp), , drop = FALSE]
  var_name <- bm$xvar.names[res$variable]
  agg <- vapply(split(abs(res$imp), var_name), mean, numeric(1))
  # Factor levels are reversed-descending so the most-important variable is
  # the LAST level (top of the plot after coord_flip).
  expected_levels <- rev(names(sort(agg, decreasing = TRUE)))
  expect_equal(levels(out$variable), expected_levels)
})

# ---- which_class semantics --------------------------------------------------

test_that("gg_beta_varpro which_class = NULL on binary resolves to last factor level", {
  vb <- .varpro_iris_binary()
  bb <- .beta_fit_iris_binary()
  out <- gg_beta_varpro(vb, beta_fit = bb)
  prov <- attr(out, "provenance")
  expect_equal(prov$which_class, "virginica")
})

test_that("gg_beta_varpro which_class explicit returns single panel", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm, which_class = "setosa")
  expect_equal(as.character(unique(out$class)), "setosa")
  expect_equal(attr(out, "provenance")$which_class, "setosa")
})

test_that("gg_beta_varpro which_class not in levels errors with levels listed", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  expect_error(
    gg_beta_varpro(vm, beta_fit = bm, which_class = "nope"),
    "is not a level of the response"
  )
})

# ---- Aggregation correctness ------------------------------------------------

test_that("gg_beta_varpro classification beta_mean equals mean(|imp.k|) per (var, class)", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)

  # Pick the first (variable, class) pair
  first_var   <- as.character(out$variable[1])
  first_class <- as.character(out$class[1])
  k_idx       <- match(first_class, levels(vm$y))
  imp_col     <- paste0("imp.", k_idx)
  v_idx       <- match(first_var, bm$xvar.names)

  vals     <- bm$results[[imp_col]][bm$results$variable == v_idx]
  expected <- mean(abs(vals[is.finite(vals)]))
  expect_equal(out$beta_mean[1], expected, tolerance = 1e-10)
})

# ---- cutoff polymorphism ----------------------------------------------------

test_that("gg_beta_varpro cutoff = NULL gives per-class mean", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  prov <- attr(out, "provenance")
  expect_type(prov$cutoff, "double")
  expect_named(prov$cutoff, levels(iris$Species))
  for (cls in levels(iris$Species)) {
    expect_equal(
      prov$cutoff[[cls]],
      mean(out$beta_mean[out$class == cls]),
      tolerance = 1e-10
    )
  }
  expect_true(prov$cutoff_default)
})

test_that("gg_beta_varpro cutoff = scalar broadcasts across classes", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm, cutoff = 0.5)
  prov <- attr(out, "provenance")
  expect_equal(unname(prov$cutoff), rep(0.5, 3))
  expect_named(prov$cutoff, levels(iris$Species))
  expect_false(prov$cutoff_default)
})

test_that("gg_beta_varpro cutoff = named vector respects per-class, falls back to mean", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm,
                       cutoff = c("setosa" = 1.5))   # only one named; others default
  prov <- attr(out, "provenance")
  expect_equal(prov$cutoff[["setosa"]], 1.5)
  expect_equal(prov$cutoff[["versicolor"]],
               mean(out$beta_mean[out$class == "versicolor"]), tolerance = 1e-10)
})

test_that("gg_beta_varpro cutoff = named vector with unknown name errors", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  expect_error(
    gg_beta_varpro(vm, beta_fit = bm, cutoff = c("bogus" = 0.5)),
    "is not a level of the response"
  )
})

# ---- Phase 4c provenance shape update --------------------------------------

test_that("gg_beta_varpro regression cutoff is now a length-1 named numeric", {
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  out <- gg_beta_varpro(v, beta_fit = b)
  prov <- attr(out, "provenance")
  expect_named(prov$cutoff, "regr")
  expect_length(prov$cutoff, 1L)
  expect_equal(prov$cutoff[["regr"]], mean(out$beta_mean), tolerance = 1e-10)
})

# ---- which_class warning on regression --------------------------------------

test_that("gg_beta_varpro which_class on regression warns and is ignored", {
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  expect_warning(
    out <- gg_beta_varpro(v, beta_fit = b, which_class = "anything"),
    "ignored for regression family"
  )
  expect_s3_class(out, "gg_beta_varpro")
  expect_false("class" %in% names(out))
})

# ---- Plot: binary single panel + multi-class faceted -----------------------

test_that("plot.gg_beta_varpro binary builds (single panel)", {
  vb <- .varpro_iris_binary()
  bb <- .beta_fit_iris_binary()
  out <- gg_beta_varpro(vb, beta_fit = bb)
  p <- plot(out)
  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("plot.gg_beta_varpro multi-class builds (faceted) with per-class cutoff lines", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  p <- plot(out)
  expect_s3_class(p, "ggplot")
  built <- ggplot2::ggplot_build(p)
  # facet_wrap engaged -> multiple panels in layout
  expect_true(nrow(built$layout$layout) >= 2L)
})

# ---- Print + summary: classification ---------------------------------------

test_that("print.gg_beta_varpro classification surfaces class info", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  txt <- utils::capture.output(print(out))
  expect_true(any(grepl("n_classes", txt) | grepl("class", txt)))
  expect_true(any(grepl("faceted", txt)))
})

test_that("summary.gg_beta_varpro classification returns per-class list", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm)
  s <- summary(out)
  expect_s3_class(s, "summary.gg_beta_varpro")
  expect_true(is.list(unclass(s)))
  expect_setequal(names(unclass(s)), levels(iris$Species))
})

# ---- Copilot review fixes (PR #98) -----------------------------------------

test_that("summary on a which_class-filtered classification only lists that class", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  out <- gg_beta_varpro(vm, beta_fit = bm, which_class = "setosa")
  s <- summary(out)
  expect_equal(names(unclass(s)), "setosa")
})

test_that("beta_fit shape guard catches missing per-class imp.<k> columns", {
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  bm_bad <- bm
  bm_bad$results <- bm_bad$results[, setdiff(names(bm_bad$results), "imp.2"),
                                   drop = FALSE]
  expect_error(
    gg_beta_varpro(vm, beta_fit = bm_bad),
    "Missing per-class column"
  )
})

test_that("empty classification fast-path returns named cutoff vector", {
  vm <- .varpro_iris_multiclass()
  empty_bm <- structure(
    list(
      results = data.frame(
        tree     = integer(0), branch = integer(0),
        variable = integer(0), n.oob  = integer(0),
        imp      = numeric(0),
        imp.1    = numeric(0), imp.2 = numeric(0), imp.3 = numeric(0)
      ),
      xvar.names = vm$xvar.names
    ),
    class = "varpro"
  )
  out <- gg_beta_varpro(vm, beta_fit = empty_bm)
  expect_s3_class(out, "gg_beta_varpro")
  expect_equal(nrow(out), 0L)
  prov <- attr(out, "provenance")
  expect_named(prov$cutoff, levels(iris$Species))
  expect_true(all(is.na(prov$cutoff)))
  expect_equal(prov$class_levels, levels(iris$Species))
})

# Visual regression tests using vdiffr.
#
# These tests generate reference SVGs on the first run and compare on
# subsequent runs. To regenerate snapshots (e.g. after an intentional
# visual change) run: vdiffr::manage_cases()
#
# All models are built with set.seed() to ensure reproducible plots.

# Guard: only register snapshot tests when explicitly opted in AND vdiffr is
# available. Set VDIFFR_RUN_TESTS=true to generate or compare visual baselines.
# This avoids failures on fresh checkouts (no _snaps/ directory) and in CI,
# and avoids a top-level skip() that would appear as an empty test in reports.
#
# To generate baselines locally:
#   1. Run Sys.setenv(VDIFFR_RUN_TESTS = "true")
#   2. Run devtools::test(filter = "snapshots")
#   3. Call testthat::snapshot_accept()
#   4. Commit tests/testthat/_snaps/ to the repo
if (requireNamespace("vdiffr", quietly = TRUE) &&
    identical(Sys.getenv("VDIFFR_RUN_TESTS"), "true")) {

## ---- Shared fixtures -------------------------------------------------------

# Classification — iris
local({
  set.seed(42L)
  rfsrc_iris <- randomForestSRC::rfsrc(
    Species ~ .,
    data = iris,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp classification", {
    gg_dta <- gg_vimp(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_vimp classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error classification", {
    gg_dta <- gg_error(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_error classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_roc classification rfsrc", {
    gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1L)
    vdiffr::expect_doppelganger("gg_roc classification rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc classification", {
    gg_dta <- gg_rfsrc(rfsrc_iris)
    vdiffr::expect_doppelganger("gg_rfsrc classification rfsrc", plot(gg_dta))
  })
})

# Regression — Boston housing
local({
  data(Boston, package = "MASS")
  boston <- Boston
  boston$chas <- as.logical(boston$chas)

  set.seed(42L)
  rfsrc_boston <- randomForestSRC::rfsrc(
    medv ~ .,
    data = boston,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp regression", {
    gg_dta <- gg_vimp(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_vimp regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error regression", {
    gg_dta <- gg_error(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_error regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc regression", {
    gg_dta <- gg_rfsrc(rfsrc_boston)
    vdiffr::expect_doppelganger("gg_rfsrc regression rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_variable regression single xvar", {
    gg_rfsrc_dta <- gg_rfsrc(rfsrc_boston)
    xvar <- "lstat"
    gg_dta <- gg_variable(rfsrc_boston, time = NULL)
    vdiffr::expect_doppelganger(
      "gg_variable regression lstat",
      plot(gg_dta, xvar = xvar)
    )
  })
})

# Survival — pbc
local({
  data(pbc, package = "randomForestSRC")
  pbc$time <- pbc$days / 364.25
  pbc_sub <- pbc[, c("time", "status", "treatment", "age", "bili", "albumin")]

  set.seed(42L)
  rfsrc_pbc <- randomForestSRC::rfsrc(
    Surv(time, status) ~ .,
    data = pbc_sub,
    importance = TRUE,
    tree.err = TRUE,
    ntree = 100L
  )

  test_that("snapshot: gg_vimp survival", {
    gg_dta <- gg_vimp(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_vimp survival rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_error survival", {
    gg_dta <- gg_error(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_error survival rfsrc", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc survival no CI", {
    gg_dta <- gg_rfsrc(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_rfsrc survival no ci", plot(gg_dta))
  })

  test_that("snapshot: gg_rfsrc survival with bootstrap CI", {
    set.seed(1L)
    gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = 0.95, bs_samples = 50L)
    vdiffr::expect_doppelganger("gg_rfsrc survival bootstrap ci", plot(gg_dta))
  })

  test_that("snapshot: gg_brier overall line", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival overall", plot(gg_dta))
  })

  test_that("snapshot: gg_brier with envelope ribbon", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival envelope",
                                plot(gg_dta, envelope = TRUE))
  })

  test_that("snapshot: gg_brier CRPS", {
    gg_dta <- gg_brier(rfsrc_pbc)
    vdiffr::expect_doppelganger("gg_brier survival crps",
                                plot(gg_dta, type = "crps"))
  })
})

  local({
    set.seed(42)
    rf_iris <- randomForest::randomForest(Species ~ ., data = iris,
                                          importance = TRUE)
    test_that("snapshot: gg_rfsrc classification rf", {
      vdiffr::expect_doppelganger("gg_rfsrc classification rf",
                                  plot(gg_rfsrc(rf_iris)))
    })
    test_that("snapshot: gg_error classification rf", {
      vdiffr::expect_doppelganger("gg_error classification rf",
                                  plot(gg_error(rf_iris)))
    })
    test_that("snapshot: gg_vimp classification rf", {
      vdiffr::expect_doppelganger("gg_vimp classification rf",
                                  plot(gg_vimp(rf_iris)))
    })
    test_that("snapshot: gg_roc classification rf", {
      vdiffr::expect_doppelganger("gg_roc classification rf",
                                  plot(gg_roc(rf_iris, which_outcome = 1)))
    })
  })
  local({
    set.seed(42)
    rf_mt <- randomForest::randomForest(mpg ~ ., data = mtcars,
                                        importance = TRUE)
    test_that("snapshot: gg_rfsrc regression rf", {
      vdiffr::expect_doppelganger("gg_rfsrc regression rf",
                                  plot(gg_rfsrc(rf_mt)))
    })
    test_that("snapshot: gg_error regression rf", {
      vdiffr::expect_doppelganger("gg_error regression rf",
                                  plot(gg_error(rf_mt)))
    })
    test_that("snapshot: gg_vimp regression rf", {
      vdiffr::expect_doppelganger("gg_vimp regression rf",
                                  plot(gg_vimp(rf_mt)))
    })
    test_that("snapshot: gg_variable regression rf", {
      vdiffr::expect_doppelganger("gg_variable regression rf",
                                  plot(gg_variable(rf_mt)))
    })
  })

  ## ---- gg_partial_varpro snapshots -------------------------------------------
  make_mock_vpro_snap <- function(n_obs = 30, n_pts = 15) {
    set.seed(42)
    list(
      age = list(
        xvirtual    = seq(30, 80, length.out = n_pts),
        xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
        yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
        yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
        yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
      ),
      sex = list(
        xvirtual    = c(0, 1),
        xorg        = sample(c(0, 1), n_obs, replace = TRUE),
        yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
        yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
        yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
      )
    )
  }

  test_that("snapshot: gg-partial-varpro-continuous", {
    result <- gg_partial_varpro(make_mock_vpro_snap(), nvars = 1)
    vdiffr::expect_doppelganger("gg-partial-varpro-continuous", plot(result))
  })

  test_that("snapshot: gg-partial-varpro-categorical", {
    mock <- make_mock_vpro_snap()
    result <- gg_partial_varpro(mock["sex"])
    vdiffr::expect_doppelganger("gg-partial-varpro-categorical", plot(result))
  })

  test_that("snapshot: gg-partial-varpro-both", {
    result <- gg_partial_varpro(make_mock_vpro_snap())
    vdiffr::expect_doppelganger("gg-partial-varpro-both", plot(result))
  })

  test_that("snapshot: gg-partial-varpro-mortality", {
    result <- gg_partial_varpro(make_mock_vpro_snap(), nvars = 1,
                                 scale = "mortality")
    vdiffr::expect_doppelganger("gg-partial-varpro-mortality", plot(result))
  })

  ## ---- gg_varpro snapshots ------------------------------------------------
  local({
    set.seed(42L)
    vp_regr  <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50L)
    vp_class <- varPro::varpro(Species ~ ., data = iris, ntree = 50L)

    test_that("snapshot: gg-varpro-default", {
      gg <- gg_varpro(vp_regr)
      vdiffr::expect_doppelganger("gg-varpro-default", plot(gg))
    })

    test_that("snapshot: gg-varpro-faithful", {
      gg <- gg_varpro(vp_regr, faithful = TRUE)
      vdiffr::expect_doppelganger("gg-varpro-faithful", plot(gg))
    })

    test_that("snapshot: gg-varpro-conditional", {
      gg <- gg_varpro(vp_class, conditional = TRUE)
      vdiffr::expect_doppelganger("gg-varpro-conditional", plot(gg))
    })
  })

  ## ---- gg_udependent snapshots ---------------------------------------------
  if (requireNamespace("ggraph", quietly = TRUE)) {
    local({
      set.seed(42L)
      uv <- varPro::uvarpro(iris[, -5L], ntree = 50L)

      test_that("snapshot: gg-udependent-default", {
        gg <- gg_udependent(uv)
        vdiffr::expect_doppelganger("gg-udependent-default", plot(gg))
      })

      test_that("snapshot: gg-udependent-undirected", {
        gg <- gg_udependent(uv, directed = FALSE)
        vdiffr::expect_doppelganger("gg-udependent-undirected", plot(gg))
      })
    })
  }

## ── randomForest classification snapshots (PR #87) ───────────────────────────
if (requireNamespace("randomForest", quietly = TRUE)) {
  local({
    set.seed(42L)
    rf_iris <- randomForest::randomForest(Species ~ ., data = iris, ntree = 50L)
    gg_iris <- gg_variable(rf_iris)

    test_that("snapshot: gg-variable-rf-classification-default", {
      vdiffr::expect_doppelganger(
        "gg-variable-rf-classification-default",
        plot(gg_iris)
      )
    })

    test_that("snapshot: gg-variable-rf-classification-smooth", {
      vdiffr::expect_doppelganger(
        "gg-variable-rf-classification-smooth",
        plot(gg_iris, xvar = "Sepal.Length", smooth = TRUE)
      )
    })
  })
}

## ── per_class ROC snapshots (PR #88) ─────────────────────────────────────
if (requireNamespace("randomForest", quietly = TRUE)) {
  local({
    set.seed(1L)
    rf_iris    <- randomForest::randomForest(Species ~ ., data = iris, ntree = 100L)
    gg_pc_iris <- gg_roc(rf_iris, per_class = TRUE)

    test_that("snapshot: gg-roc-multiclass-overlay", {
      vdiffr::expect_doppelganger(
        "gg-roc-multiclass-overlay",
        plot(gg_pc_iris, panel = "overlay")
      )
    })

    test_that("snapshot: gg-roc-multiclass-facet", {
      vdiffr::expect_doppelganger(
        "gg-roc-multiclass-facet",
        plot(gg_pc_iris, panel = "facet")
      )
    })
  })
}

## ── gg_isopro snapshots (Phase 4) ────────────────────────────────────────
if (requireNamespace("varPro", quietly = TRUE)) {
  local({
    set.seed(1L)
    fit <- varPro::isopro(data = iris[, 1:4], method = "rnd",
                          sampsize = 32, ntree = 50)
    gg  <- gg_isopro(fit)

    test_that("snapshot: gg-isopro-default", {
      vdiffr::expect_doppelganger("gg-isopro-default", plot(gg))
    })

    test_that("snapshot: gg-isopro-threshold", {
      vdiffr::expect_doppelganger("gg-isopro-threshold",
                                  plot(gg, threshold = 0.8))
    })
  })
}

## ── gg_isopro predict.isopro overlay snapshot (Phase 4b) ──────────────────
if (requireNamespace("varPro", quietly = TRUE)) {
  local({
    set.seed(1L)
    fit      <- varPro::isopro(data = iris[, 1:4], method = "rnd",
                               sampsize = 32, ntree = 50)
    test_df  <- iris[seq(1, nrow(iris), by = 3), 1:4]
    gg_train <- gg_isopro(fit)
    gg_test  <- gg_isopro(fit, newdata = test_df)
    gg_both  <- rbind(
      cbind(as.data.frame(gg_train), method = "train"),
      cbind(as.data.frame(gg_test),  method = "test")
    )
    class(gg_both) <- c("gg_isopro", "data.frame")

    test_that("snapshot: gg-isopro-predict-overlay", {
      vdiffr::expect_doppelganger("gg-isopro-predict-overlay", plot(gg_both))
    })
  })
}

## ── gg_beta_varpro default snapshot (Phase 4c) ─────────────────────────────
test_that("gg-beta-varpro-default", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  v <- .varpro_mtcars()
  b <- .beta_fit_mtcars()
  p <- plot(gg_beta_varpro(v, beta_fit = b))
  vdiffr::expect_doppelganger("gg-beta-varpro-default", p)
})

## ── gg_beta_varpro classification snapshots (Phase 4c-classification) ─────
test_that("gg-beta-varpro-class-binary", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  vb <- .varpro_iris_binary()
  bb <- .beta_fit_iris_binary()
  p <- plot(gg_beta_varpro(vb, beta_fit = bb))
  vdiffr::expect_doppelganger("gg-beta-varpro-class-binary", p)
})

test_that("gg-beta-varpro-class-multiclass", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  vm <- .varpro_iris_multiclass()
  bm <- .beta_fit_iris_multiclass()
  p <- plot(gg_beta_varpro(vm, beta_fit = bm))
  vdiffr::expect_doppelganger("gg-beta-varpro-class-multiclass", p)
})

## ── gg_ivarpro snapshots (Phase 4d) ────────────────────────────────────────
test_that("gg-ivarpro-regr-distribution", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  set.seed(1L)
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv))
  vdiffr::expect_doppelganger("gg-ivarpro-regr-distribution", p)
})

test_that("gg-ivarpro-regr-which-obs", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L))
  vdiffr::expect_doppelganger("gg-ivarpro-regr-which-obs", p)
})

test_that("gg-ivarpro-class-distribution", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  set.seed(1L)
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv))
  vdiffr::expect_doppelganger("gg-ivarpro-class-distribution", p)
})

test_that("gg-ivarpro-class-which-obs", {
  skip_if_not_installed("vdiffr")
  if (!identical(Sys.getenv("VDIFFR_RUN_TESTS", "false"), "true")) {
    skip("vdiffr snapshots skipped (set VDIFFR_RUN_TESTS=true to run)")
  }
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L))
  vdiffr::expect_doppelganger("gg-ivarpro-class-which-obs", p)
})

} # end CI guard

# Cross-function convention: every gg_* importance plot must put the
# most-important variable at the TOP of the horizontal layout. After
# coord_flip(), the topmost variable is the LAST factor level of the
# variable axis. This test pins that convention across the family so a
# future wrapper can't silently reintroduce a bottom-heavy ordering
# (the bug fixed in the v3.0.0 importance-plot-ordering work).

test_that("importance plots agree: most-important variable is the last factor level", {
  skip_if_not_installed("varPro")
  skip_if_not_installed("randomForestSRC")

  # ---- gg_vimp (rfsrc regression) ----------------------------------------
  set.seed(20260528L)
  rf <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 100,
                               importance = TRUE)
  gv <- gg_vimp(rf)
  top_vimp <- as.character(gv$vars[which.max(gv$vimp)])
  expect_equal(tail(levels(gv$vars), 1), top_vimp,
               label = "gg_vimp top factor level")

  # ---- gg_varpro (varpro regression) -------------------------------------
  v  <- .varpro_mtcars()
  gvp <- gg_varpro(v)
  top_varpro <- as.character(
    gvp$stats$variable[which.max(gvp$stats$median)]
  )
  expect_equal(tail(levels(gvp$imp$variable), 1), top_varpro,
               label = "gg_varpro top factor level")

  # ---- gg_beta_varpro (varpro regression) --------------------------------
  b  <- .beta_fit_mtcars()
  gb <- gg_beta_varpro(v, beta_fit = b)
  top_beta <- as.character(gb$variable[which.max(gb$beta_mean)])
  expect_equal(tail(levels(gb$variable), 1), top_beta,
               label = "gg_beta_varpro top factor level")

  # ---- gg_ivarpro (varpro regression) ------------------------------------
  vb <- .varpro_boston()
  iv <- .ivarpro_boston()
  gi <- gg_ivarpro(vb, ivarpro_fit = iv)
  agg_i <- tapply(abs(gi$local_imp), gi$variable, mean, na.rm = TRUE)
  top_ivar <- names(which.max(agg_i))
  expect_equal(tail(levels(gi$variable), 1), top_ivar,
               label = "gg_ivarpro top factor level")
})

# Companion contract: reversing the factor levels for the plot must NOT
# reorder the public data frames. Row order stays most-important-FIRST so
# callers inspecting $imp / $stats / the long frames (and summary()) see a
# stable, descending-importance order. Pins the regression caught in PR #101
# review (rows had followed the reversed factor into least-important-first).

test_that("importance frames keep rows most-important-first despite reversed levels", {
  skip_if_not_installed("varPro")
  skip_if_not_installed("randomForestSRC")

  # ---- gg_varpro: $imp and $stats rows descend by median z ----------------
  v   <- .varpro_mtcars()
  gvp <- gg_varpro(v)
  top_varpro <- as.character(gvp$stats$variable[which.max(gvp$stats$median)])
  expect_equal(as.character(gvp$imp$variable[1]), top_varpro,
               label = "gg_varpro $imp first row")
  expect_equal(as.character(gvp$stats$variable[1]), top_varpro,
               label = "gg_varpro $stats first row")
  expect_true(all(diff(gvp$stats$median) <= 0),
              label = "gg_varpro $stats median is non-increasing")

  # ---- gg_beta_varpro: rows descend by beta_mean --------------------------
  b  <- .beta_fit_mtcars()
  gb <- gg_beta_varpro(v, beta_fit = b)
  top_beta <- as.character(gb$variable[which.max(gb$beta_mean)])
  expect_equal(as.character(gb$variable[1]), top_beta,
               label = "gg_beta_varpro first row")
  expect_true(all(diff(gb$beta_mean) <= 0),
              label = "gg_beta_varpro beta_mean is non-increasing")

  # ---- gg_ivarpro: first long-frame row is the most-important variable ----
  vb <- .varpro_boston()
  iv <- .ivarpro_boston()
  gi <- gg_ivarpro(vb, ivarpro_fit = iv)
  agg_i <- tapply(abs(gi$local_imp), gi$variable, mean, na.rm = TRUE)
  top_ivar <- names(which.max(agg_i))
  expect_equal(as.character(gi$variable[1]), top_ivar,
               label = "gg_ivarpro long-frame first row")
})

# ---- Shape ----------------------------------------------------------------

test_that("gg_ivarpro regression returns long-format tidy frame", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)

  expect_s3_class(out, "gg_ivarpro")
  expect_s3_class(out, "data.frame")
  expect_setequal(names(out), c("obs", "variable", "local_imp", "selected"))
  expect_true(all(!is.na(out$local_imp)))  # NA cells filtered out
  expect_true(is.factor(out$variable))
})

test_that("gg_ivarpro classification adds class column", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  out <- gg_ivarpro(v, ivarpro_fit = iv)

  expect_s3_class(out, "gg_ivarpro")
  expect_true("class" %in% names(out))
  expect_setequal(as.character(unique(out$class)), levels(iris$Species))
})

test_that("gg_ivarpro variable factor levels ordered by descending mean(|local_imp|)", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  expect_true(is.factor(out$variable))

  expected <- tapply(abs(out$local_imp), out$variable, mean, na.rm = TRUE)
  # Reversed-descending: most-important variable is the LAST level (top of
  # the plot after coord_flip).
  expected_order <- rev(names(sort(expected, decreasing = TRUE)))
  expect_equal(levels(out$variable), expected_order)
})

# ---- which_obs ------------------------------------------------------------

test_that("gg_ivarpro which_obs filters to a single observation", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L)
  expect_true(all(out$obs == 1L))
  expect_equal(attr(out, "provenance")$which_obs, 1L)
})

test_that("gg_ivarpro which_obs out of range errors with valid range", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  n  <- nrow(MASS::Boston)
  expect_error(
    gg_ivarpro(v, ivarpro_fit = iv, which_obs = n + 1L),
    "out of range"
  )
})

# ---- which_class ----------------------------------------------------------

test_that("gg_ivarpro binary classification which_class = NULL defaults to last factor level", {
  v  <- .varpro_iris_binary_for_ivarpro()
  iv <- .ivarpro_iris_binary()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  prov <- attr(out, "provenance")
  expect_equal(prov$which_class, "virginica")
  expect_setequal(as.character(unique(out$class)), "virginica")
})

test_that("gg_ivarpro which_class explicit returns single class", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  out <- gg_ivarpro(v, ivarpro_fit = iv, which_class = "setosa")
  expect_equal(as.character(unique(out$class)), "setosa")
})

test_that("gg_ivarpro which_class not in levels errors", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  expect_error(
    gg_ivarpro(v, ivarpro_fit = iv, which_class = "bogus"),
    "is not a level of the response"
  )
})

test_that("gg_ivarpro which_class on regression warns and is ignored", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  expect_warning(
    out <- gg_ivarpro(v, ivarpro_fit = iv, which_class = "anything"),
    "ignored for regression family"
  )
  expect_false("class" %in% names(out))
})

# ---- cutoff polymorphism --------------------------------------------------

test_that("gg_ivarpro cutoff = NULL is per-class mean(|local_imp|) (named vector)", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  prov <- attr(out, "provenance")
  expect_named(prov$cutoff, levels(iris$Species))
  for (cls in levels(iris$Species)) {
    expect_equal(
      prov$cutoff[[cls]],
      mean(abs(out$local_imp[out$class == cls])),
      tolerance = 1e-10
    )
  }
})

test_that("gg_ivarpro scalar cutoff broadcasts across classes", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  out <- gg_ivarpro(v, ivarpro_fit = iv, cutoff = 0.5)
  prov <- attr(out, "provenance")
  expect_equal(unname(prov$cutoff), rep(0.5, 3))
})

test_that("gg_ivarpro regression cutoff is length-1 named numeric", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  prov <- attr(out, "provenance")
  expect_named(prov$cutoff, "regr")
  expect_length(prov$cutoff, 1L)
})

# ---- ivarpro_fit shape guard ----------------------------------------------

test_that("gg_ivarpro rejects malformed ivarpro_fit", {
  v <- .varpro_boston()
  expect_error(gg_ivarpro(v, ivarpro_fit = list()),
               "does not look like a varPro::ivarpro\\(\\) result")
})

test_that("gg_ivarpro classification ivarpro_fit must be list of K named frames", {
  v <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  iv_bad <- iv[c("setosa", "versicolor")]   # drop one class
  expect_error(
    gg_ivarpro(v, ivarpro_fit = iv_bad),
    "class"
  )
})

test_that("gg_ivarpro shape guard catches mismatched row counts (PR #99 Copilot)", {
  # Regression: rows mismatch
  v_regr  <- .varpro_boston()
  iv_regr <- .ivarpro_boston()
  iv_short <- iv_regr[seq_len(nrow(iv_regr) - 1L), , drop = FALSE]
  expect_error(
    gg_ivarpro(v_regr, ivarpro_fit = iv_short),
    "rows but object trained"
  )

  # Classification: per-class row count mismatch
  v_cls  <- .varpro_iris_multiclass_for_ivarpro()
  iv_cls <- .ivarpro_iris_multiclass()
  iv_cls_bad <- iv_cls
  iv_cls_bad$setosa <- iv_cls_bad$setosa[seq_len(nrow(iv_cls_bad$setosa) - 1L), ,
                                         drop = FALSE]
  expect_error(
    gg_ivarpro(v_cls, ivarpro_fit = iv_cls_bad),
    "rows but object trained"
  )

  # Classification: a per-class element is not a data.frame
  iv_cls_bad2 <- iv_cls
  iv_cls_bad2$setosa <- as.matrix(iv_cls_bad2$setosa)
  expect_error(
    gg_ivarpro(v_cls, ivarpro_fit = iv_cls_bad2),
    "is not a data.frame"
  )
})

# ---- ... + ivarpro_fit warning --------------------------------------------

test_that("gg_ivarpro warns when ... is supplied alongside ivarpro_fit", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  expect_warning(
    out <- gg_ivarpro(v, ivarpro_fit = iv, use.loo = TRUE),
    "ignored because ivarpro_fit is supplied"
  )
  expect_s3_class(out, "gg_ivarpro")
})

# ---- family guard ---------------------------------------------------------

test_that("gg_ivarpro errors on regr+ / surv families", {
  v <- .varpro_boston()
  v_fake <- v
  v_fake$family <- "surv"
  expect_error(gg_ivarpro(v_fake), "family = 'surv'")
})

# ---- plot dispatch matrix --------------------------------------------------

test_that("plot.gg_ivarpro regression distribution builds", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv))
  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("plot.gg_ivarpro regression which_obs builds (single panel)", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L))
  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("plot.gg_ivarpro classification distribution builds (faceted)", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv))
  expect_s3_class(p, "ggplot")
  built <- ggplot2::ggplot_build(p)
  expect_true(length(unique(built$layout$layout$PANEL)) >= 2L)
})

test_that("plot.gg_ivarpro classification which_obs builds (faceted)", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  p <- plot(gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L))
  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

# ---- print + summary ------------------------------------------------------

test_that("print.gg_ivarpro prints invisibly with header", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  pr <- withVisible(print(out))
  expect_false(pr$visible)
  expect_identical(pr$value, out)
})

test_that("summary.gg_ivarpro regression returns descending named numeric", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  s <- summary(out)
  expect_s3_class(s, "summary.gg_ivarpro")
  vals <- as.numeric(unclass(s))
  expect_equal(vals, sort(vals, decreasing = TRUE))
})

test_that("summary.gg_ivarpro classification returns per-class list", {
  v  <- .varpro_iris_multiclass_for_ivarpro()
  iv <- .ivarpro_iris_multiclass()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  s <- summary(out)
  expect_s3_class(s, "summary.gg_ivarpro")
  expect_true(is.list(unclass(s)))
  expect_setequal(names(unclass(s)), levels(iris$Species))
})

test_that("autoplot.gg_ivarpro matches plot", {
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()
  out <- gg_ivarpro(v, ivarpro_fit = iv)
  # The aggregate (no which_obs) regression view uses geom_jitter, whose
  # Stat re-randomises on every ggplot_build(). Seed both builds so the
  # comparison is deterministic — the test is checking that autoplot
  # delegates to plot, not testing jitter stability.
  set.seed(1L)
  d1 <- ggplot2::ggplot_build(plot(out))$data
  set.seed(1L)
  d2 <- ggplot2::ggplot_build(ggplot2::autoplot(out))$data
  expect_equal(d1, d2)
})

# ---- slow: cache equivalence ----------------------------------------------

test_that("gg_ivarpro cached and uncached paths agree (slow)", {
  testthat::skip_on_cran()
  if (!identical(Sys.getenv("GG_IVARPRO_SLOW_TESTS", "false"), "true")) {
    skip("Slow test - set GG_IVARPRO_SLOW_TESTS=true to run")
  }
  v  <- .varpro_boston()
  iv <- .ivarpro_boston()

  set.seed(20260526L)
  uncached <- gg_ivarpro(v)
  cached   <- gg_ivarpro(v, ivarpro_fit = iv)

  strip <- function(df) {
    attr(df, "provenance") <- NULL
    df
  }
  expect_equal(strip(as.data.frame(uncached)),
               strip(as.data.frame(cached)),
               tolerance = 1e-6)
  expect_false(attr(uncached, "provenance")$precomputed)
  expect_true(attr(cached,   "provenance")$precomputed)
})

####**********************************************************************
####  summary.gg_* methods + shared print.summary.gg_* printer.
####
####  Each summary method returns a list of pre-formatted lines plus a few
####  raw numeric fields, classed "summary.gg" so print.summary.gg can
####  format it uniformly.
####**********************************************************************

# Build the standard summary skeleton: header line + body lines.
.summary_skel <- function(x, class_label, body = character(0)) {
  out <- list(
    header = .gg_header(x, class_label),
    body   = body
  )
  class(out) <- c(sprintf("summary.%s", class_label), "summary.gg")
  out
}

#' Summary methods for gg_* data objects
#'
#' Where \code{print} gives you a one-line header, \code{summary} digs a level
#' deeper. Each \code{summary.gg_*()} method returns a \code{summary.gg}
#' object: a header line plus a few diagnostic statistics for that object
#' type (the OOB error curve, the top VIMP variables, a time range, the
#' integrated CRPS, and so on). \code{print.summary.gg()} renders it to the
#' console.
#'
#' @param object A \code{gg_*} data object.
#' @param x A \code{summary.gg} object (for \code{print.summary.gg}).
#' @param ... Not currently used.
#'
#' @return A \code{summary.gg} object: a list with \code{header} and
#'   \code{body} character vectors. \code{print.summary.gg} returns it
#'   invisibly.
#'
#' @seealso \code{\link{print.gg}}, \code{\link{autoplot.gg}}
#'
#' @examples
#' set.seed(42)
#' airq <- na.omit(airquality)
#' rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
#' summary(gg_error(rf))
#' summary(gg_vimp(rf))
#'
#' @name summary.gg
NULL

#' @rdname summary.gg
#' @export
print.summary.gg <- function(x, ...) {
  cat(x$header, "\n", sep = "")
  for (line in x$body) {
    cat("  ", line, "\n", sep = "")
  }
  invisible(x)
}

#' @rdname summary.gg
#' @export
summary.gg_error <- function(object, ...) {
  err_cols <- setdiff(names(object), c("ntree", "train"))
  final  <- vapply(err_cols, function(c) {
    v <- object[[c]]
    v[length(v)]
  }, numeric(1))
  min_err <- vapply(err_cols, function(c) min(object[[c]], na.rm = TRUE),
                    numeric(1))
  body <- c(
    sprintf("ntree: %d", max(object$ntree, na.rm = TRUE)),
    sprintf("final OOB error: %s",
            paste(sprintf("%s=%.4g", err_cols, final), collapse = ", ")),
    sprintf("min OOB error:   %s",
            paste(sprintf("%s=%.4g", err_cols, min_err), collapse = ", "))
  )
  .summary_skel(object, "gg_error", body)
}

#' @rdname summary.gg
#' @export
summary.gg_vimp <- function(object, ...) {
  top_n <- min(5L, nrow(object))
  top   <- utils::head(object[order(-object$vimp), , drop = FALSE], top_n)
  body  <- c(
    sprintf("variables: %d", nrow(object)),
    sprintf("positive VIMP: %d / negative: %d",
            sum(object$positive, na.rm = TRUE),
            sum(!object$positive, na.rm = TRUE)),
    sprintf("top %d: %s", top_n,
            paste(sprintf("%s (%.4g)", top$vars, top$vimp), collapse = ", "))
  )
  .summary_skel(object, "gg_vimp", body)
}

#' @rdname summary.gg
#' @export
summary.gg_rfsrc <- function(object, ...) {
  body <- c(sprintf("rows: %d, cols: %d", nrow(object), ncol(object)))
  .summary_skel(object, "gg_rfsrc", body)
}

#' @rdname summary.gg
#' @export
summary.gg_variable <- function(object, ...) {
  resp <- intersect(c("yhat", "yvar"), names(object))
  body <- c(
    sprintf("rows: %d", nrow(object)),
    sprintf("response columns: %s",
            if (length(resp) > 0) paste(resp, collapse = ", ") else "(none)")
  )
  .summary_skel(object, "gg_variable", body)
}

# Internal: shared partial-summary body builder.
.partial_body <- function(x) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  yhat_rng  <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0 &&
                   "yhat" %in% names(x$continuous)) {
    sprintf("yhat range: [%.4g, %.4g]",
            min(x$continuous$yhat, na.rm = TRUE),
            max(x$continuous$yhat, na.rm = TRUE))
  } else {
    NULL
  }
  c(sprintf("continuous: %d, categorical: %d", nvar_cont, nvar_cat),
    yhat_rng)
}

.partialpro_body <- function(x) {
  nvar_cont <- if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    length(unique(x$continuous$name))
  } else {
    0L
  }
  nvar_cat  <- if (is.data.frame(x$categorical) && nrow(x$categorical) > 0) {
    length(unique(x$categorical$name))
  } else {
    0L
  }
  # gg_partialpro has parametric / nonparametric / causal columns, not yhat.
  rng_lines <- NULL
  if (is.data.frame(x$continuous) && nrow(x$continuous) > 0) {
    for (col in c("parametric", "nonparametric", "causal")) {
      if (col %in% names(x$continuous)) {
        vals <- x$continuous[[col]]
        rng_lines <- c(rng_lines,
          sprintf("%s range: [%.4g, %.4g]",
                  col, min(vals, na.rm = TRUE), max(vals, na.rm = TRUE)))
      }
    }
  }
  c(sprintf("continuous: %d, categorical: %d", nvar_cont, nvar_cat),
    rng_lines)
}

#' @rdname summary.gg
#' @export
summary.gg_partial <- function(object, ...) {
  .summary_skel(object, "gg_partial", .partial_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_partial_rfsrc <- function(object, ...) {
  .summary_skel(object, "gg_partial_rfsrc", .partial_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_partialpro <- function(object, ...) {
  ## Deprecated-class shim.
  class(object) <- c("gg_partial_varpro",
                      setdiff(class(object), "gg_partialpro"))
  summary.gg_partial_varpro(object, ...)
}

#' @rdname summary.gg
#' @export
summary.gg_partial_varpro <- function(object, ...) {
  .summary_skel(object, "gg_partial_varpro", .partialpro_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_roc <- function(object, ...) {
  auc <- attr(object, "auc") %||% .gg_auc_trap(object)
  if ("class" %in% names(object)) {
    # per_class = TRUE path: named AUC vector, one entry per class
    n_cls   <- nlevels(object$class)
    auc_str <- paste(sprintf("%s=%.4g", names(auc), auc), collapse = ", ")
    body <- c(sprintf("classes: %d", n_cls),
              sprintf("AUC: %s", auc_str))
  } else {
    body <- c(
      sprintf("thresholds: %d", nrow(object)),
      sprintf("AUC: %.4g", auc)
    )
  }
  .summary_skel(object, "gg_roc", body)
}

# Trapezoidal AUC from a gg_roc data.frame (fpr / tpr).
.gg_auc_trap <- function(x) {
  if (!all(c("fpr", "tpr") %in% names(x))) return(NA_real_)
  ord <- order(x$fpr)
  fpr <- x$fpr[ord]
  tpr <- x$tpr[ord]
  sum((fpr[-1] - fpr[-length(fpr)]) * (tpr[-1] + tpr[-length(tpr)])) / 2
}

#' @rdname summary.gg
#' @export
summary.gg_survival <- function(object, ...) {
  body <- c(
    sprintf("time range: [%.4g, %.4g]",
            min(object$time, na.rm = TRUE),
            max(object$time, na.rm = TRUE)),
    sprintf("rows: %d", nrow(object))
  )
  .summary_skel(object, "gg_survival", body)
}

.varpro_body <- function(x) {
  prov   <- attr(x, "provenance")
  cutoff <- if (!is.null(prov)) prov$cutoff %||% 0.79 else 0.79
  n_sel  <- sum(x$imp$selected, na.rm = TRUE)
  top_n  <- min(5L, nrow(x$stats))
  top_df <- x$stats[order(-x$stats$median), ][seq_len(top_n), ]
  c(
    sprintf("variables: %d total, %d selected (z > %.2g)",
            nrow(x$imp), n_sel, cutoff),
    sprintf("top %d by median z: %s", top_n,
            paste(sprintf("%s (%.3g)",
                          as.character(top_df$variable),
                          top_df$median),
                  collapse = ", ")),
    sprintf("z range: [%.3g, %.3g]",
            min(x$imp$z, na.rm = TRUE),
            max(x$imp$z, na.rm = TRUE))
  )
}

#' @rdname summary.gg
#' @export
summary.gg_varpro <- function(object, ...) {
  .summary_skel(object, "gg_varpro", .varpro_body(object))
}

#' @rdname summary.gg
#' @export
summary.gg_udependent <- function(object, ...) {
  prov <- attr(object, "provenance")
  n    <- if (!is.null(prov)) prov$n else NA
  p    <- if (!is.null(prov)) length(prov$xvar.names) else NA
  thr  <- if (!is.null(prov)) prov$threshold else NA
  body <- c(
    sprintf("Edges: %d  Nodes: %d  Selected: %d/%d",
            nrow(object$edges), nrow(object$nodes),
            sum(object$nodes$selected, na.rm = TRUE), nrow(object$nodes)),
    sprintf("n = %s  p = %s  threshold = %s", n, p, thr)
  )
  .summary_skel(object, "gg_udependent", body)
}

#' @rdname summary.gg
#' @export
summary.gg_brier <- function(object, ...) {
  crps <- attr(object, "crps_integrated")
  envelope_mean <- mean(object$bs.upper - object$bs.lower, na.rm = TRUE)
  body <- c(
    sprintf("time range: [%.4g, %.4g]",
            min(object$time, na.rm = TRUE),
            max(object$time, na.rm = TRUE)),
    sprintf("peak Brier: %.4g at time %.4g",
            max(object$brier, na.rm = TRUE),
            object$time[which.max(object$brier)]),
    sprintf("integrated CRPS: %s",
            if (is.null(crps)) "NA" else sprintf("%.4g", crps)),
    sprintf("mean 15-85%% envelope width: %.4g", envelope_mean)
  )
  .summary_skel(object, "gg_brier", body)
}

#' @rdname summary.gg
#' @export
summary.gg_isopro <- function(object, ...) {
  q  <- stats::quantile(object$howbad, probs = c(0.05, 0.50, 0.95),
                        na.rm = TRUE, names = FALSE)
  body <- c(
    sprintf("rows: %d", nrow(object)),
    sprintf("howbad range: [%.4g, %.4g]",
            min(object$howbad, na.rm = TRUE),
            max(object$howbad, na.rm = TRUE)),
    sprintf("howbad quantiles (5%%/50%%/95%%): %.4g / %.4g / %.4g",
            q[1], q[2], q[3])
  )
  .summary_skel(object, "gg_isopro", body)
}

#' @rdname summary.gg
#' @export
summary.gg_beta_varpro <- function(object, ...) {
  prov   <- attr(object, "provenance")
  family <- if (!is.null(prov)) prov$family %||% "regr" else "regr"

  if (identical(family, "class") && "class" %in% names(object)) {
    per_class <- split(object, object$class, drop = TRUE)
    by_class  <- lapply(per_class, function(df) {
      v <- df$beta_mean
      names(v) <- as.character(df$variable)
      v <- sort(v, decreasing = TRUE)
      structure(v,
                n_rules = stats::setNames(df$n_rules, as.character(df$variable))[names(v)])
    })
    structure(by_class, class = "summary.gg_beta_varpro")
  } else {
    v <- object$beta_mean
    names(v) <- as.character(object$variable)
    v <- sort(v, decreasing = TRUE)
    structure(v,
              n_rules = stats::setNames(object$n_rules,
                                        as.character(object$variable))[names(v)],
              class   = "summary.gg_beta_varpro")
  }
}

#' @rdname summary.gg
#' @export
summary.gg_ivarpro <- function(object, ...) {
  prov   <- attr(object, "provenance")
  family <- if (!is.null(prov)) prov$family %||% "regr" else "regr"

  per_var <- function(df) {
    v <- tapply(abs(df$local_imp), df$variable, mean, na.rm = TRUE)
    v <- v[!is.na(v)]
    v <- sort(v, decreasing = TRUE)
    n_obs <- tapply(df$obs, df$variable,
                    function(o) length(unique(o)))[names(v)]
    structure(v, n_obs = n_obs)
  }

  if (identical(family, "class") && "class" %in% names(object)) {
    per_class <- split(object, object$class, drop = TRUE)
    by_class  <- lapply(per_class, per_var)
    structure(by_class, class = "summary.gg_ivarpro")
  } else {
    structure(per_var(object), class = "summary.gg_ivarpro")
  }
}

####**********************************************************************
####**********************************************************************
####
####  ----------------------------------------------------------------
####  Written by:
####  ----------------------------------------------------------------
####    John Ehrlinger, Ph.D.
####
####    email:  john.ehrlinger@gmail.com
####    URL:    https://github.com/ehrlinger/ggRandomForests
####  ----------------------------------------------------------------
####
####**********************************************************************
####**********************************************************************
#' Plot a \code{\link{gg_error}} object
#'
#' A plot of the cumulative OOB error rates of the random forest as a
#' function of number of trees.
#'
#' @param x A \code{\link{gg_error}} object created from either a
#'   \code{\link[randomForestSRC]{rfsrc}} or a
#'   \code{\link[randomForest]{randomForest}} object.  A raw forest object
#'   may also be supplied and will be passed through \code{\link{gg_error}}
#'   automatically before plotting.
#' @param ... Extra arguments forwarded to the underlying \code{ggplot2}
#'   geometry calls (e.g. \code{size}, \code{linetype}).
#'
#' @return A \code{ggplot} object with \code{ntree} on the x-axis and
#'   OOB error rate on the y-axis.  Single-outcome forests (regression,
#'   survival) produce a single line; multi-outcome forests (classification)
#'   produce one coloured line per class.
#'
#' @details The gg_error plot is used to track the convergence of the
#' randomForest. This figure is a reproduction of the error plot
#' from the \code{\link[randomForestSRC]{plot.rfsrc}} function.
#'
#' @seealso \code{\link{gg_error}} \code{\link[randomForestSRC]{rfsrc}}
#'  \code{\link[randomForestSRC]{plot.rfsrc}}
#'
#' @references
#' Breiman L. (2001). Random forests, Machine Learning, 45:5-32.
#'
#' Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
#' 7(2):25-31.
#'
#' Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
#' Regression and Classification. R package version >= 3.4.0.
#' \url{https://cran.r-project.org/package=randomForestSRC}
#'
#' @examples
#' ## Examples from RFSRC package...
#' ## ------------------------------------------------------------
#' ## classification example
#' ## ------------------------------------------------------------
#' ## ------------- iris data
#' ## You can build a randomForest
#' rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_iris)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## RandomForest example
#' rf_iris <- randomForest::randomForest(Species ~ .,
#'   data = iris,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#' gg_dta <- gg_error(rf_iris)
#' plot(gg_dta)
#'
#' gg_dta <- gg_error(rf_iris, training = TRUE)
#' plot(gg_dta)
#' ## ------------------------------------------------------------
#' ## Regression example
#' ## ------------------------------------------------------------
#' ## ------------- airq data
#' rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ .,
#'   data = airquality,
#'   na.action = "na.impute",
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_airq)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------- Boston data
#' data(Boston, package = "MASS")
#' Boston$chas <- as.logical(Boston$chas)
#' rfsrc_boston <- randomForestSRC::rfsrc(medv ~ .,
#'   data = Boston,
#'   forest = TRUE,
#'   importance = TRUE,
#'   tree.err = TRUE,
#'   save.memory = TRUE
#' )
#'
#' # Get a data.frame containing error rates
#' gg_dta <- gg_error(rfsrc_boston)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#' ## ------------- mtcars data
#' rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars,
#'   importance = TRUE,
#'   save.memory = TRUE,
#'   forest = TRUE,
#'   tree.err = TRUE)
#'
#' # Get a data.frame containing error rates
#' gg_dta<- gg_error(rfsrc_mtcars)
#'
#' # Plot the gg_error object
#' plot(gg_dta)
#'
#'
#' ## ------------------------------------------------------------
#' ## Survival example
#' ## ------------------------------------------------------------
#' ## ------------- veteran data
#' ## randomized trial of two treatment regimens for lung cancer
#' data(veteran, package = "randomForestSRC")
#' rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
#'                        tree.err = TRUE)
#'
#' gg_dta <- gg_error(rfsrc_veteran)
#' plot(gg_dta)
#'
#' ## ------------- pbc data
#' # Load a cached randomForestSRC object
#' # We need to create this dataset
#' data(pbc, package = "randomForestSRC",)
#' # For whatever reason, the age variable is in days... makes no sense to me
#' for (ind in seq_len(dim(pbc)[2])) {
#'  if (!is.factor(pbc[, ind])) {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  } else {
#'    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
#'      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
#'        pbc[, ind] <- as.logical(pbc[, ind])
#'      }
#'    }
#'  }
#'  if (!is.logical(pbc[, ind]) &
#'      length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
#'    pbc[, ind] <- factor(pbc[, ind])
#'  }
#' }
#' #Convert age to years
#' pbc$age <- pbc$age / 364.24
#'
#' pbc$years <- pbc$days / 364.24
#' pbc <- pbc[, -which(colnames(pbc) == "days")]
#' pbc$treatment <- as.numeric(pbc$treatment)
#' pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
#' pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
#' pbc$treatment <- factor(pbc$treatment)
#' dta_train <- pbc[-which(is.na(pbc$treatment)), ]
#' # Create a test set from the remaining patients
#' pbc_test <- pbc[which(is.na(pbc$treatment)), ]
#'
#' #========
#' # build the forest:
#' rfsrc_pbc <- randomForestSRC::rfsrc(
#'   Surv(years, status) ~ .,
#'  dta_train,
#'  nsplit = 10,
#'  na.action = "na.impute",
#'  tree.err = TRUE,
#'  forest = TRUE,
#'  importance = TRUE,
#'  save.memory = TRUE
#' )
#'
#'
#' gg_dta <- gg_error(rfsrc_pbc)
#' plot(gg_dta)
#'
#' @importFrom ggplot2 ggplot geom_line theme labs
#' @importFrom tidyr pivot_longer
#' @export
plot.gg_error <- function(x, ...) {
  gg_dta <- x

  # Accept a raw rfsrc object and extract error rates on the fly
  if (inherits(gg_dta, "rfsrc")) {
    gg_dta <- gg_error(gg_dta)
  }

  if (!inherits(gg_dta, "gg_error")) {
    stop("Incorrect object type: Expects a gg_error object")
  }

  # Use points instead of lines when there is only one non-NA row (e.g. a
  # forest built with a single tree, or one where only ntree=1 has an error
  # rate recorded).  A line plot with one point renders nothing visible.
  point <- FALSE
  if (nrow(na.omit(gg_dta)) < 2) {
    point <- TRUE
  }

  multi_outcome <- ncol(gg_dta) > 2
  if (multi_outcome) {
    # Multi-outcome (classification): gg_error has one column per class plus
    # the "ntree" column.  Pivot to long form so we can colour by outcome.
    gg_dta <- tidyr::pivot_longer(gg_dta, -"ntree", names_to = "variable", values_to = "value")
    gg_plt <-
      ggplot2::ggplot(na.omit(gg_dta),
                      ggplot2::aes(x = .data[["ntree"]], y = .data[["value"]],
                                   col = .data[["variable"]]))
  } else {
    # Single-outcome (regression / survival): gg_error has columns "ntree"
    # and "error".  Map directly without reshaping.
    gg_plt <-
      ggplot2::ggplot(na.omit(gg_dta), ggplot2::aes(x = .data[["ntree"]],
                                                    y = .data[["error"]]))
  }

  # Build the labs() once with the colour label only mapped on the
  # multi-outcome / classification path (avoids the "Ignoring unknown
  # labels: colour Outcome" warning on the single-outcome path where no
  # colour aesthetic is mapped). #82.
  err_labs <- if (multi_outcome) {
    ggplot2::labs(x = "Number of Trees", y = "OOB Error Rate",
                  color = "Outcome")
  } else {
    ggplot2::labs(x = "Number of Trees", y = "OOB Error Rate")
  }
  if (point) {
    gg_plt <- gg_plt + ggplot2::geom_point() + err_labs
  } else {
    gg_plt <- gg_plt + ggplot2::geom_line() + err_labs
  }

  # Hide the legend when there is only a single outcome variable: the colour
  # key adds no information and clutters the plot.  For single-outcome forests
  # (regression / survival) the data is never gathered, so there is no
  # "variable" column; suppress the legend unconditionally in that case.
  if (!"variable" %in% names(gg_dta) ||
      length(unique(gg_dta$variable)) <= 1) {
    gg_plt <- gg_plt + ggplot2::theme(legend.position = "none")
  }
  return(gg_plt)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)


## ----packages-----------------------------------------------------------------
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())


## ----data-load----------------------------------------------------------------
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter


## ----variable-labels----------------------------------------------------------
st_labs <- c(
  crim    = "Crime rate by town",
  zn      = "Residential land zoned > 25k sq ft (%)",
  indus   = "Non-retail business acres (%)",
  chas    = "Borders Charles River",
  nox     = "Nitrogen oxides (10 ppm)",
  rm      = "Rooms per dwelling",
  age     = "Units built before 1940 (%)",
  dis     = "Distance to employment centers",
  rad     = "Highway accessibility index",
  tax     = "Property tax rate per $10,000",
  ptratio = "Pupil-teacher ratio",
  black   = "Proportion of Black residents",
  lstat   = "Lower status population (%)",
  medv    = "Median home value ($1000s)"
)


## ----eda, fig.cap="EDA: each predictor vs. median home value, colored by Charles River indicator."----
dta <- Boston |>
  pivot_longer(c(-medv, -chas), names_to = "variable", values_to = "value")

ggplot(dta, aes(x = medv, y = value, color = chas)) +
  geom_point(alpha = 0.4) +
  geom_smooth(aes(x = medv, y = value), color = "grey30",
              inherit.aes = FALSE, se = FALSE) +
  labs(y = "", x = st_labs["medv"]) +
  scale_color_brewer(palette = "Set2") +
  facet_wrap(~variable, scales = "free_y", ncol = 3)


## ----rfsrc-fit----------------------------------------------------------------
rfsrc_Boston <- rfsrc(medv ~ ., data = Boston, # nolint: object_name_linter
                      ntree = 200, importance = TRUE, err.block = 5)
rfsrc_Boston


## ----error-plot, fig.height=4, fig.cap="OOB mean squared error vs. number of trees."----
gg_e <- gg_error(rfsrc_Boston)
gg_e <- gg_e |> filter(!is.na(error))
class(gg_e) <- c("gg_error", class(gg_e))
plot(gg_e)


## ----rfsrc-predicted, fig.height=4, fig.cap="OOB predicted median home values. Points are jittered; boxplot shows the distribution."----
plot(gg_rfsrc(rfsrc_Boston), alpha = 0.5) +
  coord_cartesian(ylim = c(5, 49))


## ----vimp-plot, fig.cap="VIMP ranking. Longer blue bars indicate more important variables."----
plot(gg_vimp(rfsrc_Boston), lbls = st_labs)


## ----varsel-------------------------------------------------------------------
md_Boston <- max.subtree(rfsrc_Boston) # nolint: object_name_linter


## ----select-vars--------------------------------------------------------------
xvar <- md_Boston$topvars


## ----vardep-panel, fig.cap="Variable dependence for top predictors (minimal depth rank order)."----
gg_v <- gg_variable(rfsrc_Boston)

plot(gg_v, xvar = xvar, panel = TRUE, alpha = 0.5) +
  labs(y = st_labs["medv"], x = "")


## ----vardep-chas, fig.height=4, fig.cap="Variable dependence for Charles River (categorical)."----
plot(gg_v, xvar = "chas", alpha = 0.4) +
  labs(y = st_labs["medv"])


## ----partial-dep, fig.cap="Partial dependence for top predictors."------------
pd <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = xvar)

# Quick S3 plot — works out of the box for the standard regression case
plot(pd)


## ----partial-dep-custom, fig.cap="Partial dependence (custom styling)."-------
ggplot(pd$continuous, aes(x = x, y = yhat)) +
  geom_line(color = "steelblue", linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = st_labs["medv"], x = "") +
  theme_bw()


## ----coplot-chas, fig.height=4, fig.cap="Variable dependence on lstat, conditional on Charles River."----
gg_v$chas_label <- ifelse(gg_v$chas, "Borders Charles River",
                          "Does not border")

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  facet_wrap(~chas_label)


## ----coplot-rm, fig.cap="Predicted medv vs. lstat, conditional on rooms-per-dwelling groups."----
rm_pts <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 6, intervals = TRUE)
gg_v$rm_grp <- cut(rfsrc_Boston$xvar$rm, breaks = rm_pts)
levels(gg_v$rm_grp) <- paste("rm in", levels(gg_v$rm_grp))

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~rm_grp)


## ----coplot-lstat, fig.cap="Predicted medv vs. rooms, conditional on lower-status groups."----
lstat_pts <- quantile_pts(rfsrc_Boston$xvar$lstat, groups = 6,
                          intervals = TRUE)
gg_v$lstat_grp <- cut(rfsrc_Boston$xvar$lstat, breaks = lstat_pts)
levels(gg_v$lstat_grp) <- paste("lstat in", levels(gg_v$lstat_grp))

plot(gg_v, xvar = "rm", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["rm"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~lstat_grp)


## ----surface-data, cache=TRUE-------------------------------------------------
rm_grid <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 10)

surface_list <- lapply(rm_grid, function(rm_val) {
  newx <- rfsrc_Boston$xvar
  newx$rm <- rm_val
  pd_rm <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = "lstat", newx = newx)
  df <- pd_rm$continuous
  df$rm <- rm_val
  df
})

surface_df <- bind_rows(surface_list)


## ----pd-surface, fig.cap="Partial dependence surface: median home value as a function of lstat and rm. Fill colour is the predicted median value."----
ggplot(surface_df, aes(x = x, y = rm, fill = yhat)) +
  geom_tile() +
  scale_fill_viridis_c(name = "Median Value\n($1000s)") +
  labs(x = "Lower Status (%)", y = "Rooms per Dwelling") +
  theme_bw()

---
title: "Exploring Random Forests with ggRandomForests"
author: "John Ehrlinger"
date: today
format: 
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    html-math-method: mathjax
editor: 
  markdown: 
    wrap: 80
vignette: >
  %\VignetteIndexEntry{Exploring Random Forests with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

A fitted random forest carries a lot of information, but getting at it
usually means digging through list structures that were never meant to be
plotted directly. **ggRandomForests** does that digging for you: it pulls
tidy data objects out of a `randomForestSRC` or `randomForest` fit, and
those objects drop straight into the `ggplot2` workflows you already know.
A second engine, `varPro`, powers a parallel family of functions for
release-rule importance and related diagnostics; that family is covered in
the companion vignette referenced at the end.
This vignette walks through the three objects you will reach for most often
(`gg_error`, `gg_variable`, and `gg_vimp`), plus a small helper for
cutting a predictor into evenly populated groups.

```{r pkg-setup, include=FALSE}
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}
```

## Error trajectories with `gg_error()`

```{r error-demo}
library(randomForest)
set.seed(42)
rf_iris <- randomForest(Species ~ ., data = iris, ntree = 200, keep.forest = TRUE)
err_df <- ggRandomForests::gg_error(rf_iris, training = TRUE)
head(err_df)
```

A forest's error rate settles down as trees are added, and the `gg_error()`
object lets you watch that happen. It holds the cumulative out-of-bag (OOB)
error rate for each outcome column, indexed by the `ntree` counter. Ask for
`training = TRUE` and the function reconstructs the original model frame and
adds the in-bag error trajectory (`train`) as well, so you can see both
curves at once:

```{r error-plot, fig.height=4}
plot(err_df)
```

## Marginal dependence via `gg_variable()`

```{r variable-demo}
set.seed(99)
boston <- MASS::Boston
rf_boston <- randomForest(medv ~ ., data = boston, ntree = 150)
var_df <- ggRandomForests::gg_variable(rf_boston)
str(var_df[, c("lstat", "yhat")])
```

`gg_variable()` recovers the training data straight from the model call,
so it still works when the forest was fit inside a helper function or
against a `subset()` expression, cases where the data is not sitting in
the global environment. The object you get back keeps the raw predictors
alongside the prediction: a single `yhat` column for regression, or one
`yhat.<class>` column per class for classification. To plot one predictor,
name it with `xvar`:

```{r variable-plot, fig.height=4}
plot(var_df, xvar = "lstat")
```

Survival forests can request multiple horizons using the `time` argument;
non-OOB predictions are available by setting `oob = FALSE`.

## Variable importance with `gg_vimp()`

```{r vimp-demo}
vimp_df <- ggRandomForests::gg_vimp(rf_boston)
head(vimp_df)
plot(vimp_df)
```

`gg_vimp()` measures permutation importance: each predictor is permuted in
turn, and the drop in OOB accuracy gives its score. This contrasts with the
`gg_varpro` family, which uses release-rule importance from the `varPro`
engine. Variable importance is not always stored on the fitted object. If a
`randomForest` fit is missing its importance scores, `gg_vimp()` will try to
compute them for you. When even that is not possible (the forest was grown
with `importance = FALSE` and the predictors are no longer reachable),
the function warns and returns `NA` in place of the scores, so a plot still
draws rather than failing outright.

## Balanced conditioning cuts with `quantile_pts()`

```{r quantile-demo}
rm_breaks <- ggRandomForests::quantile_pts(boston$rm, groups = 6, intervals = TRUE)
rm_groups <- cut(boston$rm, breaks = rm_breaks)
table(rm_groups)
```

When you build a coplot, you want each conditioning group to hold a roughly
equal share of the data --- equal-width bins leave the sparse tails nearly
empty. `quantile_pts()` wraps `stats::quantile()` to give you break points
that do exactly that, and they pass straight to `cut()` for the grouping or
facet labels.

## Next steps

* The full API reference lives at <https://ehrlinger.github.io/ggRandomForests/>.
* `?gg_error`, `?gg_variable`, `?gg_vimp`, and `?quantile_pts` cover the
  remaining arguments and have their own examples.
* The `gg_error`, `gg_variable`, and `gg_vimp` objects shown here are tidy
  data frames underneath, so you can skip the `plot()` methods entirely and
  build the figure yourself with `ggplot2`.
* For the full varPro toolkit (release-rule importance, lasso-refined
  importance, per-observation local importance, anomaly scores, and the
  dependency graph) walked across regression, classification, and
  survival examples, see `vignette("varpro", package = "ggRandomForests")`.

---
title: "Random Forest Regression with ggRandomForests"
author: "John Ehrlinger"
date: today
format:
  html:
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Random Forest Regression with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

::: {.callout-warning}
## Work in progress
This vignette is under active development. Code examples and narrative may
change before the next release.
:::

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)
```

# Introduction

Random forests [@Breiman:2001] are a non-parametric ensemble method that
requires no distributional or functional assumptions on how covariates relate to
the response. The method builds a large collection of de-correlated decision
trees via bootstrap aggregation (bagging) and random feature selection, then
averages their predictions to produce a stable, low-variance estimator. The
**randomForestSRC** package [@Ishwaran:RFSRC:2014] provides a unified
implementation for survival, regression, and classification forests.

**ggRandomForests** extracts tidy data objects from `rfsrc` fits and renders
them with **ggplot2** [@Wickham:2009], making it straightforward to explore how a
forest is constructed, which variables matter, and how the response depends on
individual predictors.

This vignette demonstrates a complete random forest regression workflow on the
Boston Housing data set [@Harrison:1978; @Belsley:1980]:

1. **Data exploration** --- EDA scatter panels, variable descriptions
2. **Growing the forest** --- fitting an RF, checking OOB error convergence
3. **Variable selection** --- VIMP and minimal depth via `max.subtree()`
4. **Dependence plots** --- variable dependence and partial dependence via
   `gg_variable()` and `gg_partial_rfsrc()`
5. **Variable interactions** --- conditioning plots and interactive 3-D partial
   dependence surfaces with **plotly**

```{r packages}
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())
```

# Data: Boston Housing Values

The Boston Housing data [@Harrison:1978; @Belsley:1980] is a standard benchmark
for regression. It contains data for 506 census tracts of Boston from the 1970
census. The objective is to predict the median value of owner-occupied homes
(`medv`, in \$1000s) from 13 predictors covering crime, zoning, industry,
environmental quality, and housing characteristics. We use the copy from the
**MASS** package [@mass:2002].

```{r data-load}
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
```

```{r variable-labels}
st_labs <- c(
  crim    = "Crime rate by town",
  zn      = "Residential land zoned > 25k sq ft (%)",
  indus   = "Non-retail business acres (%)",
  chas    = "Borders Charles River",
  nox     = "Nitrogen oxides (10 ppm)",
  rm      = "Rooms per dwelling",
  age     = "Units built before 1940 (%)",
  dis     = "Distance to employment centers",
  rad     = "Highway accessibility index",
  tax     = "Property tax rate per $10,000",
  ptratio = "Pupil-teacher ratio",
  black   = "Proportion of Black residents",
  lstat   = "Lower status population (%)",
  medv    = "Median home value ($1000s)"
)
```

## Exploratory data analysis

We plot each predictor against the response (`medv`), coloring by the sole
categorical variable (`chas`, whether the tract borders the Charles River). A
loess smooth highlights the marginal trend.

```{r eda, fig.cap="EDA: each predictor vs. median home value, colored by Charles River indicator."}
dta <- Boston |>
  pivot_longer(c(-medv, -chas), names_to = "variable", values_to = "value")

ggplot(dta, aes(x = medv, y = value, color = chas)) +
  geom_point(alpha = 0.4) +
  geom_smooth(aes(x = medv, y = value), color = "grey30",
              inherit.aes = FALSE, se = FALSE) +
  labs(y = "", x = st_labs["medv"]) +
  scale_color_brewer(palette = "Set2") +
  facet_wrap(~variable, scales = "free_y", ncol = 3)
```

Even from this simple view, the strong relationship between `medv` and both
`lstat` (lower status %) and `rm` (rooms per dwelling) is apparent. We expect
the random forest to confirm these as the most important predictors.

# Growing a Random Forest

We grow a regression forest using all 13 predictors. The `rfsrc()` function
detects the regression family from the continuous response.

```{r rfsrc-fit}
rfsrc_Boston <- rfsrc(medv ~ ., data = Boston, # nolint: object_name_linter
                      importance = TRUE, err.block = 5)
rfsrc_Boston
```

The forest grew `r rfsrc_Boston$ntree` trees, splitting on
`r rfsrc_Boston$mtry` randomly selected candidate variables at each node, and
stopping at a minimum terminal node size of `r rfsrc_Boston$nodesize`.

## OOB error convergence

```{r error-plot, fig.height=4, fig.cap="OOB mean squared error vs. number of trees."}
gg_e <- gg_error(rfsrc_Boston)
gg_e <- gg_e |> filter(!is.na(error))
class(gg_e) <- c("gg_error", class(gg_e))
plot(gg_e)
```

The error stabilizes well before 500 trees, indicating the forest is large
enough for reliable predictions.

## OOB predictions

```{r rfsrc-predicted, fig.height=4, fig.cap="OOB predicted median home values. Points are jittered; boxplot shows the distribution."}
plot(gg_rfsrc(rfsrc_Boston), alpha = 0.5) +
  coord_cartesian(ylim = c(5, 49))
```

Each point is a single tract's OOB prediction. The distribution is a sanity
check --- we are more interested in the *why* behind these predictions.

# Variable Selection

## Variable importance (VIMP)

VIMP measures the increase in prediction error when a variable is randomly
permuted [@Breiman:2001]. Large positive values mean the variable is essential;
negative values suggest noise is more informative.

```{r vimp-plot, fig.cap="VIMP ranking. Longer blue bars indicate more important variables."}
plot(gg_vimp(rfsrc_Boston), lbls = st_labs)
```

`lstat` and `rm` dominate, with a clear gap to the remaining predictors. All
VIMP values are positive, indicating every predictor contributes at least
marginally.

## Minimal depth

Minimal depth [@Ishwaran:2010] ranks variables by how close to the root node
they first split, on average. Variables that partition large portions of the
population early are considered most important.

```{r varsel}
md_Boston <- max.subtree(rfsrc_Boston) # nolint: object_name_linter
```

The threshold is `r round(md_Boston$threshold, 2)`, selecting
`r length(md_Boston$topvars)` variables:
`r paste(md_Boston$topvars, collapse = ", ")`.

Both VIMP and minimal depth agree on the dominance of `lstat` and `rm`. We use
the minimal depth top variables for the remainder of the analysis.

```{r select-vars}
xvar <- md_Boston$topvars
```

# Variable Dependence

## Variable dependence plots

Variable dependence shows each tract's OOB predicted `medv` plotted against a
predictor, with a loess smooth indicating the trend.

```{r vardep-panel, fig.cap="Variable dependence for top predictors (minimal depth rank order)."}
gg_v <- gg_variable(rfsrc_Boston)

plot(gg_v, xvar = xvar, panel = TRUE, alpha = 0.5) +
  labs(y = st_labs["medv"], x = "")
```

The panels confirm what EDA suggested: `medv` decreases sharply with `lstat` and
increases with `rm`, both in strongly non-linear ways. The remaining variables
show weaker but still discernible trends.

```{r vardep-chas, fig.height=4, fig.cap="Variable dependence for Charles River (categorical)."}
plot(gg_v, xvar = "chas", alpha = 0.4) +
  labs(y = st_labs["medv"])
```

Most tracts do not border the Charles River, and the predicted value
distributions largely overlap --- consistent with `chas` ranking last in both
VIMP and minimal depth.

## Partial dependence

Partial dependence integrates out the effects of all other covariates, giving a
risk-adjusted view of each predictor's marginal effect [@Friedman:2000]:

$$
\tilde{f}(x) = \frac{1}{n} \sum_{i=1}^n \hat{f}(x, x_{i,o})
$$

We use `gg_partial_rfsrc()`, which calls `randomForestSRC::partial.rfsrc()`
directly and returns a `gg_partial_rfsrc` object.  The quickest path to a plot
is `plot(pd)`, which handles continuous and categorical variables automatically.
For a custom layout we can also access `pd$continuous` directly.

```{r partial-dep, fig.cap="Partial dependence for top predictors."}
pd <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = xvar)

# Quick S3 plot — works out of the box for the standard regression case
plot(pd)
```

For a publication-ready layout with custom axis labels, access the underlying
data frame directly:

```{r partial-dep-custom, fig.cap="Partial dependence (custom styling)."}
ggplot(pd$continuous, aes(x = x, y = yhat)) +
  geom_line(color = "steelblue", linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = st_labs["medv"], x = "") +
  theme_bw()
```

`lstat` shows a strongly concave relationship, while `rm` has a flat region
below ~6 rooms before increasing sharply. These non-linear shapes are difficult
to capture with simple parametric transforms but are naturally handled by the
random forest.

# Variable Interactions and Conditioning Plots

## Conditioning on a categorical variable

The simplest coplot conditions on a categorical variable. Here we examine
`medv` vs. `lstat`, split by Charles River status:

```{r coplot-chas, fig.height=4, fig.cap="Variable dependence on lstat, conditional on Charles River."}
gg_v$chas_label <- ifelse(gg_v$chas, "Borders Charles River",
                          "Does not border")

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  facet_wrap(~chas_label)
```

The decreasing trend holds in both groups, with slightly higher values along the
Charles River at every `lstat` level.

## Conditioning on a continuous variable

To investigate the `lstat`--`rm` interaction, we bin `rm` into six quantile
groups using `quantile_pts()` and facet:

```{r coplot-rm, fig.cap="Predicted medv vs. lstat, conditional on rooms-per-dwelling groups."}
rm_pts <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 6, intervals = TRUE)
gg_v$rm_grp <- cut(rfsrc_Boston$xvar$rm, breaks = rm_pts)
levels(gg_v$rm_grp) <- paste("rm in", levels(gg_v$rm_grp))

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~rm_grp)
```

Median values decrease with `lstat` within every `rm` group, but the intercept
shifts upward with more rooms. Smaller homes in low-`lstat` (high-status)
neighborhoods still command high prices.

The complement view --- `medv` vs. `rm`, conditional on `lstat` groups ---
completes the picture:

```{r coplot-lstat, fig.cap="Predicted medv vs. rooms, conditional on lower-status groups."}
lstat_pts <- quantile_pts(rfsrc_Boston$xvar$lstat, groups = 6,
                          intervals = TRUE)
gg_v$lstat_grp <- cut(rfsrc_Boston$xvar$lstat, breaks = lstat_pts)
levels(gg_v$lstat_grp) <- paste("lstat in", levels(gg_v$lstat_grp))

plot(gg_v, xvar = "rm", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["rm"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~lstat_grp)
```

The `rm` effect is strongest in low-`lstat` tracts (bottom-left panels) and
nearly flat in high-`lstat` tracts, confirming a meaningful interaction.

# Interactive Partial Dependence Surface

To visualize the joint partial dependence of `medv` on `lstat` and `rm`, we
compute partial dependence on a grid: 25 values of `rm`, each evaluated at 25
points along `lstat`.

```{r surface-data, cache=TRUE}
rm_grid <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 25)

surface_list <- lapply(rm_grid, function(rm_val) {
  newx <- rfsrc_Boston$xvar
  newx$rm <- rm_val
  pd_rm <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = "lstat", newx = newx)
  df <- pd_rm$continuous
  df$rm <- rm_val
  df
})

surface_df <- bind_rows(surface_list)
```

```{r plotly-surface, fig.cap="Interactive partial dependence surface: median home value as a function of lstat and rm."}
if (requireNamespace("plotly", quietly = TRUE)) {
  library(plotly)

  surface_wide <- surface_df |>
    select(lstat = x, rm, medv = yhat) |>
    arrange(rm, lstat)

  lstat_vals <- sort(unique(surface_wide$lstat))
  rm_vals    <- sort(unique(surface_wide$rm))
  z_matrix   <- matrix(surface_wide$medv,
                        nrow = length(rm_vals),
                        ncol = length(lstat_vals),
                        byrow = TRUE)

  plot_ly(x = lstat_vals, y = rm_vals, z = z_matrix) |>
    add_surface(colorscale = "Viridis", showscale = TRUE) |>
    layout(
      scene = list(
        xaxis = list(title = "Lower Status (%)"),
        yaxis = list(title = "Rooms per Dwelling"),
        zaxis = list(title = "Median Value ($1000s)")
      )
    )
} else {
  message("Install the plotly package for interactive 3D surfaces.")
  ggplot(surface_df, aes(x = x, y = rm, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Median Value") +
    labs(x = "Lower Status (%)", y = "Rooms per Dwelling") +
    theme_bw()
}
```

The surface confirms the strong interaction: home values are highest when `lstat`
is low and `rm` is high (upper-left corner), dropping steeply along both axes.
The non-planar curvature --- particularly the sharp step near `rm` = 7 ---
demonstrates the kind of complex, non-linear structure that random forests
capture naturally.

# Conclusion

This vignette demonstrated a complete random forest regression analysis using
**randomForestSRC** and **ggRandomForests**:

- **OOB error** via `gg_error()` showed the forest stabilized quickly.
- **VIMP** via `gg_vimp()` and **minimal depth** via `max.subtree()` both
  identified `lstat` and `rm` as the dominant predictors, with a clear gap to
  the remaining variables.
- **Variable dependence** via `gg_variable()` revealed strongly non-linear
  predictor--response relationships that match the raw data EDA.
- **Partial dependence** via `gg_partial_rfsrc()` provided risk-adjusted
  confirmation, showing concave `lstat` and threshold-like `rm` effects.
- **Conditioning plots** and **interactive surfaces** exposed the `lstat`--`rm`
  interaction, where the room-size effect is strongest in high-status
  neighborhoods.

The **ggRandomForests** design separates data extraction from plotting: every
`gg_*()` function returns a tidy data frame that can be plotted with the
package's `plot()` methods or fed directly into custom `ggplot2` workflows.

# References

---
title: "Variable selection without an outcome: unsupervised varPro"
author: "John Ehrlinger"
date: today
format:
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    toc-depth: 2
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Variable selection without an outcome: unsupervised varPro}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
knitr::opts_chunk$set(
  message    = FALSE,
  warning    = FALSE,
  fig.width  = 7,
  fig.height = 4.5,
  cache      = TRUE,
  cache.path = "uvarpro_cache/"
)
options(mc.cores = 1, rf.cores = 1)
```

```{r libs}
library(ggplot2)

# Match the pattern used by the other vignettes: try the installed
# package first, fall back to pkgload::load_all() for the R CMD check
# vignette rebuild where the package isn't yet on .libPaths(). All varPro
# calls below are ::-qualified, so no library(varPro) is needed.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}
```

# Importance without a response

Most measures of variable importance start from a question: which predictors
help explain *this* outcome? Permutation VIMP, the varPro release rules, the
per-rule lasso weights behind `gg_beta_varpro()` — all of them score a variable
by how much it moves a response `y`. The companion
[varPro vignette](varpro.html) walks that supervised toolkit end to end.

But sometimes there is no `y`. You have a matrix of predictors and you want to
know which columns carry the structure of the data: which variables define its
shape, which travel together, and which are close to noise. That is the job of
unsupervised varPro. `varPro::uvarpro()` grows a forest on the predictor matrix
alone, with no response, and scores each variable by how much entropy it
contributes to the regions the forest carves out [@Lu2024varpro]. "Important"
here
does not mean useful for prediction. It means a variable helps reconstruct the
feature space that the others cannot.

This is a short vignette. It walks the three `gg_*` views of a single
`uvarpro()` fit, and each one answers a different question about that fit: what
depends on what, what ranks highest, and where to draw the line between signal
and noise.

# One fit, three views

We'll use `mtcars`. Its columns are all numeric, and several of them measure
closely related things — displacement, horsepower, weight, and cylinder count
all track engine size — so the unsupervised structure is easy to read.

```{r fit}
set.seed(1)
u <- varPro::uvarpro(mtcars, ntree = 50)
```

Notice what `uvarpro()` was handed: the data frame, and nothing else. No
formula, no outcome column singled out. All three views below read off this one
fit. Two of them rest on the same `get.beta.entropy()` matrix, which is the only
part worth caching, so we compute it once and pass it to both rather than paying
for it twice:

```{r beta_fit}
beta_fit <- varPro::get.beta.entropy(u)
```

## What depends on what: `gg_udependent()`

`gg_udependent()` reads cross-variable structure off the fit and draws it as a
network: nodes are variables, edges are dependencies above a configurable
threshold. The picture is built with `ggraph`, which lives in `Suggests` rather
than `Imports`, so install it if you want this view.

```{r udep, eval = requireNamespace("ggraph", quietly = TRUE)}
plot(gg_udependent(u))
```

Clusters of mutually connected variables are worth a second look. Because the
fit has no response, a cluster tells you these columns are correlated in feature
space regardless of whether any of them would matter for a prediction. When you
do have a downstream model, that is useful on its own: a variable can be
important for prediction and still sit in a tight cluster with a near-duplicate,
and in that case parsimony may favor dropping one member of the cluster without
losing much.

## What ranks highest: `gg_beta_uvarpro()`

The network shows *structure*; `gg_beta_uvarpro()` turns the same fit into a
*ranking*. It is the unsupervised analogue of `gg_beta_varpro()`: from the
entropy matrix it aggregates the per-region lasso coefficients into a mean
absolute weight per variable (`beta_mean`), orders them most-important first,
and flags the ones above a selection cutoff. Here we hand it the `beta_fit` we
already computed.

```{r beta}
plot(gg_beta_uvarpro(u, beta_fit = beta_fit))
```

Read it as the unsupervised counterpart of a VIMP bar chart. The tall bars are
the variables that most define the structure of the predictor space. Paired with
the network above, you get both halves of the story: *which* variables carry the
most unsupervised signal, and *how* they group.

## Where to draw the line: `gg_sdependent()`

A ranking invites the obvious next question: where is the cut? Which variables
are signal, and which are noise? `gg_sdependent()` answers that narrower
question off the same fit. It wraps `varPro::sdependent()` and returns one row
per candidate variable — an importance score, its degree in the dependency
graph, and a `signal` flag — drawn as a ranked lollipop.

```{r sdep}
plot(gg_sdependent(u, beta_fit = beta_fit))
```

Where `gg_beta_uvarpro()` ranks *all* the variables, `gg_sdependent()` makes the
cut explicit, separating the columns the unsupervised analysis treats as signal
from the ones it treats as noise.

# Reading the three together

The three views are one workflow, not three unrelated plots. Start with
`gg_udependent()` to see the structure, rank it with `gg_beta_uvarpro()`, then
let `gg_sdependent()` draw the signal-versus-noise line. They all derive from the
one `uvarpro()` fit, and two of them from the one `get.beta.entropy()` matrix, so
computing that matrix once (as we did above) and passing it in keeps the whole
sequence cheap.

For the supervised side of varPro — VIMP, partial dependence, per-rule lasso
refinement, local importance, and anomaly scoring — see the companion
[varPro vignette](varpro.html).

# References

---
title: "Random Forest Classification with ggRandomForests"
author: "John Ehrlinger"
date: today
format:
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Random Forest Classification with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

::: {.callout-warning}
## Work in progress
This vignette is under active development. Code examples and narrative may
change before the next release.
:::

```{r setup, include=FALSE}
# vignette dir under R CMD check, package root under an interactive knit
.fo <- c("_fig_optim.R", file.path("vignettes", "_fig_optim.R"))
if (any(file.exists(.fo))) source(.fo[file.exists(.fo)][1])
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)
```

# Introduction

Random forests [@Breiman:2001] are a non-parametric ensemble method that
requires no distributional or functional assumptions on how covariates relate
to the response. For classification, each tree votes for a class, and the
forest reports the vote fractions as class probabilities. The
**randomForestSRC** package [@Ishwaran:RFSRC:2014] provides a unified
implementation for survival, regression, and classification forests.

**ggRandomForests** extracts tidy data objects from `rfsrc` fits and renders
them with **ggplot2** [@Wickham:2009], making it straightforward to explore how
a forest is constructed, which variables matter, and how the predicted
probabilities depend on individual predictors.

This vignette demonstrates a complete random forest classification workflow on
Fisher's iris data [@Fisher:1936]:

1. **Data exploration** --- EDA scatter panels, the two variable pairs that
   separate the three species
2. **Growing the forest** --- fitting an RF, checking OOB error convergence
3. **Variable selection** --- VIMP and minimal depth via `max.subtree()`
4. **SHAP analysis** --- per-observation, per-class additive explanations via
   `gg_shap()`
5. **Dependence plots** --- variable dependence and partial dependence via
   `gg_variable()` and `gg_partial_rfsrc()`
6. **Classification performance** --- ROC curves and AUC via `gg_roc()`

```{r packages}
library(ggplot2)
library(dplyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())
```

# Data: Iris Flower Measurements

Fisher's iris data [@Fisher:1936] measures sepal and petal length and width
for 150 flowers, 50 each from three species: *setosa*, *versicolor*, and
*virginica*. It is a small data set, and that is a feature here, not a
flaw: every figure in this vignette renders in under a second, and the
structure is well-understood enough that any strange behavior stands out.

```{r data-load}
data(iris)
```

## Exploratory data analysis

Petal measurements separate the three species almost perfectly; sepal
measurements do not.

```{r eda-petal, fig.height=4, fig.cap="Petal length vs. petal width, colored by species."}
ggplot(iris, aes(x = Petal.Length, y = Petal.Width, color = Species)) +
  geom_point(size = 2, alpha = 0.7) +
  scale_color_brewer(palette = "Set2")
```

```{r eda-sepal, fig.height=4, fig.cap="Sepal length vs. sepal width, colored by species."}
ggplot(iris, aes(x = Sepal.Length, y = Sepal.Width, color = Species)) +
  geom_point(size = 2, alpha = 0.7) +
  scale_color_brewer(palette = "Set2")
```

*setosa* sits in its own corner of petal space, clean of the other two.
*versicolor* and *virginica* overlap in both plots, more so on sepals than
petals. Keep that overlap in mind --- it is the interesting case for
everything that follows, since *setosa* is trivial for the forest to get
right.

# Growing a Random Forest

We grow a classification forest using all four predictors. The `rfsrc()`
function detects the classification family from the factor response.

```{r rfsrc-fit}
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris,
                    ntree = 200, importance = TRUE, err.block = 5)
rfsrc_iris
```

The forest grew `r rfsrc_iris$ntree` trees. The confusion matrix confirms
what the EDA suggested: every *setosa* is classified correctly, while
*versicolor* and *virginica* trade three misclassifications each way, for
an overall OOB misclassification rate of 4%.

## OOB error convergence

```{r error-plot, fig.height=4, fig.cap="OOB misclassification rate vs. number of trees, overall and per class."}
plot(gg_error(rfsrc_iris))
```

The *setosa* error drops to zero almost immediately. *versicolor* and
*virginica* settle in around 6% each, and the overall rate levels off near
4% well before 200 trees --- the forest is not still learning by the time we
stop growing it.

## OOB predictions

```{r rfsrc-predicted, fig.height=5, fig.cap="OOB predicted class probabilities, faceted by the flower's true species."}
plot(gg_rfsrc(rfsrc_iris))
```

Each panel is one true species; each point is one flower's predicted
probability for one of the three classes. *setosa* flowers pin their
probability at 1 for *setosa* and 0 for the other two --- no ambiguity at
all. The *versicolor* and *virginica* panels show real spread: most flowers
are still classified confidently, but a handful sit in the 0.2--0.6 range
where the forest is genuinely unsure which of the two they are.

# Variable Selection

## Variable importance (VIMP)

VIMP, computed by [randomForestSRC](https://www.randomforestsrc.org), measures
the increase in OOB prediction error when a variable's values are randomly
permuted across the out-of-bag observations [@Breiman:2001]. For a
classification forest, `gg_vimp()` returns one column per class alongside the
overall score, and `plot()` facets on that automatically.

```{r vimp-plot, fig.cap="VIMP ranking, overall and per class."}
plot(gg_vimp(rfsrc_iris))
```

`Petal.Length` and `Petal.Width` dominate every facet, `Sepal.Length` and
`Sepal.Width` barely register --- consistent with the EDA. Look closely at the
*virginica* facet: `Sepal.Length` has a small negative VIMP there (colored
differently), meaning permuting it very slightly *helped* virginica
predictions in this fit. That kind of disagreement across facets is exactly
what a per-class VIMP breakdown is for; a single overall score would have
hidden it. (This is a distinct question from the permutation-vs-varPro
comparison covered in `vignette("varpro", package = "ggRandomForests")`,
which contrasts *how* importance is measured rather than *for which class*.)

## Minimal depth

Minimal depth [@Ishwaran:2010] ranks variables by how close to the root node
they first split, on average.

```{r varsel}
md_iris <- max.subtree(rfsrc_iris)
```

The threshold is `r round(md_iris$threshold, 2)`, selecting
`r length(md_iris$topvars)` variables:
`r paste(md_iris$topvars, collapse = ", ")`. VIMP and minimal depth agree.

# SHAP Analysis

VIMP ranks importance averaged over the whole forest, and it does not change
with the question you ask. SHAP does: a classification forest predicts one
probability per class, and `gg_shap()` explains exactly one of those
probabilities at a time, chosen with the `which.class` argument. Ask for
class 1 (*setosa*) and you get contributions to the *setosa* probability;
switch to class 3 (*virginica*) and every contribution is recomputed for
*virginica* instead. Nothing else about the method changes --- same players,
same game, different payout to split.

We target *virginica* (`which.class = 3`), the harder of the two overlapping
species. With only four predictors, `kernelshap` runs in exact mode, so
explaining all 150 flowers takes about two seconds --- no need for the
random-sample workaround the regression vignette uses on the larger, higher-
dimensional Boston housing data.

```{r shap-fit}
set.seed(43)
gg_shp <- gg_shap(rfsrc_iris, bg_n = 50, which.class = 3)
```

## SHAP importance

```{r shap-importance-plot, fig.cap="Mean absolute SHAP value per predictor, virginica probability."}
plot(gg_shp, type = "importance")
```

Same two variables on top as VIMP and minimal depth, `Petal.Length` ahead of
`Petal.Width`. Three different mechanisms, one answer for the dominant
variables --- the disagreement worth watching for is further down the
ranking, not at the top.

## SHAP beeswarm

```{r shap-beeswarm-plot, fig.cap="SHAP beeswarm for virginica probability: every dot is one flower's contribution for one predictor."}
plot(gg_shp, type = "beeswarm")
```

`Petal.Width` shows a clean gradient: the yellow (wide petal) dots sit on the
positive side, pushing the predicted *virginica* probability up; the purple
(narrow petal) dots sit on the negative side, pulling it down. That is
exactly what you would expect from the EDA scatter --- *virginica* has the
widest petals of the three species.

## SHAP dependence

```{r shap-dependence-plot, fig.cap="SHAP dependence for Petal.Width, virginica probability."}
plot(gg_shp, type = "dependence", xvar = "Petal.Width")
```

The upward slope states the same relationship as a number instead of a
color: as `Petal.Width` increases, its contribution to the *virginica*
probability climbs from clearly negative to clearly positive, crossing zero
around the point where *versicolor* and *virginica* petals start to overlap
in the EDA plot.

# Variable Dependence

## Variable dependence plots

Variable dependence shows each flower's OOB predicted class probability
plotted against a predictor. For a classification forest, `gg_variable()`
returns one probability column per class and `plot()` facets on it, the same
way `gg_vimp()` does.

```{r vardep-panel, fig.cap="Variable dependence for Petal.Width, faceted by predicted-class probability."}
gg_v <- gg_variable(rfsrc_iris)

plot(gg_v, xvar = "Petal.Width")
```

Three curves, three stories: *setosa*'s probability collapses to zero past a
petal width of about 0.6, *versicolor* peaks in the middle of the range and
falls off on both sides, and *virginica* rises steadily from zero starting
around 1.0 --- the same three-panel pattern the beeswarm and dependence plots
already showed for *virginica* alone, now visible for all three classes at
once.

## Partial dependence

Partial dependence integrates out the effects of all other covariates, giving
a risk-adjusted view of each predictor's marginal effect on one class's
predicted probability [@Friedman:2000]. Unlike `gg_shap()`, `gg_partial_rfsrc()`
has no `which.class` argument for classification forests; it defers to
`randomForestSRC::partial.rfsrc()`'s own default, which targets the first
factor level, *setosa* here.

```{r partial-dep, fig.cap="Partial dependence for petal measurements, setosa probability."}
pd <- gg_partial_rfsrc(rfsrc_iris, xvar.names = c("Petal.Length", "Petal.Width"))
plot(pd)
```

Both curves fall sharply and then flatten near zero --- the risk-adjusted
version of what the EDA already showed: past a certain petal size, a flower
is essentially never *setosa*. Getting the same kind of curve for
*versicolor* or *virginica* instead means going back to `gg_shap()`:
re-run it with `which.class = 2` or `3`, then `shap_dependence()` on that
new object picks up wherever `which.class` left off.

# Classification Performance: ROC and AUC

ROC curves and AUC have no equivalent in the regression vignette --- they are
specific to classification, where "how well does the forest separate this
class from the rest" is itself a well-posed question. `gg_roc()` computes the
curve for one class at a time, indexed the same way as `gg_shap()`'s
`which.class`, just under the name `which_outcome`.

```{r roc-plot, fig.height=4, fig.cap="ROC curve for virginica vs. the rest."}
roc_virginica <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(roc_virginica)
```

```{r auc-calc}
calc_auc(roc_virginica)
```

An AUC of `r round(calc_auc(roc_virginica), 3)` for *virginica* confirms the
forest separates it well, even given its overlap with *versicolor*. *setosa*'s ROC
curve is not worth plotting --- it is a right angle, AUC 1, the geometric
version of the confusion matrix's perfect row.

# Conclusion

We have walked a full random forest classification analysis with
**randomForestSRC** and **ggRandomForests**, and the pieces line up:

- `gg_error()` showed *setosa* converging immediately, *versicolor* and
  *virginica* settling near 6% each.
- VIMP (`gg_vimp()`) and minimal depth (`max.subtree()`) agreed:
  `Petal.Length` and `Petal.Width` dominate, for every class.
- `gg_shap()` gave the same top variables, but added something VIMP cannot:
  a signed, additive, per-flower contribution to one class's probability,
  chosen with `which.class`.
- `gg_variable()` and `gg_partial_rfsrc()` traced the marginal and
  risk-adjusted versions of the same curves the SHAP dependence plot already
  hinted at.
- `gg_roc()` and `calc_auc()` closed the loop with the classification-specific
  question SHAP and VIMP do not answer directly: how well does the forest
  separate each class from the rest.

The pattern from the regression vignette holds here too. Each `gg_*()`
function returns a tidy object; the plotting is a separate step. Use the
package's `plot()` methods when the default figure is what you want, and
reach for `ggplot2` directly when it is not.

# References

---
title: "Random Forest Survival Analysis with ggRandomForests"
author: "John Ehrlinger"
date: today
format:
  html:
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Random Forest Survival Analysis with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

::: {.callout-warning}
## Work in progress
This vignette is under active development. Code examples and narrative may
change before the next release.
:::

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)
```

# Introduction

Random forests [@Breiman:2001] are a non-parametric ensemble method that
requires no distributional assumptions on the relation between covariates and
the response. Random survival forests (RSF) [@Ishwaran:2007a; @Ishwaran:2008]
extend the method to right-censored, time-to-event data by growing trees with a
log-rank splitting rule and aggregating Kaplan--Meier estimates within terminal
nodes.

The **randomForestSRC** package [@Ishwaran:RFSRC:2014] provides a unified
implementation for survival, regression, and classification forests.
**ggRandomForests** extracts tidy data objects from `rfsrc` fits and renders
them with **ggplot2** [@Wickham:2009], making it straightforward to explore how
a forest is constructed, which variables matter, and how the response depends on
individual predictors.

This vignette demonstrates a complete random survival forest workflow on the
primary biliary cirrhosis (PBC) data set [@fleming:1991]:

1. **Data preparation and exploration** --- cleaning, EDA, Kaplan--Meier curves
2. **Growing the forest** --- fitting an RSF, checking convergence and OOB error
3. **Variable selection** --- VIMP and minimal depth via `max.subtree()`
4. **Dependence plots** --- variable dependence and partial dependence via
   `gg_variable()` and `gg_partial_rfsrc()`
5. **Variable interactions** --- conditioning plots and interactive 3-D partial
   dependence surfaces with **plotly**

```{r packages}
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)
library(survival)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())

# Plotting constants
event_marks  <- c(1, 4)
event_labels <- c("Censored", "Death")
event_colors <- c("steelblue", "firebrick")
```

# Data: Primary Biliary Cirrhosis (PBC)

The PBC study consists of 424 patients referred to Mayo Clinic between 1974 and
1984, of whom 312 were randomized into a trial of D-penicillamine (DPCA) versus
placebo. The data is described in @fleming:1991 Chapter 0.2, with a proportional
hazards model developed in Chapter 4.4. We use the copy bundled with
**randomForestSRC**.

```{r data-load}
data("pbc", package = "randomForestSRC")
```

## Data cleaning

We convert `days` to `years`, recode `treatment` as a factor, and coerce
low-cardinality numeric columns (five or fewer unique values, including binary
0/1 indicators) to factors. We avoid converting binary columns to `logical`
because `randomForestSRC::partial.rfsrc()` does not handle logical predictors
correctly in survival forests.

```{r data-clean}
pbc <- pbc |>
  mutate(
    years     = days / 365.25,
    age       = age / 365.25,
    treatment = factor(
      ifelse(treatment == 1, "DPCA",
             ifelse(treatment == 2, "Placebo", NA)),
      levels = c("DPCA", "Placebo")
    )
  ) |>
  select(-days)

# Low-cardinality numerics (including binary 0/1) to factor.
# NOTE: do NOT convert to logical — partial.rfsrc() fails with logical
# predictors in survival forests (randomForestSRC <= 3.5.1).
# Exclude the response columns (status, years) from conversion.
resp_cols <- c("status", "years")
for (nm in setdiff(names(pbc), resp_cols)) {
  v <- pbc[[nm]]
  if (is.numeric(v) && !is.factor(v) && length(unique(v[!is.na(v)])) <= 5) {
    pbc[[nm]] <- factor(v)
  }
}
```

```{r variable-labels}
# Human-readable labels for plot axes
st_labs <- c(
  status       = "Death Event",
  treatment    = "Treatment",
  age          = "Age (years)",
  sex          = "Female",
  ascites      = "Ascites",
  hepato       = "Hepatomegaly",
  spiders      = "Spiders",
  edema        = "Edema (0, 0.5, 1)",
  bili         = "Serum Bilirubin (mg/dl)",
  chol         = "Serum Cholesterol (mg/dl)",
  albumin      = "Albumin (gm/dl)",
  copper       = "Urine Copper (ug/day)",
  alk.phos     = "Alkaline Phosphatase (U/liter)",
  ast          = "SGOT (U/ml)",
  trig         = "Triglycerides (mg/dl)",
  platelet     = "Platelets (per cubic ml/1000)",
  prothrombin  = "Prothrombin Time (sec)",
  stage        = "Histologic Stage",
  years        = "Follow-up Time (years)"
)
```

## Exploratory data analysis

Good practice before modeling: scan categorical variables as stacked histograms
over follow-up time, and continuous variables as scatter plots colored by event
status.

```{r eda-categorical, fig.height=4, fig.cap="EDA for categorical variables. Bar color indicates factor level; white = missing."}
cls <- sapply(pbc, class)
cnt_idx <- which(cls %in% c("numeric", "integer"))
fct_idx <- setdiff(seq_along(pbc), cnt_idx)
fct_idx <- union(fct_idx, which(names(pbc) == "years"))

dta_cat <- suppressWarnings(
  pbc[, fct_idx] |>
    pivot_longer(-years, names_to = "variable", values_to = "value",
                 values_transform = list(value = as.character))
)

ggplot(dta_cat, aes(x = years, fill = value)) +
  geom_histogram(color = "black", binwidth = 1) +
  labs(y = "", x = st_labs["years"]) +
  scale_fill_brewer(palette = "RdBu", na.value = "white") +
  facet_wrap(~variable, scales = "free_y", nrow = 2) +
  theme(legend.position = "none")
```

```{r eda-continuous, fig.height=4, fig.cap="EDA for continuous variables. Points colored by death event; rug marks indicate missing values."}
cnt_idx <- union(cnt_idx, which(names(pbc) == "status"))
dta_cont <- pbc[, cnt_idx] |>
  pivot_longer(c(-years, -status),
               names_to = "variable", values_to = "value")

ggplot(dta_cont |> filter(!is.na(value)),
       aes(x = years, y = value, color = factor(status), shape = factor(status))) +
  geom_point(alpha = 0.4) +
  geom_rug(data = dta_cont |> filter(is.na(value)), color = "grey50") +
  labs(y = "", x = st_labs["years"], color = "Death", shape = "Death") +
  scale_color_manual(values = event_colors) +
  scale_shape_manual(values = event_marks) +
  facet_wrap(~variable, scales = "free_y", ncol = 4) +
  theme(legend.position = c(0.8, 0.2))
```

Look for patterns of missingness (white bars, rug marks) and extreme values that
fall outside the biological range.

## Kaplan--Meier survival by treatment

We restrict to the 312 trial patients and construct KM curves with `gg_survival`.

```{r km-survival, fig.cap="KM survival estimates by treatment group with 95% confidence bands."}
pbc_trial <- pbc |> filter(!is.na(treatment))
pbc_test  <- pbc |> filter(is.na(treatment))

gg_km <- gg_survival(interval = "years", censor = "status",
                     by = "treatment", data = pbc_trial,
                     conf.int = 0.95)

plot(gg_km) +
  labs(y = "Survival Probability", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.2)) +
  coord_cartesian(ylim = c(0, 1.01))
```

The curves largely overlap, consistent with the original finding that DPCA
offered no clear survival benefit over placebo [@fleming:1991].

```{r km-cumhaz, fig.cap="KM cumulative hazard estimates by treatment group."}
plot(gg_km, type = "cum_haz") +
  labs(y = "Cumulative Hazard", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.8)) +
  coord_cartesian(ylim = c(-0.02, 1.22))
```

We can also stratify on continuous variables. Here we reproduce the bilirubin
groupings from @fleming:1991 Figure 4.4.2:

```{r km-bili, fig.width=6, fig.cap="KM survival stratified by bilirubin groups."}
pbc_bili <- pbc_trial |>
  mutate(bili_grp = cut(bili, breaks = c(0, 0.8, 1.3, 3.4, 29)))

plot(gg_survival(interval = "years", censor = "status",
                 by = "bili_grp", data = pbc_bili),
     error = "none") +
  labs(y = "Survival Probability", x = "Time (years)", color = "Bilirubin")
```

Higher bilirubin strongly predicts worse survival --- an effect the random forest
will rediscover without any prior specification.

# Growing a Random Survival Forest

Several predictors in the PBC trial data contain missing values (cholesterol,
copper, triglycerides, and others). We handle these with a two-step approach:
first impute using `impute.rfsrc()`, which uses the random forest proximity
structure to fill in missing values, then fit the survival forest on the
complete imputed data. This keeps the fitted forest object free of imputation
state, which is required for `partial.rfsrc()` to work correctly.

```{r rfsrc-fit}
# Step 1: impute missing values via random forest proximity
pbc_imputed <- impute.rfsrc(Surv(years, status) ~ .,
                             data    = pbc_trial,
                             ntree   = 500,
                             nimpute = 2)

# Step 2: grow the survival forest on the complete imputed data
rfsrc_pbc <- rfsrc(Surv(years, status) ~ .,
                   data      = pbc_imputed,
                   nsplit    = 10,
                   tree.err  = TRUE,
                   importance = TRUE)
```

The forest grew `r rfsrc_pbc$ntree` trees, splitting on
`r rfsrc_pbc$mtry` randomly selected candidate variables at each node, and
stopping at a minimum terminal node size of `r rfsrc_pbc$nodesize`.

## OOB error convergence

```{r error-plot, fig.height=4, fig.cap="OOB prediction error vs. number of trees."}
plot(gg_error(rfsrc_pbc))
```

The error stabilizes well before 1000 trees, indicating the forest is large
enough for reliable predictions.

## OOB predicted survival

```{r rfsrc-predicted, fig.cap="OOB predicted survival curves. Blue = censored, red = death events."}
gg_rsf <- plot(gg_rfsrc(rfsrc_pbc), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
gg_rsf
```

Each curve represents one patient's OOB prediction, extended to the last
follow-up time. Red (death event) curves generally fall faster, confirming the
forest discriminates between risk groups.

```{r rfsrc-by-treatment, fig.cap="Median predicted survival by treatment group with 95% confidence bands."}
plot(gg_rfsrc(rfsrc_pbc, by = "treatment")) +
  theme(legend.position = c(0.2, 0.2)) +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

## Test set predictions

The non-trial patients (`pbc_test`) have substantial missing data.
`predict.rfsrc()` handles these transparently at prediction time via
`na.action = "na.impute"` — this is distinct from training-time imputation
and does not affect the fitted forest object.

```{r predict-test, fig.cap="Predicted survival for non-trial patients (test set)."}
rfsrc_pbc_test <- predict(rfsrc_pbc, newdata = pbc_test,
                          na.action = "na.impute",
                          importance = TRUE)

plot(gg_rfsrc(rfsrc_pbc_test), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

Because the training curves use OOB estimates, both plots represent
out-of-sample predictions and are directly comparable.

# Variable Selection

Random forest uses all available predictors. To understand which matter most, we
examine variable importance (VIMP) and minimal depth.

## Variable importance (VIMP)

VIMP measures the increase in prediction error when a variable is randomly
permuted. Large positive values mean the variable is essential for accuracy;
negative values suggest noise is more informative than the variable.

```{r vimp-plot, fig.width=6, fig.cap="Variable importance ranking. Blue = positive VIMP, red = negative."}
plot(gg_vimp(rfsrc_pbc), lbls = st_labs) +
  theme(legend.position = c(0.8, 0.2)) +
  labs(fill = "VIMP > 0")
```

Bilirubin ranks highest, followed by copper, prothrombin time, albumin, and age
--- closely matching the variables selected in the @fleming:1991 proportional
hazards model.

## Minimal depth

Minimal depth [@Ishwaran:2010] ranks variables by how close to the root node
they first split, on average. Variables that partition large portions of the
population early are considered most important.

```{r varsel}
md_pbc <- max.subtree(rfsrc_pbc)
```

The `max.subtree()` function computes minimal depth for each variable. The
threshold is `r round(md_pbc$threshold, 2)`, selecting
`r length(md_pbc$topvars)` variables: `r paste(md_pbc$topvars, collapse = ", ")`.

Both selection methods agree on the key predictors: `bili`, `albumin`, `copper`,
`prothrombin`, and `age`. We add `edema` (selected by the @fleming:1991 model)
for the remainder of the analysis.

```{r select-vars}
xvar     <- c("bili", "albumin", "copper", "prothrombin", "age")
xvar_cat <- "edema"
xvar_all <- c(xvar, xvar_cat)
```

# Variable Dependence

## Variable dependence plots

Variable dependence shows each patient's predicted survival at a given time
horizon plotted against a predictor of interest. Points are colored by event
status; a loess smooth indicates the trend.

```{r vardep-bili, fig.height=4, fig.cap="Variable dependence of survival at 1 and 3 years on bilirubin."}
gg_v <- gg_variable(rfsrc_pbc, time = c(1, 3),
                    time.labels = c("1 Year", "3 Years"))

plot(gg_v, xvar = "bili", alpha = 0.4) +
  labs(y = "Survival", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

Survival drops sharply with increasing bilirubin, and the 3-year curve drops
further than the 1-year curve, suggesting a non-proportional hazards effect.

```{r vardep-panel, fig.height=5, fig.cap="Variable dependence at 1 and 3 years for continuous predictors."}
plot(gg_v, xvar = xvar[-1], panel = TRUE, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.05, 1.05))
```

The plots confirm survival increases with albumin and decreases with copper,
prothrombin time, and age. The divergence between time curves for `copper`
further supports a non-proportional hazards mechanism.

```{r vardep-categorical, fig.height=4, fig.cap="Variable dependence on edema (categorical)."}
plot(gg_v, xvar = xvar_cat, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.02))
```

Patients with edema = 1 (edema despite diuretics) have markedly lower predicted
survival.

## Partial dependence

::: {.callout-warning}
**Known issue (draft):** `randomForestSRC::partial.rfsrc()` currently fails for
survival forests in randomForestSRC ≥ 3.3.  The partial dependence and
interactive surface sections below will show an error until this upstream bug
is resolved.  All other sections of this vignette are fully functional.
:::

Partial dependence integrates out the effects of other covariates, giving a
risk-adjusted view of how each predictor influences the response
[@Friedman:2000]. We use `gg_partial_rfsrc()` which calls
`randomForestSRC::partial.rfsrc()` directly and returns a `gg_partial_rfsrc`
object.  For survival forests, the result includes a `time` column so
`plot(pd)` produces one curve per predictor value over time, faceted by
variable name.

```{r partial-dep, error=TRUE, fig.height=5, fig.cap="Partial dependence of survival at approximately 1 and 3 years on continuous predictors."}
# partial.rfsrc() requires times that match the model's time.interest grid;
# gg_partial_rfsrc() snaps the requested values to the nearest observed times.
ti   <- rfsrc_pbc$time.interest
t1yr <- ti[which.min(abs(ti - 1))]
t3yr <- ti[which.min(abs(ti - 3))]

pd <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = xvar,
                       partial.time = c(t1yr, t3yr))

# Quick S3 plot — survival forests produce time-series curves per predictor value
plot(pd)
```

For a publication-ready layout with custom colour scale, access `pd$continuous`
directly:

```{r partial-dep-custom, error=TRUE, fig.height=5, fig.cap="Partial dependence (custom styling)."}
ggplot(pd$continuous, aes(x = x, y = yhat,
                          color = factor(round(time, 2)),
                          group = factor(time))) +
  geom_line(linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = "Predicted Survival", x = "", color = "Year") +
  scale_color_manual(values = setNames(c("steelblue", "firebrick"),
                                       as.character(round(c(t1yr, t3yr), 2)))) +
  theme_bw()
```

The partial dependence curves at approximately 1 and 3 years confirm the
variable dependence findings and support the log-transforms used in the
@fleming:1991 model for `bili`, `albumin`, and `prothrombin`. The divergence
between time curves for `bili` and `copper` supports non-proportional hazards.

## Conditional dependence

To investigate interactions, we can condition variable dependence on group
membership in another variable. Here we examine the dependence of survival on
bilirubin, stratified by edema group.

```{r coplot-edema, fig.width=7, fig.height=4, fig.cap="Variable dependence on bilirubin, conditional on edema group."}
gg_v1 <- gg_variable(rfsrc_pbc, time = 1)
gg_v1$edema <- paste("edema =", gg_v1$edema)

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_grid(~edema) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

The decreasing trend with bilirubin holds across all edema groups, but survival
is uniformly lower in the edema = 1 panel, confirming an additive effect.

We can also condition on a continuous variable by binning into quantile groups:

```{r coplot-albumin, fig.width=7, fig.height=5, fig.cap="Variable dependence on bilirubin, conditional on albumin groups."}
albumin_cts <- quantile_pts(gg_v1$albumin, groups = 6, intervals = TRUE)
gg_v1$albumin_grp <- cut(gg_v1$albumin, breaks = albumin_cts)
levels(gg_v1$albumin_grp) <- paste("albumin =",
                                    levels(gg_v1$albumin_grp))

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_wrap(~albumin_grp) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

The effect of bilirubin attenuates at higher albumin levels, suggesting an
interaction between these two liver function markers.

# Interactive Partial Dependence Surfaces

For a richer view of the interaction between bilirubin and albumin, we construct
a partial dependence surface. We compute partial dependence on a grid of 25
albumin values, each evaluated at 25 bilirubin points.

```{r surface-data, error=TRUE, cache=FALSE}
# Create grid of albumin values
alb_grid <- quantile_pts(pbc_trial$albumin, groups = 25)

# For each albumin value, compute partial dependence on bili at ~1 year
surface_list <- lapply(alb_grid, function(alb_val) {
  newx <- pbc_trial[, rfsrc_pbc$xvar.names, drop = FALSE]
  newx$albumin <- alb_val
  pd_alb <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = "bili",
                              newx = newx, partial.time = t1yr)
  df <- pd_alb$continuous
  df$albumin <- alb_val
  df
})

surface_df <- bind_rows(surface_list)
```

```{r plotly-surface, error=TRUE, fig.cap="Interactive partial dependence surface: survival as a function of bilirubin and albumin."}
if (!exists("surface_df")) {
  message("surface_df not available — skipping plotly surface (see surface-data chunk error above).")
} else if (requireNamespace("plotly", quietly = TRUE)) {
  # Reshape for surface
  library(plotly)

  surface_wide <- surface_df |>
    select(bili = x, albumin, survival = yhat) |>
    arrange(albumin, bili)

  # Create matrix form
  bili_vals <- sort(unique(surface_wide$bili))
  alb_vals  <- sort(unique(surface_wide$albumin))
  z_matrix  <- matrix(surface_wide$survival,
                       nrow = length(alb_vals),
                       ncol = length(bili_vals),
                       byrow = TRUE)

  plot_ly(x = bili_vals, y = alb_vals, z = z_matrix) |>
    add_surface(colorscale = "Viridis", showscale = TRUE) |>
    layout(
      scene = list(
        xaxis = list(title = "Bilirubin"),
        yaxis = list(title = "Albumin"),
        zaxis = list(title = "Survival")
      )
    )
} else {
  message("Install the plotly package for interactive 3D surfaces.")
  # Fallback: contour plot with ggplot2
  ggplot(surface_df, aes(x = x, y = albumin, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Survival") +
    labs(x = "Bilirubin", y = "Albumin") +
    theme_bw()
}
```

The surface shows that survival is highest when bilirubin is low and albumin is
high (upper-left corner), and drops steeply as bilirubin increases or albumin
decreases. The non-planar shape of the surface --- particularly the steep
gradient at low albumin and high bilirubin --- confirms the interaction detected
in the conditional plots.

## Brier Score and CRPS

The Brier score measures prediction accuracy at each point on the event-time
grid by comparing predicted survival probabilities to observed outcomes, with
inverse-probability-of-censoring weighting (IPCW) to account for
right-censoring [@graf:1999]. `gg_brier()` wraps
`randomForestSRC::get.brier.survival()` and returns a tidy data frame ready
for plotting.

```{r brier-overall, fig.width=7, fig.height=3.5, fig.cap="Time-resolved Brier score for the PBC survival forest."}
gg_bs <- gg_brier(rfsrc_pbc)
plot(gg_bs)
```

The score starts near zero (all subjects alive, predictions trivially correct),
peaks around the median event time where uncertainty is greatest, then
decreases as the remaining at-risk pool shrinks.

The 15--85% per-subject envelope shows how spread the individual Brier
contributions are at each time. A narrow ribbon indicates the forest assigns
similar predicted risks across subjects; a wide ribbon points to
heterogeneous risk predictions.

```{r brier-envelope, fig.width=7, fig.height=3.5, fig.cap="Brier score with 15--85% per-subject envelope."}
plot(gg_bs, envelope = TRUE)
```

The running CRPS (continuous ranked probability score) integrates the Brier
score over time, normalised by elapsed time. It provides a single
time-averaged accuracy measure that penalises poorly calibrated predictions
throughout the follow-up period.

```{r brier-crps, fig.width=7, fig.height=3.5, fig.cap="Running CRPS for the PBC survival forest."}
plot(gg_bs, type = "crps")
```

The integrated CRPS --- a scalar summary of overall calibration --- is stored
as an attribute and can be retrieved with:

```{r brier-crps-scalar}
attr(gg_bs, "crps_integrated")
```

# Conclusion

This vignette demonstrated a complete random survival forest analysis using
**randomForestSRC** and **ggRandomForests**:

- **Kaplan--Meier curves** via `gg_survival()` confirmed no treatment effect in
  the PBC trial, consistent with @fleming:1991.
- **OOB error** via `gg_error()` showed the forest stabilized quickly.
- **VIMP** via `gg_vimp()` and **minimal depth** via `max.subtree()` both
  identified bilirubin, albumin, copper, prothrombin, and age as key
  predictors --- matching the proportional hazards model.
- **Variable dependence** via `gg_variable()` revealed non-proportional hazards
  effects for bilirubin and copper.
- **Partial dependence** via `gg_partial_rfsrc()` provided risk-adjusted
  confirmation and supported the log-transforms used in parametric models.
- **Conditional plots** and **interactive surfaces** exposed the bilirubin--albumin
  interaction.
- **Brier score and CRPS** via `gg_brier()` quantified time-resolved prediction
  accuracy and overall calibration of the survival forest.

The **ggRandomForests** design separates data extraction from plotting: every
`gg_*()` function returns a tidy data frame that can be plotted with the
package's `plot()` methods or fed directly into custom `ggplot2` workflows.

# References

---
title: "Random Forest Survival Analysis with ggRandomForests"
author: "John Ehrlinger"
date: today
format:
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Random Forest Survival Analysis with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

::: {.callout-warning}
## Work in progress
This vignette is under active development. Code examples and narrative may
change before the next release.
:::

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)
```

# Introduction

Random forests [@Breiman:2001] are a non-parametric ensemble method that
requires no distributional assumptions on the relation between covariates and
the response. Random survival forests (RSF) [@Ishwaran:2007a; @Ishwaran:2008]
extend the method to right-censored, time-to-event data by growing trees with a
log-rank splitting rule and aggregating Kaplan--Meier estimates within terminal
nodes.

The forest returns three ensemble quantities for each subject. First, the
survival function $\hat{S}(t)$: the probability of surviving past time $t$,
bounded between 0 and 1. Second, the cumulative hazard function (CHF):
$\hat{H}(t) = -\log \hat{S}(t)$, an unbounded, monotone-increasing summary of
accumulated risk. Third, mortality: the expected number of events a subject
would accumulate if followed indefinitely under their covariate profile,
computed by summing the CHF over the observed time grid. Mortality is an
unbounded relative-risk score, not a probability, and works well as a single
scalar for ranking patients by risk.

The **[randomForestSRC](https://www.randomforestsrc.org)** package
[@Ishwaran:RFSRC:2014] provides a unified implementation for survival,
regression, and classification forests.
**ggRandomForests** extracts tidy data objects from `rfsrc` fits and renders
them with **ggplot2** [@Wickham:2009], making it straightforward to explore how
a forest is constructed, which variables matter, and how the response depends on
individual predictors.

This vignette demonstrates a complete random survival forest workflow on the
primary biliary cirrhosis (PBC) data set [@fleming:1991]:

1. **Data preparation and exploration** --- cleaning, EDA, Kaplan--Meier curves
2. **Growing the forest** --- fitting an RSF, checking convergence and OOB error
3. **Variable selection** --- VIMP and minimal depth via `max.subtree()`
4. **Dependence plots** --- variable dependence and partial dependence via
   `gg_variable()` and `gg_partial_rfsrc()`
5. **Variable interactions** --- conditioning plots and partial dependence
   surfaces

```{r packages}
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)
library(survival)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())

# Plotting constants
event_marks  <- c(1, 4)
event_labels <- c("Censored", "Death")
event_colors <- c("steelblue", "firebrick")
```

# Data: Primary Biliary Cirrhosis (PBC)

The PBC study consists of 424 patients referred to Mayo Clinic between 1974 and
1984, of whom 312 were randomized into a trial of D-penicillamine (DPCA) versus
placebo. The data is described in @fleming:1991 Chapter 0.2, with a proportional
hazards model developed in Chapter 4.4. We use the copy bundled with
**randomForestSRC**.

```{r data-load}
data("pbc", package = "randomForestSRC")
```

## Data cleaning

We convert `days` to `years`, recode `treatment` as a factor, and coerce
low-cardinality numeric columns (five or fewer unique values, including binary
0/1 indicators) to factors. We avoid converting binary columns to `logical`
because `randomForestSRC::partial.rfsrc()` does not handle logical predictors
correctly in survival forests.

```{r data-clean}
pbc <- pbc |>
  mutate(
    years     = days / 365.25,
    age       = age / 365.25,
    treatment = factor(
      ifelse(treatment == 1, "DPCA",
             ifelse(treatment == 2, "Placebo", NA)),
      levels = c("DPCA", "Placebo")
    )
  ) |>
  select(-days)

# Low-cardinality numerics (including binary 0/1) to factor.
# NOTE: do NOT convert to logical — partial.rfsrc() fails with logical
# predictors in survival forests (randomForestSRC <= 3.5.1).
# Exclude the response columns (status, years) from conversion.
resp_cols <- c("status", "years")
for (nm in setdiff(names(pbc), resp_cols)) {
  v <- pbc[[nm]]
  if (is.numeric(v) && !is.factor(v) && length(unique(v[!is.na(v)])) <= 5) {
    pbc[[nm]] <- factor(v)
  }
}
```

```{r variable-labels}
# Human-readable labels for plot axes
st_labs <- c(
  status       = "Death Event",
  treatment    = "Treatment",
  age          = "Age (years)",
  sex          = "Female",
  ascites      = "Ascites",
  hepato       = "Hepatomegaly",
  spiders      = "Spiders",
  edema        = "Edema (0, 0.5, 1)",
  bili         = "Serum Bilirubin (mg/dl)",
  chol         = "Serum Cholesterol (mg/dl)",
  albumin      = "Albumin (gm/dl)",
  copper       = "Urine Copper (ug/day)",
  alk.phos     = "Alkaline Phosphatase (U/liter)",
  ast          = "SGOT (U/ml)",
  trig         = "Triglycerides (mg/dl)",
  platelet     = "Platelets (per cubic ml/1000)",
  prothrombin  = "Prothrombin Time (sec)",
  stage        = "Histologic Stage",
  years        = "Follow-up Time (years)"
)
```

## Exploratory data analysis

Good practice before modeling: scan categorical variables as stacked histograms
over follow-up time, and continuous variables as scatter plots colored by event
status.

```{r eda-categorical, fig.height=4, fig.cap="EDA for categorical variables. Bar color indicates factor level; white = missing."}
cls <- sapply(pbc, class)
cnt_idx <- which(cls %in% c("numeric", "integer"))
fct_idx <- setdiff(seq_along(pbc), cnt_idx)
fct_idx <- union(fct_idx, which(names(pbc) == "years"))

dta_cat <- suppressWarnings(
  pbc[, fct_idx] |>
    pivot_longer(-years, names_to = "variable", values_to = "value",
                 values_transform = list(value = as.character))
)

ggplot(dta_cat, aes(x = years, fill = value)) +
  geom_histogram(color = "black", binwidth = 1) +
  labs(y = "", x = st_labs["years"]) +
  scale_fill_brewer(palette = "RdBu", na.value = "white") +
  facet_wrap(~variable, scales = "free_y", nrow = 2) +
  theme(legend.position = "none")
```

```{r eda-continuous, fig.height=4, fig.cap="EDA for continuous variables. Points colored by death event; rug marks indicate missing values."}
cnt_idx <- union(cnt_idx, which(names(pbc) == "status"))
dta_cont <- pbc[, cnt_idx] |>
  pivot_longer(c(-years, -status),
               names_to = "variable", values_to = "value")

ggplot(dta_cont |> filter(!is.na(value)),
       aes(x = years, y = value, color = factor(status), shape = factor(status))) +
  geom_point(alpha = 0.4) +
  geom_rug(data = dta_cont |> filter(is.na(value)), color = "grey50") +
  labs(y = "", x = st_labs["years"], color = "Death", shape = "Death") +
  scale_color_manual(values = event_colors) +
  scale_shape_manual(values = event_marks) +
  facet_wrap(~variable, scales = "free_y", ncol = 4) +
  theme(legend.position = c(0.8, 0.2))
```

Look for patterns of missingness (white bars, rug marks) and extreme values that
fall outside the biological range.

## Kaplan--Meier survival by treatment

The Kaplan--Meier estimate is the marginal survival curve with no covariate
adjustment. It serves as the no-covariate baseline: once we fit a forest,
comparing the KM curves to the forest's OOB predictions lets us judge how much
predictive signal the covariates add beyond the raw event rate.

We restrict to the 312 trial patients and construct KM curves with `gg_survival`.

```{r km-survival, fig.cap="KM survival estimates by treatment group with 95% confidence bands."}
pbc_trial <- pbc |> filter(!is.na(treatment))
pbc_test  <- pbc |> filter(is.na(treatment))

gg_km <- gg_survival(interval = "years", censor = "status",
                     by = "treatment", data = pbc_trial,
                     conf.int = 0.95)

plot(gg_km) +
  labs(y = "Survival Probability", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.2)) +
  coord_cartesian(ylim = c(0, 1.01))
```

The curves largely overlap, consistent with the original finding that DPCA
offered no clear survival benefit over placebo [@fleming:1991].

```{r km-cumhaz, fig.cap="KM cumulative hazard estimates by treatment group."}
plot(gg_km, type = "cum_haz") +
  labs(y = "Cumulative Hazard", x = "Time (years)",
       color = "Treatment", fill = "Treatment") +
  theme(legend.position = c(0.2, 0.8)) +
  coord_cartesian(ylim = c(-0.02, 1.22))
```

We can also stratify on continuous variables. Here we reproduce the bilirubin
groupings from @fleming:1991 Figure 4.4.2:

```{r km-bili, fig.width=6, fig.cap="KM survival stratified by bilirubin groups."}
pbc_bili <- pbc_trial |>
  mutate(bili_grp = cut(bili, breaks = c(0, 0.8, 1.3, 3.4, 29)))

plot(gg_survival(interval = "years", censor = "status",
                 by = "bili_grp", data = pbc_bili),
     error = "none") +
  labs(y = "Survival Probability", x = "Time (years)", color = "Bilirubin")
```

Higher bilirubin strongly predicts worse survival --- an effect the random forest
will rediscover without any prior specification.

# Growing a Random Survival Forest

Several predictors in the PBC trial data contain missing values (cholesterol,
copper, triglycerides, and others). We handle these with a two-step approach:
first impute using `impute.rfsrc()`, which uses the random forest proximity
structure to fill in missing values, then fit the survival forest on the
complete imputed data. This keeps the fitted forest object free of imputation
state, which is required for `partial.rfsrc()` to work correctly.

```{r rfsrc-fit}
# Step 1: impute missing values via random forest proximity
pbc_imputed <- impute.rfsrc(Surv(years, status) ~ .,
                             data    = pbc_trial,
                             ntree   = 100,
                             nimpute = 2)

# Step 2: grow the survival forest on the complete imputed data
rfsrc_pbc <- rfsrc(Surv(years, status) ~ .,
                   data      = pbc_imputed,
                   ntree     = 150,
                   nsplit    = 10,
                   tree.err  = TRUE,
                   importance = TRUE)
```

The forest grew `r rfsrc_pbc$ntree` trees, splitting on
`r rfsrc_pbc$mtry` randomly selected candidate variables at each node, and
stopping at a minimum terminal node size of `r rfsrc_pbc$nodesize`.

## OOB error convergence

```{r error-plot, fig.height=4, fig.cap="OOB prediction error vs. number of trees."}
plot(gg_error(rfsrc_pbc))
```

The error stabilizes well before 1000 trees, indicating the forest is large
enough for reliable predictions.

## OOB predicted survival

```{r rfsrc-predicted, fig.cap="OOB predicted survival curves. Blue = censored, red = death events."}
gg_rsf <- plot(gg_rfsrc(rfsrc_pbc), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
gg_rsf
```

Each curve is one patient's OOB ensemble survival function $\hat{S}(t)$,
extended to the last follow-up time. The forest never saw that patient when
building its prediction, so these are genuine out-of-sample estimates. Red
(death event) curves generally fall faster and reach lower survival
probabilities, confirming the forest separates risk groups. Comparing this
spread to the marginal KM curves from the previous section shows how much the
covariates tighten risk stratification.

```{r rfsrc-by-treatment, fig.cap="Median predicted survival by treatment group with 95% confidence bands."}
plot(gg_rfsrc(rfsrc_pbc, by = "treatment")) +
  theme(legend.position = c(0.2, 0.2)) +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

## Test set predictions

The non-trial patients (`pbc_test`) have substantial missing data.
`predict.rfsrc()` handles these transparently at prediction time via
`na.action = "na.impute"` — this is distinct from training-time imputation
and does not affect the fitted forest object.

```{r predict-test, fig.cap="Predicted survival for non-trial patients (test set)."}
rfsrc_pbc_test <- predict(rfsrc_pbc, newdata = pbc_test,
                          na.action = "na.impute",
                          importance = TRUE)

plot(gg_rfsrc(rfsrc_pbc_test), alpha = 0.2) +
  scale_color_manual(values = event_colors) +
  theme(legend.position = "none") +
  labs(y = "Survival Probability", x = "Time (years)") +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

Because the training curves use OOB estimates, both plots represent
out-of-sample predictions and are directly comparable.

# Variable Selection

Random forest uses all available predictors. To understand which matter most, we
examine variable importance (VIMP) and minimal depth.

## Variable importance (VIMP)

VIMP measures the increase in prediction error when a variable's values are
randomly permuted across the out-of-bag sample. For survival forests the
prediction error is the Harrell C-statistic complement (1 − C), so permuting
an important variable hurts C and yields a large positive VIMP. A negative VIMP
means the variable is adding noise: the forest would predict better without it.
See the [randomForestSRC](https://www.randomforestsrc.org) documentation for
details on the VIMP calculation for censored outcomes [@Ishwaran:2007a].

```{r vimp-plot, fig.width=6, fig.cap="Variable importance ranking. Blue = positive VIMP, red = negative."}
plot(gg_vimp(rfsrc_pbc), lbls = st_labs) +
  theme(legend.position = c(0.8, 0.2)) +
  labs(fill = "VIMP > 0")
```

Bilirubin ranks highest, followed by copper, prothrombin time, albumin, and age
--- closely matching the variables selected in the @fleming:1991 proportional
hazards model.

## Minimal depth

Minimal depth [@Ishwaran:2010] ranks variables by how close to the root node
they first split, on average. Variables that partition large portions of the
population early are considered most important.

```{r varsel}
md_pbc <- max.subtree(rfsrc_pbc)
```

The `max.subtree()` function computes minimal depth for each variable. The
threshold is `r round(md_pbc$threshold, 2)`, selecting
`r length(md_pbc$topvars)` variables: `r paste(md_pbc$topvars, collapse = ", ")`.

Both selection methods agree on the key predictors: `bili`, `albumin`, `copper`,
`prothrombin`, and `age`. We add `edema` (selected by the @fleming:1991 model)
for the remainder of the analysis.

```{r select-vars}
xvar     <- c("bili", "albumin", "copper", "prothrombin", "age")
xvar_cat <- "edema"
xvar_all <- c(xvar, xvar_cat)
```

# Variable Dependence

## Variable dependence plots

Variable dependence shows each patient's predicted survival probability
$\hat{S}(t_0)$ at a fixed time horizon $t_0$, plotted against a predictor of
interest. Because $\hat{S}(t_0)$ is a probability, the y-axis is always bounded
to [0, 1]. Points are colored by event status; a loess smooth traces the
overall trend while the individual points show how much variance remains after
the forest accounts for all other covariates.

```{r vardep-bili, fig.height=4, fig.cap="Variable dependence of survival at 1 and 3 years on bilirubin."}
gg_v <- gg_variable(rfsrc_pbc, time = c(1, 3),
                    time.labels = c("1 Year", "3 Years"))

plot(gg_v, xvar = "bili", alpha = 0.4) +
  labs(y = "Survival", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

Survival drops sharply with increasing bilirubin, and the 3-year curve drops
further than the 1-year curve, suggesting a non-proportional hazards effect.

```{r vardep-panel, fig.height=5, fig.cap="Variable dependence at 1 and 3 years for continuous predictors."}
plot(gg_v, xvar = xvar[-1], panel = TRUE, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.05, 1.05))
```

The plots confirm survival increases with albumin and decreases with copper,
prothrombin time, and age. The divergence between time curves for `copper`
further supports a non-proportional hazards mechanism.

```{r vardep-categorical, fig.height=4, fig.cap="Variable dependence on edema (categorical)."}
plot(gg_v, xvar = xvar_cat, alpha = 0.4) +
  labs(y = "Survival") +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  coord_cartesian(ylim = c(-0.01, 1.02))
```

Patients with edema = 1 (edema despite diuretics) have markedly lower predicted
survival.

## Partial dependence

::: {.callout-warning}
**Known issue (draft):** `randomForestSRC::partial.rfsrc()` currently fails for
survival forests in randomForestSRC ≥ 3.3.  The partial dependence and
surface sections below will show an error until this upstream bug
is resolved.  All other sections of this vignette are fully functional.
:::

Partial dependence integrates out the effects of other covariates, giving a
risk-adjusted view of how each predictor influences the response
[@Friedman:2000]. We use `gg_partial_rfsrc()` which calls
`randomForestSRC::partial.rfsrc()` directly and returns a `gg_partial_rfsrc`
object.  For survival forests, the result includes a `time` column so
`plot(pd)` produces one curve per predictor value over time, faceted by
variable name.

```{r partial-dep, error=TRUE, fig.height=5, fig.cap="Partial dependence of survival at approximately 1 and 3 years on continuous predictors."}
# partial.rfsrc() requires times that match the model's time.interest grid;
# gg_partial_rfsrc() snaps the requested values to the nearest observed times.
ti   <- rfsrc_pbc$time.interest
t1yr <- ti[which.min(abs(ti - 1))]
t3yr <- ti[which.min(abs(ti - 3))]

pd <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = xvar,
                       partial.time = c(t1yr, t3yr))

# Quick S3 plot — survival forests produce time-series curves per predictor value
plot(pd)
```

For a publication-ready layout with custom colour scale, access `pd$continuous`
directly:

```{r partial-dep-custom, error=TRUE, fig.height=5, fig.cap="Partial dependence (custom styling)."}
ggplot(pd$continuous, aes(x = x, y = yhat,
                          color = factor(round(time, 2)),
                          group = factor(time))) +
  geom_line(linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = "Predicted Survival", x = "", color = "Year") +
  scale_color_manual(values = setNames(c("steelblue", "firebrick"),
                                       as.character(round(c(t1yr, t3yr), 2)))) +
  theme_bw()
```

The partial dependence curves at approximately 1 and 3 years confirm the
variable dependence findings and support the log-transforms used in the
@fleming:1991 model for `bili`, `albumin`, and `prothrombin`. The divergence
between time curves for `bili` and `copper` supports non-proportional hazards.

## Conditional dependence

To investigate interactions, we can condition variable dependence on group
membership in another variable. Here we examine the dependence of survival on
bilirubin, stratified by edema group.

```{r coplot-edema, fig.width=7, fig.height=4, fig.cap="Variable dependence on bilirubin, conditional on edema group."}
gg_v1 <- gg_variable(rfsrc_pbc, time = 1)
gg_v1$edema <- paste("edema =", gg_v1$edema)

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_grid(~edema) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

The decreasing trend with bilirubin holds across all edema groups, but survival
is uniformly lower in the edema = 1 panel, confirming an additive effect.

We can also condition on a continuous variable by binning into quantile groups:

```{r coplot-albumin, fig.width=7, fig.height=5, fig.cap="Variable dependence on bilirubin, conditional on albumin groups."}
albumin_cts <- quantile_pts(gg_v1$albumin, groups = 6, intervals = TRUE)
gg_v1$albumin_grp <- cut(gg_v1$albumin, breaks = albumin_cts)
levels(gg_v1$albumin_grp) <- paste("albumin =",
                                    levels(gg_v1$albumin_grp))

plot(gg_v1, xvar = "bili", alpha = 0.5) +
  labs(y = "Survival at 1 Year", x = st_labs["bili"]) +
  theme(legend.position = "none") +
  scale_color_manual(values = event_colors, labels = event_labels) +
  scale_shape_manual(values = event_marks, labels = event_labels) +
  facet_wrap(~albumin_grp) +
  coord_cartesian(ylim = c(-0.01, 1.01))
```

The effect of bilirubin attenuates at higher albumin levels, suggesting an
interaction between these two liver function markers.

# Partial Dependence Surfaces

For a richer view of the interaction between bilirubin and albumin, we construct
a partial dependence surface. We compute partial dependence on a grid of 8
albumin values, each evaluated at 25 bilirubin points.

```{r surface-data, error=TRUE, cache=FALSE}
# Create grid of albumin values
alb_grid <- quantile_pts(pbc_trial$albumin, groups = 8)

# For each albumin value, compute partial dependence on bili at ~1 year
surface_list <- lapply(alb_grid, function(alb_val) {
  newx <- pbc_trial[, rfsrc_pbc$xvar.names, drop = FALSE]
  newx$albumin <- alb_val
  pd_alb <- gg_partial_rfsrc(rfsrc_pbc, xvar.names = "bili",
                              newx = newx, partial.time = t1yr)
  df <- pd_alb$continuous
  df$albumin <- alb_val
  df
})

surface_df <- bind_rows(surface_list)
```

```{r pd-surface, error=TRUE, fig.height=5, fig.cap="Partial dependence surface: survival at 1 year as a function of bilirubin and albumin. Fill colour is the predicted survival probability."}
if (!exists("surface_df")) {
  message("surface_df not available --- skipping surface (see surface-data chunk error above).")
} else {
  ggplot(surface_df, aes(x = x, y = albumin, fill = yhat)) +
    geom_tile() +
    scale_fill_viridis_c(name = "Survival") +
    labs(x = "Bilirubin", y = "Albumin") +
    theme_bw()
}
```

The surface shows that survival is highest when bilirubin is low and albumin is
high (upper-left corner), and drops steeply as bilirubin increases or albumin
decreases. The curvature of the surface --- particularly the steep
gradient at low albumin and high bilirubin --- confirms the interaction detected
in the conditional plots.

## Brier Score and CRPS

VIMP and minimal depth tell us which predictors matter, but they say nothing
about how well the forest's survival predictions are calibrated. The Brier
score answers that question. At each point on the event-time grid it measures
the mean squared difference between the predicted survival probability
$\hat{S}(t)$ and the binary outcome (survived past $t$ or not). Because
right-censored subjects have unknown true outcomes, the calculation uses
inverse-probability-of-censoring weighting (IPCW): each subject's squared
error is upweighted by the inverse probability of being uncensored at that time,
so the estimate remains unbiased even under heavy censoring [@graf:1999].

Two reference values help interpret the curve. A Brier score of 0 is perfect
prediction. A score around 0.25 is the uninformative ceiling: a model that
assigns every subject a survival probability of 0.5, regardless of their
covariates or the time horizon, scores near 0.25 by construction. A forest
that beats 0.25 is doing better than that floor; one that exceeds it has been
made worse by its covariates, which is a sign of overfitting or a poorly
specified model.

`gg_brier()` wraps `randomForestSRC::get.brier.survival()` and returns a tidy
data frame ready to plot.

```{r brier-overall, fig.width=7, fig.height=3.5, fig.cap="Time-resolved Brier score for the PBC survival forest."}
gg_bs <- gg_brier(rfsrc_pbc)
plot(gg_bs)
```

Read the curve left to right. Early on it sits near zero --- almost everyone
is still alive, so predicting survival is easy. It climbs to a peak around the
median event time, where the outcome is genuinely uncertain, then falls again
as the at-risk pool shrinks and the remaining predictions get easier.

Setting `envelope = TRUE` adds a 15--85% ribbon around that line, showing how
spread out the individual subjects' Brier contributions are at each time. A
narrow ribbon means the forest is assigning similar risks across subjects; a
wide one means the predicted risks are pulling apart.

```{r brier-envelope, fig.width=7, fig.height=3.5, fig.cap="Brier score with 15--85% per-subject envelope."}
plot(gg_bs, envelope = TRUE)
```

The running CRPS (continuous ranked probability score) is the Brier score
integrated over time and divided by elapsed time — the time-average of the
curve above. It collapses the whole trajectory into one running number, which
is handy when you want a single calibration figure rather than a curve to read.
Like the Brier score, a CRPS near 0 is good and a value near 0.25 is the
uninformative ceiling (the same constant-predictor reference as above). The final (integrated) value at the right edge of the
plot is the one most often reported.

```{r brier-crps, fig.width=7, fig.height=3.5, fig.cap="Running CRPS for the PBC survival forest."}
plot(gg_bs, type = "crps")
```

The integrated CRPS --- a scalar summary of overall calibration --- is stored
as an attribute and can be retrieved with:

```{r brier-crps-scalar}
attr(gg_bs, "crps_integrated")
```

# Conclusion

We have walked a full random survival forest analysis with
**randomForestSRC** and **ggRandomForests**, and the results hang together:

- `gg_survival()` drew Kaplan--Meier curves that showed no treatment effect in
  the PBC trial, the same conclusion reached in @fleming:1991.
- `gg_error()` showed the OOB error settling quickly as trees were added.
- VIMP (`gg_vimp()`) and minimal depth (`max.subtree()`) landed on the same
  five predictors (bilirubin, albumin, copper, prothrombin, and age),
  which is also where the proportional hazards model landed.
- `gg_variable()` exposed non-proportional hazards for bilirubin and copper,
  where the gap between time horizons widens.
- Partial dependence from `gg_partial_rfsrc()` gave the risk-adjusted version
  of those curves and backed the log-transforms used in the parametric model.
- Conditioning plots and the partial dependence surface drew out the
  bilirubin--albumin interaction.
- `gg_brier()` measured how accurate the predictions actually were, both
  across time and as a single CRPS summary.

Notice the pattern. Each `gg_*()` function returns a tidy object (often a
data frame, sometimes a small list of data frames); the plotting comes
after. Lean on the package's `plot()` methods when the
default figure works, and drop down to `ggplot2` directly when it does not.

# References

---
title: "Exploring variable importance with varPro"
author: "John Ehrlinger"
date: today
format:
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Exploring variable importance with varPro}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  message    = FALSE,
  warning    = FALSE,
  fig.width  = 7,
  fig.height = 4.5,
  cache      = TRUE,
  cache.path = "varpro_cache/"
)
options(mc.cores = 1, rf.cores = 1)
```

```{r libs}
library(ggplot2)

# Match the pattern used by the regression/survival vignettes: try the
# installed package first, fall back to pkgload::load_all() for the
# R CMD check vignette rebuild where the package isn't yet on
# .libPaths(). All varPro calls below are ::-qualified, so no
# library(varPro) is needed.
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE,
                    attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}
```

```{r precomputed, include=FALSE, cache=FALSE}
# The three gg_partial_varpro() calls and the Boston beta.varpro() fit are
# the only expensive parts of this vignette (~34s combined). They are
# precomputed by precompute_varpro.R and loaded here so the R CMD check
# vignette rebuild stays fast. If the file is absent (e.g. a fresh clone
# that hasn't run the script), each chunk falls back to live computation,
# so the vignette is reproducible either way.
.vp <- tryCatch({
  # R CMD check rebuilds the vignette in a copy whose working directory is the
  # vignette dir; an interactive knit from the package root sees vignettes/
  # instead. Try both locations, and fall back to live computation below if
  # neither is present or readRDS() fails.
  .cand <- c("varpro_precomputed.rds",
             file.path("vignettes", "varpro_precomputed.rds"))
  .hit <- .cand[file.exists(.cand)]
  if (length(.hit)) readRDS(.hit[[1]]) else list()
}, error = function(e) list())   # missing or unreadable -> live fallback below
```

# What varPro is

Random-forest variable importance has, for two decades, mostly meant
*permutation importance*: shuffle a column, measure the drop in OOB
accuracy, repeat. It works, but the score is computed against
artificially modified data that never appeared in the training set, and
when predictors are correlated the shuffled column is an implausible
counterfactual, not a neutral baseline. Knockoff-style methods clean up
some of that, but they introduce their own synthetic features and rely on
explicit distributional assumptions about the predictors.

varPro [@Lu2024varpro] takes a different path: one grounded entirely in
*observed* data. Think of each decision tree as a long chain of "if/then"
clauses. varPro harvests those clauses as *rules*, and for each rule it
identifies a specific region of the predictor space where a handful of
variables jointly constrain the response. To measure one variable's
contribution, it compares a *local estimator* (the response summary
restricted to that rule's region) against a "released" estimator where
the constraint on the tested variable has been lifted. That comparison,
averaged over rules and trees, is the variable's importance z-score. Every
observation used in the contrast is real training data; nothing is
permuted, nothing is synthesised.

The "release" metaphor is the right one. Imagine holding a rope at several
points along its length (the rule constraints). Releasing your grip at one
point (while keeping the others in place) tells you how taut that
section was. Variables that go slack when released were carrying load;
variables that stay taut were redundant given the rest.

Operationally, varPro's *guided splitting* steers tree growth so that
rules concentrate in informative regions, and the subsequent *rule
harvesting* step selects the rules that survived the importance
pre-filter. Variables below a noise threshold are dropped before
reporting (the importance z is pre-filtered), so the final ranking
contains only variables that earned a seat at the table
(see the [varPro tools reference](https://www.varprotools.org/articles/getstarted.html)
for worked implementation details). Split-weights encode each variable's
relative importance at each tree node, propagating that signal through the
release-rule contrasts and into the partial-dependence curves.

Cross-validation via `cv.varpro()` returns three views of the
importance ranking: `$imp` (the default, point-estimate ranking),
`$imp.conserve` (one-SD rule: only variables whose lower confidence
bound is positive), and `$imp.liberal` (any variable with a positive
point estimate). The conservative cut is the right default when you want
to hand a short list to a collaborator; the liberal cut is better for
exploratory screening when you'd rather miss nothing.

For classification, the importance decomposes into an `$unconditional`
score (averaged over all classes) and per-class `$conditional.z` scores.
When a variable separates *one* class from the others but is uninformative
overall, the unconditional z can be near zero while a single conditional z
is large; the conditional view catches that. One-hot consolidation via
`get.orgvimp()` and `get.topvars()` maps the per-level scores back to the
original factor, so downstream reporting stays on familiar ground.

That core machinery feeds six ggRandomForests wrappers. **`gg_varpro()`**
summarises the per-tree importance distribution. **`gg_beta_varpro()`**
refines those release-rule contrasts with a per-rule lasso. **`gg_partial_varpro()`**
turns the release machinery into partial-dependence curves.
**`gg_udependent()`** reads cross-variable dependency off a `uvarpro()`
fit. **`gg_isopro()`** scores observations for anomaly using an
isolation-forest variant. **`gg_ivarpro()`** computes per-observation
local importance.

This vignette walks all six wrappers on three worked examples: a
regression problem (Boston housing), a classification problem (iris,
binary and multi-class), and a survival problem (PBC). The closing
section is a one-page reference matrix mapping each wrapper to the
forest families it supports.

# Regression: Boston housing

We start with the Boston housing data: 506 census tracts, 13 numeric
predictors, median home value as the response.

```{r boston-fit}
data("Boston", package = "MASS")
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_boston <- if (is.null(.vp$v_boston)) {
  varPro::varpro(medv ~ ., data = Boston, ntree = 50)
} else {
  .vp$v_boston
}
v_boston
```

`varpro()` builds the importance machinery (release rules, per-tree
scores, the splitweight vector) once. Every subsequent figure in this
section reads off that single fit. The formula interface is intentional:
varPro is model-independent in the sense that it doesn't assume a linear
or parametric relationship between predictors and response, but it does
require you to name a response so the guided splitting has something to
optimise toward.

## Per-tree importance with `gg_varpro()`

The first question varPro answers is the same one permutation importance
answers: which variables matter, ranked. `gg_varpro()` extracts the
per-tree importance scores and draws their distribution as a horizontal
boxplot, one row per variable, sorted by median z-score. Variables above
a configurable cutoff (default 0.79) are highlighted.

```{r boston-gg-varpro}
plot(gg_varpro(v_boston))
```

The narrow boxes near the top are the variables varPro is confident
about: every tree agrees they matter. Wide boxes that straddle the
cutoff line are the ones to look at twice; the forest disagrees with
itself. That disagreement isn't noise to suppress: it can mean the
variable matters in some rule regions but not others, which is exactly
the kind of structured heterogeneity that model-independent methods are
built to detect. Variables that fall entirely below the cutoff were
pre-filtered by the importance z threshold; they're gone before the plot
is drawn.

## Partial dependence with `gg_partial_varpro()`

The ranking view tells you *which* variables matter. Partial dependence
asks the next question: *how* does the response change with a variable,
holding the others fixed? `gg_partial_varpro()` wraps
`varPro::partialpro()` and returns a tidy frame of parametric,
non-parametric, and causal partial-dependence curves.

```{r boston-gg-partial-varpro}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd <- if (is.null(.vp$pd_boston)) {
  gg_partial_varpro(object = v_boston)
} else {
  .vp$pd_boston
}
plot(gg_pd)
```

Each panel is a single predictor. The three curves correspond to the
three estimators varPro carries (parametric, non-parametric, and causal);
read them as a sensitivity analysis. When all three agree, you have a
stable signal; when the causal curve diverges from the others, that's a
hint that the variable's observed relationship with the response may be
partly driven by its correlation with other predictors in the rule
regions, not by a direct effect.

Because `partialpro()` operates within the *local* neighborhoods defined
by the release rules, these curves reflect what happens in the part of
the predictor space the forest actually visited, not a global
extrapolation from a fitted model. That is both a strength (no
out-of-distribution extrapolation) and a limit (sparse regions of the
predictor space will have wider effective intervals, even if the plot
doesn't always show them explicitly).

## Per-rule lasso refinement with `gg_beta_varpro()`

`gg_varpro()`'s ranking is built from the release-rule contrast.
`gg_beta_varpro()` re-asks the question variable by variable inside each
rule: it fits a one-predictor lasso of the response on the released
variable, restricted to the OOB observations in that rule's region. The
fitted coefficient (call it a "local β") captures the variable's
linear effect *within that rule's neighborhood*, not globally. Aggregating
`mean(|β|)` across rules gives one number per variable: a
regression-coefficient-flavoured importance, not a VIMP score, and not a
global slope.

That distinction matters in practice. A variable with a strong nonlinear
global relationship may have locally small β values inside any single
rule (the local-standardisation step within each rule normalises the
scale), but many rules will fire on it, so the aggregated mean is still
large. Conversely, a variable with a nearly linear global effect will
concentrate most of its weight in a handful of rules, and the
between-rule variability in β will be low.

Because `beta.varpro()` is expensive (a `glmnet` per rule), the wrapper
accepts a pre-computed `beta_fit` so you can iterate on selection,
cutoff, or class choice without re-fitting.

```{r boston-beta-fit, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_boston <- if (is.null(.vp$b_boston)) {
  varPro::beta.varpro(v_boston)
} else {
  .vp$b_boston
}
```

```{r boston-gg-beta-varpro}
plot(gg_beta_varpro(v_boston, beta_fit = b_boston))
```

Compare against `gg_varpro()` above. Disagreement is the diagnostic
signal: a variable that ranks high here but low there is one whose
local linear effect inside many rules is real even when the release
contrast is modest.

## Cross-variable dependency with `gg_udependent()`

The three views so far take one variable at a time. `gg_udependent()`
reads cross-variable structure off a `uvarpro()` fit and draws the
result as a network: nodes are variables, edges are dependencies above a
configurable threshold. The visual is built with `ggraph`, which is in
`Suggests` rather than `Imports`; install it if you want this view.

```{r boston-uvarpro, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
u_boston <- if (is.null(.vp$u_boston)) {
  varPro::uvarpro(Boston[, setdiff(names(Boston), "medv")], ntree = 50)
} else {
  .vp$u_boston
}
```

```{r boston-gg-udependent, eval = requireNamespace("ggraph", quietly = TRUE)}
plot(gg_udependent(u_boston))
```

Clusters of mutually-connected variables are worth checking for
redundancy: they may be several views of the same underlying quantity.
`uvarpro()` operates on the predictor matrix alone; there is no response
in the fit, which is what makes the network genuinely unsupervised.
Variables that cluster here are correlated in feature space regardless of
whether any of them matters for prediction. That information is
complementary to the ranking from `gg_varpro()`: a variable can be
important for prediction and still sit in a tight cluster with a
near-duplicate; in that situation, model parsimony may favour dropping
one of the cluster members without losing much predictive accuracy.

## Anomaly scoring with `gg_isopro()`

Variable importance is one axis; *observation* outlierness is another.
`gg_isopro()` wraps `varPro::isopro()` (an isolation-forest variant
that scores how anomalous each training row looks) and renders the
result as a ranked elbow plus a density of the scores. The score is
on `[0, 1]`; the wrapper's convention is "higher = more anomalous"
(opposite of varPro's native polarity; the wrapper flips it for
consistency).

```{r boston-isopro, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_boston <- if (is.null(.vp$iso_boston)) {
  varPro::isopro(data = Boston[, setdiff(names(Boston), "medv")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_boston
}
```

```{r boston-gg-isopro}
plot(gg_isopro(iso_boston))
```

The elbow flags rows that diverge from the bulk. Pair with the
domain: anomalous in feature space is not the same as wrong, but it's
often where the most interesting cases live. In a regression context,
a high-scoring row is one that the isolation forest isolated quickly
because its feature combination is rare; it may or may not be an outlier
in the response. Anomaly scoring and residual analysis answer different
questions, and it's worth doing both.

Note that `isopro()` scores are not calibrated to a universal scale: a
score of 0.7 in one dataset is not comparable to 0.7 in another. What
matters is the relative ordering within a dataset and the shape of the
elbow: a sharp kink at a small number of observations is a cleaner
signal than a gradual slope that never levels off.

## Local importance with `gg_ivarpro()`

The wrappers so far aggregate across observations. `gg_ivarpro()` does
the opposite: it returns one value per (observation, variable) pair,
capturing how much variable *v* contributed to predicting observation
*i*. The aggregate view is a distribution of those local importances
per variable; the per-observation view is a horizontal bar of one row's
local importances across variables.

`ivarpro()` is the most expensive call in varPro, so the wrapper
accepts a pre-computed `ivarpro_fit` for reuse across views.

```{r boston-ivarpro-fit, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iv_boston <- if (is.null(.vp$iv_boston)) {
  varPro::ivarpro(v_boston)
} else {
  .vp$iv_boston
}
```

```{r boston-gg-ivarpro-distribution}
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston))
```

```{r boston-gg-ivarpro-which-obs}
plot(gg_ivarpro(v_boston, ivarpro_fit = iv_boston, which_obs = 1L))
```

The distribution view tells you which variables drive predictions
*across* observations. The per-observation view answers the same
question for a specific case, useful for explaining one prediction
back to whoever asked.

The per-observation importance is computed by replaying the release-rule
contrasts restricted to the rules that fire for that observation. It
is not a Shapley value, but it shares the local-attribution spirit: each
bar shows how much that variable's release shifted the local estimator
for this specific row, averaged over the rules that covered it. Cases
that are well-represented in the training data (dense rule coverage) will
have sharper attribution than cases in sparse regions of the predictor
space.

# Classification: iris

Iris is a small data set (150 rows, four predictors, three response
classes), and that's a feature here, not a flaw: every figure renders in
under a second, and the structure is well-understood enough that any
strange behaviour stands out. It is also a good stress-test for the
conditional importance path: petal length and petal width separate
*setosa* from everything else very cleanly, but the *versicolor*/*virginica*
boundary is much softer. A method that only reports unconditional
importance would lump both cases together; the conditional decomposition
should show the asymmetry.

Two fits: a binary problem (drop *setosa*, positive class = *virginica*)
and the full three-class problem.

```{r iris-fit-binary}
iris_binary <- iris[iris$Species != "setosa", ]
iris_binary$Species <- droplevels(iris_binary$Species)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_binary <- if (is.null(.vp$v_iris_binary)) {
  varPro::varpro(Species ~ ., data = iris_binary, ntree = 50)
} else {
  .vp$v_iris_binary
}
```

```{r iris-fit-multi}
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_iris_multi <- if (is.null(.vp$v_iris_multi)) {
  varPro::varpro(Species ~ ., data = iris, ntree = 50)
} else {
  .vp$v_iris_multi
}
```

## Class-conditional importance with `gg_varpro(conditional = TRUE)`

For a classification forest, `gg_varpro()` can split the importance
view into one facet per class. Variables keep the unconditional sort
order so rows line up across facets; read along a row to see which
class a variable is informative for.

The conditional importance uses `$conditional.z` from the varPro fit:
per-class release-rule contrasts computed within the same rule regions
as the unconditional score, but with the local estimator replaced by a
per-class probability. A variable that separates one class from the rest
will have a high `$conditional.z` for that class and a near-zero or
even negative value for the others. The unconditional score in
`$unconditional` is the average, and can mask class-specific effects,
which is why the `conditional = TRUE` faceted view exists.

```{r iris-multi-gg-varpro-conditional}
plot(gg_varpro(v_iris_multi, conditional = TRUE))
```

## Partial dependence: `gg_partial_varpro()` on classification

`gg_partial_varpro()` works the same way on a classification fit, but
each predictor's curve becomes per-class probabilities. The plot
panels are organised by predictor with class encoded as colour or
facet (the wrapper defaults handle this).

```{r iris-multi-gg-partial-varpro}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_iris <- if (is.null(.vp$pd_iris_multi)) {
  gg_partial_varpro(object = v_iris_multi)
} else {
  .vp$pd_iris_multi
}
plot(gg_pd_iris)
```

Read each panel as: "as predictor X changes from low to high, how does
the probability of each class shift?" Patterns that match the
underlying biology (e.g. petal length separating setosa from the
others) act as a sanity check on the forest.

## Per-class lasso refinement with `gg_beta_varpro()`

On a classification fit, `gg_beta_varpro()` returns one row per
(variable, class) pair. For a binary fit, `which_class = NULL` defaults
to the last factor level (the positive class), so the headline view
is a single panel of that class.[^mortality]

[^mortality]: The same code pattern applies to clinical binary outcomes
such as 30-day mortality: drop the negative class, set the event class
as the last factor level, and read the figure for the event panel.

```{r iris-binary-beta-fit, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_binary <- if (is.null(.vp$b_iris_binary)) {
  varPro::beta.varpro(v_iris_binary)
} else {
  .vp$b_iris_binary
}
```

```{r iris-binary-gg-beta-varpro}
plot(gg_beta_varpro(v_iris_binary, beta_fit = b_iris_binary))
```

For a multi-class fit, the default view is faceted by class with each
class sharing the row order set by the unified ranking, same trick as
`gg_varpro(conditional = TRUE)`.

```{r iris-multi-beta-fit, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
b_iris_multi <- if (is.null(.vp$b_iris_multi)) {
  varPro::beta.varpro(v_iris_multi)
} else {
  .vp$b_iris_multi
}
```

```{r iris-multi-gg-beta-varpro}
plot(gg_beta_varpro(v_iris_multi, beta_fit = b_iris_multi))
```

`which_class = "<level>"` collapses the faceted view to a single class
when you want it.

# Survival: PBC

Survival is the family where the varPro toolchain shows its limits,
and being explicit about those limits is more useful than pretending they
don't exist. The forest-fitting and the family-agnostic wrappers all
work; the lasso-refined and individual-importance wrappers don't, because
the underlying `varPro::beta.varpro()` and `varPro::ivarpro()` calls
don't yet extend to right-censored outcomes. A per-rule local lasso for a
censored response requires a local partial-likelihood or Nelson-Aalen
estimator in place of the regression/classification local estimator;
the design work for that is tracked but not yet landed.

What does work is the core release-rule importance, the C-path partial
dependence (routed through the embedded `$rf` random survival forest),
and anomaly scoring on the predictor matrix. For many applied problems,
those three views cover the questions you actually want to answer.

The PBC (primary biliary cirrhosis) dataset from `randomForestSRC` has
418 patients, seven predictors, and a Surv-encoded outcome of days to
event (death or transplant, status ∈ {0, 1, 2}). We use a small
seven-variable subset so the vignette fits quickly. For a full
analysis including time-dependent covariates, @Lee:2021 demonstrates the
boosted nonparametric hazard framework that varPro's survival path draws
on.

```{r pbc-data}
library(survival)
data(pbc, package = "randomForestSRC")
pbc_small <- pbc[, c("days", "status", "age", "albumin", "bili",
                     "edema", "platelet")]
pbc_small <- na.omit(pbc_small)
set.seed(20260527L)
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
v_pbc <- if (is.null(.vp$v_pbc)) {
  varPro::varpro(Surv(days, status) ~ ., data = pbc_small, ntree = 50)
} else {
  .vp$v_pbc
}
```

## Variable importance: `gg_varpro()`

```{r pbc-gg-varpro}
plot(gg_varpro(v_pbc))
```

## Partial dependence: `gg_partial_varpro()` (C-path)

For survival, `gg_partial_varpro()` dispatches into the
`randomForestSRC`-backed partial-dependence machinery via the underlying
`$rf` survival forest (what the package calls the *C-path* because it
routes through the `randomForestSRC` C-code layer rather than the varPro
release-rule engine). The output shape is the same as the regression case
(parametric / non-parametric / causal panels); the y-axis carries an
"ensemble mortality" interpretation [@Ishwaran:2007a], not a survival
probability. Higher values mean higher predicted mortality at a given
predictor value, integrated over the event-time horizon in the training
data.

```{r pbc-gg-partial-varpro}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
gg_pd_pbc <- if (is.null(.vp$pd_pbc)) {
  gg_partial_varpro(object = v_pbc)
} else {
  .vp$pd_pbc
}
plot(gg_pd_pbc)
```

## Anomaly scoring: `gg_isopro()` on the X-matrix

Because `isopro()` only sees the predictor matrix, it doesn't care
about the family. The same call from section 3 works here.

```{r pbc-isopro, cache=TRUE}
# Precomputed offline (see precompute_varpro.R); falls back to a live fit.
iso_pbc <- if (is.null(.vp$iso_pbc)) {
  varPro::isopro(data = pbc_small[, c("age", "albumin", "bili", "platelet")],
                 method = "rnd", sampsize = 256, ntree = 50)
} else {
  .vp$iso_pbc
}
plot(gg_isopro(iso_pbc))
```

## Not available for survival: `gg_beta_varpro`, `gg_ivarpro`

`varPro::beta.varpro()` errors on survival fits in the current release
(it only supports `regr` and `class`). `gg_ivarpro()` for survival is
similarly deferred pending design work on the per-rule risk-scaling
story. Both are tracked for v3.1.0.

If you call either on a survival fit you'll get a clear error message
pointing at the deferred work, not a silent miscalculation. The
family-support matrix in §6 records this; the rest of the toolkit
that *does* work on survival (`gg_varpro`, `gg_partial_varpro`,
`gg_isopro` above; `gg_udependent` shown in §3 on the X-matrix).

# Cross-cutting reference

## Family-support matrix

| Wrapper | regr | class | surv | regr+ |
|---|---|---|---|---|
| `gg_partial_varpro` | ✓ | ✓ | ✓ (C-path via `$rf`) | ✗ (not audited) |
| `gg_varpro` | ✓ | ✓ (`conditional = TRUE`) | ✓ | ✗ (errors) |
| `gg_udependent` | ✓ (uvarpro on X) | ✓ (X) | ✓ (X) | ✓ (X) |
| `gg_isopro` | ✓ (X) | ✓ (X) | ✓ (X) | ✓ (X) |
| `gg_beta_varpro` | ✓ | ✓ | ✗ (upstream stop) | ✗ (deferred) |
| `gg_ivarpro` | ✓ | ✓ | ✗ (deferred) | ✗ (deferred) |

The four wrappers in the lower-right are the v3.1.0 work surface.

## Factor-level ordering

Across `gg_beta_varpro()` and `gg_ivarpro()`, the `variable` column is
stored as a factor whose levels are set by descending aggregate
importance (`mean(|imp|)` summed across classes for classification).
The default plot inherits that ordering, so faceted views show
variables in the same row order across panels. If you re-shape the
frame downstream and want the order preserved, keep `variable` as a
factor rather than coercing to character.

This convention will be propagated to `gg_vimp` and
`plot.gg_varpro(conditional = TRUE)` in a follow-up release.

## Caching the expensive calls

`varPro::beta.varpro()` and `varPro::ivarpro()` are the two heavy
calls. Both wrappers accept a pre-computed fit (`beta_fit`,
`ivarpro_fit`) so you can iterate on selection, observation index, or
cutoff without re-fitting the lasso or the local-importance machinery.
The vignette uses this throughout: every section computes the heavy
fit once in a `cache: true` chunk and re-uses it for every figure.

Provenance carries `precomputed = TRUE` when the cached path was used,
so downstream tooling can tell the two paths apart.

## Provenance shape

`attr(., "provenance")$cutoff` is always a *named numeric vector*
across the toolkit:

- regression: length 1, named `"regr"`
- classification: length K, named with the response factor levels

Downstream code that picks a value should read it as a vector
(`prov$cutoff[[class_name]]` or `prov$cutoff[[1]]`), not as a scalar.
This contract was established in v2.7.3.9012 (PR #98).

# Further reading

The release-rule framework is laid out in @Lu2024varpro. Worked
implementation examples and the full API are at the
[varPro tools reference site](https://www.varprotools.org/articles/getstarted.html).

The ensemble mortality scale used by survival partial dependence is
introduced in @Ishwaran:2007a (the Annals of Applied Statistics methods
paper for random survival forests); the R-package side is described in
@Ishwaran:2008. The boosted nonparametric hazard framework that informs
varPro's survival path (including the treatment of time-dependent
covariates) is in @Lee:2021.

Each wrapper's help page carries a "What this is doing" section that
goes one level deeper than this vignette. The cross-cutting reference
in §6 maps each wrapper to the forest families it supports and notes
which capabilities are deferred to v3.1.0.

# References

---
title: "Exploring Random Forests with ggRandomForests"
author: "John Ehrlinger"
date: today
format: 
  html:
    toc: true
    html-math-method: mathjax
editor: 
  markdown: 
    wrap: 80
vignette: >
  %\VignetteIndexEntry{Vignette's Title}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

The **ggRandomForests** package extracts tidy data objects from either
`randomForestSRC` or `randomForest` fits and feeds them into familiar
`ggplot2` workflows. This vignette highlights the most common objects—
`gg_error`, `gg_variable`, and `gg_vimp`—along with a small helper for
building balanced conditioning intervals.

```{r pkg-setup, include=FALSE}
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}
```

## Error trajectories with `gg_error()`

```{r error-demo}
library(randomForest)
set.seed(42)
rf_iris <- randomForest(Species ~ ., data = iris, ntree = 200, keep.forest = TRUE)
err_df <- ggRandomForests::gg_error(rf_iris, training = TRUE)
head(err_df)
```

The `gg_error()` object stores the cumulative OOB error rate for each
outcome column plus the `ntree` counter. When `training = TRUE`, the
function reconstructs the original model frame and appends the in-bag
error trajectory (`train`). Plotting overlays both curves by default:

```{r error-plot, fig.height=4}
plot(err_df)
```

## Marginal dependence via `gg_variable()`

```{r variable-demo}
set.seed(99)
boston <- MASS::Boston
rf_boston <- randomForest(medv ~ ., data = boston, ntree = 150)
var_df <- ggRandomForests::gg_variable(rf_boston)
str(var_df[, c("lstat", "yhat")])
```

Because the original training data are recovered from the model call,
`gg_variable()` works even when the forest was trained within helper
functions or against a `subset()` expression. The output keeps the raw
predictors plus either a continuous `yhat` column (regression) or per-class
probabilities (`yhat.<class>` for classification). Plotting a single
variable is straightforward:

```{r variable-plot, fig.height=4}
plot(var_df, xvar = "lstat")
```

Survival forests can request multiple horizons using the `time` argument;
non-OOB predictions are available by setting `oob = FALSE`.

## Variable importance with `gg_vimp()`

```{r vimp-demo}
vimp_df <- ggRandomForests::gg_vimp(rf_boston)
head(vimp_df)
plot(vimp_df)
```

If a `randomForest` object lacks stored importance scores, `gg_vimp()`
tries to compute them on the fly. When the forest truly cannot provide the
information (for example when `importance = FALSE` and the predictors are
no longer accessible), the function emits a warning and returns `NA`
placeholders so plots still render.

## Balanced conditioning cuts with `quantile_pts()`

```{r quantile-demo}
rm_breaks <- ggRandomForests::quantile_pts(boston$rm, groups = 6, intervals = TRUE)
rm_groups <- cut(boston$rm, breaks = rm_breaks)
table(rm_groups)
```

The helper wraps `stats::quantile()` to produce evenly populated strata
that drop directly into `cut()` when building coplots or facet labels.

## Next steps

* Inspect the full API reference at <https://ehrlinger.github.io/ggRandomForests/>.
* Use `?gg_error`, `?gg_variable`, `?gg_vimp`, and `?quantile_pts` for
  additional arguments and examples.
* Pair these data objects with your own `ggplot2` themes to align with your
  preferred publication style.

---
title: "Exploring Random Forests with ggRandomForests"
author: "John Ehrlinger"
date: today
format: 
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    html-math-method: mathjax
editor: 
  markdown: 
    wrap: 80
vignette: >
  %\VignetteIndexEntry{Vignette's Title}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

A fitted random forest carries a lot of information, but getting at it
usually means digging through list structures that were never meant to be
plotted directly. **ggRandomForests** does that digging for you: it pulls
tidy data objects out of a `randomForestSRC` or `randomForest` fit, and
those objects drop straight into the `ggplot2` workflows you already know.
A second engine, `varPro`, powers a parallel family of functions for
release-rule importance and related diagnostics; that family is covered in
the companion vignette referenced at the end.
This vignette walks through the three objects you will reach for most often
(`gg_error`, `gg_variable`, and `gg_vimp`), plus a small helper for
cutting a predictor into evenly populated groups.

```{r pkg-setup, include=FALSE}
if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}
```

## Error trajectories with `gg_error()`

```{r error-demo}
library(randomForest)
set.seed(42)
rf_iris <- randomForest(Species ~ ., data = iris, ntree = 200, keep.forest = TRUE)
err_df <- ggRandomForests::gg_error(rf_iris, training = TRUE)
head(err_df)
```

A forest's error rate settles down as trees are added, and the `gg_error()`
object lets you watch that happen. It holds the cumulative out-of-bag (OOB)
error rate for each outcome column, indexed by the `ntree` counter. Ask for
`training = TRUE` and the function reconstructs the original model frame and
adds the in-bag error trajectory (`train`) as well, so you can see both
curves at once:

```{r error-plot, fig.height=4}
plot(err_df)
```

## Marginal dependence via `gg_variable()`

```{r variable-demo}
set.seed(99)
boston <- MASS::Boston
rf_boston <- randomForest(medv ~ ., data = boston, ntree = 150)
var_df <- ggRandomForests::gg_variable(rf_boston)
str(var_df[, c("lstat", "yhat")])
```

`gg_variable()` recovers the training data straight from the model call,
so it still works when the forest was fit inside a helper function or
against a `subset()` expression, cases where the data is not sitting in
the global environment. The object you get back keeps the raw predictors
alongside the prediction: a single `yhat` column for regression, or one
`yhat.<class>` column per class for classification. To plot one predictor,
name it with `xvar`:

```{r variable-plot, fig.height=4}
plot(var_df, xvar = "lstat")
```

Survival forests can request multiple horizons using the `time` argument;
non-OOB predictions are available by setting `oob = FALSE`.

## Variable importance with `gg_vimp()`

```{r vimp-demo}
vimp_df <- ggRandomForests::gg_vimp(rf_boston)
head(vimp_df)
plot(vimp_df)
```

`gg_vimp()` measures permutation importance: each predictor is permuted in
turn, and the drop in OOB accuracy gives its score. This contrasts with the
`gg_varpro` family, which uses release-rule importance from the `varPro`
engine. Variable importance is not always stored on the fitted object. If a
`randomForest` fit is missing its importance scores, `gg_vimp()` will try to
compute them for you. When even that is not possible (the forest was grown
with `importance = FALSE` and the predictors are no longer reachable),
the function warns and returns `NA` in place of the scores, so a plot still
draws rather than failing outright.

## Balanced conditioning cuts with `quantile_pts()`

```{r quantile-demo}
rm_breaks <- ggRandomForests::quantile_pts(boston$rm, groups = 6, intervals = TRUE)
rm_groups <- cut(boston$rm, breaks = rm_breaks)
table(rm_groups)
```

When you build a coplot, you want each conditioning group to hold a roughly
equal share of the data --- equal-width bins leave the sparse tails nearly
empty. `quantile_pts()` wraps `stats::quantile()` to give you break points
that do exactly that, and they pass straight to `cut()` for the grouping or
facet labels.

## Next steps

* The full API reference lives at <https://ehrlinger.github.io/ggRandomForests/>.
* `?gg_error`, `?gg_variable`, `?gg_vimp`, and `?quantile_pts` cover the
  remaining arguments and have their own examples.
* The `gg_error`, `gg_variable`, and `gg_vimp` objects shown here are tidy
  data frames underneath, so you can skip the `plot()` methods entirely and
  build the figure yourself with `ggplot2`.
* For the full varPro toolkit (release-rule importance, lasso-refined
  importance, per-observation local importance, anomaly scores, and the
  dependency graph) walked across regression, classification, and
  survival examples, see `vignette("varpro", package = "ggRandomForests")`.

---
title: "Random Forest Regression with ggRandomForests"
author: "John Ehrlinger"
date: today
format:
  html:
    fig-format: png
    fig-dpi: 96
    toc: true
    toc-depth: 3
    html-math-method: mathjax
    code-fold: false
bibliography: ggRandomForests.bib
vignette: >
  %\VignetteIndexEntry{Random Forest Regression with ggRandomForests}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

::: {.callout-warning}
## Work in progress
This vignette is under active development. Code examples and narrative may
change before the next release.
:::

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
options(mc.cores = 1, rf.cores = 1)
```

# Introduction

Random forests [@Breiman:2001] are a non-parametric ensemble method that
requires no distributional or functional assumptions on how covariates relate to
the response. The method builds a large collection of de-correlated decision
trees via bootstrap aggregation (bagging) and random feature selection, then
averages their predictions to produce a stable, low-variance estimator. The
**randomForestSRC** package [@Ishwaran:RFSRC:2014] provides a unified
implementation for survival, regression, and classification forests.

**ggRandomForests** extracts tidy data objects from `rfsrc` fits and renders
them with **ggplot2** [@Wickham:2009], making it straightforward to explore how a
forest is constructed, which variables matter, and how the response depends on
individual predictors.

This vignette demonstrates a complete random forest regression workflow on the
Boston Housing data set [@Harrison:1978; @Belsley:1980]:

1. **Data exploration** --- EDA scatter panels, variable descriptions
2. **Growing the forest** --- fitting an RF, checking OOB error convergence
3. **Variable selection** --- VIMP and minimal depth via `max.subtree()`
4. **Dependence plots** --- variable dependence and partial dependence via
   `gg_variable()` and `gg_partial_rfsrc()`
5. **Variable interactions** --- conditioning plots and partial dependence
   surfaces

```{r packages}
library(ggplot2)
library(dplyr)
library(tidyr)
library(randomForestSRC)

if (requireNamespace("ggRandomForests", quietly = TRUE)) {
  library(ggRandomForests)
} else if (requireNamespace("pkgload", quietly = TRUE)) {
  pkgload::load_all(export_all = FALSE, helpers = FALSE, attach_testthat = FALSE)
} else {
  stop("Install ggRandomForests (or pkgload for dev builds) to render this vignette.")
}

theme_set(theme_bw())
```

# Data: Boston Housing Values

The Boston Housing data [@Harrison:1978; @Belsley:1980] is a standard benchmark
for regression. It contains data for 506 census tracts of Boston from the 1970
census. The objective is to predict the median value of owner-occupied homes
(`medv`, in \$1000s) from 13 predictors covering crime, zoning, industry,
environmental quality, and housing characteristics. We use the copy from the
**MASS** package [@mass:2002].

```{r data-load}
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas) # nolint: object_name_linter
```

```{r variable-labels}
st_labs <- c(
  crim    = "Crime rate by town",
  zn      = "Residential land zoned > 25k sq ft (%)",
  indus   = "Non-retail business acres (%)",
  chas    = "Borders Charles River",
  nox     = "Nitrogen oxides (10 ppm)",
  rm      = "Rooms per dwelling",
  age     = "Units built before 1940 (%)",
  dis     = "Distance to employment centers",
  rad     = "Highway accessibility index",
  tax     = "Property tax rate per $10,000",
  ptratio = "Pupil-teacher ratio",
  black   = "Proportion of Black residents",
  lstat   = "Lower status population (%)",
  medv    = "Median home value ($1000s)"
)
```

## Exploratory data analysis

We plot each predictor against the response (`medv`), coloring by the sole
categorical variable (`chas`, whether the tract borders the Charles River). A
loess smooth highlights the marginal trend.

```{r eda, fig.cap="EDA: each predictor vs. median home value, colored by Charles River indicator."}
dta <- Boston |>
  pivot_longer(c(-medv, -chas), names_to = "variable", values_to = "value")

ggplot(dta, aes(x = medv, y = value, color = chas)) +
  geom_point(alpha = 0.4) +
  geom_smooth(aes(x = medv, y = value), color = "grey30",
              inherit.aes = FALSE, se = FALSE) +
  labs(y = "", x = st_labs["medv"]) +
  scale_color_brewer(palette = "Set2") +
  facet_wrap(~variable, scales = "free_y", ncol = 3)
```

Even from this simple view, two relationships stand out: `medv` against
`lstat` (lower status %) and `medv` against `rm` (rooms per dwelling). Keep
those two in mind --- we expect the random forest to rank them as the most
important predictors, and the rest of the vignette comes back to check.

# Growing a Random Forest

We grow a regression forest using all 13 predictors. The `rfsrc()` function
detects the regression family from the continuous response.

```{r rfsrc-fit}
rfsrc_Boston <- rfsrc(medv ~ ., data = Boston, # nolint: object_name_linter
                      ntree = 200, importance = TRUE, err.block = 5)
rfsrc_Boston
```

The forest grew `r rfsrc_Boston$ntree` trees, splitting on
`r rfsrc_Boston$mtry` randomly selected candidate variables at each node, and
stopping at a minimum terminal node size of `r rfsrc_Boston$nodesize`.

## OOB error convergence

```{r error-plot, fig.height=4, fig.cap="OOB mean squared error vs. number of trees."}
gg_e <- gg_error(rfsrc_Boston)
gg_e <- gg_e |> filter(!is.na(error))
class(gg_e) <- c("gg_error", class(gg_e))
plot(gg_e)
```

The error stabilizes well before 500 trees, indicating the forest is large
enough for reliable predictions.

## OOB predictions

```{r rfsrc-predicted, fig.height=4, fig.cap="OOB predicted median home values. Points are jittered; boxplot shows the distribution."}
plot(gg_rfsrc(rfsrc_Boston), alpha = 0.5) +
  coord_cartesian(ylim = c(5, 49))
```

Each point is a single tract's OOB prediction. The distribution is a sanity
check --- we are more interested in the *why* behind these predictions.

# Variable Selection

## Variable importance (VIMP)

VIMP, computed by [randomForestSRC](https://www.randomforestsrc.org), measures
the increase in OOB prediction error when a variable's values are randomly
permuted across the out-of-bag observations [@Breiman:2001]. Permutation severs
the variable's link to the response on purpose: if breaking that link hurts
accuracy, the variable is carrying real signal. Large positive values mean the
variable is essential; negative values suggest it is no more informative than
noise.

```{r vimp-plot, fig.cap="VIMP ranking. Longer blue bars indicate more important variables."}
plot(gg_vimp(rfsrc_Boston), lbls = st_labs)
```

`lstat` and `rm` dominate, with a clear gap to the remaining predictors. All
VIMP values are positive, indicating every predictor contributes at least
marginally.

The permutation approach contrasts with varPro release-rule importance
[@Lu2024varpro], available through `gg_varpro()`. Rather than perturbing data
synthetically, varPro compares local estimators on the observed data directly:
no permutation, no manufactured feature values. Because the two methods measure
fundamentally different things, a variable can rank high under one and low under
the other. When they agree, the evidence is strong; when they disagree, that
disagreement itself is worth investigating, pointing either to a variable whose
effect is highly non-linear or to one that matters only in combination with
others.

## Minimal depth

Minimal depth [@Ishwaran:2010] ranks variables by how close to the root node
they first split, on average. Variables that partition large portions of the
population early are considered most important.

```{r varsel}
md_Boston <- max.subtree(rfsrc_Boston) # nolint: object_name_linter
```

The threshold is `r round(md_Boston$threshold, 2)`, selecting
`r length(md_Boston$topvars)` variables:
`r paste(md_Boston$topvars, collapse = ", ")`.

Both VIMP and minimal depth agree on the dominance of `lstat` and `rm`. We use
the minimal depth top variables for the remainder of the analysis.

```{r select-vars}
xvar <- md_Boston$topvars
```

# Variable Dependence

## Variable dependence plots

Variable dependence shows each tract's OOB predicted `medv` plotted against a
predictor, with a loess smooth indicating the trend.

```{r vardep-panel, fig.cap="Variable dependence for top predictors (minimal depth rank order)."}
gg_v <- gg_variable(rfsrc_Boston)

plot(gg_v, xvar = xvar, panel = TRUE, alpha = 0.5) +
  labs(y = st_labs["medv"], x = "")
```

The panels confirm what EDA suggested: `medv` decreases sharply with `lstat` and
increases with `rm`, both in strongly non-linear ways. The remaining variables
show weaker but still discernible trends.

```{r vardep-chas, fig.height=4, fig.cap="Variable dependence for Charles River (categorical)."}
plot(gg_v, xvar = "chas", alpha = 0.4) +
  labs(y = st_labs["medv"])
```

Most tracts do not border the Charles River, and the predicted value
distributions largely overlap --- consistent with `chas` ranking last in both
VIMP and minimal depth.

## Partial dependence

Partial dependence integrates out the effects of all other covariates, giving a
risk-adjusted view of each predictor's marginal effect [@Friedman:2000]:

$$
\tilde{f}(x) = \frac{1}{n} \sum_{i=1}^n \hat{f}(x, x_{i,o})
$$

We use `gg_partial_rfsrc()`, which calls `randomForestSRC::partial.rfsrc()`
directly and returns a `gg_partial_rfsrc` object. The quickest path to a
figure is `plot(pd)`, which sorts out continuous and categorical variables
for you. When you want to control the layout yourself, the underlying data
is in `pd$continuous`.

```{r partial-dep, fig.cap="Partial dependence for top predictors."}
pd <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = xvar)

# Quick S3 plot — works out of the box for the standard regression case
plot(pd)
```

For a publication-ready layout with custom axis labels, access the underlying
data frame directly:

```{r partial-dep-custom, fig.cap="Partial dependence (custom styling)."}
ggplot(pd$continuous, aes(x = x, y = yhat)) +
  geom_line(color = "steelblue", linewidth = 1) +
  facet_wrap(~name, scales = "free_x") +
  labs(y = st_labs["medv"], x = "") +
  theme_bw()
```

`lstat` shows a strongly concave relationship, while `rm` stays flat below
about 6 rooms and then climbs sharply. Shapes like these are awkward to
capture with a simple parametric transform, since you would have to guess
the form in advance, but the random forest picks them up on its own.

# Variable Interactions and Conditioning Plots

## Conditioning on a categorical variable

The simplest coplot conditions on a categorical variable. Here we examine
`medv` vs. `lstat`, split by Charles River status:

```{r coplot-chas, fig.height=4, fig.cap="Variable dependence on lstat, conditional on Charles River."}
gg_v$chas_label <- ifelse(gg_v$chas, "Borders Charles River",
                          "Does not border")

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  facet_wrap(~chas_label)
```

The decreasing trend holds in both groups, with slightly higher values along the
Charles River at every `lstat` level.

## Conditioning on a continuous variable

To investigate the `lstat`--`rm` interaction, we bin `rm` into six quantile
groups using `quantile_pts()` and facet:

```{r coplot-rm, fig.cap="Predicted medv vs. lstat, conditional on rooms-per-dwelling groups."}
rm_pts <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 6, intervals = TRUE)
gg_v$rm_grp <- cut(rfsrc_Boston$xvar$rm, breaks = rm_pts)
levels(gg_v$rm_grp) <- paste("rm in", levels(gg_v$rm_grp))

plot(gg_v, xvar = "lstat", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["lstat"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~rm_grp)
```

Median values decrease with `lstat` within every `rm` group, but the intercept
shifts upward with more rooms. Smaller homes in low-`lstat` (high-status)
neighborhoods still command high prices.

The complement view --- `medv` vs. `rm`, conditional on `lstat` groups ---
completes the picture:

```{r coplot-lstat, fig.cap="Predicted medv vs. rooms, conditional on lower-status groups."}
lstat_pts <- quantile_pts(rfsrc_Boston$xvar$lstat, groups = 6,
                          intervals = TRUE)
gg_v$lstat_grp <- cut(rfsrc_Boston$xvar$lstat, breaks = lstat_pts)
levels(gg_v$lstat_grp) <- paste("lstat in", levels(gg_v$lstat_grp))

plot(gg_v, xvar = "rm", alpha = 0.5) +
  labs(y = st_labs["medv"], x = st_labs["rm"]) +
  theme(legend.position = "none") +
  scale_color_brewer(palette = "Set3") +
  facet_wrap(~lstat_grp)
```

The `rm` effect is strongest in low-`lstat` tracts (bottom-left panels) and
nearly flat in high-`lstat` tracts, confirming a meaningful interaction.

# Partial Dependence Surface

To visualize the joint partial dependence of `medv` on `lstat` and `rm`, we
compute partial dependence on a grid: 10 values of `rm`, each evaluated at 25
points along `lstat`.

```{r surface-data, cache=TRUE}
rm_grid <- quantile_pts(rfsrc_Boston$xvar$rm, groups = 10)

surface_list <- lapply(rm_grid, function(rm_val) {
  newx <- rfsrc_Boston$xvar
  newx$rm <- rm_val
  pd_rm <- gg_partial_rfsrc(rfsrc_Boston, xvar.names = "lstat", newx = newx)
  df <- pd_rm$continuous
  df$rm <- rm_val
  df
})

surface_df <- bind_rows(surface_list)
```

```{r pd-surface, fig.cap="Partial dependence surface: median home value as a function of lstat and rm. Fill colour is the predicted median value."}
ggplot(surface_df, aes(x = x, y = rm, fill = yhat)) +
  geom_tile() +
  scale_fill_viridis_c(name = "Median Value\n($1000s)") +
  labs(x = "Lower Status (%)", y = "Rooms per Dwelling") +
  theme_bw()
```

The surface confirms the strong interaction: home values are highest when `lstat`
is low and `rm` is high (upper-left corner), dropping steeply along both axes.
The non-planar curvature --- particularly the sharp step near `rm` = 7 ---
demonstrates the kind of complex, non-linear structure that random forests
capture naturally.

# Conclusion

We have walked a full random forest regression analysis with
**randomForestSRC** and **ggRandomForests**, and the pieces line up:

- `gg_error()` showed the OOB error settling well before 500 trees.
- VIMP (`gg_vimp()`) and minimal depth (`max.subtree()`) agreed on the same
  story: `lstat` and `rm` dominate, with a clear gap to everything else.
- `gg_variable()` traced strongly non-linear predictor--response curves, the
  same shapes the raw-data EDA hinted at.
- Partial dependence from `gg_partial_rfsrc()` gave the risk-adjusted version
  of those curves: concave for `lstat`, threshold-like for `rm`.
- Conditioning plots and the partial dependence surface pulled out the `lstat`--`rm`
  interaction, with the room-size effect strongest in high-status tracts.

Notice the pattern in all of this. Each `gg_*()` function returns a tidy
object (often a data frame, sometimes a small list of data frames); the
plotting is a separate step. Use the package's `plot()`
methods when the default figure is what you want, and reach for `ggplot2`
directly when it is not.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R
\name{plot.gg_partial}
\alias{plot.gg_partial}
\title{Plot a \code{\link{gg_partial}} object}
\usage{
\method{plot}{gg_partial}(x, ...)
}
\arguments{
\item{x}{A \code{\link{gg_partial}} object (output of \code{\link{gg_partial}}).}

\item{...}{Not currently used; reserved for future arguments.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.  When only one
  variable type is present a single \code{ggplot} is returned.  When both
  continuous and categorical variables are present the two panels are
  combined vertically via \code{patchwork::wrap_plots()}, which also
  satisfies \code{inherits(p, "ggplot")}.
}
\description{
Produces ggplot2 partial dependence curves from the named list returned by
\code{\link{gg_partial}}.  Continuous predictors are shown as line plots;
categorical predictors are shown as bar charts.  Both panels are faceted by
variable name so multiple predictors can be compared at a glance.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
pv <- randomForestSRC::plot.variable(rf, partial = TRUE, show.plots = FALSE)
pd <- gg_partial(pv)
plot(pd)

}
\seealso{
\code{\link{gg_partial}}, \code{\link{plot.gg_variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R
\name{plot.gg_partial}
\alias{plot.gg_partial}
\title{Plot a \code{\link{gg_partial}} object}
\usage{
\method{plot}{gg_partial}(x, ...)
}
\arguments{
\item{x}{A \code{\link{gg_partial}} object (output of \code{\link{gg_partial}}).}

\item{...}{Not currently used; reserved for future arguments.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.  When only one
variable type is present a single \code{ggplot} is returned.  When both
continuous and categorical variables are present the two panels are
combined vertically via \code{patchwork::wrap_plots()}, which also
satisfies \code{inherits(p, "ggplot")}.
}
\description{
Turns a \code{\link{gg_partial}} object into a ggplot2 figure.  Each curve
is a partial dependence trace -- the forest's average prediction as one
predictor is swept across its range while the rest are marginalized over the
training data.  Continuous predictors appear as line plots; categorical
predictors appear as bar charts.  Both panels are faceted by variable name
so you can compare the shape and scale of each variable's effect at a
glance.
}
\details{
When a \code{model} label was attached in \code{gg_partial()}, lines are
coloured by model -- handy for overlaying results from two forests (e.g.,
one tuned, one default) in the same figure.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
pv <- randomForestSRC::plot.variable(rf, partial = TRUE, show.plots = FALSE)
pd <- gg_partial(pv)
plot(pd)

}
\seealso{
\code{\link{gg_partial}}, \code{\link{plot.gg_variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggrandomforests.news.R
\name{ggrandomforests.news}
\alias{ggrandomforests.news}
\title{Display the ggRandomForests NEWS file}
\usage{
ggrandomforests.news(...)
}
\arguments{
\item{...}{Currently unused; reserved for future arguments.}
}
\value{
Called for its side-effect of opening the NEWS file in the system
  pager (\code{file.show}). Returns \code{invisible(NULL)}.
}
\description{
Opens the package change log in the system pager so users can read the
version history without leaving their R session. The function reads
\code{NEWS.md} from the installed package root (the canonical change log
that R also surfaces via \code{utils::news()}). \code{inst/NEWS} was
removed in v2.7.1 to eliminate the duplicate-source-of-truth maintenance
hole that left the legacy file frozen at v2.4.0; if any installation
still ships an \code{inst/NEWS}, this function falls back to it.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggrandomforests.news.R
\name{ggrandomforests.news}
\alias{ggrandomforests.news}
\title{Display the ggRandomForests NEWS file}
\usage{
ggrandomforests.news(...)
}
\arguments{
\item{...}{Currently unused; reserved for future arguments.}
}
\value{
Called for its side-effect of opening the NEWS file in the system
pager (\code{file.show}). Returns \code{invisible(NULL)}.
}
\description{
Opens the package change log in the system pager so users can read the
version history without leaving their R session. The function reads
\code{NEWS.md} from the installed package root (the canonical change log
that R also surfaces via \code{utils::news()}). \code{inst/NEWS} was
removed in v2.7.1 to eliminate the duplicate-source-of-truth maintenance
hole that left the legacy file frozen at v2.4.0; if any installation
still ships an \code{inst/NEWS}, this function falls back to it.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_variable.R
\name{gg_variable}
\alias{gg_variable}
\alias{gg_variable.rfsrc}
\alias{gg_variable.randomForest}
\alias{gg_variable.random}
\title{Marginal variable dependence data object.}
\usage{
gg_variable(object, ...)
}
\arguments{
\item{object}{A \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object, or a
\code{\link[randomForestSRC]{plot.variable}} result.}

\item{...}{Optional arguments such as \code{time}, \code{time_labels}, and
\code{oob} that tailor the marginal dependence extraction.}
}
\value{
A \code{gg_variable} object: a \code{data.frame} of all predictor
  columns from the training data paired with the OOB (or in-bag) predicted
  response. For survival forests each requested time horizon produces an
  additional column named by \code{time_labels}. The object carries a
  \code{"family"} class attribute (\code{"regr"}, \code{"class"}, or
  \code{"surv"}) used by \code{\link{plot.gg_variable}} for dispatch.
}
\description{
\code{\link[randomForestSRC]{plot.variable}} generates a
\code{data.frame} containing the marginal variable dependence or the
partial variable dependence. The \code{gg_variable} function creates a
\code{data.frame} of containing the full set of covariate data (predictor
variables) and the predicted response for each observation. Marginal
dependence figures are created using the \code{\link{plot.gg_variable}}
function.
For \code{randomForest} fits the original model frame is rebuilt
from the stored call so that the same predictors can be paired with the
in-sample predictions.

Optional arguments include \code{time} (scalar or vector of survival times
of interest), \code{time_labels} (labels for multiple survival horizons) and
\code{oob} which toggles between out-of-bag and in-bag predictions when the
forest stores both.
}
\details{
The marginal variable dependence is determined by comparing
relation between the predicted response from the randomForest and a
covariate of interest.

The \code{gg_variable} function operates on a
\code{\link[randomForestSRC]{rfsrc}} object, the output from the
\code{\link[randomForestSRC]{plot.variable}} function, or on a fitted
\code{\link[randomForest]{randomForest}} object via the formula interface.
}
\examples{
## ------------------------------------------------------------
## classification (small, runs on CRAN)
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)

gg_dta <- gg_variable(rfsrc_iris)
plot(gg_dta, xvar = "Sepal.Width")

\donttest{
## ------------------------------------------------------------
## Additional classification / regression / survival examples are
## guarded with \donttest because the cumulative example time exceeds
## the 10-second CRAN budget. Run locally with `R CMD check
## --run-donttest` (or `devtools::check(run_dont_test = TRUE)`) to
## exercise them.
## ------------------------------------------------------------
plot(gg_dta, xvar = "Sepal.Length")
plot(gg_dta, xvar = rfsrc_iris$xvar.names, panel = TRUE)

## ------------------------------------------------------------
## regression
## ------------------------------------------------------------

## -------- air quality data
rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality, ntree = 50)
gg_dta <- gg_variable(rfsrc_airq)

# an ordinal variable
gg_dta[, "Month"] <- factor(gg_dta[, "Month"])

plot(gg_dta, xvar = "Wind")
plot(gg_dta, xvar = "Temp")
plot(gg_dta, xvar = "Solar.R")
plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
plot(gg_dta, xvar = "Month", notch = TRUE)

## -------- motor trend cars data
rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)

gg_dta <- gg_variable(rfsrc_mtcars)

# mtcars$cyl is an ordinal variable
gg_dta$cyl <- factor(gg_dta$cyl)
gg_dta$am <- factor(gg_dta$am)
gg_dta$vs <- factor(gg_dta$vs)
gg_dta$gear <- factor(gg_dta$gear)
gg_dta$carb <- factor(gg_dta$carb)

plot(gg_dta, xvar = "cyl")
plot(gg_dta, xvar = "disp")
plot(gg_dta, xvar = "hp")
plot(gg_dta, xvar = "wt")
plot(gg_dta, xvar = c("disp", "hp", "drat", "wt", "qsec"), panel = TRUE)
plot(gg_dta,
  xvar = c("cyl", "vs", "am", "gear", "carb"), panel = TRUE,
  notch = TRUE
)

## -------- Boston data
if (requireNamespace("MASS", quietly = TRUE)) {
  data(Boston, package = "MASS")
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
  gg_dta <- gg_variable(rf_boston)
  plot(gg_dta)
  plot(gg_dta, panel = TRUE)
}

## ------------------------------------------------------------
## survival examples
## ------------------------------------------------------------

## -------- veteran data
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., veteran,
  nsplit = 10,
  ntree = 50
)

# get the 90-day survival time.
gg_dta <- gg_variable(rfsrc_veteran, time = 90)

# Generate variable dependence plots for age and diagtime
plot(gg_dta, xvar = "age")
plot(gg_dta, xvar = "diagtime")

# Generate coplots
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE, se = FALSE)

# Compare survival at 30, 90, and 365 days simultaneously
gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
plot(gg_dta, xvar = "age")
}

}
\seealso{
\code{\link{plot.gg_variable}},
  \code{\link[randomForestSRC]{plot.variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_variable.R
\name{gg_variable}
\alias{gg_variable}
\alias{gg_variable.rfsrc}
\alias{gg_variable.randomForest}
\alias{gg_variable.random}
\title{Marginal variable dependence data object.}
\usage{
gg_variable(object, ...)
}
\arguments{
\item{object}{A \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object, or a
\code{\link[randomForestSRC]{plot.variable}} result.}

\item{...}{Optional arguments \code{time}, \code{time_labels}, and
\code{oob} that tune the marginal dependence extraction.}
}
\value{
A \code{gg_variable} object: a \code{data.frame} pairing every
training predictor column with the OOB (or in-bag) predicted response.
For survival forests, each requested time horizon adds a column named by
\code{time_labels}. The object carries a \code{"family"} class attribute
(\code{"regr"}, \code{"class"}, or \code{"surv"}) that
\code{\link{plot.gg_variable}} uses for dispatch.
}
\description{
\code{\link[randomForestSRC]{plot.variable}} generates a
\code{data.frame} containing the marginal variable dependence or the
partial variable dependence. The \code{gg_variable} function creates a
\code{data.frame} of containing the full set of covariate data (predictor
variables) and the predicted response for each observation. Marginal
dependence figures are created using the \code{\link{plot.gg_variable}}
function.
A \code{randomForest} fit does not keep the model frame, so for those
objects \code{gg_variable} rebuilds it from the stored call. That lets the
same predictors be paired with the in-sample predictions.

A few optional arguments tune the extraction: \code{time} (one survival
time, or a vector of them), \code{time_labels} (labels for multiple
survival horizons), and \code{oob}, which switches between out-of-bag and
in-bag predictions when the forest carries both.
}
\details{
The marginal variable dependence is determined by comparing
relation between the predicted response from the randomForest and a
covariate of interest.

The \code{gg_variable} function operates on a
\code{\link[randomForestSRC]{rfsrc}} object, the output from the
\code{\link[randomForestSRC]{plot.variable}} function, or on a fitted
\code{\link[randomForest]{randomForest}} object via the formula interface.
}
\examples{
## ------------------------------------------------------------
## classification (small, runs on CRAN)
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)

gg_dta <- gg_variable(rfsrc_iris)
plot(gg_dta, xvar = "Sepal.Width")

\donttest{
## ------------------------------------------------------------
## Additional classification / regression / survival examples are
## guarded with \donttest because the cumulative example time exceeds
## the 10-second CRAN budget. Run locally with `R CMD check
## --run-donttest` (or `devtools::check(run_dont_test = TRUE)`) to
## exercise them.
## ------------------------------------------------------------
plot(gg_dta, xvar = "Sepal.Length")
plot(gg_dta, xvar = rfsrc_iris$xvar.names, panel = TRUE)

## ------------------------------------------------------------
## regression
## ------------------------------------------------------------

## -------- air quality data
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality, ntree = 50)
gg_dta <- gg_variable(rfsrc_airq)

# an ordinal variable
gg_dta[, "Month"] <- factor(gg_dta[, "Month"])

plot(gg_dta, xvar = "Wind")
plot(gg_dta, xvar = "Temp")
plot(gg_dta, xvar = "Solar.R")
plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
plot(gg_dta, xvar = "Month", notch = TRUE)

## -------- motor trend cars data
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)

gg_dta <- gg_variable(rfsrc_mtcars)

# mtcars$cyl is an ordinal variable
gg_dta$cyl <- factor(gg_dta$cyl)
gg_dta$am <- factor(gg_dta$am)
gg_dta$vs <- factor(gg_dta$vs)
gg_dta$gear <- factor(gg_dta$gear)
gg_dta$carb <- factor(gg_dta$carb)

plot(gg_dta, xvar = "cyl")
plot(gg_dta, xvar = "disp")
plot(gg_dta, xvar = "hp")
plot(gg_dta, xvar = "wt")
plot(gg_dta, xvar = c("disp", "hp", "drat", "wt", "qsec"), panel = TRUE)
plot(gg_dta,
  xvar = c("cyl", "vs", "am", "gear", "carb"), panel = TRUE,
  notch = TRUE
)

## -------- Boston data
if (requireNamespace("MASS", quietly = TRUE)) {
  data(Boston, package = "MASS")
  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston)
  gg_dta <- gg_variable(rf_boston)
  plot(gg_dta)
  plot(gg_dta, panel = TRUE)
}

## ------------------------------------------------------------
## survival examples
## ------------------------------------------------------------

## -------- veteran data
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., veteran,
  nsplit = 10,
  ntree = 50
)

# get the 90-day survival time.
gg_dta <- gg_variable(rfsrc_veteran, time = 90)

# Generate variable dependence plots for age and diagtime
plot(gg_dta, xvar = "age")
plot(gg_dta, xvar = "diagtime")

# Generate coplots
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE, se = FALSE)

# Compare survival at 30, 90, and 365 days simultaneously
gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))
plot(gg_dta, xvar = "age")
}

}
\seealso{
\code{\link{plot.gg_variable}},
\code{\link[randomForestSRC]{plot.variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{.varpro_imp_stats}
\alias{.varpro_imp_stats}
\title{Compute per-variable box statistics from a per-tree importance matrix}
\usage{
.varpro_imp_stats(mat, local.std = TRUE)
}
\description{
When \code{local.std = TRUE} the columns of \code{mat} are standardised
to unit variance before computing quantiles so that the display scale
matches the aggregate z-score.  The \code{mean} column always stores raw
(unstandardised) column means.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_rfsrc.R
\name{bootstrap_survival}
\alias{bootstrap_survival}
\title{Bootstrap pointwise confidence bands for a mean survival curve}
\usage{
bootstrap_survival(gg_dta, bs_samples, level_set)
}
\arguments{
\item{gg_dta}{A wide \code{data.frame} of survival probabilities as returned
by the survival branch of \code{\link{gg_rfsrc.rfsrc}}, before the
optional pivot to long form. Columns \code{obs_id}, \code{event}, and
\code{group} (if present) are excluded from the resampling.}

\item{bs_samples}{Integer; number of bootstrap resamples.}

\item{level_set}{Numeric vector of length 2 giving the lower and upper
quantile probabilities for the confidence band (e.g. \code{c(0.025, 0.975)}
for a 95\% CI).}
}
\value{
A \code{data.frame} with one row per unique event time and columns
  \code{value} (time), \code{lower}, \code{upper}, \code{median}, and
  \code{mean}.
}
\description{
Draws \code{bs_samples} bootstrap resamples (with replacement) of the
per-observation survival curves stored in \code{gg_dta}, computes the column
means to obtain a bootstrapped mean curve per resample, then returns the
pointwise quantiles at \code{level_set} and the overall mean across
resamples.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_rfsrc.R
\name{bootstrap_survival}
\alias{bootstrap_survival}
\title{Bootstrap pointwise confidence bands for a mean survival curve}
\usage{
bootstrap_survival(gg_dta, bs_samples, level_set)
}
\arguments{
\item{gg_dta}{A wide \code{data.frame} of survival probabilities as returned
by the survival branch of \code{\link{gg_rfsrc.rfsrc}}, before the
optional pivot to long form. Columns \code{obs_id}, \code{event}, and
\code{group} (if present) are excluded from the resampling.}

\item{bs_samples}{Integer; number of bootstrap resamples.}

\item{level_set}{Numeric vector of length 2 giving the lower and upper
quantile probabilities for the confidence band (e.g. \code{c(0.025, 0.975)}
for a 95\% CI).}
}
\value{
A \code{data.frame} with one row per unique event time and columns
\code{value} (time), \code{lower}, \code{upper}, \code{median}, and
\code{mean}.
}
\description{
Draws \code{bs_samples} bootstrap resamples (with replacement) of the
per-observation survival curves stored in \code{gg_dta}, computes the column
means to obtain a bootstrapped mean curve per resample, then returns the
pointwise quantiles at \code{level_set} and the overall mean across
resamples.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_error.R
\name{plot.gg_error}
\alias{plot.gg_error}
\title{Plot a \code{\link{gg_error}} object}
\usage{
\method{plot}{gg_error}(x, ...)
}
\arguments{
\item{x}{A \code{\link{gg_error}} object created from either a
\code{\link[randomForestSRC]{rfsrc}} or a
\code{\link[randomForest]{randomForest}} object.  A raw forest object
may also be supplied and will be passed through \code{\link{gg_error}}
automatically before plotting.}

\item{...}{Extra arguments forwarded to the underlying \code{ggplot2}
geometry calls (e.g. \code{size}, \code{linetype}).}
}
\value{
A \code{ggplot} object with \code{ntree} on the x-axis and
OOB error rate on the y-axis.  Single-outcome forests (regression,
survival) produce a single line; multi-outcome forests (classification)
produce one coloured line per class.
}
\description{
A plot of the cumulative OOB error rates of the random forest as a
function of number of trees.
}
\details{
The gg_error plot is used to track the convergence of the
randomForest. This figure is a reproduction of the error plot
from the \code{\link[randomForestSRC]{plot.rfsrc}} function.
}
\examples{
## Examples from RFSRC package...
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## ------------- iris data
## You can build a randomForest
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris,
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_iris)

# Plot the gg_error object
plot(gg_dta)

## RandomForest example
rf_iris <- randomForest::randomForest(Species ~ .,
  data = iris,
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE
)
gg_dta <- gg_error(rf_iris)
plot(gg_dta)

gg_dta <- gg_error(rf_iris, training = TRUE)
plot(gg_dta)
## ------------------------------------------------------------
## Regression example
## ------------------------------------------------------------
## ------------- airq data
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ .,
  data = airquality,
  na.action = "na.impute",
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_airq)

# Plot the gg_error object
plot(gg_dta)


## ------------- Boston data
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas)
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ .,
  data = Boston,
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_boston)

# Plot the gg_error object
plot(gg_dta)

## ------------- mtcars data
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars,
  importance = TRUE,
  save.memory = TRUE,
  forest = TRUE,
  tree.err = TRUE)

# Get a data.frame containing error rates
gg_dta<- gg_error(rfsrc_mtcars)

# Plot the gg_error object
plot(gg_dta)


## ------------------------------------------------------------
## Survival example
## ------------------------------------------------------------
## ------------- veteran data
## randomized trial of two treatment regimens for lung cancer
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                       tree.err = TRUE)

gg_dta <- gg_error(rfsrc_veteran)
plot(gg_dta)

## ------------- pbc data
# Load a cached randomForestSRC object
# We need to create this dataset
data(pbc, package = "randomForestSRC",)
# For whatever reason, the age variable is in days... makes no sense to me
for (ind in seq_len(dim(pbc)[2])) {
 if (!is.factor(pbc[, ind])) {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 } else {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
     if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 }
 if (!is.logical(pbc[, ind]) &
     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
   pbc[, ind] <- factor(pbc[, ind])
 }
}
#Convert age to years
pbc$age <- pbc$age / 364.24

pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]
# Create a test set from the remaining patients
pbc_test <- pbc[which(is.na(pbc$treatment)), ]

#========
# build the forest:
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
 dta_train,
 nsplit = 10,
 na.action = "na.impute",
 tree.err = TRUE,
 forest = TRUE,
 importance = TRUE,
 save.memory = TRUE
)


gg_dta <- gg_error(rfsrc_pbc)
plot(gg_dta)

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_error}} \code{\link[randomForestSRC]{rfsrc}}
\code{\link[randomForestSRC]{plot.rfsrc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_survival.R
\name{plot.gg_survival}
\alias{plot.gg_survival}
\title{Plot a \code{\link{gg_survival}}  object.}
\usage{
\method{plot}{gg_survival}(
  x,
  type = c("surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"),
  error = c("shade", "bars", "lines", "none"),
  label = NULL,
  ...
)
}
\arguments{
\item{x}{\code{\link{gg_survival}} or a survival \code{\link{gg_rfsrc}}
object created from a \code{\link[randomForestSRC]{rfsrc}} object}

\item{type}{"surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"}

\item{error}{"shade", "bars", "lines" or "none"}

\item{label}{Modify the legend label when gg_survival has stratified samples}

\item{...}{Additional arguments forwarded to \code{geom_step()}. When
\code{alpha} is supplied it is passed to the step geom and a halved
value is used for the ribbon overlay.}
}
\value{
A \code{ggplot} object. The y-axis shows the chosen \code{type}
  (e.g. survival probability for \code{"surv"}) and the x-axis shows time.
  Confidence shading, bars, or lines are added when the input object carries
  confidence-interval columns.
}
\description{
Plot a \code{\link{gg_survival}}  object.
}
\examples{
## -------- pbc data
data(pbc, package = "randomForestSRC")
pbc$time <- pbc$days / 364.25

# This is the same as kaplan
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc
)

plot(gg_dta, error = "none")
plot(gg_dta)

# Stratified on treatment variable.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment"
)

plot(gg_dta, error = "none")
plot(gg_dta)
plot(gg_dta, label = "treatment")

# ...with smaller confidence limits.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment", conf.int = .68
)

plot(gg_dta, error = "lines")
plot(gg_dta, label = "treatment", error = "lines")

# ...with smaller confidence limits.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "sex", conf.int = .68
)

plot(gg_dta, error = "lines")
plot(gg_dta, label = "sex", error = "lines")

}
\seealso{
\code{\link{gg_survival}}, \code{\link{kaplan}},
  \code{\link{nelson}}, \code{\link{gg_rfsrc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_survival.R
\name{plot.gg_survival}
\alias{plot.gg_survival}
\title{Plot a \code{\link{gg_survival}} object.}
\usage{
\method{plot}{gg_survival}(
  x,
  type = c("surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"),
  error = c("shade", "bars", "lines", "none"),
  label = NULL,
  ...
)
}
\arguments{
\item{x}{\code{\link{gg_survival}} or a survival \code{\link{gg_rfsrc}}
object created from a \code{\link[randomForestSRC]{rfsrc}} object}

\item{type}{"surv", "cum_haz", "hazard", "density", "mid_int", "life", "proplife"}

\item{error}{"shade", "bars", "lines" or "none"}

\item{label}{Modify the legend label when gg_survival has stratified samples}

\item{...}{Additional arguments forwarded to \code{geom_step()}. When
\code{alpha} is supplied it is passed to the step geom and a halved
value is used for the ribbon overlay.}
}
\value{
A \code{ggplot} object. The y-axis shows the chosen \code{type}
(e.g. survival probability for \code{"surv"}) and the x-axis shows time.
Confidence shading, bars, or lines are added when the input object carries
confidence-interval columns.
}
\description{
Draws a Kaplan-Meier (or Nelson-Aalen) survival curve from a
\code{\link{gg_survival}} object.  You can overlay a confidence envelope
around the curve using \code{error}: \code{"shade"} fills the area between
the pointwise confidence limits, \code{"lines"} draws them as dashed step
functions, and \code{"bars"} shows them as error bars.  When \code{gg_survival}
was called with a \code{by} argument, each group gets its own step function
and the \code{label} argument renames the legend.
}
\details{
The \code{type} argument selects which quantity to plot on the y-axis --
survival probability (\code{"surv"}) is the default, but cumulative hazard,
density, and several transformed scales are available for the cases where a
linear scale reveals more about the tails.
}
\examples{
## -------- pbc data
data(pbc, package = "randomForestSRC")
pbc$time <- pbc$days / 364.25

# This is the same as kaplan
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc
)

plot(gg_dta, error = "none")
plot(gg_dta)

# Stratified on treatment variable.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment"
)

plot(gg_dta, error = "none")
plot(gg_dta)
plot(gg_dta, label = "treatment")

# ...with smaller confidence limits.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment", conf.int = .68
)

plot(gg_dta, error = "lines")
plot(gg_dta, label = "treatment", error = "lines")

# ...with smaller confidence limits.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "sex", conf.int = .68
)

plot(gg_dta, error = "lines")
plot(gg_dta, label = "sex", error = "lines")

}
\seealso{
\code{\link{gg_survival}}, \code{\link{kaplan}},
\code{\link{nelson}}, \code{\link{gg_rfsrc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_brier.R
\name{gg_brier}
\alias{gg_brier}
\title{Brier score and CRPS for survival forests}
\usage{
gg_brier(object, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} survival
forest (\code{object$family == "surv"}).}

\item{...}{Currently unused; accepted for S3 dispatch compatibility.}
}
\value{
A \code{gg_brier} \code{data.frame} with columns
  \describe{
    \item{time}{event time grid (\code{object$time.interest}).}
    \item{brier}{overall Brier score at each time.}
    \item{bs.q25, bs.q50, bs.q75, bs.q100}{Brier score within each
      mortality-risk quartile (lowest to highest risk).}
    \item{bs.lower, bs.upper}{15th and 85th percentile of per-subject
      Brier contributions at each time. Used by
      \code{plot.gg_brier(by_quartile = TRUE)} to draw an envelope
      around the overall curve.}
    \item{crps}{running CRPS (overall) at each time, normalised by
      elapsed time.}
    \item{crps.q25, crps.q50, crps.q75, crps.q100}{running CRPS within
      each mortality-risk quartile.}
    \item{crps.lower, crps.upper}{running CRPS of the 15th / 85th
      per-subject Brier percentile, normalised by elapsed time.}
  }
  The integrated CRPS (a single scalar matching
  \code{get.brier.survival()$crps}) is attached as
  \code{attr(., "crps_integrated")}.
}
\description{
Extract the time-resolved Brier score and continuous ranked probability
score (CRPS) for a survival forest grown with \code{randomForestSRC}. The
Brier score is computed at each time on \code{object$time.interest}, both
overall and stratified by mortality-risk quartile. CRPS is the running
trapezoidal integral of the Brier score, normalised by elapsed time, and
is computed within each quartile and overall.
}
\details{
Wraps \code{\link[randomForestSRC]{get.brier.survival}} and
rebuilds the quartile decomposition + running CRPS from the returned
\code{brier.matx} and \code{mort} components, mirroring the computation
in the internal \code{plot.survival} function of \pkg{randomForestSRC}. The Brier score uses
inverse-probability-of-censoring weighting; the censoring distribution
is estimated either by Kaplan-Meier (\code{cens.model = "km"}, the
default) or by a separate censoring forest (\code{cens.model = "rfsrc"}).
}
\examples{
\dontrun{
data(pbc, package = "randomForestSRC")
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(days, status) ~ ., data = pbc, nsplit = 10
)
gg_dta <- gg_brier(rfsrc_pbc)
plot(gg_dta)
plot(gg_dta, type = "crps")
plot(gg_dta, envelope = TRUE)   # overall line + 15-85\% envelope

# Multi-model comparison: stack gg_brier outputs and plot with ggplot2.
rf2 <- randomForestSRC::rfsrc(
  Surv(days, status) ~ ., data = pbc, nsplit = 10, mtry = 4
)
compare_dta <- dplyr::bind_rows(
  dplyr::mutate(gg_brier(rfsrc_pbc), model = "default"),
  dplyr::mutate(gg_brier(rf2),       model = "mtry=4")
)
ggplot2::ggplot(compare_dta,
  ggplot2::aes(x = time, y = brier, colour = model)) +
  ggplot2::geom_line()
}

}
\references{
Graf E., Schmoor C., Sauerbrei W., Schumacher M. (1999). Assessment and
comparison of prognostic classification schemes for survival data.
Statistics in Medicine, 18(17-18):2529-2545.

Gerds T.A., Schumacher M. (2006). Consistent estimation of the expected
Brier score in general survival models with right-censored event times.
Biometrical Journal, 48(6):1029-1040.
}
\seealso{
\code{\link{plot.gg_brier}},
  \code{\link[randomForestSRC]{get.brier.survival}},
  \code{\link{gg_error}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_brier.R
\name{gg_brier}
\alias{gg_brier}
\title{Brier score and CRPS for survival forests}
\usage{
gg_brier(object, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} survival
forest (\code{object$family == "surv"}).}

\item{...}{Currently unused; accepted for S3 dispatch compatibility.}
}
\value{
A \code{gg_brier} \code{data.frame} with columns
\describe{
\item{time}{event time grid (\code{object$time.interest}).}
\item{brier}{overall Brier score at each time.}
\item{bs.q25, bs.q50, bs.q75, bs.q100}{Brier score within each
mortality-risk quartile (lowest to highest risk).}
\item{bs.lower, bs.upper}{15th and 85th percentile of per-subject
Brier contributions at each time. Used by
\code{plot.gg_brier(by_quartile = TRUE)} to draw an envelope
around the overall curve.}
\item{crps}{running CRPS (overall) at each time, normalised by
elapsed time.}
\item{crps.q25, crps.q50, crps.q75, crps.q100}{running CRPS within
each mortality-risk quartile.}
\item{crps.lower, crps.upper}{running CRPS of the 15th / 85th
per-subject Brier percentile, normalised by elapsed time.}
}
The integrated CRPS (a single scalar matching
\code{get.brier.survival()$crps}) is attached as
\code{attr(., "crps_integrated")}.
}
\description{
The Brier score asks a familiar question of any probabilistic forecast:
how far did the predicted probability sit from what actually happened?
For a survival forest the forecast is the predicted survival probability
at a given moment, and the "what happened" is whether the subject was
still alive at that moment.  The score is computed at every event time,
so you get a curve rather than a single number -- lower is better
everywhere.  A perfectly calibrated forest that predicts \code{0} for
every subject who died and \code{1} for every subject who survived would
score \code{0}; a forest that predicts \code{0.5} for everyone scores
roughly \code{0.25} regardless of the true outcome -- that is the
"uninformative" ceiling.
}
\details{
This function extracts the time-resolved Brier score for a survival
forest grown with \code{randomForestSRC}, both overall and broken down
by mortality-risk quartile (lowest-risk to highest-risk subjects).  It
also returns the continuous ranked probability score (CRPS) -- the Brier
score integrated over time and divided by elapsed time, a running average
that summarises calibration up to each point on the time axis.

Because subjects are right-censored, a plain Brier score is biased:
censored subjects contribute no outcome information yet still inflate the
denominator.  The score here uses inverse-probability-of-censoring
weighting (IPCW), which up-weights uncensored observations to compensate.
The censoring distribution is estimated either by Kaplan-Meier
(\code{cens.model = "km"}, the default) or by a separate censoring
forest (\code{cens.model = "rfsrc"}) when the censoring mechanism is
itself covariate-dependent.

Internally, this wraps \code{\link[randomForestSRC]{get.brier.survival}}
and rebuilds the quartile decomposition and running CRPS from the returned
\code{brier.matx} and \code{mort} components, following the approach in
the internal \code{plot.survival} of \pkg{randomForestSRC}.
}
\note{
Brier score / CRPS is \code{randomForestSRC} survival-only; there
is no \code{randomForest} method.
}
\examples{
\donttest{
library(survival)   # Surv() must be on the search path for rfsrc()
data(pbc, package = "randomForestSRC")
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(days, status) ~ ., data = pbc, nsplit = 10
)
gg_dta <- gg_brier(rfsrc_pbc)
plot(gg_dta)
plot(gg_dta, type = "crps")
plot(gg_dta, envelope = TRUE)   # overall line + 15-85\% envelope

# Multi-model comparison: stack gg_brier outputs and plot with ggplot2.
rf2 <- randomForestSRC::rfsrc(
  Surv(days, status) ~ ., data = pbc, nsplit = 10, mtry = 4
)
compare_dta <- dplyr::bind_rows(
  dplyr::mutate(gg_brier(rfsrc_pbc), model = "default"),
  dplyr::mutate(gg_brier(rf2),       model = "mtry=4")
)
ggplot2::ggplot(compare_dta,
  ggplot2::aes(x = time, y = brier, colour = model)) +
  ggplot2::geom_line()
}

}
\references{
Graf E., Schmoor C., Sauerbrei W., Schumacher M. (1999). Assessment and
comparison of prognostic classification schemes for survival data.
Statistics in Medicine, 18(17-18):2529-2545.

Gerds T.A., Schumacher M. (2006). Consistent estimation of the expected
Brier score in general survival models with right-censored event times.
Biometrical Journal, 48(6):1029-1040.
}
\seealso{
\code{\link{plot.gg_brier}},
\code{\link[randomForestSRC]{get.brier.survival}},
\code{\link{gg_error}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_roc.R
\name{plot.gg_roc}
\alias{plot.gg_roc}
\title{ROC plot generic function for a \code{\link{gg_roc}} object.}
\usage{
\method{plot}{gg_roc}(x, which_outcome = NULL, ...)
}
\arguments{
\item{x}{A \code{\link{gg_roc}} object, or a raw
\code{\link[randomForestSRC]{rfsrc}} classification forest or
\code{\link[randomForest]{randomForest}} classification object.  When a
forest is supplied, \code{\link{gg_roc}} is called automatically.}

\item{which_outcome}{Integer; for multi-class problems, the index of the
class whose ROC curve should be plotted.  When \code{NULL} (default) and
the forest has more than two classes, ROC curves for all classes are
overlaid in a single plot.  For binary forests \code{NULL} defaults to
class index 2.}

\item{...}{Additional arguments forwarded to \code{\link{gg_roc}} when
\code{x} is a raw forest object (e.g. \code{oob = FALSE}).}
}
\value{
A \code{ggplot} object.  The x-axis shows 1 - Specificity (FPR)
  and the y-axis shows Sensitivity (TPR).  A dashed red diagonal reference
  line marks the random-classifier baseline.  The AUC value is annotated
  on the plot for single-class curves.  Multi-class plots colour and style
  each class curve distinctly.
}
\description{
ROC plot generic function for a \code{\link{gg_roc}} object.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
# Build a small classification forest (ntree=50 keeps example fast)
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)

# ROC for setosa (outcome index 1)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor (outcome index 2)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica (outcome index 3)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(gg_dta)

# Plot all three ROC curves in one call by iterating over outcome indices
n_cls <- ncol(rfsrc_iris$predicted)
for (i in seq_len(n_cls)) print(plot(gg_roc(rfsrc_iris, which_outcome = i)))

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_roc}} \code{\link{calc_roc}} \code{\link{calc_auc}}
  \code{\link[randomForestSRC]{rfsrc}}
  \code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_roc.R
\name{plot.gg_roc}
\alias{plot.gg_roc}
\title{ROC plot generic function for a \code{\link{gg_roc}} object.}
\usage{
\method{plot}{gg_roc}(x, which_outcome = NULL, ..., panel = c("overlay", "facet"))
}
\arguments{
\item{x}{A \code{\link{gg_roc}} object, or a raw
\code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} classification forest. Hand it a
forest and \code{\link{gg_roc}} is called for you.}

\item{which_outcome}{Integer; for multi-class problems, the index of the
class to plot. When \code{NULL} (default) and the forest has more than two
classes, the curves for all classes are overlaid in one plot. For binary
forests, \code{NULL} defaults to class index 2.}

\item{...}{Additional arguments passed to \code{\link{gg_roc}} when
\code{x} is a raw forest (e.g. \code{oob = FALSE}).}

\item{panel}{Character; layout for per-class ROC objects, the ones from
\code{gg_roc(..., per_class = TRUE)}. \code{"overlay"} (default) draws
every class curve in one panel, coloured by class; \code{"facet"} gives
each class its own panel. Ignored for single-class \code{gg_roc} objects.}
}
\value{
A \code{ggplot} object. The x-axis is 1 - Specificity (FPR), the
y-axis is Sensitivity (TPR), and a dashed red diagonal marks the
random-classifier baseline. Single-class curves carry the AUC as an
annotation; multi-class plots colour and style each class curve
distinctly.
}
\description{
ROC plot generic function for a \code{\link{gg_roc}} object.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
# Build a small classification forest (ntree=50 keeps example fast)
set.seed(42)
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)

# ROC for setosa (outcome index 1)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor (outcome index 2)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica (outcome index 3)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(gg_dta)

# Plot all three ROC curves in one call by iterating over outcome indices
n_cls <- ncol(rfsrc_iris$predicted)
for (i in seq_len(n_cls)) print(plot(gg_roc(rfsrc_iris, which_outcome = i)))

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_roc}} \code{\link{calc_roc}} \code{\link{calc_auc}}
\code{\link[randomForestSRC]{rfsrc}}
\code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantile_pts.R
\name{quantile_pts}
\alias{quantile_pts}
\title{Quantile-based cut points for coplots}
\usage{
quantile_pts(object, groups, intervals = FALSE)
}
\arguments{
\item{object}{Numeric vector of predictor values.}

\item{groups}{Number of quantile points (or intervals) to compute.}

\item{intervals}{Logical indicating whether to return interval boundaries
suitable for \code{cut()} (length \code{groups + 1}) or the interior
quantile points (length \code{groups}).}
}
\value{
Numeric vector of quantile points. When \code{intervals = TRUE}
  the result is strictly increasing and can be supplied to \code{cut()} to
  produce \code{groups} balanced strata.
}
\description{
This helper wraps \code{\link[stats]{quantile}} to create well-spaced
cut points for conditioning plots. When \code{intervals = TRUE} the lower
boundary is nudged down so that \code{cut()} treats the minimum value as a
valid observation.

The output can be passed directly into the breaks argument of the
\code{cut} function for creating groups for coplots.
}
\examples{
data(Boston, package = "MASS")
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston)

# To create 6 intervals, we want 7 points.
# quantile_pts will find balanced intervals
rm_pts <- quantile_pts(rfsrc_boston$xvar$rm, groups = 6, intervals = TRUE)

# Use cut to create the intervals
rm_grp <- cut(rfsrc_boston$xvar$rm, breaks = rm_pts)

summary(rm_grp)

}
\seealso{
\code{cut}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot_methods.R
\name{autoplot.gg}
\alias{autoplot.gg}
\alias{autoplot.gg_error}
\alias{autoplot.gg_vimp}
\alias{autoplot.gg_rfsrc}
\alias{autoplot.gg_variable}
\alias{autoplot.gg_partial}
\alias{autoplot.gg_partial_rfsrc}
\alias{autoplot.gg_partialpro}
\alias{autoplot.gg_roc}
\alias{autoplot.gg_survival}
\alias{autoplot.gg_brier}
\title{\code{autoplot} methods for \pkg{ggRandomForests} data objects}
\usage{
\method{autoplot}{gg_error}(object, ...)

\method{autoplot}{gg_vimp}(object, ...)

\method{autoplot}{gg_rfsrc}(object, ...)

\method{autoplot}{gg_variable}(object, ...)

\method{autoplot}{gg_partial}(object, ...)

\method{autoplot}{gg_partial_rfsrc}(object, ...)

\method{autoplot}{gg_partialpro}(object, ...)

\method{autoplot}{gg_roc}(object, ...)

\method{autoplot}{gg_survival}(object, ...)

\method{autoplot}{gg_brier}(object, ...)
}
\arguments{
\item{object}{A \code{gg_*} data object (see Details).}

\item{...}{Additional arguments forwarded to the underlying
\code{plot.gg_*()} method.}
}
\value{
A \code{ggplot} object.
}
\description{
These methods let you use \code{ggplot2::autoplot()} on any \code{gg_*}
object returned by \pkg{ggRandomForests}.  They are thin wrappers around
the corresponding \code{plot.gg_*()} S3 methods, so all arguments
accepted by those methods are forwarded via \code{...}.
}
\details{
The following \code{gg_*} classes are supported:
\describe{
  \item{\code{gg_error}}{OOB error vs. number of trees}
  \item{\code{gg_vimp}}{Variable importance ranking}
  \item{\code{gg_rfsrc}}{Predicted vs. observed values}
  \item{\code{gg_variable}}{Marginal dependence}
  \item{\code{gg_partial}}{Partial dependence (via \code{plot.variable})}
  \item{\code{gg_partial_rfsrc}}{Partial dependence (via \code{partial.rfsrc})}
  \item{\code{gg_partialpro}}{Partial dependence (via \code{varPro})}
  \item{\code{gg_roc}}{ROC curve}
  \item{\code{gg_survival}}{Survival / cumulative hazard curves}
  \item{\code{gg_brier}}{Time-resolved Brier score and CRPS}
}
}
\examples{
\donttest{
library(ggplot2)
set.seed(42)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                              ntree = 50, importance = TRUE,
                              tree.err = TRUE)
autoplot(gg_error(rf))
autoplot(gg_vimp(rf))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{gg_varpro}
\alias{gg_varpro}
\title{Variable importance data from a varPro model}
\usage{
gg_varpro(
  object,
  local.std = TRUE,
  cutoff = 0.79,
  faithful = FALSE,
  conditional = FALSE,
  nvar = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted \code{varpro} object (required).}

\item{local.std}{Logical; default \code{TRUE}.  When \code{TRUE} the
per-tree importances are put on the z-scale before the box statistics
are computed.  Set it \code{FALSE} to keep the raw importance scale,
which is what \code{type = "raw"} in \code{plot.gg_varpro} needs.}

\item{cutoff}{Numeric; the z-score above which a variable is treated as
selected.  Default \code{0.79}.  A variable with aggregate z above the
cutoff is flagged \code{selected = TRUE} in \code{$imp}.}

\item{faithful}{Logical; default \code{FALSE}.  When \code{TRUE},
\code{$imp.tree} is kept so \code{plot.gg_varpro} can scatter the
per-tree points over the box.}

\item{conditional}{Logical; default \code{FALSE}.  When \code{TRUE}, and
only for a classification forest, the \code{$conditional.z} matrix is
extracted and stored as \code{$conditional}.}

\item{nvar}{Integer; keep only the top \code{nvar} variables, ranked by
median per-tree z, after the cutoff filter has been applied.
\code{NULL} keeps all of them.}

\item{...}{Additional arguments passed to \code{varPro::importance()}.}
}
\value{
A named list of class \code{"gg_varpro"} with elements:
\describe{
\item{\code{$imp}}{Summary data frame: \code{variable} (factor whose
levels run least- to most-important by median per-tree z, so the
most-important variable sits at the top of the plot after
\code{coord_flip}), \code{z} (aggregate
z-score from \code{importance()}), \code{selected} (logical,
\code{z > cutoff}).}
\item{\code{$imp.tree}}{\code{NULL} when \code{faithful = FALSE};
otherwise an ntree x p matrix of per-tree importance values.}
\item{\code{$stats}}{Per-variable summary: \code{variable},
\code{median}, \code{q05}, \code{q15}, \code{q85}, \code{q95}
(on z-scale when \code{local.std = TRUE}, raw when \code{FALSE}),
plus \code{mean} (raw importance mean, always stored).}
\item{\code{$conditional}}{\code{NULL} when \code{conditional = FALSE};
otherwise a data frame with columns \code{variable}, \code{class},
\code{z} (one row per variable x class combination).}
}
A \code{"provenance"} attribute carries \code{family}, \code{local.std},
\code{cutoff}, \code{faithful}, \code{conditional}, \code{xvar.names},
and \code{n}.
}
\description{
Pulls the per-tree importance scores out of a fitted \code{varpro} object
and summarises them into a data structure the plot method can draw as a
boxplot.  The box hinges are the 15th and 85th percentiles and the
whiskers run to the 5th and 95th -- not the usual Tukey 1.5 IQR whiskers.
For a classification forest you can also keep the class-conditional
importances.
}
\section{What varpro is doing}{

Permutation importance asks "what happens to OOB accuracy when I scramble
this variable?" That works, but it leans on artificial data (the
permuted column) and the answer can be unstable when variables are
correlated. The varpro framework (Lu and Ishwaran, 2024) replaces
permutation with \emph{release rules}. The forest is grown with guided
splitting; from a subset of trees varpro samples a collection of
decision-rule branches; for each variable it then compares the
response inside the rule's region to the response after the rule's
constraint on that variable is "released". The size of that change,
aggregated over many rules and trees, is the variable's importance.
No synthetic covariates, no permutation: the contrast is between two
real subsets of the data.

Because varpro builds importance from rules sampled over trees, every
tree contributes its own importance value for each variable. Those are
the per-tree scores we summarise here. With \code{local.std = TRUE}
(the default) the per-tree values are standardised by their column
standard deviation so the column mean equals the aggregate z-score
returned by \code{varPro::importance()}; that z-score is the canonical
"is this variable in or out?" statistic, and \code{cutoff = 0.79} is
varpro's default selection threshold.

For a classification forest, varpro also returns a class-conditional
z table: the same importance computed restricting attention to rules
relevant to each class. \code{conditional = TRUE} keeps that table so
the plot method can show which variables matter for which class
rather than only in aggregate.
}

\section{What's in the output}{

\code{$imp} is the one-row-per-variable summary: aggregate z from
\code{varPro::importance()}, plus a \code{selected} flag for
\code{z > cutoff}. \code{$stats} holds the box quantiles
(5/15/50/85/95 percentiles, plus the raw mean) computed from the
per-tree matrix; these are what the boxplot draws. \code{$imp.tree}
is the per-tree matrix itself, kept only when \code{faithful = TRUE}
so the plot method can scatter individual tree values over the box.
\code{$conditional} is the tidy class x variable z table, present
only when \code{conditional = TRUE} and the family is
classification.
}

\section{What you use this for}{

\itemize{
\item rank candidate variables by importance and pick a working set
above varpro's z cutoff;
\item see, via the boxplot's spread and the per-tree points
(\code{faithful = TRUE}), how stable each variable's importance
is across trees: a high median with a wide box is a different
story from a high median with a tight box;
\item for a classification forest, ask which variables drive which
class (\code{conditional = TRUE}) rather than just which
variables drive the model overall.
}
The z-score is a standardised ranking statistic, not a p-value or a
probability. Two variables with the same z are "similarly important
by this method", not "equally likely to be true signal". For a
data-driven cutoff rather than the 0.79 default, see
\code{varPro::cv.varpro}.
}

\examples{
\donttest{
set.seed(42)
vp <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
gg <- gg_varpro(vp)
print(gg)
plot(gg)
}

}
\references{
Lu, M. and Ishwaran, H. (2024). Model-independent variable selection
via the rule-based variable priority framework. \emph{arXiv preprint}
\href{https://arxiv.org/abs/2409.09003}{arXiv:2409.09003}.
}
\seealso{
\code{\link{plot.gg_varpro}}, \code{\link{gg_vimp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantile_pts.R
\name{quantile_pts}
\alias{quantile_pts}
\title{Quantile-based cut points for coplots}
\usage{
quantile_pts(object, groups, intervals = FALSE)
}
\arguments{
\item{object}{Numeric vector of predictor values.}

\item{groups}{Number of quantile points (or intervals) to compute.}

\item{intervals}{Logical indicating whether to return interval boundaries
suitable for \code{cut()} (length \code{groups + 1}) or the interior
quantile points (length \code{groups}).}
}
\value{
Numeric vector of quantile points. When \code{intervals = TRUE}
the result is strictly increasing and can be supplied to \code{cut()} to
produce \code{groups} balanced strata.
}
\description{
This helper wraps \code{\link[stats]{quantile}} to create well-spaced
cut points for conditioning plots. When \code{intervals = TRUE} the lower
boundary is nudged down so that \code{cut()} treats the minimum value as a
valid observation.

The output can be passed directly into the breaks argument of the
\code{cut} function for creating groups for coplots.
}
\examples{
data(Boston, package = "MASS")
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston)

# To create 6 intervals, we want 7 points.
# quantile_pts will find balanced intervals
rm_pts <- quantile_pts(rfsrc_boston$xvar$rm, groups = 6, intervals = TRUE)

# Use cut to create the intervals
rm_grp <- cut(rfsrc_boston$xvar$rm, breaks = rm_pts)

summary(rm_grp)

}
\seealso{
\code{cut}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{ggRandomForests-package}
\alias{ggRandomForests-package}
\title{ggRandomForests: Visually Exploring Random Forests}
\description{
\code{ggRandomForests} is a utility package for
\code{randomForestSRC} (Ishwaran and Kogalur) for survival,
regression and classification forests and uses the \code{ggplot2}
(Wickham 2009) package for plotting results. \code{ggRandomForests} is
structured to extract data objects from the random forest and provides S3
functions for printing and plotting these objects.
Requires \code{randomForestSRC} >= 3.4.0.

The \code{randomForestSRC} package provides a unified treatment of
Breiman's (2001) random forests for a variety of data settings. Regression
and classification forests are grown when the response is numeric or
categorical (factor) while survival and competing risk forests
(Ishwaran et al. 2008, 2012) are grown for right-censored survival data.

Many of the figures created by the \code{ggRandomForests} package are also
available directly from within the \code{randomForestSRC} package. However,
\code{ggRandomForests} offers the following advantages:
\itemize{
\item Separation of data and figures: \code{ggRandomForest} contains
functions that operate on either the \code{\link[randomForestSRC]{rfsrc}}
forest object directly, or on the output from \code{randomForestSRC} post
processing functions (i.e. \code{plot.variable}) to generate intermediate
\code{ggRandomForests}
data objects. S3 functions are provide to further process these objects and
plot results using the \code{ggplot2} graphics package. Alternatively,
users can use these data objects for additional custom plotting or
analysis operations.

\item Each data object/figure is a single, self contained object. This
allows simple modification and manipulation of the data or \code{ggplot2}
objects to meet users specific needs and requirements.

\item The use of \code{ggplot2} for plotting. We chose to use the
\code{ggplot2} package for our figures to allow users flexibility in
modifying the figures to their liking. Each S3 plot function returns either
a single \code{ggplot2} object, or a \code{list} of \code{ggplot2} objects,
allowing users to use additional \code{ggplot2} functions or themes to
modify and customize the figures to their liking.
}

The \code{ggRandomForests} package contains the following data functions:
\itemize{
\item \code{\link{gg_rfsrc}}: randomForest[SRC] predictions.
\item \code{\link{gg_error}}: randomForest[SRC] convergence rate based on
the OOB error rate.
\item \code{\link{gg_roc}}: ROC curves for randomForest classification
models.
\item \code{\link{gg_vimp}}: Variable Importance ranking for variable
selection.
(Ishwaran et.al. 2010).
\item \code{\link{gg_variable}}: Marginal variable dependence.

\item \code{\link{gg_survival}}: Kaplan-Meier/Nelson-Aalen hazard analysis.
}

Each of these data functions has an associated S3 plot function that
returns \code{ggplot2} objects, either individually or as a list, which can
be further customized using standard \code{ggplot2} commands.
}
\references{
Breiman, L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. R News
7(2), 25--31.

Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random
survival forests. Ann. Appl. Statist. 2(3), 841--860.

Ishwaran, H., U. B. Kogalur, E. Z. Gorodeski, A. J. Minn, and M. S. Lauer
(2010). High-dimensional variable selection for survival data. J. Amer.
Statist. Assoc. 105, 205-217.

Ishwaran, H. (2007). Variable importance in binary regression trees and
forests. Electronic J. Statist., 1, 519-537.

Wickham, H. ggplot2: elegant graphics for data analysis. Springer New York,
2009.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot_methods.R
\name{autoplot.gg}
\alias{autoplot.gg}
\alias{autoplot.gg_error}
\alias{autoplot.gg_vimp}
\alias{autoplot.gg_rfsrc}
\alias{autoplot.gg_variable}
\alias{autoplot.gg_partial}
\alias{autoplot.gg_partial_rfsrc}
\alias{autoplot.gg_partialpro}
\alias{autoplot.gg_partial_varpro}
\alias{autoplot.gg_roc}
\alias{autoplot.gg_survival}
\alias{autoplot.gg_brier}
\alias{autoplot.gg_varpro}
\alias{autoplot.gg_udependent}
\alias{autoplot.gg_isopro}
\title{\code{autoplot} methods for \pkg{ggRandomForests} data objects}
\usage{
\method{autoplot}{gg_error}(object, ...)

\method{autoplot}{gg_vimp}(object, ...)

\method{autoplot}{gg_rfsrc}(object, ...)

\method{autoplot}{gg_variable}(object, ...)

\method{autoplot}{gg_partial}(object, ...)

\method{autoplot}{gg_partial_rfsrc}(object, ...)

\method{autoplot}{gg_partialpro}(object, ...)

\method{autoplot}{gg_partial_varpro}(object, ...)

\method{autoplot}{gg_roc}(object, ...)

\method{autoplot}{gg_survival}(object, ...)

\method{autoplot}{gg_brier}(object, ...)

\method{autoplot}{gg_varpro}(object, ...)

\method{autoplot}{gg_udependent}(object, ...)

\method{autoplot}{gg_isopro}(object, ...)
}
\arguments{
\item{object}{A \code{gg_*} data object (see Details).}

\item{...}{Additional arguments passed to the underlying
\code{plot.gg_*()} method.}
}
\value{
A \code{ggplot} object.
}
\description{
These let you call \code{ggplot2::autoplot()} on any \code{gg_*} object
\pkg{ggRandomForests} returns. Each is a thin wrapper around the matching
\code{plot.gg_*()} S3 method, and \code{...} passes straight through, so
every argument those plot methods take is still available here.
}
\details{
The following \code{gg_*} classes are supported:
\describe{
\item{\code{gg_error}}{OOB error vs. number of trees}
\item{\code{gg_vimp}}{Variable importance ranking}
\item{\code{gg_rfsrc}}{Predicted vs. observed values}
\item{\code{gg_variable}}{Marginal dependence}
\item{\code{gg_partial}}{Partial dependence (via \code{plot.variable})}
\item{\code{gg_partial_rfsrc}}{Partial dependence (via \code{partial.rfsrc})}
\item{\code{gg_partial_varpro}}{Partial dependence (via \code{varPro})}
\item{\code{gg_partialpro}}{Partial dependence via \code{varPro} (deprecated alias)}
\item{\code{gg_varpro}}{Variable importance from \code{varPro}}
\item{\code{gg_roc}}{ROC curve}
\item{\code{gg_survival}}{Survival / cumulative hazard curves}
\item{\code{gg_brier}}{Time-resolved Brier score and CRPS}
}
}
\examples{
\donttest{
library(ggplot2)
set.seed(42)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = na.omit(airquality),
                              ntree = 50, importance = TRUE,
                              tree.err = TRUE)
autoplot(gg_error(rf))
autoplot(gg_vimp(rf))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_beta_varpro.R
\name{plot.gg_beta_varpro}
\alias{plot.gg_beta_varpro}
\title{Plot a \code{gg_beta_varpro} object}
\usage{
\method{plot}{gg_beta_varpro}(x, ...)
}
\arguments{
\item{x}{A \code{gg_beta_varpro} object from \code{\link[=gg_beta_varpro]{gg_beta_varpro()}}.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} object.
}
\description{
Horizontal bar chart of the mean absolute coefficient
\eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)} per variable, sorted
descending so
the eye lands on the top variable first. Bars filled blue when above the
selection cutoff, grey otherwise. Dashed red line marks the cutoff.
}
\section{Reading the chart}{

Each bar is the average magnitude of a per-rule lasso coefficient for
that variable. \strong{The numeric scale carries the predictor's units.}
If "age" is in years and "creatinine" is in mg/dL, a longer bar for
age does not mean age is "more important" in any unit-free sense.
Comparisons across data sets or across variables with very different
units require keeping the units context in mind. Within one data set,
bars are comparable up to that unit caveat.

Variables above the cutoff are coloured blue and flagged \code{selected};
variables below are grey. Lasso shrinkage can drive a rule's
\eqn{\hat{\beta}}{beta hat} to
exactly zero; those rules are kept in the average, so a variable
with many shrunk-to-zero rules will sit lower in the ranking than
one whose released coefficients are consistently non-zero.

For a classification fit, variables are sorted by
\verb{mean(|sum-of-class-beta|)} descending and that ordering is shared
across every facet, so rows line up between classes for visual
comparison. Each facet has its own cutoff line.
}

\section{What this tells you}{

Use the bar chart as a selection ranking, not as an effect-size axis.
Pair it with \code{\link[=gg_varpro]{gg_varpro()}} to see where split-strength importance and
local lasso-beta importance agree or disagree; disagreement is often the
interesting signal.
}

\examples{
\donttest{
if (requireNamespace("varPro", quietly = TRUE)) {
  set.seed(1)
  v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
  plot(gg_beta_varpro(v))
}
}

}
\seealso{
\code{\link[=gg_beta_varpro]{gg_beta_varpro()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_brier.R
\name{plot.gg_brier}
\alias{plot.gg_brier}
\title{Plot a \code{\link{gg_brier}} object}
\usage{
\method{plot}{gg_brier}(x, type = c("brier", "crps"), envelope = FALSE, ...)
}
\arguments{
\item{x}{A \code{\link{gg_brier}} object.}

\item{type}{Which series to plot: \code{"brier"} (default) or
\code{"crps"}.}

\item{envelope}{Logical. When \code{TRUE}, overlays a ribbon spanning
the 15th-85th percentile of per-subject Brier (or running CRPS)
contributions at each time, around the overall line. When \code{FALSE}
(default), draws the overall series only.}

\item{...}{Extra arguments forwarded to \code{geom_line()}.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot the time-resolved Brier score (default) or running CRPS for a
survival forest, optionally overlaid with a 15-85 percent per-subject
envelope.
}
\examples{
\dontrun{
data(pbc, package = "randomForestSRC")
rf <- randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
                             nsplit = 10)
gg_dta <- gg_brier(rf)
plot(gg_dta)
plot(gg_dta, type = "crps")
plot(gg_dta, envelope = TRUE)   # adds 15-85\% envelope
}

}
\seealso{
\code{\link{gg_brier}},
  \code{\link[randomForestSRC]{get.brier.survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_roc.R
\name{calc_auc}
\alias{calc_auc}
\alias{calc_auc.gg_roc}
\title{Area Under the ROC Curve calculator}
\usage{
calc_auc(x)
}
\arguments{
\item{x}{\code{\link{gg_roc}} object}
}
\value{
AUC. 50\% is random guessing, higher is better.
}
\description{
Area Under the ROC Curve calculator
}
\details{
calc_auc uses the trapezoidal rule to calculate the area under
the ROC curve.

 This is a helper function for the \code{\link{gg_roc}} functions.
}
\examples{
##
## Taken from the gg_roc example
rfsrc_iris <- rfsrc(Species ~ ., data = iris)

gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)

calc_auc(gg_dta)

gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)

calc_auc(gg_dta)

## randomForest tests
rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)

calc_auc(gg_dta)

}
\seealso{
\code{\link{calc_roc}} \code{\link{gg_roc}}

\code{\link{plot.gg_roc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partialpro.R
\name{gg_partialpro}
\alias{gg_partialpro}
\title{Split varpro partial dependence data into continuous or categorical datasets}
\usage{
gg_partialpro(part_dta, nvars = NULL, cat_limit = 10, model = NULL)
}
\arguments{
\item{part_dta}{partial plot data from \code{varpro::partialpro}.  Each
element of the list must contain fields \code{xvirtual}, \code{xorg},
\code{yhat.par}, \code{yhat.nonpar}, and \code{yhat.causal}.}

\item{nvars}{how many variables (list elements) to process.  Defaults to
all variables in \code{part_dta}.}

\item{cat_limit}{Variables with \code{length(xvirtual)} \eqn{\le}
\code{cat_limit} are treated as categorical.  Default \code{10}.}

\item{model}{a label applied to all rows.  Useful when combining results
from multiple models in a single figure.}
}
\value{
A named list with two elements:
  \describe{
    \item{continuous}{data.frame with columns \code{variable},
      \code{parametric}, \code{nonparametric}, \code{causal}, \code{name}
      (and optionally \code{model})}
    \item{categorical}{data.frame with the same columns but one row per
      observation per category level}
  }
}
\description{
Takes the list returned by \code{varpro::partialpro} and separates variables
into two data frames: one for continuous predictors (parametric, non-
parametric, and causal effect curves) and one for categorical predictors
(one row per observation per category level).
}
\details{
The split is governed by \code{cat_limit}: a variable is treated as
continuous when \code{length(xvirtual) > cat_limit}; otherwise it is
treated as categorical and the per-category rows are stacked.
}
\examples{
## Construct mock varpro partialpro output:
##   - "age": a continuous predictor (xvirtual has > 10 points)
##   - "sex": a categorical predictor (xvirtual has 2 points)
set.seed(42)
n_obs <- 30   # number of observations (rows in yhat matrices)
n_pts <- 15   # number of evaluation points for continuous variables

mock_data <- list(
  age = list(
    # xvirtual: evaluation grid for the marginal effect curve
    xvirtual   = seq(30, 80, length.out = n_pts),
    # xorg: original observed values (used only for categorical detection)
    xorg       = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
    # yhat matrices: n_obs rows x n_pts columns (predictions at each grid pt)
    yhat.par   = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
  ),
  sex = list(
    # Two categories: the xvirtual grid has only 2 points
    xvirtual   = c(0, 1),
    xorg       = sample(c(0, 1), n_obs, replace = TRUE),
    # Two-column yhat matrices (one column per category)
    yhat.par   = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
  )
)

result <- gg_partialpro(mock_data)

## Continuous result: one row per evaluation grid point
head(result$continuous)

## Categorical result: n_obs rows per category level
head(result$categorical)

}
\seealso{
\code{\link{gg_partial}} \code{\link{varpro_feature_names}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{ggRandomForests-package}
\alias{ggRandomForests-package}
\title{ggRandomForests: Visually Exploring Random Forests}
\description{
\code{ggRandomForests} is a utility package for
\code{randomForestSRC} (Ishwaran and Kogalur) for survival,
regression and classification forests and uses the \code{ggplot2}
(Wickham 2009) package for plotting results. \code{ggRandomForests} is
structured to extract data objects from the random forest and provides S3
functions for printing and plotting these objects.
Requires \code{randomForestSRC} >= 3.4.0.

The \code{randomForestSRC} package provides a unified treatment of
Breiman's (2001) random forests for a variety of data settings. Regression
and classification forests are grown when the response is numeric or
categorical (factor) while survival and competing risk forests
(Ishwaran et al. 2008, 2012) are grown for right-censored survival data.

Many of the figures created by the \code{ggRandomForests} package are also
available directly from within the \code{randomForestSRC} package. However,
\code{ggRandomForests} offers the following advantages:
\itemize{
\item Separation of data and figures: \code{ggRandomForest} contains
functions that operate on either the \code{\link[randomForestSRC]{rfsrc}}
forest object directly, or on the output from \code{randomForestSRC} post
processing functions (i.e. \code{plot.variable}) to generate intermediate
\code{ggRandomForests}
data objects. S3 functions are provide to further process these objects and
plot results using the \code{ggplot2} graphics package. Alternatively,
users can use these data objects for additional custom plotting or
analysis operations.

\item Each data object/figure is a single, self contained object. This
allows simple modification and manipulation of the data or \code{ggplot2}
objects to meet users specific needs and requirements.

\item The use of \code{ggplot2} for plotting. We chose to use the
\code{ggplot2} package for our figures to allow users flexibility in
modifying the figures to their liking. Each S3 plot function returns either
a single \code{ggplot2} object, or a \code{list} of \code{ggplot2} objects,
allowing users to use additional \code{ggplot2} functions or themes to
modify and customize the figures to their liking.
}

The \code{ggRandomForests} package contains the following data functions:
\itemize{
\item \code{\link{gg_rfsrc}}: randomForestSRC predictions.
\item \code{\link{gg_error}}: randomForestSRC convergence rate based on
the OOB error rate.
\item \code{\link{gg_roc}}: ROC curves for randomForest classification
models.
\item \code{\link{gg_vimp}}: Variable Importance ranking for variable
selection.
(Ishwaran et.al. 2010).
\item \code{\link{gg_variable}}: Marginal variable dependence.

\item \code{\link{gg_survival}}: Kaplan-Meier/Nelson-Aalen hazard analysis.
}

Each of these data functions has an associated S3 plot function that
returns \code{ggplot2} objects, either individually or as a list, which can
be further customized using standard \code{ggplot2} commands.
}
\references{
Breiman, L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R. R News
7(2), 25--31.

Ishwaran H., Kogalur U.B., Blackstone E.H. and Lauer M.S. (2008). Random
survival forests. Ann. Appl. Statist. 2(3), 841--860.

Ishwaran, H., U. B. Kogalur, E. Z. Gorodeski, A. J. Minn, and M. S. Lauer
(2010). High-dimensional variable selection for survival data. J. Amer.
Statist. Assoc. 105, 205-217.

Ishwaran, H. (2007). Variable importance in binary regression trees and
forests. Electronic J. Statist., 1, 519-537.

Wickham, H. ggplot2: elegant graphics for data analysis. Springer New York,
2009.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_survival.R
\name{gg_survival}
\alias{gg_survival}
\alias{gg_survival.rfsrc}
\alias{gg_survival.default}
\title{Nonparametric survival estimates.}
\usage{
gg_survival(
  object = NULL,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)

\method{gg_survival}{rfsrc}(
  object,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)

\method{gg_survival}{default}(
  object = NULL,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)
}
\arguments{
\item{object}{For the \code{rfsrc} method: a fitted
\code{\link[randomForestSRC]{rfsrc}} survival forest.  For the default
method: pass \code{NULL} (or omit) and supply \code{interval},
\code{censor}, and \code{data} instead.}

\item{interval}{Character; name of the time-to-event column in \code{data}
(default method only).}

\item{censor}{Character; name of the event-indicator column in \code{data}
(1 = event, 0 = censored; default method only).}

\item{by}{Optional character; name of a grouping column for stratified
estimates.  For the \code{rfsrc} method, \code{by} must be a column in
\code{object$xvar}.}

\item{data}{A \code{data.frame} containing survival data (default method
only).}

\item{type}{One of \code{"kaplan"} (Kaplan-Meier, default) or
\code{"nelson"} (Nelson-Aalen cumulative hazard).  Default method only.}

\item{...}{Additional arguments passed to \code{\link{kaplan}} or
\code{\link{nelson}}.}
}
\value{
A \code{gg_survival} \code{data.frame} with columns \code{time},
  \code{surv}, \code{cum_haz}, \code{lower}, \code{upper}, \code{n.risk},
  and optionally \code{groups} when \code{by} is supplied.
}
\description{
Nonparametric survival estimates.
}
\details{
\code{gg_survival} is an S3 generic for generating nonparametric
survival estimates.  It dispatches on the class of its first argument:

\describe{
  \item{\code{rfsrc}}{Extracts the response data from the fitted forest and
    delegates to \code{\link{kaplan}}.  Use the \code{by} argument to
    stratify on a predictor stored in the model's \code{xvar} slot.}
  \item{default}{Accepts raw survival data columns via the \code{interval},
    \code{censor}, \code{by}, and \code{data} arguments, delegating to
    either \code{\link{kaplan}} (default) or \code{\link{nelson}}.}
}
}
\examples{
## -------- pbc data (default method — raw data columns)
data(pbc, package = "randomForestSRC")
pbc$time <- pbc$days / 364.25

gg_dta <- gg_survival(interval = "time", censor = "status", data = pbc)
plot(gg_dta, error = "none")

# Stratified
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment"
)
plot(gg_dta)

}
\seealso{
\code{\link{kaplan}} \code{\link{nelson}}

\code{\link{plot.gg_survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{.build_varpro_imp_dfs}
\alias{.build_varpro_imp_dfs}
\title{Build the tidy importance data structures from importance() output}
\usage{
.build_varpro_imp_dfs(
  imp_out,
  imp_tree_mat,
  family,
  cutoff,
  nvar,
  faithful,
  local.std,
  conditional
)
}
\description{
Build the tidy importance data structures from importance() output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partial_rfsrc.R
\name{gg_partial_rfsrc}
\alias{gg_partial_rfsrc}
\title{Partial dependence data from an rfsrc model}
\usage{
gg_partial_rfsrc(
  rf_model,
  xvar.names = NULL,
  xvar2.name = NULL,
  newx = NULL,
  partial.time = NULL,
  partial.type = c("surv", "chf", "mort"),
  cat_limit = 10,
  n_eval = 25
)
}
\arguments{
\item{rf_model}{A fitted \code{\link[randomForestSRC]{rfsrc}} object.}

\item{xvar.names}{Character vector of predictor names for which partial
dependence should be computed. Must be a subset of \code{rf_model$xvar.names}.}

\item{xvar2.name}{Optional single character name of a grouping variable in
\code{newx}. When supplied, partial dependence is computed separately for
each unique level of this variable and a \code{grp} column is appended.}

\item{newx}{Optional \code{data.frame} of predictor values to evaluate
partial effects at. Defaults to the training data stored in
\code{rf_model$xvar}. All column names must match \code{rf_model$xvar.names}.}

\item{partial.time}{Numeric vector of desired time points for survival
forests (ignored for regression/classification).  Values are automatically
snapped to the nearest entry in \code{rf_model$time.interest} — see the
\strong{Survival forests} section below.  When \code{NULL} (default),
three quartile points of \code{time.interest} are used.}

\item{partial.type}{Character; type of predicted value for survival
forests, passed through to \code{\link[randomForestSRC]{partial.rfsrc}}.
One of \code{"surv"} (default), \code{"chf"}, or \code{"mort"}. Ignored
for non-survival forests. \code{partial.rfsrc()} requires a non-\code{NULL}
value for survival families; supplying it here avoids a cryptic
\dQuote{argument is of length zero} error from the underlying C code.}

\item{cat_limit}{Variables with fewer than \code{cat_limit} unique values in
\code{newx} are treated as categorical; all others are continuous.
Defaults to 10.}

\item{n_eval}{Number of evaluation points for continuous variables. Instead
of passing all observed values (which can be slow, especially for survival
forests), continuous predictors are evaluated on a quantile grid of this
many points. Categorical variables always use all unique levels.
Defaults to 25.}
}
\value{
A named list with two elements:
  \describe{
    \item{continuous}{A \code{data.frame} with columns \code{x} (numeric),
      \code{yhat}, \code{name} (variable name), and optionally \code{grp}
      (the level of \code{xvar2.name}) and \code{time} (survival forests
      only) for all continuous predictors.}
    \item{categorical}{A \code{data.frame} with the same columns but
      \code{x} kept as character, for low-cardinality predictors.}
  }
}
\description{
Computes partial dependence for one or more predictors by calling
\code{\link[randomForestSRC]{partial.rfsrc}} internally, then splits the
results into separate data frames for continuous and categorical variables.
Unlike \code{\link{gg_partial}}, no separate \code{plot.variable} call is
required — supply the fitted \code{rfsrc} object directly.
}
\section{Survival forests and \code{partial.time}}{

\code{\link[randomForestSRC]{partial.rfsrc}} requires that every value in
\code{partial.time} be an exact member of the model's \code{time.interest}
vector (the unique observed event times stored in the fitted object).
Passing arbitrary time values — even plausible ones such as \code{c(1, 3)}
for a study measured in years — causes a C-level prediction error inside
\code{partial.rfsrc}.

\code{gg_partial_rfsrc} handles this automatically: every element of
\code{partial.time} is silently snapped to its nearest \code{time.interest}
value before the call is made.  To target a specific follow-up horizon,
find the closest grid point yourself and pass it explicitly:

\preformatted{
ti  <- rf_model$time.interest
t1  <- ti[which.min(abs(ti - 1))]   # nearest to 1 year
pd  <- gg_partial_rfsrc(rf_model, xvar.names = "x", partial.time = t1)
}
}

\section{Logical predictor columns}{

\code{\link[randomForestSRC]{partial.rfsrc}} does not handle
\code{logical} predictor columns correctly in survival forests
(randomForestSRC <= 3.5.1).  If your training data contains binary 0/1
columns, convert them to \code{\link{factor}} rather than \code{logical}
before fitting the model.
}

\examples{
## ------------------------------------------------------------
##
## regression
##
## ------------------------------------------------------------

airq.obj <- rfsrc(Ozone ~ ., data = airquality)

## partial effect for wind
prt_dta <- gg_partial_rfsrc(airq.obj,
                       xvar.names = c("Wind"))

}
\seealso{
\code{\link{gg_partial}}, \code{\link[randomForestSRC]{partial.rfsrc}},
  \code{\link[randomForestSRC]{get.partial.plot.data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_roc.R
\name{gg_roc.rfsrc}
\alias{gg_roc.rfsrc}
\alias{gg_roc}
\alias{gg_roc.randomForest}
\title{ROC (Receiver Operating Characteristic) curve data from a classification forest.}
\usage{
\method{gg_roc}{rfsrc}(object, which_outcome, oob = TRUE, ...)
}
\arguments{
\item{object}{A classification \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object. Only forests with
\code{family == "class"} (rfsrc) or \code{type == "classification"}
(randomForest) are supported.}

\item{which_outcome}{Integer index or character name of the class for which
the ROC curve is computed. For binary forests this is typically \code{1}
or \code{2}; for multi-class forests any valid class index. Use
\code{which_outcome = 0} to obtain the overall (averaged) ROC.}

\item{oob}{Logical; if \code{TRUE} (default) use out-of-bag predicted
probabilities for the curve. Set to \code{FALSE} to use full in-bag
predictions.}

\item{...}{Extra arguments (currently unused).}
}
\value{
A \code{gg_roc} \code{data.frame} with one row per unique prediction
  threshold and columns:
  \describe{
    \item{sens}{Sensitivity (true positive rate) at each threshold.}
    \item{spec}{Specificity (true negative rate) at each threshold.}
    \item{yvar}{The observed class label for each observation.}
  }
  Pass to \code{\link{calc_auc}} for the area under the curve.
}
\description{
Computes sensitivity (true positive rate) and specificity (1 - false positive
rate) across all prediction thresholds for one class of a classification
\code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- rfsrc(Species ~ ., data = iris)

# ROC for setosa
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(gg_dta)

## -------- iris data
rf_iris <- randomForest::randomForest(Species ~ ., data = iris)

# ROC for setosa
gg_dta <- gg_roc(rf_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor
gg_dta <- gg_roc(rf_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica
gg_dta <- gg_roc(rf_iris, which_outcome = 3)
plot(gg_dta)

}
\seealso{
\code{\link{plot.gg_roc}}, \code{\link{calc_roc}},
  \code{\link{calc_auc}},
  \code{\link[randomForestSRC]{rfsrc}},
  \code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_brier.R
\name{plot.gg_brier}
\alias{plot.gg_brier}
\title{Plot a \code{\link{gg_brier}} object}
\usage{
\method{plot}{gg_brier}(x, type = c("brier", "crps"), envelope = FALSE, ...)
}
\arguments{
\item{x}{A \code{\link{gg_brier}} object.}

\item{type}{Which series to plot: \code{"brier"} (default) or
\code{"crps"}.}

\item{envelope}{Logical. When \code{TRUE}, overlays a ribbon spanning
the 15th-85th percentile of per-subject Brier (or running CRPS)
contributions at each time, around the overall line. When \code{FALSE}
(default), draws the overall series only.}

\item{...}{Extra arguments forwarded to \code{geom_line()}.}
}
\value{
A \code{ggplot} object.
}
\description{
Draws the time-resolved Brier score or the running CRPS from a
\code{\link{gg_brier}} object.  The curve moves across the event-time
grid on the x-axis; lower values mean the forest's predicted survival
probabilities are closer to what actually happened.  Think of
\code{0} as "perfect" and roughly \code{0.25} as "uninformative" -- a
forest that predicts \code{0.5} for every subject regardless of
prognosis would sit near that ceiling.
}
\details{
Set \code{envelope = TRUE} to add a ribbon around the overall curve
spanning the 15th to 85th percentile of the per-subject Brier
contributions at each time.  The ribbon shows how heterogeneous the
scoring is across subjects: a narrow ribbon means most subjects are
predicted equally well (or equally poorly); a wide ribbon means a
minority of subjects are driving the average.
}
\examples{
\donttest{
library(survival)   # Surv() must be on the search path for rfsrc()
data(pbc, package = "randomForestSRC")
rf <- randomForestSRC::rfsrc(Surv(days, status) ~ ., data = pbc,
                             nsplit = 10)
gg_dta <- gg_brier(rf)
plot(gg_dta)
plot(gg_dta, type = "crps")
plot(gg_dta, envelope = TRUE)   # adds 15-85\% envelope
}

}
\seealso{
\code{\link{gg_brier}},
\code{\link[randomForestSRC]{get.brier.survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_vimp.R
\name{plot.gg_vimp}
\alias{plot.gg_vimp}
\title{Plot a \code{\link{gg_vimp}} object, extracted variable importance of a
\code{\link[randomForestSRC]{rfsrc}} object}
\usage{
\method{plot}{gg_vimp}(x, relative, lbls, ...)
}
\arguments{
\item{x}{\code{\link{gg_vimp}} object created from a
\code{\link[randomForestSRC]{rfsrc}} object}

\item{relative}{should we plot vimp or relative vimp. Defaults to vimp.}

\item{lbls}{A vector of alternative variable labels. Item names should be
the same as the variable names.}

\item{...}{optional arguments passed to gg_vimp if necessary}
}
\value{
\code{ggplot} object
}
\description{
Plot a \code{\link{gg_vimp}} object, extracted variable importance of a
\code{\link[randomForestSRC]{rfsrc}} object
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- rfsrc(Species ~ ., data = iris)
gg_dta <- gg_vimp(rfsrc_iris)
plot(gg_dta)

## ------------------------------------------------------------
## regression example
## ------------------------------------------------------------
## -------- air quality data
rfsrc_airq <- rfsrc(Ozone ~ ., airquality)
gg_dta <- gg_vimp(rfsrc_airq)
plot(gg_dta)


}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
R, Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_vimp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_udependent.R
\name{gg_udependent}
\alias{gg_udependent}
\title{Variable dependency graph from a uvarpro model}
\usage{
gg_udependent(
  object,
  threshold = 0.25,
  q.signal = 0.75,
  directed = TRUE,
  min.degree = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted \code{uvarpro} object (required).}

\item{threshold}{Numeric; the positive dependency threshold passed on to
\code{sdependent()}.  An edge \eqn{i \to j} is drawn when
\code{I[i, j] >= threshold}.  Default \code{0.25}.}

\item{q.signal}{Quantile threshold (0--1) for picking out the signal
variables; passed on to \code{sdependent()}.  Default \code{0.75}.}

\item{directed}{Logical; \code{TRUE} (default) builds a directed igraph.}

\item{min.degree}{Integer or \code{NULL}.  When set, only nodes with
degree \eqn{\ge} \code{min.degree} are kept in \code{$nodes},
\code{$edges}, and \code{$graph}.}

\item{...}{Additional arguments forwarded to \code{varPro::sdependent()}.}
}
\value{
A named list of class \code{"gg_udependent"} with elements:
\describe{
\item{\code{$edges}}{Data frame: \code{variable_from}, \code{variable_to},
\code{weight} (raw cross-importance value).}
\item{\code{$nodes}}{Data frame: \code{variable} (factor, levels by
descending degree), \code{degree} (integer; out-degree when
\code{directed = TRUE}, total degree when \code{directed = FALSE}),
\code{selected} (logical, \code{TRUE} if in \code{sdependent}'s
signal set).}
\item{\code{$graph}}{igraph object. \code{NULL} if no dependencies
detected.}
}
A \code{"provenance"} attribute carries \code{threshold}, \code{q.signal},
\code{directed}, \code{min.degree}, \code{xvar.names}, and \code{n}.
}
\description{
A \code{uvarpro} fit records how strongly each variable depends on the
others.  This function pulls those cross-variable dependency scores from
the fit with \code{\link[varPro]{get.beta.entropy}} and
\code{\link[varPro]{sdependent}}, and returns them as a tidy list that
\code{plot.gg_udependent} can draw as a network.
}
\section{What cross-variable dependency is doing}{

UVarPro (Zhou, Lu and Ishwaran, 2026) extends the varpro framework to
the unsupervised setting: grow a forest without a response, then use
the same region-release contrasts varpro uses for supervised
importance to ask, "which variables explain the structure in the
data?" The lasso-driven variant frames each region-release contrast
as a classification task (does an observation belong to the region
or to its release?) and fits a lasso logistic regression with the
other variables as predictors. The coefficient on variable \eqn{j}
in the model for variable \eqn{i}'s region-release contrast is the
entry \eqn{I[i, j]} of the matrix \code{varPro::get.beta.entropy()}
returns.

Read that entry as "how much does knowing \eqn{j} help separate
\eqn{i}'s region from its release". A large \eqn{I[i, j]} says
\eqn{j} carries information about the structure varpro picked up in
\eqn{i}. \code{varPro::sdependent} thresholds that matrix at a
user-chosen cut and returns the set of "signal" variables: the
nodes with high enough out-degree to be worth keeping. We pass the
threshold through to \code{sdependent} and use the same matrix to
weight the edges of the resulting graph.

The graph is directed by default because \eqn{I[i, j]} and
\eqn{I[j, i]} are separate lasso coefficients and need not agree;
setting \code{directed = FALSE} collapses each pair by taking the
larger of the two, which is appropriate when you only want to see
that two variables are dependent, not which way the dependency
reads.
}

\section{What's in the output}{

\code{$edges} has one row per surviving edge with the raw weight
\code{I[i, j]} (or, for undirected graphs, the max of the two
directions). \code{$nodes} has one row per surviving variable with
its degree (out-degree for directed, total degree for undirected)
and a \code{selected} flag for membership in the \code{sdependent}
signal set. \code{$graph} is the same information packaged as an
\code{igraph} object, with \code{weight}, \code{degree}, and
\code{selected} attached so \code{plot.gg_udependent} can render it
without recomputing anything.
}

\section{What you use this for}{

\itemize{
\item screen a wide unsupervised dataset for the small set of
variables UVarPro thinks are carrying the signal: the nodes
with high degree, or those flagged \code{selected = TRUE};
\item spot clusters of mutually dependent variables (hubs and the
spokes around them) that may be measuring the same underlying
construct;
\item compare two datasets, or two preprocessing pipelines, by
looking at how their dependency graphs change.
}
An edge in this graph is a statistical dependency in the unsupervised
decomposition of the data. It is not a causal arrow. A high
\eqn{I[i, j]} says \eqn{j} predicts \eqn{i}'s region membership,
not that \eqn{j} causes \eqn{i}.
}

\examples{
\donttest{
set.seed(42)
uv <- varPro::uvarpro(iris[, -5], ntree = 50)
gg <- gg_udependent(uv)
print(gg)
}

}
\references{
Zhou, L., Lu, M. and Ishwaran, H. (2026). Variable priority for
unsupervised variable selection. \emph{Pattern Recognition},
172:112727.
}
\seealso{
\code{\link{plot.gg_udependent}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_roc.R
\name{calc_roc.rfsrc}
\alias{calc_roc.rfsrc}
\alias{calc_roc.randomForest}
\alias{calc_roc}
\title{Receiver Operator Characteristic calculator}
\usage{
\method{calc_roc}{rfsrc}(object, dta, which_outcome = "all", oob = TRUE, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}},
\code{\link[randomForestSRC]{predict.rfsrc}}, or
\code{\link[randomForest]{randomForest}} classification object containing
predicted class probabilities.}

\item{dta}{A factor (or coercible to factor) of the true observed class
labels, one per observation. Typically \code{object$yvar} for rfsrc or
\code{object$y} for randomForest.}

\item{which_outcome}{Integer index of the class for which the ROC curve is
computed (e.g. \code{1} for the first class, \code{2} for the second).
Use \code{"all"} to request all classes (currently falls back to class 1
with a warning).}

\item{oob}{Logical; if \code{TRUE} (default for rfsrc) use OOB predicted
probabilities. Forced to \code{FALSE} for \code{randomForest} objects.}

\item{...}{Extra arguments passed to helper functions (currently unused).}
}
\value{
A \code{gg_roc} \code{data.frame} with columns \code{sens}
  (sensitivity), \code{spec} (specificity), and \code{pct} (the probability
  threshold), with one row per unique prediction value. Suitable for passing
  to \code{\link{calc_auc}} or \code{\link{plot.gg_roc}}.
}
\description{
Receiver Operator Characteristic calculator
}
\details{
For a randomForestSRC prediction and the actual
response value, calculate the specificity (1-False Positive Rate) and
sensitivity (True Positive Rate) of a predictor.

This is a helper function for the \code{\link{gg_roc}} functions, and
not intended for use by the end user.
}
\examples{
## Taken from the gg_roc example
rfsrc_iris <- rfsrc(Species ~ ., data = iris)

gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
  which_outcome = 1, oob = TRUE
)
gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
  which_outcome = 1, oob = FALSE
)

rf_iris <- randomForest(Species ~ ., data = iris)
gg_dta <- calc_roc(rf_iris, rf_iris$yvar,
  which_outcome = 1
)
gg_dta <- calc_roc(rf_iris, rf_iris$yvar,
  which_outcome = 2
)

}
\seealso{
\code{\link{calc_auc}} \code{\link{gg_roc}}

\code{\link{plot.gg_roc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_roc.R
\name{calc_auc}
\alias{calc_auc}
\alias{calc_auc.gg_roc}
\title{Area Under the ROC Curve calculator}
\usage{
calc_auc(x)
}
\arguments{
\item{x}{\code{\link{gg_roc}} object}
}
\value{
AUC. 50\\% is random guessing, higher is better.
}
\description{
Area Under the ROC Curve calculator
}
\details{
calc_auc uses the trapezoidal rule to calculate the area under
the ROC curve.

This is a helper function for the \code{\link{gg_roc}} functions.
}
\examples{
##
## Taken from the gg_roc example
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)

gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)

calc_auc(gg_dta)

gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)

calc_auc(gg_dta)

## randomForest tests
rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)

calc_auc(gg_dta)

}
\seealso{
\code{\link{calc_roc}} \code{\link{gg_roc}}

\code{\link{plot.gg_roc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_survival.R
\name{gg_survival}
\alias{gg_survival}
\alias{gg_survival.rfsrc}
\alias{gg_survival.default}
\title{Nonparametric survival estimates.}
\usage{
gg_survival(
  object = NULL,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)

\method{gg_survival}{rfsrc}(
  object,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)

\method{gg_survival}{default}(
  object = NULL,
  interval = NULL,
  censor = NULL,
  by = NULL,
  data = NULL,
  type = c("kaplan", "nelson"),
  ...
)
}
\arguments{
\item{object}{For the \code{rfsrc} method: a fitted
\code{\link[randomForestSRC]{rfsrc}} survival forest.  For the default
method: pass \code{NULL} (or omit) and supply \code{interval},
\code{censor}, and \code{data} instead.}

\item{interval}{Character; name of the time-to-event column in \code{data}
(default method only).}

\item{censor}{Character; name of the event-indicator column in \code{data}
(1 = event, 0 = censored; default method only).}

\item{by}{Optional character; name of a grouping column for stratified
estimates.  For the \code{rfsrc} method, \code{by} must be a column in
\code{object$xvar}.}

\item{data}{A \code{data.frame} containing survival data (default method
only).}

\item{type}{One of \code{"kaplan"} (Kaplan-Meier, default) or
\code{"nelson"} (Nelson-Aalen cumulative hazard).  Default method only.}

\item{...}{Additional arguments passed to \code{\link{kaplan}} or
\code{\link{nelson}}.}
}
\value{
A \code{gg_survival} \code{data.frame} with columns \code{time},
\code{surv}, \code{cum_haz}, \code{lower}, \code{upper}, \code{n.risk},
and optionally \code{groups} when \code{by} is supplied.
}
\description{
Nonparametric survival estimates.
}
\details{
Comparing the forest's ensemble survival curve to the marginal
Kaplan-Meier baseline is a quick sanity check: if they diverge the forest
has found structure the predictors carry; if they track each other closely
the predictors may add little.  \code{gg_survival} computes
the nonparametric baseline -- the Kaplan-Meier or Nelson-Aalen estimate --
so you can place it on the same canvas as the forest predictions from
\code{\link{gg_rfsrc}}.

\code{gg_survival} is an S3 generic that dispatches on the class of its
first argument:

\describe{
\item{\code{rfsrc}}{Extracts the outcome columns from the fitted
forest's \code{$yvar} slot (time in column 1, event indicator in
column 2) and delegates to \code{\link{kaplan}}.  Use \code{by} to
stratify on a predictor from \code{$xvar}: you get one
Kaplan-Meier curve per group, ready to compare against the forest's
group-specific ensemble curves.}
\item{default}{Accepts raw survival columns directly via
\code{interval}, \code{censor}, and \code{data}.  Delegates to
\code{\link{kaplan}} (the default) or \code{\link{nelson}} depending
on \code{type}.}
}
}
\note{
Survival estimation is \code{randomForestSRC}-only; \code{randomForest}
has no survival forest, so no \code{randomForest} method exists.
}
\examples{
## -------- pbc data (default method, raw data columns)
data(pbc, package = "randomForestSRC")
pbc$time <- pbc$days / 364.25

gg_dta <- gg_survival(interval = "time", censor = "status", data = pbc)
plot(gg_dta, error = "none")

# Stratified
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment"
)
plot(gg_dta)

}
\seealso{
\code{\link{kaplan}} \code{\link{nelson}}

\code{\link{plot.gg_survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partial.R
\name{gg_partial}
\alias{gg_partial}
\title{Split partial dependence data into continuous or categorical datasets}
\usage{
gg_partial(part_dta, nvars = NULL, cat_limit = 10, model = NULL)
}
\arguments{
\item{part_dta}{partial plot data from \code{rfsrc::plot.variable}}

\item{nvars}{how many of the partial plot variables to calculate}

\item{cat_limit}{Categorical features are built when there are fewer than
\code{cat_limit} unique feature values.}

\item{model}{a label name applied to all features. Useful when combining
multiple partial plot objects in figures.}
}
\value{
A named list with two elements:
  \describe{
    \item{continuous}{data.frame with columns \code{x}, \code{yhat},
      \code{name} (and optionally \code{model}) for continuous variables}
    \item{categorical}{data.frame with the same columns but with \code{x}
      as a factor, for low-cardinality / categorical variables}
  }
}
\description{
Takes the list returned by \code{rfsrc::plot.variable(partial = TRUE)} and
separates the variables into two data frames: one for continuous predictors
and one for categorical (factor-like) predictors.  The split is controlled
by \code{cat_limit}: variables with more unique x-values than this threshold
are treated as continuous; all others are categorical.
}
\examples{
## Build a small regression forest on the airquality dataset
set.seed(42)
airq <- na.omit(airquality)
rf <- rfsrc(Ozone ~ ., data = airq, ntree = 50)

## Compute partial dependence via plot.variable (show.plots = FALSE to
## suppress the base-graphics output — we only want the data)
pv <- randomForestSRC::plot.variable(rf, partial = TRUE,
                                      show.plots = FALSE)

## Split into continuous and categorical data frames
result <- gg_partial(pv)
head(result$continuous)

## Label this model for later comparison with a second forest
result_labelled <- gg_partial(pv, model = "airq_model")
unique(result_labelled$continuous$model)

}
\seealso{
\code{\link{gg_partial_rfsrc}} \code{\link{gg_partialpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_error.R
\name{gg_error}
\alias{gg_error}
\alias{gg_error.rfsrc}
\alias{gg_error.randomForest}
\alias{gg_error.randomForest.formula}
\title{Random forest error trajectory data object}
\usage{
gg_error(object, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object.}

\item{...}{Optional arguments passed to the methods. Set
\code{training = TRUE} to append the in-bag error trajectory when
supported.}
}
\value{
A \code{gg_error} \code{data.frame} containing at least the
  cumulative OOB error columns and an \code{ntree} counter. When
  \code{training = TRUE} is honored an additional \code{train} column is
  included.
}
\description{
Extract the cumulative out-of-bag (OOB) or in-bag training error rate from
\code{randomForestSRC} and \code{randomForest} fits as a function of the
number of grown trees.
}
\details{
For \code{randomForestSRC} objects the function reshapes the
\code{\link[randomForestSRC]{rfsrc}$err.rate} matrix and annotates it with
the tree index required by \code{\link{plot.gg_error}}. When supplied a
\code{\link[randomForest]{randomForest}} object, the method inspects either
the \code{$mse} or \code{$err.rate} component and, when
\code{training = TRUE} is requested, reconstructs the original training set
via the model call to compute an in-bag error curve using per-tree
predictions. Training curves are only available when the forest was stored
(\code{keep.forest = TRUE}) and the original data can be recovered.
}
\examples{
## Examples from RFSRC package...
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## ------------- iris data
## You can build a randomForest
rfsrc_iris <- rfsrc(Species ~ ., data = iris, tree.err = TRUE)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_iris)

# Plot the gg_error object
plot(gg_dta)

## RandomForest example
rf_iris <- randomForest::randomForest(Species ~ .,
  data = iris,
  tree.err = TRUE,
)
gg_dta <- gg_error(rf_iris)
plot(gg_dta)

gg_dta <- gg_error(rf_iris, training = TRUE)
plot(gg_dta)
## ------------------------------------------------------------
## Regression example
## ------------------------------------------------------------

## ------------- airq data
rfsrc_airq <- rfsrc(Ozone ~ .,
  data = airquality,
  na.action = "na.impute", tree.err = TRUE,
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_airq)

# Plot the gg_error object
plot(gg_dta)


## ------------- Boston data
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas)
rfsrc_boston <- rfsrc(medv ~ .,
  data = Boston,
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_boston)

# Plot the gg_error object
plot(gg_dta)


## ------------- mtcars data
rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, tree.err = TRUE)

# Get a data.frame containing error rates
gg_dta<- gg_error(rfsrc_mtcars)

# Plot the gg_error object
plot(gg_dta)


## ------------------------------------------------------------
## Survival example
## ------------------------------------------------------------
## ------------- veteran data
## randomized trial of two treatment regimens for lung cancer
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran,
                       tree.err = TRUE)

gg_dta <- gg_error(rfsrc_veteran)
plot(gg_dta)

## ------------- pbc data
# Load a cached randomForestSRC object
# We need to create this dataset
data(pbc, package = "randomForestSRC",)
# For whatever reason, the age variable is in days... makes no sense to me
for (ind in seq_len(dim(pbc)[2])) {
 if (!is.factor(pbc[, ind])) {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 } else {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
     if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 }
 if (!is.logical(pbc[, ind]) &
     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
   pbc[, ind] <- factor(pbc[, ind])
 }
}
#Convert age to years
pbc$age <- pbc$age / 364.24

pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]
# Create a test set from the remaining patients
pbc_test <- pbc[which(is.na(pbc$treatment)), ]

#========
# build the forest:
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
 dta_train,
 nsplit = 10,
 na.action = "na.impute",
 tree.err = TRUE,
 forest = TRUE,
 importance = TRUE,
 save.memory = TRUE
)


gg_dta <- gg_error(rfsrc_pbc)
plot(gg_dta)

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{plot.gg_error}}, \code{\link{gg_vimp}},
  \code{\link{gg_variable}},
  \code{\link[randomForestSRC]{rfsrc}},
  \code{\link[randomForest]{randomForest}},
  \code{\link[randomForestSRC]{plot.rfsrc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partial_rfsrc.R
\name{gg_partial_rfsrc}
\alias{gg_partial_rfsrc}
\title{Partial dependence data from an rfsrc model}
\usage{
gg_partial_rfsrc(
  rf_model,
  xvar.names = NULL,
  xvar2.name = NULL,
  newx = NULL,
  partial.time = NULL,
  partial.type = c("surv", "chf", "mort"),
  cat_limit = 10,
  n_eval = 25
)
}
\arguments{
\item{rf_model}{A fitted \code{\link[randomForestSRC]{rfsrc}} object.}

\item{xvar.names}{Character vector of predictor names for which partial
dependence should be computed. Must be a subset of \code{rf_model$xvar.names}.}

\item{xvar2.name}{Optional single character name of a grouping variable in
\code{newx}. When supplied, partial dependence is computed separately for
each unique level of this variable and a \code{grp} column is appended.}

\item{newx}{Optional \code{data.frame} of predictor values to evaluate
partial effects at. Defaults to the training data stored in
\code{rf_model$xvar}. All column names must match \code{rf_model$xvar.names}.}

\item{partial.time}{Numeric vector of desired time points for survival
forests (ignored for regression/classification).  Values are automatically
snapped to the nearest entry in \code{rf_model$time.interest}; see the
\strong{Survival forests} section below.  When \code{NULL} (default),
three quartile points of \code{time.interest} are used.}

\item{partial.type}{Character; type of predicted value for survival
forests, passed through to \code{\link[randomForestSRC]{partial.rfsrc}}.
One of \code{"surv"} (default), \code{"chf"}, or \code{"mort"}. Ignored
for non-survival forests. \code{partial.rfsrc()} requires a non-\code{NULL}
value for survival families; supplying it here avoids a cryptic
\dQuote{argument is of length zero} error from the underlying C code.}

\item{cat_limit}{Variables with fewer than \code{cat_limit} unique values in
\code{newx} are treated as categorical; all others are continuous.
Defaults to 10.}

\item{n_eval}{Number of evaluation points for continuous variables. Instead
of passing all observed values (which can be slow, especially for survival
forests), continuous predictors are evaluated on a quantile grid of this
many points. Categorical variables always use all unique levels.
Defaults to 25.}
}
\value{
A named list with two elements:
\describe{
\item{continuous}{A \code{data.frame} with columns \code{x} (numeric),
\code{yhat}, \code{name} (variable name), and optionally \code{grp}
(the level of \code{xvar2.name}) and \code{time} (survival forests
only) for all continuous predictors.}
\item{categorical}{A \code{data.frame} with the same columns but
\code{x} kept as character, for low-cardinality predictors.}
}
}
\description{
A partial dependence curve marginalizes the forest's prediction over all
other predictors: for each evaluation point of the target variable, the
forest scores every training observation with that value substituted in,
then averages the result.  What you get is the average effect of the target
variable after "integrating out" the rest -- a curve that would be flat if
the variable carried no signal.
}
\details{
This function builds those curves for one or more predictors by calling
\code{\link[randomForestSRC]{partial.rfsrc}} and then tidy-stacking the
results into separate data frames for continuous and categorical variables.
Unlike \code{\link{gg_partial}} (which wraps \code{plot.variable}), you
pass the fitted \code{rfsrc} object directly -- no intermediate
\code{plot.variable} step.

For survival forests, the marginalized quantity depends on
\code{partial.type}: survival probability (\code{"surv"}), cumulative
hazard function (\code{"chf"}), or expected mortality (\code{"mort"}).
You can request the curve at one or more time horizons via
\code{partial.time}; the resulting data have a \code{time} column so the
plot layers them as separate coloured lines.
}
\section{Survival forests and \code{partial.time}}{

\code{\link[randomForestSRC]{partial.rfsrc}} expects every value in
\code{partial.time} to be an exact member of the model's
\code{time.interest} vector, the unique observed event times stored in the
fitted object.  Pass an arbitrary time, even a plausible one such as
\code{c(1, 3)} for a study measured in years, and you get a C-level
prediction error from inside \code{partial.rfsrc}.

\code{gg_partial_rfsrc} takes care of this: every element of
\code{partial.time} is silently snapped to its nearest
\code{time.interest} value before the call.  To target a specific
follow-up horizon, find the closest grid point yourself and pass it
explicitly:

\preformatted{
ti  <- rf_model$time.interest
t1  <- ti[which.min(abs(ti - 1))]   # nearest to 1 year
pd  <- gg_partial_rfsrc(rf_model, xvar.names = "x", partial.time = t1)
}
}

\section{Logical predictor columns}{

\code{\link[randomForestSRC]{partial.rfsrc}} does not handle
\code{logical} predictor columns correctly in survival forests
(randomForestSRC <= 3.5.1).  If your training data contains binary 0/1
columns, convert them to \code{\link{factor}} rather than \code{logical}
before fitting the model.
}

\examples{
## ------------------------------------------------------------
##
## regression
##
## ------------------------------------------------------------

airq.obj <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality)

## partial effect for wind
prt_dta <- gg_partial_rfsrc(airq.obj,
                       xvar.names = c("Wind"))

}
\seealso{
\code{\link{gg_partial}}, \code{\link[randomForestSRC]{partial.rfsrc}},
\code{\link[randomForestSRC]{get.partial.plot.data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_beta_varpro.R
\name{gg_beta_varpro}
\alias{gg_beta_varpro}
\title{Per-variable lasso-beta importance from a varPro fit}
\usage{
gg_beta_varpro(object, ..., cutoff = NULL, beta_fit = NULL, which_class = NULL)
}
\arguments{
\item{object}{A \code{varpro} fit from \code{\link[varPro:varpro]{varPro::varpro()}} (regression or
classification family).}

\item{...}{Forwarded to \code{\link[varPro:beta.varpro]{varPro::beta.varpro()}} when \code{beta_fit = NULL};
ignored otherwise (with a warning). Documented forwardables: \code{use.cv},
\code{use.1se}, \code{nfolds}, \code{maxit}, \code{thresh}, \code{max.rules.tree}, \code{max.tree}.}

\item{cutoff}{Selection threshold on \code{beta_mean}. \code{NULL} (default) means
\code{mean(beta_mean)} across released variables. Numeric scalar otherwise.}

\item{beta_fit}{Optional pre-computed \code{\link[varPro:beta.varpro]{varPro::beta.varpro()}} result for
the same \code{object}. \code{NULL} (default) means the wrapper runs \code{beta.varpro()}
itself. When supplied, must be a \code{varpro}-class object whose \verb{$results}
has columns \code{tree / branch / variable / n.oob / imp}.}

\item{which_class}{For a classification fit, name of a single response
level to subset on. \code{NULL} (default) returns all classes (binary fits
resolve to the \emph{last} factor level, the positive-class convention
used by \code{glm} and \code{gg_roc}). Ignored with a warning on regression
fits.}
}
\value{
A \code{data.frame} of class \code{c("gg_beta_varpro", "data.frame")}.
For a regression fit: one row per released variable, sorted by
\code{beta_mean} descending. For a classification fit: long-format with
an extra \code{class} column, one row per (variable, class) pair;
\code{variable} is a factor whose levels are set by
\verb{mean(|sum-of-class-beta|)} descending so every facet / panel shares
the same row order. \code{which_class} (or the binary default
last-factor-level) collapses the output to a single class.
}
\description{
Tidy wrapper around \code{\link[varPro:beta.varpro]{varPro::beta.varpro()}} for the regression or
classification family.
Aggregates the per-rule lasso coefficient \eqn{\hat{\beta}}{beta hat} by
variable into the mean absolute value
\eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)} and flags variables
above a scalar cutoff. Optional
\code{beta_fit} argument lets callers compute the expensive
\code{beta.varpro()} step once and reuse the result.
}
\note{
Multivariate regression (\verb{regr+}) and survival families are out
of scope for this release and tracked for v3.1.0. The
unsupported-family path errors with a message pointing at that work.
}
\section{What this is doing}{

Think of the varPro release-rule mechanism as asking: "given a region of
the feature space that the forest carved out, what changes when I remove
the constraint on this one variable and let observations leave?" The
standard importance answer (from \code{\link[=gg_varpro]{gg_varpro()}}) measures that change as a
z-scored contrast between local estimators: no synthetic data, no
permutation. \code{beta.varpro()} asks the same question with a different
ruler: for each rule (a tree-branch pair), it fits a one-predictor lasso
regression of the response on the released variable's values, restricted
to the OOB observations inside the rule's region. The wrapper aggregates
those per-rule coefficients into one number per variable.

The key distinction from \code{\link[=gg_vimp]{gg_vimp()}}, which measures Breiman-Cutler
permutation importance by perturbing a variable's values and watching OOB
error climb, is that neither \code{\link[=gg_varpro]{gg_varpro()}} nor \code{gg_beta_varpro()}
touches the data synthetically: all contrasts are between real subsets
defined by the forest's rules.
}

\section{What \code{imp} actually is (pedantic, because the column name is misleading)}{

The \code{imp} column on \code{beta.varpro()}'s \verb{$results} is \strong{not} a
variable-importance score in the conventional sense. It is a regularised
regression coefficient. Specifically:
\itemize{
\item Per rule, \code{glmnet} fits a one-predictor lasso of the response on
the released variable inside the rule's OOB region. \code{use.cv = TRUE}
selects \eqn{\lambda}{lambda} by 10-fold CV (default \code{nfolds = 10});
\code{use.1se = TRUE} (default) picks \code{lambda.1se}. \code{use.cv = FALSE} uses the
full \eqn{\lambda}{lambda} path.
\item \code{imp} is the \strong{fitted coefficient} \eqn{\hat{\beta}}{beta hat} at the
chosen \eqn{\lambda}{lambda}. \strong{Sign is
real} (direction of local association). \strong{Magnitude depends on
the predictor's units} (raw \code{x}, no standardisation); a predictor
in millimetres has a smaller \eqn{|\hat{\beta}|}{|beta hat|} than the
same predictor in metres.
\item Lasso shrinkage can drive \eqn{\hat{\beta}}{beta hat} to \strong{exactly zero}.
Those zeros are
data, not missingness, and are kept in the aggregation. Convergence
failures land as \code{NA_real_} and are dropped.
\item The per-variable aggregate is \code{beta_mean},
\eqn{\mathrm{mean}(|\hat{\beta}|)}{mean(|beta hat|)}, across the
rules where this variable was released. It is \strong{not} a permutation
importance, \strong{not} a split-strength importance, and \strong{not} directly
comparable on the same numeric axis to \code{gg_varpro()}'s z-scores.
Disagreement with \code{gg_varpro} is often diagnostic, not a bug.
}

In code form: \code{imp_r} is the glmnet coefficient \eqn{\hat{\beta}}{beta hat}
fit on \code{y ~ x_v} restricted to rule r, with \eqn{\lambda}{lambda} chosen by
\code{use.cv} / \code{use.1se}.
}

\section{What's in the output}{

One row per released variable. Columns:
\itemize{
\item \code{variable}: predictor name.
\item \code{beta_mean}: mean of \eqn{|\hat{\beta}|}{|beta hat|} across that
variable's rules.
\item \code{n_rules}: count of rules contributing (zero-beta rules included; only
\code{NA} failures excluded).
\item \code{selected}: \code{beta_mean >= cutoff}.
}

Provenance attribute carries \code{source}, \code{family}, \code{ntree}, \code{cutoff},
\code{cutoff_default}, \code{use.cv}, \code{n_rules_total}, \code{n_rules_nonzero},
\code{precomputed}, and \code{xvar.names}.
}

\section{What you use this for}{

Picking variables when local effects matter more than aggregate
split-strength contribution. Compare side-by-side with \code{\link[=gg_varpro]{gg_varpro()}}:
a variable that scores high here but low in \code{gg_varpro} is one whose
local linear effect inside many rules is real even though its
release-rule contrast is modest.
}

\section{Caching}{

\code{beta.varpro()} is the expensive call (per-rule \code{glmnet} / \code{cv.glmnet},
often minutes on real data). Compute it once and reuse:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 200)
b <- varPro::beta.varpro(v, use.cv = TRUE)        # expensive, once
gg_a <- gg_beta_varpro(v, beta_fit = b)            # cheap
gg_b <- gg_beta_varpro(v, beta_fit = b, cutoff = 0.5)
}\if{html}{\out{</div>}}

Provenance carries \code{precomputed = TRUE} when \code{beta_fit} was supplied.
}

\section{Classification}{

For a varpro classification fit (\code{object$family == "class"},
binary or multi-class), the returned frame is long-format with an
extra \code{class} column: one row per (variable, class) pair. The
\code{beta_mean} column aggregates the \strong{per-class lasso coefficient}
\eqn{\hat{\beta}}{beta hat} stored in
\code{beta.varpro()}'s \verb{imp.<k>} columns (one per class level). Same
pedantic-beta semantics as regression, applied independently to each
class.

\strong{Binary default}: \code{which_class = NULL} resolves to the \emph{last}
factor level of the response, the positive-class convention used
by \code{glm} and \code{gg_roc}. For a 30-day-mortality outcome with levels
\code{c("no", "yes")}, that means the wrapper shows you \code{"yes"} (the
event) by default.

\strong{Multi-class default}: \code{which_class = NULL} returns all K
classes; the plot method renders \code{facet_wrap(~ class)} with one
cutoff line per facet.

\strong{\code{which_class = "<name>"}} filters to a single class regardless
of K. Errors if the name isn't in the response levels.

\strong{Per-class cutoffs}: \code{cutoff = NULL} resolves to each class's
\code{mean(beta_mean)}. A scalar broadcasts. A named numeric vector
overrides per class; missing names fall back to that class's mean.

Example (30-day mortality, binary):

\if{html}{\out{<div class="sourceCode r">}}\preformatted{fit <- varPro::varpro(event_30d ~ ., data = clinical, ntree = 200)
gg  <- gg_beta_varpro(fit)   # default: "yes" panel
plot(gg)
}\if{html}{\out{</div>}}
}

\section{Reproducibility}{

Byte-for-byte agreement between cached (\code{beta_fit = b}) and uncached
(\code{beta_fit = NULL}) outputs requires that \code{b} was computed by
\code{beta.varpro(object, ...)} on the same \code{object}; \code{set.seed()} alone is
not sufficient, because \code{beta.varpro}'s internal \code{cv.glmnet} fits can
pick slightly different folds across separate calls. Reuse \code{beta_fit}
when reproducibility matters.
}

\examples{
\donttest{
if (requireNamespace("varPro", quietly = TRUE)) {
  set.seed(1)
  v <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
  b <- varPro::beta.varpro(v)
  gg <- gg_beta_varpro(v, beta_fit = b)
  plot(gg)
}
}

}
\seealso{
\code{\link[=gg_varpro]{gg_varpro()}}, \code{\link[=gg_vimp]{gg_vimp()}}, \code{\link[=plot.gg_beta_varpro]{plot.gg_beta_varpro()}}, \code{\link[varPro:beta.varpro]{varPro::beta.varpro()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_rfsrc.R
\name{plot.gg_rfsrc}
\alias{plot.gg_rfsrc}
\title{Predicted response plot from a \code{\link{gg_rfsrc}} object.}
\usage{
\method{plot}{gg_rfsrc}(x, notch = TRUE, ...)
}
\arguments{
\item{x}{A \code{\link{gg_rfsrc}} object, or a raw
\code{\link[randomForestSRC]{rfsrc}} object (which will be passed through
\code{\link{gg_rfsrc}} automatically before plotting).}

\item{notch}{Logical; whether to draw notched boxplots for regression and
classification forests (default \code{TRUE}).  Set \code{notch = FALSE}
to suppress notches when sample sizes are too small for reliable
confidence intervals on the median.}

\item{...}{Additional arguments forwarded to the underlying
\code{ggplot2} geometry calls.  Commonly useful arguments include:
\describe{
  \item{\code{alpha}}{Numeric in \eqn{[0,1]}; point/ribbon transparency.
    For survival plots with confidence bands the ribbon alpha is
    automatically halved relative to the value supplied here.}
  \item{\code{size}}{Point or line size passed to \code{geom_jitter},
    \code{geom_step}, etc.}
}
Arguments that control \code{\link{gg_rfsrc}} (e.g. \code{conf.int},
\code{surv_type}, \code{by}) should be applied when constructing the
\code{gg_rfsrc} object before calling \code{plot()}.}
}
\value{
A \code{ggplot} object.  The plot appearance depends on the forest
  family stored in \code{x}:
  \describe{
    \item{Regression (\code{"regr"})}{Jitter + notched boxplot of OOB
      predicted values.  If a \code{group} column is present the x-axis
      shows each group label; otherwise observations are collapsed to a
      single x-position.}
    \item{Classification (\code{"class"})}{Binary: jitter + notched
      boxplot of the predicted class probability.  Multi-class: jitter
      plot with one panel per class (class probabilities in long form).}
    \item{Survival (\code{"surv"})}{Step curves of the ensemble survival
      function.  When \code{gg_rfsrc} was called with \code{conf.int},
      a shaded ribbon is added.  When called with \code{by}, curves are
      coloured by group.}
  }
}
\description{
Plot the predicted response from a \code{\link{gg_rfsrc}} object, the
\code{\link[randomForestSRC]{rfsrc}} prediction, using the OOB prediction
from the forest.  The plot type adapts automatically to the forest family:
jitter + boxplot for regression and classification, step curves for
survival.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
# Build a small classification forest (ntree=50 keeps example fast)
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_iris)

plot(gg_dta)

## ------------------------------------------------------------
## Regression example
## ------------------------------------------------------------
## -------- air quality data
# na.action = "na.impute" handles missing Ozone / Solar.R values
set.seed(42)
rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_airq)

plot(gg_dta)

## -------- mtcars data
set.seed(42)
rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_mtcars)

plot(gg_dta)

## ------------------------------------------------------------
## Survival example
## ------------------------------------------------------------
## -------- veteran data
## randomized trial of two treatment regimens for lung cancer
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_veteran)
plot(gg_dta)

# With 95\% pointwise bootstrap confidence bands
gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = .95)
plot(gg_dta)

# Stratified by treatment arm
gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt")
plot(gg_dta)

## -------- pbc data (larger dataset -- skipped on CRAN)
\donttest{
data(pbc, package = "randomForestSRC")
# For whatever reason, the age variable is in days; convert to years
for (ind in seq_len(dim(pbc)[2])) {
  if (!is.factor(pbc[, ind])) {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  } else {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  }
  if (!is.logical(pbc[, ind]) &
    length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
    pbc[, ind] <- factor(pbc[, ind])
  }
}
# Convert age from days to years
pbc$age <- pbc$age / 364.24
pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
# Remove test-set patients (those with no assigned treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]

set.seed(42)
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
  dta_train,
  nsplit = 10,
  na.action = "na.impute",
  forest = TRUE,
  importance = TRUE,
  save.memory = TRUE
)

gg_dta <- gg_rfsrc(rfsrc_pbc)
plot(gg_dta)

gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = .95)
plot(gg_dta)

gg_dta <- gg_rfsrc(rfsrc_pbc, by = "treatment")
plot(gg_dta)
}

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
R, Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_rfsrc}} \code{\link[randomForestSRC]{rfsrc}}
  \code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_roc.R
\name{gg_roc.rfsrc}
\alias{gg_roc.rfsrc}
\alias{gg_roc}
\alias{gg_roc.randomForest}
\title{ROC (Receiver Operating Characteristic) curve data from a classification forest.}
\usage{
\method{gg_roc}{rfsrc}(object, which_outcome, oob = TRUE, per_class = FALSE, ...)
}
\arguments{
\item{object}{A classification \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object. Only forests with
\code{family == "class"} (rfsrc) or \code{type == "classification"}
(randomForest) are supported.}

\item{which_outcome}{Integer index or character name of the class to score.
For binary forests this is usually \code{1} or \code{2}; for multi-class
forests, any valid class index or level name. \code{which_outcome = "all"}
or \code{0} behaves differently by engine:
\describe{
\item{\code{randomForest} method}{Returns a macro-averaged
one-vs-rest ROC computed over the per-class probabilities.}
\item{\code{rfsrc} method}{Warns and falls back to class 1. The
macro-average and per-class faceting for the \code{rfsrc} path
are tracked separately under issue #72.}
}}

\item{oob}{Logical; if \code{TRUE} (default), build the curve from
out-of-bag predicted probabilities, otherwise from full in-bag
predictions. For \code{randomForest}, \code{TRUE} uses the out-of-bag
vote probabilities in \code{object$votes}; \code{FALSE} uses in-bag
\code{predict(type = "prob")}.}

\item{per_class}{Logical; if \code{TRUE} and the forest has more than two
classes, return one ROC curve per class, each class scored against all
the others. The result is a long-format \code{data.frame} with a
\code{class} factor column and a named AUC vector attribute, ordered by
descending AUC. Binary forests treat \code{per_class = TRUE} as a no-op.
Honoured by the \code{randomForest} method only.}

\item{...}{Extra arguments (currently unused).}
}
\value{
A \code{gg_roc} \code{data.frame}, one row per unique prediction
threshold, with columns:
\describe{
\item{sens}{Sensitivity (true positive rate) at the threshold.}
\item{spec}{Specificity (true negative rate) at the threshold.}
\item{pct}{The probability threshold used for that row.}
}
Pass it to \code{\link{calc_auc}} for the area under the curve.
}
\description{
A classifier does not hand you a class; it hands you a predicted probability,
and you pick a threshold. Slide that threshold from 0 to 1 and the trade-off
between catching the positives and crying wolf shifts the whole way. The ROC
curve traces that trade-off. For one class of a classification
\code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} forest, \code{gg_roc} walks every
threshold and records sensitivity (the true positive rate) against
specificity (1 minus the false positive rate).
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)

# ROC for setosa
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica
gg_dta <- gg_roc(rfsrc_iris, which_outcome = 3)
plot(gg_dta)

## -------- iris data
rf_iris <- randomForest::randomForest(Species ~ ., data = iris)

# ROC for setosa
gg_dta <- gg_roc(rf_iris, which_outcome = 1)
plot(gg_dta)

# ROC for versicolor
gg_dta <- gg_roc(rf_iris, which_outcome = 2)
plot(gg_dta)

# ROC for virginica
gg_dta <- gg_roc(rf_iris, which_outcome = 3)
plot(gg_dta)

}
\seealso{
\code{\link{plot.gg_roc}}, \code{\link{calc_roc}},
\code{\link{calc_auc}},
\code{\link[randomForestSRC]{rfsrc}},
\code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_variable.R
\name{plot.gg_variable}
\alias{plot.gg_variable}
\title{Plot a \code{\link{gg_variable}} object,}
\usage{
\method{plot}{gg_variable}(
  x,
  xvar,
  time,
  time_labels,
  panel = FALSE,
  oob = TRUE,
  points = TRUE,
  smooth = TRUE,
  ...
)
}
\arguments{
\item{x}{\code{\link{gg_variable}} object created from a
\code{\link[randomForestSRC]{rfsrc}} object}

\item{xvar}{variable (or list of variables) of interest.}

\item{time}{For survival, one or more times of interest}

\item{time_labels}{string labels for times}

\item{panel}{Should plots be faceted along multiple xvar?}

\item{oob}{oob estimates (boolean)}

\item{points}{plot the raw data points (boolean)}

\item{smooth}{include a smooth curve (boolean)}

\item{...}{arguments passed to the \code{ggplot2} functions.}
}
\value{
A single \code{ggplot} object when \code{length(xvar) == 1} or
  \code{panel = TRUE}. Otherwise a named list of \code{ggplot} objects, one
  per variable in \code{xvar}.
}
\description{
Plot a \code{\link{gg_variable}} object,
}
\examples{
## ------------------------------------------------------------
## classification
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)

gg_dta <- gg_variable(rfsrc_iris)
plot(gg_dta, xvar = "Sepal.Width")
plot(gg_dta, xvar = "Sepal.Length")

## Panel plot across all predictors
plot(gg_dta,
  xvar = rfsrc_iris$xvar.names,
  panel = TRUE, se = FALSE
)

## ------------------------------------------------------------
## regression
## ------------------------------------------------------------
## -------- air quality data
# na.action = "na.impute" handles missing Ozone / Solar.R values
set.seed(42)
rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
gg_dta <- gg_variable(rfsrc_airq)

# Treat Month as an ordinal factor for better visualisation
gg_dta[, "Month"] <- factor(gg_dta[, "Month"])

plot(gg_dta, xvar = "Wind")
plot(gg_dta, xvar = "Temp")
plot(gg_dta, xvar = "Solar.R")

# Panel plot across continuous predictors
plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)

# Factor variable uses notched boxplots
plot(gg_dta, xvar = "Month", notch = TRUE)

## ------------------------------------------------------------
## survival examples
## ------------------------------------------------------------
## -------- veteran data
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., veteran,
  nsplit = 10,
  ntree = 50
)

# Marginal survival at 90 days
gg_dta <- gg_variable(rfsrc_veteran, time = 90)

# Single-variable dependence plots
plot(gg_dta, xvar = "age")
plot(gg_dta, xvar = "diagtime")

# Panel coplot for two predictors at a single time
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)

# Compare survival at 30, 90, and 365 days simultaneously
gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))

# Single-variable plot (one facet per time point)
plot(gg_dta, xvar = "age")

# Panel coplot across two predictors and three time points
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_variable}}, \code{\link{gg_partial}},
  \code{\link[randomForestSRC]{plot.variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R
\name{plot.gg_partialpro}
\alias{plot.gg_partialpro}
\title{Plot a \code{\link{gg_partialpro}} object}
\usage{
\method{plot}{gg_partialpro}(x, type = c("parametric", "nonparametric", "causal"), ...)
}
\arguments{
\item{x}{A \code{\link{gg_partialpro}} object.}

\item{type}{Character vector; one or more of \code{"parametric"},
\code{"nonparametric"}, \code{"causal"}.  Defaults to all three.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.  When both continuous
  and categorical variables are present the two panels are combined
  vertically via \code{patchwork::wrap_plots()}.
}
\description{
Produces ggplot2 partial dependence curves from the named list returned by
\code{\link{gg_partialpro}}, which wraps \code{varpro::partialpro} output.
}
\details{
Each variable produces up to three effect curves: parametric, nonparametric,
and causal.  The \code{type} argument controls which are shown.
}
\examples{
## ggRandomForests does not depend on the varpro package; we construct a
## minimal mock of the partialpro() output so the example runs everywhere.
set.seed(42)
n_obs <- 30
n_pts <- 15
mock_data <- list(
  age = list(
    xvirtual    = seq(30, 80, length.out = n_pts),
    xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
  ),
  sex = list(
    xvirtual    = c(0, 1),
    xorg        = sample(c(0, 1), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
  )
)

pp <- gg_partialpro(mock_data)

# Returns a single ggplot/patchwork figure combining continuous and
# categorical panels when both variable types are present.
plot(pp)

# Restrict to one or two effect types
plot(pp, type = "nonparametric")
plot(pp, type = c("parametric", "causal"))

}
\seealso{
\code{\link{gg_partialpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_vimp.R
\name{plot.gg_vimp}
\alias{plot.gg_vimp}
\title{Plot a \code{\link{gg_vimp}} object, extracted variable importance of a
\code{\link[randomForestSRC]{rfsrc}} object}
\usage{
\method{plot}{gg_vimp}(x, relative, lbls, ...)
}
\arguments{
\item{x}{\code{\link{gg_vimp}} object created from a
\code{\link[randomForestSRC]{rfsrc}} object}

\item{relative}{should we plot vimp or relative vimp. Defaults to vimp.}

\item{lbls}{A vector of alternative variable labels. Item names should be
the same as the variable names.}

\item{...}{optional arguments passed to gg_vimp if necessary}
}
\value{
\code{ggplot} object
}
\description{
Draws a horizontal bar chart of the VIMP scores extracted by
\code{\link{gg_vimp}}.  Each bar represents one predictor; bar length is
proportional to its permutation VIMP -- the average rise in OOB prediction
error when that predictor's OOB values are randomly shuffled.  Predictors
are sorted in descending order of importance so the most influential
variables appear at the top.
}
\details{
Bars are coloured by the \code{positive} flag: a bar at or below zero
(non-positive VIMP) is colour-coded differently to flag predictors that
\emph{hurt} OOB accuracy when their signal is removed -- usually a sign of
collinearity or a very noisy variable.  In a well-behaved forest most bars
are positive; the colour distinction matters when a handful are not.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)
gg_dta <- gg_vimp(rfsrc_iris)
plot(gg_dta)

## ------------------------------------------------------------
## regression example
## ------------------------------------------------------------
## -------- air quality data
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., airquality)
gg_dta <- gg_vimp(rfsrc_airq)
plot(gg_dta)


}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
R, Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_vimp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_roc.R
\name{calc_roc.rfsrc}
\alias{calc_roc.rfsrc}
\alias{calc_roc.randomForest}
\alias{calc_roc}
\title{Receiver Operator Characteristic calculator}
\usage{
\method{calc_roc}{rfsrc}(object, dta, which_outcome = "all", oob = TRUE, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}},
\code{\link[randomForestSRC]{predict.rfsrc}}, or
\code{\link[randomForest]{randomForest}} classification object containing
predicted class probabilities.}

\item{dta}{A factor (or coercible to factor) of the true observed class
labels, one per observation. Typically \code{object$yvar} for rfsrc or
\code{object$y} for randomForest.}

\item{which_outcome}{Integer index of the class for which the ROC curve is
computed (e.g. \code{1} for the first class, \code{2} for the second).
Use \code{"all"} to request all classes (currently falls back to class 1
with a warning).}

\item{oob}{Logical; if \code{TRUE} (default for rfsrc) use OOB predicted
probabilities. Forced to \code{FALSE} for \code{randomForest} objects.}

\item{...}{Extra arguments passed to helper functions (currently unused).}
}
\value{
A \code{gg_roc} \code{data.frame} with columns \code{sens}
(sensitivity), \code{spec} (specificity), and \code{pct} (the probability
threshold), with one row per unique prediction value. Suitable for passing
to \code{\link{calc_auc}} or \code{\link{plot.gg_roc}}.
}
\description{
Receiver Operator Characteristic calculator
}
\details{
For a randomForestSRC prediction and the actual
response value, calculate the specificity (1-False Positive Rate) and
sensitivity (True Positive Rate) of a predictor.

This is a helper function for the \code{\link{gg_roc}} functions, and
not intended for use by the end user.
}
\examples{
## Taken from the gg_roc example
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris)

gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
  which_outcome = 1, oob = TRUE
)
gg_dta <- calc_roc(rfsrc_iris, rfsrc_iris$yvar,
  which_outcome = 1, oob = FALSE
)

rf_iris <- randomForest::randomForest(Species ~ ., data = iris)
# randomForest stores the response in $y (rfsrc uses $yvar); pass the
# original training factor so calc_roc has the class labels.
gg_dta <- calc_roc(rf_iris, iris$Species,
  which_outcome = 1
)
gg_dta <- calc_roc(rf_iris, iris$Species,
  which_outcome = 2
)

}
\seealso{
\code{\link{calc_auc}} \code{\link{gg_roc}}

\code{\link{plot.gg_roc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R
\name{plot.gg_partial_rfsrc}
\alias{plot.gg_partial_rfsrc}
\title{Plot a \code{\link{gg_partial_rfsrc}} object}
\usage{
\method{plot}{gg_partial_rfsrc}(x, ...)
}
\arguments{
\item{x}{A \code{\link{gg_partial_rfsrc}} object.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.  When both continuous
  and categorical variables are present the two panels are combined
  vertically via \code{patchwork::wrap_plots()}.
}
\description{
Produces ggplot2 partial dependence curves from the named list returned by
\code{\link{gg_partial_rfsrc}}.
}
\details{
For standard (non-survival) forests: continuous predictors are line plots,
categorical predictors are bar charts, both faceted by variable name.

For survival forests (when a \code{time} column is present): each evaluation
time point is a separate curve over the predictor's value, faceted by
variable name. The y-axis label adapts to the \code{partial.type} stored on
the object (\dQuote{Predicted Survival}, \dQuote{Predicted CHF}, or
\dQuote{Predicted Mortality}).

For two-variable surface plots (when a \code{grp} column is present):
each group level is a separate line, faceted by primary predictor name.
}
\examples{
## ------------------------------------------------------------
## Regression forest -- one continuous curve per variable
## ------------------------------------------------------------
set.seed(42)
airq <- na.omit(airquality)
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)

pd <- gg_partial_rfsrc(rfsrc_airq, xvar.names = c("Wind", "Temp"),
                       n_eval = 10)
plot(pd)

\donttest{
## ------------------------------------------------------------
## Survival forest -- one curve per requested time horizon,
## faceted by variable. Y-axis label tracks `partial.type`.
## ------------------------------------------------------------
# randomForestSRC's formula parser requires the unqualified Surv() symbol;
# it Depends on `survival`, so Surv is on the search path once
# randomForestSRC is loaded.
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_v <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
                                  data = veteran, ntree = 50)
ti  <- rfsrc_v$time.interest
t30 <- ti[which.min(abs(ti - 30))]
t90 <- ti[which.min(abs(ti - 90))]

# Default partial.type = "surv" -> y-axis "Predicted Survival"
pd_s <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
                         partial.time = c(t30, t90), n_eval = 8)
plot(pd_s)

# partial.type = "chf" -> y-axis "Predicted CHF"
pd_c <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
                         partial.time = c(t30, t90),
                         partial.type = "chf", n_eval = 8)
plot(pd_c)
}

}
\seealso{
\code{\link{gg_partial_rfsrc}}, \code{\link{plot.gg_partial}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_methods.R
\name{summary.gg}
\alias{summary.gg}
\alias{print.summary.gg}
\alias{summary.gg_error}
\alias{summary.gg_vimp}
\alias{summary.gg_rfsrc}
\alias{summary.gg_variable}
\alias{summary.gg_partial}
\alias{summary.gg_partial_rfsrc}
\alias{summary.gg_partialpro}
\alias{summary.gg_roc}
\alias{summary.gg_survival}
\alias{summary.gg_brier}
\title{Summary methods for gg_* data objects}
\usage{
\method{print}{summary.gg}(x, ...)

\method{summary}{gg_error}(object, ...)

\method{summary}{gg_vimp}(object, ...)

\method{summary}{gg_rfsrc}(object, ...)

\method{summary}{gg_variable}(object, ...)

\method{summary}{gg_partial}(object, ...)

\method{summary}{gg_partial_rfsrc}(object, ...)

\method{summary}{gg_partialpro}(object, ...)

\method{summary}{gg_roc}(object, ...)

\method{summary}{gg_survival}(object, ...)

\method{summary}{gg_brier}(object, ...)
}
\arguments{
\item{x}{A \code{summary.gg} object (for \code{print.summary.gg}).}

\item{...}{Not currently used.}

\item{object}{A \code{gg_*} data object.}
}
\value{
A \code{summary.gg} object (a list with \code{header} and
  \code{body} character vectors), returned invisibly from
  \code{print.summary.gg}.
}
\description{
Each \code{summary.gg_*()} method returns a \code{summary.gg} object
containing a header line and per-class diagnostic statistics (OOB error
curve, top VIMP variables, time range, integrated CRPS, etc.).
\code{print.summary.gg()} renders the object to the console.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
summary(gg_error(rf))
summary(gg_vimp(rf))

}
\seealso{
\code{\link{print.gg}}, \code{\link{autoplot.gg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R, R/plot.gg_partial_varpro.R
\name{plot.gg_partial_varpro}
\alias{plot.gg_partial_varpro}
\alias{plot.gg_partialpro}
\title{Plot a \code{\link{gg_partial_varpro}} object}
\usage{
\method{plot}{gg_partialpro}(x, type = c("parametric", "nonparametric", "causal"), ...)

\method{plot}{gg_partial_varpro}(x, type = c("parametric", "nonparametric", "causal"), ...)
}
\arguments{
\item{x}{A \code{\link{gg_partial_varpro}} object.}

\item{type}{Character vector; one or more of \code{"parametric"},
\code{"nonparametric"}, \code{"causal"}.  Defaults to all three.
Ignored for path-C objects.}

\item{...}{Unused for path-A objects; forwarded to
\code{plot.gg_partial_rfsrc} for path-C objects.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.
}
\description{
Draws the partial dependence curves from the list that
\code{\link{gg_partial_varpro}} returns.  Continuous predictors get
overlaid line curves, one per effect type; categorical predictors get
side-by-side boxplots.  Survival path-C objects (the ones you get when
\code{scale \%in\% c("surv","chf")} was passed to the extractor) are
handed off to \code{\link{plot.gg_partial_rfsrc}} for drawing.
}
\details{
\strong{Ensemble mortality (scale = "mortality"):} when the provenance scale
is \code{"mortality"}, the y-axis is labelled
\emph{"Ensemble mortality (expected events)"}.  The wording is
deliberate: this is an \strong{unbounded relative-risk score}, not a
survival probability and not \eqn{1 - S(t)} (Ishwaran, Kogalur,
Blackstone & Lauer, 2008 \url{doi:10.1214/08-AOAS169}).
}
\section{Reading the partial dependence}{

For a continuous variable the x-axis is the variable's grid of values
and the y-axis is the partial prediction; each of the three effect
types (\code{parametric}, \code{nonparametric}, \code{causal}) is
drawn as its own line. The shape of the line is the story: a clear
slope says the model uses the variable, a flat line says it
essentially does not, and a U-shape or a threshold says the effect
is nonlinear in a way a single coefficient would miss. For a
categorical variable the picture is a boxplot per level; here the
eye is looking at level-to-level shifts in the centre of each box.

Where the three effect types track each other, the parametric story
is a fair summary of what the forest is doing. Where they fan
apart (typically the parametric curve smoother than the
nonparametric, or the causal curve flatter than either) the
variable is one to inspect more carefully before reading a single
effect off the plot.
}

\section{What this tells you}{

Use these curves to describe how the model uses each variable, not
to claim how the world works. They are a window into the fitted
relationship; they do not by themselves establish that intervening
on the variable would move the outcome. For survival path-C
(\code{scale = "surv"} or \code{"chf"}), the y-axis is on the
probability or cumulative-hazard scale, which is usually the scale
you want to report to a clinical audience.
}

\examples{
set.seed(42)
n_obs <- 30; n_pts <- 15
mock_data <- list(
  age = list(
    xvirtual    = seq(30, 80, length.out = n_pts),
    xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
  ),
  sex = list(
    xvirtual    = c(0, 1),
    xorg        = sample(c(0, 1), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
  )
)
pp <- gg_partial_varpro(mock_data)
plot(pp)
plot(pp, type = "parametric")

}
\references{
Ishwaran H, Kogalur UB, Blackstone EH, Lauer MS (2008).
Random survival forests. \emph{The Annals of Applied Statistics},
\bold{2}(3), 841--860. \doi{10.1214/08-AOAS169}.
}
\seealso{
\code{\link{gg_partial_varpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partial.R
\name{gg_partial}
\alias{gg_partial}
\title{Split partial dependence data into continuous or categorical datasets}
\usage{
gg_partial(part_dta, nvars = NULL, cat_limit = 10, model = NULL)
}
\arguments{
\item{part_dta}{partial plot data from \code{rfsrc::plot.variable}}

\item{nvars}{how many of the partial plot variables to calculate}

\item{cat_limit}{Categorical features are built when there are fewer than
\code{cat_limit} unique feature values.}

\item{model}{a label name applied to all features. Useful when combining
multiple partial plot objects in figures.}
}
\value{
A named list with two elements:
\describe{
\item{continuous}{data.frame with columns \code{x}, \code{yhat},
\code{name} (and optionally \code{model}) for continuous variables}
\item{categorical}{data.frame with the same columns but with \code{x}
as a factor, for low-cardinality / categorical variables}
}
}
\description{
A partial dependence curve answers a what-if question about a forest: hold
every other predictor at its observed value, sweep one of them across its
range, and watch how the ensemble prediction moves.  Marginalized over the
joint distribution of the other variables, the resulting curve isolates the
average effect of the swept predictor alone.
}
\details{
\code{gg_partial} handles the bookkeeping step after you've already called
\code{rfsrc::plot.variable(partial = TRUE)}: it takes the list that function
returns and separates the variables into two tidy data frames -- one for
continuous predictors (plotted as lines) and one for categorical predictors
(plotted as bar charts).  The split is controlled by \code{cat_limit}:
variables with more unique x-values than this threshold are treated as
continuous; all others are categorical.

If you'd rather skip the \code{plot.variable} step and pass the fitted
forest directly, see \code{\link{gg_partial_rfsrc}}, which calls
\code{partial.rfsrc} for you.
}
\note{
Partial-dependence extraction is \code{randomForestSRC}-only;
there is no \code{randomForest} method (the \code{randomForest} package
provides no comparable partial-dependence interface).
}
\examples{
## Build a small regression forest on the airquality dataset
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)

## Compute partial dependence via plot.variable (show.plots = FALSE to
## suppress the base-graphics output, we only want the data)
pv <- randomForestSRC::plot.variable(rf, partial = TRUE,
                                      show.plots = FALSE)

## Split into continuous and categorical data frames
result <- gg_partial(pv)
head(result$continuous)

## Label this model for later comparison with a second forest
result_labelled <- gg_partial(pv, model = "airq_model")
unique(result_labelled$continuous$model)

}
\seealso{
\code{\link{gg_partial_rfsrc}} \code{\link{gg_partialpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_rfsrc.R
\name{gg_rfsrc.rfsrc}
\alias{gg_rfsrc.rfsrc}
\alias{gg_rfsrc}
\title{Predicted response data object}
\usage{
\method{gg_rfsrc}{rfsrc}(object, oob = TRUE, by, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object.}

\item{oob}{Logical; if \code{TRUE} (default) return out-of-bag predictions.
Set to \code{FALSE} to use full in-bag (training) predictions. Forced to
\code{FALSE} automatically for \code{predict.rfsrc} objects, which carry
no OOB estimates.}

\item{by}{Optional stratifying variable. Either a character column name
present in the training data, or a vector/factor of the same length as
the training set. When supplied, a \code{group} column is added to the
returned data and bootstrap CI bands (survival) are computed per group.
Omit or leave missing to return an unstratified result.}

\item{...}{Additional arguments controlling output for specific forest
families:
\describe{
  \item{surv_type}{Character; one of \code{"surv"} (default),
    \code{"chf"}, or \code{"mortality"} for survival forests.}
  \item{conf.int}{Numeric coverage probability (e.g. \code{0.95}) to
    request bootstrap pointwise confidence bands for survival forests.
    Triggers wide-format output with \code{lower}, \code{upper},
    \code{median}, and \code{mean} columns.}
  \item{bs.sample}{Integer; number of bootstrap resamples when
    \code{conf.int} is set. Defaults to the number of observations.}
}}
}
\value{
A \code{gg_rfsrc} object (a classed \code{data.frame}) whose
  structure depends on the forest family:
  \describe{
    \item{regression}{Columns \code{yhat} and the response name; optionally
      a \code{group} column when \code{by} is supplied.}
    \item{classification}{One column per class with predicted probabilities;
      a \code{y} column with observed class labels; optionally \code{group}.}
    \item{survival (no CI / grouping)}{Long-format with columns
      \code{variable} (event time), \code{value} (survival probability),
      \code{obs_id}, and \code{event}.}
    \item{survival (with \code{conf.int} or \code{by})}{Wide-format with
      pointwise bootstrap CI columns (\code{lower}, \code{upper},
      \code{median}, \code{mean}) per time point; a \code{group} column
      when \code{by} is supplied.}
  }
  The object carries class attributes for the forest family so that
  \code{\link{plot.gg_rfsrc}} dispatches correctly.
}
\description{
Extracts the predicted response values from the
\code{\link[randomForestSRC]{rfsrc}} object, and formats data for plotting
the response using \code{\link{plot.gg_rfsrc}}.
}
\details{
For survival forests, use the \code{surv_type} argument
  (\code{"surv"}, \code{"chf"}, or \code{"mortality"}) to select the
  predicted quantity. Bootstrap confidence bands are requested by passing
  \code{conf.int} (e.g. \code{conf.int = 0.95}); the number of resamples
  is controlled by \code{bs.sample}.
}
\examples{
## ------------------------------------------------------------
## classification example (small, runs on CRAN)
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- rfsrc(Species ~ ., data = iris, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_iris)
plot(gg_dta)

\donttest{
## ------------------------------------------------------------
## Additional regression / survival examples are guarded with
## \donttest because the cumulative example time exceeds the
## 10-second CRAN budget. Run locally with `R CMD check --run-donttest`
## (or `devtools::check(run_dont_test = TRUE)`) to exercise them.
## ------------------------------------------------------------

## -------- air quality data (regression)
rfsrc_airq <- rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
plot(gg_rfsrc(rfsrc_airq))

## -------- Boston data (rfsrc + randomForest)
if (requireNamespace("MASS", quietly = TRUE)) {
  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas)
  rfsrc_boston <- rfsrc(medv ~ ., data = Boston, ntree = 50,
                        forest = TRUE, importance = TRUE,
                        tree.err = TRUE, save.memory = TRUE)
  plot(gg_rfsrc(rfsrc_boston))

  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
                                          ntree = 50)
  plot(gg_rfsrc(rf_boston))
}

## -------- mtcars data
rfsrc_mtcars <- rfsrc(mpg ~ ., data = mtcars, ntree = 50)
plot(gg_rfsrc(rfsrc_mtcars))

## -------- veteran data (survival; with CI and group-by)
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- rfsrc(Surv(time, status) ~ ., data = veteran,
                       ntree = 50)
plot(gg_rfsrc(rfsrc_veteran))
plot(gg_rfsrc(rfsrc_veteran, conf.int = .95))
plot(gg_rfsrc(rfsrc_veteran, by = "trt"))
}

}
\seealso{
\code{\link{plot.gg_rfsrc}},
  \code{\link[randomForestSRC]{rfsrc}},
  \code{\link{gg_survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_methods.R
\name{print.gg}
\alias{print.gg}
\alias{print.gg_error}
\alias{print.gg_vimp}
\alias{print.gg_rfsrc}
\alias{print.gg_variable}
\alias{print.gg_partial}
\alias{print.gg_partial_rfsrc}
\alias{print.gg_partialpro}
\alias{print.gg_roc}
\alias{print.gg_survival}
\alias{print.gg_brier}
\title{Print methods for gg_* data objects}
\usage{
\method{print}{gg_error}(x, ...)

\method{print}{gg_vimp}(x, ...)

\method{print}{gg_rfsrc}(x, ...)

\method{print}{gg_variable}(x, ...)

\method{print}{gg_partial}(x, ...)

\method{print}{gg_partial_rfsrc}(x, ...)

\method{print}{gg_partialpro}(x, ...)

\method{print}{gg_roc}(x, ...)

\method{print}{gg_survival}(x, ...)

\method{print}{gg_brier}(x, ...)
}
\arguments{
\item{x}{A \code{gg_*} data object.}

\item{...}{Not currently used.}
}
\value{
The object \code{x}, invisibly.
}
\description{
Each \code{print.gg_*()} method emits a single-line header containing the
class label and, when available, forest provenance metadata (source package,
family, ntree, n). The object is returned invisibly so \code{print()} calls
chain cleanly in pipes.
}
\details{
To inspect rows use \code{head()}.  To retrieve per-class diagnostics use
\code{\link{summary.gg}}.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
print(gg_error(rf))
print(gg_vimp(rf))

}
\seealso{
\code{\link{summary.gg}}, \code{\link{autoplot.gg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_error.R
\name{gg_error}
\alias{gg_error}
\alias{gg_error.rfsrc}
\alias{gg_error.randomForest}
\alias{gg_error.randomForest.formula}
\title{Random forest error trajectory data object}
\usage{
gg_error(object, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object.}

\item{...}{Optional arguments passed to the methods. Set
\code{training = TRUE} to append the in-bag error trajectory when
supported.}
}
\value{
A \code{gg_error} \code{data.frame} containing at least the
cumulative OOB error columns and an \code{ntree} counter. When
\code{training = TRUE} is honored an additional \code{train} column is
included.
}
\description{
Extract the cumulative out-of-bag (OOB) or in-bag training error rate from
\code{randomForestSRC} and \code{randomForest} fits as a function of the
number of grown trees.
}
\details{
For \code{randomForestSRC} objects the function reshapes the
\code{\link[randomForestSRC]{rfsrc}$err.rate} matrix and annotates it with
the tree index required by \code{\link{plot.gg_error}}. When supplied a
\code{\link[randomForest]{randomForest}} object, the method inspects either
the \code{$mse} or \code{$err.rate} component and, when
\code{training = TRUE} is requested, reconstructs the original training set
via the model call to compute an in-bag error curve using per-tree
predictions. Training curves are only available when the forest was stored
(\code{keep.forest = TRUE}) and the original data can be recovered.
}
\examples{
## Examples from RFSRC package...
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## ------------- iris data
## You can build a randomForest
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, tree.err = TRUE)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_iris)

# Plot the gg_error object
plot(gg_dta)

## RandomForest example
rf_iris <- randomForest::randomForest(Species ~ .,
  data = iris,
  tree.err = TRUE,
)
gg_dta <- gg_error(rf_iris)
plot(gg_dta)

gg_dta <- gg_error(rf_iris, training = TRUE)
plot(gg_dta)
## ------------------------------------------------------------
## Regression example
## ------------------------------------------------------------

## ------------- airq data
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ .,
  data = airquality,
  na.action = "na.impute", tree.err = TRUE,
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_airq)

# Plot the gg_error object
plot(gg_dta)


## ------------- Boston data
data(Boston, package = "MASS")
Boston$chas <- as.logical(Boston$chas)
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ .,
  data = Boston,
  forest = TRUE,
  importance = TRUE,
  tree.err = TRUE,
  save.memory = TRUE
)

# Get a data.frame containing error rates
gg_dta <- gg_error(rfsrc_boston)

# Plot the gg_error object
plot(gg_dta)


## ------------- mtcars data
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, tree.err = TRUE)

# Get a data.frame containing error rates
gg_dta<- gg_error(rfsrc_mtcars)

# Plot the gg_error object
plot(gg_dta)


## ------------------------------------------------------------
## Survival example
## ------------------------------------------------------------
## ------------- veteran data
## randomized trial of two treatment regimens for lung cancer
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                       tree.err = TRUE)

gg_dta <- gg_error(rfsrc_veteran)
plot(gg_dta)

## ------------- pbc data
# Load a cached randomForestSRC object
# We need to create this dataset
data(pbc, package = "randomForestSRC",)
# For whatever reason, the age variable is in days... makes no sense to me
for (ind in seq_len(dim(pbc)[2])) {
 if (!is.factor(pbc[, ind])) {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 } else {
   if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
     if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
     if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
       pbc[, ind] <- as.logical(pbc[, ind])
     }
   }
 }
 if (!is.logical(pbc[, ind]) &
     length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
   pbc[, ind] <- factor(pbc[, ind])
 }
}
#Convert age to years
pbc$age <- pbc$age / 364.24

pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]
# Create a test set from the remaining patients
pbc_test <- pbc[which(is.na(pbc$treatment)), ]

#========
# build the forest:
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
 dta_train,
 nsplit = 10,
 na.action = "na.impute",
 tree.err = TRUE,
 forest = TRUE,
 importance = TRUE,
 save.memory = TRUE
)


gg_dta <- gg_error(rfsrc_pbc)
plot(gg_dta)

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R,
Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{plot.gg_error}}, \code{\link{gg_vimp}},
\code{\link{gg_variable}},
\code{\link[randomForestSRC]{rfsrc}},
\code{\link[randomForest]{randomForest}},
\code{\link[randomForestSRC]{plot.rfsrc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_isopro.R
\name{plot.gg_isopro}
\alias{plot.gg_isopro}
\title{Plot a varPro isolation-forest anomaly score}
\usage{
\method{plot}{gg_isopro}(
  x,
  panel = c("both", "elbow", "density"),
  threshold = NULL,
  top_n_pct = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{gg_isopro} object from \code{\link{gg_isopro}}.}

\item{panel}{One of \code{"both"} (default: a \code{patchwork} of
elbow + density), \code{"elbow"}, or \code{"density"} (each returns a
single \code{ggplot}).}

\item{threshold}{Numeric in \code{[0, 1]}, or \code{NULL} (default). If
set, draws a reference line at that \code{howbad} value on the elbow
and density.}

\item{top_n_pct}{Numeric in \code{(0, 100)}, or \code{NULL} (default).
If set, resolves to the matching \code{howbad} quantile and draws the
same reference line. If both \code{threshold} and \code{top_n_pct}
are supplied, \code{threshold} wins with a \code{message()}.}

\item{...}{Currently unused.}
}
\value{
A \code{ggplot} (single panel) or a \code{patchwork}
(\code{panel = "both"}).
}
\description{
Renders a \code{gg_isopro} object as a ranked elbow (observations
sorted by anomaly score), a density of scores, or both side-by-side.
Optionally annotates a threshold either in score-space
(\code{threshold}) or in quantile-space (\code{top_n_pct}).
}
\section{Reading the elbow}{

The elbow plot is the canonical anomaly-detection picture. The x-axis
is observation rank (observations sorted from most ordinary to most
anomalous) and the y-axis is the \code{howbad} score. For a clean
population the curve sits flat near zero across the bulk of the data
and then bends sharply upward in the right tail; that bend is where
the anomalous observations live. The point of the plot is not to read
off a single score, it is to \emph{see where the curve breaks}. Pick a
cutoff there. Pass it back in as \code{threshold} (for a score) or
\code{top_n_pct} (for "the top 5\\%") and the plot draws a dashed
reference line so you can record the choice you made.
}

\section{Reading the density}{

The density panel is the same scores viewed as a distribution. A
single tight mode near zero with a long thin right tail is the
picture you hope for: bulk of the data ordinary, a few clear
anomalies. A bimodal density says you may have two populations rather
than one clean cluster plus outliers, and the cutoff question becomes
harder. Either way, this panel is a sanity check on what the elbow
suggests.
}

\section{Comparing methods}{

When the input object carries a \code{method} column (because you
bound several \code{\link{gg_isopro}} calls together), both panels
colour by method automatically. The point of comparing \code{"rnd"},
\code{"unsupv"}, and \code{"auto"} is not to pick a winner from the
figure alone, it is to see whether the methods agree on which
observations are anomalous. Curves that overlap in the right tail and
elbow at roughly the same rank are telling you the same story three
ways. Curves that diverge are telling you the score is
method-sensitive, which is itself useful information.
}

\seealso{
\code{\link{gg_isopro}}, \code{\link[varPro]{isopro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_ivarpro.R
\name{gg_ivarpro}
\alias{gg_ivarpro}
\title{Individual (local) variable importance from a varPro fit}
\usage{
gg_ivarpro(
  object,
  ...,
  which_obs = NULL,
  which_class = NULL,
  cutoff = NULL,
  ivarpro_fit = NULL
)
}
\arguments{
\item{object}{A \code{varpro} fit from \code{\link[varPro:varpro]{varPro::varpro()}} (regression or
classification family).}

\item{...}{Forwarded to \code{\link[varPro:ivarpro]{varPro::ivarpro()}} when \code{ivarpro_fit = NULL};
ignored otherwise (with a warning). Documented forwardables:
\code{adaptive}, \code{cut}, \code{cut.max}, \code{ncut}, \code{nmin}, \code{nmax}, \code{noise.na},
\code{max.rules.tree}, \code{max.tree}, \code{use.loo}, \code{use.abs}, \code{scale}.}

\item{which_obs}{Optional integer scalar - 1-based row index into the
training data. \code{NULL} (default) returns the aggregate view.}

\item{which_class}{Optional response level name. \code{NULL} default on a
binary classification fit resolves to the last factor level
(positive-class convention). Ignored with a warning on regression
fits.}

\item{cutoff}{Selection threshold on \verb{|local_imp|}. \code{NULL} (default)
resolves to the per-class \verb{mean(|local_imp|)} (or per-frame mean
for regression). A numeric scalar broadcasts. A named numeric
vector (names a subset of class levels) overrides per class with
fallback to the per-class mean for missing names.}

\item{ivarpro_fit}{Optional pre-computed \code{\link[varPro:ivarpro]{varPro::ivarpro()}} result
for the same \code{object}. Shape-validated.}
}
\value{
A \code{data.frame} of class \code{c("gg_ivarpro", "data.frame")}.
Regression: columns \code{obs / variable / local_imp / selected}.
Classification: long-format with an extra \code{class} column.
\code{variable} is a factor whose levels are set by
\verb{mean(|local_imp|)} descending across all rows (the unified
ranking axis shared across facets / panels).
}
\description{
Tidy wrapper around \code{\link[varPro:ivarpro]{varPro::ivarpro()}} for the regression or
classification family. Returns one row per (observation, variable)
pair where the local-importance cell is non-NA; classification adds
a \code{class} column. \code{which_obs} collapses to a per-observation
profile; \code{which_class} collapses to a single class. Optional
\code{ivarpro_fit} argument lets callers cache the expensive
\code{ivarpro()} call.
}
\note{
Multivariate regression (\verb{regr+}) and survival families are
out of scope for this release. The non-regression / non-class
path errors with a message naming v3.1.0 as the tracker.
}
\section{What this is doing}{

The varPro framework builds importance from release rules: for a given
rule region, it compares a local estimator inside that region to what
the estimator becomes after the constraint on the tested variable is
removed ("released"). That contrast is summed over many rules and trees
to get a global z-score: the quantity \code{\link[=gg_varpro]{gg_varpro()}} shows. What
\code{ivarpro()} adds is a per-observation view of the same mechanism.

Concretely: \code{ivarpro()} walks the forest's rules and, for each
(observation, variable) pair, computes a scaled per-rule contribution
to predicting that observation. Per-rule LOO removes the observation
from its own rule before scoring, so the contribution is not inflated
by the observation having helped define the region. Per-region scaling
(\code{scale = "local"}, default) standardises the contribution by the
rule's local response standard deviation so values are comparable
across rules of different size. Aggregating those per-rule scores into
one number per (obs, variable) pair gives the \code{local_imp} cell.

No permutation, no synthetic data: the contrast is always between real
subsets of the observed data, defined by the forest's own rules. This
is the same no-synthetic-features property that distinguishes
\code{\link[=gg_varpro]{gg_varpro()}} from \code{\link[=gg_vimp]{gg_vimp()}}'s Breiman-Cutler permutation importance.
}

\section{What \code{local_imp} actually is (pedantic)}{

\code{local_imp[i, v]} is the \strong{scaled aggregated rule contribution} of
variable \code{v} to predicting observation \code{i}, NOT a permutation
importance and NOT a SHAP value. \strong{Sign carries direction} of the
local response shift inside the rule's region. \strong{Magnitude is on
the response scale} when \code{scale = "global"}, or unit-free when
\code{scale = "local"} (the default). The matrix is \strong{heavily sparse} -
an observation contributes only to rules that retain it as OOB; on
real data, per-variable NA fractions of 50-95\% are common.
Comparison with \code{gg_varpro()} (aggregate split-strength) and
\code{gg_beta_varpro()} (per-rule lasso beta) is diagnostic: a variable
that's important globally but has low per-observation contribution
for a specific case is interesting; the inverse - high local but
low global - flags a regime-specific signal.
}

\section{What's in the output}{

Long-format tidy frame. Regression has columns \code{obs}, \code{variable},
\code{local_imp}, \code{selected}. Classification adds a \code{class} column
(factor in response-level order). \code{variable} is a factor whose
levels are set by \verb{mean(|local_imp|)} descending across all rows;
for classification that aggregate is across all (obs, class) so
every facet / panel shows variables in the same row order. NA
cells are filtered out - the source matrix is sparse, and the
tidy frame only carries the cells where local importance is
defined.

Provenance attribute carries \code{source}, \code{family}, \code{ntree}, \code{cutoff}
(named numeric vector - length 1 named \code{"regr"} for regression,
length K named with class levels for classification),
\code{cutoff_default}, \code{use.loo}, \code{scale}, \code{n_train}, \code{n_obs}, \code{n_var},
\code{precomputed}, \code{xvar.names}, \code{class_levels} (classification only),
\code{which_obs}, \code{which_class}.
}

\section{What you use this for}{

Per-observation interpretation ("which variables drive \emph{this}
prediction?"), variable-selection diagnostics via the aggregate
distribution view, and side-by-side comparison against
\code{\link[=gg_varpro]{gg_varpro()}} / \code{\link[=gg_beta_varpro]{gg_beta_varpro()}} to spot variables that matter
locally but not globally (or vice versa).
}

\section{Caching}{

\code{ivarpro()} is \strong{the most expensive call in varPro} (per-rule
leave-one-out + per-region scaling, often minutes on real data).
Compute it once and reuse:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{v   <- varPro::varpro(medv ~ ., data = Boston, ntree = 200)
iv  <- varPro::ivarpro(v, scale = "local")              # expensive, once
gg_aggregate <- gg_ivarpro(v, ivarpro_fit = iv)          # cheap
gg_case1     <- gg_ivarpro(v, ivarpro_fit = iv, which_obs = 1L)
}\if{html}{\out{</div>}}

Provenance carries \code{precomputed = TRUE} when \code{ivarpro_fit} was supplied.
}

\section{Classification}{

For a classification fit, \code{ivarpro()} returns a list of K matrices
(one per class) for multi-class, or a flat data.frame for binary
(positive-class importances only - the wrapper normalises this to
a single-element list under the last factor level). The wrapper
stacks per-class frames into a long-format frame with a \code{class}
column. \code{which_class = NULL} returns all classes (binary defaults
to the last factor level, the positive-class convention used by
\code{glm} and \code{gg_roc}); \code{which_class = "<name>"} filters to a single
class. \code{cutoff} polymorphism mirrors \code{\link[=gg_beta_varpro]{gg_beta_varpro()}} - \code{NULL}
is per-class mean(|local_imp|), a scalar broadcasts, a named
numeric vector overrides per class with fallback to that class's
mean.
}

\section{Reproducibility}{

Byte-for-byte agreement between cached (\code{ivarpro_fit = iv}) and
uncached (\code{ivarpro_fit = NULL}) outputs requires reusing the same
\code{ivarpro()} result. \code{set.seed()} alone is not sufficient because
per-rule LOO subsampling can drift across separate calls. Reuse
\code{ivarpro_fit} when reproducibility matters.
}

\examples{
\donttest{
if (requireNamespace("varPro", quietly = TRUE) &&
    requireNamespace("MASS", quietly = TRUE)) {
  set.seed(1)
  v <- varPro::varpro(medv ~ ., data = MASS::Boston, ntree = 50)
  iv <- varPro::ivarpro(v)
  gg <- gg_ivarpro(v, ivarpro_fit = iv)
  plot(gg)
}
}

}
\seealso{
\code{\link[=gg_varpro]{gg_varpro()}}, \code{\link[=gg_vimp]{gg_vimp()}}, \code{\link[=gg_beta_varpro]{gg_beta_varpro()}}, \code{\link[varPro:ivarpro]{varPro::ivarpro()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_partial.rfsrc.R
\name{surv_partial.rfsrc}
\alias{surv_partial.rfsrc}
\title{Survival partial dependence data for one or more predictors}
\usage{
surv_partial.rfsrc(rforest, var_list, npts = 25, partial.type = "surv")
}
\arguments{
\item{rforest}{A fitted \code{\link[randomForestSRC]{rfsrc}} survival or
competing-risk forest object.}

\item{var_list}{Character vector of predictor names for which partial
dependence should be computed. Each must appear in
\code{rforest$xvar.names}.}

\item{npts}{Integer; the number of predictor grid points to evaluate
(default 25). Evenly-spaced unique values are sampled from each predictor.}

\item{partial.type}{The prediction type to return. For survival forests one
of \code{"surv"} (default), \code{"mort"}, or \code{"chf"}. For competing
risk forests one of \code{"years.lost"}, \code{"cif"}, or \code{"chf"}.
See \code{\link[randomForestSRC]{partial.rfsrc}} for full details.}
}
\value{
A named list with one element per variable in \code{var_list}. Each
  element is itself a list with:
  \describe{
    \item{name}{The predictor variable name (character).}
    \item{dta}{The raw output of
      \code{\link[randomForestSRC]{get.partial.plot.data}}, a list containing
      at minimum \code{x} (predictor values) and \code{yhat} (partial
      predictions), and for survival/competing risk, \code{partial.time}.}
  }
}
\description{
\strong{Deprecated.} Use \code{\link{gg_partial_rfsrc}} instead, which
returns a classed \code{gg_partial_rfsrc} object with a dedicated
\code{plot()} method.
}
\details{
Computes partial dependence curves for a survival or competing-risk
\code{\link[randomForestSRC]{rfsrc}} forest by calling
\code{\link[randomForestSRC]{partial.rfsrc}} at \code{npts} evenly-spaced
unique values of each predictor across all stored event times.
}
\examples{
## ------------------------------------------------------------
## survival
## ------------------------------------------------------------

data(veteran, package = "randomForestSRC")
v.obj <- randomForestSRC::rfsrc(Surv(time,status)~.,
  veteran, nsplit = 10, ntree = 100)

spart <- surv_partial.rfsrc(v.obj, var_list="age", partial.type = "mort")

## partial effect of age on mortality
partial.obj <- partial(v.obj,
                       partial.type = "mort",
                       partial.xvar = "age",
                       partial.values = v.obj$xvar$age,
                       partial.time = v.obj$time.interest)
pdta <- get.partial.plot.data(partial.obj)

plot(lowess(pdta$x, pdta$yhat, f = 1/3),
     type = "l", xlab = "age", ylab = "adjusted mortality")

## example where x is discrete - partial effect of age on mortality
## we use the granule=TRUE option
partial.obj <- partial(v.obj,
                       partial.type = "mort",
                       partial.xvar = "trt",
                       partial.values = v.obj$xvar$trt,
                       partial.time = v.obj$time.interest)
pdta <- get.partial.plot.data(partial.obj, granule = TRUE)
boxplot(pdta$yhat ~ pdta$x, xlab = "treatment", ylab = "partial effect")


## partial effects of karnofsky score on survival
karno <- quantile(v.obj$xvar$karno)
partial.obj <- partial(v.obj,
                       partial.type = "surv",
                       partial.xvar = "karno",
                       partial.values = karno,
                       partial.time = v.obj$time.interest)
pdta <- get.partial.plot.data(partial.obj)

matplot(pdta$partial.time, t(pdta$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "karnofsky adjusted survival")
legend("topright", legend = paste0("karnofsky = ", karno), fill = 1:5)


## ------------------------------------------------------------
## competing risk
## ------------------------------------------------------------

data(follic, package = "randomForestSRC")
follic.obj <- rfsrc(Surv(time, status) ~ ., follic, nsplit = 3, ntree = 100)

## partial effect of age on years lost
partial.obj <- partial(follic.obj,
                       partial.type = "years.lost",
                       partial.xvar = "age",
                       partial.values = follic.obj$xvar$age,
                       partial.time = follic.obj$time.interest)
pdta1 <- get.partial.plot.data(partial.obj, target = 1)
pdta2 <- get.partial.plot.data(partial.obj, target = 2)

# Save and restore the user's graphical parameters per CRAN policy.
oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar))
par(mfrow = c(2, 2))
plot(lowess(pdta1$x, pdta1$yhat),
     type = "l", xlab = "age", ylab = "adjusted years lost relapse")
plot(lowess(pdta2$x, pdta2$yhat),
     type = "l", xlab = "age", ylab = "adjusted years lost death")

## partial effect of age on cif
partial.obj <- partial(follic.obj,
                       partial.type = "cif",
                       partial.xvar = "age",
                       partial.values = quantile(follic.obj$xvar$age),
                       partial.time = follic.obj$time.interest)
pdta1 <- get.partial.plot.data(partial.obj, target = 1)
pdta2 <- get.partial.plot.data(partial.obj, target = 2)

matplot(pdta1$partial.time, t(pdta1$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "age adjusted cif for relapse")
matplot(pdta2$partial.time, t(pdta2$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "age adjusted cif for death")

}
\seealso{
\code{\link{gg_partial_rfsrc}},
  \code{\link[randomForestSRC]{partial.rfsrc}},
  \code{\link[randomForestSRC]{get.partial.plot.data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_rfsrc.R
\name{plot.gg_rfsrc}
\alias{plot.gg_rfsrc}
\title{Predicted response plot from a \code{\link{gg_rfsrc}} object.}
\usage{
\method{plot}{gg_rfsrc}(x, notch = TRUE, ...)
}
\arguments{
\item{x}{A \code{\link{gg_rfsrc}} object, or a raw
\code{\link[randomForestSRC]{rfsrc}} object (which will be passed through
\code{\link{gg_rfsrc}} automatically before plotting).}

\item{notch}{Logical; whether to draw notched boxplots for regression and
classification forests (default \code{TRUE}).  Set \code{notch = FALSE}
to suppress notches when sample sizes are too small for reliable
confidence intervals on the median.}

\item{...}{Additional arguments forwarded to the underlying
\code{ggplot2} geometry calls.  Commonly useful arguments include:
\describe{
\item{\code{alpha}}{Numeric in \eqn{[0,1]}; point/ribbon transparency.
For survival plots with confidence bands the ribbon alpha is
automatically halved relative to the value supplied here.}
\item{\code{size}}{Point or line size passed to \code{geom_jitter},
\code{geom_step}, etc.}
}
Arguments that control \code{\link{gg_rfsrc}} (e.g. \code{conf.int},
\code{surv_type}, \code{by}) should be applied when constructing the
\code{gg_rfsrc} object before calling \code{plot()}.}
}
\value{
A \code{ggplot} object.  The plot appearance depends on the forest
family stored in \code{x}:
\describe{
\item{Regression (\code{"regr"})}{Jitter + notched boxplot of OOB
predicted values.  If a \code{group} column is present the x-axis
shows each group label; otherwise observations are collapsed to a
single x-position.}
\item{Classification (\code{"class"})}{Binary: jitter + notched
boxplot of the predicted class probability.  Multi-class: jitter
plot with one panel per class (class probabilities in long form).}
\item{Survival (\code{"surv"})}{Step curves of the ensemble survival
function.  When \code{gg_rfsrc} was called with \code{conf.int},
a shaded ribbon is added.  When called with \code{by}, curves are
coloured by group.}
}
}
\description{
Visualizes the ensemble predictions extracted by \code{\link{gg_rfsrc}}.
By default those are out-of-bag (OOB) predictions -- the forest's built-in
cross-validation estimate, averaging only over the trees that left a given
observation out of their bootstrap sample.
}
\details{
The geometry adapts to the forest family.  For regression or
classification, you get a jitter-and-boxplot: every observation is a dot
placed at its OOB predicted value, coloured by observed response, with a
notched boxplot overlaid to show the central tendency and spread.  For a
survival forest, each observation contributes one ensemble survival curve
(or CHF / mortality, whichever \code{surv_type} was chosen in
\code{gg_rfsrc}); the bundle of step functions shows the spread of
predicted risk across the cohort.  Pass \code{by} to \code{gg_rfsrc}
beforehand to colour curves by a predictor group, or \code{conf.int} to
replace the individual curves with a mean curve and bootstrap ribbon.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
# Build a small classification forest (ntree=50 keeps example fast)
set.seed(42)
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_iris)

plot(gg_dta)

## ------------------------------------------------------------
## Regression example
## ------------------------------------------------------------
## -------- air quality data
# na.action = "na.impute" handles missing Ozone / Solar.R values
set.seed(42)
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_airq)

plot(gg_dta)

## -------- mtcars data
set.seed(42)
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_mtcars)

plot(gg_dta)

## ------------------------------------------------------------
## Survival example
## ------------------------------------------------------------
## -------- veteran data
## randomized trial of two treatment regimens for lung cancer
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_veteran)
plot(gg_dta)

# With 95\% pointwise bootstrap confidence bands
gg_dta <- gg_rfsrc(rfsrc_veteran, conf.int = .95)
plot(gg_dta)

# Stratified by treatment arm
gg_dta <- gg_rfsrc(rfsrc_veteran, by = "trt")
plot(gg_dta)

## -------- pbc data (larger dataset -- skipped on CRAN)
\donttest{
data(pbc, package = "randomForestSRC")
# For whatever reason, the age variable is in days; convert to years
for (ind in seq_len(dim(pbc)[2])) {
  if (!is.factor(pbc[, ind])) {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  } else {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  }
  if (!is.logical(pbc[, ind]) &
    length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
    pbc[, ind] <- factor(pbc[, ind])
  }
}
# Convert age from days to years
pbc$age <- pbc$age / 364.24
pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
# Remove test-set patients (those with no assigned treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]

set.seed(42)
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
  dta_train,
  nsplit = 10,
  na.action = "na.impute",
  forest = TRUE,
  importance = TRUE,
  save.memory = TRUE
)

gg_dta <- gg_rfsrc(rfsrc_pbc)
plot(gg_dta)

gg_dta <- gg_rfsrc(rfsrc_pbc, conf.int = .95)
plot(gg_dta)

gg_dta <- gg_rfsrc(rfsrc_pbc, by = "treatment")
plot(gg_dta)
}

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for
R, Rnews, 7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_rfsrc}} \code{\link[randomForestSRC]{rfsrc}}
\code{\link[randomForest]{randomForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varpro_feature_names.R
\name{varpro_feature_names}
\alias{varpro_feature_names}
\title{Recover original variable names from varpro one-hot encoded feature names}
\usage{
varpro_feature_names(varpro_names, dataset)
}
\arguments{
\item{varpro_names}{character vector of names as output by varpro (may
include one-hot encoded suffixed names such as \code{"sex0"}, \code{"sex1"})}

\item{dataset}{the original data frame passed to varpro, used to look up
valid column names}
}
\value{
character vector of unique original variable names (no suffixes)
}
\description{
\code{varpro} one-hot encodes factor variables, appending a numeric suffix
for each level (e.g., \code{sex} becomes \code{sex0} and \code{sex1}).
This function strips those suffixes iteratively until every name in
\code{varpro_names} can be matched back to a column in \code{dataset}.
}
\examples{
## ------------------------------------------------------------------
## Simple case: one continuous variable + one binary factor
## ------------------------------------------------------------------
ds <- data.frame(age = c(25, 30, 45), sex = c("M", "F", "M"))

# varpro one-hot encodes 'sex' into 'sex0' and 'sex1'
varpro_names <- c("age", "sex0", "sex1")
varpro_feature_names(varpro_names, ds)
# Returns: c("age", "sex")

## ------------------------------------------------------------------
## Multi-level factor: three-level 'group' variable
## ------------------------------------------------------------------
ds2 <- data.frame(score = 1:6,
                  group = factor(rep(c("A", "B", "C"), 2)))

# varpro appends 0/1/2 for each level
vn2 <- c("score", "group0", "group1", "group2")
varpro_feature_names(vn2, ds2)
# Returns: c("score", "group")

## ------------------------------------------------------------------
## Already-clean names pass through unchanged
## ------------------------------------------------------------------
ds3 <- data.frame(x = 1:5, y = 1:5)
varpro_feature_names(c("x", "y"), ds3)
# Returns: c("x", "y")

}
\seealso{
\code{\link{gg_partialpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nelson.R
\name{nelson}
\alias{nelson}
\title{nonparametric Nelson-Aalen estimates}
\usage{
nelson(interval, censor, data, by = NULL, weight = NULL, ...)
}
\arguments{
\item{interval}{name of the interval variable in the training dataset.}

\item{censor}{name of the censoring variable in the training dataset.}

\item{data}{name of the survival training data.frame}

\item{by}{stratifying variable in the training dataset, defaults to NULL}

\item{weight}{for each observation (default=NULL)}

\item{...}{arguments passed to the \code{survfit} function}
}
\value{
\code{\link{gg_survival}} object
}
\description{
nonparametric Nelson-Aalen estimates
}
\examples{
# These get run through the gg_survival examples.
data(pbc, package = "randomForestSRC")
pbc$time <- pbc$days / 364.25

# This is the same as gg_survival
gg_dta <- nelson(
  interval = "time", censor = "status",
  data = pbc
)

plot(gg_dta, error = "none")
plot(gg_dta)

# Stratified on treatment variable.
gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment"
)

plot(gg_dta, error = "none")
plot(gg_dta, error = "lines")
plot(gg_dta)

gg_dta <- gg_survival(
  interval = "time", censor = "status",
  data = pbc, by = "treatment",
  type = "nelson"
)

plot(gg_dta, error = "bars")
plot(gg_dta)

}
\seealso{
\code{\link{gg_survival}} \code{\link{nelson}}
\code{\link{plot.gg_survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_variable.R
\name{plot.gg_variable}
\alias{plot.gg_variable}
\title{Plot a \code{\link{gg_variable}} object,}
\usage{
\method{plot}{gg_variable}(
  x,
  xvar,
  time,
  time_labels,
  panel = FALSE,
  oob = TRUE,
  points = TRUE,
  smooth = TRUE,
  ...
)
}
\arguments{
\item{x}{\code{\link{gg_variable}} object created from a
\code{\link[randomForestSRC]{rfsrc}} object}

\item{xvar}{variable (or list of variables) of interest.}

\item{time}{For survival, one or more times of interest}

\item{time_labels}{string labels for times}

\item{panel}{Should plots be faceted along multiple xvar?}

\item{oob}{oob estimates (boolean)}

\item{points}{plot the raw data points (boolean)}

\item{smooth}{include a smooth curve (boolean)}

\item{...}{arguments passed to the \code{ggplot2} functions.}
}
\value{
A single \code{ggplot} object when \code{length(xvar) == 1} or
\code{panel = TRUE}; otherwise a \code{patchwork} composite that stacks
one panel per variable in \code{xvar}. Either way the result is one
plottable object, never a bare list, so it composes with
\code{patchwork} and dispatches through \code{ggplot2::autoplot()}.
For the patchwork case, to inspect one panel with
\code{ggplot2::layer_data()} pull that panel out first
(e.g. \code{ggplot2::layer_data(p[[1]])}).
}
\description{
Plot a \code{\link{gg_variable}} object,
}
\examples{
## ------------------------------------------------------------
## classification
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)

gg_dta <- gg_variable(rfsrc_iris)
plot(gg_dta, xvar = "Sepal.Width")
plot(gg_dta, xvar = "Sepal.Length")

## Panel plot across all predictors
plot(gg_dta,
  xvar = rfsrc_iris$xvar.names,
  panel = TRUE, se = FALSE
)

## ------------------------------------------------------------
## regression
## ------------------------------------------------------------
## -------- air quality data
# na.action = "na.impute" handles missing Ozone / Solar.R values
set.seed(42)
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
gg_dta <- gg_variable(rfsrc_airq)

# Treat Month as an ordinal factor for better visualisation
gg_dta[, "Month"] <- factor(gg_dta[, "Month"])

plot(gg_dta, xvar = "Wind")
plot(gg_dta, xvar = "Temp")
plot(gg_dta, xvar = "Solar.R")

# Factor variable uses notched boxplots
plot(gg_dta, xvar = "Month", notch = TRUE)

\donttest{
# Panel plot across continuous predictors (loess smooths; slower)
plot(gg_dta, xvar = c("Solar.R", "Wind", "Temp", "Day"), panel = TRUE)
}

## ------------------------------------------------------------
## survival examples
## ------------------------------------------------------------
## -------- veteran data
\donttest{
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., veteran,
  nsplit = 10,
  ntree = 50
)

# Marginal survival at 90 days
gg_dta <- gg_variable(rfsrc_veteran, time = 90)

# Single-variable dependence plots
plot(gg_dta, xvar = "age")
plot(gg_dta, xvar = "diagtime")

# Panel coplot for two predictors at a single time
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)

# Compare survival at 30, 90, and 365 days simultaneously
gg_dta <- gg_variable(rfsrc_veteran, time = c(30, 90, 365))

# Single-variable plot (one facet per time point)
plot(gg_dta, xvar = "age")

# Panel coplot across two predictors and three time points
plot(gg_dta, xvar = c("age", "diagtime"), panel = TRUE)
}

}
\references{
Breiman L. (2001). Random forests, Machine Learning, 45:5-32.

Ishwaran H. and Kogalur U.B. (2007). Random survival forests for R, Rnews,
7(2):25-31.

Ishwaran H. and Kogalur U.B. randomForestSRC: Random Forests for Survival,
Regression and Classification. R package version >= 3.4.0.
\url{https://cran.r-project.org/package=randomForestSRC}
}
\seealso{
\code{\link{gg_variable}}, \code{\link{gg_partial}},
\code{\link[randomForestSRC]{plot.variable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_udependent.R
\name{plot.gg_udependent}
\alias{plot.gg_udependent}
\title{Plot a \code{gg_udependent} variable dependency graph}
\usage{
\method{plot}{gg_udependent}(x, layout = "fr", ...)
}
\arguments{
\item{x}{A \code{gg_udependent} object from \code{\link{gg_udependent}}.}

\item{layout}{Character; the igraph/ggraph layout algorithm.  Common
choices are \code{"fr"} (Fruchterman-Reingold, the default),
\code{"kk"} (Kamada-Kawai), \code{"stress"}, \code{"circle"}, and
\code{"grid"}.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} object (built via ggraph).
}
\description{
Draws the dependency graph held in a \code{gg_udependent} object as a
ggraph network.  Node colour marks whether a variable made the signal
set, and the width and opacity of an edge tell you how strong the
dependency between its two variables is.
}
\details{
This plot needs the \pkg{ggraph} package, which is in \code{Suggests}
rather than installed for you.  If it is missing, run
\code{install.packages("ggraph")}.

A signal variable (\code{selected = TRUE}) gets a blue node
(\code{#4e8fcd}); the rest are grey (\code{#888888}).  Node size grows
with degree.  Edge width and opacity both grow with the raw dependency
weight \code{I[i,j]}.
}
\section{Reading the network}{

Each node is a variable; each edge is a cross-variable dependency
that cleared the threshold passed to \code{gg_udependent}. The
Fruchterman-Reingold layout (the default) places mutually connected
variables near each other, so the picture tends to show hubs and
the clusters around them rather than a tidy ring. The eye usually
goes first to the largest blue node: a variable that is both in
the signal set and connects to many others is a hub of the
dependency structure. Edges with wider, more opaque strokes are
stronger dependencies; thin, faint edges sit near the threshold and
are the ones that would disappear first if you raised it.

Grey, low-degree nodes are the ones UVarPro thinks are not
contributing much to the structure. (Truly isolated nodes are
dropped by \code{gg_udependent()} before the graph is drawn; what you
see is the connected component.) A cluster of mutually
connected variables is worth checking for redundancy; they may be
several views of the same underlying quantity.
}

\section{What this tells you}{

Use the figure to pick a working set of variables: the hubs and
their immediate neighbours are the candidates UVarPro flags as
carrying structure. If a cluster of high-degree variables looks
like it might be measuring the same thing, that is a cue to look at
their pairwise correlations or fit them as a block rather than
individually. The threshold and layout are recorded in the caption
so a different choice is easy to spot in a later figure.
}

\examples{
\donttest{
if (requireNamespace("ggraph", quietly = TRUE)) {
  set.seed(42)
  uv <- varPro::uvarpro(iris[, -5], ntree = 50)
  plot(gg_udependent(uv))
}
}

}
\seealso{
\code{\link{gg_udependent}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_isopro.R
\name{gg_isopro}
\alias{gg_isopro}
\title{Tidy data from a varPro isolation-forest fit}
\usage{
gg_isopro(object, ..., newdata = NULL)
}
\arguments{
\item{object}{An \code{isopro} fit returned by
\code{\link[varPro]{isopro}}.}

\item{...}{Currently unused. Present before \code{newdata} so that
\code{newdata} is only matched by name, preserving backward
compatibility with callers of the PR #94 signature
\code{gg_isopro(object, ...)}.}

\item{newdata}{Optional \code{data.frame} of new observations to score
against the fit. Must be passed by name. When \code{NULL} (default)
the extractor returns the in-sample tidy frame from the fit's stored
\code{$case.depth} and \code{$howbad}. When supplied, each row is
scored via \code{\link[varPro]{predict.isopro}} and the same tidy
shape is returned for the test data.}
}
\value{
A \code{data.frame} of class \code{c("gg_isopro", "data.frame")},
one row per observation. Columns:
\describe{
\item{obs}{Integer; observation index \code{1..n}, in the same
order as the rows of the data passed to
\code{\link[varPro]{isopro}}.}
\item{case.depth}{Numeric; mean isolation depth across the forest.
Lower means the observation was isolated quickly, so more
anomalous.}
\item{howbad}{Numeric in \code{[0, 1]}; the \code{case.depth}
values pushed through their own empirical CDF and flipped so
higher means more anomalous. This is the score the plot method
draws by default.}
}
A \code{provenance} attribute records
\code{source = "varPro::isopro"}, the observation count \code{n}, and
the number of trees \code{ntree}.
}
\description{
Pulls per-observation anomaly scores out of a \code{\link[varPro]{isopro}}
fit so you can plot them, sort them, or write them to disk without having
to know the internal shape of the fit.
}
\section{What isopro is doing}{

An isolation forest (Liu, Ting and Zhou 2008) is a random forest grown
on very small subsamples of the data and asked to split until each
observation lands in its own terminal node. The intuition is geometric:
a typical observation sits in the dense middle of the feature cloud and
takes many splits to isolate, while an unusual observation sits out
near an edge and gets cut off after only a few. So \strong{the depth at
which an observation is isolated is a proxy for how typical it is}:
shallow depth means anomalous, deep depth means ordinary. Average a
single observation's depth across many trees and the noise washes out,
leaving a stable per-observation rank.

\code{\link[varPro]{isopro}} supports three flavours of isolation
forest, which differ in how the splits are chosen:
\describe{
\item{\code{"rnd"}}{The original Liu/Ting/Zhou method: each tree node
picks a variable at random and a split point uniformly at random
in the variable's range. Fast, no model, surprisingly effective.}
\item{\code{"unsupv"}}{Unsupervised splitting from
\code{randomForestSRC}: splits are chosen to separate the data
along the directions of highest variance. More structured than
\code{"rnd"}; sometimes more accurate, especially when the
anomalies follow a coherent direction.}
\item{\code{"auto"}}{An auto-encoder formulation that grows a
multivariate forest predicting each feature from the others. Most
expressive, slowest, best suited to low-dimensional data.}
}
No method is universally best. The varPro authors recommend trying at
least two and comparing the score distributions; the plot method here
colours per-method curves automatically when you stack the results.
}

\section{What's in the output}{

The fit gives back two parallel per-observation vectors:
\code{case.depth} is the raw mean isolation depth (units of "splits",
lower = more anomalous) and \code{howbad} is the same information
transformed onto a \code{[0, 1]} scale via the empirical CDF of
\code{case.depth} (higher = more anomalous). Both columns are kept so
you can plot in either space and have the raw depth on hand for
diagnostics; \code{howbad} is the canonical score and is what the plot
method uses by default.
}

\section{What you use this for}{

This is screening, not inference. Reach for it when you want to:
\itemize{
\item flag observations that may be data-entry errors, out-of-range
measurements, or distinct subpopulations before fitting a primary
model;
\item check whether a held-out cohort sits inside the training
distribution before scoring with a model trained elsewhere;
\item give the analyst a ranked list of "look at these first" cases
for a manual review;
\item score a held-out cohort or a fresh batch of incoming data
against a fitted model and compare the test scores to the training
distribution.
}
The score is a \emph{rank}, not a probability of being an outlier: two
observations with \code{howbad = 0.92} are both unusual, not "92\\%
likely to be anomalous". Pick a cutoff by looking at where the elbow
rises; \code{\link{plot.gg_isopro}} can annotate either a score
(\code{threshold}) or a top-percent (\code{top_n_pct}) for you.
}

\section{Scoring new data}{

Pass a \code{data.frame} as \code{newdata} and the extractor calls
\code{\link[varPro]{predict.isopro}} twice: once with
\code{quantiles = FALSE} to get the raw mean case depth per row, and once
with \code{quantiles = TRUE} to get the per-row quantile of that depth
against the training-data depth distribution.

varPro's \code{predict.isopro} returns quantiles where \emph{smaller is
more anomalous}, which is the opposite polarity of the wrapper's
\code{howbad} (where \emph{higher} is more anomalous). The wrapper
exposes both conventions so nothing is hidden:
\itemize{
\item \code{case.depth} carries varPro's native polarity, \emph{lower
= more anomalous}. This is the unmodified output of
\code{predict(object, newdata, quantiles = FALSE)}. Use it to
cross-reference against raw varPro output.
\item \code{howbad} is the flipped, wrapper-convention version. The
relationship is \code{howbad = 1 - predict(object, newdata, quantiles = TRUE)}.
}

To overlay training and test scores in one plot, bind the two extractor
calls with a \code{method} label column (the same column
\code{\link{plot.gg_isopro}} uses to colour rnd / unsupv / auto
comparisons):

\preformatted{
gg_train <- gg_isopro(fit)
gg_test  <- gg_isopro(fit, newdata = test_df)
gg_both  <- rbind(cbind(gg_train, method = "train"),
                  cbind(gg_test,  method = "test"))
class(gg_both) <- c("gg_isopro", "data.frame")
plot(gg_both)
}
}

\section{Comparing methods}{

To compare methods (\code{"rnd"}, \code{"unsupv"}, \code{"auto"}), call
\code{\link{gg_isopro}} on each fit and \code{dplyr::bind_rows()} the
results with a \code{method} label column. The plot method auto-detects
\code{method} and colours the curves.
}

\examples{
\donttest{
if (requireNamespace("varPro", quietly = TRUE)) {
  set.seed(1)
  fit <- varPro::isopro(data = iris[, 1:4], method = "rnd",
                        sampsize = 32, ntree = 50)
  gg <- gg_isopro(fit)
  plot(gg)
}
}

}
\references{
Liu, F. T., Ting, K. M., and Zhou, Z. H. (2008). Isolation Forest.
\emph{Eighth IEEE International Conference on Data Mining}, 413-422.

Ishwaran, H., Mantero, A., and Lu, M. (2025). varPro: Model-Independent
Variable Selection via the Rule-Based Variable Priority Framework.
\emph{R package version 3.x}.
}
\seealso{
\code{\link{plot.gg_isopro}}, \code{\link[varPro]{isopro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_vimp.R
\name{gg_vimp}
\alias{gg_vimp}
\alias{gg_vimp.rfsrc}
\alias{gg_vimp.randomForest}
\alias{gg_vimp.randomForest.formula}
\title{Variable Importance (VIMP) data object}
\usage{
gg_vimp(object, nvar, ...)
}
\arguments{
\item{object}{A \code{\link[randomForestSRC]{rfsrc}} object, the output from
\code{\link[randomForestSRC]{vimp}}, or a fitted
\code{\link[randomForest]{randomForest}}.}

\item{nvar}{argument to control the number of variables included in the
output.}

\item{...}{arguments passed to the \code{\link[randomForestSRC]{vimp.rfsrc}}
function if the \code{\link[randomForestSRC]{rfsrc}} object does not contain
importance information.}
}
\value{
\code{gg_vimp} object. A \code{data.frame} of VIMP measures, in rank
  order, optionally containing class-specific scores and a relative importance
  column. When \code{randomForest} objects lack stored importance values a
  warning is issued and \code{NA} placeholders are returned so plots remain
  reproducible.
}
\description{
\code{gg_vimp} Extracts the variable importance (VIMP) information from a
\code{\link[randomForestSRC]{rfsrc}} or \code{\link[randomForest]{randomForest}}
object and reshapes it into a tidy data set.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- rfsrc(Species ~ .,
  data = iris,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_iris)
plot(gg_dta)

## ------------------------------------------------------------
## regression example
## ------------------------------------------------------------

## -------- air quality data
rfsrc_airq <- rfsrc(Ozone ~ ., airquality,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_airq)
plot(gg_dta)


## -------- Boston data
data(Boston, package = "MASS")
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_boston)
plot(gg_dta)

## -------- Boston data
rf_boston <- randomForest::randomForest(medv ~ ., Boston)
gg_dta <- gg_vimp(rf_boston)
plot(gg_dta)


## -------- mtcars data
rfsrc_mtcars <- rfsrc(mpg ~ .,
  data = mtcars,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_mtcars)
plot(gg_dta)

## ------------------------------------------------------------
## survival example
## ------------------------------------------------------------

## -------- veteran data
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- rfsrc(Surv(time, status) ~ .,
  data = veteran,
  ntree = 100,
  importance = TRUE
)

gg_dta <- gg_vimp(rfsrc_veteran)
plot(gg_dta)

## -------- pbc data
# We need to create this dataset
data(pbc, package = "randomForestSRC", )
# For whatever reason, the age variable is in days...
# makes no sense to me
for (ind in seq_len(dim(pbc)[2])) {
  if (!is.factor(pbc[, ind])) {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  } else {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  }
  if (!is.logical(pbc[, ind]) &
    length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
    pbc[, ind] <- factor(pbc[, ind])
  }
}
# Convert age to years
pbc$age <- pbc$age / 364.24

pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]
# Create a test set from the remaining patients
pbc_test <- pbc[which(is.na(pbc$treatment)), ]

# ========
# build the forest:
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
  dta_train,
  nsplit = 10,
  na.action = "na.impute",
  forest = TRUE,
  importance = TRUE,
  save.memory = TRUE
)

gg_dta <- gg_vimp(rfsrc_pbc)
plot(gg_dta)

# Restrict to only the top 10.
gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10)
plot(gg_dta)

}
\references{
Ishwaran H. (2007). Variable importance in binary regression trees and
forests, \emph{Electronic J. Statist.}, 1:519-537.
}
\seealso{
\code{\link{plot.gg_vimp}} \code{\link[randomForestSRC]{rfsrc}}

\code{\link[randomForestSRC]{vimp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_varpro.R
\name{.build_varpro_imp_tree}
\alias{.build_varpro_imp_tree}
\title{Build per-tree importance matrix from varpro object results}
\usage{
.build_varpro_imp_tree(object)
}
\description{
Reconstructs the ntree x p per-tree importance matrix from
\code{object$results}, replicating the aggregation performed internally
by \code{varPro:::.importance.varpro.workhorse}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_partial.R
\name{plot.gg_partial_rfsrc}
\alias{plot.gg_partial_rfsrc}
\title{Plot a \code{\link{gg_partial_rfsrc}} object}
\usage{
\method{plot}{gg_partial_rfsrc}(x, ...)
}
\arguments{
\item{x}{A \code{\link{gg_partial_rfsrc}} object.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} (or \code{patchwork}) object.  When both continuous
and categorical variables are present the two panels are combined
vertically via \code{patchwork::wrap_plots()}.
}
\description{
Renders the partial dependence curves from \code{\link{gg_partial_rfsrc}}
as a ggplot2 figure.  The layout adapts automatically to what the object
contains.
}
\details{
For a standard regression or classification forest, continuous predictors
are drawn as line plots and categorical predictors as bar charts, both
faceted by variable name -- the same arrangement as
\code{\link{plot.gg_partial}}.

For a survival forest, each call to \code{partial.rfsrc} returns a predicted
quantity (survival probability, cumulative hazard function, or mortality) at
one or more chosen time horizons.  When a \code{time} column is present in
the data, each horizon becomes a separate coloured curve over the predictor's
value, still faceted by variable.  The y-axis label (\dQuote{Predicted
Survival}, \dQuote{Predicted CHF}, or \dQuote{Predicted Mortality}) tracks
the \code{partial.type} attribute set by \code{gg_partial_rfsrc()}.

For a two-variable interaction surface (when \code{xvar2.name} was supplied
to \code{gg_partial_rfsrc}), the secondary variable's levels become
separate coloured lines, faceted by the primary predictor.
}
\examples{
## ------------------------------------------------------------
## Regression forest -- one continuous curve per variable
## ------------------------------------------------------------
set.seed(42)
airq <- na.omit(airquality)
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)

pd <- gg_partial_rfsrc(rfsrc_airq, xvar.names = c("Wind", "Temp"),
                       n_eval = 10)
plot(pd)

\donttest{
## ------------------------------------------------------------
## Survival forest -- one curve per requested time horizon,
## faceted by variable. Y-axis label tracks `partial.type`.
## ------------------------------------------------------------
# randomForestSRC's formula parser requires the unqualified Surv() symbol;
# it Depends on `survival`, so Surv is on the search path once
# randomForestSRC is loaded.
data(veteran, package = "randomForestSRC")
set.seed(42)
rfsrc_v <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
                                  data = veteran, ntree = 50)
ti  <- rfsrc_v$time.interest
t30 <- ti[which.min(abs(ti - 30))]
t90 <- ti[which.min(abs(ti - 90))]

# Default partial.type = "surv" -> y-axis "Predicted Survival"
pd_s <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
                         partial.time = c(t30, t90), n_eval = 8)
plot(pd_s)

# partial.type = "chf" -> y-axis "Predicted CHF"
pd_c <- gg_partial_rfsrc(rfsrc_v, xvar.names = "age",
                         partial.time = c(t30, t90),
                         partial.type = "chf", n_eval = 8)
plot(pd_c)
}

}
\seealso{
\code{\link{gg_partial_rfsrc}}, \code{\link{plot.gg_partial}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_methods.R
\name{summary.gg}
\alias{summary.gg}
\alias{print.summary.gg}
\alias{summary.gg_error}
\alias{summary.gg_vimp}
\alias{summary.gg_rfsrc}
\alias{summary.gg_variable}
\alias{summary.gg_partial}
\alias{summary.gg_partial_rfsrc}
\alias{summary.gg_partialpro}
\alias{summary.gg_partial_varpro}
\alias{summary.gg_roc}
\alias{summary.gg_survival}
\alias{summary.gg_varpro}
\alias{summary.gg_udependent}
\alias{summary.gg_brier}
\alias{summary.gg_isopro}
\alias{summary.gg_beta_varpro}
\alias{summary.gg_ivarpro}
\title{Summary methods for gg_* data objects}
\usage{
\method{print}{summary.gg}(x, ...)

\method{summary}{gg_error}(object, ...)

\method{summary}{gg_vimp}(object, ...)

\method{summary}{gg_rfsrc}(object, ...)

\method{summary}{gg_variable}(object, ...)

\method{summary}{gg_partial}(object, ...)

\method{summary}{gg_partial_rfsrc}(object, ...)

\method{summary}{gg_partialpro}(object, ...)

\method{summary}{gg_partial_varpro}(object, ...)

\method{summary}{gg_roc}(object, ...)

\method{summary}{gg_survival}(object, ...)

\method{summary}{gg_varpro}(object, ...)

\method{summary}{gg_udependent}(object, ...)

\method{summary}{gg_brier}(object, ...)

\method{summary}{gg_isopro}(object, ...)

\method{summary}{gg_beta_varpro}(object, ...)

\method{summary}{gg_ivarpro}(object, ...)
}
\arguments{
\item{x}{A \code{summary.gg} object (for \code{print.summary.gg}).}

\item{...}{Not currently used.}

\item{object}{A \code{gg_*} data object.}
}
\value{
A \code{summary.gg} object: a list with \code{header} and
\code{body} character vectors. \code{print.summary.gg} returns it
invisibly.
}
\description{
Where \code{print} gives you a one-line header, \code{summary} digs a level
deeper. Each \code{summary.gg_*()} method returns a \code{summary.gg}
object: a header line plus a few diagnostic statistics for that object
type (the OOB error curve, the top VIMP variables, a time range, the
integrated CRPS, and so on). \code{print.summary.gg()} renders it to the
console.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
summary(gg_error(rf))
summary(gg_vimp(rf))

}
\seealso{
\code{\link{print.gg}}, \code{\link{autoplot.gg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_rfsrc.R
\name{gg_rfsrc.rfsrc}
\alias{gg_rfsrc.rfsrc}
\alias{gg_rfsrc}
\title{Predicted response data object}
\usage{
\method{gg_rfsrc}{rfsrc}(object, oob = TRUE, by, ...)
}
\arguments{
\item{object}{A fitted \code{\link[randomForestSRC]{rfsrc}} or
\code{\link[randomForest]{randomForest}} object.}

\item{oob}{Logical; if \code{TRUE} (default) return out-of-bag predictions.
Set to \code{FALSE} to use full in-bag (training) predictions. Forced to
\code{FALSE} automatically for \code{predict.rfsrc} objects, which carry
no OOB estimates.}

\item{by}{Optional stratifying variable. Either a character column name
present in the training data, or a vector/factor of the same length as
the training set. When supplied, a \code{group} column is added to the
returned data and bootstrap CI bands (survival) are computed per group.
Omit or leave missing to return an unstratified result.}

\item{...}{Additional arguments controlling output for specific forest
families:
\describe{
\item{surv_type}{Character; one of \code{"surv"} (default),
\code{"chf"}, or \code{"mortality"} for survival forests.}
\item{conf.int}{Numeric coverage probability (e.g. \code{0.95}) to
request bootstrap pointwise confidence bands for survival forests.
Triggers wide-format output with \code{lower}, \code{upper},
\code{median}, and \code{mean} columns.}
\item{bs.sample}{Integer; number of bootstrap resamples when
\code{conf.int} is set. Defaults to the number of observations.}
}}
}
\value{
A \code{gg_rfsrc} object (a classed \code{data.frame}) whose
structure depends on the forest family:
\describe{
\item{regression}{Columns \code{yhat} and the response name; optionally
a \code{group} column when \code{by} is supplied.}
\item{classification}{One column per class with predicted probabilities;
a \code{y} column with observed class labels; optionally \code{group}.}
\item{survival (no CI / grouping)}{Long-format with columns
\code{variable} (event time), \code{value} (survival probability),
\code{obs_id}, and \code{event}.}
\item{survival (with \code{conf.int} or \code{by})}{Wide-format with
pointwise bootstrap CI columns (\code{lower}, \code{upper},
\code{median}, \code{mean}) per time point; a \code{group} column
when \code{by} is supplied.}
}
The object carries class attributes for the forest family so that
\code{\link{plot.gg_rfsrc}} dispatches correctly.
}
\description{
Every tree in a random forest makes its own prediction, and the forest's
"ensemble" prediction is the average across all trees.  The out-of-bag
(OOB) variant averages only over the trees that did not include a given
observation in their bootstrap sample -- a built-in cross-validation
estimate that requires no held-out test set.  \code{gg_rfsrc} pulls those
ensemble predictions out of the fitted forest and arranges them for plotting
with \code{\link{plot.gg_rfsrc}}.
}
\details{
The structure of the returned data depends on the forest family.  For a
regression forest you get a scatter of OOB predicted values against the
observed response.  For classification you get predicted class probabilities
alongside the observed class label.  For a survival forest you get the
ensemble survival function (or cumulative hazard, or mortality, controlled
by \code{surv_type}) at each unique event time -- one curve per
observation -- which together trace the range of predicted risk in the
cohort.  Pass \code{conf.int} to add pointwise bootstrap confidence bands
around the mean survival curve, or \code{by} to stratify all of the above
by a predictor group.

For survival forests, use the \code{surv_type} argument
(\code{"surv"}, \code{"chf"}, or \code{"mortality"}) to select the
predicted quantity. Bootstrap confidence bands are requested by passing
\code{conf.int} (e.g. \code{conf.int = 0.95}); the number of resamples
is controlled by \code{bs.sample}.
}
\examples{
## ------------------------------------------------------------
## classification example (small, runs on CRAN)
## ------------------------------------------------------------
## -------- iris data
set.seed(42)
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ ., data = iris, ntree = 50)
gg_dta <- gg_rfsrc(rfsrc_iris)
plot(gg_dta)

\donttest{
## ------------------------------------------------------------
## Additional regression / survival examples are guarded with
## \donttest because the cumulative example time exceeds the
## 10-second CRAN budget. Run locally with `R CMD check --run-donttest`
## (or `devtools::check(run_dont_test = TRUE)`) to exercise them.
## ------------------------------------------------------------

## -------- air quality data (regression)
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., data = airquality,
                    na.action = "na.impute", ntree = 50)
plot(gg_rfsrc(rfsrc_airq))

## -------- Boston data (rfsrc + randomForest)
if (requireNamespace("MASS", quietly = TRUE)) {
  data(Boston, package = "MASS")
  Boston$chas <- as.logical(Boston$chas)
  rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., data = Boston, ntree = 50,
                        forest = TRUE, importance = TRUE,
                        tree.err = TRUE, save.memory = TRUE)
  plot(gg_rfsrc(rfsrc_boston))

  rf_boston <- randomForest::randomForest(medv ~ ., data = Boston,
                                          ntree = 50)
  plot(gg_rfsrc(rf_boston))
}

## -------- mtcars data
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ ., data = mtcars, ntree = 50)
plot(gg_rfsrc(rfsrc_mtcars))

## -------- veteran data (survival; with CI and group-by)
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ ., data = veteran,
                       ntree = 50)
plot(gg_rfsrc(rfsrc_veteran))
plot(gg_rfsrc(rfsrc_veteran, conf.int = .95))
plot(gg_rfsrc(rfsrc_veteran, by = "trt"))
}

}
\seealso{
\code{\link{plot.gg_rfsrc}},
\code{\link[randomForestSRC]{rfsrc}},
\code{\link{gg_survival}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_methods.R
\name{print.gg}
\alias{print.gg}
\alias{print.gg_error}
\alias{print.gg_vimp}
\alias{print.gg_rfsrc}
\alias{print.gg_variable}
\alias{print.gg_partial}
\alias{print.gg_partial_rfsrc}
\alias{print.gg_partialpro}
\alias{print.gg_partial_varpro}
\alias{print.gg_roc}
\alias{print.gg_survival}
\alias{print.gg_brier}
\alias{print.gg_udependent}
\alias{print.summary.gg_udependent}
\alias{print.gg_varpro}
\alias{print.gg_isopro}
\alias{print.gg_beta_varpro}
\alias{print.gg_ivarpro}
\title{Print methods for gg_* data objects}
\usage{
\method{print}{gg_error}(x, ...)

\method{print}{gg_vimp}(x, ...)

\method{print}{gg_rfsrc}(x, ...)

\method{print}{gg_variable}(x, ...)

\method{print}{gg_partial}(x, ...)

\method{print}{gg_partial_rfsrc}(x, ...)

\method{print}{gg_partialpro}(x, ...)

\method{print}{gg_partial_varpro}(x, ...)

\method{print}{gg_roc}(x, ...)

\method{print}{gg_survival}(x, ...)

\method{print}{gg_brier}(x, ...)

\method{print}{gg_udependent}(x, ...)

\method{print}{summary.gg_udependent}(x, ...)

\method{print}{gg_varpro}(x, ...)

\method{print}{gg_isopro}(x, ...)

\method{print}{gg_beta_varpro}(x, ...)

\method{print}{gg_ivarpro}(x, ...)
}
\arguments{
\item{x}{A \code{gg_*} data object.}

\item{...}{Not currently used.}
}
\value{
The object \code{x}, invisibly.
}
\description{
Each \code{print.gg_*()} method prints a one-line header: the class label
and, where the forest recorded it, provenance (source package, family,
ntree, n). It returns the object invisibly, so \code{print()} sits cleanly
in a pipe.
}
\details{
To see the rows themselves, use \code{head()}; for per-class diagnostics,
use \code{\link{summary.gg}}.
}
\examples{
set.seed(42)
airq <- na.omit(airquality)
rf <- randomForestSRC::rfsrc(Ozone ~ ., data = airq, ntree = 50)
print(gg_error(rf))
print(gg_vimp(rf))

}
\seealso{
\code{\link{summary.gg}}, \code{\link{autoplot.gg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_varpro.R
\name{plot.gg_varpro}
\alias{plot.gg_varpro}
\title{Plot a \code{gg_varpro} variable importance object}
\usage{
\method{plot}{gg_varpro}(x, type, ...)
}
\arguments{
\item{x}{A \code{gg_varpro} object from \code{\link{gg_varpro}}.}

\item{type}{Character; the display scale.  Leave it off and it is read
from \code{provenance$local.std}: \code{"z"} when \code{local.std =
TRUE} (the default), \code{"raw"} when \code{local.std = FALSE}.
Asking for a scale that the extract step did not prepare raises an
error.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} object.
}
\description{
Draws a horizontal boxplot of the per-tree importance z-scores, or of the
raw importances if you asked for those.  Set \code{faithful = TRUE} at
extract time and the per-tree points are scattered over the box; for a
classification forest, \code{conditional = TRUE} splits the plot into one
facet per class.
}
\details{
\strong{Boxplot geometry:} the hinges are the 15th and 85th percentiles of the
per-tree z-distribution, and the whiskers run to the 5th and 95th.  This
is \strong{not} a Tukey boxplot, and the plot carries a caption that says
so.

\strong{\code{faithful = TRUE}:} the per-tree values are jittered over the box
as semi-transparent points, on the same scale as the box itself (z when
\code{local.std = TRUE}, raw when \code{local.std = FALSE}).  The box is
drawn faint to let the points show through, and a white-outlined dot
marks the mean.

\strong{\code{conditional = TRUE}:} the class-conditional importances
(\code{$conditional}) are shown as a faceted bar chart
(\code{facet_wrap(~class, nrow = 1)}).  Variables keep the sort order set
by the unconditional median z in \code{$stats}, so the facets line up.
}
\section{Reading the boxplot}{

Variables are sorted top to bottom by descending median per-tree
importance, so the eye lands on the most important variable first.
For each variable the box spans the 15th to 85th percentile of the
per-tree scores, the centre line is the median, and the whiskers run
out to the 5th and 95th percentile, not the usual Tukey 1.5 IQR
whiskers. The dashed vertical line is the selection \code{cutoff}
(default \code{0.79}). On the default z-score axis
(\code{local.std = TRUE}) that line is a z; on the raw-importance
axis (\code{local.std = FALSE}, \code{type = "raw"}) it is the same
numeric value but in raw-importance units. Boxes whose aggregate
value sits above the line are coloured blue and flagged
\code{selected = TRUE}, the rest are grey. A
selected variable with a tight, high box is a variable the forest
agrees on across trees. A selected variable with a wide box that
straddles the cutoff is one to look at twice before relying on it.

With \code{faithful = TRUE} the box is drawn faint and the per-tree
values are jittered over it as semi-transparent points, on the same
scale as the box (z when \code{local.std = TRUE}, raw otherwise). A
white-outlined dot marks the mean. Use this view when you want to
see how individual trees voted rather than just the summary.

For a classification forest with \code{conditional = TRUE} the plot
splits into one facet per class. Variables keep the unconditional
sort order, so the rows line up across facets and you can read
across to see which class a variable is informative for.
}

\section{What this tells you}{

Take the variables above the cutoff as your candidate set. Use the
width of the box and the per-tree overlay to gauge confidence: a
narrow box well above the cutoff is a confident pick, a wide box
that crosses it is a coin flip you should not lean on. For
classification, conditional importance tells you which variables
drive which class; a variable that is unconditionally important but
only important for one class out of several is still useful, just
useful for a narrower question.
}

\examples{
\donttest{
set.seed(42)
vp <- varPro::varpro(mpg ~ ., data = mtcars, ntree = 50)
plot(gg_varpro(vp))
plot(gg_varpro(vp, faithful = TRUE))
}

}
\seealso{
\code{\link{gg_varpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_partial.rfsrc.R
\name{surv_partial.rfsrc}
\alias{surv_partial.rfsrc}
\title{Survival partial dependence data for one or more predictors}
\usage{
surv_partial.rfsrc(rforest, var_list, npts = 25, partial.type = "surv")
}
\arguments{
\item{rforest}{A fitted \code{\link[randomForestSRC]{rfsrc}} survival or
competing-risk forest object.}

\item{var_list}{Character vector of predictor names for which partial
dependence should be computed. Each must appear in
\code{rforest$xvar.names}.}

\item{npts}{Integer; the number of predictor grid points to evaluate
(default 25). Evenly-spaced unique values are sampled from each predictor.}

\item{partial.type}{The prediction type to return. For survival forests one
of \code{"surv"} (default), \code{"mort"}, or \code{"chf"}. For competing
risk forests one of \code{"years.lost"}, \code{"cif"}, or \code{"chf"}.
See \code{\link[randomForestSRC]{partial.rfsrc}} for full details.}
}
\value{
A named list with one element per variable in \code{var_list}. Each
element is itself a list with:
\describe{
\item{name}{The predictor variable name (character).}
\item{dta}{The raw output of
\code{\link[randomForestSRC]{get.partial.plot.data}}, a list containing
at minimum \code{x} (predictor values) and \code{yhat} (partial
predictions), and for survival/competing risk, \code{partial.time}.}
}
}
\description{
\strong{Deprecated.} Use \code{\link{gg_partial_rfsrc}} instead, which
returns a classed \code{gg_partial_rfsrc} object with a dedicated
\code{plot()} method.
}
\details{
Computes partial dependence curves for a survival or competing-risk
\code{\link[randomForestSRC]{rfsrc}} forest.  For each predictor it calls
\code{\link[randomForestSRC]{partial.rfsrc}} at \code{npts} evenly-spaced
unique values, across every stored event time.
}
\examples{
## ------------------------------------------------------------
## survival
## ------------------------------------------------------------

data(veteran, package = "randomForestSRC")
v.obj <- randomForestSRC::rfsrc(Surv(time,status)~.,
  veteran, nsplit = 10, ntree = 100)

spart <- surv_partial.rfsrc(v.obj, var_list="age", partial.type = "mort")

## partial effect of age on mortality
partial.obj <- randomForestSRC::partial(v.obj,
                       partial.type = "mort",
                       partial.xvar = "age",
                       partial.values = v.obj$xvar$age,
                       partial.time = v.obj$time.interest)
pdta <- randomForestSRC::get.partial.plot.data(partial.obj)

plot(lowess(pdta$x, pdta$yhat, f = 1/3),
     type = "l", xlab = "age", ylab = "adjusted mortality")

## example where x is discrete - partial effect of age on mortality
## we use the granule=TRUE option
partial.obj <- randomForestSRC::partial(v.obj,
                       partial.type = "mort",
                       partial.xvar = "trt",
                       partial.values = v.obj$xvar$trt,
                       partial.time = v.obj$time.interest)
pdta <- randomForestSRC::get.partial.plot.data(partial.obj, granule = TRUE)
boxplot(pdta$yhat ~ pdta$x, xlab = "treatment", ylab = "partial effect")


## partial effects of karnofsky score on survival
karno <- quantile(v.obj$xvar$karno)
partial.obj <- randomForestSRC::partial(v.obj,
                       partial.type = "surv",
                       partial.xvar = "karno",
                       partial.values = karno,
                       partial.time = v.obj$time.interest)
pdta <- randomForestSRC::get.partial.plot.data(partial.obj)

matplot(pdta$partial.time, t(pdta$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "karnofsky adjusted survival")
legend("topright", legend = paste0("karnofsky = ", karno), fill = 1:5)


## ------------------------------------------------------------
## competing risk
## ------------------------------------------------------------

data(follic, package = "randomForestSRC")
follic.obj <- randomForestSRC::rfsrc(Surv(time, status) ~ ., follic, nsplit = 3, ntree = 100)

## partial effect of age on years lost
partial.obj <- randomForestSRC::partial(follic.obj,
                       partial.type = "years.lost",
                       partial.xvar = "age",
                       partial.values = follic.obj$xvar$age,
                       partial.time = follic.obj$time.interest)
pdta1 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 1)
pdta2 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 2)

# Save and restore the user's graphical parameters per CRAN policy.
oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar))
par(mfrow = c(2, 2))
plot(lowess(pdta1$x, pdta1$yhat),
     type = "l", xlab = "age", ylab = "adjusted years lost relapse")
plot(lowess(pdta2$x, pdta2$yhat),
     type = "l", xlab = "age", ylab = "adjusted years lost death")

## partial effect of age on cif
partial.obj <- randomForestSRC::partial(follic.obj,
                       partial.type = "cif",
                       partial.xvar = "age",
                       partial.values = quantile(follic.obj$xvar$age),
                       partial.time = follic.obj$time.interest)
pdta1 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 1)
pdta2 <- randomForestSRC::get.partial.plot.data(partial.obj, target = 2)

matplot(pdta1$partial.time, t(pdta1$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "age adjusted cif for relapse")
matplot(pdta2$partial.time, t(pdta2$yhat), type = "l", lty = 1,
        xlab = "time", ylab = "age adjusted cif for death")

}
\seealso{
\code{\link{gg_partial_rfsrc}},
\code{\link[randomForestSRC]{partial.rfsrc}},
\code{\link[randomForestSRC]{get.partial.plot.data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varpro_feature_names.R
\name{varpro_feature_names}
\alias{varpro_feature_names}
\title{Recover original variable names from varpro one-hot encoded feature names}
\usage{
varpro_feature_names(varpro_names, dataset)
}
\arguments{
\item{varpro_names}{character vector of names as output by varpro (may
include one-hot encoded suffixed names such as \code{"sex0"}, \code{"sex1"})}

\item{dataset}{the original data frame passed to varpro, used to look up
valid column names}
}
\value{
character vector of unique original variable names (no suffixes)
}
\description{
\code{varpro} one-hot encodes factor variables, appending a numeric suffix
for each level -- \code{sex} becomes \code{sex0} and \code{sex1}.  To map
those back, this function strips the suffix one character at a time until
every name in \code{varpro_names} matches a column in \code{dataset}.
}
\examples{
## ------------------------------------------------------------------
## Simple case: one continuous variable + one binary factor
## ------------------------------------------------------------------
ds <- data.frame(age = c(25, 30, 45), sex = c("M", "F", "M"))

# varpro one-hot encodes 'sex' into 'sex0' and 'sex1'
varpro_names <- c("age", "sex0", "sex1")
varpro_feature_names(varpro_names, ds)
# Returns: c("age", "sex")

## ------------------------------------------------------------------
## Multi-level factor: three-level 'group' variable
## ------------------------------------------------------------------
ds2 <- data.frame(score = 1:6,
                  group = factor(rep(c("A", "B", "C"), 2)))

# varpro appends 0/1/2 for each level
vn2 <- c("score", "group0", "group1", "group2")
varpro_feature_names(vn2, ds2)
# Returns: c("score", "group")

## ------------------------------------------------------------------
## Already-clean names pass through unchanged
## ------------------------------------------------------------------
ds3 <- data.frame(x = 1:5, y = 1:5)
varpro_feature_names(c("x", "y"), ds3)
# Returns: c("x", "y")

}
\seealso{
\code{\link{gg_partialpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_partial_varpro.R, R/gg_partialpro.R
\name{gg_partial_varpro}
\alias{gg_partial_varpro}
\alias{gg_partialpro}
\title{Partial dependence data from a varPro model}
\usage{
gg_partial_varpro(
  part_dta = NULL,
  object = NULL,
  scale = c("auto", "rmst", "mortality", "surv", "chf"),
  time = NULL,
  nvars = NULL,
  cat_limit = 10,
  model = NULL
)

gg_partialpro(
  part_dta,
  object = NULL,
  scale = c("auto", "rmst", "mortality", "surv", "chf"),
  time = NULL,
  nvars = NULL,
  cat_limit = 10,
  model = NULL
)
}
\arguments{
\item{part_dta}{Passed to \code{\link{gg_partial_varpro}}.}

\item{object}{Passed to \code{\link{gg_partial_varpro}}.}

\item{scale}{Passed to \code{\link{gg_partial_varpro}}.}

\item{time}{Passed to \code{\link{gg_partial_varpro}}.}

\item{nvars}{Passed to \code{\link{gg_partial_varpro}}.}

\item{cat_limit}{Passed to \code{\link{gg_partial_varpro}}.}

\item{model}{Passed to \code{\link{gg_partial_varpro}}.}
}
\value{
A named list of class \code{"gg_partial_varpro"} with elements:
\describe{
\item{continuous}{data.frame with columns \code{variable},
\code{parametric}, \code{nonparametric}, \code{causal}, \code{name}
(and optionally \code{model}).}
\item{categorical}{data.frame with the same columns, one row per
observation per category level.}
}
A \code{"provenance"} attribute carries \code{source}, \code{family},
\code{ntree}, \code{n}, \code{scale}, \code{rmst_tau},
\code{xvar.names}, and \code{path}.

A \code{gg_partial_varpro} object (see
\code{\link{gg_partial_varpro}}).
}
\description{
\code{varPro::partialpro} returns one list, with continuous and
categorical predictors mixed together. This function splits that list into
two tidy data frames, one for each kind, and resolves the y-axis label the
plot method will use.

\emph{Deprecated.} \code{gg_partialpro()} has been superseded by
\code{\link{gg_partial_varpro}()} and is now a thin alias for it.  It will
be removed in the release after \pkg{ggRandomForests} v3.0.0; use
\code{\link{gg_partial_varpro}()} directly.
}
\details{
\strong{Scale detection:} with \code{scale = "auto"} and an \code{object} in
hand, the scale resolves to \code{"mortality"} for a survival forest and
\code{"generic"} for a regression or classification forest.  The RMST
horizon \eqn{\tau} is \emph{not} stored in the \code{varpro} object
(varPro 3.1.0), so for RMST-labeled output you have to pass
\code{scale = "rmst", time = tau} yourself.

\strong{Ensemble mortality (scale = "mortality"):} here the y-axis is
\emph{ensemble mortality}, the expected number of events a subject would
see if they were exposed to the study-average cumulative hazard.  It is
the same quantity as the \code{rfsrc} \code{predicted} value for survival
forests (Ishwaran, Kogalur, Blackstone & Lauer, 2008
\url{doi:10.1214/08-AOAS169}).  This is an \strong{unbounded relative-risk
score}, \emph{not} a survival probability and not \eqn{1 - S(t)}; don't
read it as one.  If you want output on the probability scale, refit with
\code{varpro(..., rmst = tau)} and use \code{scale = "rmst"}.
}
\section{What partialpro is doing}{

A partial dependence curve answers the question, "if I hold a single
variable at a grid of values and average out everything else, how does
the model's prediction move?" That is the same question \code{rfsrc}
partial dependence answers. What \code{varPro::partialpro} adds is two
wrinkles that are worth understanding before you read the curves.

First, \code{partialpro} filters the partial grid through an isolation
forest (Unlimited Virtual Twins, or UVT) so that unlikely combinations
of the focal variable with the rest of the data are downweighted. The
\code{rfsrc} version, by contrast, averages over the full marginal grid
regardless of plausibility. So when a covariate is highly correlated
with others, the two methods can disagree, and \code{partialpro}'s
curve is the one restricted to the data manifold.

Second, \code{partialpro} fits a local polynomial model to the
predicted values rather than just plotting their mean. That gives
three parallel curves per variable, stored as \code{yhat.par},
\code{yhat.nonpar}, and \code{yhat.causal}, which the plot method
overlays so you can see whether a smooth parametric story and the
raw forest predictions are telling you the same thing.

Interpretation of the y-axis depends on the outcome (per
\code{varPro::partialpro}): response scale for regression, log-odds of
the target class for classification, and either ensemble mortality
(default) or RMST (if the original \code{varpro} call set
\code{rmst}) for survival.
}

\section{What's in the output}{

We split \code{partialpro}'s mixed list into two tidy data frames so
the plot method does not have to. A variable with more than
\code{cat_limit} distinct grid points goes into \code{$continuous},
one row per grid point with the column means of \code{yhat.par},
\code{yhat.nonpar}, and \code{yhat.causal} stored as
\code{parametric}, \code{nonparametric}, and \code{causal}. A
variable at or below \code{cat_limit} goes into \code{$categorical},
one row per observation per category level, carrying the same three
columns unaveraged so the plot method can draw boxplots. Path C
(\code{scale \%in\% c("surv","chf")}) takes a different route: we
hand the underlying \code{rfsrc} forest to \code{gg_partial_rfsrc} so
you get a survival-probability or cumulative-hazard curve on the
usual rfsrc scale instead.
}

\section{What you use this for}{

\itemize{
\item read the marginal shape of a relationship the varpro model
found important: monotone, threshold, U-shape, flat;
\item compare the three partialpro estimators on the same variable
and flag the ones where parametric and nonparametric disagree,
those are the candidates for closer inspection;
\item report a survival partial dependence on the probability or
cumulative-hazard scale (\code{scale = "surv"} or \code{"chf"})
rather than the unbounded mortality scale.
}
A varpro partial dependence curve is a description of the model, not
a causal effect. The \code{causal} column is varpro's local
estimator, not a structural causal claim about the data-generating
process.
}

\examples{
set.seed(42)
n_obs <- 30; n_pts <- 15
mock_data <- list(
  age = list(
    xvirtual    = seq(30, 80, length.out = n_pts),
    xorg        = sample(seq(30, 80, by = 5), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * n_pts), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * n_pts), nrow = n_obs)
  ),
  sex = list(
    xvirtual    = c(0, 1),
    xorg        = sample(c(0, 1), n_obs, replace = TRUE),
    yhat.par    = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.nonpar = matrix(rnorm(n_obs * 2), nrow = n_obs),
    yhat.causal = matrix(rnorm(n_obs * 2), nrow = n_obs)
  )
)
result <- gg_partial_varpro(mock_data)
head(result$continuous)
head(result$categorical)

}
\references{
Ishwaran H, Kogalur UB, Blackstone EH, Lauer MS (2008).
Random survival forests. \emph{The Annals of Applied Statistics},
\bold{2}(3), 841--860. \doi{10.1214/08-AOAS169}.
}
\seealso{
\code{\link{plot.gg_partial_varpro}},
\code{\link{gg_varpro}}, \code{\link{gg_vimp}},
\code{\link{gg_partialpro}} (deprecated),
\code{\link{gg_partial_rfsrc}}, \code{\link{varpro_feature_names}}

\code{\link{gg_partial_varpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_vimp.R
\name{gg_vimp}
\alias{gg_vimp}
\alias{gg_vimp.rfsrc}
\alias{gg_vimp.randomForest}
\alias{gg_vimp.randomForest.formula}
\title{Variable Importance (VIMP) data object}
\usage{
gg_vimp(object, nvar, ...)
}
\arguments{
\item{object}{A \code{\link[randomForestSRC]{rfsrc}} object, the output from
\code{\link[randomForestSRC]{vimp}}, or a fitted
\code{\link[randomForest]{randomForest}}.}

\item{nvar}{argument to control the number of variables included in the
output.}

\item{...}{arguments passed to the \code{\link[randomForestSRC]{vimp.rfsrc}}
function if the \code{\link[randomForestSRC]{rfsrc}} object does not contain
importance information.}
}
\value{
\code{gg_vimp} object. A \code{data.frame} of VIMP measures, in rank
order, optionally containing class-specific scores and a relative importance
column. When \code{randomForest} objects lack stored importance values a
warning is issued and \code{NA} placeholders are returned so plots remain
reproducible.
}
\description{
\code{gg_vimp} Extracts the variable importance (VIMP) information from a
\code{\link[randomForestSRC]{rfsrc}} or \code{\link[randomForest]{randomForest}}
object and reshapes it into a tidy data set.
}
\details{
\code{gg_vimp()} shows \strong{permutation (Breiman-Cutler) variable
importance}: the forest permutes a variable's observed values across the
out-of-bag (OOB) cases, runs those perturbed cases down the already-grown
trees, and measures how much the OOB prediction error climbs.  That
perturbation is synthetic (the variable's link to the response is broken
on purpose) so a large increase means the variable was carrying genuine
signal; near-zero or negative values mean it added noise or nothing at all.

\code{\link{gg_varpro}()} takes the opposite route, comparing local
estimators on real observed data through varPro's release rules, with no
permutation and no synthetic features.  The two approaches answer "which
variables matter?" by opposite mechanisms, so a variable can rank
differently under each, and that disagreement is itself informative: it
often signals interaction structure or non-monotone effects that one
mechanism surfaces and the other obscures.

For survival forests, VIMP is measured against the ensemble cumulative
hazard function (CHF); the error metric is one minus the concordance index
(C-statistic).  Variables with non-positive VIMP are flagged in the
\code{positive} column and colored differently by
\code{\link{plot.gg_vimp}}.
}
\examples{
## ------------------------------------------------------------
## classification example
## ------------------------------------------------------------
## -------- iris data
rfsrc_iris <- randomForestSRC::rfsrc(Species ~ .,
  data = iris,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_iris)
plot(gg_dta)

## ------------------------------------------------------------
## regression example
## ------------------------------------------------------------

## -------- air quality data
rfsrc_airq <- randomForestSRC::rfsrc(Ozone ~ ., airquality,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_airq)
plot(gg_dta)


## -------- Boston data
data(Boston, package = "MASS")
rfsrc_boston <- randomForestSRC::rfsrc(medv ~ ., Boston,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_boston)
plot(gg_dta)

## -------- Boston data
rf_boston <- randomForest::randomForest(medv ~ ., Boston)
gg_dta <- gg_vimp(rf_boston)
plot(gg_dta)


## -------- mtcars data
rfsrc_mtcars <- randomForestSRC::rfsrc(mpg ~ .,
  data = mtcars,
  importance = TRUE
)
gg_dta <- gg_vimp(rfsrc_mtcars)
plot(gg_dta)

## ------------------------------------------------------------
## survival example
## ------------------------------------------------------------

## -------- veteran data
data(veteran, package = "randomForestSRC")
rfsrc_veteran <- randomForestSRC::rfsrc(Surv(time, status) ~ .,
  data = veteran,
  ntree = 100,
  importance = TRUE
)

gg_dta <- gg_vimp(rfsrc_veteran)
plot(gg_dta)

## -------- pbc data
# We need to create this dataset
data(pbc, package = "randomForestSRC", )
# For whatever reason, the age variable is in days...
# makes no sense to me
for (ind in seq_len(dim(pbc)[2])) {
  if (!is.factor(pbc[, ind])) {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(range(pbc[, ind], na.rm = TRUE) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  } else {
    if (length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 2) {
      if (sum(sort(unique(pbc[, ind])) == c(0, 1)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
      if (sum(sort(unique(pbc[, ind])) == c(FALSE, TRUE)) == 2) {
        pbc[, ind] <- as.logical(pbc[, ind])
      }
    }
  }
  if (!is.logical(pbc[, ind]) &
    length(unique(pbc[which(!is.na(pbc[, ind])), ind])) <= 5) {
    pbc[, ind] <- factor(pbc[, ind])
  }
}
# Convert age to years
pbc$age <- pbc$age / 364.24

pbc$years <- pbc$days / 364.24
pbc <- pbc[, -which(colnames(pbc) == "days")]
pbc$treatment <- as.numeric(pbc$treatment)
pbc$treatment[which(pbc$treatment == 1)] <- "DPCA"
pbc$treatment[which(pbc$treatment == 2)] <- "placebo"
pbc$treatment <- factor(pbc$treatment)
dta_train <- pbc[-which(is.na(pbc$treatment)), ]
# Create a test set from the remaining patients
pbc_test <- pbc[which(is.na(pbc$treatment)), ]

# ========
# build the forest:
rfsrc_pbc <- randomForestSRC::rfsrc(
  Surv(years, status) ~ .,
  dta_train,
  nsplit = 10,
  na.action = "na.impute",
  forest = TRUE,
  importance = TRUE,
  save.memory = TRUE
)

gg_dta <- gg_vimp(rfsrc_pbc)
plot(gg_dta)

# Restrict to only the top 10.
gg_dta <- gg_vimp(rfsrc_pbc, nvar = 10)
plot(gg_dta)

}
\references{
Ishwaran H. (2007). Variable importance in binary regression trees and
forests, \emph{Electronic J. Statist.}, 1:519-537.
}
\seealso{
\code{\link{plot.gg_vimp}} \code{\link[randomForestSRC]{rfsrc}}

\code{\link[randomForestSRC]{vimp}} \code{\link{gg_varpro}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.gg_ivarpro.R
\name{plot.gg_ivarpro}
\alias{plot.gg_ivarpro}
\title{Plot a \code{gg_ivarpro} object}
\usage{
\method{plot}{gg_ivarpro}(x, ...)
}
\arguments{
\item{x}{A \code{gg_ivarpro} object from \code{\link[=gg_ivarpro]{gg_ivarpro()}}.}

\item{...}{Not currently used.}
}
\value{
A \code{ggplot} object.
}
\description{
Branches on the presence of \code{which_obs} provenance and the \code{class}
column. Distribution view: jittered points showing per-observation
local importances per variable. Per-observation view: horizontal bar
chart of one observation's local importances across variables.
Classification: faceted by class unless \code{which_class} collapses to
a single class.
}
\section{Reading the chart}{

Each point in the distribution view is one observation's local
importance for that variable. Variables are sorted by descending
\verb{mean(|local_imp|)}. The cutoff line picks the variables whose local
importance is, on average, large enough to flag. For a classification
fit, every facet shares the same row order so you can read across.

For a classification fit, variables are sorted by descending
\verb{mean(|local_imp|)} across all (obs, class) rows and that ordering
is shared across every facet, so rows line up between classes for
visual comparison. Each facet has its own cutoff line.

The per-observation view (\code{which_obs}) is a horizontal bar chart of
one observation's local importances; bars below the cutoff are grey,
above are blue. The visual resembles a SHAP waterfall, but the values
are release-rule contributions: scaled per-rule contrasts on observed
data, not Shapley values and not permutation-based.
}

\examples{
\donttest{
if (requireNamespace("varPro", quietly = TRUE) &&
    requireNamespace("MASS", quietly = TRUE)) {
  set.seed(1)
  v <- varPro::varpro(medv ~ ., data = MASS::Boston, ntree = 50)
  plot(gg_ivarpro(v))
}
}

}
\seealso{
\code{\link[=gg_ivarpro]{gg_ivarpro()}}.
}
